<?php
$params = array_merge(
    require(__DIR__ . '/../../common/config/params.php'),
    require(__DIR__ . '/../../common/config/params-local.php'),
    require(__DIR__ . '/params.php'),
    require(__DIR__ . '/params-local.php')
);

$baseUrl = str_replace('/web', '', (new \yii\web\Request)->getBaseUrl());

return [
    'id' => 'app-api',
    'basePath' => dirname(__DIR__),
    'controllerNamespace' => 'api\controllers',
    'bootstrap' => ['log'],
    'modules' => [
        'v1' => [
            'class' => 'api\modules\v1\Module',
        ],
    ],
    'components' => [
        'user' => [
            'identityClass' => 'common\modules\user\models\UserIdentity',
            'enableAutoLogin' => true,
            'enableSession' => false, 
            'loginUrl' => null, 
        ],
        'log' => [
            'traceLevel' => YII_DEBUG ? 3 : 0,
            'targets' => [
                [
                    'class' => 'yii\log\FileTarget',
                    'levels' => ['error', 'warning'],
                ],
            ],
        ],
        'errorHandler' => [
            'errorAction' => 'site/error',
        ],
        'urlManager' => [
            'enablePrettyUrl' => true,
            'enableStrictParsing' => true,
            'showScriptName' => false,
            'rules' => [
                [
                    'class' => 'yii\rest\UrlRule', 
                    'controller' => 'v1/user', 
                    'extraPatterns' => [
                        'POST login' => 'login', 
                    ],
                ], 
                [
                    'class' => 'yii\rest\UrlRule', 
                    'controller' => 'v1/course', 
                    'extraPatterns' => [],
                ], 
                [
                    'class' => 'yii\rest\UrlRule', 
                    'controller' => 'v1/user-course', 
                    'extraPatterns' => [],
                ], 
                [
                    'class' => 'yii\rest\UrlRule', 
                    'controller' => 'v1/course-activity', 
                    'extraPatterns' => [],
                ], 
                [
                    'class' => 'yii\rest\UrlRule', 
                    'controller' => 'v1/user-course-activity', 
                    'extraPatterns' => [],
                ], 
            ],
        ], 
        'request' => [
            'baseUrl' => $baseUrl,
            'csrfParam' => '_csrf-api',
            'parsers' => [
                'application/json' => 'yii\web\JsonParser',
            ]
        ], 
    ],
    'params' => $params,
];
