<?php

namespace api\modules\v1\controllers;

use Yii;
use yii\data\ActiveDataProvider;
use yii\web\BadRequestHttpException;
use yii\web\UnauthorizedHttpException;
use yii\filters\auth\HttpBearerAuth;
use yii\helpers\Json;
use common\modules\core\models\CoreSite;
use common\modules\user\models\User;
use common\modules\user\models\UserRole;

class UserController extends \yii\rest\ActiveController
{
	public $modelClass = 'common\modules\user\models\User';

	/**
	 * Checks the privilege of the current user.
	 *
	 * This method should be overridden to check whether the current user has the privilege
	 * to run the specified action against the specified data model.
	 * If the user does not have access, a [[ForbiddenHttpException]] should be thrown.
	 *
	 * @param string $action the ID of the action to be executed
	 * @param \yii\base\Model $model the model to be accessed. If null, it means no specific model is being accessed.
	 * @param array $params additional parameters
	 * @throws ForbiddenHttpException if the user does not have access
	 */
	public function checkAccess($action, $model = null, $params = [])
	{
	    // check if the user can access $action and $model
	    // throw ForbiddenHttpException if access should be denied
	}

	public function actions()
	{
	    $actions = parent::actions();

	    // disable the "delete" and "create" actions
	    unset($actions['create']);

	    // customize the data provider preparation with the "prepareDataProvider()" method
	    // $actions['index']['prepareDataProvider'] = [$this, 'prepareDataProvider'];

	    return $actions;
	}

	public function prepareDataProvider()
	{
	    // prepare and return a data provider for the "index" action
	}

	public function behaviors()
	{
	    $behaviors = parent::behaviors();

	    $behaviors['authenticator'] = [
	        'class' => HttpBearerAuth::className(), 
	    ];

	    // remove authentication filter
	    $auth = $behaviors['authenticator'];
	    
	    unset($behaviors['authenticator']);
	    
	    // add CORS filter
	    $behaviors['corsFilter'] = [
	        'class' => \yii\filters\Cors::className(),
	    ];
	    
	    // re-add authentication filter
	    $behaviors['authenticator'] = $auth;
	    // avoid authentication on CORS-pre-flight requests (HTTP OPTIONS method)
	    $behaviors['authenticator']['except'] = ['options'];

	    $behaviors['authenticator']['except'] = ['login', 'create'];

	    return $behaviors;
	}

	public function actionCreate()
	{
		$request = Yii::$app->request;
		$module = \Yii::$app->controller->module;
		$modelUser = new User();
		$data = array();

		$data = $request->bodyParams;

		$next_user_id = User::find()->select('id')->orderBy(['id' => SORT_DESC])->asArray()->one()['id'];

		$post_data['User']['email'] = $data['email'];
		$post_data['User']['password_hash'] = $data['password_hash'];
		$post_data['User']['role_id'] = UserRole::getAttr(['id'], ['name' => 'user']);
		$post_data['User']['created_by'] = $next_user_id;
		$post_data['User']['updated_by'] = $next_user_id;
		$post_data['User']['site_id'] = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);

		$post_data['UserProfile']['first_name'] = $data['first_name'];
		$post_data['UserProfile']['last_name'] = $data['last_name'];
		$post_data['UserProfile']['gender'] = $data['gender'];
		$post_data['UserProfile']['date_of_birth'] = $data['date_of_birth'];
		$post_data['UserProfile']['ba_address'] = $data['ba_address'];
		$post_data['UserProfile']['ba_city'] = $data['ba_city'];
		$post_data['UserProfile']['ba_country'] = $data['ba_country'];
		$post_data['UserProfile']['ba_zip_code'] = $data['ba_zip_code'];
		$post_data['UserProfile']['sa_address'] = $data['sa_address'];
		$post_data['UserProfile']['sa_city'] = $data['sa_city'];
		$post_data['UserProfile']['sa_country'] = $data['sa_country'];
		$post_data['UserProfile']['sa_zip_code'] = $data['sa_zip_code'];
		$post_data['UserProfile']['phone'] = $data['phone'];
		$post_data['UserProfile']['newsletters'] = $data['newsletters'];
		$post_data['UserProfile']['created_by'] = $next_user_id;
		$post_data['UserProfile']['updated_by'] = $next_user_id;
		
		if (User::find()->where(['email' => $data['email'], 'status' => 'active'])->exists()) {
			$user_id = $modelUser->saveUser($post_data, User::getAttr(['id'], ['email' => $data['email']]));
		} else {
			$user_id = $modelUser->saveUser($post_data);
		}
		
		return $user_id;
	}

    	public function actionLogin()
	{
		$modelUser = new User();
		$module = \Yii::$app->controller->module;
		$request = Yii::$app->request;
		
		$email = $request->getBodyParam('email');
		$password = $request->getBodyParam('password');

		if (!isset($email, $password) || empty($email) || empty($password)) {
			throw new BadRequestHttpException("Email and/or Password not provided.", 400);
		} else {
			if (!User::find()->where(['email' => $email, 'status' => 'active'])->exists()) {
				throw new UnauthorizedHttpException("Email does not exist.", 401);
			}

			$modelUser = User::find()->where(['email' => $email, 'status' => 'active'])->one();

			if (Yii::$app->getSecurity()->validatePassword($password, $modelUser->password_hash)) {
				$access_token = $modelUser->generateAccessToken($modelUser->id);	// all good, generating access token

				$data = User::find()->where(['email' => $email, 'status' => 'active'])->asArray()->one();

				return $data;
			} else {
				throw new UnauthorizedHttpException("Invalid password.", 401);	// wrong password
			}
		}
	}
}
