<?php

namespace api\modules\v1\controllers;

use Yii;
use yii\data\ActiveDataProvider;
use yii\web\BadRequestHttpException;
use yii\web\UnauthorizedHttpException;
use yii\filters\auth\HttpBearerAuth;
use yii\helpers\Json;
use common\modules\lms\models\LmsUserCourseActivity;

class UserCourseActivityController extends \yii\rest\ActiveController
{
	public $modelClass = 'common\modules\lms\models\LmsUserCourseActivity';

	/**
	 * Checks the privilege of the current user.
	 *
	 * This method should be overridden to check whether the current user has the privilege
	 * to run the specified action against the specified data model.
	 * If the user does not have access, a [[ForbiddenHttpException]] should be thrown.
	 *
	 * @param string $action the ID of the action to be executed
	 * @param \yii\base\Model $model the model to be accessed. If null, it means no specific model is being accessed.
	 * @param array $params additional parameters
	 * @throws ForbiddenHttpException if the user does not have access
	 */
	public function checkAccess($action, $model = null, $params = [])
	{
	    // check if the user can access $action and $model
	    // throw ForbiddenHttpException if access should be denied
	}

	public function actions()
	{
	    $actions = parent::actions();

	    // disable the "delete" and "create" actions
	    unset($actions['create']);

	    // customize the data provider preparation with the "prepareDataProvider()" method
	    // $actions['index']['prepareDataProvider'] = [$this, 'prepareDataProvider'];

	    return $actions;
	}

	public function prepareDataProvider()
	{
	    // prepare and return a data provider for the "index" action
	}

	public function behaviors()
	{
	    $behaviors = parent::behaviors();

	    $behaviors['authenticator'] = [
	        'class' => HttpBearerAuth::className(), 
	    ];

	    // remove authentication filter
	    $auth = $behaviors['authenticator'];
	    
	    unset($behaviors['authenticator']);
	    
	    // add CORS filter
	    $behaviors['corsFilter'] = [
	        'class' => \yii\filters\Cors::className(),
	    ];
	    
	    // re-add authentication filter
	    $behaviors['authenticator'] = $auth;
	    // avoid authentication on CORS-pre-flight requests (HTTP OPTIONS method)
	    $behaviors['authenticator']['except'] = ['options'];

	    return $behaviors;
	}

	public function actionCreate()
	{
		$request = Yii::$app->request;
		$modelLmsUserCourseActivity = new LmsUserCourseActivity();
		$data = array();
		
		$data = $request->bodyParams;

		$data['LmsUserCourseActivity']['user_id'] = Yii::$app->user->identity->id;
		$data['LmsUserCourseActivity']['created_by'] = Yii::$app->user->identity->id;
		$data['LmsUserCourseActivity']['updated_by'] = Yii::$app->user->identity->id;

		$data['LmsUserCourseActivityAttempt']['user_id'] = Yii::$app->user->identity->id;
		$data['LmsUserCourseActivityAttempt']['created_by'] = Yii::$app->user->identity->id;
		$data['LmsUserCourseActivityAttempt']['updated_by'] = Yii::$app->user->identity->id;

		if (!isset($data) || empty($data)) {
			throw new BadRequestHttpException("Unable to save this attempt.", 400);
		} else {
			$user_course_activity_id = $modelLmsUserCourseActivity->saveUserCourseActivity($data);

			if (!$user_course_activity_id) {
			    return 'Attempt not saved';
			} else {
			    return $user_course_activity_id;
			}
		}
	}
}
