<?php

namespace api\modules\v1\controllers;

use Yii;
use yii\data\ActiveDataProvider;
use yii\web\BadRequestHttpException;
use yii\web\UnauthorizedHttpException;
use yii\filters\auth\HttpBearerAuth;
use common\modules\lms\models\LmsUserCourse;
use Yadakhov\Json;

class UserCourseController extends \yii\rest\ActiveController
{
	public $modelClass = 'common\modules\lms\models\LmsUserCourse';

	/**
	 * Checks the privilege of the current user.
	 *
	 * This method should be overridden to check whether the current user has the privilege
	 * to run the specified action against the specified data model.
	 * If the user does not have access, a [[ForbiddenHttpException]] should be thrown.
	 *
	 * @param string $action the ID of the action to be executed
	 * @param \yii\base\Model $model the model to be accessed. If null, it means no specific model is being accessed.
	 * @param array $params additional parameters
	 * @throws ForbiddenHttpException if the user does not have access
	 */
	public function checkAccess($action, $model = null, $params = [])
	{
	    // check if the user can access $action and $model
	    // throw ForbiddenHttpException if access should be denied
	}

	public function actions()
	{
	    $actions = parent::actions();

	    // disable the "delete" and "create" actions
	    unset($actions['update']);

	    // customize the data provider preparation with the "prepareDataProvider()" method
	    // $actions['index']['prepareDataProvider'] = [$this, 'prepareDataProvider'];

	    return $actions;
	}

	public function prepareDataProvider()
	{
	    // prepare and return a data provider for the "index" action
	}

	public function behaviors()
	{
	    $behaviors = parent::behaviors();

	    $behaviors['authenticator'] = [
	        'class' => HttpBearerAuth::className(), 
	    ];

	    // remove authentication filter
	    $auth = $behaviors['authenticator'];
	    
	    unset($behaviors['authenticator']);
	    
	    // add CORS filter
	    $behaviors['corsFilter'] = [
	        'class' => \yii\filters\Cors::className(),
	    ];
	    
	    // re-add authentication filter
	    $behaviors['authenticator'] = $auth;
	    // avoid authentication on CORS-pre-flight requests (HTTP OPTIONS method)
	    $behaviors['authenticator']['except'] = ['options'];

	    return $behaviors;
	}

	public function actionUpdate()
	{
		$request = Yii::$app->request;
		$module = \Yii::$app->controller->module;
		$modelLmsUserCourse = new LmsUserCourse();
		$data = array();

		if (!LmsUserCourse::find()->where(['id' => $request->get(), 'status' => 'active'])->exists()) {
			throw new NotFoundHttpException('Course subscription not found.');
		}

		$modelLmsUserCourse = LmsUserCourse::find()->where(['id' => $request->get(), 'status' => 'active'])->one();

		$data = $request->bodyParams;

		if (isset($data['tok'])) {
			unset($data['tok']);
		}

		if (isset($data['reCaptcha'])) {
			unset($data['reCaptcha']);
		}

		$modelLmsUserCourse->metadata = json_encode($data);

		if ($modelLmsUserCourse->validate()) {
			$modelLmsUserCourse->save();
		} else {
			return 'Course subscription not updated';
		}

		return $modelLmsUserCourse->id;
	}
}
