<?php

use \yii\web\Request;

$params = array_merge(
    require __DIR__ . '/../../common/config/params.php',
    require __DIR__ . '/../../common/config/params-local.php',
    require __DIR__ . '/params.php',
    require __DIR__ . '/params-local.php'
);

$baseUrl = str_replace('/backend/web', '/admin', (new Request)->getBaseUrl());

return [
    'id' => 'app-backend',
    'name' => 'GeneSys Admin',
    'basePath' => dirname(__DIR__),
    'controllerNamespace' => 'backend\controllers',
    'bootstrap' => ['log'],
    'modules' => [
        'core' => [
            'isBackend' => true,
        ],
        'user' => [
            'isBackend' => true,
        ],
        'site' => [
            'isBackend' => true,
        ],
        'cms' => [
            'isBackend' => true,
        ],
        'shop' => [
            'isBackend' => true,
        ],
        'payment' => [
            'isBackend' => true,
        ],
        'logs' => [
            'isBackend' => true,
        ],
		'mailer' => [
            'isBackend' => true,
        ],
		'lms' => [
            'isBackend' => true,
        ],
        'inventory' => [
            'isBackend' => true,
        ],
        'accounting' => [
            'isBackend' => true,
        ],
        'affiliate' => [
            'isBackend' => true,
        ],
        'qms' => [
            'isBackend' => true,
        ],

        'notifications' => [
            'class' => 'webzop\notifications\Module',
            'channels' => [
                'genesys-screen' => [
                    'class' => 'common\channels\GenesysScreenChannel',
                    'enable' => true,
                ],
                'genesys-email' => [
                    'class' => 'common\channels\GenesysEmailChannel',
                    'enable' => false,
                ],
            ],
        ],
    ],
    'components' => [
        'request' => [
            'baseUrl' => $baseUrl,
            'cookieValidationKey' => 'lHylqZyR4mHR5rs0VEht',
            'csrfParam' => '_csrf-backend',
        ],
        'user' => [
            'identityClass' => 'common\modules\user\models\UserIdentity',
            'enableAutoLogin' => true,
            'identityCookie' => ['name' => '_identity-backend', 'httpOnly' => true],
        ],
        'session' => [
            // this is the name of the session cookie used for login on the backend
            'name' => 'advanced-backend',
        ],
        'log' => [
            'traceLevel' => YII_DEBUG ? 3 : 0,
            'targets' => [
                [
                    'class' => 'yii\log\FileTarget',
                    'levels' => ['error', 'warning'],
                ],
            ],
        ],
        'errorHandler' => [
            'errorAction' => 'site/site/error',
        ],
        'urlManager' => [
            'enablePrettyUrl' => true,
            'showScriptName' => false,
            'rules' => [
                '/' => 'site/site/index',
                '/site/login' => 'site/site/login',
            ],
        ],
        'view' => [
            'theme' => [
                'pathMap' => [
                    'backend/views' => '@vendor/hail812/yii2-adminlte3/src/views'
                ],
            ],
        ],
        'assetManager' => [
            'bundles' => [
                'wbraganca\dynamicform\DynamicFormAsset' => [
                    'sourcePath' => null,
                    'basePath' => '@webroot',
                    'baseUrl' => '@web',
                    'js' => [
                        'js/yii2-dynamic-form.js'
                    ],
                ],
                'kartik\select2\Select2Asset' => [
                    'sourcePath' => null,
                    'basePath' => '@webroot',
                    'baseUrl' => '@web',
                    'css' => [
                        'css/select2.css'
                    ],
                    'js' => [
                        'js/select2.full.js'
                    ],
                ],
            ]
        ]
    ],
    'params' => $params,
    'layoutPath' => '@common/modules/site/views/backend/layouts',
];
