<?php

namespace common\channels;

use Yii;
use yii\helpers\Inflector;
use webzop\notifications\Channel;
use webzop\notifications\Notification;

use common\modules\site\components\Email;
use common\modules\user\models\User;

class GenesysEmailChannel extends Channel
{

    public $enable = true;
    
    /**
     * Send the given notification.
     *
     * @param Notification $notification
     * @return void
     */
    public function send(Notification $notification)
    {
        // use $notification->getTitle() ou $notification->getDescription();
        // Send your notification in this channel...
        
        if ($this->enable === false) {
            return false;
        }
        
        if ($notification->user_id == 1) {
            return false;
        }

        if (empty(User::getAttr(['email'], ['id' => $notification->user_id]))) {
            return false;
        }

        return Email::send(
            'notification/notification-' . $notification->entity_class, 
            [
                'html' => 'notification/notification-' . $notification->entity_class . '-html', 
                'text' => 'notification/notification-' . $notification->entity_class . '-text'
            ], 
            [
                'to' => User::getAttr(['email'], ['id' => $notification->user_id]), 
                'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
            ], 
            'Notification - ' . Inflector::titleize($notification->entity_class, true), 
            [
                'data' => [
                    'user_id' => $notification->user_id, 
                    'entity_class' => $notification->entity_class, 
                    'entity_id' => $notification->entity_id, 
                    'app_alias' => $notification->app_alias, 
                    'model' => $notification->model, 
                    'message' => (string) $notification->getTitle(),
                    'route' => serialize($notification->getRoute()),
                    'notif' => $notification->notification_id,
                ]
            ], 
            $notification->app_alias, 
            (isset(Yii::$app->params['site_code']) && !empty(Yii::$app->params['site_code'])) ? Yii::$app->params['site_code'] : 'genesys'
        );
    }
}
