<?php

namespace common\components;

use Yii;
use yii\base\Component;
use yii\helpers\Url;
use MongoDB;
use linslin\yii2\curl;

class WhoisComponent extends Component 
{

    public static function lookup($domain = null)
    {
        $config = Yii::$app->getComponents();
        $collection = (new MongoDB\Client($config['mongodb']['dsn']))->dncorp2->whois;

        $curl = new curl\Curl();
        $whois_data = array();
        $url = '';

        $domain = trim($domain);
        $url = strpos($domain, 'http') !== 0 ? "http://$domain" : $domain;

        if (isset($domain) && !empty($domain) && filter_var($url, FILTER_VALIDATE_URL)) {
            if (Yii::$app->params['mode'] == 'live') {
                if ($collection->count(['domain' => $domain]) > 0) {  // check if whois-data locally available
                    $response = $collection->findOne(['domain' => $domain])->data;

                    $whois_data = json_decode($response, true);

                    if (isset($whois_data['status']) && !empty($whois_data['status']) && $whois_data['status'] == 1) {  // all good, proceed
                        // save the lookup-attempt locally
                        $collection->updateOne(
                            ['domain' => $domain], 
                            ['$set' => ['lookup' => intval($collection->findOne(['domain' => $domain])->lookup + 1), 'updated_at' => date('Y-m-d H:s:i')]]
                        );
                    } else {    // lookup failed for some reason; explicitly inserting domain-name
                        $whois_data['domain_name'] = $domain;
                    }
                } else {    // fetch whois-data through API if not available locally
                    $response = $curl->setGetParams([
                        'key' => Yii::$app->params['whoxy']['api_key'],
                        'whois' => $domain
                    ])->get('http://api.whoxy.com/');

                    $whois_data = json_decode($response, true);

                    if (isset($whois_data['status']) && !empty($whois_data['status']) && $whois_data['status'] != 0) {  // all good, proceed
                        // save the lookup locally
                        $collection->insertOne([
                            'domain' => $domain, 
                            'lookup' => 1, 
                            'data' => $response, 
                            'api' => 'whoxy', 
                            'funnel' => Yii::$app->params['site_code'], 
                            'created_at' => date('Y-md-d H:s:i'), 
                            'updated_at' => NULL, 
                        ]);
                    } else {    // lookup failed for some reason; explicitly inserting domain-name
                        $whois_data['domain_name'] = $domain;
                    }
                }
            } else {
                $whois_data = '{
                    "status": 1,
                    "domain_name": "thaivisa.com",
                    "query_time": "2017-09-09 17:20:18",
                    "whois_server": "whois.dynadot.com",
                    "domain_registered": "yes",
                    "create_date": "1998-06-12",
                    "update_date": "2017-07-30",
                    "expiry_date": "2021-06-11",
                    "domain_registrar": {
                        "iana_id": 472,
                        "registrar_name": "DYNADOT LLC",
                        "whois_server": "whois.dynadot.com",
                        "website_url": "http://www.dynadot.com",
                        "email_address": "abuse@dynadot.com",
                        "phone_number": "+1.6502620100"
                    },
                    "registrant_contact": {
                        "full_name": "Tessanet Limited c/o Dynadot",
                        "mailing_address": "PO Box 701",
                        "city_name": "San Mateo",
                        "state_name": "California",
                        "zip_code": "94401",
                        "country_name": "United States",
                        "country_code": "US",
                        "email_address": "privacy@dynadot.com",
                        "phone_number": "+1.6505854708"
                    },
                    "administrative_contact": {
                        "full_name": "Tessanet Limited c/o Dynadot",
                        "mailing_address": "PO Box 701",
                        "city_name": "San Mateo",
                        "state_name": "California",
                        "zip_code": "94401",
                        "country_name": "United States",
                        "country_code": "US",
                        "email_address": "privacy@dynadot.com",
                        "phone_number": "+1.6505854708"
                    },
                    "technical_contact": {
                        "full_name": "Tessanet Limited c/o Dynadot",
                        "mailing_address": "PO Box 701",
                        "city_name": "San Mateo",
                        "state_name": "California",
                        "zip_code": "94401",
                        "country_name": "United States",
                        "country_code": "US",
                        "email_address": "privacy@dynadot.com",
                        "phone_number": "+1.6505854708"
                    },
                    "name_servers": [
                        "1133.awsdns",
                        "1832.awsdns",
                        "397.awsdns",
                        "661.awsdns"
                    ],
                    "domain_status": [
                        "clientTransferProhibited"
                    ],
                    "raw_whois": "Domain Name: THAIVISA.COM\nRegistry Domain ID: 1411618_DOMAIN_COM-VRSN\nRegistrar WHOIS Server: whois.dynadot.com\nRegistrar URL: http://www.dynadot.com\nUpdated Date: 2017-07-30T11:39:23.0Z\nCreation Date: 1998-06-12T04:00:00.0Z\nRegistrar Registration Expiration Date: 2021-06-11T04:00:00.0Z\nRegistrar: DYNADOT LLC\nRegistrar IANA ID: 472\nRegistrar Abuse Contact Email: abuse@dynadot.com\nRegistrar Abuse Contact Phone: +1.6502620100\nDomain Status: clientTransferProhibited\nRegistry Registrant ID: \nRegistrant Name: Tessanet Limited c/o Dynadot\nRegistrant Street: PO Box 701\nRegistrant City: San Mateo\nRegistrant State/Province: California\nRegistrant Postal Code: 94401\nRegistrant Country: US\nRegistrant Phone: +1.6505854708\nRegistrant Email: privacy@dynadot.com\nRegistry Admin ID: \nAdmin Name: Tessanet Limited c/o Dynadot\nAdmin Street: PO Box 701\nAdmin City: San Mateo\nAdmin State/Province: California\nAdmin Postal Code: 94401\nAdmin Country: US\nAdmin Phone: +1.6505854708\nAdmin Email: privacy@dynadot.com\nRegistry Tech ID: \nTech Name: Tessanet Limited c/o Dynadot\nTech Street: PO Box 701\nTech City: San Mateo\nTech State/Province: California\nTech Postal Code: 94401\nTech Country: US\nTech Phone: +1.6505854708\nTech Email: privacy@dynadot.com\nName Server: ns-1133.awsdns-13.org\nName Server: ns-1832.awsdns-37.co.uk\nName Server: ns-397.awsdns-49.com\nName Server: ns-661.awsdns-18.net\nDNSSEC: unsigned\nURL of the ICANN WHOIS Data Problem Reporting System: http://wdprs.internic.net/\n>>> Last update of WHOIS database: 2017-09-09 10:17:36 -0700 <<<"
                }';

                $whois_data = json_decode($whois_data, true);
            }
        }

        return self::filter($whois_data);
    }

    public static function filter($whois_data = array())
    {
        $whois_data_filtered = array();

        /****** Domain ******/
        if (isset($whois_data['domain_name']) && !empty($whois_data['domain_name'])) {
            $whois_data_filtered['domain']['name'] = $whois_data['domain_name'];
        }

        /****** Registrant Contact ******/
        if (isset($whois_data['registrant_contact']) && !empty($whois_data['registrant_contact'])) {
            $whois_data_filtered['registrant_contact'] = $whois_data['registrant_contact'];
        } else {
            $whois_data_filtered['registrant_contact'] = [
                'name' => '', 
                'email' => '', 
                'address' => '', 
                'city' => '', 
                'state' => '', 
                'zip' => '', 
                'country' => '', 
                'phone' => '', 
            ];
        }

        /****** Administrative Contact ******/
        if (isset($whois_data['administrative_contact']) && !empty($whois_data['administrative_contact'])) {
            $whois_data_filtered['administrative_contact'] = $whois_data['administrative_contact'];
        }

        /****** Technical Contact ******/
        if (isset($whois_data['technical_contact']) && !empty($whois_data['technical_contact'])) {
            $whois_data_filtered['technical_contact'] = $whois_data['technical_contact'];
        }

        return $whois_data_filtered;
    }
}
