<?php
/**
 * This is the template for generating CRUD search class of the specified model.
 */

use yii\helpers\StringHelper;


/* @var $this yii\web\View */
/* @var $generator yii\gii\generators\crud\Generator */

$modelClass = StringHelper::basename($generator->modelClass);
$searchModelClass = StringHelper::basename($generator->searchModelClass);
if ($modelClass === $searchModelClass) {
    $modelAlias = $modelClass . 'Model';
}
$rules = $generator->generateSearchRules();
$labels = $generator->generateSearchLabels();
$searchAttributes = $generator->getSearchAttributes();
$searchConditions = $generator->generateSearchConditions();

if (isset($searchConditions[1])) {
    $searchConditions[1] = str_replace("'like', '", "'like', '" . $generator->modelClass::getTableSchema()->name . ".", $searchConditions[1]);
}

$search_conditions = $search_conditions_tmp = array();

$search_conditions = explode(',', str_replace("[", '', strstr(strstr($searchConditions[0], "]", true), "[")));

foreach ($search_conditions as $search_condition) {
    $tmp = explode(' => ', $search_condition);

    if (isset($tmp[0], $tmp[1])) {
        $search_conditions_tmp[trim($tmp[0])] = trim($tmp[1]);
    }
}

$search_conditions_str = '';

foreach ($search_conditions_tmp as $k => $v) {
    $k = str_replace("'", "", $k);
    $search_conditions_str .= "\n            ->andFilterWhere(['like', '" . $generator->modelClass::getTableSchema()->name . ".$k', $v])";
}

$search_conditions_str .= ';';

if (isset($searchConditions[1])) {
    $searchConditions[0] = str_replace(';', $search_conditions_str, $searchConditions[1]);
}

$searchConditions[0] = str_replace("->andFilterWhere(['like', 'status', \$this->status])", "", $searchConditions[0]);

unset($searchConditions[1]);

echo "<?php\n";
?>

namespace <?= StringHelper::dirname(ltrim($generator->searchModelClass, '\\')) ?>;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use <?= ltrim($generator->modelClass, '\\') . (isset($modelAlias) ? " as $modelAlias" : "") ?>;

/**
 * <?= $searchModelClass ?> represents the model behind the search form of `<?= $generator->modelClass ?>`.
 */
class <?= $searchModelClass ?> extends <?= isset($modelAlias) ? $modelAlias : $modelClass ?>

{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            <?= implode(",\n            ", $rules) ?>,
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
	<?php if (in_array('status', $searchAttributes)) { ?>
	$query = <?= isset($modelAlias) ? $modelAlias : $modelClass ?>::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	<?php } else { ?>
	$query = <?= isset($modelAlias) ? $modelAlias : $modelClass ?>::find();
	<?php } ?>

        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        <?= implode("\n        ", $searchConditions) ?>

        return $dataProvider;
    }
}
