<?php
/**
 * This is the template for generating CRUD search class of the specified model.
 */

use yii\helpers\Inflector;
use yii\helpers\StringHelper;


/* @var $this yii\web\View */
/* @var $generator yii\gii\generators\crud\Generator */

$modelClass = StringHelper::basename($generator->modelClass);
$searchModelClass = StringHelper::basename($generator->searchModelClass);
if ($modelClass === $searchModelClass) {
    $modelAlias = $modelClass . 'Model';
}
$rules = $generator->generateSearchRules();
$labels = $generator->generateSearchLabels();
$searchAttributes = $generator->getSearchAttributes();
$searchConditions = $generator->generateSearchConditions();

$columns = array();

if (($tableSchema = $generator->getTableSchema()) === false) {
    foreach ($generator->getColumnNames() as $name) {
        $columns[] = $name;
    }
} else {
    foreach ($tableSchema->columns as $column) {
        $columns[] = $column->name;
    }
}

echo "<?php\n";
?>

namespace <?= StringHelper::dirname(ltrim($generator->searchModelClass, '\\')) ?>;

use Yii;

/**
 * Our data model extends yii\base\Model class so we can get easy to use and yet 
 * powerful Yii 2 validation mechanism.
 */
class <?= $searchModelClass ?> extends \yii\base\Model
{
    /**
     * We plan to get two columns in our grid that can be filtered.
     * Add more if required. You don't have to add all of them.
     */
    <?php 
    foreach ($columns as $column) {
        if ($column == 'id') {
            echo "public \$$column;\n";

            continue;
        }

        if ($column == 'status') {
            continue;
        }

        if ($column == 'metadata' && isset(Yii::$app->params[Inflector::camel2id(StringHelper::basename($generator->viewPath)) . '_metadata'])) {
            foreach (Yii::$app->params[Inflector::camel2id(StringHelper::basename($generator->viewPath)) . '_metadata'] as $key => $value) {
                echo "    public \${$column}_$key;\n";
            }

            continue;
        }

        echo "    public \$$column;\n";
    }
    ?>

    /**
     * Here we can define validation rules for each filtered column.
     * See http://www.yiiframework.com/doc-2.0/guide-input-validation.html
     * for more information about validation.
     */
    public function rules()
    {
        return [
            [[<?php foreach ($columns as $column) {
                if ($column == 'status') {
                    continue;
                }
                
                if ($column == 'metadata' && isset(Yii::$app->params[Inflector::camel2id(StringHelper::basename($generator->viewPath)) . '_metadata'])) {
                    foreach (Yii::$app->params[Inflector::camel2id(StringHelper::basename($generator->viewPath)) . '_metadata'] as $key => $value) {
                        echo "'{$column}_$key', ";
                    }

                    continue;
                }

                echo "'$column', ";
            } ?>], 'string'],
            // our columns are just simple string, nothing fancy
        ];
    }

    /**
     * In this example we keep this special property to know if columns should be 
     * filtered or not. See search() method below.
     */
    private $_filtered = false;

    /**
     * This method returns ArrayDataProvider.
     * Filtered and sorted if required.
     */
    public function search($params)
    {
        /**
         * $params is the array of GET parameters passed in the actionExample().
         * These are being loaded and validated.
         * If validation is successful _filtered property is set to true to prepare
         * data source. If not - data source is displayed without any filtering.
         */
        if ($this->load($params) && $this->validate()) {
            $this->_filtered = true;
        }

        return new \yii\data\ArrayDataProvider([
            // ArrayDataProvider here takes the actual data source
            'allModels' => $this->getData(),
            'key' => 'id',
            'sort' => [
                // we want our columns to be sortable:
                'attributes' => [<?php foreach ($columns as $column) {
                if ($column == 'status') {
                    continue;
                }
                
                if ($column == 'metadata' && isset(Yii::$app->params[Inflector::camel2id(StringHelper::basename($generator->viewPath)) . '_metadata'])) {
                    foreach (Yii::$app->params[Inflector::camel2id(StringHelper::basename($generator->viewPath)) . '_metadata'] as $key => $value) {
                        echo "'{$column}_$key', ";
                    }

                    continue;
                }

                echo "'$column', ";
            } ?>],
                'defaultOrder' => ['id' => SORT_DESC],
            ],
        ]);
    }

    /**
     * Here we are preparing the data source and applying the filters
     * if _filtered property is set to true.
     */
    protected function getData()
    {
        $data = <?= isset($modelAlias) ? $modelAlias : $modelClass ?>::find()->where(['status' => 'active'])->asArray()->all();

        <?php if ($column == 'metadata') { ?>
        foreach ($data as $k => $v) {
            $json_arr = json_decode($v['metadata'], true);

            foreach (Yii::$app->params['<?= Inflector::camel2id(StringHelper::basename($generator->viewPath)) ?>_metadata'] as $key => $value) {
                $data[$k]['metadata_' . $key] = null;
            }

            if (!is_array($json_arr)) {
                continue;
            }

            foreach ($json_arr as $_k => $_v) {
                $data[$k]['metadata_' . $_k] = $_v;
            }
        }
        <?php } ?>

        if ($this->_filtered) {
            $data = array_filter($data, function ($value) {
                $conditions = [true];

                <?php 
                foreach ($columns as $column) {
                    if ($column == 'status') {
                        continue;
                    }
                    
                    if ($column == 'metadata' && isset(Yii::$app->params[Inflector::camel2id(StringHelper::basename($generator->viewPath)) . '_metadata'])) {
                        foreach (Yii::$app->params[Inflector::camel2id(StringHelper::basename($generator->viewPath)) . '_metadata'] as $key => $value) {
                ?>
if (!empty($this-><?= $column ?>_<?= $key ?>)) {
                    $conditions[] = stripos($value['<?= $column ?>_<?= $key ?>'], $this-><?= $column ?>_<?= $key ?>) !== false;
                }

                <?php
                        }

                        continue;
                    }

                    if ($column == 'created_by') {
                ?>if (!empty($this->created_by)) {
                    $conditions[] = stripos(\common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $value['created_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $value['created_by']]), $this->created_by) !== false;
                }

                <?php

                        continue;
                    }

                    if ($column == 'updated_by') {
                ?>if (!empty($this->updated_by)) {
                    $conditions[] = stripos(\common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $value['updated_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $value['updated_by']]), $this->updated_by) !== false;
                }

                <?php

                        continue;
                    }

                ?>if (!empty($this-><?= $column ?>)) {
                    $conditions[] = stripos($value['<?= $column ?>'], $this-><?= $column ?>) !== false;
                }

                <?php
                } 
                ?>return array_product($conditions);
            });
        }

        return $data;
    }
}