<?php
/**
 * This is the template for generating the model class of a specified table.
 */

/* @var $this yii\web\View */
/* @var $generator yii\gii\generators\model\Generator */
/* @var $tableName string full table name */
/* @var $className string class name */
/* @var $queryClassName string query class name */
/* @var $tableSchema yii\db\TableSchema */
/* @var $properties array list of properties (property => [type, name. comment]) */
/* @var $labels string[] list of attribute labels (name => label) */
/* @var $rules string[] list of validation rules */
/* @var $relations array list of relations (name => relation declaration) */

$modelClassNames = (array) \Yii::$app->getModules()['gii']->generators['model'];
$modelClassNames = array_flip($modelClassNames[chr(0) . '*' . chr(0) . 'classNames']);

foreach ($relations as $name => $relation) {
    if (strpos($relation[0], "::") !== false) {
        $modelClassName = str_replace('(', '', strstr(strstr($relation[0], "::", true), "("));
        $moduleName = explode('_', $modelClassNames[$modelClassName]);

        if ($modelClassName == 'AssetManager') {
            $moduleName = implode('-', [$moduleName[1], $moduleName[2]]);

            $modelClass = str_replace('controllers', 'models', \Yii::$app->getModule($moduleName)->controllerNamespace);
        } else {
            $moduleName = $moduleName[1];

            $modelClass = str_replace('controllers\backend', 'models', \Yii::$app->getModule($moduleName)->controllerNamespace);
        }

        if (strpos($relation[0], "$modelClassName::") !== false) {
            $relations[$name][0] = str_replace("$modelClassName::", "\\$modelClass\\$modelClassName::", $relation[0]);
        }
    }
}

foreach ($rules as $key => $value) {
    if (strpos($value, "'targetClass' => ") !== false) {
        $modelClassName = str_replace("'targetClass' => ", '', strstr(strstr($value, "::", true), "'targetClass' => "));
        $moduleName = explode('_', $modelClassNames[$modelClassName]);

        if ($modelClassName == 'AssetManager') {
            $moduleName = implode('-', [$moduleName[1], $moduleName[2]]);

            $modelClass = str_replace('controllers', 'models', \Yii::$app->getModule($moduleName)->controllerNamespace);
        } else {
            $moduleName = $moduleName[1];

            $modelClass = str_replace('controllers\backend', 'models', \Yii::$app->getModule($moduleName)->controllerNamespace);
        }

        if (strpos($value, "$modelClassName::") !== false) {
            $rules[$key] = str_replace("$modelClassName::", "\\$modelClass\\$modelClassName::", $value);
        }
    }
}

foreach ($rules as $key => $value) {
    if (strpos($value, ", 'created_by', 'updated_by', 'created_at', 'updated_at'") !== false) {
        $rules[$key] = str_replace(", 'created_by', 'updated_by', 'created_at', 'updated_at'", "", $value);
    } else if (strpos($value, "'created_by', 'updated_by', 'created_at', 'updated_at'") !== false) {
        $rules[$key] = str_replace("'created_by', 'updated_by', 'created_at', 'updated_at'", "", $value);
    }
}

foreach ($rules as $key => $value) {
    if (strpos($value, ", 'created_at', 'updated_at'") !== false) {
        $rules[$key] = str_replace(", 'created_at', 'updated_at'", "", $value);
    } else if (strpos($value, "'created_by', 'updated_by', 'created_at', 'updated_at'") !== false) {
        $rules[$key] = str_replace("'created_by', 'updated_by', 'created_at', 'updated_at'", "", $value);
    }
}

echo "<?php\n";
?>

namespace <?= $generator->ns ?>;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "<?= $generator->generateTableName($tableName) ?>".
 *
<?php foreach ($properties as $property => $data): ?>
 * @property <?= "{$data['type']} \${$property}"  . ($data['comment'] ? ' ' . strtr($data['comment'], ["\n" => ' ']) : '') . "\n" ?>
<?php endforeach; ?>
<?php if (!empty($relations)): ?>
 *
<?php foreach ($relations as $name => $relation): ?>
 * @property <?= $relation[1] . ($relation[2] ? '[]' : '') . ' $' . lcfirst($name) . "\n" ?>
<?php endforeach; ?>
<?php endif; ?>
 */
class <?= $className ?> extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '<?= $generator->generateTableName($tableName) ?>';
    }
<?php if ($generator->db !== 'db'): ?>

    /**
     * @return \yii\db\Connection the database connection used by this AR class.
     */
    public static function getDb()
    {
        return Yii::$app->get('<?= $generator->db ?>');
    }
<?php endif; ?>

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [<?= empty($rules) ? '' : ("\n            " . implode(",\n            ", $rules) . ",\n        ") ?>];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
<?php foreach ($labels as $name => $label): ?>
            <?= "'$name' => " . $generator->generateString($label) . ",\n" ?>
<?php endforeach; ?>
        ];
    }
<?php foreach ($relations as $name => $relation): ?>

    /**
     * @return \yii\db\ActiveQuery
     */
    public function get<?= $name ?>()
    {
        <?= $relation[0] . "\n" ?>
    }
<?php endforeach; ?>
<?php if ($queryClassName): ?>
<?php
    $queryClassFullName = ($generator->ns === $generator->queryNs) ? $queryClassName : '\\' . $generator->queryNs . '\\' . $queryClassName;
    echo "\n";
?>
    /**
     * {@inheritdoc}
     * @return <?= $queryClassFullName ?> the active query used by this AR class.
     */
    public static function find()
    {
        return new <?= $queryClassFullName ?>(get_called_class());
    }
<?php endif; ?>
}