<?php

namespace common\modules\accounting\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

use common\modules\accounting\models\AccountingDocument;
use common\modules\accounting\models\AccountingDocumentSearch;
use common\modules\accounting\models\AccountingDocumentItem;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * DocumentController implements the CRUD actions for AccountingDocument model.
 */
class DocumentController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all AccountingDocument models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new AccountingDocumentSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single AccountingDocument model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new AccountingDocument model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new AccountingDocument();
        $modelAccountingDocumentItem = [new AccountingDocumentItem];
        $data = array();

        if (Yii::$app->request->isPost) {
            if ($document_id = $model->saveDocument(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $document_id]);   
            }
        }

        $data['model'] = $model;
        $data['modelAccountingDocumentItem'] = (empty($modelAccountingDocumentItem)) ? [new AccountingDocumentItem] : $modelAccountingDocumentItem;

        return $this->render('create', [
            'data' => $data,
        ]);
    }

    /**
     * Updates an existing AccountingDocument model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $modelAccountingDocumentItem = [new AccountingDocumentItem];
        $data = array();

        $modelAccountingDocumentItem = AccountingDocumentItem::findAll(['document_id' => $id, 'status' => 'active']);
        
        if (Yii::$app->request->isPost) {
            if ($document_id = $model->saveDocument(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $document_id]);   
            }
        }

        $data['model'] = $model;
        $data['modelAccountingDocumentItem'] = (empty($modelAccountingDocumentItem)) ? [new AccountingDocumentItem] : $modelAccountingDocumentItem;

        return $this->render('update', [
            'data' => $data,
        ]);
    }

    /**
     * Deletes an existing AccountingDocument model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the AccountingDocument model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return AccountingDocument the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = AccountingDocument::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
