<?php

namespace common\modules\accounting\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%accounting_account}}".
 *
 * @property int $id
 * @property string $name
 * @property string $code
 * @property string $type
 * @property int $account_id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property AccountingAccount $account
 * @property AccountingAccount[] $accountingAccounts
 * @property AccountingLedger[] $accountingLedgers
 */
class AccountingAccount extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%accounting_account}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'code'], 'required'],
            [['type', 'metadata', 'status'], 'string'],
            [['account_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['name', 'code'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['account_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingAccount::className(), 'targetAttribute' => ['account_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'code' => 'Code',
            'type' => 'Type',
            'account_id' => 'Account ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccount()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingAccount::className(), ['id' => 'account_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingAccounts()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingAccount::className(), ['account_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingLedgers()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingLedger::className(), ['account_id' => 'id']);
    }
}