<?php

namespace common\modules\accounting\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\accounting\models\AccountingContact;

/**
 * AccountingContactSearch represents the model behind the search form of `common\modules\accounting\models\AccountingContact`.
 */
class AccountingContactSearch extends AccountingContact
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'legal_entity_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'email', 'phone', 'address', 'city', 'state', 'country', 'zip_code', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = AccountingContact::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_accounting_contact.name', $this->name])
            ->andFilterWhere(['like', 'mod_accounting_contact.email', $this->email])
            ->andFilterWhere(['like', 'mod_accounting_contact.phone', $this->phone])
            ->andFilterWhere(['like', 'mod_accounting_contact.address', $this->address])
            ->andFilterWhere(['like', 'mod_accounting_contact.city', $this->city])
            ->andFilterWhere(['like', 'mod_accounting_contact.state', $this->state])
            ->andFilterWhere(['like', 'mod_accounting_contact.country', $this->country])
            ->andFilterWhere(['like', 'mod_accounting_contact.zip_code', $this->zip_code])
            ->andFilterWhere(['like', 'mod_accounting_contact.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_accounting_contact.status', $this->status])
            ->andFilterWhere(['like', 'mod_accounting_contact.id', $this->id])
            ->andFilterWhere(['like', 'mod_accounting_contact.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_accounting_contact.legal_entity_id', $this->legal_entity_id])
            ->andFilterWhere(['like', 'mod_accounting_contact.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_accounting_contact.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_accounting_contact.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_accounting_contact.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
