<?php

namespace common\modules\accounting\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\accounting\models\AccountingCurrency;

/**
 * AccountingCurrencySearch represents the model behind the search form of `common\modules\accounting\models\AccountingCurrency`.
 */
class AccountingCurrencySearch extends AccountingCurrency
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'code', 'symbol', 'symbol_position', 'rate', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = AccountingCurrency::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_accounting_currency.name', $this->name])
            ->andFilterWhere(['like', 'mod_accounting_currency.code', $this->code])
            ->andFilterWhere(['like', 'mod_accounting_currency.symbol', $this->symbol])
            ->andFilterWhere(['like', 'mod_accounting_currency.symbol_position', $this->symbol_position])
            ->andFilterWhere(['like', 'mod_accounting_currency.rate', $this->rate])
            ->andFilterWhere(['like', 'mod_accounting_currency.status', $this->status])
            ->andFilterWhere(['like', 'mod_accounting_currency.id', $this->id])
            ->andFilterWhere(['like', 'mod_accounting_currency.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_accounting_currency.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_accounting_currency.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_accounting_currency.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
