<?php

namespace common\modules\accounting\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%accounting_document}}".
 *
 * @property int $id
 * @property int $document_type_id
 * @property string $number
 * @property string $date
 * @property string $description
 * @property string $amount
 * @property int $contact_id
 * @property int $legal_entity_id
 * @property int $currency_id
 * @property string $currency_rate
 * @property int $order_id
 * @property string $order_number
 * @property int $voucher_id
 * @property string $discount
 * @property string $notes
 * @property string $comments
 * @property string $remarks
 * @property int $asset_id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property AccountingContact $contact
 * @property AccountingLegalEntity $legalEntity
 * @property AccountingCurrency $currency
 * @property ShopOrder $order
 * @property ShopVoucher $voucher
 * @property AssetManager $asset
 * @property AccountingDocumentType $documentType
 * @property AccountingDocumentItem[] $accountingDocumentItems
 * @property AccountingTransaction[] $accountingTransactions
 * @property InventoryItem[] $inventoryItems
 */
class AccountingDocument extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%accounting_document}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['document_type_id', 'contact_id', 'legal_entity_id', 'currency_id', 'order_id', 'voucher_id', 'asset_id', 'created_by', 'updated_by'], 'integer'],
            [['date'], 'required'],
            [['date'], 'safe'],
            [['notes', 'comments', 'remarks', 'metadata', 'status'], 'string'],
            [['number', 'description', 'amount', 'currency_rate', 'order_number', 'discount'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['contact_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingContact::className(), 'targetAttribute' => ['contact_id' => 'id']],
            [['legal_entity_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingLegalEntity::className(), 'targetAttribute' => ['legal_entity_id' => 'id']],
            [['currency_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingCurrency::className(), 'targetAttribute' => ['currency_id' => 'id']],
            [['order_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopOrder::className(), 'targetAttribute' => ['order_id' => 'id']],
            [['voucher_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopVoucher::className(), 'targetAttribute' => ['voucher_id' => 'id']],
            [['asset_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['asset_id' => 'id']],
            [['document_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingDocumentType::className(), 'targetAttribute' => ['document_type_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'document_type_id' => 'Document Type ID',
            'number' => 'Number',
            'date' => 'Date',
            'description' => 'Description',
            'amount' => 'Amount',
            'contact_id' => 'Contact ID',
            'legal_entity_id' => 'Legal Entity ID',
            'currency_id' => 'Currency ID',
            'currency_rate' => 'Currency Rate',
            'order_id' => 'Order ID',
            'order_number' => 'Order Number',
            'voucher_id' => 'Voucher ID',
            'discount' => 'Discount',
            'notes' => 'Notes',
            'comments' => 'Comments',
            'remarks' => 'Remarks',
            'asset_id' => 'Asset ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getContact()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingContact::className(), ['id' => 'contact_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLegalEntity()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingLegalEntity::className(), ['id' => 'legal_entity_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCurrency()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingCurrency::className(), ['id' => 'currency_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrder()
    {
        return $this->hasOne(\common\modules\shop\models\ShopOrder::className(), ['id' => 'order_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getVoucher()
    {
        return $this->hasOne(\common\modules\shop\models\ShopVoucher::className(), ['id' => 'voucher_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAsset()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'asset_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDocumentType()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingDocumentType::className(), ['id' => 'document_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingDocumentItems()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingDocumentItem::className(), ['document_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingTransactions()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingTransaction::className(), ['document_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryItems()
    {
        return $this->hasMany(\common\modules\inventory\models\InventoryItem::className(), ['bill' => 'id']);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                if (AccountingDocumentItem::find()->where(['document_id' => $this->id, 'status' => 'active'])->exists()) {
                    $modelAccountingDocumentItem = AccountingDocumentItem::find()->where(['document_id' => $this->id, 'status' => 'active'])->all();

                    foreach ($modelAccountingDocumentItem as $key => $value) {
                        $value->softDelete();
                    }
                }

                $transaction->commit();
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveDocument($post_data, $document_id = 0)
    {
        $session = Yii::$app->session;

        $modelAccountingDocument = new AccountingDocument();
        $modelAccountingDocumentItem = new AccountingDocumentItem();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();
    
        try {
            if (isset($document_id) && $document_id > 0 && AccountingDocument::find()->where(['id' => $document_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelAccountingDocument = new AccountingDocument();

                $modelAccountingDocument = AccountingDocument::findOne(['id' => $document_id, 'status' => 'active']);

                $modelAccountingDocument->load($post_data);

                $modelAccountingDocument->metadata = isset($post_data['AccountingDocument']['AccountingDocumentMetadata']) ? Globals::setJson($post_data['AccountingDocument']['AccountingDocumentMetadata']) : '';
                $modelAccountingDocument->updated_by = self::setUpdatedBy($post_data['AccountingDocument']);
            } else {    // Create a new model
                $modelAccountingDocument = new AccountingDocument();

                $modelAccountingDocument->load($post_data);

                $modelAccountingDocument->metadata = isset($post_data['AccountingDocument']['AccountingDocumentMetadata']) ? Globals::setJson($post_data['AccountingDocument']['AccountingDocumentMetadata']) : '';
                $modelAccountingDocument->created_by = self::setCreatedBy($post_data['AccountingDocument']);
                $modelAccountingDocument->updated_by = self::setUpdatedBy($post_data['AccountingDocument']);
                $modelAccountingDocument->status = 'active';
            }

            if ($modelAccountingDocument->validate()) {
                $modelAccountingDocument->save();

                foreach (AccountingDocumentItem::findAll(['document_id' => $modelAccountingDocument->id, 'status' => 'active']) as $documentItemKey => $documentItemValue) {
                    $documentItemValue->softDelete();
                }

                foreach ($post_data['AccountingDocumentItem'] as $documentItemKey => $documentItemValue) {
                    if (isset($documentItemValue['id']) && $documentItemValue['id'] > 0 && AccountingDocumentItem::find()->where(['id' => $documentItemValue['id'], 'status' => 'active'])->exists()) {   // Model already exists
                        $modelAccountingDocumentItem = new AccountingDocumentItem();

                        $modelAccountingDocumentItem = AccountingDocumentItem::findOne(['id' => $documentItemValue['id'], 'status' => 'active']);

                        $modelAccountingDocumentItem->document_id = $modelAccountingDocument->id;
                        $modelAccountingDocumentItem->price = Globals::formatNumber($documentItemValue['price'], ['round' => true, 'precision' => 2]);
                        $modelAccountingDocumentItem->tax = Globals::formatNumber($documentItemValue['tax'], ['round' => true, 'precision' => 2]);

                        if (strpos($documentItemValue['discount'], '%') === false) {
                            $modelAccountingDocumentItem->discount = Globals::formatNumber($documentItemValue['discount'], ['round' => true, 'precision' => 2]);
                        }

                        $modelAccountingDocumentItem->total = Globals::formatNumber($documentItemValue['total'], ['round' => true, 'precision' => 2]);
                        $modelAccountingDocumentItem->metadata = isset($post_data['AccountingDocumentItem']['AccountingDocumentItemMetadata']) ? Globals::setJson($post_data['AccountingDocumentItem']['AccountingDocumentItemMetadata']) : '';
                        $modelAccountingDocumentItem->updated_by = $modelAccountingDocument->updated_by;
                        $modelAccountingDocumentItem->status = 'active';
                    } else {    // Create a new model
                        $modelAccountingDocumentItem = new AccountingDocumentItem();

                        $modelAccountingDocumentItem->load($documentItemValue, '');
                        
                        $modelAccountingDocumentItem->document_id = $modelAccountingDocument->id;
                        $modelAccountingDocumentItem->price = Globals::formatNumber($documentItemValue['price'], ['round' => true, 'precision' => 2]);
                        $modelAccountingDocumentItem->tax = Globals::formatNumber($documentItemValue['tax'], ['round' => true, 'precision' => 2]);

                        if (strpos($documentItemValue['discount'], '%') === false) {
                            $modelAccountingDocumentItem->discount = Globals::formatNumber($documentItemValue['discount'], ['round' => true, 'precision' => 2]);
                        }

                        $modelAccountingDocumentItem->total = Globals::formatNumber($documentItemValue['total'], ['round' => true, 'precision' => 2]);
                        $modelAccountingDocumentItem->metadata = isset($post_data['AccountingDocumentItem']['AccountingDocumentItemMetadata']) ? Globals::setJson($post_data['AccountingDocumentItem']['AccountingDocumentItemMetadata']) : '';
                        $modelAccountingDocumentItem->created_by = $modelAccountingDocument->created_by;
                        $modelAccountingDocumentItem->updated_by = $modelAccountingDocument->updated_by;
                        $modelAccountingDocumentItem->status = 'active';
                    }

                    if ($modelAccountingDocumentItem->validate()) {
                        $modelAccountingDocumentItem->save();
                    } else {
                        $breakFlag = true;

                        Yii::$app->session->setFlash('error', "Unable to save document item " . Globals::renderErrors($modelAccountingDocumentItem->getErrors()));

                        break;
                    }
                }
            } else {
                $breakFlag = true;

                Yii::$app->session->setFlash('error', "Unable to save document: " . Globals::renderErrors($modelAccountingDocument->getErrors()));
            }

            if (!$breakFlag) {
                $transaction->commit();

                Yii::$app->session->setFlash('success', "Document saved successfully.");

                return $modelAccountingDocument->id;
            } else {
                $transaction->rollBack();

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}