<?php

namespace common\modules\accounting\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%accounting_document_item}}".
 *
 * @property int $id
 * @property int $document_id
 * @property int $item_id
 * @property int $product_id
 * @property int $order_item_id
 * @property string $quantity
 * @property string $price
 * @property string $tax
 * @property string $discount
 * @property string $total
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property AccountingDocument $document
 * @property InventoryItem $item
 * @property ShopProduct $product
 * @property ShopOrderItem $orderItem
 */
class AccountingDocumentItem extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%accounting_document_item}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['document_id'], 'required'],
            [['document_id', 'item_id', 'product_id', 'order_item_id', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['quantity', 'price', 'tax', 'discount', 'total'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['document_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingDocument::className(), 'targetAttribute' => ['document_id' => 'id']],
            [['item_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\inventory\models\InventoryItem::className(), 'targetAttribute' => ['item_id' => 'id']],
            [['product_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopProduct::className(), 'targetAttribute' => ['product_id' => 'id']],
            [['order_item_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopOrderItem::className(), 'targetAttribute' => ['order_item_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'document_id' => 'Document ID',
            'item_id' => 'Item ID',
            'product_id' => 'Product ID',
            'order_item_id' => 'Order Item ID',
            'quantity' => 'Quantity',
            'price' => 'Price',
            'tax' => 'Tax',
            'discount' => 'Discount',
            'total' => 'Total',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDocument()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingDocument::className(), ['id' => 'document_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getItem()
    {
        return $this->hasOne(\common\modules\inventory\models\InventoryItem::className(), ['id' => 'item_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProduct()
    {
        return $this->hasOne(\common\modules\shop\models\ShopProduct::className(), ['id' => 'product_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrderItem()
    {
        return $this->hasOne(\common\modules\shop\models\ShopOrderItem::className(), ['id' => 'order_item_id']);
    }
}