<?php

namespace common\modules\accounting\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\accounting\models\AccountingDocument;

/**
 * AccountingDocumentSearch represents the model behind the search form of `common\modules\accounting\models\AccountingDocument`.
 */
class AccountingDocumentSearch extends AccountingDocument
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'document_type_id', 'contact_id', 'legal_entity_id', 'currency_id', 'order_id', 'voucher_id', 'asset_id', 'created_by', 'updated_by'], 'integer'],
            [['number', 'date', 'description', 'amount', 'currency_rate', 'order_number', 'discount', 'notes', 'comments', 'remarks', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = AccountingDocument::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_accounting_document.number', $this->number])
            ->andFilterWhere(['like', 'mod_accounting_document.description', $this->description])
            ->andFilterWhere(['like', 'mod_accounting_document.amount', $this->amount])
            ->andFilterWhere(['like', 'mod_accounting_document.currency_rate', $this->currency_rate])
            ->andFilterWhere(['like', 'mod_accounting_document.order_number', $this->order_number])
            ->andFilterWhere(['like', 'mod_accounting_document.discount', $this->discount])
            ->andFilterWhere(['like', 'mod_accounting_document.notes', $this->notes])
            ->andFilterWhere(['like', 'mod_accounting_document.comments', $this->comments])
            ->andFilterWhere(['like', 'mod_accounting_document.remarks', $this->remarks])
            ->andFilterWhere(['like', 'mod_accounting_document.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_accounting_document.status', $this->status])
            ->andFilterWhere(['like', 'mod_accounting_document.id', $this->id])
            ->andFilterWhere(['like', 'mod_accounting_document.document_type_id', $this->document_type_id])
            ->andFilterWhere(['like', 'mod_accounting_document.date', $this->date])
            ->andFilterWhere(['like', 'mod_accounting_document.contact_id', $this->contact_id])
            ->andFilterWhere(['like', 'mod_accounting_document.legal_entity_id', $this->legal_entity_id])
            ->andFilterWhere(['like', 'mod_accounting_document.currency_id', $this->currency_id])
            ->andFilterWhere(['like', 'mod_accounting_document.order_id', $this->order_id])
            ->andFilterWhere(['like', 'mod_accounting_document.voucher_id', $this->voucher_id])
            ->andFilterWhere(['like', 'mod_accounting_document.asset_id', $this->asset_id])
            ->andFilterWhere(['like', 'mod_accounting_document.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_accounting_document.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_accounting_document.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_accounting_document.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
