<?php

namespace common\modules\accounting\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%accounting_ledger}}".
 *
 * @property int $id
 * @property int $account_id
 * @property int $transaction_id
 * @property int $journal_entry_id
 * @property string $entry_type
 * @property string $amount
 * @property int $legal_entity_id
 * @property int $contact_id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property AccountingAccount $account
 * @property AccountingTransaction $transaction
 * @property AccountingJournalEntry $journalEntry
 * @property AccountingLegalEntity $legalEntity
 * @property AccountingContact $contact
 */
class AccountingLedger extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%accounting_ledger}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['account_id', 'transaction_id', 'journal_entry_id', 'entry_type', 'amount'], 'required'],
            [['account_id', 'transaction_id', 'journal_entry_id', 'legal_entity_id', 'contact_id', 'created_by', 'updated_by'], 'integer'],
            [['entry_type', 'metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['amount'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['account_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingAccount::className(), 'targetAttribute' => ['account_id' => 'id']],
            [['transaction_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingTransaction::className(), 'targetAttribute' => ['transaction_id' => 'id']],
            [['journal_entry_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingJournalEntry::className(), 'targetAttribute' => ['journal_entry_id' => 'id']],
            [['legal_entity_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingLegalEntity::className(), 'targetAttribute' => ['legal_entity_id' => 'id']],
            [['contact_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingContact::className(), 'targetAttribute' => ['contact_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'account_id' => 'Account ID',
            'transaction_id' => 'Transaction ID',
            'journal_entry_id' => 'Journal Entry ID',
            'entry_type' => 'Entry Type',
            'amount' => 'Amount',
            'legal_entity_id' => 'Legal Entity ID',
            'contact_id' => 'Contact ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccount()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingAccount::className(), ['id' => 'account_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTransaction()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingTransaction::className(), ['id' => 'transaction_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getJournalEntry()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingJournalEntry::className(), ['id' => 'journal_entry_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLegalEntity()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingLegalEntity::className(), ['id' => 'legal_entity_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getContact()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingContact::className(), ['id' => 'contact_id']);
    }
}