<?php

namespace common\modules\accounting\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\accounting\models\AccountingLedger;

/**
 * AccountingLedgerSearch represents the model behind the search form of `common\modules\accounting\models\AccountingLedger`.
 */
class AccountingLedgerSearch extends AccountingLedger
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'account_id', 'transaction_id', 'journal_entry_id', 'legal_entity_id', 'contact_id', 'created_by', 'updated_by'], 'integer'],
            [['entry_type', 'amount', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = AccountingLedger::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_accounting_ledger.entry_type', $this->entry_type])
            ->andFilterWhere(['like', 'mod_accounting_ledger.amount', $this->amount])
            ->andFilterWhere(['like', 'mod_accounting_ledger.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_accounting_ledger.status', $this->status])
            ->andFilterWhere(['like', 'mod_accounting_ledger.id', $this->id])
            ->andFilterWhere(['like', 'mod_accounting_ledger.account_id', $this->account_id])
            ->andFilterWhere(['like', 'mod_accounting_ledger.transaction_id', $this->transaction_id])
            ->andFilterWhere(['like', 'mod_accounting_ledger.journal_entry_id', $this->journal_entry_id])
            ->andFilterWhere(['like', 'mod_accounting_ledger.legal_entity_id', $this->legal_entity_id])
            ->andFilterWhere(['like', 'mod_accounting_ledger.contact_id', $this->contact_id])
            ->andFilterWhere(['like', 'mod_accounting_ledger.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_accounting_ledger.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_accounting_ledger.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_accounting_ledger.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
