<?php

namespace common\modules\accounting\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%accounting_legal_entity}}".
 *
 * @property int $id
 * @property string $name
 * @property int $business_type_id
 * @property string $cin
 * @property string $tin
 * @property string $address
 * @property string $city
 * @property string $state
 * @property string $country
 * @property string $zip_code
 * @property string $email
 * @property string $phone
 * @property string $website
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property AccountingContact[] $accountingContacts
 * @property AccountingDocument[] $accountingDocuments
 * @property AccountingLedger[] $accountingLedgers
 * @property User $createdBy
 * @property User $updatedBy
 * @property AccountingBusinessType $businessType
 * @property InventoryManufacturer[] $inventoryManufacturers
 * @property InventorySupplier[] $inventorySuppliers
 * @property InventoryWarehouse[] $inventoryWarehouses
 */
class AccountingLegalEntity extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%accounting_legal_entity}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'city', 'country'], 'required'],
            [['business_type_id', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['name', 'cin', 'tin', 'address', 'city', 'state', 'country', 'zip_code', 'email', 'phone', 'website'], 'string', 'max' => 255],
            [['name'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['business_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingBusinessType::className(), 'targetAttribute' => ['business_type_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'business_type_id' => 'Business Type ID',
            'cin' => 'Cin',
            'tin' => 'Tin',
            'address' => 'Address',
            'city' => 'City',
            'state' => 'State',
            'country' => 'Country',
            'zip_code' => 'Zip Code',
            'email' => 'Email',
            'phone' => 'Phone',
            'website' => 'Website',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingContacts()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingContact::className(), ['legal_entity_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingDocuments()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingDocument::className(), ['legal_entity_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingLedgers()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingLedger::className(), ['legal_entity_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBusinessType()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingBusinessType::className(), ['id' => 'business_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryManufacturers()
    {
        return $this->hasMany(\common\modules\inventory\models\InventoryManufacturer::className(), ['legal_entity_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventorySuppliers()
    {
        return $this->hasMany(\common\modules\inventory\models\InventorySupplier::className(), ['legal_entity_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryWarehouses()
    {
        return $this->hasMany(\common\modules\inventory\models\InventoryWarehouse::className(), ['legal_entity_id' => 'id']);
    }
}