<?php

namespace common\modules\accounting\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\accounting\models\AccountingLegalEntity;

/**
 * AccountingLegalEntitySearch represents the model behind the search form of `common\modules\accounting\models\AccountingLegalEntity`.
 */
class AccountingLegalEntitySearch extends AccountingLegalEntity
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'business_type_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'cin', 'tin', 'address', 'city', 'state', 'country', 'zip_code', 'email', 'phone', 'website', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = AccountingLegalEntity::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_accounting_legal_entity.name', $this->name])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.cin', $this->cin])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.tin', $this->tin])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.address', $this->address])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.city', $this->city])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.state', $this->state])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.country', $this->country])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.zip_code', $this->zip_code])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.email', $this->email])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.phone', $this->phone])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.website', $this->website])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.status', $this->status])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.id', $this->id])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.business_type_id', $this->business_type_id])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_accounting_legal_entity.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
