<?php

namespace common\modules\accounting\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\accounting\models\AccountingTax;

/**
 * AccountingTaxSearch represents the model behind the search form of `common\modules\accounting\models\AccountingTax`.
 */
class AccountingTaxSearch extends AccountingTax
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'jurisdiction_country', 'jurisdiction_state', 'value', 'applicable', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = AccountingTax::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_accounting_tax.name', $this->name])
            ->andFilterWhere(['like', 'mod_accounting_tax.jurisdiction_country', $this->jurisdiction_country])
            ->andFilterWhere(['like', 'mod_accounting_tax.jurisdiction_state', $this->jurisdiction_state])
            ->andFilterWhere(['like', 'mod_accounting_tax.value', $this->value])
            ->andFilterWhere(['like', 'mod_accounting_tax.applicable', $this->applicable])
            ->andFilterWhere(['like', 'mod_accounting_tax.status', $this->status])
            ->andFilterWhere(['like', 'mod_accounting_tax.id', $this->id])
            ->andFilterWhere(['like', 'mod_accounting_tax.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_accounting_tax.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_accounting_tax.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_accounting_tax.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_accounting_tax.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
