<?php

namespace common\modules\affiliate\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%affiliate}}".
 *
 * @property int $id
 * @property int $user_id
 * @property string $uuid
 * @property string $commission
 * @property string $approved
 * @property int $approved_by
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $user
 * @property User $approvedBy
 * @property AffiliateConversion[] $affiliateConversions
 * @property SiteUrl[] $siteUrls
 */
class Affiliate extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%affiliate}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'uuid', 'approved_by'], 'required'],
            [['user_id', 'approved_by', 'created_by', 'updated_by'], 'integer'],
            [['approved', 'metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['uuid', 'commission'], 'string', 'max' => 255],
            [['uuid'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['approved_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['approved_by' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'uuid' => 'Uuid',
            'commission' => 'Commission',
            'approved' => 'Approved',
            'approved_by' => 'Approved By',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'approved_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAffiliateConversions()
    {
        return $this->hasMany(\common\modules\affiliate\models\AffiliateConversion::className(), ['affiliate_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSiteUrls()
    {
        return $this->hasMany(\common\modules\site\models\SiteUrl::className(), ['affiliate_id' => 'id']);
    }
}