<?php

namespace common\modules\affiliate\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%affiliate_conversion}}".
 *
 * @property int $id
 * @property int $affiliate_id
 * @property int $promotion_id
 * @property string $session_id
 * @property int $logs_visitor_id
 * @property int $order_id
 * @property string $commission
 * @property string $approved
 * @property int $approved_by
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property Affiliate $affiliate
 * @property AffiliatePromotion $promotion
 * @property LogsVisitor $logsVisitor
 * @property ShopOrder $order
 * @property User $approvedBy
 */
class AffiliateConversion extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%affiliate_conversion}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['affiliate_id', 'promotion_id', 'commission', 'approved_by'], 'required'],
            [['affiliate_id', 'promotion_id', 'logs_visitor_id', 'order_id', 'approved_by', 'created_by', 'updated_by'], 'integer'],
            [['approved', 'metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['session_id', 'commission'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['affiliate_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\affiliate\models\Affiliate::className(), 'targetAttribute' => ['affiliate_id' => 'id']],
            [['promotion_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\affiliate\models\AffiliatePromotion::className(), 'targetAttribute' => ['promotion_id' => 'id']],
            [['logs_visitor_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\logs\models\LogsVisitor::className(), 'targetAttribute' => ['logs_visitor_id' => 'id']],
            [['order_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopOrder::className(), 'targetAttribute' => ['order_id' => 'id']],
            [['approved_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['approved_by' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'affiliate_id' => 'Affiliate ID',
            'promotion_id' => 'Promotion ID',
            'session_id' => 'Session ID',
            'logs_visitor_id' => 'Logs Visitor ID',
            'order_id' => 'Order ID',
            'commission' => 'Commission',
            'approved' => 'Approved',
            'approved_by' => 'Approved By',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAffiliate()
    {
        return $this->hasOne(\common\modules\affiliate\models\Affiliate::className(), ['id' => 'affiliate_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPromotion()
    {
        return $this->hasOne(\common\modules\affiliate\models\AffiliatePromotion::className(), ['id' => 'promotion_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLogsVisitor()
    {
        return $this->hasOne(\common\modules\logs\models\LogsVisitor::className(), ['id' => 'logs_visitor_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrder()
    {
        return $this->hasOne(\common\modules\shop\models\ShopOrder::className(), ['id' => 'order_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'approved_by']);
    }
}