<?php

namespace common\modules\affiliate\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%affiliate_promotion}}".
 *
 * @property int $id
 * @property string $type
 * @property int $image
 * @property string $video
 * @property string $html_attributes
 * @property int $url_id
 * @property string $commission
 * @property string $available
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property AffiliateConversion[] $affiliateConversions
 * @property User $createdBy
 * @property User $updatedBy
 * @property AssetManager $image0
 * @property SiteUrl $url
 */
class AffiliatePromotion extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%affiliate_promotion}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['type', 'video', 'html_attributes', 'available', 'metadata', 'status'], 'string'],
            [['image', 'url_id', 'created_by', 'updated_by'], 'integer'],
            [['url_id', 'commission'], 'required'],
            [['created_at', 'updated_at'], 'safe'],
            [['commission'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['image' => 'id']],
            [['url_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\site\models\SiteUrl::className(), 'targetAttribute' => ['url_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'type' => 'Type',
            'image' => 'Image',
            'video' => 'Video',
            'html_attributes' => 'Html Attributes',
            'url_id' => 'Url ID',
            'commission' => 'Commission',
            'available' => 'Available',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAffiliateConversions()
    {
        return $this->hasMany(\common\modules\affiliate\models\AffiliateConversion::className(), ['promotion_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getImage0()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'image']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUrl()
    {
        return $this->hasOne(\common\modules\site\models\SiteUrl::className(), ['id' => 'url_id']);
    }
}