<?php

namespace common\modules\affiliate\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\affiliate\models\Affiliate;

/**
 * AffiliateSearch represents the model behind the search form of `common\modules\affiliate\models\Affiliate`.
 */
class AffiliateSearch extends Affiliate
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'approved_by', 'created_by', 'updated_by'], 'integer'],
            [['uuid', 'commission', 'approved', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = Affiliate::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_affiliate.uuid', $this->uuid])
            ->andFilterWhere(['like', 'mod_affiliate.commission', $this->commission])
            ->andFilterWhere(['like', 'mod_affiliate.approved', $this->approved])
            ->andFilterWhere(['like', 'mod_affiliate.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_affiliate.status', $this->status])
            ->andFilterWhere(['like', 'mod_affiliate.id', $this->id])
            ->andFilterWhere(['like', 'mod_affiliate.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_affiliate.approved_by', $this->approved_by])
            ->andFilterWhere(['like', 'mod_affiliate.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_affiliate.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_affiliate.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_affiliate.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
