<?php

namespace common\modules\assetManager;

use Yii;

/**
 * asset-manager module definition class
 */
class Module extends \yii\base\Module
{
    const DEFAULT_THUMB_ALIAS = 'thumb';

    /**
     * {@inheritdoc}
     */
    public $controllerNamespace = 'common\modules\assetManager\controllers';

    /**
     *  Set true if you want to rename files if the name is already in use 
     * @var boolean
     */
    public $rename = true;
    
     /**
     *  Set true to enable autoupload
     * @var boolean
     */
    public $autoUpload = false;

    /**
     * Thumbnails name template.
     * Possible vars: {original}, {width}, {height}, {alias}, {extension}
     * Note: "am" alias is reserved for default thumbnails
     * @var string
     */
    public $thumbFilenameTemplate = '{original}-{alias}.{extension}';
    
    /**
     * @var array upload routes
     */
    public $routes = [
        // base absolute path to web directory
        'baseUrl' => '',
        // base web directory url
        'basePath' => '@assets',
        // path for uploaded files in web directory
        'uploadPath' => 'uploads',
    ];

    /**
     * @var array thumbnails info
     */
    public $thumbs = [
        'small' => [
            'name' => 'Small size',
            'size' => [120, 80],
        ],
        'medium' => [
            'name' => 'Medium size',
            'size' => [400, 300],
        ],
        'large' => [
            'name' => 'Large size',
            'size' => [800, 600],
        ],
    ];

    /**
     * @var array default thumbnail size, using in assetManager view.
     */
    private static $defaultThumbSize = [128, 128];

    /**
     * {@inheritdoc}
     */
    public function init()
    {
        parent::init();

        $this->registerTranslations();
    }

    public function registerTranslations()
    {
        Yii::$app->i18n->translations['modules/assetManager/*'] = [
            'class' => 'yii\i18n\PhpMessageSource',
            'sourceLanguage' => 'en-US',
            'basePath' => '@common/modules/assetManager/messages',
            'fileMap' => [
                'modules/assetManager/main' => 'main.php',
            ],
        ];
    }

    public static function t($category, $message, $params = [], $language = null)
    {
        if (!isset(Yii::$app->i18n->translations['modules/assetManager/*'])) {
            return $message;
        }

        return Yii::t("modules/assetManager/$category", $message, $params, $language);
    }

    /**
     * @return array default thumbnail size. Using in assetManager view.
     */
    public static function getDefaultThumbSize()
    {
        return self::$defaultThumbSize;
    }
}
