<?php

namespace common\modules\assetManager\models;

use Yii;
use yii\db\Expression;
use yii\web\UploadedFile;
use yii\data\ActiveDataProvider;
use yii\helpers\Inflector;
use yii\bootstrap4\Html;
use yii\imagine\Image;
use Imagine\Image\ImageInterface;
use common\modules\assetManager\Module;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%asset_manager}}".
 *
 * @property int $id
 * @property string $name
 * @property string $alt
 * @property string $description
 * @property string $app_alias
 * @property string $path
 * @property string $mime_type
 * @property string $size
 * @property string $thumbs
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property CmsAlbum[] $cmsAlbums
 * @property CmsBlog[] $cmsBlogs
 * @property CmsEntityAsset[] $cmsEntityAssets
 * @property CmsEvent[] $cmsEvents
 * @property CmsPage[] $cmsPages
 * @property CmsTeamMember[] $cmsTeamMembers
 * @property CmsTestimonial[] $cmsTestimonials
 * @property LmsCourse[] $lmsCourses
 * @property LmsCourseActivity[] $lmsCourseActivities
 * @property ShopCategory[] $shopCategories
 * @property ShopProduct[] $shopProducts
 * @property ShopSubcategory[] $shopSubcategories
 * @property ShopVoucher[] $shopVouchers
 * @property UserProfile[] $userProfiles
 */
class AssetManager extends \common\modules\core\models\GenesysActiveRecord
{
    public $file;

    public static $imageFileTypes = ['image/gif', 'image/jpeg', 'image/png'];

    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%asset_manager}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'app_alias', 'path', 'mime_type', 'size'], 'required'],
            [['description', 'thumbs', 'metadata', 'status'], 'string'],
            [['size', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['name', 'alt', 'app_alias', 'path', 'mime_type'], 'string', 'max' => 255],
            [['path'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['file'], 'file'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'alt' => 'Alt',
            'description' => 'Description',
            'app_alias' => 'App Alias',
            'path' => 'Path',
            'mime_type' => 'Mime Type',
            'size' => 'Size',
            'thumbs' => 'Thumbs',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsAlbums()
    {
        return $this->hasMany(\common\modules\cms\models\CmsAlbum::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsBlogs()
    {
        return $this->hasMany(\common\modules\cms\models\CmsBlog::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsEntityAssets()
    {
        return $this->hasMany(\common\modules\cms\models\CmsEntityAsset::className(), ['asset_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsEvents()
    {
        return $this->hasMany(\common\modules\cms\models\CmsEvent::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsPages()
    {
        return $this->hasMany(\common\modules\cms\models\CmsPage::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsTeamMembers()
    {
        return $this->hasMany(\common\modules\cms\models\CmsTeamMember::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsTestimonials()
    {
        return $this->hasMany(\common\modules\cms\models\CmsTestimonial::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsCourses()
    {
        return $this->hasMany(\common\modules\lms\models\LmsCourse::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsCourseActivities()
    {
        return $this->hasMany(\common\modules\lms\models\LmsCourseActivity::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopCategories()
    {
        return $this->hasMany(\common\modules\shop\models\ShopCategory::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopProducts()
    {
        return $this->hasMany(\common\modules\shop\models\ShopProduct::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopSubcategories()
    {
        return $this->hasMany(\common\modules\shop\models\ShopSubcategory::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopVouchers()
    {
        return $this->hasMany(\common\modules\shop\models\ShopVoucher::className(), ['featured_image' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserProfiles()
    {
        return $this->hasMany(\common\modules\user\models\UserProfile::className(), ['avatar' => 'id']);
    }

    public function behaviors()
    {
        return [
            [
                'class' => \yii\behaviors\TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }

    public function afterSave($insert, $changedAttributes)
    {
        parent::afterSave($insert, $changedAttributes);
    }

    /**
     * Save just uploaded file
     * @param array $routes routes from module settings
     * @param bool $rename
     * @return bool
     */
    public function saveAsset(array $routes, $rename = false, $app_alias = 'assets')
    {
        // get file instance
        $this->file = UploadedFile::getInstance($this, 'file');
        $asset_class_dir = '';
        $subcategories = array();
        /*$modelSubcategory = Subcategory::findAll(['status' => 'active']);

        foreach ($modelSubcategory as $subcategoryKey => $subcategoryValue) {
            $subcategories[] = Inflector::slug(Value::getEntityValue('subcategory', 'name', $subcategoryValue->id, 'en'));
        }*/

        foreach (Yii::$app->params['asset_classes'] as $asset_classes_key => $asset_classes_value) {
            if (preg_match("/^$asset_classes_value-(\w+)/", $this->file->basename)) {
                if ($asset_classes_value == 'product') {
                    foreach ($subcategories as $subcategoriesKey => $subcategoriesValue) {
                        if (preg_match("/^$asset_classes_value-$subcategoriesValue-(\w+)/", $this->file->basename)) {
                            $asset_class_dir = $asset_classes_value . '/' . $subcategoriesValue;

                            break;
                        } else {
                            $asset_class_dir = date('Y', time()) . '/' . date('m', time());
                        }
                    }    
                } else {
                    $asset_class_dir = $asset_classes_value;
                }

                break;
            } else {
                $asset_class_dir = date('Y', time()) . '/' . date('m', time());
            }
        }

        $structure = "$routes[baseUrl]/$routes[uploadPath]/$asset_class_dir";
        $basePath = Yii::getAlias($routes['basePath']);
        $absolutePath = "$basePath/$structure";

        // create actual directory structure "yyyy/mm"
        if (!file_exists($absolutePath)) {
            mkdir($absolutePath, 0777, true);
        }
        
        // if a file with the same name already exist append a number
        $counter = 0;
        
        do {
            if ($counter == 0) {
                $name = Inflector::slug($this->file->baseName) . '.' . $this->file->extension;
            } else {
                // if we don't want to rename we finish the call here
                if ($rename == false) {
                    return false;
                }

                $name = Inflector::slug($this->file->baseName) . $counter . '.' . $this->file->extension;
            }

            $path = "$structure/$name";
            
            $counter++;
        } while(self::findByUrl($path)); // checks for existing path in db

        // save original uploaded file
        $this->file->saveAs("$absolutePath/$name");

        $this->name = $name;
        $this->app_alias = $app_alias;
        $this->path = $path;
        $this->mime_type = $this->file->type;
        $this->size = $this->file->size;
        $this->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
        $this->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
        $this->status = 'active';

        if ($this->validate()) {
            return $this->save();
        } else {
            return false;
        }
    }

    /**
     * Create thumbs for this image
     *
     * @param array $routes see routes in module config
     * @param array $presets thumbs presets. See in module config
     * @return bool
     */
    public function createThumbs(array $routes, array $presets)
    {
        $thumbs = [];
        $basePath = $basePath = Yii::getAlias($routes['basePath']);
        $originalFile = pathinfo($this->path);
        $dirname = $originalFile['dirname'];
        $filename = $originalFile['filename'];
        $extension = $originalFile['extension'];

        Image::$driver = [Image::DRIVER_GD2, Image::DRIVER_GMAGICK, Image::DRIVER_IMAGICK];

        foreach ($presets as $alias => $preset) {
            $width = $preset['size'][0];
            $height = $preset['size'][1];
            $mode = (isset($preset['mode']) ? $preset['mode'] : ImageInterface::THUMBNAIL_OUTBOUND);

            $thumbUrl = "$dirname/" . $this->getThumbFilename($filename, $extension, $alias, $width, $height);

            Image::thumbnail("$basePath/{$this->path}", $width, $height, $mode)->save("$basePath/$thumbUrl");

            $thumbs[$alias] = $thumbUrl;
        }

        $this->thumbs = serialize($thumbs);
        
        $this->detachBehavior('timestamp');

        // create default thumbnail
        $this->createDefaultThumb($routes);

        if ($this->validate()) {
            return $this->save();
        } else {
            return false;
        }
    }

    /**
     * Create default thumbnail
     *
     * @param array $routes see routes in module config
     */
    public function createDefaultThumb(array $routes)
    {
        $originalFile = pathinfo($this->path);
        $dirname = $originalFile['dirname'];
        $filename = $originalFile['filename'];
        $extension = $originalFile['extension'];

        Image::$driver = [Image::DRIVER_GD2, Image::DRIVER_GMAGICK, Image::DRIVER_IMAGICK];

        $size = Module::getDefaultThumbSize();

        $width = $size[0];
        $height = $size[1];
        $thumbUrl = "$dirname/" . $this->getThumbFilename($filename, $extension, Module::DEFAULT_THUMB_ALIAS, $width, $height);
        $basePath = Yii::getAlias($routes['basePath']);

        Image::thumbnail("$basePath/{$this->path}", $width, $height)->save("$basePath/$thumbUrl");
    }

    /**
     * @return bool if type of this media file is image, return true;
     */
    public function isImage()
    {
        return in_array($this->mime_type, self::$imageFileTypes);
    }

    /**
     * @param $baseUrl
     * @return string default thumbnail for image
     */
    public function getDefaultThumbUrl($baseUrl = '')
    {
        if ($this->isImage()) {
            $size = Module::getDefaultThumbSize();
            
            $originalFile = pathinfo($this->path);
            $dirname = $originalFile['dirname'];
            $filename = $originalFile['filename'];
            $extension = isset($originalFile['extension']) ? $originalFile['extension'] : '';
            $width = $size[0];
            $height = $size[1];

            return Yii::getAlias('@assetsurl') . "$dirname/" . $this->getThumbFilename($filename, $extension, Module::DEFAULT_THUMB_ALIAS, $width, $height);
        }

        return "$baseUrl/images/file.png";
    }

    /**
     * @param $baseUrl
     * @return string default thumbnail for image
     */
    public function getDefaultUploadThumbUrl($baseUrl = '')
    {
        $size = Module::getDefaultThumbSize();

        $originalFile = pathinfo($this->path);
        $dirname = $originalFile['dirname'];
        $filename = $originalFile['filename'];
        $extension = $originalFile['extension'];
        $width = $size[0];
        $height = $size[1];

        return Yii::getAlias('@assetsurl') . "$dirname/" . $this->getThumbFilename($filename, $extension, Module::DEFAULT_THUMB_ALIAS, $width, $height);
    }

    /**
     * Returns thumbnail name
     *
     * @param $original
     * @param $extension
     * @param $alias
     * @param $width
     * @param $height
     *
     * @return string
     */
    protected function getThumbFilename($original, $extension, $alias, $width, $height)
    {
        /** @var AssetManager $module */
        $module = Module::getInstance();

        return strtr($module->thumbFilenameTemplate, [
            '{width}'     => $width,
            '{height}'    => $height,
            '{alias}'     => $alias,
            '{original}'  => $original,
            '{extension}' => $extension,
        ]);
    }

    /**
     * @return array thumbnails
     */
    public function getThumbs()
    {
        return unserialize($this->thumbs) ?: [];
    }

    /**
     * @param string $alias thumb alias
     * @return string thumb url
     */
    public function getThumbUrl($alias)
    {
        $thumbs = $this->getThumbs();

        if ($alias === 'original') {
            return $this->path;
        }

        return !empty($thumbs[$alias]) ? $thumbs[$alias] : '';
    }

    /**
     * Thumbnail image html tag
     *
     * @param string $alias thumbnail alias
     * @param array $options html options
     * @return string Html image tag
     */
    public function getThumbImage($alias, $options = [])
    {
        $url = $this->getThumbUrl($alias);

        if (empty($url)) {
            return '';
        }

        if (empty($options['alt'])) {
            $options['alt'] = $this->alt;
        }

        return Html::img($url, $options);
    }

    /**
     * @param AssetManager $module
     * @return array images list
     */
    public function getImagesList(Module $module)
    {
        $thumbs = $this->getThumbs();
        $list = [];
        $originalImageSize = $this->getOriginalImageSize($module->routes);
        $list[$this->path] = Module::t('main', 'Original') . ' ' . $originalImageSize;

        foreach ($thumbs as $alias => $url) {
            $preset = $module->thumbs[$alias];
            $list[$url] = $preset['name'] . ' ' . $preset['size'][0] . ' × ' . $preset['size'][1];
        }

        return $list;
    }

    /**
     * Delete thumbnails for current image
     * @param array $routes see routes in module config
     */
    public function deleteThumbs(array $routes)
    {
        $basePath = Yii::getAlias($routes['basePath']);

        foreach ($this->getThumbs() as $thumbUrl) {
            if (is_file("$basePath/$thumbUrl")) {
                unlink("$basePath/$thumbUrl");
            }
        }
        
        $defaultThumbPath = "$basePath/{$this->getDefaultThumbUrl()}";

        if (is_file($defaultThumbPath)) {
            unlink($defaultThumbPath);
        }
    }

    /**
     * Delete file
     * @param array $routes see routes in module config
     * @return bool
     */
    public function deleteFile(array $routes)
    {
        $basePath = Yii::getAlias($routes['basePath']);
        $filePath = "$basePath/{$this->path}";

        return is_file($filePath) ? unlink($filePath) : false;
    }

    /**
     * @return int last changes timestamp
     */
    public function getLastChanges()
    {
        return !empty($this->updated_at) ? $this->updated_at : $this->created_at;
    }

    /**
     * This method wrap getimagesize() function
     * @param array $routes see routes in module config
     * @param string $delimiter delimiter between width and height
     * @return string image size like '1366x768'
     */
    public function getOriginalImageSize(array $routes, $delimiter = ' × ')
    {
        $imageSizes = $this->getOriginalImageSizes($routes);
        
        return "$imageSizes[0]$delimiter$imageSizes[1]";
    }

    /**
     * This method wrap getimagesize() function
     * @param array $routes see routes in module config
     * @return array
     */
    public function getOriginalImageSizes(array $routes)
    {
        $basePath = Yii::getAlias($routes['basePath']);
        
        return getimagesize("$basePath/{$this->path}");
    }

    /**
     * @return string file size
     */
    public function getFileSize()
    {
        Yii::$app->formatter->sizeFormatBase = 1000;
        
        return Yii::$app->formatter->asShortSize($this->size, 0);
    }

    /**
     * Find model by url
     *
     * @param $url
     * @return static
     */
    public static function findByUrl($url)
    {
        return self::findOne(['path' => $url]);
    }

    /**
     * Search models by file types
     * @param array $types file types
     * @return array|\yii\db\ActiveRecord[]
     */
    public static function findByTypes(array $types)
    {
        return self::find()->filterWhere(['in', 'mime_type', $types])->all();
    }

    public static function getAssetUrl($asset_id)
    {
        $modelAsset = new self();

        if (self::find()->where(['id' => $asset_id, 'status' => 'active'])->exists()) {
            $modelAsset = self::findOne(['id' => $asset_id, 'status' => 'active']);

            return Yii::getAlias('@' . $modelAsset->app_alias . 'url') . $modelAsset->path;
        } else {
            return '';
        }
    }

    public static function getAssetAlt($asset_id)
    {
        $modelAsset = new self();

        if (self::find()->where(['id' => $asset_id, 'status' => 'active'])->exists()) {
            $modelAsset = self::findOne(['id' => $asset_id, 'status' => 'active']);

            return $modelAsset->alt;
        } else {
            return '';
        }
    }
}
