<?php

namespace common\modules\assetManager\models;

use Yii;
use yii\web\UploadedFile;
use yii\behaviors\TimestampBehavior;
use yii\db\ActiveRecord;
use yii\imagine\Image;
use yii\data\ActiveDataProvider;
use yii\bootstrap4\Html;
use yii\helpers\Inflector;
use common\modules\assetManager\Module;
use common\modules\assetManager\models\Owners;
use Imagine\Image\ImageInterface;

/**
 * This is the model class for table "assetManager_tag".
 *
 * @property integer $id
 * @property string $name
 *
 * relations
 * @property Mediafile[] $mediafiles
 */
class Tag extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'assetManager_tag';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['name'], 'required'],
            [['name'], 'string', 'max' => 100],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => Module::t('main', 'Tag'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMediafiles() {
        return $this->hasMany(Mediafile::className(), ['id' => 'mediafile_id'])
            ->viaTable('assetManager_mediafile_tag', ['tag_id' => 'id']);
    }

	/**
	 * Removes unused tags
	 *
	 * @return int
	 * @throws \yii\db\Exception
	 */
	public static function removeUnusedTags()
	{
		return Yii::$app->db->createCommand(
			'DELETE assetManager_tag
			FROM
				assetManager_tag
			LEFT JOIN assetManager_mediafile_tag ON id = tag_id
			WHERE
				ISNULL(mediafile_id)'
		)->execute();
	}
}
