<?php

use yii\helpers\ArrayHelper;
use yii\bootstrap4\Html;
use yii\bootstrap4\ActiveForm;

use common\modules\assetManager\Module;
use common\modules\assetManager\assets\AssetManagerAsset;

$bundle = AssetManagerAsset::register($this);
?>

<div class="row">
    <div class="col-md-4">
        <?= Html::img($model->getDefaultThumbUrl($bundle->baseUrl), ['class' => 'img-thumbnail']) ?>
    </div>

    <div class="col-md-8">
        <ul class="detail">
            <li><strong><?= $model->name ?></strong></li>
            <li class="text-muted"><small><?= $model->mime_type ?></small></li>
            <li class="text-muted"><small><?= $model->getFileSize() ?></small></li>

            <?php if ($model->isImage()) : ?>
                <li class="text-muted"><small><?= $model->getOriginalImageSize($this->context->module->routes) ?></small></li>
            <?php endif; ?>

            <li class="text-muted"><small><?= $model->getLastChanges() ?></small></li>
            <li>
                
            </li>
        </ul>
    </div>
</div>

<br>

<div class="row">
    <div class="col-md-12">
        <?php
        $form = ActiveForm::begin([
            'id' => 'control-form',
            'action' => [
                'asset/update', 
                'id' => $model->id
            ],
            'enableClientValidation' => false,
        ]);
        ?>
            <?php if ($model->isImage()) : ?>
                <?= $form->field($model, 'alt')->textInput(['class' => 'form-control form-control-sm', 'placeholder' => 'Alt'])->label(false) ?>
            <?php endif; ?>

            <?= $form->field($model, 'description')->textarea(['class' => 'form-control form-control-sm', 'placeholder' => 'Description'])->label(false) ?>

            <?php if ($model->isImage()) : ?>
                <div class="form-group<?= $strictThumb ? ' hidden' : '' ?>">
                    <?= Html::dropDownList('path', $model->getThumbUrl($strictThumb), $model->getImagesList($this->context->module), ['class' => 'form-control form-control-sm', 'prompt' => 'Select image size']) ?>

                    <div class="help-block"></div>
                </div>
            <?php else : ?>
                <?= Html::hiddenInput('path', $model->path) ?>
            <?php endif; ?>

            <?= Html::hiddenInput('id', $model->id) ?>

            <?= Html::button(Module::t('main', 'Insert'), ['id' => 'insert-btn', 'class' => 'btn btn-primary btn-sm']) ?>

            <?= Html::submitButton(Module::t('main', 'Save'), ['class' => 'btn btn-success btn-sm']) ?>

            <?= Html::a(
                Module::t('main', 'Delete'), 
                [
                    'asset/delete/', 
                    'id' => $model->id
                ],
                [
                    'class' => 'btn btn-danger btn-sm',
                    'data-message' => Yii::t('yii', 'Are you sure you want to delete this item?'),
                    'data-id' => $model->id,
                    'role' => 'delete',
                ]
            ) ?>

            <?php if ($message = Yii::$app->session->getFlash('mediafileUpdateResult')) : ?>
                <div class="text-success"><?= $message ?></div>
            <?php endif; ?>
        <?php ActiveForm::end(); ?>
    </div>
</div>

<br>