<?php

namespace common\modules\cms\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\cms\models\CmsAlbum;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

class AlbumController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'view'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'index' => ['get'],
                    'view' => ['get'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionIndex()
    {
        if (!CmsAlbum::find()->where(['status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Page not found.', 404);
        }

        $albums = $this->getAllAlbum();

        return $this->render('index', [
            'modelValue' => $albums,
        ]);
    }

    public function actionView($id)
    {
        if (!CmsAlbum::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Page not found.', 404);
        }

        $modelEntityClass = new CoreEntityClass();
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'album', 'status' => 'active']);

        $model = CmsAlbum::findOne(['id' => $id, 'status' => 'active']);

        $albums = $this->getAllAlbum();
        $album_arr = array();
        
        if(array_key_exists($id, $albums)) {
            $album_arr = $albums[$id];
        }

        unset($albums[$id]);

        $modelAlbumImage = \common\modules\cms\models\CmsEntityAsset::find()->where(['entity_id' => $model->id, 'entity_class_id' => $modelEntityClass->id,'status' => 'active'])->asArray()->all();

        return $this->render('view', [
            'model' => $model, 
            'albumDetails' => $album_arr, 
            'modelValue' => $albums,
            'modelAlbumImage' => $modelAlbumImage,
        ]);
    }

    protected function getAllAlbum() 
    {
        $modelEntityClass = new CoreEntityClass();
        $modelAttributeClass = new CoreAttributeClass();
        $modelValue = new CoreValue();

        $modelAlbum = CmsAlbum::find()->where(['status' => 'active'])->asArray()->all();
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'album', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::find()->where(['name' => array('name', 'title', 'meta_description', 'url')])->asArray()->all();
        $attr_class_ids = array();
        $attr = array();
        $albums = array();

        foreach ($modelAttributeClass as $key => $value) {
            array_push($attr_class_ids, $value['id']);

            $attr[$value['id']] = $value['name'];
        }

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $attr_class_ids, 'status' => 'active'])->asArray()->all();

        foreach ($tmpArray as $key => $value) {
            if(array_key_exists($value['attribute_class_id'], $attr)) {
                $albums[$value['entity_id']][$attr[$value['attribute_class_id']]] = $value['value'];
            }
        }
        
        foreach($modelAlbum as $key => $value) {
            $albums[$value['id']]['date'] = $value['date'];
            $albums[$value['id']]['location'] = $value['location'];
            $albums[$value['id']]['featured_img'] = $value['featured_image'];
            $albums[$value['id']]['created_at'] = $value['created_at'];
        }

        return $albums;
    }
}
