<?php

namespace common\modules\cms\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\cms\models\CmsBlog;
use common\modules\cms\models\CmsEntityKeyword;
use common\modules\core\models\CoreKeyword;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

class BlogController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'view-magazine'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'view', 'view-magazine'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'index' => ['get'],
                    'view' => ['get'],
                    'view-magazine' => ['get'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionIndex($keywords = '')
    {
        $blog_data = \common\modules\cms\models\CmsBlog::getData([], [], true, 'blog', false);
        $pagination = new \yii\data\Pagination(['totalCount' => count($blog_data), 'pageSize' => 12]);

        $provider = new \yii\data\ArrayDataProvider([
            'allModels' => $blog_data,
            'sort' => [
                'attributes' => ['created_at'],
            ],
            'pagination' => [
                'pageSize' => $pagination->limit,
            ],
        ]);

        $blogs = $provider->getModels();

        return $this->render('index', [
            'blogs' => $blogs,
            'pagination' => $pagination,
        ]);
    }

    public function actionView($id)
    {
        if (!CmsBlog::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Blog not found.', 404);
        }

        return $this->render('view', [
            'model' => \common\modules\cms\models\CmsBlog::findOne(['id' => $id, 'status' => 'active']),
            'blog' => \common\modules\cms\models\CmsBlog::getData(['id' => $id], [], true, 'blog', false)[0], 
        ]);
    }
}
