<?php

namespace common\modules\cms\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\cms\models\CmsEvent;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

class EventController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['view'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index','view'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'view' => ['get'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionIndex()
    {
        if (!CmsEvent::find()->where(['status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Page not found.', 404);
        }

        $eventValue = $this->getAllEvent();
        
        $events_arr = array();
        //echo "<pre/>";print_r($eventValue);die;
        foreach ($eventValue as $key => $value) {
            array_push($events_arr, array('title'=>$value['name'], 'start'=>$value['start'],'end' => $value['end'],'allDay' => 'true','url' => '/event/'.$value['url']));
        }
        return $this->render('index', [
            'modelValue' => $eventValue,
            'events' => $events_arr,
        ]);

        
    }

    public function actionView($id)
    {
        if (!CmsEvent::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Page not found.', 404);
        }

        $model = CmsEvent::findOne(['id' => $id, 'status' => 'active']);

        $name = CoreValue::getEntityValue('event', 'name', $model->id);
        $description = CoreValue::getEntityValue('event', 'description', $model->id);
        $starttimestmp = strtotime($model->start);
        $endtimestmp = strtotime($model->end);
        $startime = date('F d, Y', $starttimestmp);
        $endtime = '';
        if($starttimestmp !== $endtimestmp) {
            $endtime = date('F d, Y', $endtimestmp);
        }
        $ispast = 0;
        if ($endtimestmp < time()) {
            $ispast = 1;
        }

        $details = array('name'=> $name, 'description' => $description, 'start' => $startime, 'end' => $endtime, 'venue' => $model->venue,'organizer' => $model->organizer, 'fee' => $model->entry_fee, 'ispast' => $ispast);

        return $this->render('view', [
            'model' => $model,
            'details' => $details
        ]);
    }

    protected function getAllEvent() {
        $modelValue = new CoreValue();
        $modelEntityClass = new CoreEntityClass();
        $modelAttributeClass = new CoreAttributeClass();

        $model = CmsEvent::find()->where(['status' => 'active'])->asArray()->all();
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'event', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::find()->where(['name' => array('name','url')])->asArray()->all();
        $attrClassIdArray = array();
        $attrArray = array();
        $eventDetailsArr = array();

        foreach ($modelAttributeClass as $key => $value) {
            array_push($attrClassIdArray, $value['id']);
            $attrArray[$value['id']] = $value['name'];
        }
        
        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $attrClassIdArray, 'status' => 'active'])->asArray()->all();

        foreach ($tmpArray as $key => $value) {
            if(array_key_exists($value['attribute_class_id'], $attrArray)) {
                $eventDetailsArr[$value['entity_id']][$attrArray[$value['attribute_class_id']]] = $value['value'];
            }
        }
        
        foreach($model as $key => $value) {
            $eventDetailsArr[$value['id']]['start'] = $value['start'];
            $eventDetailsArr[$value['id']]['end'] = $value['end'];
        }
        //echo json_encode($modelValue);die;
        //$modeljson = json_encode($modelValue);
        return $eventDetailsArr;

    }
}
