<?php

namespace common\modules\cms\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\cms\models\CmsNews;
use common\modules\cms\models\CmsEntityKeyword;
use common\modules\core\models\CoreKeyword;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

class NewsController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'view'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'index' => ['get'],
                    'view' => ['get'],
                    'view-magazine' => ['get'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionIndex($keywords = '')
    {
        $news_data = \common\modules\cms\models\CmsNews::getData([], [], true, 'cms_news', false);
        $pagination = new \yii\data\Pagination(['totalCount' => count($news_data), 'pageSize' => 12]);
        $news_data_tmp = array();

        foreach ($news_data as $key => $value) {
            $news_data_tmp[date('Y-m', strtotime($value['publication_date']))][] = $value;
        }

        $news_data = $news_data_tmp;

        krsort($news_data);

        return $this->render('index', [
            'news' => $news_data,
        ]);
    }

    public function actionView($id)
    {
        if (!CmsNews::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('News not found.', 404);
        }

        return $this->render('view', [
            'model' => \common\modules\cms\models\CmsNews::findOne(['id' => $id, 'status' => 'active']),
            'news' => \common\modules\cms\models\CmsNews::getData(['id' => $id], [], true, 'cms_news', false)[0], 
        ]);
    }
}
