<?php

namespace common\modules\cms\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\cms\models\CmsPage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

class PageController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['view'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['view'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'view' => ['get', 'post'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionView($id)
    {
        if (!CmsPage::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Page not found.', 404);
        }

        $request = Yii::$app->request;

        $modelFormLead = new \common\modules\mailer\models\FormLead();
        $tmp_msg = '';

        if ($request->isPost) {
            $modelFormLead->load($request->post());

            if ($modelFormLead->save()) {
                $tmp_msg = Yii::$app->params['modules']['mailer']['contact']['lead-gen']['message'][0]['success'];

                Yii::$app->session->setFlash('success', $tmp_msg);
            } else {
                $tmp_msg = Yii::$app->params['modules']['mailer']['contact']['lead-gen']['message'][0]['error'];

                Yii::$app->session->setFlash('error', $tmp_msg);
            }

            return $this->redirect(['/mailer/contact/lead-gen-track', 'msg' => $tmp_msg]);
        }

        $page_details = array();
        $attr_class_ids = array();
        $attr = array();

        $model = CmsPage::findOne(['id' => $id, 'status' => 'active']);
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'page', 'status' => 'active']);
        $attributeClassArr = CoreEntityAttribute::getAttributeClassesOfEntityClass('page');
        $modelAttributeClass = CoreAttributeClass::find()->where(['name' => $attributeClassArr])->asArray()->all();
        
        foreach ($modelAttributeClass as $key => $value) {
            array_push($attr_class_ids, $value['id']);

            $attr[$value['id']] = $value['name'];
        }

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $attr_class_ids, 'entity_id' => $id, 'status' => 'active'])->asArray()->all();
		
		foreach ($tmpArray as $key => $value) {
            if(array_key_exists($value['attribute_class_id'], $attr)) {
                $page_details[$attr[$value['attribute_class_id']]] = $value['value'];
            }
        }
		
		return $this->render('view', [
            'model' => $model, 
            'page_details' => $page_details, 
            'modelFormLead' => $modelFormLead,
        ]);
    }
}
