<?php

namespace common\modules\cms\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\cms\models\CmsTeamMember;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

class TeamMemberController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['view'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index','view'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'view' => ['get'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionIndex()
    {
        if (!CmsTeamMember::find()->where(['status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Page not found.', 404);
        }

        $modelValue = new CoreValue();
        $modelEntityClass = new CoreEntityClass();
        $modelAttributeClass = new CoreAttributeClass();

        $model = CmsTeamMember::find()->where(['status' => 'active'])->asArray()->all();
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'team_member', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::find()->where(['name' => array('name','team_member_designation','url')])->asArray()->all();
        $attrClassIdArray = array();
        $attrArray = array();
        $memberDetailsArr = array();

        foreach ($modelAttributeClass as $key => $value) {
            array_push($attrClassIdArray, $value['id']);
            $attrArray[$value['id']] = $value['name'];
        }
        
        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $attrClassIdArray, 'status' => 'active'])->asArray()->all();

        foreach ($tmpArray as $key => $value) {
            if(array_key_exists($value['attribute_class_id'], $attrArray)) {
                $memberDetailsArr[$value['entity_id']][$attrArray[$value['attribute_class_id']]] = $value['value'];
            }
        }

        foreach($model as $key => $value) {
            $memberDetailsArr[$value['id']]['featured_img'] = $value['featured_image'];
        }
        
        return $this->render('index', [
            'modelValue' => $memberDetailsArr,
        ]);
    }

    public function actionView($id)
    {
        if (!CmsTeamMember::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Page not found.', 404);
        }

        $model = CmsTeamMember::findOne(['id' => $id, 'status' => 'active']);

        $memberDetailsArr = array();

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'team_member', 'status' => 'active']);

        $modelAttributeClass = CoreAttributeClass::find()->where(['name' => array('name','title','meta_description','url', 'team_member_designation', 'team_member_bio')])->asArray()->all();

        $attrClassIdArray = array();
        $attrArray = array();
        
        foreach ($modelAttributeClass as $key => $value) {
            array_push($attrClassIdArray, $value['id']);
            $attrArray[$value['id']] = $value['name'];
        }

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $attrClassIdArray, 'entity_id' => $id, 'status' => 'active'])->asArray()->all();

        foreach ($tmpArray as $key => $value) {
            if(array_key_exists($value['attribute_class_id'], $attrArray)) {
                $memberDetailsArr[$attrArray[$value['attribute_class_id']]] = $value['value'];
            }
        }
        
        return $this->render('view', [
            'memberDetails' => $memberDetailsArr,
            'model' => $model
        ]);
    }
}
