<?php

namespace common\modules\cms\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\cms\models\CmsTestimonial;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

class TestimonialController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['view'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index','view'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'view' => ['get'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionIndex()
    {
        if (!CmsTestimonial::find()->where(['status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Page not found.', 404);
        }

        $modelValue = new CoreValue();
        $modelEntityClass = new CoreEntityClass();
        $modelAttributeClass = new CoreAttributeClass();

        $model = CmsTestimonial::find()->where(['status' => 'active'])->asArray()->all();
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'testimonial', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::find()->where(['name' => array('name','testimonial_role', 'testimonial_company', 'testimonial_content')])->asArray()->all();
        $attrClassIdArray = array();
        $attrArray = array();
        $testimonialsDetailsArr = array();
        
        foreach ($modelAttributeClass as $key => $value) {
            array_push($attrClassIdArray, $value['id']);
            $attrArray[$value['id']] = $value['name'];
        }
        
        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $attrClassIdArray, 'status' => 'active'])->asArray()->all();
        
        foreach ($tmpArray as $key => $value) {
            if(array_key_exists($value['attribute_class_id'], $attrArray)) {
                $testimonialsDetailsArr[$value['entity_id']][$attrArray[$value['attribute_class_id']]] = $value['value'];
            }
        }

        foreach($model as $key => $value) {
            $testimonialsDetailsArr[$value['id']]['featured_img'] = $value['featured_image'];
            $testimonialsDetailsArr[$value['id']]['type'] = $value['type'];
            if($value['type'] == 'video') {
                $testimonialsDetailsArr[$value['id']]['video'] = \common\modules\cms\models\CmsEntityVideo::getAttr(['embed_url'], ['entity_class_id' => $modelEntityClass->id, 'entity_id' => $value['id']]);
            }
        }
        
        return $this->render('index', [
            'modelValue' => $testimonialsDetailsArr,
        ]);

    }
}
