<?php

namespace common\modules\cms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%cms_album}}".
 *
 * @property int $id
 * @property int $featured_image
 * @property string $location
 * @property string $date
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property AssetManager $featuredImage
 */
class CmsAlbum extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%cms_album}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['featured_image', 'created_by', 'updated_by'], 'integer'],
            [['date'], 'safe'],
            [['metadata', 'status'], 'string'],
            [[], 'required'],
            [['location'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['featured_image' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'featured_image' => 'Featured Image',
            'location' => 'Location',
            'date' => 'Date',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'featured_image']);
    }
    
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'album', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'album', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
    
    public function saveAlbum($post_data, $album_id = 0)
    {
        $session = Yii::$app->session;

        $modelCmsAlbum = new self();
        $modelCoreValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($album_id) && $album_id > 0 && self::find()->where(['id' => $album_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelCmsAlbum = self::findOne(['id' => $album_id, 'status' => 'active']);

                $modelCmsAlbum->location = $post_data['CmsAlbum']['location'];
                $modelCmsAlbum->date = $post_data['CmsAlbum']['date'];
                $modelCmsAlbum->featured_image = $post_data['CmsAlbum']['featured_image'];
            } else {    // Create a new model
                $modelCmsAlbum = new self();

                $modelCmsAlbum->location = $post_data['CmsAlbum']['location'];
                $modelCmsAlbum->date = $post_data['CmsAlbum']['date'];
                $modelCmsAlbum->featured_image = $post_data['CmsAlbum']['featured_image'];
                $modelCmsAlbum->status = 'active';
            }

            if ($modelCmsAlbum->validate()) {
                $modelCmsAlbum->save();

                foreach ($post_data['CmsAlbum'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }
                    
                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();

                            if ($key == 'url') {                                
                                if (!isset($codeValue) || empty($codeValue)) {
                                    $codeValue = (string) '?r=cms/album/view&id=' . $modelCmsAlbum->id;
                                }                                
                            }

                            $modelCoreValue = CoreValue::setEntityValue('album', $key, $modelCmsAlbum->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }                    
                }

                // Saving album images
                $asset_arr = array();
				
				if (isset($post_data['CmsEntityAsset'])) {
					foreach ($post_data['CmsEntityAsset'] as $assetKey=>$assetValue) {
						$asset_arr[$assetKey]['asset_id'] = $assetValue;
					}
				}
                
                $is_Save = CmsEntityAsset::saveEntityAssets('album', $modelCmsAlbum->id, $asset_arr);

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Album saved successfully.");

                    return $modelCmsAlbum->id;
                } else {
                    $transaction->rollBack();
                    
                    Yii::$app->session->setFlash('error', "Unable to save value " . Globals::renderErrors($modelCoreValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save album: " . Globals::renderErrors($modelCmsAlbum->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
