<?php

namespace common\modules\cms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%cms_entity_asset}}".
 *
 * @property int $id
 * @property int $entity_class_id
 * @property int $entity_id
 * @property int $asset_id
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property CoreEntityClass $entityClass
 * @property AssetManager $asset
 * @property User $createdBy
 * @property User $updatedBy
 */
class CmsEntityAsset extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%cms_entity_asset}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['entity_class_id', 'entity_id'], 'required'],
            [['entity_class_id', 'entity_id', 'asset_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['entity_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreEntityClass::className(), 'targetAttribute' => ['entity_class_id' => 'id']],
            [['asset_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['asset_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'entity_class_id' => 'Entity Class ID',
            'entity_id' => 'Entity ID',
            'asset_id' => 'Asset ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEntityClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreEntityClass::className(), ['id' => 'entity_class_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAsset()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'asset_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    public function saveEntityAssets($entity_class, $entity_id, $asset_id_arr) {
        $breakFlag = false;

        // Saving product images
        if (is_numeric($entity_class)) {
            $modelCoreEntityClass = \common\modules\core\models\CoreEntityClass::findOne(['id' => $entity_class, 'status' => 'active']);
        } else {
            $modelCoreEntityClass = \common\modules\core\models\CoreEntityClass::findOne(['name' => $entity_class, 'status' => 'active']);
        }

        $entity_assets = isset($asset_id_arr) ? array_filter(array_map('current', $asset_id_arr)) : [];
        $entity_assets_tmp = array_map('current', CmsEntityAsset::find()->select('asset_id')->where(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $entity_id, 'status' => 'active'])->andWhere(['IN', 'asset_id', $entity_assets])->asArray()->all());

        $counts = array_count_values($entity_assets_tmp);
        $entity_assets_tmp = array_filter($entity_assets, function($o) use (&$counts) {
            return empty($counts[$o]) || !$counts[$o]--;
        });
        
        foreach ($entity_assets_tmp as $entityImageKey => $entityImageValue) {
            $modelEntityImage = new CmsEntityAsset();
            $modelEntityImage->entity_class_id = $modelCoreEntityClass->id;
            $modelEntityImage->entity_id = $entity_id;
            $modelEntityImage->asset_id = $entityImageValue;
            $modelEntityImage->status = 'active';

            if ($modelEntityImage->validate()) {
                $modelEntityImage->save();
            } else {
                $transaction->rollBack();
                $breakFlag = true;
            }
        }

        // Deleting left-out product images
        if (!$breakFlag) {
            foreach (CmsEntityAsset::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $entity_id, 'status' => 'active'])->andWhere(['NOT IN', 'asset_id', $entity_assets])->orWhere(['entity_id' => $entity_id, 'asset_id' => null, 'status' => 'active'])->all() as $key => $value) {
                $value->softDelete();
            }
        }
        
        return $breakFlag;
    }
}
