<?php

namespace common\modules\cms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%cms_entity_video}}".
 *
 * @property int $id
 * @property int $entity_class_id
 * @property int $entity_id
 * @property string $embed_url
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property CoreEntityClass $entityClass
 * @property User $createdBy
 * @property User $updatedBy
 */
class CmsEntityVideo extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%cms_entity_video}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['entity_class_id', 'entity_id'], 'required'],
            [['entity_class_id', 'entity_id', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['embed_url'], 'string', 'max' => 255],
            [['entity_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreEntityClass::className(), 'targetAttribute' => ['entity_class_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'entity_class_id' => 'Entity Class ID',
            'entity_id' => 'Entity ID',
            'embed_url' => 'Embed Url',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEntityClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreEntityClass::className(), ['id' => 'entity_class_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    public static function setEntityVideo($entity_class, $entity_id, $embed_url)
    {
        $modelCmsEntityVideo = new self();

        if (is_numeric($entity_class)) {
            $modelCoreEntityClass = \common\modules\core\models\CoreEntityClass::findOne(['id' => $entity_class, 'status' => 'active']);
        } else {
            $modelCoreEntityClass = \common\modules\core\models\CoreEntityClass::findOne(['name' => $entity_class, 'status' => 'active']);
        }
        
        if (self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $entity_id, 'status' => 'active'])->exists()) {   // Model already exists
            $modelCmsEntityVideo = self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $entity_id, 'status' => 'active'])->one();

            $modelCmsEntityVideo->embed_url = $embed_url['value'];
        } else {    // Create a new model
            $modelCmsEntityVideo->entity_class_id = $modelCoreEntityClass->id;
            $modelCmsEntityVideo->entity_id = $entity_id;
            
            $modelCmsEntityVideo->embed_url = $embed_url['value'];
            $modelCmsEntityVideo->status = 'active';
        }

        return $modelCmsEntityVideo;
    }
}
