<?php

namespace common\modules\cms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%cms_event}}".
 *
 * @property int $id
 * @property int $featured_image
 * @property string $start
 * @property string $end
 * @property string $venue
 * @property string $organizer
 * @property string $entry_fee
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property AssetManager $featuredImage
 * @property Value $values
 */
class CmsEvent extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%cms_event}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['featured_image', 'created_by', 'updated_by'], 'integer'],
            [['start', 'end'], 'required'],
            [['start', 'end'], 'safe'],
            [['metadata', 'status'], 'string'],
            [['venue', 'organizer', 'entry_fee'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['featured_image' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'featured_image' => 'Featured Image',
            'start' => 'Start',
            'end' => 'End',
            'venue' => 'Venue',
            'organizer' => 'Organizer',
            'entry_fee' => 'Entry Fee',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'featured_image']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'event', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'event', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveEvent($post_data, $event_id = 0)
    {
        $session = Yii::$app->session;

        $modelCmsEvent = new self();
        $modelCoreValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();
        
        try {
            if (isset($event_id) && $event_id > 0 && self::find()->where(['id' => $event_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelCmsEvent = self::findOne(['id' => $event_id, 'status' => 'active']);

                $modelCmsEvent->start = $post_data['CmsEvent']['start'];
                $modelCmsEvent->end = $post_data['CmsEvent']['end'];
                $modelCmsEvent->venue = $post_data['CmsEvent']['venue'];
                $modelCmsEvent->organizer = $post_data['CmsEvent']['organizer'];
                $modelCmsEvent->entry_fee = $post_data['CmsEvent']['entry_fee'];
                $modelCmsEvent->featured_image = $post_data['CmsEvent']['featured_image'];
            } else {    // Create a new model
                $modelCmsEvent = new self();

                $modelCmsEvent->start = $post_data['CmsEvent']['start'];
                $modelCmsEvent->end = $post_data['CmsEvent']['end'];
                $modelCmsEvent->venue = $post_data['CmsEvent']['venue'];
                $modelCmsEvent->organizer = $post_data['CmsEvent']['organizer'];
                $modelCmsEvent->entry_fee = $post_data['CmsEvent']['entry_fee'];
                $modelCmsEvent->featured_image = $post_data['CmsEvent']['featured_image'];
                $modelCmsEvent->status = 'active';
            }

            if ($modelCmsEvent->validate()) {
                $modelCmsEvent->save();

                foreach ($post_data['CmsEvent'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }

                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();
                            $modelCoreValue = CoreValue::setEntityValue('event', $key, $modelCmsEvent->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }
                }

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Event saved successfully.");

                    return $modelCmsEvent->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Yii::$app->Globals->renderErrors($modelCoreValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save event: " . Yii::$app->Globals->renderErrors($modelCmsEvent->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
