<?php

namespace common\modules\cms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%cms_faq}}".
 *
 * @property int $id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property Value $values
 */
class CmsFaq extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%cms_faq}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['metadata', 'status'], 'string'],
            [[], 'required'],
            [['created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'faq', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'faq_question', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'faq', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
    
    public function saveFaq($post_data, $faq_id = 0)
    {
        $session = Yii::$app->session;

        $modelCmsFaq = new self();
        $modelCmsEntityKeyword = new CmsEntityKeyword();
        $modelCoreValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($faq_id) && $faq_id > 0 && self::find()->where(['id' => $faq_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelCmsFaq = self::findOne(['id' => $faq_id, 'status' => 'active']);

            } else {    // Create a new model
                $modelCmsFaq = new self();

                $modelCmsFaq->status = 'active';
            }

            if ($modelCmsFaq->validate()) {
                $modelCmsFaq->save();

                foreach ($post_data['CmsFaq'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }

                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();
                            $modelCoreValue = CoreValue::setEntityValue('faq', $key, $modelCmsFaq->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }
                    }
                }

                // Save faq keywords
                foreach ((array) $post_data['CmsEntityKeyword']['keyword_id'] as $key => $value) {
                    if (isset($value) && !empty($value)) {
                        if (CmsEntityKeyword::find()->where(['entity_class_id' => CoreEntityClass::getAttr(['id'], ['name' => 'faq']), 'entity_id' => $modelCmsFaq->id, 'keyword_id' => $value, 'status' => 'active'])->exists()) {
                            $modelCmsEntityKeyword = CmsEntityKeyword::find()->where(['entity_class_id' => CoreEntityClass::getAttr(['id'], ['name' => 'faq']), 'entity_id' => $modelCmsFaq->id, 'keyword_id' => $value, 'status' => 'active'])->one();

                            $modelCmsEntityKeyword->keyword_id = $value;
                        } else {
                            $modelCmsEntityKeyword = new CmsEntityKeyword();

                            $modelCmsEntityKeyword->entity_class_id = CoreEntityClass::getAttr(['id'], ['name' => 'faq']);
                            $modelCmsEntityKeyword->entity_id = $modelCmsFaq->id;
                            $modelCmsEntityKeyword->keyword_id = $value;
                            $modelCmsEntityKeyword->status = 'active';
                        }

                        if ($modelCmsEntityKeyword->validate()) {
                            $modelCmsEntityKeyword->save();
                        } else {
                            $transaction->rollBack();

                            $breakFlag = true;
                        }
                    }
                }

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Faq saved successfully.");

                    return $modelCmsFaq->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Yii::$app->Globals->renderErrors($modelCoreValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save Blog: " . Yii::$app->Globals->renderErrors($modelCmsFaq->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
