<?php

namespace common\modules\cms\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\cms\models\CmsFaq;

/**
 * CmsFaqSearch represents the model behind the search form of `common\modules\cms\models\CmsFaq`.
 */
class CmsFaqSearch extends CmsFaq
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'created_at', 'updated_at', 'status', 'faq_question'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['faq_question']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = CmsFaq::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['values']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['name'] = ['asc' => ['core_value.value' => SORT_ASC], 'desc' => ['core_value.value' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_cms_faq.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_cms_faq.status', $this->status])
            ->andFilterWhere(['like', 'mod_cms_faq.id', $this->id])
            ->andFilterWhere(['like', 'mod_core_value.value', $this->faq_question])
            ->andFilterWhere(['like', 'mod_cms_faq.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_cms_faq.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_cms_faq.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_cms_faq.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
