<?php

namespace common\modules\cms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%cms_news}}".
 *
 * @property int $id
 * @property int|null $featured_image
 * @property string|null $media
 * @property string|null $source
 * @property string|null $publication_date
 * @property int|null $event_id
 * @property string|null $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property CmsEvent $event
 * @property AssetManager $featuredImage
 * @property User $updatedBy
 */
class CmsNews extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%cms_news}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['featured_image', 'event_id', 'created_by', 'updated_by'], 'integer'],
            [['publication_date'], 'safe'],
            [['metadata', 'status'], 'string'],
            [[], 'required'],
            [['media', 'source'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::class, 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::class, 'targetAttribute' => ['updated_by' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::class, 'targetAttribute' => ['featured_image' => 'id']],
            [['event_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\cms\models\CmsEvent::class, 'targetAttribute' => ['event_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'featured_image' => 'Featured Image',
            'media' => 'Media',
            'source' => 'Source',
            'publication_date' => 'Publication Date',
            'event_id' => 'Event ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::class, ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEvent()
    {
        return $this->hasOne(\common\modules\cms\models\CmsEvent::class, ['id' => 'event_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::class, ['id' => 'featured_image']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::class, ['id' => 'updated_by']);
    }

    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'cms_news', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'cms_news', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveNews($post_data, $news_id = 0)
    {
        $session = Yii::$app->session;

        $modelCmsNews = new self();
        $modelCmsEntityKeyword = new CmsEntityKeyword();
        $modelCoreValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($news_id) && $news_id > 0 && self::find()->where(['id' => $news_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelCmsNews = self::findOne(['id' => $news_id, 'status' => 'active']);

                $modelCmsNews->featured_image = $post_data['CmsNews']['featured_image'];
                $modelCmsNews->media = $post_data['CmsNews']['media'];
                $modelCmsNews->source = $post_data['CmsNews']['source'];
                $modelCmsNews->publication_date = $post_data['CmsNews']['publication_date'];
                $modelCmsNews->event_id = $post_data['CmsNews']['event_id'];
                $modelCmsNews->metadata = isset($post_data['CmsNewsMetadata']) ? Globals::setJson($post_data['CmsNewsMetadata']) : '';
            } else {    // Create a new model
                $modelCmsNews = new CmsNews();

                $modelCmsNews->featured_image = $post_data['CmsNews']['featured_image'];
                $modelCmsNews->media = $post_data['CmsNews']['media'];
                $modelCmsNews->source = $post_data['CmsNews']['source'];
                $modelCmsNews->publication_date = $post_data['CmsNews']['publication_date'];
                $modelCmsNews->event_id = $post_data['CmsNews']['event_id'];
                $modelCmsNews->metadata = isset($post_data['CmsNewsMetadata']) ? Globals::setJson($post_data['CmsNewsMetadata']) : '';
                $modelCmsNews->status = 'active';
            }

            if ($modelCmsNews->validate()) {
                $modelCmsNews->save();

                foreach ($post_data['CmsNews'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }

                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();
                            $modelCoreValue = CoreValue::setEntityValue('cms_news', $key, $modelCmsNews->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }                     
                }

                // Save news keywords
                foreach ((array) $post_data['CmsEntityKeyword']['keyword_id'] as $key => $value) {
                    if (isset($value) && !empty($value)) {
                        if (CmsEntityKeyword::find()->where(['entity_class_id' => CoreEntityClass::getAttr(['id'], ['name' => 'cms_news']), 'entity_id' => $modelCmsNews->id, 'keyword_id' => $value, 'status' => 'active'])->exists()) {
                            $modelCmsEntityKeyword = CmsEntityKeyword::find()->where(['entity_class_id' => CoreEntityClass::getAttr(['id'], ['name' => 'cms_news']), 'entity_id' => $modelCmsNews->id, 'keyword_id' => $value, 'status' => 'active'])->one();

                            $modelCmsEntityKeyword->keyword_id = $value;
                        } else {
                            $modelCmsEntityKeyword = new CmsEntityKeyword();

                            $modelCmsEntityKeyword->entity_class_id = CoreEntityClass::getAttr(['id'], ['name' => 'cms_news']);
                            $modelCmsEntityKeyword->entity_id = $modelCmsNews->id;
                            $modelCmsEntityKeyword->keyword_id = $value;
                            $modelCmsEntityKeyword->status = 'active';
                        }

                        if ($modelCmsEntityKeyword->validate()) {
                            $modelCmsEntityKeyword->save();
                        } else {
                            $transaction->rollBack();

                            $breakFlag = true;
                        }
                    }
                }

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "News saved successfully.");

                    return $modelCmsNews->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Globals::renderErrors($modelCoreValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save news: " . Globals::renderErrors($modelCmsNews->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}