<?php

namespace common\modules\cms\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\cms\models\CmsNews;

/**
 * CmsNewsSearch represents the model behind the search form of `common\modules\cms\models\CmsNews`.
 */
class CmsNewsSearch extends CmsNews
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'featured_image', 'event_id', 'created_by', 'updated_by'], 'integer'],
            [['media', 'source', 'publication_date', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = CmsNews::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_cms_news.media', $this->media])
            ->andFilterWhere(['like', 'mod_cms_news.source', $this->source])
            ->andFilterWhere(['like', 'mod_cms_news.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_cms_news.status', $this->status])
            ->andFilterWhere(['like', 'mod_cms_news.id', $this->id])
            ->andFilterWhere(['like', 'mod_cms_news.featured_image', $this->featured_image])
            ->andFilterWhere(['like', 'mod_cms_news.publication_date', $this->publication_date])
            ->andFilterWhere(['like', 'mod_cms_news.event_id', $this->event_id])
            ->andFilterWhere(['like', 'mod_cms_news.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_cms_news.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_cms_news.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_cms_news.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
