<?php

namespace common\modules\cms\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\cms\models\CmsTeamMember;

/**
 * CmsTeamMemberSearch represents the model behind the search form of `common\modules\cms\models\CmsTeamMember`.
 */
class CmsTeamMemberSearch extends CmsTeamMember
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'featured_image', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'created_at', 'updated_at', 'status', 'name'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['name']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = CmsTeamMember::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['values']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['name'] = ['asc' => ['core_value.value' => SORT_ASC], 'desc' => ['core_value.value' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_cms_team_member.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_cms_team_member.status', $this->status])
            ->andFilterWhere(['like', 'mod_cms_team_member.id', $this->id])
            ->andFilterWhere(['like', 'mod_cms_team_member.featured_image', $this->featured_image])
            ->andFilterWhere(['like', 'mod_core_value.value', $this->name])
            ->andFilterWhere(['like', 'mod_cms_team_member.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_cms_team_member.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_cms_team_member.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_cms_team_member.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
