<?php

namespace common\modules\core\components;

use Yii;
use yii\base\Component;
use yii\helpers\Url;
use yii\helpers\Inflector;
use yii\helpers\VarDumper;

class Globals extends Component 
{

    public static function cliProgress($done, $total, $info = "", $width = 100) 
    {
        $perc = round(($done * 100) / $total);
        $bar = round(($width * $perc) / 100);
        
        return sprintf("%s [%s>%s] %s%%\r", $info, str_repeat("=", $bar), str_repeat(" ", $width - $bar), $perc);
    }

    public static function renderErrors($errors)
    {
        $errorStr = '';
        
        $errorStr .= '<ul>';
        
        foreach ($errors as $errors_key => $errors_value) {
            foreach ($errors_value as $key => $value) {
                $errorStr .= '<li>' . $value . '</li>';
            }
        }

        $errorStr .= '</ul>';

        return $errorStr;
    }

    public static function truncateString($string, $length)
    {
        return strlen($string) > $length ? substr($string, 0, $length) . "..." : $string;
    }

	public static function getAssetUploadsAbsolutePath($appAlias = '@backend', $actionId = '') 
	{
		$baseUploadsPath = Yii::getAlias($appAlias);
		$relativePath = '/uploads/';

		return $baseUploadsPath . $relativePath . $actionId;
	}

	public static function getAssetUploadsRelativePath($actionId = '') 
	{
		$relativePath = '/uploads/';

		return $relativePath . $actionId . '/';
	}

	public static function getAssetUrl($asset_id)
	{
		$modelAsset = Asset::findOne(['id' => $asset_id, 'status' => 'active']);

		if (!isset($modelAsset) || empty($modelAsset)) {
			return null;
		} else {
			return Url::to(Yii::getAlias('@baseurl') . '/' . str_replace('@', '', $modelAsset->app_alias) . $modelAsset->path . $modelAsset->name, true);
		}	
	}

	public static function getAssetPath($asset_id)
	{
		$modelAsset = Asset::findOne(['id' => $asset_id, 'status' => 'active']);

		if (!isset($modelAsset) || empty($modelAsset)) {
			return null;
		} else {
			return Yii::getAlias($modelAsset->app_alias) . $modelAsset->path . $modelAsset->name;
		}	
	}

	public static function isDateInRange($date, $start_date, $end_date)
	{
		// Convert to timestamp
		$start_ts = strtotime($start_date);
		$end_ts = strtotime($end_date);
		$user_ts = strtotime($date);

		// Check that user date is between start & end
		return (($user_ts >= $start_ts) && ($user_ts <= $end_ts));
	}

    public static function hasExpired($expiration_date)
    {
        // Convert to timestamp
        $expiration_ts = strtotime($expiration_date);
        $user_ts = strtotime(date('Y-m-d H:s:i'));
		
        return ($user_ts > $expiration_ts);
    }

	public static function recordLoginAttempt($user, $success, $app_name)
    {
        $modelLoginLogs = new LoginLogs();

        $modelLoginLogs->user = $user;
        $modelLoginLogs->app_name = $app_name;
        $modelLoginLogs->success = $success;
        $modelLoginLogs->ip_address = $this->getIp();
        $modelLoginLogs->timestamp = date('Y-m-d h:m:s');
        $modelLoginLogs->created_at = date('Y-m-d h:m:s');
        $modelLoginLogs->updated_at = date('Y-m-d h:m:s');

        if ($modelLoginLogs->validate()) {
            $modelLoginLogs->save();
        }
    }

    public static function sendMail($mailTemplate = array(), $data = array(), $to = '', $subject)
    {
        Yii::$app->mailer->compose(['html' => $mailTemplate['html'], 'text' => $mailTemplate['text']], ['data' => $data])
            ->setFrom(Yii::$app->params['email.from'])
            ->setTo($to)
            ->setSubject($subject)
            ->send();
    }

    public static function isStringNull($string)
    {
        if (!isset($string) || empty($string) || trim($string) == '') {
            return '';
        } else {
            return $string;
        }
    }

    public static function renderAppData($string)
    {
        return stripslashes(htmlentities($string));
    }

    public static function isUserSubscriptionValid($status, $expiration)
    {
        if ($status == 'active' && self::hasExpired(date('Y-m-d'), $expiration)) {
            return true;
        } else {
            return false;
        }
    }

    public static function renderPageTitle($filename)
    {
        return ucwords(str_replace('-', ' ', $filename));
    }

    public static function renderTagsAsLabels($tags, $css_class = 'default')
    {
        $tags_html = '';
        $temp_tags = explode(',', $tags);

        foreach ($temp_tags as $tag_key => $tag_value) {
            $tags_html .= '<span class="label label-' . $css_class . '">';
            $tags_html .= $tag_value;
            $tags_html .= '</span> ';
        }

        return $tags_html;
    }

    public static function isHomePage()
    {
        $controller = Yii::$app->controller;
        $default_controller = Yii::$app->defaultRoute;
        
        return (($controller->id === $default_controller) && ($controller->action->id === $controller->defaultAction)) ? true : false;
    }

    public static function countDecimalDigits($number)
    {
        $tmp = explode('.', $number);
        $count = 0;

        if (isset($tmp[1])) {
            $count = strlen($tmp[1]);
        } else {
            $count = 0;
        }

        if ($count < 2) {
            $count = 2;
        }

        return $count;
    }

    public static function customNumberFormat($number, $decimals = '', $sep1 = '', $sep2 = '') 
    {
        if (($number * pow(10 , $decimals + 1) % 10 ) == 5) // if next not significant digit is 5
            $number -= pow(10 , - ($decimals + 1));

        return number_format($number, $decimals, $sep1, $sep2);
    }

    public static function formatNumber($number = 0, $round = array('round' => true, 'precision' => 2), $dec_point = ".", $thousands_sep = "")
    {
        if ($round['round']) {
            $number = number_format((float) $number, $round['precision'], $dec_point, $thousands_sep);
        } else {
            $number = self::customNumberFormat($number, self::countDecimalDigits($number), $dec_point, $thousands_sep);
        }

        return $number;
    }

    public static function setJson($json_array = array()) 
    {
        if (!empty($json_array)) {
            $json = json_encode($json_array);

            return $json;
        } else {
            return '';
        }
    }

    public static function getArray($array_json = '') 
    {
        if (!empty($array_json)) {
            $arr = json_decode($array_json, true);

            return $arr;
        } else {
            return '';
        }
    }

    public static function dnsExists($record = '', $value = '')
    {
        if (isset($record, $value) && !empty($record) && !empty($value)) {
            switch ($record) {
                case 'MX':
                    list($username, $domain) = explode('@', $value);

                    return checkdnsrr($domain, $record);

                    break;
                
                default:
                    return checkdnsrr($value, $record);

                    break;
            }
        }

        return false;
    }
	
    public static function getMerchantDescriptor($domain = '', $is_fallback = false, $override = '')
    {
        $payment_gateway = Yii::$app->params['payment_gateway'];

        if ($is_fallback === true) {
            $payment_gateway = Yii::$app->params['payment_gateway_fallback'];
        }

        if (!empty($override)) {
            $payment_gateway = $override;
        }

        if ($payment_gateway == 'stripe') {
            $domain = str_replace('www.', '', strtolower($domain));
            $domain = str_replace('.', '', $domain);

            if (Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] == 'MSTRD*') {    // mustardim.com
                $domain = Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] . time();

                return $domain;
            } elseif (Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] == 'VPN ') {    // buranvpn.com
                $domain = substr($domain, 0, 18);

                $domain = Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] . strtoupper($domain);

                return $domain;
            } elseif (Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] == 'WEBS*') {    // dynawebhosting.com
                $domain = substr($domain, 0, 17);

                $domain = Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] . strtoupper($domain);

                return $domain;
            } elseif (Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] == ' MNGH') {    // manganohost.com
                $domain = substr($domain, 0, 17);

                $domain = strtoupper($domain) . Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'];

                return $domain;
            } elseif (Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] == 'HOSTING') {    // hostingbeans.com
                $domain = 'HOSTING';

                return $domain;
            } elseif (Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] == 'KWHSTZ') {    // kiwihostzing.com
                $domain = substr($domain, 0, 22);

                $domain = strtoupper($domain);

                return $domain;
            } elseif (Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] == 'NZHST') {    // newzealhost.com
                $domain = substr($domain, 0, 16);

                $domain = strtoupper($domain) . '*NZHST';

                return $domain;
            } elseif (Yii::$app->params['payment_gateways']['stripe'][Yii::$app->params['mode']]['merchant_descriptor'] == 'WEB VPN') {    // stratovpn.com
                $domain = 'WEB VPN';

                return $domain;
            }
        } else if ($payment_gateway == 'buranvpn_com:domain' || $payment_gateway == 'buranvpn_com:ssl' || $payment_gateway == 'buranvpn_com:privacy') {
            $tmp = explode(':', $payment_gateway);
            $payment_gateway = $tmp[0];

            switch ($tmp[1]) {
                case 'domain':
                    $domain = substr($domain, 0, 18);
                    $domain = 'VPN ' . strtoupper($domain);

                    break;

                case 'ssl':
                    $domain = substr($domain, 0, 15);
                    $domain = 'VPNSEC ' . strtoupper($domain);
                    
                    break;

                case 'privacy':
                    $domain = substr($domain, 0, 15);
                    $domain = 'VPNSEC ' . strtoupper($domain);
                    
                    break;
                
                default:
                    $domain = substr($domain, 0, 18);
                    $domain = 'VPN ' . strtoupper($domain);

                    break;
            }

            return $domain;
        } else if ($payment_gateway == 'mustardim_com:domain' || $payment_gateway == 'mustardim_com:ssl' || $payment_gateway == 'mustardim_com:privacy') {
            $tmp = explode(':', $payment_gateway);
            $payment_gateway = $tmp[0];

            switch ($tmp[1]) {
                case 'domain':
                    $domain = substr($domain, 0, 18);
                    $domain = 'VPN ' . strtoupper($domain);

                    break;

                case 'ssl':
                    $domain = substr($domain, 0, 15);
                    $domain = 'VPNSEC ' . strtoupper($domain);
                    
                    break;

                case 'privacy':
                    $domain = substr($domain, 0, 15);
                    $domain = 'VPNSEC ' . strtoupper($domain);
                    
                    break;
                
                default:
                    $domain = substr($domain, 0, 18);
                    $domain = 'VPN ' . strtoupper($domain);

                    break;
            }

            return $domain;
        } else if ($payment_gateway == 'netweb_io:domain' || $payment_gateway == 'netweb_io:ssl' || $payment_gateway == 'netweb_io:privacy') {
            $tmp = explode(':', $payment_gateway);
            $payment_gateway = $tmp[0];

            switch ($tmp[1]) {
                case 'domain':
                    $domain = substr($domain, 0, 18);
                    $domain = 'VPN ' . strtoupper($domain);

                    break;

                case 'ssl':
                    $domain = substr($domain, 0, 15);
                    $domain = 'VPNSEC ' . strtoupper($domain);
                    
                    break;

                case 'privacy':
                    $domain = substr($domain, 0, 15);
                    $domain = 'VPNSEC ' . strtoupper($domain);
                    
                    break;
                
                default:
                    $domain = substr($domain, 0, 18);
                    $domain = 'VPN ' . strtoupper($domain);

                    break;
            }

            return $domain;
        } else if ($payment_gateway == 'authorizenet') {
            return 'WEB RENEW';
        } else {
            return Yii::$app->params['merchant_descriptor'];
        }
    }

    public static function limitExceeded($payment_gateway = '')
    {
        $payment_gateway = (isset($payment_gateway) && !empty($payment_gateway)) ? $payment_gateway : Yii::$app->params['payment_gateway'];

        $fhtpay = LogsFhtpay::find()->andWhere(['>', 'created_at', date("Y-m-d")])->count();
        $stripe = LogsStripe::find()->andWhere(['>', 'created_at', date("Y-m-d")])->count();
        $authorizenet = LogsAuthorizenet::find()->andWhere(['>', 'created_at', date("Y-m-d")])->count();
        
        if (Yii::$app->params[$payment_gateway][Yii::$app->params['mode']]['limit'] == 0) {
            return false;
        } elseif ($$payment_gateway < Yii::$app->params[$payment_gateway][Yii::$app->params['mode']]['limit']) {
            return false;
        } else {
            return true;
        }
    }

    public static function getTimezones()
    {
        $timezones = array();
        $t_ids = \DateTimeZone::listIdentifiers(\DateTimeZone::ALL);

        foreach ($t_ids as $t_id) {
            $timezones[$t_id] = $t_id;
        }

        return $timezones;
    }

    public static function dynagridMetadataColumns($metadata_name = '', $invisible = [])
    {
        if (!isset($metadata_name) || empty($metadata_name)) {
            return [];
        }

        $metadata_columns = array();

        foreach (Yii::$app->params[$metadata_name] as $key => $value) {
            $visible = $filter = $filter_type = $filter_widget_options = $filter_input_options = '';

            switch ($value['type']) {
                case 'select':
                    $filter = Yii::$app->params[$metadata_name][$key]['data'];
                    $visible = (in_array($key, $invisible)) ? false : true;
                    
                    break;

                case 'select2':
                    $filter = Yii::$app->params[$metadata_name][$key]['data'];
                    $filter_type = \kartik\grid\GridView::FILTER_SELECT2;
                    $filter_widget_options = ['pluginOptions' => ['allowClear' => true]];
                    $filter_input_options = ['placeholder' => 'Select'];
                    $visible = (in_array($key, $invisible)) ? false : true;
                    
                    break;

                default:
                    $visible = (in_array($key, $invisible)) ? false : true;

                    break;
            }

            $tmp = array();

            $tmp = [
                'attribute' => 'metadata_' . $key, 
                'label' => Inflector::titleize($key, true),
                'visible' => $visible,
            ];

            if (isset(Yii::$app->params[$metadata_name][$key]['template'])) {
                $tmp['value'] = function ($m, $k, $i, $w) use ($metadata_name, $key) {
                    if (isset($m['metadata_' . $key]) && !empty($m['metadata_' . $key])) {
                        if (isset(Yii::$app->params[$metadata_name][$key]['template'][$m['metadata_' . $key]])) {
                            return str_replace('{{' . $metadata_name . '.' . $key . '}}', $m['metadata_' . $key], Yii::$app->params[$metadata_name][$key]['template'][$m['metadata_' . $key]]);
                        } else {
                            return $m['metadata_' . $key];
                        }
                    } else {
                        return '';
                    }
                };

                $tmp['format'] = 'raw';
            }

            if (isset($filter) && !empty($filter)) {
                $tmp['filter'] = $filter;
            }

            if (isset($filter_type) && !empty($filter_type)) {
                $tmp['filterType'] = $filter_type;
            }

            if (isset($filter_widget_options) && !empty($filter_widget_options)) {
                $tmp['filterWidgetOptions'] = $filter_widget_options;
            }

            if (isset($filter_input_options) && !empty($filter_input_options)) {
                $tmp['filterInputOptions'] = $filter_input_options;
            }

            $metadata_columns[] = $tmp;
        }

        return $metadata_columns;
    }
}
