<?php

namespace common\modules\core\components;

use Yii;
use yii\base\Component;

class Shortcode extends Component 
{

    public static function renderShortcode($content = null, $ignore_html = false)
    {
        $shortcode_tags = Yii::$app->params['shortcode_arr'];

        if (false === strpos($content, '[')) {
           return $content;
        }

        preg_match_all('@\[([^<>&/\[\]\x00-\x20=]++)@', $content, $matches);
        $tagnames = array_intersect(array_keys($shortcode_tags), $matches[1]);

        if (empty($tagnames)) {
            return $content;
        }

        $trans = array('&#91;' => '&#091;', '&#93;' => '&#093;');
        $content = strtr($content, $trans);
        $trans = array('[' => '&#91;', ']' => '&#93;');

        $pattern = self::getShortcodeRegex($tagnames);  
        $textarr = self::splitHtml($content);
        $content = implode('', $textarr);
        global $shortCodeNo;
        $shortCodeNo = 0;
		
        $content = preg_replace_callback("/$pattern/", function($pattern) use ($shortcode_tags) {
            if ($pattern[1] == '[' && $pattern[6] == ']') {
                return substr($pattern[0], 1, -1);
            }

            $tag = $pattern[2];
			
            if (array_key_exists($tag,$shortcode_tags)) {
                $GLOBALS['shortCodeNo']++;
				
                $func = $shortcode_tags[$tag]['function'];
                $content = self::$func($pattern, $GLOBALS['shortCodeNo']);
            }

            return $content;
        }, $content);
        
        return $content;
    }

    public static function splitHtml($input) 
    {
        return preg_split(self::getSplitHtmlRegex(), $input, -1, PREG_SPLIT_DELIM_CAPTURE);
    }

    public static function getShortcodeRegex($tagnames = null)
    {
        $tagregexp = join('|', array_map('preg_quote', $tagnames));

        return
              '\\['                              // Opening bracket
            . '(\\[?)'                           // 1: Optional second opening bracket for escaping shortcodes: [[tag]]
            . "($tagregexp)"                     // 2: Shortcode name
            . '(?![\\w-])'                       // Not followed by word character or hyphen
            . '('                                // 3: Unroll the loop: Inside the opening shortcode tag
            .     '[^\\]\\/]*'                   // Not a closing bracket or forward slash
            .     '(?:'
            .         '\\/(?!\\])'               // A forward slash not followed by a closing bracket
            .         '[^\\]\\/]*'               // Not a closing bracket or forward slash
            .     ')*?'
            . ')'
            . '(?:'
            .     '(\\/)'                        // 4: Self closing tag ...
            .     '\\]'                          // ... and closing bracket
            . '|'
            .     '\\]'                          // Closing bracket
            .     '(?:'
            .         '('                        // 5: Unroll the loop: Optionally, anything between the opening and closing shortcode tags
            .             '[^\\[]*+'             // Not an opening bracket
            .             '(?:'
            .                 '\\[(?!\\/\\2\\])' // An opening bracket not followed by the closing shortcode tag
            .                 '[^\\[]*+'         // Not an opening bracket
            .             ')*+'
            .         ')'
            .         '\\[\\/\\2\\]'             // Closing shortcode tag
            .     ')?'
            . ')'
            . '(\\]?)';                          // 6: Optional second closing brocket for escaping shortcodes: [[tag]]
    }

    public static function getSplitHtmlRegex()
    {
        static $regex;

        if (!isset($regex)) {
            $comments =
                      '!'           // Start of comment, after the <.
                    . '(?:'         // Unroll the loop: Consume everything until --> is found.
                    .     '-(?!->)' // Dash not followed by end of comment.
                    .     '[^\-]*+' // Consume non-dashes.
                    . ')*+'         // Loop possessively.
                    . '(?:-->)?';   // End of comment. If not found, match all input. 
            $cdata =
                      '!\[CDATA\['  // Start of comment, after the <.
                    . '[^\]]*+'     // Consume non-].
                    . '(?:'         // Unroll the loop: Consume everything until ]]> is found.
                    .     '](?!]>)' // One ] not followed by end of comment.
                    .     '[^\]]*+' // Consume non-].
                    . ')*+'         // Loop possessively.
                    . '(?:]]>)?';   // End of comment. If not found, match all input. 
            $escaped =
                      '(?='           // Is the element escaped?
                    .    '!--'
                    . '|'
                    .    '!\[CDATA\['
                    . ')'
                    . '(?(?=!-)'      // If yes, which type?
                    .     $comments
                    . '|'
                    .     $cdata
                    . ')'; 
            $regex =
                      '/('              // Capture the entire match.
                    .     '<'           // Find start of element.
                    .     '(?'          // Conditional expression follows.
                    .         $escaped  // Find end of escaped element.
                    .     '|'           // ... else ...
                    .         '[^>]*>?' // Find end of normal element.
                    .     ')'
                    . ')/';
        }

        return $regex;
    }

    public static function getAttrValue($element, $attr)
    {
        $attr_exp = join('|', array_map('preg_quote', $attr));

        preg_match_all('/(' . $attr_exp . ')="([^"]*)"/U', $element, $match, PREG_SET_ORDER);
        
        $attr_array = array();

        foreach($match as $key => $value) {
            $attr_array[$value[1]] = $value[2];
        }

        return $attr_array;
    }

    public static function bsGrid($pattern)
    {
        $content = $pattern[0];

        $tag_pattern = self::getShortcodeRegex(array('bsGrid'));
        $content = preg_replace_callback("/$tag_pattern/", function($tag_pattern) {
            $template = '';
            $attr_arr = array('container');
            $val_arr = self::getAttrValue($tag_pattern[3], $attr_arr);
            $container = '';

            if (array_key_exists('container', $val_arr)) {
                $container = $val_arr['container'];
            }

            if ($tag_pattern[2] == 'bsGrid' && !empty($tag_pattern[5])) {
                if (empty($container)) {
                    $template = '<div>' . $tag_pattern[5] . '</div>';
                } else {
                    $template = '<div class="' . $container . '">' . $tag_pattern[5] . '</div>';
                }
            }

            return $template;
        }, $content);

        $tag_pattern = self::getShortcodeRegex(array('bsGrid-row'));
        $content = preg_replace_callback("/$tag_pattern/", function($tag_pattern) {
            $template = '';
            $attr_arr = array('class');
            $val_arr = self::getAttrValue($tag_pattern[3], $attr_arr);
            $class = '';

            if (array_key_exists('class', $val_arr)) {
                $class = $val_arr['class'];
            }

            if ($tag_pattern[2] == 'bsGrid-row' && !empty($tag_pattern[5])) {
                $template = '<div class="' . $class . '">' . $tag_pattern[5] . '</div>';
            }

            return $template;
        }, $content);

        $tag_pattern = self::getShortcodeRegex(array('bsGrid-column'));
        $content = preg_replace_callback("/$tag_pattern/", function($tag_pattern) {
            $template = '';
            $attr_arr = array('class');
            $val_arr = self::getAttrValue($tag_pattern[3], $attr_arr);
            $class = '';

            if (array_key_exists('class', $val_arr)) {
                $class = $val_arr['class'];
            }

            if ($tag_pattern[2] == 'bsGrid-column') {
                $template = '<div class="' . $class . '">' . $tag_pattern[5] . '</div>';
            }

            return $template;
        }, $content);
        
        return $content;
    }

    public static function bsButton($pattern)
    {
        $content = $pattern[0];

        $tag_pattern = self::getShortcodeRegex(array('bsButton'));
        $content = preg_replace_callback("/$tag_pattern/", function($tag_pattern) {
            $template = '';
            $attr_arr = array('theme');
            $val_arr = self::getAttrValue($tag_pattern[3], $attr_arr);
            $theme = 'default';

            if (array_key_exists('theme', $val_arr)) {
                $theme = $val_arr['theme'];
            }

            if ($tag_pattern[2] == 'bsButton' && !empty($tag_pattern[5])) {
                $template = '<a class="btn btn-' . $theme . '">' . $tag_pattern[5] . '</a>';
            }

            return $template;
        }, $content);
        
        return $content;
    }

    public static function bsImage($pattern)
    {
        $content = $pattern[0];

        $tag_pattern = self::getShortcodeRegex(array('bsImage'));
        $content = preg_replace_callback("/$tag_pattern/", function($tag_pattern) {
            $template = '';
            $attr_arr = array('src', 'alt', 'class');
            $val_arr = self::getAttrValue($tag_pattern[3], $attr_arr);
            $src = '#';
            $alt = '';
            $class = '';

            if (array_key_exists('src', $val_arr)) {
                $src = $val_arr['src'];
            }

            if (array_key_exists('alt', $val_arr)) {
                $alt = $val_arr['alt'];
            }

            if (array_key_exists('class', $val_arr)) {
                $class = $val_arr['class'];
            }

            if ($tag_pattern[2] == 'bsImage') {
                $template = '<img src="' . $src . '" class="' . $class . '" alt="' . $alt . '">';
            }

            return $template;
        }, $content);
        
        return $content;
    }

    public static function accordion($pattern, $codeNo)
    {
        global $shortCodeNo;
        
		$shortCodeNo = $codeNo;
        $tagnames = array('accordion-item');
        $content = $pattern[0];

        // Replacing [accordion] short code with html structure
        $tag_pattern = self::getShortcodeRegex(array('accordion'));
        
        global $isfirstopen;
        $isfirstopen = 0;

        $content = preg_replace_callback("/$tag_pattern/", function($tag_pattern) {
            $m = '';
            $attr_arr = array('isfirstopen');
            $val_arr = self::getAttrValue($tag_pattern[3], $attr_arr);

            if (array_key_exists('isfirstopen', $val_arr) && $val_arr['isfirstopen'] == 'true') {
                $GLOBALS['isfirstopen'] = 1;
            }

            if ($tag_pattern[2] == 'accordion' && !empty($tag_pattern[5])) {
                $m = '<div class="panel-group" aria-multiselectable="true" role="tablist" id="accordion-'.$GLOBALS['shortCodeNo'].'">' . $tag_pattern[5] . '</div>';
            }

            return $m;
        }, $content);

        // Replacing [accordion-item] with html tags
        $accpattern = self::getShortcodeRegex($tagnames);
        $textarr = self::splitHtml($content);
        $content = implode('', $textarr);
        
        global $value;
        $value = 1;
        global $item_no;
        $item_no = 1;

        $content = preg_replace_callback("/$accpattern/", function($accpattern) use ($isfirstopen) {
            $attr_arr = array('title', 'type', 'class');
            $val_arr = self::getAttrValue($accpattern[3], $attr_arr);
            $id = $title = '';
            $m = '<div class="panel panel-default">';

            if (array_key_exists('id', $val_arr)) {
                $id = $val_arr['id'];
            } else {
                $id = 'accordion-item-' . $GLOBALS['shortCodeNo'] . $GLOBALS['value'];
                
                $GLOBALS['value']++;
            }

            if (array_key_exists('title', $val_arr)) {
                $title = $val_arr['title'];
            } else {
                $title = 'Please set title attribute...';
            }

            $m .= '<div class="panel-heading"><h4 class="panel-title"><a class="accordion-toggle" data-toggle="collapse" data-parent="#accordion-'.$GLOBALS['shortCodeNo'].'" href="#' . $id . '">' . $title . '</a></h4></div>';
            $m .= '<div id="' . $id . '" class="accordion-body collapse';

            if ($isfirstopen && $GLOBALS['item_no'] == '1') {
                $m .= ' in';
            }

            $m .= '"><div class="panel-body"><p>' . $accpattern[5] . '</p></div></div>';
            $m .= '</div>';

            $GLOBALS['item_no']++;

            return $m;
        }, $content);

        return $content;
    }

    public static function toggle($pattern)
    {
        $tagnames = array('toggle-topic', 'toggle-desp');
        $trans = array('&#91;' => '&#091;', '&#93;' => '&#093;');
        $content = strtr($pattern[0], $trans);
        $trans = array('[' => '&#91;', ']' => '&#93;');
        $tag_pattern = self::getShortcodeRegex(array('toggle'));
        $textarr = self::splitHtml($content);
        $content = implode('', $textarr);

        $content = preg_replace_callback("/$tag_pattern/", function($tag_pattern) use ($content) {
            $m = $content;

            if ($tag_pattern[2] == 'toggle' && !empty($tag_pattern[5])) {
                $m = '<div class="toggle toggle-primary" data-plugin-toggle>' . $tag_pattern[5] . '</div>';
            }

            return $m;
        }, $content);

        $accpattern = self::getShortcodeRegex($tagnames);  
        $textarr = self::splitHtml($content);
        $content = implode('', $textarr);

        $content = preg_replace_callback("/$accpattern/", function($accpattern) {
            if ($accpattern[2] == 'toggle-topic' && !empty($accpattern[5])) {
                $m = '<section class="toggle"><label>' . $accpattern[5] . '</label>';
            }

            if ($accpattern[2] == 'toggle-desp' && !empty($accpattern[5])) {
                $m = '<div class="toggle-content">' . $accpattern[5] . '</div></section>';
            }

            return $m;
        }, $content);

        return $content;
    }

    public static function testimonial($pattern)
    {
        $tagnames = array('testimonial-item');
        $content = $pattern[0];

        // Replacing [accordion] short code with html structure
        $tag_pattern = self::getShortcodeRegex(array('testimonials'));
        
        $content = preg_replace_callback("/$tag_pattern/", function($tag_pattern) {
            $m = '';

            if ($tag_pattern[2] == 'testimonials' && !empty($tag_pattern[5])) {
                $m = '<div class="testimonials-wrap">' . $tag_pattern[5] . '</div>';
            }

            return $m;
        }, $content);

        $accpattern = self::getShortcodeRegex($tagnames);  
        $textarr = self::splitHtml($content);
        $content = implode('', $textarr);

        global $value;
        $value = 1;
        global $item_no;
        $item_no = 1;

        $content = preg_replace_callback("/$accpattern/", function($accpattern) {
            $attr_arr = array('author', 'design');
            $val_arr = self::getAttrValue($accpattern[3], $attr_arr);
            $author = 'A Top Executive';
            $class = 'col-sm-8';
            $design = '';

            if ($GLOBALS['item_no'] % 2 == 0) {
                $class = 'col-sm-offset-4 col-sm-8';
            }

            if (array_key_exists('author', $val_arr)) {
                $author = $val_arr['author'];
            }

            if (array_key_exists('design', $val_arr)) {
                $design = $val_arr['design'];
            }

            $m = '<div class="row">
                    <div class="' . $class . '">
                        <div class="testimonial testimonial-primary">
                            <blockquote>
                                <p>' . $accpattern[5] . '</p>
                            </blockquote>
                            
                            <div class="testimonial-arrow-down"></div>

                            <div class="testimonial-author">
                                <div class="testimonial-author-thumbnail img-thumbnail">
                                    <img src="/img/profile.png" alt="Profile">
                                </div>

                                <p><strong>' . $author . '</strong><span>' . $design . '</span></p>
                            </div>
                        </div>
                    </div>
                </div>';
                
            $GLOBALS['item_no']++;

            return $m;
        }, $content);

        return $content;
    }

    public static function stripContent($content, $letters = 300)
    {
        $content = strip_tags($content);    // strip tags to avoid breaking any html

        if (strlen($content) > 500) {
            $stringCut = substr($content, 0, $letters); // truncate content
            $content = substr($stringCut, 0, strrpos($stringCut, ' ')); // make sure it ends in a word so assassinate doesn't become ass...
        }

        return $content;
    }
}
