<?php

namespace common\modules\core\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;

/**
 * This is the model class for table "{{%core_entity_attribute}}".
 *
 * @property int $id
 * @property int $entity_class_id
 * @property int $attribute_class_id
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreEntityClass $entityClass
 * @property CoreAttributeClass $attributeClass
 */
class CoreEntityAttribute extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%core_entity_attribute}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['entity_class_id', 'attribute_class_id'], 'required'],
            [['entity_class_id', 'attribute_class_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['entity_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreEntityClass::className(), 'targetAttribute' => ['entity_class_id' => 'id']],
            [['attribute_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreAttributeClass::className(), 'targetAttribute' => ['attribute_class_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'entity_class_id' => 'Entity Class ID',
            'attribute_class_id' => 'Attribute Class ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEntityClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreEntityClass::className(), ['id' => 'entity_class_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAttributeClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreAttributeClass::className(), ['id' => 'attribute_class_id']);
    }

    public static function getAttributeClassesOfEntityClass($entity_class)
    {
        $modelCoreEntityClass = new CoreEntityClass();
        $modelCoreEntityAttribute = new self();
        $attributeClasses = array();

        if (is_numeric($entity_class)) {
            if (CoreEntityClass::find()->where(['id' => $entity_class, 'status' => 'active'])->exists()) {
                $modelCoreEntityClass = CoreEntityClass::findOne(['id' => $entity_class, 'status' => 'active']);
            } else {
                return [];
            }
        } else {
            if (CoreEntityClass::find()->where(['name' => $entity_class, 'status' => 'active'])->exists()) {
                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => $entity_class, 'status' => 'active']);
            } else {
                return [];
            }
        }

        $modelCoreEntityAttribute = self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'status' => 'active'])->all();

        foreach ($modelCoreEntityAttribute as $key => $value) {
            $attributeClasses[$value->attribute_class_id] = \common\modules\core\models\CoreAttributeClass::getAttr(['name'], ['id' => $value->attribute_class_id, 'status' => 'active']);
        }

        return $attributeClasses;
    }
}
