<?php

namespace common\modules\core\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%core_entity_class}}".
 *
 * @property int $id
 * @property string $name
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property CmsEntityAsset[] $cmsEntityAssets
 * @property CmsEntityKeyword[] $cmsEntityKeywords
 * @property CmsEntityVideo[] $cmsEntityVideos
 * @property CoreEntityAttribute[] $coreEntityAttributes
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreEntitySite[] $coreEntitySites
 * @property CoreValue[] $coreValues
 * @property SiteNotification[] $siteNotifications
 * @property SiteReview[] $siteReviews
 */
class CoreEntityClass extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%core_entity_class}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name'], 'required'],
            [['created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['name'], 'string', 'max' => 255],
            [['name'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsEntityAssets()
    {
        return $this->hasMany(\common\modules\cms\models\CmsEntityAsset::className(), ['entity_class_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsEntityKeywords()
    {
        return $this->hasMany(\common\modules\cms\models\CmsEntityKeyword::className(), ['entity_class_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCmsEntityVideos()
    {
        return $this->hasMany(\common\modules\cms\models\CmsEntityVideo::className(), ['entity_class_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCoreEntityAttributes()
    {
        return $this->hasMany(\common\modules\core\models\CoreEntityAttribute::className(), ['entity_class_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCoreEntitySites()
    {
        return $this->hasMany(\common\modules\core\models\CoreEntitySite::className(), ['entity_class_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCoreValues()
    {
        return $this->hasMany(\common\modules\core\models\CoreValue::className(), ['entity_class_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSiteNotifications()
    {
        return $this->hasMany(\common\modules\site\models\SiteNotification::className(), ['entity_class_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSiteReviews()
    {
        return $this->hasMany(\common\modules\site\models\SiteReview::className(), ['entity_class_id' => 'id']);
    }
}