<?php

namespace common\modules\core\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\core\models\CoreEntitySite;

/**
 * CoreEntitySiteSearch represents the model behind the search form of `common\modules\core\models\CoreEntitySite`.
 */
class CoreEntitySiteSearch extends CoreEntitySite
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'entity_class_id', 'entity_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = CoreEntitySite::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_core_entity_site.status', $this->status])
            ->andFilterWhere(['like', 'mod_core_entity_site.id', $this->id])
            ->andFilterWhere(['like', 'mod_core_entity_site.entity_class_id', $this->entity_class_id])
            ->andFilterWhere(['like', 'mod_core_entity_site.entity_id', $this->entity_id])
            ->andFilterWhere(['like', 'mod_core_entity_site.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_core_entity_site.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_core_entity_site.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_core_entity_site.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_core_entity_site.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
