<?php

namespace common\modules\core\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\core\models\CoreLog;

/**
 * CoreLogSearch represents the model behind the search form of `common\modules\core\models\CoreLog`.
 */
class CoreLogSearch extends CoreLog
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'level'], 'integer'],
            [['category', 'prefix', 'message'], 'safe'],
            [['log_time'], 'number'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = CoreLog::find();
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_core_log.category', $this->category])
            ->andFilterWhere(['like', 'mod_core_log.prefix', $this->prefix])
            ->andFilterWhere(['like', 'mod_core_log.message', $this->message])
            ->andFilterWhere(['like', 'mod_core_log.id', $this->id])
            ->andFilterWhere(['like', 'mod_core_log.level', $this->level])
            ->andFilterWhere(['like', 'mod_core_log.log_time', $this->log_time]);

        return $dataProvider;
    }
}
