<?php

namespace common\modules\core\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%core_site}}".
 *
 * @property int $id
 * @property string $name
 * @property string $code
 * @property int $language_id
 * @property string $title
 * @property string $meta_description
 * @property string $url
 * @property string $mode
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property CoreEntitySite[] $coreEntitySites
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreLanguage $language
 * @property LogsVisitor[] $logsVisitors
 * @property MailerCampaign[] $mailerCampaigns
 * @property MailerContact[] $mailerContacts
 * @property ShopCart[] $shopCarts
 * @property ShopOrder[] $shopOrders
 * @property ShopRefund[] $shopRefunds
 * @property AccountingTax[] $accountingTaxes
 * @property ShopWishlist[] $shopWishlists
 * @property SiteContact[] $siteContacts
 * @property SiteMail[] $siteMails
 * @property SiteNewsletters[] $siteNewsletters
 * @property SiteNotification[] $siteNotifications
 * @property SiteReview[] $siteReviews
 * @property User[] $users
 * @property UserLogsLogin[] $userLogsLogins
 */
class CoreSite extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%core_site}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'code', 'language_id', 'title', 'meta_description', 'url'], 'required'],
            [['language_id', 'created_by', 'updated_by'], 'integer'],
            [['mode', 'metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['name', 'code', 'title', 'meta_description', 'url'], 'string', 'max' => 255],
            [['name'], 'unique'],
            [['code'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['language_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreLanguage::className(), 'targetAttribute' => ['language_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'code' => 'Code',
            'language_id' => 'Language ID',
            'title' => 'Title',
            'meta_description' => 'Meta Description',
            'url' => 'Url',
            'mode' => 'Mode',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCoreEntitySites()
    {
        return $this->hasMany(\common\modules\core\models\CoreEntitySite::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLanguage()
    {
        return $this->hasOne(\common\modules\core\models\CoreLanguage::className(), ['id' => 'language_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLogsVisitors()
    {
        return $this->hasMany(\common\modules\logs\models\LogsVisitor::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMailerCampaigns()
    {
        return $this->hasMany(\common\modules\mailer\models\MailerCampaign::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMailerContacts()
    {
        return $this->hasMany(\common\modules\mailer\models\MailerContact::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopCarts()
    {
        return $this->hasMany(\common\modules\shop\models\ShopCart::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopOrders()
    {
        return $this->hasMany(\common\modules\shop\models\ShopOrder::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopRefunds()
    {
        return $this->hasMany(\common\modules\shop\models\ShopRefund::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingTaxes()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingTax::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopWishlists()
    {
        return $this->hasMany(\common\modules\shop\models\ShopWishlist::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSiteContacts()
    {
        return $this->hasMany(\common\modules\site\models\SiteContact::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSiteMails()
    {
        return $this->hasMany(\common\modules\site\models\SiteMail::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSiteNewsletters()
    {
        return $this->hasMany(\common\modules\site\models\SiteNewsletters::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSiteNotifications()
    {
        return $this->hasMany(\common\modules\site\models\SiteNotification::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSiteReviews()
    {
        return $this->hasMany(\common\modules\site\models\SiteReview::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUsers()
    {
        return $this->hasMany(\common\modules\user\models\User::className(), ['site_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserLogsLogins()
    {
        return $this->hasMany(\common\modules\user\models\UserLogsLogin::className(), ['site_id' => 'id']);
    }
}