<?php

namespace common\modules\core\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\core\models\CoreSite;

/**
 * CoreSiteSearch represents the model behind the search form of `common\modules\core\models\CoreSite`.
 */
class CoreSiteSearch extends CoreSite
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'language_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'code', 'title', 'meta_description', 'url', 'mode', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = CoreSite::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_core_site.name', $this->name])
            ->andFilterWhere(['like', 'mod_core_site.code', $this->code])
            ->andFilterWhere(['like', 'mod_core_site.title', $this->title])
            ->andFilterWhere(['like', 'mod_core_site.meta_description', $this->meta_description])
            ->andFilterWhere(['like', 'mod_core_site.url', $this->url])
            ->andFilterWhere(['like', 'mod_core_site.mode', $this->mode])
            ->andFilterWhere(['like', 'mod_core_site.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_core_site.status', $this->status])
            ->andFilterWhere(['like', 'mod_core_site.id', $this->id])
            ->andFilterWhere(['like', 'mod_core_site.language_id', $this->language_id])
            ->andFilterWhere(['like', 'mod_core_site.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_core_site.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_core_site.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_core_site.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
