<?php

namespace common\modules\core\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use \common\modules\core\models\CoreLanguage;
use \common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityClass;

/**
 * This is the model class for table "{{%core_value}}".
 *
 * @property int $id
 * @property int $entity_class_id
 * @property int $attribute_class_id
 * @property int $entity_id
 * @property string $value
 * @property int $language_id
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property CoreEntityClass $entityClass
 * @property CoreAttributeClass $attributeClass
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreLanguage $language
 */
class CoreValue extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%core_value}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['entity_class_id', 'attribute_class_id', 'entity_id', 'language_id'], 'required'],
            [['entity_class_id', 'attribute_class_id', 'entity_id', 'language_id', 'created_by', 'updated_by'], 'integer'],
            [['value', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['entity_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreEntityClass::className(), 'targetAttribute' => ['entity_class_id' => 'id']],
            [['attribute_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreAttributeClass::className(), 'targetAttribute' => ['attribute_class_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['language_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreLanguage::className(), 'targetAttribute' => ['language_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'entity_class_id' => 'Entity Class ID',
            'attribute_class_id' => 'Attribute Class ID',
            'entity_id' => 'Entity ID',
            'value' => 'Value',
            'language_id' => 'Language ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEntityClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreEntityClass::className(), ['id' => 'entity_class_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAttributeClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreAttributeClass::className(), ['id' => 'attribute_class_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLanguage()
    {
        return $this->hasOne(\common\modules\core\models\CoreLanguage::className(), ['id' => 'language_id']);
    }

    public static function getEntity($entity_class, $attribute_class, $value)
    {
        if (is_numeric($entity_class)) {
            if (CoreEntityClass::find()->where(['id' => $entity_class, 'status' => 'active'])->exists()) {
                $modelCoreEntityClass = CoreEntityClass::findOne(['id' => $entity_class, 'status' => 'active']);
            } else {
                return false;
            }
        } else {
            if (CoreEntityClass::find()->where(['name' => $entity_class, 'status' => 'active'])->exists()) {
                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => $entity_class, 'status' => 'active']);
            } else {
                return false;
            }
        }

        if (is_numeric($attribute_class)) {
            if (CoreAttributeClass::find()->where(['id' => $attribute_class, 'status' => 'active'])->exists()) {
                $modelCoreAttributeClass = CoreAttributeClass::findOne(['id' => $attribute_class, 'status' => 'active']);
            } else {
                return false;
            }
        } else {
            if (CoreAttributeClass::find()->where(['name' => $attribute_class, 'status' => 'active'])->exists()) {
                $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => $attribute_class, 'status' => 'active']);
            } else {
                return false;
            }
        }
        
        $modelCoreValue = self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'value' => $value, 'status' => 'active'])->asArray()->one();

        return $modelCoreValue['entity_id'];
    }

    public static function getEntityValue($entity_class, $attribute_class, $entity_id, $lang = 'en')
    {
        if (is_numeric($entity_class)) {
            if (CoreEntityClass::find()->where(['id' => $entity_class, 'status' => 'active'])->exists()) {
                $modelCoreEntityClass = CoreEntityClass::findOne(['id' => $entity_class, 'status' => 'active']);
            } else {
                return false;
            }
        } else {
            if (CoreEntityClass::find()->where(['name' => $entity_class, 'status' => 'active'])->exists()) {
                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => $entity_class, 'status' => 'active']);
            } else {
                return false;
            }
        }

        if (is_numeric($lang)) {
            $modelCoreLanguage = CoreLanguage::findOne(['id' => $lang, 'status' => 'active']);
        } else {
            $modelCoreLanguage = CoreLanguage::findOne(['code' => $lang, 'status' => 'active']);
        }

        if (isset($attribute_class) && !empty($attribute_class)) {
            if (is_numeric($attribute_class)) {
                $modelCoreAttributeClass = CoreAttributeClass::findOne(['id' => $attribute_class, 'status' => 'active']);
            } else {
                $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => $attribute_class, 'status' => 'active']);
            }

            if (!self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'entity_id' => $entity_id, 'language_id' => $modelCoreLanguage->id, 'status' => 'active'])->exists()) {
                return false;
            }
            
            $modelCoreValue = self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'entity_id' => $entity_id, 'language_id' => $modelCoreLanguage->id, 'status' => 'active'])->asArray()->one();
            
            return $modelCoreValue['value'];
        } else {
            if (!self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $entity_id, 'language_id' => $modelCoreLanguage->id, 'status' => 'active'])->exists()) {
                return false;
            }
            
            $modelCoreValue = self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $entity_id, 'language_id' => $modelCoreLanguage->id, 'status' => 'active'])->asArray()->all();
            
            return $modelCoreValue;
        }
    }

    public static function setEntityValue($entity_class, $attribute_class, $entity_id, $value, $lang = 'en')
    {
        $modelCoreValue = new self();
		
        if (is_numeric($entity_class)) {
            $modelCoreEntityClass = CoreEntityClass::findOne(['id' => $entity_class, 'status' => 'active']);
        } else {
            $modelCoreEntityClass = CoreEntityClass::findOne(['name' => $entity_class, 'status' => 'active']);
        }

        if (is_numeric($attribute_class)) {
            $modelCoreAttributeClass = CoreAttributeClass::findOne(['id' => $attribute_class, 'status' => 'active']);
        } else {
            $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => $attribute_class, 'status' => 'active']);
        }

        if (is_numeric($lang)) {
            $modelCoreLanguage = CoreLanguage::findOne(['id' => $lang, 'status' => 'active']);
        } else {
            $modelCoreLanguage = CoreLanguage::findOne(['code' => $lang, 'status' => 'active']);
        }
        
		$lang = $modelCoreLanguage->id;
		
        if (self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'entity_id' => $entity_id, 'language_id' => $lang, 'status' => 'active'])->exists()) {   // Model already exists
            $modelCoreValue = self::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'entity_id' => $entity_id, 'language_id' => $lang, 'status' => 'active'])->one();

            $modelCoreValue->value = $value;
        } else {    // Create a new model
            $modelCoreValue->entity_class_id = $modelCoreEntityClass->id;
            $modelCoreValue->attribute_class_id = $modelCoreAttributeClass->id;
            $modelCoreValue->entity_id = $entity_id;
            $modelCoreValue->language_id = $lang;
            
            $modelCoreValue->value = $value;
            $modelCoreValue->status = 'active';
        }

        return $modelCoreValue;
    }

    public static function deleteEntityValue($where)
    {
        $modelCoreValue = new self();

        $where['status'] = 'active';

        $modelCoreValue = self::find()->where($where)->all();

        foreach ($modelCoreValue as $key => $value) {
            $value->softDelete();
        }

        return true;
    }

    public static function getEavData($entity, $attribute = null, $entity_id = null)
    {
        $data = array();

        if (!isset($entity) || empty($entity)) {
            return $data;
        }

        $attributes = array();
        $valueArray = array();
        $modelCoreLanguage = new CoreLanguage();

        $modelCoreLanguage = CoreLanguage::find()->where(['status' => 'active'])->all();

        foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass($entity) as $attributeClassKey => $attributeClassValue) {
            $tmp = array();

            if (isset($attribute, $entity_id) && !empty($attribute) && !empty($entity_id)) {
                $$attributeClassValue = array();
                
                $temp = self::getEntityValue($entity, $attributeClassValue, $entity_id);
                
                if (!isset($temp)) {
                    continue;
                }
            }

            foreach ($modelCoreLanguage as $langKey => $langValue) {
                if (isset($attribute, $entity_id) && !empty($attribute) && !empty($entity_id)) {
                    $tmp[$langValue->id] = self::getEntityValue($entity, $attributeClassValue, $entity_id, $langValue->id);
                } else {
                    $tmp[$langValue->id] = '';
                }
            }
        
            $$attributeClassValue = $tmp;
            $attributes[] = $attributeClassValue;
        }

        $data['modelCoreLanguage'] = $modelCoreLanguage;

        foreach ($attributes as $key => $value) {
            $data[$value] = $$value;
        }

        if (isset($attribute, $entity_id) && !empty($attribute) && !empty($entity_id)) {
            $modelEntityClass = CoreEntityClass::findOne(['name' => $entity, 'status' => 'active']);
            $modelAttributeClass = CoreAttributeClass::findOne(['name' => $attribute, 'status' => 'active']);

            $tmpArray = self::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'status' => 'active'])->asArray()->all();

            foreach ($tmpArray as $key => $value) {
                $valueArray[$value['entity_id']] = $value['value'];
            }

            $data['valueArray'] = $valueArray;
        }

        return $data;
    }
}
