<?php

namespace common\modules\core\models;

use Yii;
use yii\db\Expression;
use yii\behaviors\TimestampBehavior;
use yii\behaviors\BlameableBehavior;
use yii\helpers\Inflector;

use common\modules\cms\models\CmsEntityKeyword;

class GenesysActiveRecord extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
			[
                'class' => BlameableBehavior::className(),
                'createdByAttribute' => 'created_by',
                'updatedByAttribute' => 'updated_by',
                'defaultValue' => 1,
            ],
            [
                'class' => TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }

    public static function setCreatedBy($post_data = array())
    {
        if (isset($post_data['created_by']) && !empty($post_data['created_by'])) {
            return $post_data['created_by'];
        } else {
            if (Yii::$app->user->isGuest) {
                return 1;
            } else {
                return Yii::$app->user->identity->id;
            }
        }
    }

    public static function setUpdatedBy($post_data = array())
    {
        if (isset($post_data['updated_by']) && !empty($post_data['updated_by'])) {
            return $post_data['updated_by'];
        } else {
            if (Yii::$app->user->isGuest) {
                return 1;
            } else {
                return Yii::$app->user->identity->id;
            }
        }
    }

    public function softDelete()
    {
        $this->status = 'inactive';
        
        if ($this->validate()) {
            $this->save();
        } else {
            throw new \yii\web\HttpException(500, 'Unable to delete.');
        }
    }

    public static function getData($where = array(), $andWhere = array(), $eav = false, $entity_class = false, $attribute_class = false)
    {
        $where['status'] = 'active';
        $data = array();
        
        $model = self::find()->where($where)->andWhere($andWhere)->asArray()->all();

        foreach ($model as $key => $record) {
            if ($eav !== false) {
                $data[$key] = $record;
                
                if ($entity_class !== false && !empty($entity_class)) {
                    if ($attribute_class !== false && !empty($attribute_class)) {
                        $tmp = CoreValue::getEntityValue($entity_class, $attribute_class, $record['id']);
                        $data[$key][CoreEntityClass::getAttr(['name'], ['id' => $tmp['entity_class_id']])] = $tmp['value'];
                    } else {
                        $tmp = CoreValue::getEntityValue($entity_class, false, $record['id']);

                        foreach ($tmp as $k => $v) {
                            if (is_array($v)) {
                                $data[$key][CoreAttributeClass::getAttr(['name'], ['id' => $v['attribute_class_id']])] = $v['value'];

                                unset($tmp[$k]);
                            }
                        }
                    }
                }
            } else {
                $data[$key] = $record;
            }
        }

        return $data;
    }

    public static function getAll(
        $k = '', 
        $v = '', 
        $eav = false, 
        $entity_class = '', 
        $attribute_class = '', 
        $where = array(), 
        $andWhere = array(), 
        $relation = array()
    )
    {
        $where['status'] = 'active';
        $result = array();
        
        $model = self::find()->where($where)->andWhere($andWhere)->all();

        if (isset($k) && !empty($k) && isset($v) && !empty($v)) {
            foreach ($model as $modelKey => $modelValue) {
                if (
                    isset($modelValue->$k) && 
                    (
                        (
                            (!is_array($v) && isset($modelValue->$v)) || 
                            (is_array($v) && isset($modelValue->{$v[0]}))
                        ) || 
                        $eav
                    )
                ) {
                    if ($eav && isset($entity_class, $attribute_class) && !empty($entity_class) && !empty($attribute_class)) {
                        $result[$modelValue->$k] = CoreValue::getEntityValue($entity_class, $attribute_class, $modelValue->id);
                    } else {
                        if (empty($relation)) {
                            if (is_array($v)) {
                                $v_tmp = array();

                                foreach ($v as $v_k => $v_v) {
                                    $v_tmp[] = $modelValue->$v_v;
                                }

                                $result[$modelValue->$k] = implode(' ', $v_tmp);
                            } else {
                                $result[$modelValue->$k] = $modelValue->$v;
                            }
                        } else {
                            $v_tmp = array();

                            if (is_array($v)) {
                                foreach ($v as $v_k => $v_v) {
                                    $v_tmp[] = $modelValue->$v_v;
                                }
                            } else {
                                $v_tmp[] = $modelValue->$v;
                            }

                            if (is_array($relation[0])) {
                                $relation_tmp = array();

                                foreach ($relation as $relation_k => $relation_v) {
                                    $v_tmp[] = isset($modelValue->{$relation_v[0]}->{$relation_v[1]}) ? $modelValue->{$relation_v[0]}->{$relation_v[1]} : null;
                                }

                                $result[$modelValue->$k] = implode(' ', $v_tmp);
                            } else {
                                $result[$modelValue->$k] = implode(' ', $v_tmp) . isset($modelValue->{$relation[0]}->{$relation[1]}) ? $modelValue->{$relation[0]}->{$relation[1]} : null;
                            }
                        }
                    }
                }
            }
        }

        return $result;
    }

    public static function getAttr($attr = array(), $where = array())
    {
        $where['status'] = 'active';

        $model = self::findOne($where);

        $result = array();

        if (isset($model->id) && !empty($model->id)) {
            if (count($attr) < 1) {
                $result = false;
            } else if (count($attr) < 2) {
                foreach ($attr as $key => $value) {
                    $result = isset($model->$value) && !empty($model->$value) ? $model->$value : false;
                }
            } else {
                foreach ($attr as $key => $value) {
                    $result[$value] = isset($model->$value) && !empty($model->$value) ? $model->$value : false;
                }

                if (in_array(false, $result)) {
                    $result = false;
                }
            }
        } else {
            $result = false;
        }

        return $result;
    }

    public static function findByKeywords($keywords = array())
    {
        $entities = array();

        foreach ($keywords as $keyword) {
            $keyword = ucwords(str_replace('-', ' ', $keyword));

            $keyword_id = CoreKeyword::getAttr(['id'], ['name' => $keyword]);
            $entity_ids = CmsEntityKeyword::find()->select('entity_id')->where(['keyword_id' => $keyword_id, 'status' => 'active'])->asArray()->all();
            
            $entity_ids = array_map('current', $entity_ids);

            $entities[] = self::find()->where(['status' => 'active'])->andWhere(['IN', 'id', $entity_ids])->orderBy('created_at', 'DESC')->asArray()->all();
        }

        return $entities;
    }

    public static function getChartDataInterval($interval = 'daily', $multi_axis = false, $select = array(), $where = array(), $is_logs = false)
    {
        $date_format = '%Y %m %d';
        $select[] = 'COUNT(*) AS `count`';

        if ($is_logs == false) {
            $where['status'] = 'active';
        }
        
        switch ($interval) {
            case 'daily':
                $date_format = '%Y-%m-%d';

                break;

            case 'monthly':
                $date_format = '%Y-%m';
                
                break;

            case 'yearly':
                $date_format = '%Y';
                
                break;
            
            default:
                $date_format = '%Y-%m-%d';

                break;
        }

        $select[] = 'DATE_FORMAT(DATE(`created_at`), "' . $date_format . '") AS `created_at`';
        $data = array();

        $model = self::find()
            ->select($select)
            ->where($where)
            ->groupBy(['DATE_FORMAT(DATE(`created_at`), "' . $date_format . '")'])
            ->asArray()
            ->all();

        if ($multi_axis !== true) {
            foreach ($model as $key => $value) {
                $data[$value['created_at']] = $value['count'];
            }
        } else {
            foreach ($model as $key => $value) {
                $data['created_at'][$key] = $value['created_at'];
                
                foreach ($select as $k => $v) {
                    preg_match('/AS(.*)/', $v, $matches);

                    $data[trim(str_replace('`', '', $matches[1]))][$key] = $value[trim(str_replace('`', '', $matches[1]))];
                }
            }
        }

        return $data;
    }
}
