<?php

namespace common\modules\inventory\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%inventory_item}}".
 *
 * @property int $id
 * @property string $sku
 * @property string $name
 * @property string $description
 * @property int $unit_id
 * @property string $quantity
 * @property int $subcategory_id
 * @property int $warehouse_id
 * @property int $supplier_id
 * @property int $manufacturer_id
 * @property int $brand_id
 * @property int $currency_id
 * @property int $tax_id
 * @property string $tax
 * @property string $hsn_code
 * @property string $cost_price
 * @property string $shipping_cost
 * @property string $packaging_cost
 * @property string $repairing_cost
 * @property string $mrp
 * @property string $selling_price
 * @property string $upc
 * @property string $ean
 * @property string $mpn
 * @property string $asin
 * @property string $isbn_10
 * @property string $isbn_13
 * @property string $incoming_date
 * @property string $remarks
 * @property int $bill
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property AccountingDocumentItem[] $accountingDocumentItems
 * @property User $createdBy
 * @property AccountingCurrency $currency
 * @property AccountingTax $tax0
 * @property User $updatedBy
 * @property InventoryUnit $unit
 * @property ShopSubcategory $subcategory
 * @property InventorySupplier $supplier
 * @property InventoryManufacturer $manufacturer
 * @property InventoryBrand $brand
 * @property AccountingDocument $bill0
 * @property InventoryWarehouse $warehouse
 * @property InventoryItemTraitValue[] $inventoryItemTraitValues
 * @property InventoryLog[] $inventoryLogs
 */
class InventoryItem extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%inventory_item}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['sku', 'name', 'unit_id', 'quantity', 'subcategory_id', 'warehouse_id', 'currency_id', 'cost_price', 'shipping_cost', 'packaging_cost', 'repairing_cost', 'mrp', 'selling_price', 'incoming_date'], 'required'],
            [['description', 'remarks', 'metadata', 'status'], 'string'],
            [['unit_id', 'subcategory_id', 'warehouse_id', 'supplier_id', 'manufacturer_id', 'brand_id', 'currency_id', 'tax_id', 'bill', 'created_by', 'updated_by'], 'integer'],
            [['incoming_date'], 'safe'],
            [['sku', 'name', 'quantity', 'tax', 'hsn_code', 'cost_price', 'shipping_cost', 'packaging_cost', 'repairing_cost', 'mrp', 'selling_price', 'upc', 'ean', 'mpn', 'asin', 'isbn_10', 'isbn_13'], 'string', 'max' => 255],
            [['sku'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['currency_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingCurrency::className(), 'targetAttribute' => ['currency_id' => 'id']],
            [['tax_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingTax::className(), 'targetAttribute' => ['tax_id' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['unit_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\inventory\models\InventoryUnit::className(), 'targetAttribute' => ['unit_id' => 'id']],
            [['subcategory_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopSubcategory::className(), 'targetAttribute' => ['subcategory_id' => 'id']],
            [['supplier_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\inventory\models\InventorySupplier::className(), 'targetAttribute' => ['supplier_id' => 'id']],
            [['manufacturer_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\inventory\models\InventoryManufacturer::className(), 'targetAttribute' => ['manufacturer_id' => 'id']],
            [['brand_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\inventory\models\InventoryBrand::className(), 'targetAttribute' => ['brand_id' => 'id']],
            [['bill'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingDocument::className(), 'targetAttribute' => ['bill' => 'id']],
            [['warehouse_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\inventory\models\InventoryWarehouse::className(), 'targetAttribute' => ['warehouse_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'sku' => 'Sku',
            'name' => 'Name',
            'description' => 'Description',
            'unit_id' => 'Unit ID',
            'quantity' => 'Quantity',
            'subcategory_id' => 'Subcategory ID',
            'warehouse_id' => 'Warehouse ID',
            'supplier_id' => 'Supplier ID',
            'manufacturer_id' => 'Manufacturer ID',
            'brand_id' => 'Brand ID',
            'currency_id' => 'Currency ID',
            'tax_id' => 'Tax ID',
            'tax' => 'Tax',
            'hsn_code' => 'Hsn Code',
            'cost_price' => 'Cost Price',
            'shipping_cost' => 'Shipping Cost',
            'packaging_cost' => 'Packaging Cost',
            'repairing_cost' => 'Repairing Cost',
            'mrp' => 'Mrp',
            'selling_price' => 'Selling Price',
            'upc' => 'Upc',
            'ean' => 'Ean',
            'mpn' => 'Mpn',
            'asin' => 'Asin',
            'isbn_10' => 'Isbn 10',
            'isbn_13' => 'Isbn 13',
            'incoming_date' => 'Incoming Date',
            'remarks' => 'Remarks',
            'bill' => 'Bill',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountingDocumentItems()
    {
        return $this->hasMany(\common\modules\accounting\models\AccountingDocumentItem::className(), ['item_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCurrency()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingCurrency::className(), ['id' => 'currency_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTax0()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingTax::className(), ['id' => 'tax_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUnit()
    {
        return $this->hasOne(\common\modules\inventory\models\InventoryUnit::className(), ['id' => 'unit_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSubcategory()
    {
        return $this->hasOne(\common\modules\shop\models\ShopSubcategory::className(), ['id' => 'subcategory_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSupplier()
    {
        return $this->hasOne(\common\modules\inventory\models\InventorySupplier::className(), ['id' => 'supplier_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManufacturer()
    {
        return $this->hasOne(\common\modules\inventory\models\InventoryManufacturer::className(), ['id' => 'manufacturer_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBrand()
    {
        return $this->hasOne(\common\modules\inventory\models\InventoryBrand::className(), ['id' => 'brand_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBill0()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingDocument::className(), ['id' => 'bill']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getWarehouse()
    {
        return $this->hasOne(\common\modules\inventory\models\InventoryWarehouse::className(), ['id' => 'warehouse_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryItemTraitValues()
    {
        return $this->hasMany(\common\modules\inventory\models\InventoryItemTraitValue::className(), ['item_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryLogs()
    {
        return $this->hasMany(\common\modules\inventory\models\InventoryLog::className(), ['item_id' => 'id']);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                // soft-deleting traits
                if (\common\modules\inventory\models\InventoryItemTraitValue::find()->where(['item_id' => $this->id, 'status' => 'active'])->exists()) {
                    $modelInventoryItemTraitValue = \common\modules\inventory\models\InventoryItemTraitValue::find()->where(['item_id' => $this->id, 'status' => 'active'])->all();

                    foreach ($modelInventoryItemTraitValue as $key => $value) {
                        $value->softDelete();
                    }
                }

                $transaction->commit();
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveItem($post_data, $item_id = 0)
    {
        $session = Yii::$app->session;

        $modelInventoryItem = new self();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($item_id) && $item_id > 0 && self::find()->where(['id' => $item_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelInventoryItem = self::findOne(['id' => $item_id, 'status' => 'active']);

                $modelInventoryItem->load($post_data);

                $modelInventoryItem->metadata = isset($post_data['InventoryItem']['InventoryItemMetadata']) ? Globals::setJson($post_data['InventoryItem']['InventoryItemMetadata']) : '';
                $modelInventoryItem->updated_by = self::setUpdatedBy($post_data['InventoryItem']);
            } else {    // Create a new model
                $modelInventoryItem = new InventoryItem();

                $modelInventoryItem->load($post_data);

                $modelInventoryItem->metadata = isset($post_data['InventoryItem']['InventoryItemMetadata']) ? Globals::setJson($post_data['InventoryItem']['InventoryItemMetadata']) : '';
                $modelInventoryItem->created_by = self::setCreatedBy($post_data['InventoryItem']);
                $modelInventoryItem->updated_by = self::setUpdatedBy($post_data['InventoryItem']);
                $modelInventoryItem->status = 'active';
            }
            
            if ($modelInventoryItem->validate()) {
                $modelInventoryItem->save();

                // saving trait-values
                if (isset($post_data['Trait']['trait']) && !empty($post_data['Trait']['trait'])) {
                    $trait_value_ids = $item_trait_value_ids = array();

                    foreach ($post_data['Trait']['trait'] as $trait_id) {
                        if (isset($post_data['Trait'][$trait_id])) {
                            foreach ($post_data['Trait'][$trait_id] as $trait_value_id) {
                                array_push($trait_value_ids, $trait_value_id);

                                if (\common\modules\inventory\models\InventoryItemTraitValue::find()->where([
                                    'item_id' => $modelInventoryItem->id,
                                    'trait_id' => $trait_id,
                                    'trait_value_id' => $trait_value_id,
                                    'status' => 'active',
                                ])->exists()) {
                                    $modelInventoryItemTraitValue = \common\modules\inventory\models\InventoryItemTraitValue::find()->where([
                                        'item_id' => $modelInventoryItem->id,
                                        'trait_id' => $trait_id,
                                        'trait_value_id' => $trait_value_id,
                                        'status' => 'active',
                                    ])->one();

                                    $modelInventoryItemTraitValue->updated_by = $modelInventoryItem->updated_by;
                                } else {
                                    $modelInventoryItemTraitValue = new \common\modules\inventory\models\InventoryItemTraitValue();

                                    $modelInventoryItemTraitValue->item_id = $modelInventoryItem->id;
                                    $modelInventoryItemTraitValue->trait_id = $trait_id;
                                    $modelInventoryItemTraitValue->trait_value_id = $trait_value_id;
                                    $modelInventoryItemTraitValue->created_by = $modelInventoryItem->created_by;
                                    $modelInventoryItemTraitValue->updated_by = $modelInventoryItem->updated_by;
                                    $modelInventoryItemTraitValue->status = 'active';
                                }

                                if ($modelInventoryItemTraitValue->validate()) {
                                    $modelInventoryItemTraitValue->save();

                                    array_push($item_trait_value_ids, $modelInventoryItemTraitValue->id);
                                } else {
                                    $transaction->rollBack();

                                    $breakFlag = true;
                                }
                            }
                        }
                    }

                    // deleting remaining traits-values
                    if (!$breakFlag) {
                        $item_trait_values = \common\modules\inventory\models\InventoryItemTraitValue::find()
                            ->where(['item_id' => $modelInventoryItem->id, 'status' => 'active'])
                            ->andWhere(['NOT IN', 'id', $item_trait_value_ids])
                            ->all();

                        foreach ($item_trait_values as $item_trait_value) {
                            $item_trait_value->delete();
                        }
                    }
                }

                $transaction->commit();

                Yii::$app->session->setFlash('success', "Item saved successfully.");

                return $modelInventoryItem->id;
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save item: " . Globals::renderErrors($modelInventoryItem->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}