<?php

namespace common\modules\inventory\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\inventory\models\InventoryItem;

/**
 * InventoryItemSearch represents the model behind the search form of `common\modules\inventory\models\InventoryItem`.
 */
class InventoryItemSearch extends InventoryItem
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'unit_id', 'subcategory_id', 'warehouse_id', 'supplier_id', 'manufacturer_id', 'brand_id', 'currency_id', 'tax_id', 'bill', 'created_by', 'updated_by'], 'integer'],
            [['sku', 'name', 'description', 'quantity', 'tax', 'hsn_code', 'cost_price', 'shipping_cost', 'packaging_cost', 'repairing_cost', 'mrp', 'selling_price', 'upc', 'ean', 'mpn', 'asin', 'isbn_10', 'isbn_13', 'incoming_date', 'remarks', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = InventoryItem::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_inventory_item.sku', $this->sku])
            ->andFilterWhere(['like', 'mod_inventory_item.name', $this->name])
            ->andFilterWhere(['like', 'mod_inventory_item.description', $this->description])
            ->andFilterWhere(['like', 'mod_inventory_item.quantity', $this->quantity])
            ->andFilterWhere(['like', 'mod_inventory_item.tax', $this->tax])
            ->andFilterWhere(['like', 'mod_inventory_item.hsn_code', $this->hsn_code])
            ->andFilterWhere(['like', 'mod_inventory_item.cost_price', $this->cost_price])
            ->andFilterWhere(['like', 'mod_inventory_item.shipping_cost', $this->shipping_cost])
            ->andFilterWhere(['like', 'mod_inventory_item.packaging_cost', $this->packaging_cost])
            ->andFilterWhere(['like', 'mod_inventory_item.repairing_cost', $this->repairing_cost])
            ->andFilterWhere(['like', 'mod_inventory_item.mrp', $this->mrp])
            ->andFilterWhere(['like', 'mod_inventory_item.selling_price', $this->selling_price])
            ->andFilterWhere(['like', 'mod_inventory_item.upc', $this->upc])
            ->andFilterWhere(['like', 'mod_inventory_item.ean', $this->ean])
            ->andFilterWhere(['like', 'mod_inventory_item.mpn', $this->mpn])
            ->andFilterWhere(['like', 'mod_inventory_item.asin', $this->asin])
            ->andFilterWhere(['like', 'mod_inventory_item.isbn_10', $this->isbn_10])
            ->andFilterWhere(['like', 'mod_inventory_item.isbn_13', $this->isbn_13])
            ->andFilterWhere(['like', 'mod_inventory_item.remarks', $this->remarks])
            ->andFilterWhere(['like', 'mod_inventory_item.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_inventory_item.status', $this->status])
            ->andFilterWhere(['like', 'mod_inventory_item.id', $this->id])
            ->andFilterWhere(['like', 'mod_inventory_item.unit_id', $this->unit_id])
            ->andFilterWhere(['like', 'mod_inventory_item.subcategory_id', $this->subcategory_id])
            ->andFilterWhere(['like', 'mod_inventory_item.warehouse_id', $this->warehouse_id])
            ->andFilterWhere(['like', 'mod_inventory_item.supplier_id', $this->supplier_id])
            ->andFilterWhere(['like', 'mod_inventory_item.manufacturer_id', $this->manufacturer_id])
            ->andFilterWhere(['like', 'mod_inventory_item.brand_id', $this->brand_id])
            ->andFilterWhere(['like', 'mod_inventory_item.currency_id', $this->currency_id])
            ->andFilterWhere(['like', 'mod_inventory_item.tax_id', $this->tax_id])
            ->andFilterWhere(['like', 'mod_inventory_item.incoming_date', $this->incoming_date])
            ->andFilterWhere(['like', 'mod_inventory_item.bill', $this->bill])
            ->andFilterWhere(['like', 'mod_inventory_item.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_inventory_item.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_inventory_item.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_inventory_item.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
