<?php

namespace common\modules\inventory\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%inventory_item_trait_value}}".
 *
 * @property int $id
 * @property int $item_id
 * @property int $trait_id
 * @property int $trait_value_id
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property InventoryItem $item
 * @property ShopTraitValue $traitValue
 * @property User $createdBy
 * @property User $updatedBy
 * @property ShopTrait $trait
 */
class InventoryItemTraitValue extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%inventory_item_trait_value}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['item_id', 'trait_id', 'trait_value_id'], 'required'],
            [['item_id', 'trait_id', 'trait_value_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['item_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\inventory\models\InventoryItem::className(), 'targetAttribute' => ['item_id' => 'id']],
            [['trait_value_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopTraitValue::className(), 'targetAttribute' => ['trait_value_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['trait_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopTrait::className(), 'targetAttribute' => ['trait_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'item_id' => 'Item ID',
            'trait_id' => 'Trait ID',
            'trait_value_id' => 'Trait Value ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getItem()
    {
        return $this->hasOne(\common\modules\inventory\models\InventoryItem::className(), ['id' => 'item_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTraitValue()
    {
        return $this->hasOne(\common\modules\shop\models\ShopTraitValue::className(), ['id' => 'trait_value_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTrait()
    {
        return $this->hasOne(\common\modules\shop\models\ShopTrait::className(), ['id' => 'trait_id']);
    }
}