<?php

namespace common\modules\inventory\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%inventory_manufacturer}}".
 *
 * @property int $id
 * @property int $legal_entity_id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property InventoryItem[] $inventoryItems
 * @property User $createdBy
 * @property User $updatedBy
 * @property AccountingLegalEntity $legalEntity
 */
class InventoryManufacturer extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%inventory_manufacturer}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['legal_entity_id'], 'required'],
            [['legal_entity_id', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['legal_entity_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\accounting\models\AccountingLegalEntity::className(), 'targetAttribute' => ['legal_entity_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'legal_entity_id' => 'Legal Entity ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInventoryItems()
    {
        return $this->hasMany(\common\modules\inventory\models\InventoryItem::className(), ['manufacturer_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLegalEntity()
    {
        return $this->hasOne(\common\modules\accounting\models\AccountingLegalEntity::className(), ['id' => 'legal_entity_id']);
    }
}