<?php

namespace common\modules\inventory\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\inventory\models\InventoryManufacturer;

/**
 * InventoryManufacturerSearch represents the model behind the search form of `common\modules\inventory\models\InventoryManufacturer`.
 */
class InventoryManufacturerSearch extends InventoryManufacturer
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'legal_entity_id', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = InventoryManufacturer::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_inventory_manufacturer.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_inventory_manufacturer.status', $this->status])
            ->andFilterWhere(['like', 'mod_inventory_manufacturer.id', $this->id])
            ->andFilterWhere(['like', 'mod_inventory_manufacturer.legal_entity_id', $this->legal_entity_id])
            ->andFilterWhere(['like', 'mod_inventory_manufacturer.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_inventory_manufacturer.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_inventory_manufacturer.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_inventory_manufacturer.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
