<?php

namespace common\modules\lms\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

use common\modules\lms\models\LmsAttendance;
use common\modules\lms\models\LmsAttendanceSearch;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * AttendanceController implements the CRUD actions for LmsAttendance model.
 */
class AttendanceController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete', 'mark'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete', 'mark'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all LmsAttendance models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new LmsAttendanceSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single LmsAttendance model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new LmsAttendance model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new LmsAttendance();
        $modelLmsCohort = new \common\modules\lms\models\LmsCohort;
        $modelLmsCohortUserCourse = [new \common\modules\lms\models\LmsCohortUserCourse];
        $data = array();

        if (Yii::$app->request->isPost) {
            if ($attendance_id = $model->saveAttendance(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $attendance_id]);
            }
        }

        $data['model'] = $model;
        $data['modelLmsCohort'] = $modelLmsCohort;
        $data['modelLmsCohortUserCourse'] = empty($modelLmsCohortUserCourse) ? [new \common\modules\lms\models\LmsCohortUserCourse] : $modelLmsCohortUserCourse;

        return $this->render('create', [
            'data' => $data,
        ]);
    }

    /**
     * Updates an existing LmsAttendance model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $modelLmsCohort = \common\modules\lms\models\LmsCohort::find()->where(['id' => $model->cohortUserCourse->cohort_id, 'status' => 'active'])->one();
        $modelLmsCohortUserCourse = \common\modules\lms\models\LmsCohortUserCourse::find()->where(['cohort_id' => $model->cohortUserCourse->cohort_id, 'status' => 'active'])->all();
        $data = array();

        if (Yii::$app->request->isPost) {
            if ($attendance_id = $model->saveAttendance(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $attendance_id]);
            }
        }

        $data['model'] = $model;
        $data['modelLmsCohort'] = empty($modelLmsCohort) ? new \common\modules\lms\models\LmsCohort : $modelLmsCohort;
        $data['modelLmsCohortUserCourse'] = empty($modelLmsCohortUserCourse) ? [new \common\modules\lms\models\LmsCohortUserCourse] : $modelLmsCohortUserCourse;

        return $this->render('update', [
            'data' => $data,
        ]);
    }

    /**
     * Deletes an existing LmsAttendance model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the LmsAttendance model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return LmsAttendance the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = LmsAttendance::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionMark($cohort_id)
    {
        $modelLmsCohort = \common\modules\lms\models\LmsCohort::find()->where(['id' => $cohort_id, 'status' => 'active'])->one();
        $modelLmsCohortUserCourse = \common\modules\lms\models\LmsCohortUserCourse::find()->where(['cohort_id' => $cohort_id, 'status' => 'active'])->all();
        $data = array();
        
        if (LmsAttendance::find()
            ->where(['status' => 'active'])
            ->andWhere([
                'IN', 
                'cohort_user_course_id', 
                \common\modules\lms\models\LmsCohortUserCourse::find()->select(['id'])->where(['cohort_id' => $cohort_id, 'status' => 'active'])->column()
            ])
            ->exists()
        ) {
            $model = LmsAttendance::find()
            ->where(['status' => 'active'])
            ->andWhere([
                'IN', 
                'cohort_user_course_id', 
                \common\modules\lms\models\LmsCohortUserCourse::find()->select(['id'])->where(['cohort_id' => $cohort_id, 'status' => 'active'])->column()
            ])
            ->one();
        } else {
            $model = new LmsAttendance;
        }

        if (Yii::$app->request->isPost) {
            if ($attendance_id = $model->saveAttendance(Yii::$app->request->post(), $cohort_id)) {
                return $this->redirect(['/lms/cohort/index']);
            }
        }

        $data['model'] = $model;
        $data['modelLmsCohort'] = empty($modelLmsCohort) ? new \common\modules\lms\models\LmsCohort : $modelLmsCohort;
        $data['modelLmsCohortUserCourse'] = empty($modelLmsCohortUserCourse) ? [new \common\modules\lms\models\LmsCohortUserCourse] : $modelLmsCohortUserCourse;

        return $this->render('mark', [
            'data' => $data,
        ]);
    }
}
