<?php

namespace common\modules\lms\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

use common\modules\lms\models\LmsCohort;
use common\modules\lms\models\LmsCohortSearch;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * CohortController implements the CRUD actions for LmsCohort model.
 */
class CohortController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all LmsCohort models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new LmsCohortSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        $modelTeachers = $modelTenants = new \common\modules\user\models\User();
        $modelCourses = new \common\modules\lms\models\LmsCourse();
        $courses = array();

        $modelTeachers = \common\modules\user\models\User::find()->where(['status' => 'active', 'role_id' => \common\modules\user\models\UserRole::getAttr(['id'], ['name' => 'teacher'])])->asArray()->all();
        $modelTenants = \common\modules\user\models\User::find()->where(['status' => 'active', 'role_id' => \common\modules\user\models\UserRole::getAttr(['id'], ['name' => 'tenant'])])->asArray()->all();

        $modelCourses = CoreValue::find()->where([
            'entity_class_id' => \common\modules\core\models\CoreEntityClass::getAttr(['id'], ['name' => 'course']), 
            'attribute_class_id' => \common\modules\core\models\CoreAttributeClass::getAttr(['id'], ['name' => 'name']), 
            'status' => 'active'])
        ->asArray()
        ->all();

        foreach ($modelCourses as $k => $v) {
            $courses[$v['entity_id']] = $v['value'];
        }

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'modelTeachers' => \yii\helpers\ArrayHelper::map($modelTeachers, 'id', 'email'),
            'modelTenants' => \yii\helpers\ArrayHelper::map($modelTenants, 'id', 'email'),
            'courses' => $courses,
        ]);
    }

    /**
     * Displays a single LmsCohort model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new LmsCohort model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new LmsCohort();
        $modelLmsCohortUserCourse = [new \common\modules\lms\models\LmsCohortUserCourse];
        $data = array();

        if (Yii::$app->request->isPost) {
            if ($cohort_id = $model->saveCohort(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $cohort_id]);
            }
        }
        
        $data['model'] = $model;
        $data['modelLmsCohortUserCourse'] = (empty($modelLmsCohortUserCourse)) ? [new \common\modules\lms\models\LmsCohortUserCourse] : $modelLmsCohortUserCourse;

        return $this->render('create', [
            'data' => $data,
        ]);
    }

    /**
     * Updates an existing LmsCohort model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $modelLmsCohortUserCourse = \common\modules\lms\models\LmsCohortUserCourse::find()->where(['cohort_id' => $id, 'status' => 'active'])->all();
        $data = array();

        if (Yii::$app->request->isPost) {
            if ($cohort_id = $model->saveCohort(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $cohort_id]);
            }
        }
        
        $data['model'] = $model;
        $data['modelLmsCohortUserCourse'] = (empty($modelLmsCohortUserCourse)) ? [new \common\modules\lms\models\LmsCohortUserCourse] : $modelLmsCohortUserCourse;

        return $this->render('update', [
            'data' => $data,
        ]);
    }

    /**
     * Deletes an existing LmsCohort model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the LmsCohort model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return LmsCohort the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = LmsCohort::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
