<?php

namespace common\modules\lms\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use Dompdf\Dompdf;
use Dompdf\Options;

use common\modules\lms\models\LmsCourseActivity;
use common\modules\lms\models\LmsCourseActivitySearch;
use common\modules\lms\models\LmsCourse;
use common\modules\lms\models\LmsActivity;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * CourseActivityController implements the CRUD actions for LmsCourseActivity model.
 */
class CourseActivityController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all LmsCourseActivity models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new LmsCourseActivitySearch();
        $modelCoreValue = new CoreValue();
        $modelCoreValueCourse = new CoreValue();
        $modelCoreEntityClass = new CoreEntityClass();
        $modelCoreAttributeClass = new CoreAttributeClass();
        $activityArr = array();
        
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course_activity', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);
        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $modelCoreValue = array();

        foreach ($tmpArray as $key => $value) {
            $modelCoreValue[$value['entity_id']] = $value['value'];
        }

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $modelCoreValueCourse = array();

        foreach ($tmpArray as $key => $value) {
            $modelCoreValueCourse[$value['entity_id']] = $value['value'];
        }

        $modelActivity = LmsActivity::find()->where(['status' => 'active'])->asArray()->all();
        $activityArr = \yii\helpers\ArrayHelper::map($modelActivity, 'id', 'name');

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'modelCoreValue' => $modelCoreValue,
            'modelCoreValueCourse' => $modelCoreValueCourse,
            'modelActivity' => $activityArr,
        ]);
    }

    /**
     * Displays a single LmsCourseActivity model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new LmsCourseActivity model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new LmsCourseActivity();
        $modelCoreValue = new CoreValue();
        $modelCoreLanguage = new CoreLanguage();
        $attributes = array();
        $data = array();

        $modelCoreLanguage = CoreLanguage::find()->where(['status' => 'active'])->all();

        foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass('course_activity') as $attributeClassKey => $attributeClassValue) {
            $tmp = array();
            
            foreach ($modelCoreLanguage as $langKey => $langValue) {
                $tmp[$langValue->id] = '';            
            }
            
            $$attributeClassValue = $tmp;
            $attributes[] = $attributeClassValue;
        }

        if (Yii::$app->request->isPost) {
            if ($course_id = $model->saveCourseActivity(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $course_id]);
            } else {
                $data['model'] = $model;
                $data['modelCoreValue'] = $modelCoreValue;
                $data['modelCoreLanguage'] = $modelCoreLanguage;
                
                foreach ($attributes as $key => $value) {
                    $data[$value] = $$value;
                }

                return $this->render('create', [
                    'data' => $data,
                ]);    
            }
        } else {
            $data['model'] = $model;
            $data['modelCoreValue'] = $modelCoreValue;
            $data['modelCoreLanguage'] = $modelCoreLanguage;
            
            foreach ($attributes as $key => $value) {
                $data[$value] = $$value;
            }
            
            return $this->render('create', [
                'data' => $data,
            ]);
        }
    }

    /**
     * Updates an existing LmsCourseActivity model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $modelCoreValue = new CoreValue();
        $modelCoreLanguage = new CoreLanguage();
        $attributes = array();
        $data = array();

        $model = $this->findModel($id);
        $modelCoreLanguage = CoreLanguage::find()->where(['status' => 'active'])->all();
        
        foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass('course_activity') as $attributeClassKey => $attributeClassValue) {
            $$attributeClassValue = array();
            $tmp = array();

            $temp = CoreValue::getEntityValue('course_activity', $attributeClassValue, $id);
            
            if (!isset($temp)) {
                continue;
            }
            
            foreach ($modelCoreLanguage as $langKey => $langValue) {
                $tmp[$langValue->id] = CoreValue::getEntityValue('course_activity', $attributeClassValue, $id, $langValue->id);            
            }
            
            $$attributeClassValue = $tmp;
            $attributes[] = $attributeClassValue;
        }

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'course_activity', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $valueArray = array();

        foreach ($tmpArray as $key => $value) {
            $valueArray[$value['entity_id']] = $value['value'];
        }

        if (Yii::$app->request->isPost) {
            if ($course_id = $model->saveCourseActivity(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $course_id]);
            } else {
                $data['model'] = $model;
                $data['modelCoreValue'] = $modelCoreValue;
                $data['valueArray'] = $valueArray;
                $data['modelCoreLanguage'] = $modelCoreLanguage;

                foreach ($attributes as $key => $value) {
                    $data[$value] = $$value;
                }

                return $this->render('update', [
                    'data' => $data,
                ]);    
            }
        } else {
            $data['model'] = $model;
            $data['modelCoreValue'] = $modelCoreValue;
            $data['valueArray'] = $valueArray;
            $data['modelCoreLanguage'] = $modelCoreLanguage;

            foreach ($attributes as $key => $value) {
                $data[$value] = $$value;
            }

            return $this->render('update', [
                'data' => $data,
            ]);
        }
    }

    /**
     * Deletes an existing LmsCourseActivity model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the LmsCourseActivity model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return LmsCourseActivity the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = LmsCourseActivity::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionQuizExportPdf($id, $export = false)
    {
        $modelLmsCourseActivity = new LmsCourseActivity();
        
        if (LmsCourseActivity::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            $modelLmsCourseActivity = LmsCourseActivity::find()->where(['id' => $id, 'status' => 'active'])->asArray()->one();

            $userCourseActivityMetadata = json_decode($modelLmsCourseActivity['metadata'], true);
            
            if (isset($userCourseActivityMetadata) && !empty($userCourseActivityMetadata)) {
                $total_question = count($userCourseActivityMetadata['quiz']);
                $sno = 1;

                foreach ($userCourseActivityMetadata['quiz'] as $question){
                    $quiz[$sno]['sno'] = $question['sno'];
                    $quiz[$sno]['type'] = $question['type'];
                    $quiz[$sno]['question'] = $question['question'];
                    $quiz[$sno]['option'] = $question['option'];
                    $quiz[$sno]['answer'] = $question['answer'];
                    $quiz[$sno]['explanation'] = $question['explanation'];
                    $quiz[$sno]['assets'] = $question['assets'];

                    $sno ++;
                }
            }
        }

        // echo "<pre>"; print_r($quiz); die();

        if ($export) {
            // instantiate and use the dompdf class
            $dompdf = new Dompdf();
            $options = new Options();

            $options->setIsRemoteEnabled(true);
            $options->setIsPhpEnabled(true);
            
            $dompdf->setOptions($options);
            $dompdf->loadHtml($this->renderPartial('quiz-export-pdf', [
                'modelLmsCourseActivity' => $modelLmsCourseActivity,
                'total_question' => $total_question,
                'quiz' => $quiz,
            ]));
            //$dompdf->setPaper('A4', 'portrait');    // (Optional) Setup the paper size and orientation      
            @$dompdf->render();  // Render the HTML as PDF
            //$canvas = $dompdf->getCanvas();
            //print the pdf file to the screen for saving
            $dompdf->stream("quiz_id_" . $modelLmsCourseActivity['id'] . ".pdf", array("Attachment" => true));
            //$dompdf->stream("pdf_filename_".rand(10,1000).".pdf", array("Attachment" => false));

            $dompdf->stream("dompdf_out", array("Attachment" => false));    // Output the generated PDF to Browser

            exit(0);
        } else {
            return $this->render('quiz-export-pdf',[
                'modelLmsCourseActivity' => $modelLmsCourseActivity,
                'total_question' => $total_question,
                'quiz' => $quiz,
            ]);    
        }
        
    }
}
