<?php

namespace common\modules\lms\controllers\backend;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use yii\helpers\ArrayHelper;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\lms\models\LmsUserCourseActivity;
use common\modules\lms\models\LmsUserCourseActivityAttempt;
use common\modules\lms\models\LmsCourseActivity;
use common\modules\lms\models\LmsCourseActivitySearch;
use common\modules\lms\models\LmsReportsCourseCompletionSearch;
use common\modules\lms\models\LmsReportsQuizCompletionSearch;
use common\modules\lms\models\LmsReportsAssignmentCompletionSearch;
use Dompdf\Dompdf;
use Dompdf\Options;

/**
 * LmsReportController implements the CRUD actions for Scorm model.
 */
class LmsReportController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['course-completion', 'quiz-completion', 'assignment-completion', 'quiz-report', 'generate-report', 'cumulative-report', 'cumulative-reports', 'average-report'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['course-completion', 'quiz-completion', 'assignment-completion', 'quiz-report', 'generate-report', 'cumulative-report', 'cumulative-reports', 'average-report'],
                        'roles' => ['@'],
                    ],
                ],
            ],
        ];
    }
    
    public function actionCourseCompletion()
    {
        $searchModel = new LmsReportsCourseCompletionSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('course-completion', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    public function actionQuizCompletion()
    {
        $searchModel = new LmsReportsQuizCompletionSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('quiz-completion', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    public function actionAssignmentCompletion()
    {
        $searchModel = new LmsReportsAssignmentCompletionSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('assignment-completion', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    public function actionQuizCompletionReport($id, $user_id, $user_course_activity_id, $export = false)
    {
        $modelUserArr = new User();
        $modelUserProfileArr = new UserProfile();
        $modelLmsUserCourseActivityAttemptArr = new LmsUserCourseActivityAttempt();
        $user_activity_arr = array();
        $total_question = 0;
        $correct_answers = 0;
        $user_activity_attempt = '';
        
        if (LmsUserCourseActivityAttempt::find()->where(['id' => $id, 'user_id' => $user_id, 'user_course_activity_id' => $user_course_activity_id, 'status' => 'active'])->exists()) {
            $modelLmsUserCourseActivityAttemptArr = LmsUserCourseActivityAttempt::find()->where(['id' => $id, 'user_id' => $user_id, 'user_course_activity_id' => $user_course_activity_id, 'status' => 'active'])->asArray()->one();
            $modelUserArr = User::find()->where(['id' => $modelLmsUserCourseActivityAttemptArr['user_id'], 'status' => 'active'])->asArray()->one();
            $modelUserProfileArr = UserProfile::find()->where(['user_id' => $modelLmsUserCourseActivityAttemptArr['user_id'], 'status' => 'active'])->asArray()->one();

            $userCourseActivityMetadata = $modelLmsUserCourseActivityAttemptArr['metadata'];
            $user_activity_attempt = json_decode($userCourseActivityMetadata,true);
            
            if (isset($user_activity_attempt) && !empty($user_activity_attempt)) {
                $total_question = count($user_activity_attempt);
                $sno = 1;

                foreach ($user_activity_attempt as $key => $attempt_values) {
                    $user_activity_arr[$sno]['qno'] = $attempt_values['sno'];
                    $user_activity_arr[$sno]['question'] = $attempt_values['question'];
                    $user_activity_arr[$sno]['answer'] = $attempt_values['answer'];

                    $user_activity_arr[$sno]['response'] = (isset($attempt_values['response']) ? $attempt_values['response'] : '');

                    if (isset($attempt_values['result'])) {
                        $user_activity_arr[$sno]['result'] = $attempt_values['result'];
                        
                        $correct_answers += ($attempt_values['result'] == 'correct') ? 1 : 0;
                    } else {
                        $user_activity_arr[$sno]['result'] = '';
                    }

                    $sno ++;
                }
            }
        }

        if ($export) {
            // instantiate and use the dompdf class
            $dompdf = new Dompdf();
            $options = new Options();

            $options->setIsRemoteEnabled(true);
            $options->setIsPhpEnabled(true);
            
            $dompdf->setOptions($options);
            $dompdf->loadHtml($this->renderPartial('_export-quiz-completion-report', [
                'modelUserArr' => $modelUserArr,
                'modelUserProfileArr' => $modelUserProfileArr,
                'modelLmsUserCourseActivityAttemptArr' => $modelLmsUserCourseActivityAttemptArr,
                'total_question' => $total_question,
                'correct_answers' => $correct_answers,
                'user_activity_arr' => $user_activity_arr,
            ]));
            //$dompdf->setPaper('A4', 'portrait');    // (Optional) Setup the paper size and orientation      
            @$dompdf->render();  // Render the HTML as PDF
            //$canvas = $dompdf->getCanvas();
            //print the pdf file to the screen for saving
            $dompdf->stream("quiz_completion_report_id_" . $modelLmsUserCourseActivityAttemptArr['id'] . ".pdf", array("Attachment" => true));
            //$dompdf->stream("pdf_filename_".rand(10,1000).".pdf", array("Attachment" => false));

            $dompdf->stream("dompdf_out", array("Attachment" => false));    // Output the generated PDF to Browser

            exit(0);
        } else {
            return $this->render('quiz-completion-report',[
                'modelUserArr' => $modelUserArr,
                'modelUserProfileArr' => $modelUserProfileArr,
                'modelLmsUserCourseActivityAttemptArr' => $modelLmsUserCourseActivityAttemptArr,
                'total_question' => $total_question,
                'correct_answers' => $correct_answers,
                'user_activity_arr' => $user_activity_arr,
            ]);    
        }
        
    }

    public function actionCumulativeQuizReports($course_id, $activity_id) 
    {
        $searchModel = new LmsCourseActivitySearch();
        $params = array('LmsCourseActivitySearch' => array('course_id' => $course_id, 'activity_id' => $activity_id));
        $dataProvider = $searchModel->search($params);
    
        return $this->render('cumulative-quiz-reports',[
            'dataProvider' => $dataProvider,
        ]);
    }

    public function actionFeedbackCourseEvaluation($course_activity_id = 258, $export = false)
    {
        $modelLmsUserCourseActivity = new LmsUserCourseActivityAttempt();
        $user_course_activity_ids = array();

        $questions = $feedback_arr = $chart_arr = array();
        $modelLmsCourseActivity = LmsCourseActivity::find()->select(['metadata'])->where(['id' => $course_activity_id, 'status' => 'active'])->asArray()->one();

        if ($modelLmsCourseActivity !== null) {
            $questions = json_decode($modelLmsCourseActivity['metadata'], true);
            $questions = $questions['quiz'];
        }

        $modelLmsUserCourseActivity = LmsUserCourseActivity::find()->where(['course_activity_id' => $course_activity_id, 'status' => 'active'])->asArray()->all();
        $user_course_activity_ids = ArrayHelper::getColumn($modelLmsUserCourseActivity, 'id');
        
        $modelLmsUserCourseActivityAttempt = LmsUserCourseActivityAttempt::find()->where(['status' => 'active'])->andWhere(['IN', 'user_course_activity_id', $user_course_activity_ids])->asArray()->all();
        $total_attempts = count($modelLmsUserCourseActivityAttempt);

        foreach ($questions as $question) {
            $feedback = $chart_details = array();
            $response_arr = array();
            $option_label_arr = $response_percent_arr = array();

            foreach ($question['option'] as $optionKey => $optionValue) {
                if ($question['type'] == 'sa') {
                    continue;
                }

                $response_arr[$optionKey]['value'] = $optionValue;
                $response_arr[$optionKey]['response_num'] = 0;
            }

            foreach ($modelLmsUserCourseActivityAttempt as $k => $v) {
                $user_questions_metadata = $v['metadata'];
                $user_questions_metadata = json_decode($user_questions_metadata, true);

                foreach ($user_questions_metadata as $user_question_metadata) {
                    if (($question['type'] == 'sa') && ($question['sno'] == $user_question_metadata['sno']) && ($question['question'] == $user_question_metadata['question']) && isset($user_question_metadata['response'])) {
                        $response_arr[0]['value'] = $user_question_metadata['question'];
                        $response_arr[0]['response_num'][] = $user_question_metadata['response'];
                    } elseif (($question['sno'] == $user_question_metadata['sno']) && ($question['question'] == $user_question_metadata['question']) && isset($user_question_metadata['response'])) {
                        if (is_array($user_question_metadata['response']) || is_object($user_question_metadata['response'])) {
                            foreach ($user_question_metadata['response'] as $user_question_metadata_value) {
                                $questionArrayKey = array_search($user_question_metadata_value, $question['option']);
                                $response_arr[$questionArrayKey]['response_num'] += 1;
                            }
                        } else {
                            $questionArrayKey = array_search($user_question_metadata['response'], $question['option']);
                            $response_arr[$questionArrayKey]['response_num'] += 1;
                        }
                    }       
                } 
            }

            foreach ($response_arr as $response_arr_key => $response_arr_value) {
                if (!is_numeric($response_arr_value['response_num'])) {
                    if (isset($response_arr_value['value']) && !empty($response_arr_value['value'])) {
                        array_push($option_label_arr, $response_arr_value['value']);
                    }

                    foreach ($response_arr_value['response_num'] as $_k => $_v) {
                        array_push($response_percent_arr, $_v);
                    }

                    continue;
                }

                array_push($option_label_arr, $response_arr_value['value']);

                $data_percent = ($response_arr_value['response_num'] * 100) / $total_attempts;

                array_push($response_percent_arr, $data_percent);
            }
            
            $chart_details['question'] = $question['question'];
            $chart_details['label'] = $option_label_arr;
            $chart_details['data'] = $response_percent_arr;

            array_push($chart_arr, $chart_details);
            array_push($feedback_arr, $feedback);
        }

        if ($export) {
            // instantiate and use the dompdf class
            $dompdf = new Dompdf();
            $options = new Options();

            $options->setIsRemoteEnabled(true);
            $options->setIsPhpEnabled(true);
            
            $dompdf->setOptions($options);
            $dompdf->loadHtml($this->renderPartial('_export-cumulative-quiz-report', [
                'feedbackReport' => $feedback_arr,
                'totalAttempts' => $total_attempts,
                'chart_data' => $chart_arr,
            ]));
            //$dompdf->setPaper('A4', 'portrait');    // (Optional) Setup the paper size and orientation      
            @$dompdf->render();  // Render the HTML as PDF
            //$canvas = $dompdf->getCanvas();
            //print the pdf file to the screen for saving
            $dompdf->stream("cumulative_quiz_report_" . $course_activity_id . ".pdf", array("Attachment" => true));
            //$dompdf->stream("pdf_filename_".rand(10,1000).".pdf", array("Attachment" => false));

            $dompdf->stream("dompdf_out", array("Attachment" => false));    // Output the generated PDF to Browser

            exit(0);
        } else {
            return $this->render('feedback-course-evaluation', [
                'course_id' => LmsCourseActivity::getAttr(['course_id'], ['id' => $course_activity_id]),
                'feedbackReport' => $feedback_arr,
                'totalAttempts' => $total_attempts,
                'chart_data' => $chart_arr,
            ]);
        }
    }

    public function actionFeedbackContentEvaluation($course_activity_id_bfr = 256, $course_activity_id_aft = 257, $export = false)
    {
        $chart_arr = array();
        $average_chart_report_1 = $average_chart_report_2 = array();

        $total_attempts = 0;
        $feedback_details = $this->getFeedbackResponseDetails($course_activity_id_bfr);

        if (isset($feedback_details['chart_details'])) {
            $chart_arr = $feedback_details['chart_details'];
        }

        if (isset($feedback_details['total_attempts'])) {
            $total_attempts = $feedback_details['total_attempts'];
        }

        $average_chart_report_1['label'] = $average_chart_report_1['option'] = $average_chart_report_1['data'] = array();

        foreach ($chart_arr as $chartKey => $chartValue) {
            $sum = 0;

            foreach($chartValue['label'] as $labelKey => $labelValue) {
                if (!in_array($labelValue, $average_chart_report_1['option'])) {
                    array_push($average_chart_report_1['option'], $labelValue);
                }
            }

            foreach($chartValue['data'] as $key => $value) {
                $sum += $value;
            }

            $avg = round($sum / $total_attempts, 2);

            array_push($average_chart_report_1['data'], $avg);
            array_push($average_chart_report_1['label'], $chartValue['question']);
        }

        $total_attempts = 0;
        $feedback_details = $this->getFeedbackResponseDetails($course_activity_id_aft);

        if (isset($feedback_details['chart_details'])) {
            $chart_arr = $feedback_details['chart_details'];
        }

        if (isset($feedback_details['total_attempts'])) {
            $total_attempts = $feedback_details['total_attempts'];
        }

        $average_chart_report_2['label'] = $average_chart_report_2['option'] = $average_chart_report_2['data'] = array();

        foreach ($chart_arr as $chartKey => $chartValue) {
            $sum = 0;

            foreach($chartValue['label'] as $labelKey => $labelValue) {
                if (!in_array($labelValue, $average_chart_report_2['option'])) {
                    array_push($average_chart_report_2['option'], $labelValue);
                }
            }

            foreach($chartValue['data'] as $key => $value) {
                $sum += $value;
            }

            $avg = round($sum / $total_attempts, 2);

            array_push($average_chart_report_2['data'], $avg);
            array_push($average_chart_report_2['label'], $chartValue['question']);
        }

        // TODO: Export report
        if ($export) {
            // instantiate and use the dompdf class
            $dompdf = new Dompdf();
            $options = new Options();

            $options->setIsRemoteEnabled(true);
            $options->setIsPhpEnabled(true);
            
            $dompdf->setOptions($options);
            $dompdf->loadHtml($this->renderPartial('_export-cumulative-quiz-report', [
                // 'feedbackReport' => $feedback_arr,
                'totalAttempts' => $total_attempts,
                'chart_data' => $chart_arr,
            ]));
            //$dompdf->setPaper('A4', 'portrait');    // (Optional) Setup the paper size and orientation      
            @$dompdf->render();  // Render the HTML as PDF
            //$canvas = $dompdf->getCanvas();
            //print the pdf file to the screen for saving
            $dompdf->stream("cumulative_quiz_report_" . $id . ".pdf", array("Attachment" => true));
            //$dompdf->stream("pdf_filename_".rand(10,1000).".pdf", array("Attachment" => false));

            $dompdf->stream("dompdf_out", array("Attachment" => false));    // Output the generated PDF to Browser

            exit(0);
        } else {
            return $this->render('feedback-content-evaluation', [
                'course_id' => LmsCourseActivity::getAttr(['course_id'], ['id' => $course_activity_id_bfr]),
                'course_activity_id' => $course_activity_id_bfr,
                'average_chart_report_1' => $average_chart_report_1,
                'average_chart_report_2' => $average_chart_report_2,
            ]);
        }
    }

    public function actionEvaluationAssessment($course_activity_id_pre = 259, $course_activity_id_post = 260, $export = false)
    {
        $chart_arr = array();
        $chart_data_pre = $chart_data_post = array();

        $total_attempts = 0;
        $assessment_details = $this->getAssessmentResponseDetails($course_activity_id_pre);
        $chart_data_pre = $assessment_details;

        $chart_data_pre = array();

        foreach ($assessment_details as $k => $v) {
            $chart_data_pre[$k] = $v;
        }

        $total_attempts = 0;
        $assessment_details = $this->getAssessmentResponseDetails($course_activity_id_post);
        $chart_data_post = $assessment_details;

        $chart_data_post = array();

        foreach ($assessment_details as $k => $v) {
            $chart_data_post[$k] = $v;
        }

        $tmp = array_intersect_key($chart_data_pre, $chart_data_post);

        return $this->render('evaluation-assessment', [
            'course_id' => LmsCourseActivity::getAttr(['course_id'], ['id' => $course_activity_id_pre]),
            'course_activity_id' => $course_activity_id_pre,
            'chart_data_pre' => array_intersect_key($chart_data_pre, $tmp),
            'chart_data_post' => array_intersect_key($chart_data_post, $tmp),
        ]);
    }

    public function getFeedbackResponseDetails($id)
    {
        $modelLmsUserCourseActivity = new LmsUserCourseActivityAttempt();
        $user_course_activity_ids = array();

        $questions = $chart_arr = array();
        $modelLmsCourseActivity = LmsCourseActivity::find()->select(['metadata'])->where(['id' => $id, 'status' => 'active'])->asArray()->one();

        if ($modelLmsCourseActivity !== null) {
            $questions = json_decode($modelLmsCourseActivity['metadata'], true);
            $questions = $questions['quiz'];
        }

        $modelLmsUserCourseActivity = LmsUserCourseActivity::find()->where(['course_activity_id' => $id, 'status' => 'active'])->asArray()->all();
        $user_course_activity_ids = ArrayHelper::getColumn($modelLmsUserCourseActivity, 'id');
        
        $modelLmsUserCourseActivityAttempt = LmsUserCourseActivityAttempt::find()->where(['status' => 'active'])->andWhere(['IN', 'user_course_activity_id', $user_course_activity_ids])->asArray()->all();
        $total_attempts = count($modelLmsUserCourseActivityAttempt);

        foreach ($questions as $question) {
            $chart_details = array();
            $response_arr = array();
            $option_label_arr = $response_percent_arr = array();

            foreach ($question['option'] as $optionKey => $optionValue) {
                $response_arr[$optionKey]['value'] = $optionValue;
                $response_arr[$optionKey]['response_num'] = 0;
            }

            foreach ($modelLmsUserCourseActivityAttempt as $k => $v) {
                $user_questions_metadata = $v['metadata'];
                $user_questions_metadata = json_decode($user_questions_metadata, true);

                foreach ($user_questions_metadata as $user_question_metadata) {
                    if (($question['sno'] == $user_question_metadata['sno']) && ($question['question'] == $user_question_metadata['question']) && isset($user_question_metadata['response'])) {
                        if (is_array($user_question_metadata['response']) || is_object($user_question_metadata['response'])) {
                            foreach ($user_question_metadata['response'] as $user_question_metadata_value) {
                                $questionArrayKey = array_search($user_question_metadata_value, $question['option']);
                                $response_arr[$questionArrayKey]['response_num'] += 1;
                            }
                        } else {
                            $questionArrayKey = array_search($user_question_metadata['response'], $question['option']);
                            $response_arr[$questionArrayKey]['response_num'] += 1;
                        }
                        
                    }       
                } 
            }

            foreach ($response_arr as $response_arr_value) {
                array_push($option_label_arr, $response_arr_value['value']);

                $data_percent = ($response_arr_value['response_num'] * $response_arr_value['value']);

                array_push($response_percent_arr, $data_percent);
            }

            $chart_details['sno'] = $question['sno'];
            $chart_details['question'] = $question['question'];
            $chart_details['label'] = $option_label_arr;
            $chart_details['data'] = $response_percent_arr;

            array_push($chart_arr, $chart_details);
        }

        return ['chart_details' => $chart_arr, 'total_attempts' => $total_attempts];
    }

    public function getAssessmentResponseDetails($id)
    {
        $modelLmsUserCourseActivity = new LmsUserCourseActivityAttempt();
        $user_course_activity_ids = array();

        $modelLmsUserCourseActivity = LmsUserCourseActivity::find()->where(['course_activity_id' => $id, 'status' => 'active'])->asArray()->all();
        $user_course_activity_ids = ArrayHelper::getColumn($modelLmsUserCourseActivity, 'id');

        $modelLmsUserCourseActivityAttempt = LmsUserCourseActivityAttempt::find()->where(['status' => 'active'])->andWhere(['IN', 'user_course_activity_id', $user_course_activity_ids])->asArray()->all();
        $total_attempts = count($modelLmsUserCourseActivityAttempt);
        
        foreach ($modelLmsUserCourseActivityAttempt as $_k => $_v) {
            $user_course_activity_attempt = $_v['metadata'];
            $user_course_activity_attempt = json_decode($user_course_activity_attempt, true);

            $score = 0;

            foreach ($user_course_activity_attempt as $__k => $__v) {
                if (isset($__v['result']) && $__v['result'] == 'correct') {
                    $score++;
                }
            }

            $modelLmsUserCourseActivityAttempt[$_k]['score'] = $score;
        }

        $tmp = array();

        foreach ($modelLmsUserCourseActivityAttempt as $_k => $_v) {
            $tmp[$_v['user_id']] = $_v['score'];
        }
        
        return $tmp;
    }
}
