<?php

namespace common\modules\lms\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

use common\modules\lms\models\LmsUserCourse;
use common\modules\lms\models\LmsUserCourseSearch;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * UserCourseController implements the CRUD actions for LmsUserCourse model.
 */
class UserCourseController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all LmsUserCourse models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new LmsUserCourseSearch();
        $modelCoreValue = new CoreValue();
        $modelCoreEntityClass = new CoreEntityClass();
        $modelCoreAttributeClass = new CoreAttributeClass();
        $modelUser = new \common\modules\user\models\User();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        $userArr = array();

        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);
        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $modelCoreValue = array();

        foreach ($tmpArray as $key => $value) {
            $modelCoreValue[$value['entity_id']] = $value['value'];
        }

        $modelUser = \common\modules\user\models\User::find()->where(['status' => 'active'])->asArray()->all();
        $userArr = \yii\helpers\ArrayHelper::map($modelUser, 'id', 'email');

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'modelCoreValue' => $modelCoreValue,
            'modelUser' => $userArr
        ]);
    }

    /**
     * Displays a single LmsUserCourse model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new LmsUserCourse model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new LmsUserCourse();
        $data = array();

        if (Yii::$app->request->isPost) {
            if ($user_course_id = $model->saveUserCourse(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $user_course_id]);
            }
        }

        $data['model'] = $model;

        return $this->render('create', [
            'data' => $data,
        ]);
    }

    /**
     * Updates an existing LmsUserCourse model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $data = array();

        if (Yii::$app->request->isPost) {
            if ($user_course_id = $model->saveUserCourse(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $user_course_id]);
            }
        }

        $data['model'] = $model;

        return $this->render('update', [
            'data' => $data,
        ]);
    }

    /**
     * Deletes an existing LmsUserCourse model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the LmsUserCourse model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return LmsUserCourse the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = LmsUserCourse::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionCourseEnrollments()
    {
        Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $out = [];

        if (isset($_POST['depdrop_parents'])) {
            $course_id = end($_POST['depdrop_parents']);
            
            $modelLmsUserCourse = LmsUserCourse::find()->where([
                'course_id' => $course_id, 
                'status' => 'active'
            ])
            ->asArray()
            ->all();

            $selected = null;
            
            if ($course_id != null && count($modelLmsUserCourse) > 0) {
                $selected = '';

                foreach ($modelLmsUserCourse as $k => $v) {
                    $out[] = [
                        'id' => $v['id'], 
                        'name' => \common\modules\user\models\User::getAttr(['email'], ['id' => $v['user_id']])
                    ];
                }
                
                return ['output' => $out, 'selected' => $selected];
            }
        }

        return ['output' => '', 'selected' => ''];
    }
}
