<?php

namespace common\modules\lms\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\web\UnauthorizedHttpException;
use yii\web\UploadedFile;
use Yadakhov\Json;

use common\modules\lms\models\LmsCourseActivity;
use common\modules\lms\models\LmsCourse;
use common\modules\lms\models\LmsUserCourse;
use common\modules\lms\models\LmsUserCourseActivity;
use common\modules\lms\models\LmsUserCourseActivityAttempt;
use common\modules\lms\models\FormAssignment;
use common\modules\lms\models\FormCaseStudy;
use common\modules\core\components\Globals;
use common\modules\core\components\GenesysRestClient;
use common\modules\core\models\FormFileUpload;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\user\models\User;
use common\modules\assetManager\models\AssetManager;

/**
 * CourseActivityController implements the CRUD actions for CourseActivity model.
 */
class CourseActivityController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['post'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'quiz'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index'],
                        'roles' => ['@'],
                    ],
                    [
                        'allow' => true,
                        'actions' => ['quiz'],
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function beforeAction($action)
    {      
        if ($action->id == 'save-quiz-response') {
            $this->enableCsrfValidation = false;
        }

        return parent::beforeAction($action);
    }

    /**
     * Displays a single CourseActivity model.
     * @param string $id
     * @return mixed
     */
    public function actionIndex($course_id)
    {
        if (Yii::$app->user->isGuest) {
            Url::remember();

            return $this->redirect('login', 302);
        } else {
            $modelLmsCourse = new LmsCourse();
            $modelLmsCourseActivity = new LmsCourseActivity();
            
            $modelLmsCourseActivityArr = array();
            $course_activity_details_arr = array();

            // validating course
            if (!LmsCourse::find()->where(['id' => $course_id, 'status' => 'active'])->exists()) {
                throw new NotFoundHttpException('Course does not exist.');
            } else {
                $modelLmsCourse = LmsCourse::findOne(['id' => $course_id, 'status' => 'active']);
            }

            // validating user-course
            if (!LmsUserCourse::find()->where(['user_id' => Yii::$app->user->identity->id, 'course_id' => $course_id, 'status' => 'active'])->exists()) {
                throw new UnauthorizedHttpException("You are not subscribed to this course.", 401);
            } else {
                $modelLmsUserCourse = LmsUserCourse::find()->where(['user_id' => Yii::$app->user->identity->id, 'course_id' => $course_id, 'status' => 'active'])->one();

                if (Globals::hasExpired($modelLmsUserCourse->expiration_date)) {
                    throw new UnauthorizedHttpException("Your subscription for this course has expired.", 401);
                }
            }

            // validating course-activity
            if (!LmsCourseActivity::find()->where(['course_id' => $course_id, 'status' => 'active'])->exists()) {
                throw new NotFoundHttpException('Course activity does not exist.');
            }

            $modelLmsCourseActivityArr = LmsCourseActivity::find()->where(['course_id' => $course_id, 'status' => 'active'])->asArray()->all();

            if (!empty($modelLmsCourseActivityArr)) {
                $entity_ids = \yii\helpers\ArrayHelper::getColumn($modelLmsCourseActivityArr, 'id');
                $course_activity_details_arr = $modelLmsCourseActivity->getCourseActivities($modelLmsCourseActivityArr);
            }

            return $this->render('index', [
                'modelLmsCourse' => $modelLmsCourse,
                'course_activities' => array_values($course_activity_details_arr),
            ]);            
        }
    }

    public function actionScorm12($course_activity_id)
    {
        if (Yii::$app->user->isGuest) {
            Url::remember();

            return $this->redirect('login.html', 302);
        } else {
            $modelLmsCourse = new LmsCourse();
            $modelLmsCourseActivity = new LmsCourseActivity();
            
            $modelLmsCourseActivityArr = array();
            $course_activity_details_arr = array();

            // Validating course-activity
            if (!LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->exists()) {
                throw new NotFoundHttpException('Course activity does not exist.');
            } else {
                $modelLmsCourseActivity = LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->asArray()->one();
            }

            // validating course
            if (!LmsCourse::find()->where(['id' => $modelLmsCourseActivity['course_id'], 'status' => 'active'])->exists()) {
                throw new NotFoundHttpException('Course does not exist.');
            } else {
                $modelLmsCourse = LmsCourse::find()->where(['id' => $modelLmsCourseActivity['course_id'], 'status' => 'active'])->asArray()->one();
            }

            // validating user-course
            if (!LmsUserCourse::find()->where(['user_id' => Yii::$app->user->identity->id, 'course_id' => $modelLmsCourse['id'], 'status' => 'active'])->exists()) {
                throw new UnauthorizedHttpException("You are not subscribed to this course.", 401);
            } else {
                $modelLmsUserCourse = LmsUserCourse::find()->where(['user_id' => Yii::$app->user->identity->id, 'course_id' => $modelLmsCourse['id'], 'status' => 'active'])->one();

                if (Globals::hasExpired($modelLmsUserCourse->expiration_date)) {
                    throw new UnauthorizedHttpException("Your subscription for this course has expired.", 401);
                }
            }

            return $this->render('scorm12', [
                'modelLmsCourse' => $modelLmsCourse,
                'course_activity' => $modelLmsCourseActivity,
            ]);            
        }
    }
    
    public function actionJournal()
    {
        $modelUser = new User;
        $modelLmsUserCourseActivityAttempt = new LmsUserCourseActivityAttempt;

        if (Yii::$app->user->isGuest) {
            Url::remember();

            return $this->redirect('login', 302);
        } else {
            $modelUser = User::find()->where(['id' => Yii::$app->user->identity->id, 'status' => 'active'])->one();

            $journal_data = $modelLmsUserCourseActivityAttempt->getUserData($modelUser->id);

            return $this->render('journal', [
                'journal_data' => $journal_data, 
                'modelUser' => $modelUser, 
            ]);
        }
    }

    public function actionWebObject($id = '')
    {
        return $this->renderPartial('web-object', [
            'id' => $id, 
        ]);
    }

    public function actionAssignment($course_activity_id)
    {
        $session = Yii::$app->session;

        $modelLmsCourseActivity = new LmsCourseActivity();
        $modelLmsUserCourseActivity = new LmsUserCourseActivity();
        $modelAssetManager = new AssetManager();
        $modelFormAssignment = new FormAssignment();
        $modelFormFileUpload = new FormFileUpload();
        $data = array();
        $assignment = array();
        $assignment_data = array();

        if (!LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException('Course activity does not exist.');
        }

        $modelLmsCourseActivity = LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->one();

        $assignment = $modelLmsCourseActivity->metadata;

        $assignment = json_decode($assignment, true);

        if (Yii::$app->request->isPost) {
            $modelFormFileUpload->file = UploadedFile::getInstance($modelFormFileUpload, 'file');
            
            if (!empty($modelFormFileUpload->file)) {
                if ($tempRelativePath = $modelFormFileUpload->upload('assignment', Yii::$app->user->identity->id . '_' . $modelLmsCourseActivity->course_id . '_' . $course_activity_id . '.' . $modelFormFileUpload->file->extension)) {
                    $transaction = Yii::$app->db->beginTransaction();

                    try {
                        if (!$session->has('uploadedFileName')) {
                            $transaction->rollBack();

                            throw new HttpException(500, 'Unable to save file.');
                        }

                        if (AssetManager::find()->where(['path' => $tempRelativePath . $session->get('uploadedFileName')])->exists()) {
                            $modelAssetManager = AssetManager::find()->where(['path' => $tempRelativePath . $session->get('uploadedFileName')])->one();
                        }

                        $modelAssetManager->name = Yii::$app->user->identity->id . '_' . $modelLmsCourseActivity->course_id . '_' . $course_activity_id . '.' . $modelFormFileUpload->file->extension;
                        $modelAssetManager->app_alias = 'assets';
                        $modelAssetManager->path = $tempRelativePath . $session->get('uploadedFileName');
                        $modelAssetManager->mime_type = $modelFormFileUpload->file->type;
                        $modelAssetManager->size = $modelFormFileUpload->file->size;
                        $modelAssetManager->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                        $modelAssetManager->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                        $modelAssetManager->status = 'active';

                        if ($modelAssetManager->validate()) {           
                            $modelAssetManager->save();

                            $assignment_data['answer'] = Yii::$app->request->post('FormAssignment')['answer'];
                            $assignment_data['file'] = $modelAssetManager->path;

                            $data = [
                                'LmsUserCourseActivity' => [
                                    'course_activity_id' => $course_activity_id, 
                                    'user_id' => Yii::$app->user->identity->id, 
                                    'created_by' => Yii::$app->user->identity->id, 
                                    'updated_by' => Yii::$app->user->identity->id, 
                                    'status' => 'active', 
                                ], 
                                'LmsUserCourseActivityAttempt' => [
                                    'user_course_activity_id' => NULL, 
                                    'user_id' => Yii::$app->user->identity->id, 
                                    'attempt' => 0, 
                                    'metadata' => json_encode($assignment_data), 
                                    'created_by' => Yii::$app->user->identity->id, 
                                    'updated_by' => Yii::$app->user->identity->id, 
                                    'status' => 'active', 
                                ],
                            ];

                            if (!$modelLmsUserCourseActivity->saveUserCourseActivity($data)) {
                                Yii::$app->session->setFlash('error', "Unable to submit assignment " . Globals::renderErrors($modelBespoke->getErrors()));
                            } else {
                                $transaction->commit();

                                Yii::$app->session->setFlash('success', "Assignment submitted successfully.");
                            }
                        } else {
                            Yii::$app->session->setFlash('error', "Unable to submit assignment " . Globals::renderErrors($modelAssetManager->getErrors()));
                        }
                    } catch (\Exception $e) {
                        $transaction->rollBack();

                        throw $e;
                    }
                }
            }

            return $this->render('assignment', [
                'assignment' => $assignment,
                'course_id' => $modelLmsCourseActivity->course_id,
                'course_activity_id' => $modelLmsCourseActivity->id,
                'modelAssetManager' => $modelAssetManager,
                'modelFormAssignment' => $modelFormAssignment,
                'modelFormFileUpload' => $modelFormFileUpload,
            ]);
        } else {
            return $this->render('assignment', [
                'assignment' => $assignment,
                'course_id' => $modelLmsCourseActivity->course_id,
                'course_activity_id' => $modelLmsCourseActivity->id,
                'modelAssetManager' => $modelAssetManager,
                'modelFormAssignment' => $modelFormAssignment,
                'modelFormFileUpload' => $modelFormFileUpload,
            ]);
        }
    }

    public function actionCaseStudy($course_activity_id)
    {
        $session = Yii::$app->session;

        $modelLmsCourseActivity = new LmsCourseActivity();
        $modelLmsUserCourseActivity = new LmsUserCourseActivity();
        $modelAssetManager = new AssetManager();
        $modelFormCaseStudy = new FormCaseStudy();
        $modelFormFileUpload = new FormFileUpload();
        $data = array();
        $case_study = array();
        $case_study_data = array();

        if (!LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException('Course activity does not exist.');
        }

        $modelLmsCourseActivity = LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->one();

        $case_study = $modelLmsCourseActivity->metadata;

        $case_study = json_decode($case_study, true);

        if (Yii::$app->request->isPost) {
            $modelFormFileUpload->file = UploadedFile::getInstance($modelFormFileUpload, 'file');
            
            if (!empty($modelFormFileUpload->file)) {
                if ($tempRelativePath = $modelFormFileUpload->upload('case-study', Yii::$app->user->identity->id . '_' . $modelLmsCourseActivity->course_id . '_' . $course_activity_id . '.' . $modelFormFileUpload->file->extension)) {
                    $transaction = Yii::$app->db->beginTransaction();

                    try {
                        if (!$session->has('uploadedFileName')) {
                            $transaction->rollBack();

                            throw new HttpException(500, 'Unable to save file.');
                        }

                        if (AssetManager::find()->where(['path' => $tempRelativePath . $session->get('uploadedFileName')])->exists()) {
                            $modelAssetManager = AssetManager::find()->where(['path' => $tempRelativePath . $session->get('uploadedFileName')])->one();
                        }

                        $modelAssetManager->name = Yii::$app->user->identity->id . '_' . $modelLmsCourseActivity->course_id . '_' . $course_activity_id . '.' . $modelFormFileUpload->file->extension;
                        $modelAssetManager->app_alias = 'assets';
                        $modelAssetManager->path = $tempRelativePath . $session->get('uploadedFileName');
                        $modelAssetManager->mime_type = $modelFormFileUpload->file->type;
                        $modelAssetManager->size = $modelFormFileUpload->file->size;
                        $modelAssetManager->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                        $modelAssetManager->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                        $modelAssetManager->status = 'active';

                        if ($modelAssetManager->validate()) {           
                            $modelAssetManager->save();

                            $case_study_data['answer'] = Yii::$app->request->post('FormCaseStudy')['answer'];
                            $case_study_data['file'] = $modelAssetManager->path;

                            $data = [
                                'LmsUserCourseActivity' => [
                                    'course_activity_id' => $course_activity_id, 
                                    'user_id' => Yii::$app->user->identity->id, 
                                    'created_by' => Yii::$app->user->identity->id, 
                                    'updated_by' => Yii::$app->user->identity->id, 
                                    'status' => 'active', 
                                ], 
                                'LmsUserCourseActivityAttempt' => [
                                    'user_course_activity_id' => NULL, 
                                    'user_id' => Yii::$app->user->identity->id, 
                                    'attempt' => 0, 
                                    'metadata' => json_encode($case_study_data), 
                                    'created_by' => Yii::$app->user->identity->id, 
                                    'updated_by' => Yii::$app->user->identity->id, 
                                    'status' => 'active', 
                                ],
                            ];

                            if (!$modelLmsUserCourseActivity->saveUserCourseActivity($data)) {
                                Yii::$app->session->setFlash('error', "Unable to submit case-study " . Globals::renderErrors($modelBespoke->getErrors()));
                            } else {
                                $transaction->commit();

                                Yii::$app->session->setFlash('success', "Case-study submitted successfully.");
                            }
                        } else {
                            Yii::$app->session->setFlash('error', "Unable to submit case-study " . Globals::renderErrors($modelAssetManager->getErrors()));
                        }
                    } catch (\Exception $e) {
                        $transaction->rollBack();

                        throw $e;
                    }
                }
            }

            return $this->render('case-study', [
                'case_study' => $case_study,
                'course_id' => $modelLmsCourseActivity->course_id,
                'course_activity_id' => $modelLmsCourseActivity->id,
                'modelAssetManager' => $modelAssetManager,
                'modelFormCaseStudy' => $modelFormCaseStudy,
                'modelFormFileUpload' => $modelFormFileUpload,
            ]);
        } else {
            return $this->render('case-study', [
                'case_study' => $case_study,
                'course_id' => $modelLmsCourseActivity->course_id,
                'course_activity_id' => $modelLmsCourseActivity->id,
                'modelAssetManager' => $modelAssetManager,
                'modelFormCaseStudy' => $modelFormCaseStudy,
                'modelFormFileUpload' => $modelFormFileUpload,
            ]);
        }
    }

    public function actionQuiz($course_activity_id)
    {
        $modelLmsCourseActivity = new LmsCourseActivity();
        $questions = array();
        $duration = '-1';

        if (!LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException('Course activity does not exist.');
        }

        $modelLmsCourseActivity = LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->one();

        $questions = $modelLmsCourseActivity->metadata;

        $questions = json_decode($questions);

        if (isset($questions->duration_in_minutes) && $questions->duration_in_minutes !== null && !empty($questions->duration_in_minutes)) {
            $duration = $questions->duration_in_minutes;
        }

        if (isset($questions->quiz)) {
            $questions = $questions->quiz;
        } else {
            $questions = array();
        }
         
        return $this->render('quiz', [
            'questions' => $questions,
            'duration' => $duration,
            'course_id' => $modelLmsCourseActivity->course_id,
            'course_activity_id' => $modelLmsCourseActivity->id,
        ]);
    }

    public function actionLaunchQuizRest($tok, $course_activity_id)
    {
        $restClient = new GenesysRestClient($tok);

        if (!isset($course_activity_id) || empty($course_activity_id)) {
            throw new NotFoundHttpException('Course activity does not exist.');
        }
         
        return $this->render('launch-quiz', [
            'tok' => $tok,
            'course_activity_id' => $course_activity_id,
        ]);
    }

    public function actionQuizHomeRest($tok)
    {
        $restClient = new GenesysRestClient($tok);

        $course_ids = LmsUserCourse::find()->select('course_id')->where(['user_id' => User::getAttr(['id'], ['access_token' => $tok]), 'status' => 'active'])->asArray()->all();

        $course_ids_tmp = $course_ids;

        $course_ids = array();

        foreach ($course_ids_tmp as $v) {
            $course_ids[] = $v['course_id'];
        }

        $course_ids = array_unique($course_ids);

        $course_activity_ids = LmsCourseActivity::find()->select('id')->where(['status' => 'active'])->andWhere(['IN', 'course_id', $course_ids])->asArray()->all();

        $course_activity_ids_tmp = $course_activity_ids;

        $course_activity_ids = array();

        foreach ($course_activity_ids_tmp as $v) {
            $course_activity_ids[] = $v['id'];
        }

        $course_activity_ids = array_unique($course_activity_ids);
         
        return $this->render('quiz-home', [
            'tok' => $tok,
            'course_ids' => $course_ids,
            'course_activity_ids' => $course_activity_ids,
        ]);
    }

    public function actionQuizRest($tok, $course_activity_id)
    {
        if (Yii::$app->request->get('fromNode', 0) == 1) {
            $data = (array) Yii::$app->request->get('data', []);
            echo json_encode($data); die;
        }
        
        $restClient = new GenesysRestClient($tok);
        $questions = array();
        $duration = -1;

        $response = new Json($restClient->callApi('course-activities', 'GET', $course_activity_id));

        $course_activity_id = $response->get('data.id');

        if (!isset($course_activity_id) || empty($course_activity_id)) {
            throw new NotFoundHttpException('Course activity does not exist.');
        }

        $metadata = new Json($response->get('data.metadata'));
        $metadata = $metadata->toArray();

        if (isset($metadata['duration_in_minutes']) && $metadata['duration_in_minutes'] !== null && !empty($metadata['duration_in_minutes'])) {
            $duration = $metadata['duration_in_minutes'];
        }

        if (isset($metadata['quiz'])) {
            $questions = $metadata['quiz'];
        }
         
        return $this->render('quiz', [
            'tok' => $tok,
            'questions' => $questions,
            'duration' => $duration,
            'course_id' => $response->get('data.course_id'),
            'course_activity_id' => $course_activity_id,
        ]);
    }

    public function actionSaveQuizResponse()
    {
        $session = Yii::$app->session;
        
        $session->open();

        // Handling data in JSON format on the server-side using PHP
        header("Content-Type: application/json");

        $modelLmsUserCourseActivity = new LmsUserCourseActivity();

        $quiz_data = array();
        $data = array();
        $metadata = array();
        
        foreach (explode('&', file_get_contents("php://input")) as $key => $value) {
            switch ($key) {
                case 0:
                    $tmp_quiz_data = explode('questions=', $value);
                    $quiz_data['questions'] = urldecode($tmp_quiz_data[1]);

                    break;

                case 1:
                    $tmp_quiz_data = explode('course_id=', $value);
                    $quiz_data['course_id'] = urldecode($tmp_quiz_data[1]);
                    
                    break;

                case 2:
                    $tmp_quiz_data = explode('course_activity_id=', $value);
                    $quiz_data['course_activity_id'] = urldecode($tmp_quiz_data[1]);
                    
                    break;
                
                default:
                    break;
            }
        }

        $metadata = json_decode($quiz_data['questions']);

        $data = [
            'LmsUserCourseActivity' => [
                'course_activity_id' => $quiz_data['course_activity_id'], 
                'user_id' => Yii::$app->user->identity->id, 
                'created_by' => Yii::$app->user->identity->id, 
                'updated_by' => Yii::$app->user->identity->id, 
                'status' => 'active', 
            ], 
            'LmsUserCourseActivityAttempt' => [
                'user_course_activity_id' => null, 
                'user_id' => Yii::$app->user->identity->id, 
                'attempt' => 0, 
                'metadata' => json_encode($metadata), 
                'created_by' => Yii::$app->user->identity->id, 
                'updated_by' => Yii::$app->user->identity->id, 
                'status' => 'active', 
            ],
        ];

        if (!$modelLmsUserCourseActivity->saveUserCourseActivity($data)) {
            return 'Unable to record progress of this session';
        } else {
            return 'ok';
        }
    }

    public function actionSaveQuizResponseRest($tok)
    {
        $restClient = new GenesysRestClient($tok);
        $quiz_data = array();
        $data = array();
        $metadata = array();

        // Handling data in JSON format on the server-side using PHP
        header("Content-Type: application/json");

        foreach (explode('&', file_get_contents("php://input")) as $key => $value) {
            switch ($key) {
                case 0:
                    $tmp_quiz_data = explode('questions=', $value);
                    $quiz_data['questions'] = urldecode($tmp_quiz_data[1]);

                    break;

                case 1:
                    $tmp_quiz_data = explode('course_id=', $value);
                    $quiz_data['course_id'] = urldecode($tmp_quiz_data[1]);
                    
                    break;

                case 2:
                    $tmp_quiz_data = explode('course_activity_id=', $value);
                    $quiz_data['course_activity_id'] = urldecode($tmp_quiz_data[1]);
                    
                    break;
                
                default:
                    break;
            }
        }

        $metadata = json_decode($quiz_data['questions']);

        $data = [
            'LmsUserCourseActivity' => [
                'course_activity_id' => $quiz_data['course_activity_id'], 
                'user_id' => null, 
                'created_by' => null, 
                'updated_by' => null, 
                'status' => 'active', 
            ], 
            'LmsUserCourseActivityAttempt' => [
                'user_course_activity_id' => null, 
                'user_id' => null, 
                'attempt' => 0, 
                'metadata' => json_encode($metadata), 
                'created_by' => null, 
                'updated_by' => null, 
                'status' => 'active', 
            ],
        ];

        $response = new Json($restClient->callApi('user-course-activities', 'POST', '', $data));

        if (!empty($response->get('data')) && is_int($response->get('data'))) {
            return 'ok';
        } else {
            return $response->get('data');
        }
    }

    public function actionEbook($course_activity_id)
    {
        $modelLmsCourseActivity = new LmsCourseActivity();

        if (!LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException('Course activity does not exist.');
        }

        $modelLmsCourseActivity = LmsCourseActivity::find()->where(['id' => $course_activity_id, 'status' => 'active'])->one();

        $json = new Json($modelLmsCourseActivity->metadata);

        return $this->render('ebook', [
            'json' => $json,
            'modelLmsCourseActivity' => $modelLmsCourseActivity,
            'course_id' => $modelLmsCourseActivity->course_id,
            'course_activity_id' => $modelLmsCourseActivity->id,
        ]);
    }
}
