<?php

namespace common\modules\lms\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\lms\models\LmsCourse;
use common\modules\lms\models\LmsUserCourse;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

class CourseController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'preview'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['index', 'preview'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'index' => ['get'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionIndex()
    {
        if (Yii::$app->user->isGuest) {
            \yii\helpers\Url::remember();

            return $this->redirect('login', 302);
        }

        $modelLmsUserCourse = new LmsUserCourse();
        $courses = $modelLmsUserCourse->getUserCourses();

        return $this->render('index', [
            'courses' => $courses,
        ]);
    }
    
    public function actionAddUserCourse($course_id, $user_id)
    {
        $modelCourse = new LmsUserCourse();

        if ($course_id !== null && $user_id !== null) {
            $modelCourse->user_id = $user_id;
            $modelCourse->course_id = $course_id;
            $modelCourse->expiration_date = date('Y-m-d h:m:s',strtotime(date("Y-m-d h:m:s", mktime()) . " + 365 day"));
            $modelCourse->valid = 'yes';
            $modelCourse->status = 'active';

            if ($modelCourse->validate()) {
                $modelCourse->save();
                return Yii::$app->response->redirect(['/site/site/thank-you', 'msg' => 'Thank you for registering with us....']);
            }
        }
    }

    public function actionPreview()
    {
        return $this->render('preview');
    }
}
