<?php

namespace common\modules\lms\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\web\BadRequestHttpException;
use Yadakhov\Json;

use common\modules\user\models\UserProfile;
use common\modules\lms\models\LmsCourseActivity;
use common\modules\lms\models\LmsUserCourseActivity;
use common\modules\lms\models\LmsUserCourseActivityAttempt;

/**
 * ScormController implements the CRUD actions for Scorm model.
 */
class ScormController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete', 'save-cmi-data', 'get-cmi-data', 'get-student', 'save-user-data', 'get-user-data'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['save-cmi-data', 'get-cmi-data', 'get-student', 'save-user-data', 'get-user-data'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function beforeAction($action)
    {      
        if ($action->id == 'save-cmi-data') {
            $this->enableCsrfValidation = false;
        }

        if ($action->id == 'get-cmi-data') {
            $this->enableCsrfValidation = false;
        }
		
		if ($action->id == 'get-student') {
            $this->enableCsrfValidation = false;
        }
		
		if ($action->id == 'save-user-data') {
            $this->enableCsrfValidation = false;
        }

        if ($action->id == 'get-user-data') {
            $this->enableCsrfValidation = false;
        }

        return parent::beforeAction($action);
    }

    /**
     * Lists all Scorm models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ScormSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Scorm model.
     * @param string $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Scorm model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new LmsUserCourseActivity();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        } else {
            return $this->render('create', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Updates an existing Scorm model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param string $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        } else {
            return $this->render('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing Scorm model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param string $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Scorm model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param string $id
     * @return Scorm the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = LmsUserCourseActivity::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }

    public function actionPlayer($id, $launch_file)
    {
        if (Yii::$app->user->isGuest) {
            \yii\helpers\Url::remember();

            return $this->redirect('login.html', 302);
        }

        $attempts = 0;
        $max_attempts = 0;

        if (!LmsCourseActivity::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException("Course activity not found.", 404);
        }

        $modelLmsCourseActivity = LmsCourseActivity::find()->where(['id' => $id, 'status' => 'active'])->one();
        $json = new Json($modelLmsCourseActivity->metadata);
        
        if (!empty($json->get('max_attempts'))) {
            $max_attempts = $json->get('max_attempts');
		}
		
		if (LmsUserCourseActivity::find()->where(['course_activity_id' => $id, 'user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
			$modelLmsUserCourseActivity = LmsUserCourseActivity::find()->where(['course_activity_id' => $id, 'user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->one();
			
			if (LmsUserCourseActivityAttempt::find()->where(['user_course_activity_id' => $modelLmsUserCourseActivity->id, 'user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
				$attempts = LmsUserCourseActivityAttempt::find()->where(['user_course_activity_id' => $modelLmsUserCourseActivity->id, 'user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->count();
			}
		}
		
		return $this->render('player', [
            'id' => $id,
            'launch_file' => $launch_file,
			'max_attempts' => $max_attempts,
			'attempts' => $attempts,
        ]);
    }

    public function actionSaveCmiData()
    {
        $session = Yii::$app->session;
		
		$session->open();
		
		// Handling data in JSON format on the server-side using PHP
        header("Content-Type: application/json");

        $modelLmsUserCourseActivity = new LmsUserCourseActivity();

        $scorm_data = array();
        $data = array();
        $metadata = array();
        $loop_counter = 0;

        foreach (explode('&', file_get_contents("php://input")) as $key => $value) {
            $loop_counter++; 

            switch ($loop_counter) {
                case 1:
                    $tmp_scorm_data = explode('cmi_data=', $value);        
                    $scorm_data['cmi_data'] = $tmp_scorm_data[1];

                    break;

                case 2:
                    $tmp_scorm_data = explode('id=', $value);        
                    $scorm_data['id'] = $tmp_scorm_data[1];
                    
                    break;

                case 3:
                    $tmp_scorm_data = explode('course_activity_id=', $value);        
                    $scorm_data['course_activity_id'] = $tmp_scorm_data[1];
                    
                    break;
                
                default:
                    break;
            }
        }

        $metadata = [
            'cmi_data' => json_decode($scorm_data['cmi_data']), 
            'user_data' => json_decode(isset($_SESSION['sco.user_data']) ? $_SESSION['sco.user_data'] : ''), 
        ];

        $data = [
            'LmsUserCourseActivity' => [
                'course_activity_id' => $scorm_data['course_activity_id'], 
                'user_id' => Yii::$app->user->identity->id, 
                'created_by' => Yii::$app->user->identity->id, 
                'updated_by' => Yii::$app->user->identity->id, 
                'status' => 'active', 
            ], 
            'LmsUserCourseActivityAttempt' => [
                'user_course_activity_id' => NULL, 
                'user_id' => Yii::$app->user->identity->id, 
                'attempt' => 0, 
                'metadata' => json_encode($metadata), 
                'created_by' => Yii::$app->user->identity->id, 
                'updated_by' => Yii::$app->user->identity->id, 
                'status' => 'active', 
            ],
        ];

        if (!$modelLmsUserCourseActivity->saveUserCourseActivity($data)) {
            return 'Unable to record progress of this session';
        } else {
            return 'ok';
        }
    }

    public function actionGetCmiData()
    {
        // Handling data in JSON format on the server-side using PHP
        header("Content-Type: application/json");

        $modelLmsUserCourseActivity = new LmsUserCourseActivity();
        $modelLmsUserCourseActivityAttempt = new LmsUserCourseActivityAttempt();

        $query_data = array();
        $loop_counter = 0;

        foreach (explode('&', file_get_contents("php://input")) as $key => $value) {
            $loop_counter++; 

            switch ($loop_counter) {
                case 2:
                    $tmp_query_data = explode('id=', $value);
                    $query_data['id'] = $tmp_query_data[1];

                    break;
                
                default:
                    break;
            }
        }

        if (Yii::$app->user->isGuest) {
            return 'notfound';
        }

        $user_course_activity_id = LmsUserCourseActivity::find()->select(['id'])->where(['user_id' => Yii::$app->user->identity->id, 'course_activity_id' => $query_data['id'], 'status' => 'active'])->column()[0];

        if (!isset($user_course_activity_id) || empty($user_course_activity_id)) {
            return 'notfound';
        }

        $modelLmsUserCourseActivityAttempt = LmsUserCourseActivityAttempt::find()->where(['user_id' => Yii::$app->user->identity->id, 'user_course_activity_id' => $user_course_activity_id, 'status' => 'active'])->orderBy('attempt DESC')->all();

        if (!isset($modelLmsUserCourseActivityAttempt[0]->metadata) || empty($modelLmsUserCourseActivityAttempt[0]->metadata)) {
            return 'notok';
        } else {
            $json = new Json($modelLmsUserCourseActivityAttempt[0]->metadata);
            $json = new Json($json->get('cmi_data'));
            
            return $json->toString();
        }
    }

    public function actionGetStudent()
    {
        $modelUserProfile = new UserProfile();

        $return_data = array();

        if (Yii::$app->user->isGuest) {
            return 'notfound';
        }

        $modelUserProfile = UserProfile::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->one();

        $return_data = $modelUserProfile;
        $return_data['email'] = Yii::$app->user->identity->email;

        return json_encode($return_data);
    }
	
	public function actionSaveUserData()
    {
        $session = Yii::$app->session;
        $request = Yii::$app->request;
		
		$session->open();
		
		$modelLmsCourseActivity = new LmsCourseActivity();
        $modelLmsUserCourseActivity = new LmsUserCourseActivity();

        // validating request params
        if (!$request->post('id')) {
            throw new BadRequestHttpException("Course activity missing.", 400);
        }

        // validating course-activity
        if (!LmsCourseActivity::find()->where(['id' => $request->post('id'), 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException('Course activity does not exist.');
        }
		
		$json = new Json($request->post('journal_data'));
        $_SESSION['sco.user_data'] = $json->toString();

        return $_SESSION['sco.user_data'];
    }

    public function actionGetUserData()
    {
        $session = Yii::$app->session;
		$request = Yii::$app->request;
		
		$session->open();
		
		$modelLmsCourseActivity = new LmsCourseActivity();
        $modelLmsUserCourseActivity = new LmsUserCourseActivity();
        $modelLmsUserCourseActivityAttempt = new LmsUserCourseActivityAttempt();

        // validating request params
        if (!$request->post('id')) {
            throw new BadRequestHttpException("Course activity missing.", 400);
        }

        // validating user
        if (Yii::$app->user->isGuest) {
            throw new \yii\web\UnauthorizedHttpException("Unauthorized user.", 401);
        }

        // validating course-activity
        if (!LmsCourseActivity::find()->where(['id' => $request->post('id'), 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException('Course activity does not exist.');
        }

        // validating scorm
        if (!LmsUserCourseActivity::find()->where(['course_activity_id' => $request->post('id'), 'user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
            // fetching user data from the last attempted available scorm
            if (LmsUserCourseActivity::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
                $modelLmsUserCourseActivity = LmsUserCourseActivity::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->orderBy(['id' => SORT_DESC])->one();
            } else {
                throw new NotFoundHttpException('Scorm does not exist.');
            }
        } else {
            $modelLmsUserCourseActivity = LmsUserCourseActivity::find()->where(['course_activity_id' => $request->post('id'), 'user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->one();
        }
        
        // validating scorm record
		if (!LmsUserCourseActivityAttempt::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
            throw new NotFoundHttpException('Scorm record does not exist.');
        } else {
			$modelLmsUserCourseActivityAttempt = LmsUserCourseActivityAttempt::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->orderBy(['id' => SORT_DESC])->one();
		}
        
        $json = new Json($modelLmsUserCourseActivityAttempt->metadata);
        $json = new Json($json->get('user_data'));
        
        echo $json->toString();
    }
}
