<?php

namespace common\modules\lms\controllers\frontend;

use Yii;
use yii\web\Controller;
use Yadakhov\Json;

use common\modules\lms\models\LmsCourse;
use common\modules\lms\models\LmsUserCourse;
use common\modules\lms\models\FormMetadataUserCourse;
use common\modules\core\components\GenesysRestClient;
use common\modules\user\models\User;

/**
 * UserCourseController implements the CRUD actions for UserCourse model.
 */
class UserCourseController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index'],
                        'roles' => ['@'],
                    ],
                    [
                        'allow' => true,
                        'actions' => ['view'],
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
        ];
    }

    /**
     * Displays a single CourseActivity model.
     * @param string $id
     * @return mixed
     */
    public function actionAddUserCourse($course_id, $userid)
    {
        $modelCourse = new LmsUserCourse();
        $modelCourseActivity = new LmsUserCourseActivity();
		$modelCourseActivityArr = array();

        if(Yii::$app->user->isGuest) {
            return $this->render('view', [
                'modelCourseActivityArr' => $modelCourseActivityArr,
            ]);   
        }

        $user_id = Yii::$app->user->id;
        $modelUser = User::findOne(['id' => $user_id, 'status' => 'active']);

        // validating access token
        if (isset($modelUser->id) && !empty($modelUser->id)) {
            // validating course
            if (!LmsCourse::find()->where(['id' => $course_id, 'status' => 'active'])->exists()) {
                throw new \yii\web\NotFoundHttpException('Course does not exist.');
            } else {
                $modelCourse = LmsCourse::findOne(['id' => $course_id, 'status' => 'active']); 
            }
            // validating course-activity
            if (!LmsCourseActivity::find()->where(['course_id' => $course_id, 'status' => 'active'])->exists()) {
                throw new \yii\web\NotFoundHttpException('Activity does not exist.');
            }
            
            $modelUserCourse = LmsUserCourse::findOne(['user_id' => $modelUser->id, 'course_id' => $course_id, 'status' => 'active']);			

            // validating user-course subscription
            if (isset($modelUserCourse->id) && !empty($modelUserCourse->id) && $modelUserCourse->valid == 'yes') {
                if (date('Y-m-d h:m:s') >= $modelUserCourse->expiration_date) {
                    throw new \yii\web\UnauthorizedHttpException("Your subscription for this course has expired.", 401);
                }
            } else {
                throw new \yii\web\UnauthorizedHttpException("You are not subscribed to this course.", 401);
            }
            $modelCourseActivityArr = LmsCourseActivity::find()->where(['course_id' => $course_id, 'status' => 'active'])->asArray()->all();
            $courseActivityDetailsArr = array();
            if(!empty($modelCourseActivityArr)) {
                $entity_id_arr = \yii\helpers\ArrayHelper::getColumn($modelCourseActivityArr, 'id');
                $courseActivityDetailsArr = $this->getCourseActivityDetails($modelCourseActivityArr);
            }
            
            return $this->render('index', [
                'courseActivityDetails' => $courseActivityDetailsArr,
                'modelCourse' => $modelCourse,
            ]);   
        } else {
            throw new \yii\web\UnauthorizedHttpException("Invalid User.", 401);
        }
    }

    public function actionUpdateMetadataRest($tok)
    {
        $model = new FormMetadataUserCourse();
        $restClient = new GenesysRestClient($tok);
        $data = array();
        $metadata = array();

        if (Yii::$app->request->isPost) {
            $response = new Json($restClient->callApi('user-courses', 'PUT', User::getAttr(['id'], ['access_token' => $tok]), Yii::$app->request->post('FormMetadataUserCourse')));

            Yii::$app->session->setFlash('success', "Exam enrollment successful.");

            // TODO: Exam enrollment confirmation emailed to the student

            return $this->redirect('/', 302);
        } else {
            return $this->render('update-metadata', [
                'tok' => $tok,
                'model' => $model,
            ]);
        }
    }

    public function actionConvertTimeToTimezone()
    {
        Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;

        $out = [];

        if (isset($_POST['depdrop_parents'])) {
            $parents = $_POST['depdrop_parents'];

            if ($parents != null) {
                $cat_id = $parents[0];
                $out = self::getSubCatList($cat_id); 
                // the getSubCatList function will query the database based on the
                // cat_id and return an array like below:
                // [
                //    ['id'=>'<sub-cat-id-1>', 'name'=>'<sub-cat-name1>'],
                //    ['id'=>'<sub-cat_id_2>', 'name'=>'<sub-cat-name2>']
                // ]
                return ['output' => $out, 'selected' => ''];
            }
        }

        return ['output' => '', 'selected' => ''];
    }
}
