<?php

namespace common\modules\lms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%lms_attendance}}".
 *
 * @property int $id
 * @property int $cohort_user_course_id
 * @property string $date
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property LmsCohortUserCourse $cohortUserCourse
 */
class LmsAttendance extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%lms_attendance}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['cohort_user_course_id', 'date'], 'required'],
            [['cohort_user_course_id', 'created_by', 'updated_by'], 'integer'],
            [['date'], 'safe'],
            [['status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['cohort_user_course_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsCohortUserCourse::className(), 'targetAttribute' => ['cohort_user_course_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'cohort_user_course_id' => 'Cohort User Course ID',
            'date' => 'Date',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCohortUserCourse()
    {
        return $this->hasOne(\common\modules\lms\models\LmsCohortUserCourse::className(), ['id' => 'cohort_user_course_id']);
    }

    public function getCohort()
    {
        return $this->hasOne(\common\modules\lms\models\LmsCohort::className(), ['id' => 'cohort_id'])->via('cohortUserCourse');
    }

    public function getUserCourse()
    {
        return $this->hasOne(\common\modules\lms\models\LmsUserCourse::className(), ['id' => 'user_course_id'])->via('cohortUserCourse');
    }

    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::class, ['id' => 'user_id'])->via('userCourse');
    }

    public function saveAttendance($post_data, $attendance_id = 0)
    {
        $session = Yii::$app->session;

        $modelLmsAttendance = new LmsAttendance();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();
    
        try {
            foreach ($post_data['LmsCohortUserCourse']['cohort_user_course_id'] as $cohort_user_course_k => $cohort_user_course_v) {
                $modelLmsAttendance = new \common\modules\lms\models\LmsAttendance();

                if (\common\modules\lms\models\LmsAttendance::find()
                    ->where([
                        'cohort_user_course_id' => $cohort_user_course_v, 
                        'date' => $post_data['LmsAttendance']['date'], 
                        'status' => 'active'
                    ])
                    ->exists()
                ) { // Model already exists
                    $modelLmsAttendance = \common\modules\lms\models\LmsAttendance::find()
                    ->where([
                        'cohort_user_course_id' => $cohort_user_course_v, 
                        'date' => $post_data['LmsAttendance']['date'], 
                        'status' => 'active'
                    ])
                    ->one();
                } else {    // Create a new model
                    $modelLmsAttendance->cohort_user_course_id = $cohort_user_course_v;
                    $modelLmsAttendance->date = $post_data['LmsAttendance']['date'];
                    $modelLmsAttendance->status = 'active';
                }

                if ($modelLmsAttendance->validate()) {
                    $modelLmsAttendance->save();
                } else {
                    $breakFlag = true;echo "<pre>"; print_r($modelLmsAttendance->getErrors()); die();

                    Yii::$app->session->setFlash('error', "Unable to save attendance " . Globals::renderErrors($modelLmsAttendance->getErrors()));

                    break;
                }
            }

            // delete remaining attendances
            if (!$breakFlag) {
                $attendances = \common\modules\lms\models\LmsAttendance::find()
                    ->where([
                        'date' => $post_data['LmsAttendance']['date'], 
                        'status' => 'active'
                    ])
                    ->andWhere(['NOT IN', 'cohort_user_course_id', $post_data['LmsCohortUserCourse']['cohort_user_course_id']])
                    ->all();

                foreach ($attendances as $attendance) {
                    $attendance->delete();
                }
            }

            if (!$breakFlag) {
                $transaction->commit();

                Yii::$app->session->setFlash('success', "Attendance saved successfully.");

                return $modelLmsAttendance->id;
            } else {
                $transaction->rollBack();

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}