<?php

namespace common\modules\lms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%lms_cohort}}".
 *
 * @property int $id
 * @property string $name
 * @property int $course_id
 * @property int $teacher_id
 * @property int $tenant_id
 * @property string $stage
 * @property string $start_date
 * @property string $end_date
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property LmsCourse $course
 * @property User $teacher
 * @property User $tenant
 * @property LmsCohortUserCourse[] $lmsCohortUserCourses
 * @property LmsCohortUserCourseActivity[] $lmsCohortUserCourseActivities
 * @property CoreValue $courseValue
 */
class LmsCohort extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%lms_cohort}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'course_id', 'teacher_id'], 'required'],
            [['course_id', 'teacher_id', 'tenant_id', 'created_by', 'updated_by'], 'integer'],
            [['stage', 'metadata', 'status'], 'string'],
            [['start_date', 'end_date'], 'safe'],
            [['name'], 'string', 'max' => 255],
            [['name'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['course_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsCourse::className(), 'targetAttribute' => ['course_id' => 'id']],
            [['teacher_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['teacher_id' => 'id']],
            [['tenant_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['tenant_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'course_id' => 'Course ID',
            'teacher_id' => 'Teacher ID',
            'tenant_id' => 'Tenant ID',
            'stage' => 'Stage',
            'start_date' => 'Start Date',
            'end_date' => 'End Date',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCourse()
    {
        return $this->hasOne(\common\modules\lms\models\LmsCourse::className(), ['id' => 'course_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTeacher()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'teacher_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTenant()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'tenant_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsCohortUserCourses()
    {
        return $this->hasMany(\common\modules\lms\models\LmsCohortUserCourse::className(), ['cohort_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsCohortUserCourseActivities()
    {
        return $this->hasMany(\common\modules\lms\models\LmsCohortUserCourseActivity::className(), ['cohort_id' => 'id']);
    }

    public function getCourseValue()
    {
        $modelCoreEntityClass = \common\modules\core\models\CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);
        $modelCoreAttributeClass = \common\modules\core\models\CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(\common\modules\core\models\CoreValue::className(), ['entity_id' => 'course_id'])->andOnCondition(['cv.entity_class_id' => $modelCoreEntityClass->id, 'cv.attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                if (\common\modules\lms\models\LmsCohortUserCourse::find()->where(['cohort_id' => $this->id, 'status' => 'active'])->exists()) {
                    $modelLmsCohortUserCourse = \common\modules\lms\models\LmsCohortUserCourse::find()->where(['cohort_id' => $this->id, 'status' => 'active'])->all();

                    foreach ($modelLmsCohortUserCourse as $key => $value) {
                        $value->softDelete();
                    }
                }

                $transaction->commit();
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveCohort($post_data, $cohort_id = 0)
    {
        $session = Yii::$app->session;

        $modelLmsCohort = new LmsCohort();
        $modelLmsCohortUserCourse = new \common\modules\lms\models\LmsCohortUserCourse();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();
    
        try {
            if (isset($cohort_id) && $cohort_id > 0 && LmsCohort::find()->where(['id' => $cohort_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelLmsCohort = new LmsCohort();

                $modelLmsCohort = LmsCohort::find()->where(['id' => $cohort_id, 'status' => 'active'])->one();

                $modelLmsCohort->load($post_data);

                $modelLmsCohort->metadata = isset($post_data['LmsCohort']['LmsCohortMetadata']) ? Globals::setJson($post_data['LmsCohort']['LmsCohortMetadata']) : '';
            } else {    // Create a new model
                $modelLmsCohort = new LmsCohort();

                $modelLmsCohort->load($post_data);

                $modelLmsCohort->metadata = isset($post_data['LmsCohort']['LmsCohortMetadata']) ? Globals::setJson($post_data['LmsCohort']['LmsCohortMetadata']) : '';
                $modelLmsCohort->status = 'active';
            }

            if ($modelLmsCohort->validate()) {
                $modelLmsCohort->save();

                if (is_array($post_data['LmsCohortUserCourse']['user_course_id']) && !empty($post_data['LmsCohortUserCourse']['user_course_id'])) {
                    foreach ($post_data['LmsCohortUserCourse']['user_course_id'] as $cohort_user_course_k => $cohort_user_course_v) {
                        $modelLmsCohortUserCourse = new \common\modules\lms\models\LmsCohortUserCourse();

                        if (\common\modules\lms\models\LmsCohortUserCourse::find()
                            ->where([
                                'cohort_id' => $modelLmsCohort->id, 
                                'user_course_id' => $cohort_user_course_v, 
                                'status' => 'active'
                            ])
                            ->exists()
                        ) { // Model already exists
                            $modelLmsCohortUserCourse = \common\modules\lms\models\LmsCohortUserCourse::find()
                            ->where([
                                'cohort_id' => $modelLmsCohort->id, 
                                'user_course_id' => $cohort_user_course_v, 
                                'status' => 'active'
                            ])
                            ->one();
                        } else {    // Create a new model
                            $modelLmsCohortUserCourse->cohort_id = $modelLmsCohort->id;
                            $modelLmsCohortUserCourse->user_course_id = $cohort_user_course_v;
                            $modelLmsCohortUserCourse->status = 'active';
                        }

                        if ($modelLmsCohortUserCourse->validate()) {
                            $modelLmsCohortUserCourse->save();
                        } else {
                            $breakFlag = true;

                            Yii::$app->session->setFlash('error', "Unable to save cohort enrollment " . Globals::renderErrors($modelLmsCohortUserCourse->getErrors()));

                            break;
                        }
                    }
                }

                // delete remaining cohort enrollments
                if (!$breakFlag && isset($modelLmsCohort->id) && $modelLmsCohort->id > 0) {
                    $cohort_user_courses = \common\modules\lms\models\LmsCohortUserCourse::find()
                        ->where([
                            'cohort_id' => $modelLmsCohort->id, 
                            'status' => 'active'
                        ])
                        ->andWhere(['NOT IN', 'user_course_id', $post_data['LmsCohortUserCourse']['user_course_id']])
                        ->all();

                    foreach ($cohort_user_courses as $cohort_user_course) {
                        $cohort_user_course->delete();
                    }
                }
            } else {
                $breakFlag = true;

                Yii::$app->session->setFlash('error', "Unable to save cohort: " . Globals::renderErrors($modelLmsCohort->getErrors()));
            }

            if (!$breakFlag) {
                $transaction->commit();

                Yii::$app->session->setFlash('success', "Cohort saved successfully.");

                return $modelLmsCohort->id;
            } else {
                $transaction->rollBack();

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}