<?php

namespace common\modules\lms\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\lms\models\LmsCohort;

/**
 * LmsCohortSearch represents the model behind the search form of `common\modules\lms\models\LmsCohort`.
 */
class LmsCohortSearch extends LmsCohort
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'course_id', 'teacher_id', 'tenant_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'stage', 'start_date', 'end_date', 'metadata', 'created_at', 'updated_at', 'status', 'course', 'teacher', 'tenant'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['course', 'teacher', 'tenant']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = LmsCohort::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['courseValue as cv', 'teacher', 'tenant']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['course'] = ['asc' => ['cv.value' => SORT_ASC], 'desc' => ['cv.value' => SORT_DESC]];
        $dataProvider->sort->attributes['teacher'] = ['asc' => ['mod_user.email' => SORT_ASC], 'desc' => ['mod_user.email' => SORT_DESC]];
        $dataProvider->sort->attributes['tenant'] = ['asc' => ['mod_user.email' => SORT_ASC], 'desc' => ['mod_user.email' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_lms_cohort.name', $this->name])
            ->andFilterWhere(['mod_lms_cohort.stage' => $this->stage])
            ->andFilterWhere(['like', 'mod_lms_cohort.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_lms_cohort.status', $this->status])
            ->andFilterWhere(['like', 'mod_lms_cohort.id', $this->id])
            ->andFilterWhere(['like', 'mod_lms_cohort.course_id', $this->course_id])
            ->andFilterWhere(['like', 'mod_lms_cohort.teacher_id', $this->teacher_id])
            ->andFilterWhere(['like', 'mod_lms_cohort.tenant_id', $this->tenant_id])
            ->andFilterWhere(['like', 'mod_lms_cohort.start_date', $this->start_date])
            ->andFilterWhere(['like', 'mod_lms_cohort.end_date', $this->end_date])
            ->andFilterWhere(['like', 'cv.value', $this->course])
            ->andFilterWhere(['like', 'mod_user.email', $this->teacher])
            ->andFilterWhere(['like', 'mod_user.email', $this->tenant])
            ->andFilterWhere(['like', 'mod_lms_cohort.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_lms_cohort.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_lms_cohort.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_lms_cohort.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
