<?php

namespace common\modules\lms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%lms_cohort_user_course}}".
 *
 * @property int $id
 * @property int $cohort_id
 * @property int $user_course_id
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property LmsAttendance[] $lmsAttendances
 * @property User $createdBy
 * @property User $updatedBy
 * @property LmsCohort $cohort
 * @property LmsUserCourse $userCourse
 * @property User $user
 * @property LmsCourse $course
 * @property User $teacher
 * @property User $tenant
 */
class LmsCohortUserCourse extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%lms_cohort_user_course}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['cohort_id', 'user_course_id'], 'required'],
            [['cohort_id', 'user_course_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['cohort_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsCohort::className(), 'targetAttribute' => ['cohort_id' => 'id']],
            [['user_course_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsUserCourse::className(), 'targetAttribute' => ['user_course_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'cohort_id' => 'Cohort ID',
            'user_course_id' => 'User Course ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsAttendances()
    {
        return $this->hasMany(\common\modules\lms\models\LmsAttendance::className(), ['cohort_user_course_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCohort()
    {
        return $this->hasOne(\common\modules\lms\models\LmsCohort::className(), ['id' => 'cohort_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserCourse()
    {
        return $this->hasOne(\common\modules\lms\models\LmsUserCourse::className(), ['id' => 'user_course_id']);
    }

    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::class, ['id' => 'user_id'])->via('userCourse');
    }

    public function getCourse()
    {
        return $this->hasOne(\common\modules\lms\models\LmsCourse::className(), ['id' => 'course_id'])->via('cohort');
    }

    public function getTeacher()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'teacher_id'])->via('cohort');
    }

    public function getTenant()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'tenant_id'])->via('cohort');
    }
}