<?php

namespace common\modules\lms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%lms_course}}".
 *
 * @property int $id
 * @property string $code
 * @property int $featured_image
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property AssetManager $featuredImage
 * @property LmsCourseActivity[] $lmsCourseActivities
 * @property LmsUserCourse[] $lmsUserCourses
 */
class LmsCourse extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%lms_course}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['code'], 'required'],
            [['featured_image', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['code'], 'string', 'max' => 255],
            [['code'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['featured_image' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'code' => 'Code',
            'featured_image' => 'Featured Image',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'featured_image']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsCourseActivities()
    {
        return $this->hasMany(\common\modules\lms\models\LmsCourseActivity::className(), ['course_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsUserCourses()
    {
        return $this->hasMany(\common\modules\lms\models\LmsUserCourse::className(), ['course_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
    
    public function saveCourse($post_data, $course_id = 0)
    {
        $session = Yii::$app->session;

        $modelLmsCourse = new self();
        $modelCoreValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($course_id) && $course_id > 0 && self::find()->where(['id' => $course_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelLmsCourse = self::findOne(['id' => $course_id, 'status' => 'active']);

                $modelLmsCourse->code = $post_data['LmsCourse']['code'];
                $modelLmsCourse->metadata = isset($post_data['LmsCourseMetadata']) ? Globals::setJson($post_data['LmsCourseMetadata']) : '';
                $modelLmsCourse->featured_image = $post_data['LmsCourse']['featured_image'];
            } else {    // Create a new model
                $modelLmsCourse = new LmsCourse();

                $modelLmsCourse->code = $post_data['LmsCourse']['code'];
                $modelLmsCourse->metadata = isset($post_data['LmsCourseMetadata']) ? Globals::setJson($post_data['LmsCourseMetadata']) : '';
                $modelLmsCourse->featured_image = $post_data['LmsCourse']['featured_image'];
                $modelLmsCourse->status = 'active';
            }

            if ($modelLmsCourse->validate()) {
                $modelLmsCourse->save();

                foreach ($post_data['LmsCourse'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }

                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();
                            $modelCoreValue = CoreValue::setEntityValue('course', $key, $modelLmsCourse->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }                     
                }

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Course saved successfully.");

                    return $modelLmsCourse->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Globals::renderErrors($modelCoreValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save course: " . Globals::renderErrors($modelLmsCourse->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
