<?php

namespace common\modules\lms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%lms_course_activity}}".
 *
 * @property int $id
 * @property int $course_id
 * @property int $activity_id
 * @property int $featured_image
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property LmsCourse $course
 * @property LmsActivity $activity
 * @property AssetManager $featuredImage
 * @property LmsUserCourseActivity[] $lmsUserCourseActivities
 */
class LmsCourseActivity extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%lms_course_activity}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['course_id', 'activity_id'], 'required'],
            [['course_id', 'activity_id', 'featured_image', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['course_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsCourse::className(), 'targetAttribute' => ['course_id' => 'id']],
            [['activity_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsActivity::className(), 'targetAttribute' => ['activity_id' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['featured_image' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'course_id' => 'Course ID',
            'activity_id' => 'Activity ID',
            'featured_image' => 'Featured Image',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCourse()
    {
        return $this->hasOne(\common\modules\lms\models\LmsCourse::className(), ['id' => 'course_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getActivity()
    {
        return $this->hasOne(\common\modules\lms\models\LmsActivity::className(), ['id' => 'activity_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'featured_image']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsUserCourseActivities()
    {
        return $this->hasMany(\common\modules\lms\models\LmsUserCourseActivity::className(), ['course_activity_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course_activity', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id]);
    }
	
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCourseValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'course_id'])->andOnCondition(['cv.entity_class_id' => $modelCoreEntityClass->id, 'cv.attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course_activity', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
    
    public function saveCourseActivity($post_data, $course_id = 0)
    {
        $session = Yii::$app->session;

        $modelLmsCourseActivity = new self();
        $modelCoreValue = new CoreValue();
        $breakFlag = false;

        // Json decode metadata
        foreach ($post_data['LmsCourseActivityMetadata'] as $_k => $_v) {
            if (is_array($_v)) {
                foreach ($_v as $__k => $__v) {
                    if (empty($__v)) {
                        unset($post_data['LmsCourseActivityMetadata'][$_k][$__k]);
                    } else {
                        $post_data['LmsCourseActivityMetadata'][$_k][$__k] = json_decode($__v, true);
                    }
                }
            }
        }

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($course_id) && $course_id > 0 && self::find()->where(['id' => $course_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelLmsCourseActivity = self::findOne(['id' => $course_id, 'status' => 'active']);

                $modelLmsCourseActivity->course_id = $post_data['LmsCourseActivity']['course_id'];
                $modelLmsCourseActivity->activity_id = $post_data['LmsCourseActivity']['activity_id'];
                $modelLmsCourseActivity->metadata = isset($post_data['LmsCourseActivityMetadata']) ? Globals::setJson($post_data['LmsCourseActivityMetadata']) : '';
                $modelLmsCourseActivity->featured_image = $post_data['LmsCourseActivity']['featured_image'];
            } else {    // Create a new model
                $modelLmsCourseActivity = new LmsCourseActivity();

                $modelLmsCourseActivity->course_id = $post_data['LmsCourseActivity']['course_id'];
                $modelLmsCourseActivity->activity_id = $post_data['LmsCourseActivity']['activity_id'];
                $modelLmsCourseActivity->metadata = isset($post_data['LmsCourseActivityMetadata']) ? Globals::setJson($post_data['LmsCourseActivityMetadata']) : '';
                $modelLmsCourseActivity->featured_image = $post_data['LmsCourseActivity']['featured_image'];
                $modelLmsCourseActivity->status = 'active';
            }

            if ($modelLmsCourseActivity->validate()) {
                $modelLmsCourseActivity->save();

                foreach ($post_data['LmsCourseActivity'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }

                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();
                            $modelCoreValue = CoreValue::setEntityValue('course_activity', $key, $modelLmsCourseActivity->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }                     
                }

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Course Activity saved successfully.");

                    return $modelLmsCourseActivity->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Globals::renderErrors($modelCoreValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save Course Activity: " . Globals::renderErrors($modelLmsCourseActivity->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
	}

    public function getCourseActivities($course_activities = array()) 
    {
        $modelCoreEntityClass = new CoreEntityClass();
        $modelCoreAttributeClass = new CoreAttributeClass();
        $modelCoreValue = new CoreValue();

        $attr = array();
        $attr_class_ids = array();
        $attribute_class_arr = array();
        $entity_ids = array();
        $courses = array();

        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course_activity', 'status' => 'active']);
        $attribute_class_arr = CoreEntityAttribute::getAttributeClassesOfEntityClass('course_activity');
        $entity_ids = \yii\helpers\ArrayHelper::getColumn($course_activities, 'id');

        if (!empty($attribute_class_arr)) {
            $attr_class_ids = array_keys($attribute_class_arr);
        }

        $tmp_arr = CoreValue::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $attr_class_ids, 'entity_id' => $entity_ids, 'status' => 'active'])->asArray()->all();

        foreach ($tmp_arr as $key => $value) {
            if (array_key_exists($value['attribute_class_id'], $attribute_class_arr)) {
                $courses[$value['entity_id']][$attribute_class_arr[$value['attribute_class_id']]] = $value['value'];
            }
        }

        foreach ($course_activities as $key => $value) {
            $courses[$value['id']]['id'] = $value['id'];
            $courses[$value['id']]['course_id'] = $value['course_id'];
            $courses[$value['id']]['activity_id'] = $value['activity_id'];
            $courses[$value['id']]['featured_image'] = $value['featured_image'];
            $courses[$value['id']]['metadata'] = $value['metadata'];
            $courses[$value['id']]['created_by'] = $value['created_by'];
            $courses[$value['id']]['updated_by'] = $value['updated_by'];
            $courses[$value['id']]['created_at'] = $value['created_at'];
            $courses[$value['id']]['updated_at'] = $value['updated_at'];

            if (!Yii::$app->user->isGuest) {
                $courses[$value['id']]['user_course_activity_id'] = LmsUserCourseActivity::getAttr(['id'], ['user_id' => Yii::$app->user->identity->id, 'course_activity_id' => $value['id'], 'status' => 'active']);
            } else {
                $courses[$value['id']]['user_course_activity_id'] = false;
            }
        }
        
        return $courses;
    }
}
