<?php

namespace common\modules\lms\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\lms\models\LmsCourseActivity;

/**
 * LmsCourseActivitySearch represents the model behind the search form of `common\modules\lms\models\LmsCourseActivity`.
 */
class LmsCourseActivitySearch extends LmsCourseActivity
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'course_id', 'activity_id', 'featured_image', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'created_at', 'updated_at', 'status', 'name', 'course', 'activity'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['name', 'course', 'activity']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = LmsCourseActivity::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['activity', 'values as v', 'courseValues as cv']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);
        $dataProvider->sort->attributes['name'] = ['asc' => ['v.value' => SORT_ASC], 'desc' => ['v.value' => SORT_DESC]];
        $dataProvider->sort->attributes['course'] = ['asc' => ['cv.value' => SORT_ASC], 'desc' => ['cv.value' => SORT_DESC]];
        $dataProvider->sort->attributes['activity'] = ['asc' => ['mod_lms_activity.name' => SORT_ASC], 'desc' => ['mod_lms_activity.name' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_lms_course_activity.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_lms_course_activity.status', $this->status])
            ->andFilterWhere(['like', 'mod_lms_course_activity.id', $this->id])
            ->andFilterWhere(['like', 'mod_lms_course_activity.course_id', $this->course_id])
            ->andFilterWhere(['like', 'mod_lms_course_activity.activity_id', $this->activity_id])
            ->andFilterWhere(['like', 'mod_lms_course_activity.featured_image', $this->featured_image])
            ->andFilterWhere(['like', 'mod_lms_activity.name', $this->activity])
            ->andFilterWhere(['like', 'v.value', $this->name])
            ->andFilterWhere(['like', 'cv.value', $this->course])
            ->andFilterWhere(['like', 'mod_lms_course_activity.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_lms_course_activity.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_lms_course_activity.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_lms_course_activity.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
