<?php

namespace common\modules\lms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;

/**
 * LmsReportsCourseCompletionSearch represents the model behind the search form about `common\models\Scorm`.
 */
class LmsReportsCourseCompletionSearch extends LmsUserCourseActivityAttempt
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'user_course_activity_id', 'attempt', 'created_by', 'updated_by'], 'integer'],
            [['attempt', 'created_at', 'updated_at', 'status', 'courseName', 'courseActivityName', 'userEmail', 'completionStatus', 'score'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['courseName', 'courseActivityName', 'userEmail', 'completionStatus', 'score']);
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = LmsUserCourseActivityAttempt::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);

        // add conditions that should always apply here

        $query->joinWith(['user', 'values', 'courseActivity', 'course']);
        $query->andWhere(['mod_lms_course_activity.activity_id' => LmsActivity::getAttr(['id'], ['name' => 'scorm-1.2'])]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['courseName'] = ['asc' => ['mod_core_value.value' => SORT_ASC], 'desc' => ['mod_core_value.value' => SORT_DESC]];
        $dataProvider->sort->attributes['courseActivityName'] = ['asc' => ['JSON_EXTRACT(`mod_lms_course_activity`.`metadata`, "$.launch_file")' => SORT_ASC], 'desc' => ['JSON_EXTRACT(`mod_lms_course_activity`.`metadata`, "$.launch_file")' => SORT_DESC]];
        $dataProvider->sort->attributes['userEmail'] = ['asc' => ['mod_user.email' => SORT_ASC], 'desc' => ['mod_user.email' => SORT_DESC]];
        $dataProvider->sort->attributes['attempt'] = ['asc' => ['mod_lms_user_course_activity_attempt.attempt' => SORT_ASC], 'desc' => ['mod_lms_user_course_activity_attempt.attempt' => SORT_DESC]];
        $dataProvider->sort->attributes['userEmail'] = ['asc' => ['mod_user.email' => SORT_ASC], 'desc' => ['mod_user.email' => SORT_DESC]];
        $dataProvider->sort->attributes['completionStatus'] = ['asc' => ['JSON_EXTRACT(`mod_lms_user_course_activity_attempt`.`metadata`, "$.cmi_data.\"cmi.core.lesson_status\"")' => SORT_ASC], 'desc' => ['JSON_EXTRACT(`mod_lms_user_course_activity_attempt`.`metadata`, "$.cmi_data.\"cmi.core.lesson_status\"")' => SORT_DESC]];
        $dataProvider->sort->attributes['score'] = ['asc' => ['JSON_EXTRACT(`mod_lms_user_course_activity_attempt`.`metadata`, "$.cmi_data.\"cmi.core.score.raw\"")' => SORT_ASC], 'desc' => ['JSON_EXTRACT(`mod_lms_user_course_activity_attempt`.`metadata`, "$.cmi_data.\"cmi.core.score.raw\"")' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
        ]);

        $query->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.id', $this->id])
            ->andFilterWhere(['like', 'mod_core_value.value', $this->courseName])
            ->andFilterWhere(['like', new \yii\db\Expression('JSON_EXTRACT(`mod_lms_course_activity`.`metadata`, "$.launch_file")'), $this->courseActivityName])
            ->andFilterWhere(['like', 'mod_user.email', $this->userEmail])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.attempt', $this->attempt])
            ->andFilterWhere(['like', new \yii\db\Expression('JSON_EXTRACT(`mod_lms_user_course_activity_attempt`.`metadata`, "$.cmi_data.\"cmi.core.lesson_status\"")'), $this->completionStatus])
            ->andFilterWhere(['like', new \yii\db\Expression('JSON_EXTRACT(`mod_lms_user_course_activity_attempt`.`metadata`, "$.cmi_data.\"cmi.core.score.raw\"")'), $this->score])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
