<?php

namespace common\modules\lms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;

/**
 * LmsReportsQuizCompletionSearch represents the model behind the search form about `common\models\Scorm`.
 */
class LmsReportsQuizCompletionSearch extends LmsUserCourseActivityAttempt
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'user_course_activity_id', 'attempt', 'created_by', 'updated_by'], 'integer'],
            [['attempt', 'created_at', 'updated_at', 'status', 'courseName', 'courseActivityName', 'userEmail', 'firstName', 'lastName'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['courseName', 'courseActivityName', 'userEmail', 'firstName', 'lastName']);
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = LmsUserCourseActivityAttempt::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);

        // add conditions that should always apply here

        $query->joinWith(['user', 'userProfile', 'courseValues', 'courseActivityValues', 'courseActivity', 'course']);
        $query->andWhere(['mod_lms_course_activity.activity_id' => LmsActivity::getAttr(['id'], ['name' => 'quiz'])]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['courseName'] = ['asc' => ['course_value.value' => SORT_ASC], 'desc' => ['course_value.value' => SORT_DESC]];
        $dataProvider->sort->attributes['courseActivityName'] = ['asc' => ['course_activity_value.value' => SORT_ASC], 'desc' => ['course_activity_value.value' => SORT_DESC]];
        $dataProvider->sort->attributes['userEmail'] = ['asc' => ['mod_user.email' => SORT_ASC], 'desc' => ['mod_user.email' => SORT_DESC]];
        $dataProvider->sort->attributes['firstName'] = ['asc' => ['mod_user_profile.first_name' => SORT_ASC], 'desc' => ['mod_user_profile.first_name' => SORT_DESC]];
        $dataProvider->sort->attributes['lastName'] = ['asc' => ['mod_user_profile.last_name' => SORT_ASC], 'desc' => ['mod_user_profile.last_name' => SORT_DESC]];
        $dataProvider->sort->attributes['attempt'] = ['asc' => ['mod_lms_user_course_activity_attempt.attempt' => SORT_ASC], 'desc' => ['mod_lms_user_course_activity_attempt.attempt' => SORT_DESC]];
        $dataProvider->sort->attributes['userEmail'] = ['asc' => ['mod_user.email' => SORT_ASC], 'desc' => ['mod_user.email' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
        ]);

        $query->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.id', $this->id])
            ->andFilterWhere(['like', 'course_value.value', $this->courseName])
            ->andFilterWhere(['like', 'course_activity_value.value', $this->courseActivityName])
            ->andFilterWhere(['like', 'mod_user.email', $this->userEmail])
            ->andFilterWhere(['like', 'mod_user_profile.first_name', $this->firstName])
            ->andFilterWhere(['like', 'mod_user_profile.last_name', $this->lastName])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.attempt', $this->attempt])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
