<?php

namespace common\modules\lms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%lms_user_course}}".
 *
 * @property int $id
 * @property int $user_id
 * @property int $course_id
 * @property string $expiration_date
 * @property string $valid
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $user
 * @property LmsCourse $course
 */
class LmsUserCourse extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%lms_user_course}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'course_id', 'expiration_date'], 'required'],
            [['user_id', 'course_id', 'created_by', 'updated_by'], 'integer'],
            [['expiration_date'], 'safe'],
            [['valid', 'metadata', 'status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['course_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsCourse::className(), 'targetAttribute' => ['course_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'course_id' => 'Course ID',
            'expiration_date' => 'Expiration Date',
            'valid' => 'Valid',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCourse()
    {
        return $this->hasOne(\common\modules\lms\models\LmsCourse::className(), ['id' => 'course_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['v.entity_class_id' => $modelCoreEntityClass->id, 'v.attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function getUserCourses() 
    {
        $modelCoreEntityClass = new CoreEntityClass();
        $modelCoreAttributeClass = new CoreAttributeClass();
        $modelCoreValue = new CoreValue();
        $modelLmsCourse = new LmsCourse();
        $modelLmsUserCourse = new self();

        $modelLmsCourseArr = array();
        $modelLmsUserCourseArr = array();
        $courses = array();
        $query = array();
        $attr_class_ids = array();

        $query['user_id'] = Yii::$app->user->identity->id;
        $query['valid'] = 'yes';
        $query['status'] = 'active';

        $modelLmsUserCourseArr = self::find()->where($query)->asArray()->all();
        $user_course_ids = array_unique(\yii\helpers\ArrayHelper::getColumn($modelLmsUserCourseArr, 'course_id'));

        $modelLmsCourseArr = LmsCourse::find()->where(['id' => $user_course_ids, 'status' => 'active'])->asArray()->all();
        $course_ids = \yii\helpers\ArrayHelper::getColumn($modelLmsCourseArr, 'id');

        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);
        $attributeClassArr = CoreEntityAttribute::getAttributeClassesOfEntityClass('course');

        if (!empty($attributeClassArr)) {
            $attr_class_ids = array_keys($attributeClassArr);
        }

        $modelCoreValueArr = CoreValue::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $attr_class_ids, 'entity_id' => $course_ids,'status' => 'active'])->asArray()->all();

        foreach ($modelCoreValueArr as $key => $value) {
            if (array_key_exists($value['attribute_class_id'], $attributeClassArr)) {
                $courses[$value['entity_id']][$attributeClassArr[$value['attribute_class_id']]] = $value['value'];
            }
        }

        foreach ($modelLmsCourseArr as $key => $value) {
            $courses[$value['id']]['id'] = $value['id'];
            $courses[$value['id']]['code'] = $value['code'];
            $courses[$value['id']]['featured_image'] = $value['featured_image'];
            $courses[$value['id']]['metadata'] = $value['metadata'];
            $courses[$value['id']]['created_by'] = $value['created_by'];
            $courses[$value['id']]['updated_by'] = $value['updated_by'];
            $courses[$value['id']]['created_at'] = $value['created_at'];
            $courses[$value['id']]['updated_at'] = $value['updated_at'];
        }

        return array_values($courses);
    }

    public function saveUserCourse($post_data, $user_course_id = 0)
    {
        $session = Yii::$app->session;

        if ($post_data instanceof \yii\base\Event) {
            $post_data = $post_data->data->postData;
        }

        $modelLmsUserCourse = new self();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($user_course_id) && $user_course_id > 0 && self::find()->where(['id' => $user_course_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelLmsUserCourse = self::findOne(['id' => $user_course_id, 'status' => 'active']);

                $modelLmsUserCourse->user_id = $post_data['LmsUserCourse']['user_id'];
                $modelLmsUserCourse->course_id = $post_data['LmsUserCourse']['course_id'];
                $modelLmsUserCourse->expiration_date = $post_data['LmsUserCourse']['expiration_date'];
                $modelLmsUserCourse->valid = $post_data['LmsUserCourse']['valid'];
                $modelLmsUserCourse->metadata = isset($post_data['LmsUserCourse']['LmsUserCourseMetadata']) ? Globals::setJson($post_data['LmsUserCourse']['LmsUserCourseMetadata']) : '';
            } else {    // Create a new model
                $modelLmsUserCourse = new LmsUserCourse();

                $modelLmsUserCourse->user_id = $post_data['LmsUserCourse']['user_id'];
                $modelLmsUserCourse->course_id = $post_data['LmsUserCourse']['course_id'];
                $modelLmsUserCourse->expiration_date = $post_data['LmsUserCourse']['expiration_date'];
                $modelLmsUserCourse->valid = $post_data['LmsUserCourse']['valid'];
                $modelLmsUserCourse->metadata = isset($post_data['LmsUserCourse']['LmsUserCourseMetadata']) ? Globals::setJson($post_data['LmsUserCourse']['LmsUserCourseMetadata']) : '';
                $modelLmsUserCourse->status = 'active';
            }

            if ($modelLmsUserCourse->validate()) {
                $modelLmsUserCourse->save();
                
                $transaction->commit();

                Yii::$app->session->setFlash('success', "User-course saved successfully.");

                return $modelLmsUserCourse->id;
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save user-course: " . Globals::renderErrors($modelLmsUserCourse->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
