<?php

namespace common\modules\lms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%lms_user_course_activity}}".
 *
 * @property int $id
 * @property int $user_id
 * @property int $course_activity_id
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $user
 * @property LmsCourseActivity $courseActivity
 * @property LmsUserCourseActivityAttempt[] $lmsUserCourseActivityAttempts
 */
class LmsUserCourseActivity extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%lms_user_course_activity}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'course_activity_id'], 'required'],
            [['user_id', 'course_activity_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['course_activity_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsCourseActivity::className(), 'targetAttribute' => ['course_activity_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'course_activity_id' => 'Course Activity ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCourseActivity()
    {
        return $this->hasOne(\common\modules\lms\models\LmsCourseActivity::className(), ['id' => 'course_activity_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLmsUserCourseActivityAttempts()
    {
        return $this->hasMany(\common\modules\lms\models\LmsUserCourseActivityAttempt::className(), ['user_course_activity_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserCourseActivityAttempts()
    {
        return $this->hasMany(LmsUserCourseActivityAttempt::className(), ['user_course_activity_id' => 'id']);
    }

    public function saveUserCourseActivity($post_data)
    {
        $session = Yii::$app->session;

        $modelLmsUserCourseActivity = new LmsUserCourseActivity();
        $modelLmsUserCourseActivityAttempt = new LmsUserCourseActivityAttempt();

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (LmsUserCourseActivity::find()->where(['user_id' => $post_data['LmsUserCourseActivity']['user_id'], 'course_activity_id' => $post_data['LmsUserCourseActivity']['course_activity_id'], 'status' => 'active'])->exists()) {    // Model already exists
                $modelLmsUserCourseActivity = LmsUserCourseActivity::find()->where(['user_id' => $post_data['LmsUserCourseActivity']['user_id'], 'course_activity_id' => $post_data['LmsUserCourseActivity']['course_activity_id'], 'status' => 'active'])->one();

                $modelLmsUserCourseActivity->load($post_data);
            } else {    // Create a new model
                $modelLmsUserCourseActivity = new LmsUserCourseActivity();

                $modelLmsUserCourseActivity->load($post_data);
            }
            
            if ($modelLmsUserCourseActivity->validate()) {
                $modelLmsUserCourseActivity->save();

                $modelLmsUserCourseActivityAttempt = new LmsUserCourseActivityAttempt();

                $modelLmsUserCourseActivityAttempt->load($post_data);
                $modelLmsUserCourseActivityAttempt->user_course_activity_id = $modelLmsUserCourseActivity->id;
                $modelLmsUserCourseActivityAttempt->attempt = LmsUserCourseActivityAttempt::find()->where(['user_id' => $post_data['LmsUserCourseActivityAttempt']['user_id'], 'user_course_activity_id' => $modelLmsUserCourseActivity->id, 'status' => 'active'])->count() + 1;

                if ($modelLmsUserCourseActivityAttempt->validate()) {
                    $modelLmsUserCourseActivityAttempt->save();

                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "User course activity attempt saved successfully.");

                    return $modelLmsUserCourseActivity->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save user course activity attempt " . Yii::$app->Globals->renderErrors($modelLmsUserCourseActivityAttempt->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();
                
                Yii::$app->session->setFlash('error', "Unable to save user course activity: " . Yii::$app->Globals->renderErrors($modelLmsUserCourseActivity->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
