<?php

namespace common\modules\lms\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%lms_user_course_activity_attempt}}".
 *
 * @property int $id
 * @property int $user_id
 * @property int $user_course_activity_id
 * @property int $attempt
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $user
 * @property LmsUserCourseActivity $userCourseActivity
 * @property CourseValue $courseValues
 * @property CourseActivityValue $courseActivityValues
 * @property CourseActivity $courseActivity
 * @property Course $course
 */
class LmsUserCourseActivityAttempt extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%lms_user_course_activity_attempt}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'user_course_activity_id'], 'required'],
            [['user_id', 'user_course_activity_id', 'attempt', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['user_course_activity_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\lms\models\LmsUserCourseActivity::className(), 'targetAttribute' => ['user_course_activity_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'user_course_activity_id' => 'User Course Activity ID',
            'attempt' => 'Attempt',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserProfile()
    {
        return $this->hasOne(\common\modules\user\models\UserProfile::className(), ['user_id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserCourseActivity()
    {
        return $this->hasOne(\common\modules\lms\models\LmsUserCourseActivity::className(), ['id' => 'user_course_activity_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCourseValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'course_id'])->andOnCondition(['course_value.entity_class_id' => $modelCoreEntityClass->id, 'course_value.attribute_class_id' => $modelCoreAttributeClass->id])->via('courseActivity')->from(['course_value' => CoreValue::tableName()]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCourseActivityValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'course_activity', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'course_activity_id'])->andOnCondition(['course_activity_value.entity_class_id' => $modelCoreEntityClass->id, 'course_activity_value.attribute_class_id' => $modelCoreAttributeClass->id])->via('userCourseActivity')->from(['course_activity_value' => CoreValue::tableName()]);
    }

    public function getCourseActivity()
    {
        return $this->hasOne(LmsCourseActivity::className(), ['id' => 'course_activity_id'])->via('userCourseActivity');
    }

    public function getCourse()
    {
        return $this->hasOne(LmsCourse::className(), ['id' => 'course_id'])->via('courseActivity');
    }

    public function getUserData()
    {
        if (!isset(Yii::$app->user->identity->id) || empty(Yii::$app->user->identity->id)) {
            if (!Yii::$app->user->isGuest) {
                Yii::$app->user->identity->id = Yii::$app->user->identity->id;
            } else {
                throw new \yii\web\BadRequestHttpException("No user provided.", 400);
            }
        }

        $modelLmsUserCourseActivityAttempt = self::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->orderBy(['created_at' => SORT_DESC])->limit(10)->asArray()->all();

        return $modelLmsUserCourseActivityAttempt;
    }
}
