<?php

namespace common\modules\lms\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\lms\models\LmsUserCourseActivityAttempt;

/**
 * LmsUserCourseActivityAttemptSearch represents the model behind the search form of `common\modules\lms\models\LmsUserCourseActivityAttempt`.
 */
class LmsUserCourseActivityAttemptSearch extends LmsUserCourseActivityAttempt
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'user_course_activity_id', 'attempt', 'created_by', 'updated_by'], 'integer'],
            [['metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = LmsUserCourseActivityAttempt::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.status', $this->status])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.id', $this->id])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.user_course_activity_id', $this->user_course_activity_id])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.attempt', $this->attempt])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_lms_user_course_activity_attempt.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
