<?php

namespace common\modules\lms\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\lms\models\LmsUserCourse;

/**
 * LmsUserCourseSearch represents the model behind the search form of `common\modules\lms\models\LmsUserCourse`.
 */
class LmsUserCourseSearch extends LmsUserCourse
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'course_id', 'created_by', 'updated_by'], 'integer'],
            [['expiration_date', 'valid', 'metadata', 'created_at', 'updated_at', 'status', 'name', 'email'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['name', 'email']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = LmsUserCourse::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['user', 'values as v']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['name'] = ['asc' => ['v.value' => SORT_ASC], 'desc' => ['v.value' => SORT_DESC]];
        $dataProvider->sort->attributes['email'] = ['asc' => ['mod_user.email' => SORT_ASC], 'desc' => ['mod_user.email' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_lms_user_course.valid', $this->valid])
            ->andFilterWhere(['like', 'mod_lms_user_course.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_lms_user_course.status', $this->status])
            ->andFilterWhere(['like', 'mod_lms_user_course.id', $this->id])
            ->andFilterWhere(['like', 'mod_lms_user_course.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_lms_user_course.course_id', $this->course_id])
            ->andFilterWhere(['like', 'mod_lms_user_course.expiration_date', $this->expiration_date])
            ->andFilterWhere(['like', 'mod_user.email', $this->email])
            ->andFilterWhere(['like', 'v.value', $this->name])
            ->andFilterWhere(['like', 'mod_lms_user_course.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_lms_user_course.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_lms_user_course.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_lms_user_course.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
