<?php

use yii\bootstrap4\Html;
use yii\grid\GridView;
use yii\widgets\Pjax;
/* @var $this yii\web\View */
/* @var $searchModel common\modules\lms\models\LmsCohortSearch */
/* @var $dataProvider yii\data\ActiveDataProvider */

$this->title = 'Cohorts';
$this->params['breadcrumbs'][] = $this->title;
?>

<div class="container-fluid lms-cohort-index">
    <div class="row">
        <div class="col-md-12">
            <div class="card">
                <div class="card-body">
                    <div class="row mb-2">
                        <div class="col-md-12">
                            <?= Html::a('Create Cohort', ['create'], ['class' => 'btn btn-success']) ?>
                        </div>
                    </div>

                    <?php Pjax::begin(); ?>
                    <?php // echo $this->render('_search', ['model' => $searchModel]); ?>

                    <?= GridView::widget([
                        'dataProvider' => $dataProvider,
                        'options' => ['class' => 'table-responsive'],
                        'filterModel' => $searchModel,
                        'columns' => [
                            ['class' => 'yii\grid\SerialColumn'],

                            ['attribute' => 'id', 'label' => 'Cohort ID', 'value' => 'id'],
                            'name',
                            [
                                'attribute' => 'course',
                                'label' => 'Course',
                                'value' => function($data) use ($courses) {                    
                                    return isset($courses[$data['course_id']]) ? $courses[$data['course_id']] : null;
                                }
                            ],
                            [
                                'attribute' => 'teacher',
                                'label' => 'Teacher',
                                'value' => function($data) use ($modelTeachers) {
                                    return isset($modelTeachers[$data['teacher_id']]) ? $modelTeachers[$data['teacher_id']] : null;
                                }
                            ],
                            [
                                'attribute' => 'tenant',
                                'label' => 'Tenant',
                                'value' => function($data) use ($modelTenants) {
                                    return isset($modelTenants[$data['tenant_id']]) ? $modelTenants[$data['tenant_id']] : null;
                                }
                            ],
                            [
                                'attribute' => 'stage',
                                'filter' => ['unscheduled' => 'Unscheduled', 'scheduled' => 'Scheduled', 'ongoing' => 'Ongoing', 'completed' => 'Completed', 'cancelled' => 'Cancelled', 'rescheduled' => 'Rescheduled'],
                            ],
                            'start_date',
                            'end_date',
                            // 'metadata:ntext',
                            // 'created_by',
                            // 'updated_by',
                            'created_at',
                            'updated_at',
                            // 'status',

                            [
                                'class' => 'hail812\adminlte3\yii\grid\ActionColumn', 
                                'template' => '{view} {update} {delete} {attendance}', 
                                'buttons' => [
                                    'attendance' => function ($url, $model, $key) {
                                        return Html::a('<span class="fa fa-user-check"></span>', \yii\helpers\Url::toRoute(['/lms/attendance/mark', 'cohort_id' => $model['id']]), ['title' => 'Attendance', 'class' => '', 'data-pjax' => '0', 'target' => '_blank']);
                                    },
                                ],
                                'header' => false,
                            ],
                        ],
                        'summaryOptions' => ['class' => 'summary mb-2'],
                        'pager' => [
                            'class' => 'yii\bootstrap4\LinkPager',
                        ]
                    ]); ?>

                    <?php Pjax::end(); ?>
                </div>
                <!--.card-body-->
            </div>
            <!--.card-->
        </div>
        <!--.col-md-12-->
    </div>
    <!--.row-->
</div>