<?php

use yii\bootstrap4\Html;
use yii\bootstrap4\ActiveForm;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\assetManager\widgets\FileInput;
use common\modules\assetManager\models\AssetManager;

/* @var $this yii\web\View */
/* @var $model common\modules\lms\models\LmsCourse */
/* @var $form yii\bootstrap4\ActiveForm */

$data_tmp = $data;

extract($data_tmp);
?>

<div class="lms-course-form">

    <?php $form = ActiveForm::begin(); ?>

    <?= $form->field($model, 'code')->textInput(['maxlength' => true]) ?>

    <?= $this->render('@common/views/_form-eav', [
        'data' => $data,
        'entity_name' => 'course',
        'modelCoreLanguage' => $modelCoreLanguage,
    ]) ?>

    <?php //= $form->field($model, 'metadata')->textarea(['rows' => 6]) ?>

    <?php 
    $tmpPreviewImage = '';

    if (isset($model->featured_image) && !empty($model->featured_image)) {
        $tmpPreviewImage = '<div class="preview-img"><img src="' . AssetManager::getAssetUrl($model->featured_image) . '" alt="' . AssetManager::getAttr(['alt'], ['id' => $model->featured_image, 'status' => 'active']) . '" class="well"></div><div class="input-group">{input}<span class="input-group-btn">{button}</span></div>';
    } else {
        $tmpPreviewImage = '<div class="preview-img"></div><div class="input-group">{input}<span class="input-group-btn">{button}</span></div>';
    }
    ?>

    <?= $form->field($model, 'featured_image')->widget(FileInput::className(), [
        'buttonTag' => 'button',
        'buttonName' => 'Browse',
        'buttonOptions' => ['class' => 'btn btn-primary'],
        'options' => ['class' => 'form-control disabled', 'readonly' => 'readonly'],
        // Widget template
        'template' => $tmpPreviewImage,
        // Optional, if set, only this image can be selected by user
        'thumb' => 'original',
        // Optional, if set, in container will be inserted selected image
        'imageContainer' => '.preview-img',
        // Default to FileInput::DATA_URL. This data will be inserted in input field
        'pasteData' => FileInput::DATA_ID,
        'frameSrc' => ['/asset-manager/asset/asset-manager'],
        // JavaScript function, which will be called before insert file data to input.
        // Argument data contains file data.
        // data example: [alt: "Ведьма с кошкой", description: "123", url: "/uploads/2014/12/vedma-100x100.jpeg", id: "45"]
        'callbackBeforeInsert' => 'function(e, data) {
            console.log(data);
        }',
    ]); ?>

    <?php 
    if (isset(Yii::$app->params['course_metadata']) && !empty(Yii::$app->params['course_metadata'])) {
        $tmpData = '';
        $courseMetadata = array();
        
        if (!empty($model->metadata)) {
            $courseMetadata = Globals::getArray($model->metadata);           
        }
    ?>
        <?php
        foreach (Yii::$app->params['course_metadata'] as $field => $detailsArr) {
            $is_true = false;
            
            if (array_key_exists('type', $detailsArr)) {
                if ($detailsArr['mode'] == 'required') {
                    $is_true = true;
                }
        ?>
                <div class="form-group">
                    <label for="activity-name_value_<?= $field ?>"><?= \yii\helpers\Inflector::titleize(ucfirst($field), true)  ?></label>

                    <br>

                    <?php 
                    $tmpData = isset($courseMetadata[$field]) ? $courseMetadata[$field] : '';

                    switch ($detailsArr['type']) {
                        case 'input_text':
                            echo Html::input('text', 'LmsCourseMetadata[' . $field . ']', isset($tmpData) ? $tmpData : '', ['class' => 'form-control', 'required' => $is_true]);
                            
                            break;

                        case 'wysiwyg':
                            echo \dosamigos\tinymce\TinyMce::widget([
                                'name' => 'LmsCourseMetadata[' . $field . ']',
                                'value' => (isset($tmpData) && !empty($tmpData)) ? $tmpData : '',
                                'options' => ['rows' => 3],
                                'language' => 'en_GB',
                                'clientOptions' => [
                                    'plugins' => [
                                        "advlist autolink lists link charmap print preview anchor hr",
                                        "searchreplace visualblocks code fullscreen",
                                        "insertdatetime media table contextmenu paste"
                                    ],
                                    'toolbar' => "undo redo | styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image"
                                ]
                            ]);
                            
                            break;

                        case 'json':
                            for ($i = 0; $i < 33; $i++) {
                                echo Html::label('Question #' . intval($i + 1), null, ['style' => 'font-size: 0.7em;']);

                                echo Html::textarea('LmsCourseMetadata[' . $field . '][' . $i . ']', isset($tmpData[$i]) ? json_encode($tmpData[$i], JSON_PRETTY_PRINT) : '', ['class' => 'form-control', 'rows' => 6]);

                                echo '</div>';

                                if ($i < 33 - 1) {
                                    echo '<div class="form-group">';
                                }
                            }

                            break;
                        
                        default:
                            echo Html::input('text', 'LmsCourseMetadata[' . $field . ']', isset($tmpData) ? $tmpData : '', ['class' => 'form-control']);
                            
                            break;
                    }
                    ?>
                </div>
            <?php
            }
        }
    }
    ?>

    <div class="form-group">
        <?= Html::submitButton('Save', ['class' => 'btn btn-success']) ?>
    </div>

    <?php ActiveForm::end(); ?>

</div>
