<?php

use yii\bootstrap4\Html;
use yii\grid\GridView;
use yii\widgets\Pjax;
use common\modules\core\models\CoreValue;
use common\modules\lms\models\LmsCourse;
use common\modules\lms\models\LmsCourseActivity;
use dosamigos\chartjs\ChartJs;

$this->title = 'Cumulative Content Feedback Quiz Report';
$this->params['breadcrumbs'][] = $this->title;

?>

<div class="cumulative-quiz-report">

	<h1><?= $this->title ?></h1>

	<div class="row">
		<div class="col-xs-12">
			<div class="chart-wrap">
				<?= ChartJs::widget([
				    'type' => 'horizontalBar',
				    'data' => [
				        'labels' => $average_chart_report_1['label'],
				        'datasets' => [
				            [
				            	'label' => 'Before Training',
				                'data' => $average_chart_report_1['data'],
				                'backgroundColor' => '#ccc',
				            ],
				            [
				            	'label' => 'After Training',
				                'data' => $average_chart_report_2['data'],
				                'backgroundColor' => '#999',
				            ]

				        ]
				    ],
				    'clientOptions' => [
				    	'barValueSpacing' => 20,
				        'legend' => [
				            'display' => true,
				            'position' => 'bottom',
				            'labels' => [
				                'fontSize' => 14,
				                'fontColor' => "#425062",
				            ]
				        ],
				        'tooltips' => [
				            'enabled' => true,
				            'intersect' => true
				        ],
				        'hover' => [
				            'mode' => false
				        ],
				        'scales' => [
					        'yAxes' => [
					            'ticks' => [
					                'min' => 0,
					            ]
					        ]
					    ],
				        // 'maintainAspectRatio' => false,
				    ],
				]) ?>
			</div>
		</div>
	</div>
</div>

<?php 
$script = <<< JS
$('#downloadPdf').click(function(event) {
	// get size of report page
	var reportPageHeight = $('.cumulative-quiz-report').innerHeight();
	var reportPageWidth = $('.cumulative-quiz-report').innerWidth();

	// create a new canvas object that we will populate with all other canvas objects
	var pdfCanvas = $('<canvas />').attr({
		id: "canvaspdf",
		width: reportPageWidth,
		height: reportPageHeight
	});

	// keep track canvas position
	var pdfctx = $(pdfCanvas)[0].getContext('2d');
	var pdfctxX = 0;
	var pdfctxY = 0;
	var buffer = 100;

	// for each chart.js chart
	$("canvas").each(function(index) {
		// get the chart height/width
		var canvasHeight = $(this).innerHeight();
		var canvasWidth = $(this).innerWidth();

		// draw the chart into the new canvas
		pdfctx.drawImage($(this)[0], pdfctxX, pdfctxY, canvasWidth, canvasHeight);
		pdfctxX += canvasWidth + buffer;

		// our report page is in a grid pattern so replicate that in the new canvas
		if (index % 2 === 1) {
			pdfctxX = 0;
			pdfctxY += canvasHeight + buffer;
		}
	});

	// create new pdf and add our new canvas as an image
	var pdf = new jsPDF('l', 'pt', [reportPageWidth, reportPageHeight]);
	pdf.addImage($(pdfCanvas)[0], 'PNG', 0, 0);

	// download the pdf
	pdf.save('filename.pdf');
});
JS;

$this->registerJs($script);
?>
