<?php

use yii\bootstrap4\Html;
use yii\helpers\Url;
use yii\grid\GridView;
use yii\widgets\Pjax;
use common\modules\core\models\CoreValue;
use common\modules\lms\models\LmsCourse;
use common\modules\lms\models\LmsCourseActivity;
use kartik\export\ExportMenu;
use Yadakhov\Json;

$this->title = 'Quiz Completion';
$this->params['breadcrumbs'][] = $this->title;
?>
<div class="course-completion-index">

    <h1><?= Html::encode($this->title) ?></h1>

    <?php
    $gridColumns = [
        ['class' => 'yii\grid\SerialColumn'],

        'id',
        [
            'attribute' => 'courseName', 
            'label' => 'Course', 
            'value' => function($model) {
                if (!empty(LmsCourseActivity::getAttr(['course_id'], ['id' => $model->userCourseActivity->course_activity_id]))) {
                    return CoreValue::getEntityValue('course', 'name', LmsCourseActivity::getAttr(['course_id'], ['id' => $model->userCourseActivity->course_activity_id]));
                } else {
                    return null;
                }
            }
        ],
        [
            'attribute' => 'courseActivityName', 
            'label' => 'Course Activity', 
            'value' => function($model) {
                $json = new Json(LmsCourseActivity::getAttr(['metadata'], ['id' => $model->userCourseActivity->course_activity_id]));

                if (strpos($json->get('launch_url'), '/index.php?r=lms/course-activity/quiz&course_activity_id=') !== false) {
                    return CoreValue::getEntityValue('course_activity', 'name', str_replace('/index.php?r=lms/course-activity/quiz&course_activity_id=', '', $json->get('launch_url')));
                } else {
                    return null;
                }
            }
        ],
        ['attribute' => 'userEmail', 'label' => 'Email', 'value' => 'user.email'],
        [
            'attribute' => 'firstName', 
            'label' => 'First Name', 
            'value' => function($model) {
                return $model->userProfile->first_name;
            }
        ],
        [
            'attribute' => 'lastName', 
            'label' => 'Last Name', 
            'value' => function($model) {
                return $model->userProfile->last_name;
            }
        ],
        'attempt',
        [
            'attribute' => 'completionStatus',
            'label' => 'Status',
            'value' => function($model) {
                $tmp_metadata = json_decode($model->metadata, true);

                return count($tmp_metadata);
            },
        ],
        [
            'attribute' => 'score',
            'label' => 'Score',
            'value' => function($model) {
                $tmp_metadata = json_decode($model->metadata, true);
                $score = 0;

                foreach ($tmp_metadata as $key => $value) {
                    $score += (isset($value['result']) && $value['result'] == 'correct') ? 1 : 0;
                }

                return $score;
            },
        ],
        ['attribute' => 'updated_at', 'label' => 'Last Accessed'],
        [
            'class' => 'yii\grid\ActionColumn', 
            'template' => '{view} {export}', 
            'buttons' => [
                'view' => function ($url, $model, $key) {
                    return Html::a('<span class="fa fa-eye"></span>', Url::toRoute(['lms-report/quiz-completion-report', 'id' => $model['id'], 'user_id' => $model['user_id'], 'user_course_activity_id' => $model['user_course_activity_id']]), ['title' => 'View', 'class' => '', 'data-pjax' => '0', 'target' => '_blank']);
                },
                'export' => function ($url, $model, $key) {
                    return Html::a('<span class="fa fa-print"></span>', Url::toRoute(['lms-report/quiz-completion-report', 'id' => $model['id'], 'user_id' => $model['user_id'], 'user_course_activity_id' => $model['user_course_activity_id'], 'export'=> true]), ['title' => 'Export', 'class' => '', 'data-pjax' => '0', 'target' => '_blank']);
                },
            ],
            'header' => false,
        ],
    ];
    ?>

    <?= ExportMenu::widget([
        'dataProvider' => $dataProvider,
        'columns' => $gridColumns
    ]);
    ?>

<?php Pjax::begin(); ?>    <?= GridView::widget([
        'dataProvider' => $dataProvider,
        'options' => ['class' => 'table-responsive'],
        'filterModel' => $searchModel,
        'columns' => $gridColumns,
    ]); ?>
<?php Pjax::end(); ?></div>
