<?php

use yii\bootstrap4\Html;
use yii\helpers\Inflector;

use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;

/* @var $this yii\web\View */
/* @var $model common\modules\lms\models\LmsUserCourse */

$entity_value = CoreValue::getEntityValue('user_course', 'name', $model->id);

if (isset($entity_value) && empty($entity_value)) {
    unset($entity_value);
}

$this->title = 'View Enrollment: ' . (isset($entity_value) ? $entity_value : $model->id);
$this->params['breadcrumbs'][] = ['label' => 'Enrollments', 'url' => ['index']];
$this->params['breadcrumbs'][] = (isset($entity_value) ? $entity_value : $model->id);
?>

<div class="container-fluid lms-user-course-view">
    <div class="card">
        <div class="card-body">
            <div class="row">
                <div class="col-md-12">
                    <p>
                        <?= Html::a('Update', ['update', 'id' => $model->id], ['class' => 'btn btn-primary']) ?>
                        <?= Html::a('Delete', ['delete', 'id' => $model->id], [
                            'class' => 'btn btn-danger',
                            'data' => [
                                'confirm' => 'Are you sure you want to delete this item?',
                                'method' => 'post',
                            ],
                        ]) ?>
                    </p>

                    <?php
                    $attributes = array();

                    $attributes[] = ['attribute' => 'id', 'label' => 'User Course ID'];
                    $attributes[] = ['label' => 'User', 'attribute' => 'user_id', 'value' => User::getAttr(['email'], ['id' => $model->user_id]) . ' ' . UserProfile::getAttr(['last_name'], ['user_id' => $model->user_id])];
                    $attributes[] = ['label' => 'Course', 'attribute' => 'course_id', 'value' => CoreValue::getEntityValue('course', 'name', $model->course_id)];
                    $attributes[] = 'expiration_date';
                    $attributes[] = 'valid';
                    $attributes[] = 'metadata:ntext';
                    $attributes[] = [
                        'attribute' => 'created_by',
                        'label' => 'Created By',
                        'value' => UserProfile::getAttr(['first_name'], ['user_id' => $model->created_by]) . ' ' . UserProfile::getAttr(['last_name'], ['user_id' => $model->created_by])
                    ];
                    $attributes[] = [
                        'attribute' => 'updated_by',
                        'label' => 'Updated By',
                        'value' => UserProfile::getAttr(['first_name'], ['user_id' => $model->updated_by]) . ' ' . UserProfile::getAttr(['last_name'], ['user_id' => $model->updated_by])
                    ];
                    $attributes[] = 'created_at';
                    $attributes[] = 'updated_at';

                    foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass('user_course') as $attributeClassKey => $attributeClassValue) {
                        foreach (CoreLanguage::find()->where(['status' => 'active'])->all() as $languageKey => $languageValue) {
                            if (Yii::$app->params['attributeFormFields'][$attributeClassValue] == 'wysiwyg') {
                                $attributes[] = [
                                    'attribute' => $attributeClassValue . '_' . $languageValue->code,
                                    'label' => Inflector::titleize(ucfirst($attributeClassValue), true) . ' (' . strtoupper($languageValue->code) . ')',
                                    'value' => CoreValue::getEntityValue('user_course', $attributeClassValue, $model->id, $languageValue->id),
                                    'format' => 'html',
                                ];
                            } else {
                                $attributes[] = [
                                    'attribute' => $attributeClassValue . '_' . $languageValue->code,
                                    'label' => Inflector::titleize(ucfirst($attributeClassValue), true) . ' (' . strtoupper($languageValue->code) . ')',
                                    'value' => CoreValue::getEntityValue('user_course', $attributeClassValue, $model->id, $languageValue->id)
                                ];
                            }
                        }
                    }
                    ?>

                    <?= \yii\widgets\DetailView::widget([
                        'model' => $model,
                        'attributes' => $attributes,
                    ]) ?>
                </div>
                <!--.col-md-12-->
            </div>
            <!--.row-->
        </div>
        <!--.card-body-->
    </div>
    <!--.card-->
</div>
