<?php

namespace common\modules\logs\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

use common\modules\logs\models\LogsVisitor;
use common\modules\logs\models\LogsVisitorSearch;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * VisitorController implements the CRUD actions for LogsVisitor model.
 */
class VisitorController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete', 'chart'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete', 'chart'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all LogsVisitor models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new LogsVisitorSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single LogsVisitor model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new LogsVisitor model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new LogsVisitor();
        $data = array();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        $data['model'] = $model;

        return $this->render('create', [
            'data' => $data,
        ]);
    }

    /**
     * Updates an existing LogsVisitor model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $data = array();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        $data['model'] = $model;

        return $this->render('update', [
            'data' => $data,
        ]);
    }

    /**
     * Deletes an existing LogsVisitor model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the LogsVisitor model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return LogsVisitor the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = LogsVisitor::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public static function logVisitor()
    {
        $session = Yii::$app->session;

        if ($session->has('flag.visitorLog') && $session->get('flag.visitorLog') == 'true') {
            return;
        }

        $modelLogsVisitor = new LogsVisitor();
        $visit_count = 1;

        if (LogsVisitor::find()->where(['ip_address' => Yii::$app->request->getUserIP()])->exists()) {
            $visit_count = LogsVisitor::getAttr(['visits'], ['ip_address' => Yii::$app->request->getUserIP()]);

            $visit_count++;

            $modelLogsVisitor = LogsVisitor::find()->where(['ip_address' => Yii::$app->request->getUserIP()])->one();
        }
        
        $modelLogsVisitor->session_id = Yii::$app->session->getId();
        $modelLogsVisitor->site_id = \common\modules\core\models\CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
        $modelLogsVisitor->visits = $visit_count;
        $modelLogsVisitor->ip_address = Yii::$app->request->getUserIP();
        $modelLogsVisitor->referrer_url = Yii::$app->request->referrer;
        $modelLogsVisitor->user_agent = Yii::$app->request->userAgent;
        $modelLogsVisitor->created_by = isset(Yii::$app->user->identity->id) ? Yii::$app->user->identity->id : 1;
        $modelLogsVisitor->updated_by = isset(Yii::$app->user->identity->id) ? Yii::$app->user->identity->id : 1;

        if ($modelLogsVisitor->validate()) {
            $modelLogsVisitor->save();

            $session->set('flag.visitorLog', 'true');
        }
    }

    public function actionChart()
    {
        return $this->render('chart');
    }
}
