<?php

namespace common\modules\logs\models;

use Yii;
use yii\db\Expression;
use yii\behaviors\TimestampBehavior;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%logs_visitor}}".
 *
 * @property int $id
 * @property int $entity_class_id
 * @property int $entity_id
 * @property string $form_model
 * @property string $session_id
 * @property int $url_id
 * @property int $user_id
 * @property int $site_id
 * @property int $visits
 * @property string $ip_address
 * @property string $referrer_url
 * @property string $user_agent
 * @property int $logs_visitor_id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 *
 * @property AffiliateConversion[] $affiliateConversions
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreEntityClass $entityClass
 * @property SiteUrl $url
 * @property User $user
 * @property CoreSite $site
 * @property LogsVisitor $logsVisitor
 * @property LogsVisitor[] $logsVisitors
 */
class LogsVisitor extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%logs_visitor}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['entity_class_id', 'entity_id', 'url_id', 'user_id', 'site_id', 'visits', 'logs_visitor_id', 'created_by', 'updated_by'], 'integer'],
            [['site_id'], 'required'],
            [['referrer_url', 'metadata'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['form_model', 'session_id', 'ip_address', 'user_agent'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['entity_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreEntityClass::className(), 'targetAttribute' => ['entity_class_id' => 'id']],
            [['url_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\site\models\SiteUrl::className(), 'targetAttribute' => ['url_id' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::className(), 'targetAttribute' => ['site_id' => 'id']],
            [['logs_visitor_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\logs\models\LogsVisitor::className(), 'targetAttribute' => ['logs_visitor_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'entity_class_id' => 'Entity Class ID',
            'entity_id' => 'Entity ID',
            'form_model' => 'Form Model',
            'session_id' => 'Session ID',
            'url_id' => 'Url ID',
            'user_id' => 'User ID',
            'site_id' => 'Site ID',
            'visits' => 'Visits',
            'ip_address' => 'Ip Address',
            'referrer_url' => 'Referrer Url',
            'user_agent' => 'User Agent',
            'logs_visitor_id' => 'Logs Visitor ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAffiliateConversions()
    {
        return $this->hasMany(\common\modules\affiliate\models\AffiliateConversion::className(), ['logs_visitor_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEntityClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreEntityClass::className(), ['id' => 'entity_class_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUrl()
    {
        return $this->hasOne(\common\modules\site\models\SiteUrl::className(), ['id' => 'url_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::className(), ['id' => 'site_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLogsVisitor()
    {
        return $this->hasOne(\common\modules\logs\models\LogsVisitor::className(), ['id' => 'logs_visitor_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getLogsVisitors()
    {
        return $this->hasMany(\common\modules\logs\models\LogsVisitor::className(), ['logs_visitor_id' => 'id']);
    }

    public function behaviors()
    {
        return [
            [
                'class' => TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }

    public static function getAttr($attr = array(), $where = array())
    {
        $model = self::findOne($where);

        $result = array();

        if (isset($model->id) && !empty($model->id)) {
            if (count($attr) < 1) {
                $result = false;
            } else if (count($attr) < 2) {
                foreach ($attr as $key => $value) {
                    $result = isset($model->$value) && !empty($model->$value) ? $model->$value : false;
                }
            } else {
                foreach ($attr as $key => $value) {
                    $result[$value] = isset($model->$value) && !empty($model->$value) ? $model->$value : false;
                }

                if (in_array(false, $result)) {
                    $result = false;
                }
            }
        } else {
            $result = false;
        }

        return $result;
    }
}
