<?php

namespace common\modules\logs\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\logs\models\LogsVisitor;

/**
 * LogsVisitorSearch represents the model behind the search form of `common\modules\logs\models\LogsVisitor`.
 */
class LogsVisitorSearch extends LogsVisitor
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'entity_class_id', 'entity_id', 'url_id', 'user_id', 'site_id', 'visits', 'logs_visitor_id', 'created_by', 'updated_by'], 'integer'],
            [['form_model', 'session_id', 'ip_address', 'referrer_url', 'user_agent', 'metadata', 'created_at', 'updated_at'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = LogsVisitor::find();
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_logs_visitor.form_model', $this->form_model])
            ->andFilterWhere(['like', 'mod_logs_visitor.session_id', $this->session_id])
            ->andFilterWhere(['like', 'mod_logs_visitor.ip_address', $this->ip_address])
            ->andFilterWhere(['like', 'mod_logs_visitor.referrer_url', $this->referrer_url])
            ->andFilterWhere(['like', 'mod_logs_visitor.user_agent', $this->user_agent])
            ->andFilterWhere(['like', 'mod_logs_visitor.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_logs_visitor.id', $this->id])
            ->andFilterWhere(['like', 'mod_logs_visitor.entity_class_id', $this->entity_class_id])
            ->andFilterWhere(['like', 'mod_logs_visitor.entity_id', $this->entity_id])
            ->andFilterWhere(['like', 'mod_logs_visitor.url_id', $this->url_id])
            ->andFilterWhere(['like', 'mod_logs_visitor.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_logs_visitor.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_logs_visitor.visits', $this->visits])
            ->andFilterWhere(['like', 'mod_logs_visitor.logs_visitor_id', $this->logs_visitor_id])
            ->andFilterWhere(['like', 'mod_logs_visitor.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_logs_visitor.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_logs_visitor.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_logs_visitor.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
