<?php

namespace common\modules\mailer\controllers\backend;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;

use common\modules\mailer\notifications\backend\ContactNotification;
use common\modules\core\models\CoreSite;
use common\modules\core\models\CoreEntityClass;
use common\modules\site\models\SiteNotification;
use common\modules\user\models\User;
use common\modules\mailer\models\MailerContact;
use common\modules\mailer\models\MailerContactSearch;

/**
 * NotificationController implements the CRUD actions for MailerContact model.
 */
class NotificationController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['follow-up-date'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['follow-up-date'],
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
        ];
    }

    public function actionFollowUpDate($event)
    {
        $channel_screen = new \common\channels\GenesysScreenChannel('genesys-screen', ['enable' => Yii::$app->getModule('notifications')->channels['genesys-screen']['enable']]);
        $channel_email = new \common\channels\GenesysEmailChannel('genesys-email', ['enable' => Yii::$app->getModule('notifications')->channels['genesys-email']['enable']]);
        $modelMailerContact = new MailerContact;
        $modelSiteNotification = new SiteNotification;
        $class_name = 'ContactNotification';

        $user_id = isset($event->data->userId) ? $event->data->userId : 1;

        if (User::find()->where(['id' => $user_id, 'status' => 'active'])->exists()) {
            $lead_owners = MailerContact::find()
                ->select(["id", "CAST(JSON_EXTRACT(`metadata`, '$.lead_owner') AS UNSIGNED) AS `lead_owner`"])
                ->where(['status' => 'active'])
                ->andWhere(['or', 'created_by=:user_id', 'updated_by=:user_id'])
                ->andWhere(['<', "DATEDIFF(CAST(JSON_EXTRACT(`metadata`, '$.follow_up_date') AS DATE), CURDATE())", 2])
                ->andWhere(['not', ["JSON_EXTRACT(`metadata`, '$.lead_status')" => 'converted']])
                ->addParams([':user_id' => $user_id])
                ->asArray()
                ->all();

            $lead_owners_tmp = array();

            foreach ($lead_owners as $lead_owner) {
                if ($lead_owner['lead_owner'] == $user_id) {
                    $lead_owners_tmp[]['id'] = $lead_owner['id'];
                }
            }

            $lead_owners = $lead_owners_tmp;

            $modelMailerContact = MailerContact::find()
                ->select(['id'])
                ->where(['status' => 'active'])
                ->andWhere(['or', 'created_by=:user_id', 'updated_by=:user_id'])
                ->andWhere(['<', "DATEDIFF(CAST(JSON_EXTRACT(`metadata`, '$.follow_up_date') AS DATE), CURDATE())", 2])
                ->andWhere(['not', ["JSON_EXTRACT(`metadata`, '$.lead_status')" => 'converted']])
                ->addParams([':user_id' => $user_id])
                ->asArray()
                ->all();

            $modelMailerContact = array_unique(array_column(array_merge($lead_owners, $modelMailerContact), 'id'));

            $modelSiteNotification = SiteNotification::find()->select(['entity_id'])->where([
                'class' => strtolower(substr($class_name, strrpos($class_name, '\\'), -12)), 
                'key' => ContactNotification::KEY_FOLLOW_UP_DATE,
                'is_read' => 'no',
                'user_id' => $user_id,
                'entity_class_id' => CoreEntityClass::getAttr(['id'], ['name' => 'mailer_contact']),
                'app_alias' => 'backend',
                'site_id' => CoreSite::getAttr(['id'], ['code' => 'genesys']),
            ])->asArray()->all();

            $modelSiteNotification = array_unique(array_column($modelSiteNotification, 'entity_id'));

            $modelMailerContact = array_diff($modelMailerContact, $modelSiteNotification);

            rsort($modelMailerContact);

            $counter = 1;

            foreach ($modelMailerContact as $key => $value) {
                $counter++;

                $notification_id = $channel_screen->send(ContactNotification::create(
                    ContactNotification::KEY_FOLLOW_UP_DATE, 
                    [
                        'user_id' => $user_id,
                        'entity_class' => 'mailer_contact',
                        'entity_id' => $value,
                        'app_alias' => 'backend',
                        'model' => MailerContact::findOne(['id' => $value, 'status' => 'active']),
                    ]
                ));

                $channel_email->send(ContactNotification::create(
                    ContactNotification::KEY_FOLLOW_UP_DATE, 
                    [
                        'user_id' => $user_id,
                        'entity_class' => 'mailer_contact',
                        'entity_id' => $value,
                        'app_alias' => 'backend',
                        'model' => MailerContact::findOne(['id' => $value, 'status' => 'active']),
                        'notification_id' => $notification_id,
                    ]
                ));

                if ($counter > 999) {
                    break;
                }
            }
        }
    }
}
