<?php

namespace common\modules\mailer\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\mailer\models\MailerCampaign;
use common\modules\mailer\models\MailerCampaignSearch;

/**
 * CampaignController implements the CRUD actions for MailerCampaign model.
 */
class CampaignController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['track'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['track'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'track' => ['get'],
                ],
            ],
        ];
    }

    public function actionTpxl()
    {
        if (Yii::$app->request->isGet) {
            $query_params = Yii::$app->request->get();
            
            unset($query_params['r']);

            if (MailerCampaign::find()->where($query_params)->exists()) {
                $modelMailerCampaign = MailerCampaign::find()->where($query_params)->one();

                $modelMailerCampaign->opens = intval($modelMailerCampaign->opens) + 1;
                $modelMailerCampaign->timestamp_open = (string) time();
                $modelMailerCampaign->ip_address = Yii::$app->request->userIP;
                $modelMailerCampaign->referrer_url = Yii::$app->request->referrer;
                $modelMailerCampaign->user_agent = Yii::$app->request->userAgent;

                if ($modelMailerCampaign->validate()) {
                    $modelMailerCampaign->save();

                    return $modelMailerCampaign->id;
                } else {
                    throw new \yii\web\HttpException(422, 'Unable to track.');
                }
            }
        }
    }

    public function actionTrack()
    {
        $modelMailerCampaign = new MailerCampaign();

        if (!isset($_GET['t']) || empty($_GET['t'])) {
            throw new \yii\web\BadRequestHttpException('Insufficient parameters to track.');
        }

        if (!MailerCampaign::find()->where(['timestamp' => $_GET['t']])->exists()) {
            throw new \yii\web\NotFoundHttpException('Tracking source not found.', 404);
        }

        $modelMailerCampaign = MailerCampaign::find()->where(['timestamp' => $_GET['t']])->one();

        $modelMailerCampaign->timestamp_click = (string) time();
        $modelMailerCampaign->clicks = $modelMailerCampaign->clicks + 1;
        $modelMailerCampaign->ip_address = Yii::$app->request->userIP;
        $modelMailerCampaign->referrer_url = Yii::$app->request->referrer;
        $modelMailerCampaign->user_agent = Yii::$app->request->userAgent;

        if ($modelMailerCampaign->validate()) {
            $modelMailerCampaign->save();
        } else {
            throw new \yii\web\HttpException(422, 'Unable to track.');
        }
    }

    public function actionTclk()
    {
        $request = Yii::$app->request;

        $modelMailerCampaign = new MailerCampaign();

        if ($request->isGet) {
            if ($request->get('ts') && $request->get('rdr')) {
                if (MailerCampaign::find()->where(['timestamp' => $request->get('ts')])->exists()) {
                    $modelMailerCampaign = MailerCampaign::find()->where(['timestamp' => $request->get('ts')])->one();

                    $modelMailerCampaign->timestamp_click = (string) time();
                    $modelMailerCampaign->clicks = $modelMailerCampaign->clicks + 1;
                    $modelMailerCampaign->ip_address = Yii::$app->request->userIP;
                    $modelMailerCampaign->referrer_url = Yii::$app->request->referrer;
                    $modelMailerCampaign->user_agent = Yii::$app->request->userAgent;

                    if ($modelMailerCampaign->validate()) {
                        $modelMailerCampaign->save();
                    }
                }

                $this->redirect($request->get('rdr'));
            }
        }
    }

    public function actionTrack2()
    {
        $modelMailerCampaign = new MailerCampaign();

        $config = Yii::$app->getComponents();

        if (!isset($_GET['col']) || empty($_GET['col'])) {
            throw new \yii\web\BadRequestHttpException('Insufficient parameters to track.');
        }

        if (!isset($_GET['t']) || empty($_GET['t'])) {
            throw new \yii\web\BadRequestHttpException('Insufficient parameters to track.');
        }

        $col_name = 'whois_data_scraper' . $_GET['col'];

        $$col_name = (new \MongoDB\Client($config['mongodb']['dsn']))->whoisrestapi->$col_name;

        if ($$col_name->find(['timestamp' => $_GET['t']])->count() <= 0) {
            throw new \yii\web\NotFoundHttpException('Tracking source not found.', 404);
        }

        $updateResult = $$col_name->updateOne(
            ['timestamp' => $_GET['t']],
            ['$set' => [
                    'timestamp_click' => (string) time(), 
                    'updated_at' => date('Y-m-d H:s:i')
                ]
            ]
        );
    }
}
