<?php

namespace common\modules\mailer\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\mailer\models\MailerContact;
use common\modules\mailer\models\MailerContactSearch;
use common\modules\mailer\models\MailerCampaign;
use common\modules\mailer\models\FormLeadGen;
use common\modules\mailer\models\FormLead;

/**
 * CampaignController implements the CRUD actions for MailerCampaign model.
 */
class ContactController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['unsubscribe'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['unsubscribe', 'lead-gen', 'lead-gen-track', 'lead'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'unsubscribe' => ['get'],
                ],
            ],
        ];
    }

    public function actionUnsubscribe()
    {
        $request = Yii::$app->request;

        $modelMailerContact = new MailerContact();
        $modelMailerCampaign = new MailerCampaign();
        $msg = 'Unable to unsubscribe';

        if ($request->isGet) {
            if ($request->get('ts')) {
                if (!MailerCampaign::find()->where(['timestamp' => $request->get('ts')])->exists()) {
                    throw new \yii\web\NotFoundHttpException('Unsubscription source not found.', 404);
                }

                $modelMailerCampaign = MailerCampaign::find()->where(['timestamp' => $request->get('ts')])->one();
                $modelMailerContact = MailerContact::find()->where(['id' => $modelMailerCampaign->identifier])->one();

                $modelMailerContact->subscribed = 'no';

                if ($modelMailerContact->validate()) {
                    $modelMailerContact->save();

                    $msg = 'Successfully unsubscribed';
                } else {
                    throw new \yii\web\HttpException(422, 'Unable to unsubscribe.');
                }
            }
        }

        return $this->render('unsubscribe', [
            'msg' => $msg, 
        ]);
    }

    public function actionLeadGen()
    {
        $request = Yii::$app->request;

        $modelFormLeadGen = new FormLeadGen();
        $modelMailerContact = new MailerContact();
        $tmp_msg = '';

        if ($request->isPost) {
            $modelFormLeadGen->load($request->post());

            if ($modelFormLeadGen->save()) {
                $tmp_msg = Yii::$app->params['modules']['mailer']['contact']['lead-gen']['message'][0]['success'];

                Yii::$app->session->setFlash('success', $tmp_msg);
            } else {
                $tmp_msg = Yii::$app->params['modules']['mailer']['contact']['lead-gen']['message'][0]['error'];

                Yii::$app->session->setFlash('error', $tmp_msg);
            }

            return $this->redirect(['contact/lead-gen-track', 'msg' => $tmp_msg]);
        } else {
            return $this->render('lead-gen', [
                'modelFormLeadGen' => $modelFormLeadGen,
            ]);
        }
    }

    public function actionLeadGenTrack($msg = '')
    {
        return $this->render('lead-gen-track', [
            'msg' => $msg, 
        ]);
    }

    public function actionLead()
    {
        $request = Yii::$app->request;

        $modelFormLead = new FormLead();
        $modelFormLeadGen = new FormLeadGen();
        $modelMailerContact = new MailerContact();
        $tmp_msg = '';

        if ($request->isGet) {
            $modelFormLead->load($request->get(), '');

            if ($modelFormLead->save()) {
                $tmp_msg = Yii::$app->params['modules']['mailer']['contact']['lead'][$request->get('mailer_campaign')]['success'];

                Yii::$app->session->setFlash('success', $tmp_msg);
            } else {
                $tmp_msg = Yii::$app->params['modules']['mailer']['contact']['lead'][$request->get('mailer_campaign')]['error'];

                Yii::$app->session->setFlash('error', $tmp_msg);
            }

            return $this->redirect(['contact/lead-gen-track', 'msg' => $tmp_msg]);
        } else {
            return $this->render('lead-gen', [
                'modelFormLeadGen' => $modelFormLeadGen,
            ]);
        }
    }
}
