<?php

namespace common\modules\mailer\models;

use Yii;
use yii\base\Model;
use common\modules\site\components\Email;
use common\modules\core\models\CoreSite;
use common\modules\user\models\User;

/**
 * FormLead is the model behind the contact form.
 */
class FormLead extends Model
{
    public $email;
    public $first_name;
    public $last_name;
    public $phone;
    public $organization;
    public $designation;
    public $country;
    public $lead_type;
    public $product;
    public $source;
    public $mailer_campaign;
    public $lead_status;
    public $reCaptcha;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['email', 'first_name', 'last_name', 'phone', 'organization', 'designation', 'country', 'lead_type', 'product', 'source', 'mailer_campaign'], 'trim'],

            ['email', 'required', 'when' => function($model, $attribute) {
                return (isset($model->email) && !empty($model->email));
            }, 'whenClient' => "function (attribute, value) {
                return !$('#formlead-email').length || $('#formlead-email').val().length === 0;
            }"],
            ['first_name', 'required', 'when' => function($model, $attribute) {
                return (isset($model->first_name) && !empty($model->first_name));
            }, 'whenClient' => "function (attribute, value) {
                return !$('#formlead-first_name').length || $('#formlead-first_name').val().length === 0;
            }"],
            ['last_name', 'required', 'when' => function($model, $attribute) {
                return (isset($model->last_name) && !empty($model->last_name));
            }, 'whenClient' => "function (attribute, value) {
                return !$('#formlead-last_name').length || $('#formlead-last_name').val().length === 0;
            }"],
            ['reCaptcha', 'required', 'when' => function($model, $attribute) {
                return (isset($model->reCaptcha) && !empty($model->reCaptcha));
            }, 'whenClient' => "function (attribute, value) {
                return !$('#formlead-recaptcha').length || $('#formlead-recaptcha').val().length === 0;
            }"],

            ['email', 'email'],

            ['first_name', 'string', 'min' => 2, 'max' => 255, 'when' => function($model, $attribute) {
                return (isset($model->first_name) && !empty($model->first_name));
            }, 'whenClient' => "function (attribute, value) {
                return !$('#formlead-first_name').length || $('#formlead-first_name').val().length === 0;
            }"],
            ['last_name', 'string', 'min' => 2, 'max' => 255, 'when' => function($model, $attribute) {
                return (isset($model->last_name) && !empty($model->last_name));
            }, 'whenClient' => "function (attribute, value) {
                return !$('#formlead-last_name').length || $('#formlead-last_name').val().length === 0;
            }"],

            [['phone', 'organization', 'designation'], 'string', 'max' => 255],
            
            [['email', 'first_name', 'last_name', 'phone', 'organization', 'designation', 'country', 'lead_type', 'product', 'source', 'mailer_campaign', 'lead_status'], 'safe'],
            
            [['reCaptcha'], \himiklab\yii2\recaptcha\ReCaptchaValidator::className(), 'when' => function($model, $attribute) {
                return (isset($model->reCaptcha) && !empty($model->reCaptcha));
            }, 'whenClient' => "function (attribute, value) {
                return !$('#formlead-recaptcha').length || $('#formlead-recaptcha').val().length === 0;
            }"],
        ];
    }

    public function save()
    {
        if (!$this->validate()) {
            return false;
        }

        $post_data = array();
        $modelMailerContact = new MailerContact();
        $next_mailer_contact_id = MailerContact::find()->select('id')->orderBy(['id' => SORT_DESC])->asArray()->one()['id'];

        $post_data['MailerContact']['email'] = $this->email;
        $post_data['MailerContact']['first_name'] = $this->first_name;
        $post_data['MailerContact']['last_name'] = $this->last_name;
        $post_data['MailerContact']['phone'] = $this->phone;
        $post_data['MailerContact']['MailerContactMetadata']['organization'] = $this->organization;
        $post_data['MailerContact']['MailerContactMetadata']['designation'] = $this->designation;
        $post_data['MailerContact']['MailerContactMetadata']['country'] = $this->country;
        $post_data['MailerContact']['MailerContactMetadata']['lead_type'] = $this->lead_type;
        $post_data['MailerContact']['MailerContactMetadata']['product'] = $this->product;
        $post_data['MailerContact']['MailerContactMetadata']['source'] = $this->source;
        $post_data['MailerContact']['MailerContactMetadata']['mailer_campaign'] = $this->mailer_campaign;
        $post_data['MailerContact']['MailerContactMetadata']['lead_status'] = $this->lead_status;
        $post_data['MailerContact']['tag'] = null;
        $post_data['MailerContact']['subscribed'] = 'yes';
        $post_data['MailerContact']['site_id'] = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
        $post_data['MailerContact']['created_by'] = 1;
        $post_data['MailerContact']['updated_by'] = 1;

        if (MailerContact::find()
            ->where(['email' => $this->email])
            ->andWhere(["JSON_EXTRACT(`metadata`, '$.product')" => $post_data['MailerContact']['MailerContactMetadata']['product']])
            ->exists()
        ) {
            $modelMailerContact = MailerContact::find()
                ->where(['email' => $this->email])
                ->andWhere(["JSON_EXTRACT(`metadata`, '$.product')" => $post_data['MailerContact']['MailerContactMetadata']['product']])
                ->one();

            $contact_id = $modelMailerContact->saveContact($post_data, $modelMailerContact->id);
        } else {
            $contact_id = $modelMailerContact->saveContact($post_data);
        }

        return $contact_id ? $contact_id : false;
    }
}
