<?php

namespace common\modules\mailer\models;

use Yii;
use yii\db\Expression;
use yii\behaviors\TimestampBehavior;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%mailer_campaign}}".
 *
 * @property int $id
 * @property string $campaign
 * @property string|null $uuid
 * @property string|null $identifier_name
 * @property int|null $identifier
 * @property int $count
 * @property string|null $timestamp
 * @property int|null $opens
 * @property string|null $timestamp_open
 * @property int|null $clicks
 * @property string|null $timestamp_click
 * @property string|null $in_queue
 * @property int $site_id
 * @property string|null $ip_address
 * @property string|null $referrer_url
 * @property string|null $user_agent
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 *
 * @property User $createdBy
 * @property CoreSite $site
 * @property User $updatedBy
 * @property ShopOrder $order
 * @property User $user
 */
class MailerCampaign extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%mailer_campaign}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['campaign', 'site_id'], 'required'],
            [['identifier', 'count', 'opens', 'clicks', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['in_queue', 'referrer_url'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['campaign', 'uuid', 'identifier_name', 'timestamp', 'timestamp_open', 'timestamp_click', 'ip_address', 'user_agent'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::class, 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::class, 'targetAttribute' => ['updated_by' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::class, 'targetAttribute' => ['site_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'campaign' => 'Campaign',
            'uuid' => 'Uuid',
            'identifier_name' => 'Identifier Name',
            'identifier' => 'Identifier',
            'count' => 'Count',
            'timestamp' => 'Timestamp',
            'opens' => 'Opens',
            'timestamp_open' => 'Timestamp Open',
            'clicks' => 'Clicks',
            'timestamp_click' => 'Timestamp Click',
            'in_queue' => 'In Queue',
            'site_id' => 'Site ID',
            'ip_address' => 'Ip Address',
            'referrer_url' => 'Referrer Url',
            'user_agent' => 'User Agent',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::class, ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::class, ['id' => 'site_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::class, ['id' => 'updated_by']);
    }

    public function getOrder()
    {
        return $this->hasOne(\common\modules\shop\models\ShopOrder::class, ['id' => 'identifier']);
    }

    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::class, ['id' => 'user_id'])->via('order');
    }

    public function behaviors()
    {
        return [
            [
                'class' => TimestampBehavior::class,
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }
}
