<?php

namespace common\modules\mailer\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\mailer\models\MailerCampaign;

/**
 * MailerCampaignSearch represents the model behind the search form of `common\modules\mailer\models\MailerCampaign`.
 */
class MailerCampaignSearch extends MailerCampaign
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'identifier', 'count', 'opens', 'clicks', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['campaign', 'uuid', 'identifier_name', 'timestamp', 'timestamp_open', 'timestamp_click', 'in_queue', 'ip_address', 'referrer_url', 'user_agent', 'created_at', 'updated_at'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = MailerCampaign::find();
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_mailer_campaign.campaign', $this->campaign])
            ->andFilterWhere(['like', 'mod_mailer_campaign.uuid', $this->uuid])
            ->andFilterWhere(['like', 'mod_mailer_campaign.identifier_name', $this->identifier_name])
            ->andFilterWhere(['like', 'mod_mailer_campaign.timestamp', $this->timestamp])
            ->andFilterWhere(['like', 'mod_mailer_campaign.timestamp_open', $this->timestamp_open])
            ->andFilterWhere(['like', 'mod_mailer_campaign.timestamp_click', $this->timestamp_click])
            ->andFilterWhere(['like', 'mod_mailer_campaign.in_queue', $this->in_queue])
            ->andFilterWhere(['like', 'mod_mailer_campaign.ip_address', $this->ip_address])
            ->andFilterWhere(['like', 'mod_mailer_campaign.referrer_url', $this->referrer_url])
            ->andFilterWhere(['like', 'mod_mailer_campaign.user_agent', $this->user_agent])
            ->andFilterWhere(['like', 'mod_mailer_campaign.id', $this->id])
            ->andFilterWhere(['like', 'mod_mailer_campaign.identifier', $this->identifier])
            ->andFilterWhere(['like', 'mod_mailer_campaign.count', $this->count])
            ->andFilterWhere(['like', 'mod_mailer_campaign.opens', $this->opens])
            ->andFilterWhere(['like', 'mod_mailer_campaign.clicks', $this->clicks])
            ->andFilterWhere(['like', 'mod_mailer_campaign.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_mailer_campaign.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_mailer_campaign.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_mailer_campaign.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_mailer_campaign.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
