<?php

namespace common\modules\mailer\models;

use Yii;
use yii\db\Expression;
use yii\helpers\Json;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%mailer_contact}}".
 *
 * @property int $id
 * @property string $email
 * @property string $first_name
 * @property string $last_name
 * @property string $phone
 * @property string $tag
 * @property string $subscribed
 * @property int $site_id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreSite $site
 */
class MailerContact extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%mailer_contact}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['email', 'site_id'], 'required'],
            [['subscribed', 'metadata', 'status'], 'string'],
            [['site_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['email', 'first_name', 'last_name', 'phone', 'tag'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::className(), 'targetAttribute' => ['site_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'email' => 'Email',
            'first_name' => 'First Name',
            'last_name' => 'Last Name',
            'phone' => 'Phone',
            'tag' => 'Tag',
            'subscribed' => 'Subscribed',
            'site_id' => 'Site ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::className(), ['id' => 'site_id']);
    }

    public function behaviors()
    {
        return [
            [
                'class' => \yii\behaviors\TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }
    
    public function saveContact($post_data, $contact_id = 0)
    {
        $session = Yii::$app->session;

        $modelMailerContact = new self();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($contact_id) && $contact_id > 0 && self::find()->where(['id' => $contact_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelMailerContact = self::findOne(['id' => $contact_id, 'status' => 'active']);

                $modelMailerContact->load($post_data);

                $modelMailerContact->metadata = isset($post_data['MailerContact']['MailerContactMetadata']) ? Globals::setJson($post_data['MailerContact']['MailerContactMetadata']) : '';
                $modelMailerContact->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
            } else {    // Create a new model
                $modelMailerContact = new MailerContact();

                $modelMailerContact->load($post_data);

                $modelMailerContact->metadata = isset($post_data['MailerContact']['MailerContactMetadata']) ? Globals::setJson($post_data['MailerContact']['MailerContactMetadata']) : '';
                $modelMailerContact->created_by = (isset($post_data['MailerContact']['created_by']) && !empty($post_data['MailerContact']['created_by'])) ? $post_data['MailerContact']['created_by'] : (Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id);
                $modelMailerContact->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                $modelMailerContact->status = 'active';
            }
            
            if ($modelMailerContact->validate()) {
                $modelMailerContact->save();

                $transaction->commit();

                Yii::$app->session->setFlash('success', "Contact saved successfully.");

                return $modelMailerContact->id;
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save contact: " . Globals::renderErrors($modelMailerContact->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function importMailerContact($data = array(), $tag = null)
    {
        $modelMailerContact = new MailerContact();
        $data_tmp = array();

        $data_tmp = $data;

        if (empty($tag)) {
            $tag = (isset($data_tmp['tag']) && !empty($data_tmp['tag'])) ? $data_tmp['tag'] : null;
        }

        if (isset($data_tmp['follow_up_date']) && !empty($data_tmp['follow_up_date'])) {
            $data_tmp['follow_up_date'] = date('Y-m-d', strtotime($data_tmp['follow_up_date']));
        }

        unset($data_tmp['']);
        unset($data_tmp['email']);

        if (isset($data_tmp['sno']) || empty($data_tmp['sno'])) { unset($data_tmp['sno']); }
        if (isset($data_tmp['first_name']) || empty($data_tmp['first_name'])) { unset($data_tmp['first_name']); }
        if (isset($data_tmp['last_name']) || empty($data_tmp['last_name'])) { unset($data_tmp['last_name']); }
        if (isset($data_tmp['phone']) || empty($data_tmp['phone'])) { unset($data_tmp['phone']); }
        if (isset($data_tmp['tag']) || empty($data_tmp['tag'])) { unset($data_tmp['tag']); }
        if (isset($data_tmp['subscribed']) || empty($data_tmp['subscribed'])) { unset($data_tmp['subscribed']); }
        if (isset($data_tmp['site_id']) || empty($data_tmp['site_id'])) { unset($data_tmp['site_id']); }

        foreach ($data_tmp as $key => $value) {
            if (empty($value)) {
                unset($data_tmp[$key]);
            }
        }

        $modelMailerContact->email = (string) strtolower(trim($data['email']));
        $modelMailerContact->first_name = (string) ucwords(trim((isset($data['first_name']) && !empty($data['first_name'])) ? $data['first_name'] : null));
        $modelMailerContact->last_name = (string) ucwords(trim((isset($data['last_name']) && !empty($data['last_name'])) ? $data['last_name'] : null));
        $modelMailerContact->phone = (string) trim((isset($data['phone']) && !empty($data['phone'])) ? $data['phone'] : null);
        $modelMailerContact->metadata = (count($data_tmp) > 0) ? Json::encode($data_tmp) : null;
        $modelMailerContact->tag = (string) trim((isset($tag) && !empty($tag)) ? $tag : null);
        $modelMailerContact->subscribed = 'yes';
        $modelMailerContact->site_id = intval(trim((isset($data['site_id']) && !empty($data['site_id'])) ? $data['site_id'] : 1));
        $modelMailerContact->created_by = 1;
        $modelMailerContact->updated_by = 1;
        $modelMailerContact->status = 'active';

        if ($modelMailerContact->validate()) {
            $modelMailerContact->save();

            return $modelMailerContact->id;
        } else {
            return false;
        }
    }

    public function importUser($data = array())
    {
        $modelUser = new User();
        $post_data = array();

        $post_data['User']['email'] = (string) strtolower(trim($data['email']));
        $post_data['User']['password_hash'] = (string) strtolower(trim($data['email']));
        $post_data['User']['role_id'] = 2;
        $post_data['User']['site_id'] = 1;
        $post_data['User']['created_by'] = 1;
        $post_data['User']['updated_by'] = 1;
        $post_data['User']['status'] = 'active';

        $post_data['UserProfile']['first_name'] = (string) ucwords(trim(empty($data['first_name']) ? 'NA' : $data['first_name']));
        $post_data['UserProfile']['last_name'] = (string) ucwords(trim(empty($data['last_name']) ? 'NA' : $data['last_name']));
        $post_data['UserProfile']['gender'] = 'male';
        $post_data['UserProfile']['ba_address'] = 'NA'; 
        $post_data['UserProfile']['ba_city'] = 'NA';
        $post_data['UserProfile']['ba_country'] = 'NA';
        $post_data['UserProfile']['ba_zip_code'] = 'NA';
        $post_data['UserProfile']['sa_address'] = 'NA';
        $post_data['UserProfile']['sa_city'] = 'NA';
        $post_data['UserProfile']['sa_country'] = 'NA';
        $post_data['UserProfile']['sa_zip_code'] = 'NA';
        $post_data['UserProfile']['phone'] = (string) trim($data['phone']);
        $post_data['UserProfile']['newsletters'] = 'no';
        $post_data['UserProfile']['created_by'] = 1;
        $post_data['UserProfile']['updated_by'] = 1;
        $post_data['UserProfile']['status'] = 'active';

        return $modelUser->saveUser($post_data);
    }
}
