<?php

namespace common\modules\mailer\models;

use Yii;

/**
 * Our data model extends yii\base\Model class so we can get easy to use and yet 
 * powerful Yii 2 validation mechanism.
 */
class MailerContactSearch extends \yii\base\Model
{
    /**
     * We plan to get two columns in our grid that can be filtered.
     * Add more if required. You don't have to add all of them.
     */
    public $id;
    public $email;
    public $first_name;
    public $last_name;
    public $phone;
    public $tag;
    public $subscribed;
    public $site_id;
    public $metadata_lead_owner;
    public $metadata_organization;
    public $metadata_designation;
    public $metadata_country;
    public $metadata_lead_type;
    public $metadata_product;
    public $metadata_lead_status;
    public $metadata_source;
    public $metadata_pitched_price;
    public $metadata_reason;
    public $metadata_follow_up_date;
    public $metadata_remarks;
    public $metadata_industry_segment;
    public $metadata_industry_segment_challenges;
    public $created_by;
    public $updated_by;
    public $created_at;
    public $updated_at;

    /**
     * Here we can define validation rules for each filtered column.
     * See http://www.yiiframework.com/doc-2.0/guide-input-validation.html
     * for more information about validation.
     */
    public function rules()
    {
        return [
            [['id', 'email', 'first_name', 'last_name', 'phone', 'tag', 'subscribed', 'site_id', 'metadata_lead_owner', 'metadata_organization', 'metadata_designation', 'metadata_country', 'metadata_lead_type', 'metadata_product', 'metadata_lead_status', 'metadata_source', 'metadata_pitched_price', 'metadata_reason', 'metadata_follow_up_date', 'metadata_remarks', 'metadata_industry_segment', 'metadata_industry_segment_challenges', 'created_by', 'updated_by', 'created_at', 'updated_at', ], 'string'],
            // our columns are just simple string, nothing fancy
        ];
    }

    /**
     * In this example we keep this special property to know if columns should be 
     * filtered or not. See search() method below.
     */
    private $_filtered = false;

    /**
     * This method returns ArrayDataProvider.
     * Filtered and sorted if required.
     */
    public function search($params)
    {
        /**
         * $params is the array of GET parameters passed in the actionExample().
         * These are being loaded and validated.
         * If validation is successful _filtered property is set to true to prepare
         * data source. If not - data source is displayed without any filtering.
         */
        if ($this->load($params) && $this->validate()) {
            $this->_filtered = true;
        }

        return new \yii\data\ArrayDataProvider([
            // ArrayDataProvider here takes the actual data source
            'allModels' => $this->getData(),
            'key' => 'id',
            'sort' => [
                // we want our columns to be sortable:
                'attributes' => ['id', 'email', 'first_name', 'last_name', 'phone', 'tag', 'subscribed', 'site_id', 'metadata_lead_owner', 'metadata_organization', 'metadata_designation', 'metadata_country', 'metadata_lead_type', 'metadata_product', 'metadata_lead_status', 'metadata_source', 'metadata_pitched_price', 'metadata_reason', 'metadata_follow_up_date', 'metadata_remarks', 'metadata_industry_segment', 'metadata_industry_segment_challenges', 'created_by', 'updated_by', 'created_at', 'updated_at', ],
                'defaultOrder' => ['id' => SORT_DESC],
            ],
        ]);
    }

    /**
     * Here we are preparing the data source and applying the filters
     * if _filtered property is set to true.
     */
    protected function getData()
    {
        $data = MailerContact::find()->where(['status' => 'active'])->asArray()->all();

        foreach ($data as $k => $v) {
            $json_arr = json_decode($v['metadata'], true);

            foreach (Yii::$app->params['contact_metadata'] as $key => $value) {
                $data[$k]['metadata_' . $key] = null;
            }

            if (!is_array($json_arr)) {
                continue;
            }

            foreach ($json_arr as $_k => $_v) {
                $data[$k]['metadata_' . $_k] = $_v;
            }
        }

        if ($this->_filtered) {
            $data = array_filter($data, function ($value) {
                $conditions = [true];

                if (!empty($this->id)) {
                    $conditions[] = stripos($value['id'], $this->id) !== false;
                }

                if (!empty($this->email)) {
                    $conditions[] = stripos($value['email'], $this->email) !== false;
                }

                if (!empty($this->first_name)) {
                    $conditions[] = stripos($value['first_name'], $this->first_name) !== false;
                }

                if (!empty($this->last_name)) {
                    $conditions[] = stripos($value['last_name'], $this->last_name) !== false;
                }

                if (!empty($this->phone)) {
                    $conditions[] = stripos($value['phone'], $this->phone) !== false;
                }

                if (!empty($this->tag)) {
                    $conditions[] = stripos($value['tag'], $this->tag) !== false;
                }

                if (!empty($this->subscribed)) {
                    $conditions[] = stripos($value['subscribed'], $this->subscribed) !== false;
                }

                if (!empty($this->site_id)) {
                    $conditions[] = stripos(\common\modules\core\models\CoreSite::getAttr(['name'], ['id' => $value['site_id']]), \common\modules\core\models\CoreSite::getAttr(['name'], ['id' => $this->site_id])) !== false;
                }

                if (!empty($this->metadata_organization)) {
                    $conditions[] = stripos($value['metadata_organization'], $this->metadata_organization) !== false;
                }

                if (!empty($this->metadata_designation)) {
                    $conditions[] = stripos($value['metadata_designation'], $this->metadata_designation) !== false;
                }

                if (!empty($this->metadata_country)) {
                    $conditions[] = stripos($value['metadata_country'], $this->metadata_country) !== false;
                }

                if (!empty($this->metadata_lead_type)) {
                    $conditions[] = stripos($value['metadata_lead_type'], $this->metadata_lead_type) !== false;
                }

                if (!empty($this->metadata_product)) {
                    $conditions[] = stripos($value['metadata_product'], $this->metadata_product) !== false;
                }

                if (!empty($this->metadata_lead_status)) {
                    $conditions[] = stripos($value['metadata_lead_status'], $this->metadata_lead_status) !== false;
                }

                if (!empty($this->metadata_source)) {
                    $conditions[] = stripos($value['metadata_source'], $this->metadata_source) !== false;
                }

                if (!empty($this->metadata_pitched_price)) {
                    $conditions[] = stripos($value['metadata_pitched_price'], $this->metadata_pitched_price) !== false;
                }

                if (!empty($this->metadata_reason)) {
                    $conditions[] = stripos($value['metadata_reason'], $this->metadata_reason) !== false;
                }

                if (!empty($this->metadata_follow_up_date)) {
                    $conditions[] = stripos($value['metadata_follow_up_date'], $this->metadata_follow_up_date) !== false;
                }

                if (!empty($this->metadata_remarks)) {
                    $conditions[] = stripos($value['metadata_remarks'], $this->metadata_remarks) !== false;
                }

                if (!empty($this->metadata_industry_segment)) {
                    $conditions[] = stripos($value['metadata_industry_segment'], $this->metadata_industry_segment) !== false;
                }

                if (!empty($this->metadata_industry_segment_challenges)) {
                    $conditions[] = stripos($value['metadata_industry_segment_challenges'], $this->metadata_industry_segment) !== false;
                }

                if (!empty($this->created_by)) {
                    $conditions[] = stripos(\common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $value['created_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $value['created_by']]), $this->created_by) !== false;
                }

                if (!empty($this->updated_by)) {
                    $conditions[] = stripos(\common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $value['updated_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $value['updated_by']]), $this->updated_by) !== false;
                }

                if (!empty($this->created_at)) {
                    $conditions[] = stripos($value['created_at'], $this->created_at) !== false;
                }

                if (!empty($this->updated_at)) {
                    $conditions[] = stripos($value['updated_at'], $this->updated_at) !== false;
                }

                return array_product($conditions);
            });
        }

        return $data;
    }
}
