<?php

use yii\bootstrap4\Html;
use kartik\grid\GridView;
use kartik\dynagrid\DynaGrid;

/* @var $this yii\web\View */
/* @var $searchModel common\modules\mailer\models\MailerContactSearch */
/* @var $dataProvider yii\data\ActiveDataProvider */

$this->title = 'Contacts';
$this->params['breadcrumbs'][] = $this->title;

$columns = [
    ['class' => 'kartik\grid\SerialColumn', 'order' => DynaGrid::ORDER_FIX_LEFT],

    [
        'attribute' => 'id',
        'label' => 'Contact ID',
        'value' => 'id'
    ],
    'email:email',
    'first_name',
    'last_name',
    'phone',
    'tag',
    [
        'attribute' => 'subscribed',
        'value' => function ($model, $key, $index, $widget) {
            return ucfirst($model['subscribed']);
        },
        'filter' => ['yes' => 'Yes', 'no' => 'No'],
        'visible' => false
    ],
    [
        'attribute' => 'site_id',
        'label' => 'Site',
        'value' => function ($model, $key, $index, $widget) {
            return \common\modules\core\models\CoreSite::getAttr(['name'], ['id' => $model['site_id']]);
        },
        'filter' => \common\modules\core\models\CoreSite::getAll($k = 'id', $v = 'name'),
        'visible' => false
    ],
    [
        'attribute' => 'created_by',
        'value' => function ($model, $key, $index, $widget) {
            return \common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $model['created_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $model['created_by']]);
        },
        'visible' => false
    ],
    [
        'attribute' => 'updated_by',
        'value' => function ($model, $key, $index, $widget) {
            return \common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $model['updated_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $model['updated_by']]);
        },
        'visible' => false
    ],
    'created_at',
    'updated_at',

    ['class' => 'kartik\grid\ActionColumn', 'order' => DynaGrid::ORDER_FIX_RIGHT],
];

$columns = array_merge($columns, \common\modules\core\components\Globals::dynagridMetadataColumns('contact_metadata', ['pitched_price', 'reason', 'remarks']));
?>

<div class="container-fluid mailer-contact-index">
    <div class="row">
        <div class="col-md-12">
            <div class="card">
                <div class="card-body">
                    <div class="row mb-2">
                        <div class="col-md-12">
                            <?= Html::a('Create Mailer Contact', ['create'], ['class' => 'btn btn-success']) ?>
                        </div>
                    </div>

                    <?= DynaGrid::widget([
                        'columns' => $columns,
                        'storage' => DynaGrid::TYPE_SESSION,
                        'theme' => 'simple-condensed',
                        'allowPageSetting' => true,
                        'allowThemeSetting' => false,
                        'allowFilterSetting' => false,
                        'allowSortSetting' => false,
                        'gridOptions' => [
                            'dataProvider' => $dataProvider,
                            'filterModel' => $searchModel,
                            'showPageSummary' => false,
                            'floatHeader' => false,
                            'pjax' => true,
                            'responsiveWrap' => true,
                            'panel' => [
                                'after' => false
                            ],
                            'toolbar' =>  [
                                ['content' => 
                                    Html::a('<i class="fas fa-plus"></i>', ['create'], [
                                        'type' => 'button', 
                                        'title' => 'Add User', 
                                        'class' => 'btn btn-success', 
                                        'data-pjax' => 0, 
                                    ]) . 
                                    Html::a('<i class="fas fa-redo"></i>', ['index'], [
                                        'type' => 'button', 
                                        'title' => 'Reset Grid', 
                                        'class' => 'btn btn-outline-secondary', 
                                        'data-pjax' => 0, 
                                    ]) . 
                                    '{dynagrid}' . 
                                    '{export}'
                                ],
                            ]
                        ],
                        'options' => ['id' => 'dynagrid-mailer-contact']
                    ]);
                    ?>
                </div>
                <!--.card-body-->
            </div>
            <!--.card-->
        </div>
        <!--.col-md-12-->
    </div>
    <!--.row-->
</div>
