<?php

namespace common\modules\payment\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\payment\components\CCAvenue;
use common\modules\payment\models\PaymentMethod;
use common\modules\payment\models\PaymentLogs;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreSite;
use common\modules\user\models\UserIdentity;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\user\models\UserRole;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopOrderStatus;
use common\modules\accounting\models\AccountingCurrency;
use common\modules\lms\models\LmsCourse;

/**
 * Payment controller
 */
class CcavenueController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['pay'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['pay'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [

                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    public function beforeAction($action) 
    {
        if (Yii::$app->controller->action->id == 'pay') {
            $this->enableCsrfValidation = false;
        }

        return parent::beforeAction($action);
    }

    public function actionPay()
    {
        $session = Yii::$app->session->open();

        /*if (!isset($_SESSION['gateway_call']) || $_SESSION['gateway_call'] != 'true') {
            throw new \yii\web\HttpException(422, 'Payment gateway not called.');
        }*/

        $modelUser = new User();
        $modelUserProfile = new UserProfile();
        $modelShopOrder = new ShopOrder();
        $modelShopOrderItem = new ShopOrderItem();

        $workingKey = Yii::$app->params['payment_gateways'][Yii::$app->params['payment_gateway']][Yii::$app->params['mode']]['working_key'];     //Working Key should be provided here.
        $encResponse = $_POST["encResp"];         //This is the response sent by the CCAvenue Server
        $rcvdString = CCAvenue::decrypt($encResponse, $workingKey);      //Crypto Decryption used as per the specified working key.
        $order_status = "";
        /*$rcvdString = 'order_id=dhrvo4gdcceu7cnr70g0fiaddg&tracking_id=109014602115&bank_ref_no=544915&order_status=Success&failure_message=&payment_mode=Credit Card&card_name=MasterCard&status_code=00&status_message=Approved¤cy=AED&amount=999.0&billing_name=Pranshu Praneta&billing_address=LIG-30 B, Kadam Vihar&billing_city=Mathura&billing_state=Uttar Pradesh&billing_zip=281006&billing_country=India&billing_tel=8791112279&billing_email=pranetapranshu@gmail.com&delivery_name=Pranshu Praneta&delivery_address=LIG-30 B, Kadam Vihar&delivery_city=Mathura&delivery_state=Uttar Pradesh&delivery_zip=281006&delivery_country=India&delivery_tel=8791112279&merchant_param1=&merchant_param2=&merchant_param3=&merchant_param4=&merchant_param5=&vault=N&offer_type=null&offer_code=null&discount_value=0.0&mer_amount=999.0&eci_value=05&card_holder_name=&bank_qsi_no=30000013102&bank_receipt_no=022501544915&merchant_param6=5204740014';*/
        $decryptValues = explode('&', $rcvdString);
        $dataSize = sizeof($decryptValues);

        for ($i = 0; $i < $dataSize; $i++) {
            $information[] = explode('=', $decryptValues[$i]);
            
            if ($i == 3) {
                $order_status = $information[3][1];
            }
        }

        foreach ($information as $v) {
            $tmp[$v[0]] = $v[1];
        }

        $information = $tmp;

        $identity = UserIdentity::findIdentity($information['merchant_param1']);
        
        Yii::$app->user->login($identity);

        if (Yii::$app->user->isGuest) {
            throw new \yii\web\UnauthorizedHttpException("Unauthenticated user.", 401);
        }

        $modelUser = User::find()->where(['email' => Yii::$app->user->identity->email, 'status' => 'active'])->asArray()->one();
        $modelUserProfile = UserProfile::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->one();

        $post_data['User'] = $modelUser;
        $post_data['User']['updated_by'] = Yii::$app->user->identity->id;
        $post_data['User']['updated_at'] = date('Y-m-d H:s:i');
        
        $post_data['UserProfile'] = $modelUserProfile;
        $post_data['UserProfile']['updated_by'] = Yii::$app->user->identity->id;
        $post_data['UserProfile']['updated_at'] = date('Y-m-d H:s:i');

        $modelUser = new User();

        $user_id = $modelUser->saveUser($post_data, Yii::$app->user->identity->id);

        // save order
        $order_id = $this->saveOrder($information, $rcvdString, $user_id);

        $modelShopOrder = ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->asArray()->one();
        $modelShopOrderItem = ShopOrderItem::find()->where(['order_id' => $order_id, 'status' => 'active'])->asArray()->all();

        if ($order_status === "Success") {
            // unset($_SESSION['gateway_call']);
            
            return $this->render('thank-you', [
                'message' => 'Thank you for shopping with us. Your credit card has been charged and your transaction is successful.', 
                'information' => $information,
                'modelShopOrder' => $modelShopOrder,
                'modelShopOrderItem' => $modelShopOrderItem,
            ]);
        } else if ($order_status === "Aborted") {
            return $this->render('thank-you-failure', [
                'message' => 'Thank you for shopping with us. We will keep you posted regarding the status of your order through e-mail.', 
                'information' => $information,
                'modelShopOrder' => $modelShopOrder,
                'modelShopOrderItem' => $modelShopOrderItem,
            ]);
        } else if ($order_status === "Failure") {
            return $this->render('thank-you-failure', [
                'message' => 'Payment Failed.', 
                'reason' => 'The transaction has been declined.', 
                'information' => $information,
            ]);
        } else {
            return $this->render('thank-you-failure', [
                'message' => 'Security Error', 
                'reason' => 'Illegal access detected.', 
                'information' => $information,
            ]);
        }
    }

    public function saveOrder($form_data = array(), $return_data, $user_id = null)
    {
        $session = Yii::$app->session->open();

        if ($form_data instanceof \yii\base\Event) {
            $return_data = $form_data->data->returnData;
            $form_data = $form_data->data->formData;
        }

        $modelUser = new User();
        $modelShopOrder = new ShopOrder();
        $modelPaymentLogs = new PaymentLogs();
        $post_data = array();
        $posted_data = array();
        $modelUserArr = array();
        $modelUserProfileArr = array();
        $modelShopOrderArr = array();
        $modelShopOrderItemArr = array();
        $flag_new_user = true;
        $last_order_id = 1;

        $posted_data = $form_data;

        // validating response from square
        if (!isset($return_data) || empty($return_data)) {
            throw new \yii\web\BadRequestHttpException('Empty response from payment gateway.');
        }

        // get last order id inserted (for invoice number)
        if (ShopOrder::find()->orderBy(['id' => SORT_DESC])->exists()) {
            $last_order = ShopOrder::find()->orderBy(['id' => SORT_DESC])->one();
            $last_order_id = $last_order->id;
        } else {
            $last_order_id = 1;
        }

        $transaction = Yii::$app->db->beginTransaction();

        try {
            // validating successful user creation
            if (!isset($user_id) || empty($user_id)) {
                throw new \yii\web\HttpException(422, 'User not created.');
            }

            $modelUserArr = User::find()->where(['id' => $user_id, 'status' => 'active'])->asArray()->one();
            $modelUserProfileArr = UserProfile::find()->where(['user_id' => $user_id, 'status' => 'active'])->asArray()->one();

            // Create order
            $post_data['ShopOrder']['user_id'] = $user_id;
            $post_data['ShopOrder']['first_name'] = $modelUserProfileArr['first_name'];
            $post_data['ShopOrder']['last_name'] = $modelUserProfileArr['last_name'];
            $post_data['ShopOrder']['order_status_id'] = ShopOrderStatus::getAttr(['id'], ['name' => 'New']);
            $post_data['ShopOrder']['voucher_id'] = (string) ((isset($_SESSION['cart']['voucher_id']) && !empty($_SESSION['cart']['voucher_id'])) ? $_SESSION['cart']['voucher_id'] : null);
            $post_data['ShopOrder']['amount'] = (string) ((isset($posted_data['amount']) && !empty($posted_data['amount'])) ? Globals::formatNumber($posted_data['amount'], ['round' => true, 'precision' => 2]) : 0);
            $post_data['ShopOrder']['exchange_rate'] = AccountingCurrency::getAttr(['rate'], ['code' => 'usd']);
            $post_data['ShopOrder']['invoice_number'] = Yii::$app->params['invoice_prefix'] . date('Ymd') . str_pad(mt_rand(0, 10000), 5, '0', STR_PAD_LEFT);
            $post_data['ShopOrder']['credit_note_number'] = null;
            $post_data['ShopOrder']['session_id'] = Yii::$app->session->getId();
            $post_data['ShopOrder']['order_date'] = date('Y-m-d');
            $post_data['ShopOrder']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
            $post_data['ShopOrder']['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['status'] = 'active';

            $product_ids = explode('|', $posted_data['merchant_param2']);

            if (isset($_SESSION['cart']['size']) && $_SESSION['cart']['size'] > 0) {
                foreach ($_SESSION['cart'] as $cartKey => $cartValue) {
                    if (!is_array($cartValue)) { continue; }

                    $post_data['ShopOrderItem'][$cartKey]['product_id'] = $product_ids[$cartKey];

                    if (isset($posted_data['custom']) && !empty($posted_data['custom'])) {
                        $post_data['ShopOrderItem'][$cartKey]['product_name'] = (string) $posted_data['custom'];
                    }
                    
                    $post_data['ShopOrderItem'][$cartKey]['cart_id'] = (isset($cartValue['cart_id']) && !empty($cartValue['cart_id'])) ? $cartValue['cart_id'] : null;
                    $post_data['ShopOrderItem'][$cartKey]['quantity'] = (isset($cartValue['quantity']) && !empty($cartValue['quantity'])) ? $cartValue['quantity'] : null;
                    $post_data['ShopOrderItem'][$cartKey]['price'] = (string) (isset($cartValue['price']) && !empty($cartValue['price'])) ? $cartValue['price'] : null;
                    $post_data['ShopOrderItem'][$cartKey]['traits'] = null;
                    $post_data['ShopOrderItem'][$cartKey]['created_by'] = isset($user_id) ? $user_id : 1;
                    $post_data['ShopOrderItem'][$cartKey]['updated_by'] = isset($user_id) ? $user_id : 1;
                    $post_data['ShopOrderItem'][$cartKey]['status'] = 'active';
                }
            } else {
                $post_data['ShopOrderItem'][0]['product_id'] = $product_ids[0];

                if (isset($posted_data['custom']) && !empty($posted_data['custom'])) {
                    $post_data['ShopOrderItem'][0]['product_name'] = (string) $posted_data['custom'];
                }
                
                $post_data['ShopOrderItem'][0]['cart_id'] = (isset($_SESSION['cart'][0]['cart_id']) && !empty($_SESSION['cart'][0]['cart_id'])) ? $_SESSION['cart'][0]['cart_id'] : null;
                $post_data['ShopOrderItem'][0]['quantity'] = 1;
                $post_data['ShopOrderItem'][0]['price'] = (string) $post_data['ShopOrder']['amount'];
                $post_data['ShopOrderItem'][0]['traits'] = null;
                $post_data['ShopOrderItem'][0]['created_by'] = isset($user_id) ? $user_id : 1;
                $post_data['ShopOrderItem'][0]['updated_by'] = isset($user_id) ? $user_id : 1;
                $post_data['ShopOrderItem'][0]['status'] = 'active';
            }

            $order_id = $modelShopOrder->saveOrder($post_data);

            // validating successful order creation
            if (!isset($order_id) || empty($order_id)) {
                throw new \yii\web\HttpException(422, 'Order not created.');
            } else {
                // Create user-course subscription, in case of course order
                if (ShopProduct::getAttr(['code'], ['id' => 1]) == LmsCourse::getAttr(['code'], ['id' => 1])) {
                    $data['LmsUserCourse']['user_id'] = $user_id;
                    $data['LmsUserCourse']['course_id'] = LmsCourse::getAttr(['id'], ['code' => ShopProduct::getAttr(['code'], ['id' => $post_data['ShopOrderItem'][0]['product_id']])]);
                    $data['LmsUserCourse']['expiration_date'] = date('Y-m-d H:s:i', strtotime('+365 days'));
                    $data['LmsUserCourse']['valid'] = 'yes';

                    $eventLmsEvents = new \common\modules\lms\components\LmsEvents;

                    $eventLmsEvents->eventModLmsCreateUserCourse($data);
                }
            }

            $modelShopOrderArr = ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->asArray()->one();
            $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $order_id, 'status' => 'active'])->asArray()->all();

            // saving order id into session
            $_SESSION['order_id'] = $order_id;

            // Save paypal logs
            $modelPaymentLogs->order_id = $order_id;
            $modelPaymentLogs->payment_method_id = PaymentMethod::getAttr(['id'], ['name' => 'paypal']);
            $modelPaymentLogs->response = $return_data;
            $modelPaymentLogs->created_by = $user_id;
            $modelPaymentLogs->updated_by = $user_id;

            if ($modelPaymentLogs->validate()) {
                $modelPaymentLogs->save();

                $transaction->commit();

                /*// order confirmation email to customer
                \common\modules\site\components\Email::send(
                    'order-confirmation-customer', 
                    [
                        'html' => 'orderConfirmationCustomer-html', 
                        'text' => 'orderConfirmationCustomer-text'
                    ], 
                    [
                        'to' => $modelUserArr['email'], 
                        'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                    ], 
                    null, 
                    ['user_data' => [
                        'user' => $modelUserArr, 
                        'order' => $modelShopOrderArr, 
                        'order_item' => $modelShopOrderItemArr, 
                    ]], 
                    Yii::$app->params['site_code'], 
                    Yii::$app->params['site_code']
                );*/

                return $order_id;
            } else {
                $transaction->rollBack();
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
