<?php

namespace common\modules\payment\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\helpers\Json;

use common\modules\payment\models\FormFhtpay;
use common\modules\payment\models\PaymentMethod;
use common\modules\payment\models\PaymentLogs;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreSite;
use common\modules\site\components\Email;
use common\modules\user\models\User;
use common\modules\user\models\UserRole;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopOrderStatus;
use common\models\EmailMap;

/**
 * Payment controller
 */
class FhtpayController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'pay'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'pay'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [

                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    public function actionIndex($data)
    {
        $modelFormFhtpay = new FormFhtpay();

        $data = Json::decode($data);

        return $this->renderAjax('_form.php', [
            'modelFormFhtpay' => $modelFormFhtpay, 
            'amount' => (isset($data['amount']) ? $data['amount'] : 0), 
            'description' => (isset($data['description']) ? $data['description'] : ''), 
        ]);
    }

    public function actionPay($ext_auth_key = '')
    {
        $session = Yii::$app->session->open();

        $_SESSION['gateway_call'] = 'true';

        $form_data = array();
        $return_data = array();
        $fullname = array();
        $response = null;
        $first_name = '';
        $last_name = '';

        $mandatory_keys = ['cardNumber', 'cardExpiry', 'cardCVC', 'grPerName', 'grEmail', 'grphoneNumber', 'grAddress', 'grCity', 'grCountry', 'grZipCode', 'apiType', 'merremark', 'merMgrURL', 'returnURL', 'webInfo', 'language', 'goodsString', 'EncryptionMode', 'CharacterSet', 'merNo', 'terNo', 'orderNo', 'currencyCode', 'amount', 'payIP', 'transType', 'transModel', 'hashcode'];

        if (!$_POST || !isset($_POST['FormFhtpay']) || empty($_POST['FormFhtpay'])) {
            throw new \yii\web\BadRequestHttpException('Insufficient parameters to process payment.');
        }

        // validating mandatory fields
        foreach ($mandatory_keys as $mandatory_key) {
            if (!isset($_POST['FormFhtpay'][$mandatory_key]) || empty($_POST['FormFhtpay'][$mandatory_key])) {
                throw new \yii\web\BadRequestHttpException('Insufficient parameters to process payment.');
            }
        }

        $fullname = explode(' ', $_POST['FormFhtpay']['grPerName']);

        if (count($fullname) < 2) { // last name not provided
            $first_name = $fullname[0];
            $last_name = '';
        } else {
            $first_name = $fullname[0];

            unset($fullname[0]);

            $last_name = implode(' ', $fullname);
        }

        $_POST['FormFhtpay']['first_name'] = $first_name;
        $_POST['FormFhtpay']['last_name'] = $last_name;

        $form_data = $_POST['FormFhtpay'];

        $fht = [
            'url'               => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['url'],
            'apiType'           => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['apiType'],
            'EncryptionMode'    => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['EncryptionMode'],
            'CharacterSet'      => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['CharacterSet'],
            'merNo'             => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['merNo'],
            'terNo'             => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['terNo'],
            'currencyCode'      => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['currencyCode'],
            'returnURL'         => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['returnURL'],
            'language'          => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['language'],
            'merMgrURL'         => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['merMgrURL'],
            'hashcode'          => Yii::$app->params['fhtpay'][Yii::$app->params['mode']]['hashcode'],
        ];

        $exp = str_replace(' ', '', $_POST['FormFhtpay']['cardExpiry']);
        $exp = explode('/', $exp);
        $expMonth = $exp[0]; 
        $expYear = ((strlen($exp[1]) == 2) ? '20' . $exp[1] : $exp[1]);

        $data = [
            'apiType'           => $_POST['FormFhtpay']['apiType'],
            'language'          => $_POST['FormFhtpay']['language'],
            'returnURL'         => $_POST['FormFhtpay']['returnURL'],
            'webInfo'           => $_POST['FormFhtpay']['webInfo'],
            'cardCountry'       => $_POST['FormFhtpay']['grCountry'],
            'cardCity'          => $_POST['FormFhtpay']['grCity'],
            'cardAddress'       => $_POST['FormFhtpay']['grAddress'],
            'cardZipCode'       => $_POST['FormFhtpay']['grZipCode'],
            'cardFullPhone'     => $_POST['FormFhtpay']['grphoneNumber'],
            'cardFullName'      => $_POST['FormFhtpay']['grPerName'],
            'grCountry'         => $_POST['FormFhtpay']['grCountry'],
            'grCity'            => $_POST['FormFhtpay']['grCity'],
            'grAddress'         => $_POST['FormFhtpay']['grAddress'],
            'grZipCode'         => $_POST['FormFhtpay']['grZipCode'],
            'grEmail'           => EmailMap::imitateEmail($_POST['FormFhtpay']['grEmail']),
            'grphoneNumber'     => $_POST['FormFhtpay']['grphoneNumber'],
            'grPerName'         => $_POST['FormFhtpay']['grPerName'],
            'goodsString'       => $_POST['FormFhtpay']['goodsString'],
            'cardNO'            => $_POST['FormFhtpay']['cardNumber'],
            'expYear'           => $expYear,
            'expMonth'          => $expMonth,
            'cvv'               => $_POST['FormFhtpay']['cardCVC'],
        ];

        $arrHashCode = [
            'EncryptionMode'    => $_POST['FormFhtpay']['EncryptionMode'],
            'CharacterSet'      => $_POST['FormFhtpay']['CharacterSet'],
            'merNo'             => $_POST['FormFhtpay']['merNo'],
            'terNo'             => $_POST['FormFhtpay']['terNo'],
            'orderNo'           => $_POST['FormFhtpay']['orderNo'],
            'currencyCode'      => $_POST['FormFhtpay']['currencyCode'],
            'amount'            => $_POST['FormFhtpay']['amount'],
            'payIP'             => $_POST['FormFhtpay']['payIP'],
            'transType'         => $_POST['FormFhtpay']['transType'],
            'transModel'        => $_POST['FormFhtpay']['transModel'],
        ];

        if (Yii::$app->params['mode'] == 'live') {
            // call payment processor
            $response = Yii::$app->Fhtpay->pay($fht, $arrHashCode, $data);

            $result = JSON::decode($response);        
            $return_data = array(
                'transType' => (isset($result['transType']) ? $result['transType'] : ''),
                'orderNo' => (isset($result['orderNo']) ? $result['orderNo'] : ''),
                'merNo' => (isset($result['merNo']) ? $result['merNo'] : ''),
                'terNo' => (isset($result['terNo']) ? $result['terNo'] : ''),
                'currencyCode' => (isset($result['currencyCode']) ? $result['currencyCode'] : ''),
                'amount' => (isset($result['amount']) ? $result['amount'] : ''),
                'tradeNo' => (isset($result['tradeNo']) ? $result['tradeNo'] : ''),
                'respCode' => $result['respCode'],
                'respMsg' => $result['respMsg']
            );
        } else {
            // a stub
            $return_data = array(
                'transType' => $_POST['FormFhtpay']['transType'],
                'orderNo' => $_POST['FormFhtpay']['orderNo'],
                'merNo' => $_POST['FormFhtpay']['merNo'],
                'terNo' => $_POST['FormFhtpay']['terNo'],
                'currencyCode'=> $_POST['FormFhtpay']['currencyCode'],
                'amount' => $_POST['FormFhtpay']['amount'],
                'tradeNo' => 'FHT1512302310022875',
                'respCode' => '00',
                'respMsg' => '00:succeed'
            );
        }

        if ($return_data['respCode'] != '00') {
            // Retry with fallback payment processor
            if (Yii::$app->params['payment_gateway_fallback_enabled'] === true) {
                $tmp_post = $_POST;

                foreach (Yii::$app->params['payment_gateway_fallback'] as $payment_gateway_fallback) {  // iterating over fallback processors
                    $_POST = $tmp_post;
                    
                    switch (Yii::$app->params['payment_gateway_fallback'][$_SESSION['payment_gateway_fallback_index']]) {
                        case 'stripe':
                            if (Yii::$app->Globals->limitExceeded('stripe') && ($_SESSION['payment_gateway_fallback_index'] < (count(Yii::$app->params['payment_gateway_fallback']) - 1))) {
                                $_SESSION['payment_gateway_fallback_index']++;

                                continue 2;
                            }

                            $_POST = [
                                'name' => $_POST['FormFhtpay']['grPerName'], 
                                'email' => $_POST['FormFhtpay']['grEmail'], 
                                'amount' => (isset($_POST['FormFhtpay']['amount']) ? intval($_POST['FormFhtpay']['amount'] * 100) : 0), 
                                'domain' => $_SESSION['whois']['domain']['name'], 
                                'card' => [
                                    'number' => $_POST['FormFhtpay']['cardNumber'], 
                                    'exp_month' => $expMonth, 
                                    'exp_year' => $expYear, 
                                    'cvc' => $_POST['FormFhtpay']['cardCVC'], 
                                ],
                                "description" => $_POST['FormFhtpay']['merremark'], 
                                "address_line1" => $_POST['FormFhtpay']['grAddress'], 
                                "address_line2" => '', 
                                "address_city" => $_POST['FormFhtpay']['grCity'], 
                                "address_state" => '', 
                                "address_zip" => $_POST['FormFhtpay']['grZipCode'], 
                                "address_country" => $_POST['FormFhtpay']['grCountry'], 
                            ];

                            $result = Yii::$app->runAction('payment/stripe/pay', ['post' => $_POST, 'retry' => true]);

                            if ($result == 'Payment Declined' && ($_SESSION['payment_gateway_fallback_index'] < (count(Yii::$app->params['payment_gateway_fallback']) - 1))) {
                                $_SESSION['payment_gateway_fallback_index']++;

                                continue 2;
                            }

                            return $result;
                            
                            break;

                        case 'authorizenet':
                            if (Yii::$app->Globals->limitExceeded('authorizenet') && ($_SESSION['payment_gateway_fallback_index'] < (count(Yii::$app->params['payment_gateway_fallback']) - 1))) {
                                $_SESSION['payment_gateway_fallback_index']++;

                                continue 2;
                            }
                            
                            $_POST = [
                                'AuthorizenetForm' => [
                                    'cardNumber' => $_POST['FormFhtpay']['cardNumber'], 
                                    'cardExpiry' => $_POST['FormFhtpay']['cardExpiry'], 
                                    'grPerName' => $_POST['FormFhtpay']['grPerName'], 
                                    'grEmail' => $_POST['FormFhtpay']['grEmail'], 
                                    'amount' => (isset($_POST['FormFhtpay']['amount']) ? $_POST['FormFhtpay']['amount'] : 0), 
                                    'domain' => $_SESSION['whois']['domain']['name'], 
                                    'card' => [
                                        'number' => $_POST['FormFhtpay']['cardNumber'], 
                                        'exp_month' => $expMonth, 
                                        'exp_year' => $expYear, 
                                        'cvc' => $_POST['FormFhtpay']['cardCVC'], 
                                    ],
                                    "description" => $_POST['FormFhtpay']['merremark'], 
                                    "address_line1" => $_POST['FormFhtpay']['grAddress'], 
                                    "address_line2" => '', 
                                    "address_city" => $_POST['FormFhtpay']['grCity'], 
                                    "address_state" => '', 
                                    "address_zip" => $_POST['FormFhtpay']['grZipCode'], 
                                    "address_country" => $_POST['FormFhtpay']['grCountry'], 
                                ], 
                            ];

                            $result = Yii::$app->runAction('payment/authorizenet/pay', ['post' => $_POST, 'retry' => true]);

                            if ($result == 'Payment Declined' && ($_SESSION['payment_gateway_fallback_index'] < (count(Yii::$app->params['payment_gateway_fallback']) - 1))) {
                                $_SESSION['payment_gateway_fallback_index']++;

                                continue 2;
                            }

                            return $result;
                            
                            break;
                        
                        default:
                            return 'Payment Declined';

                            break;
                    }
                }
            }
            
            return 'Payment Declined';
        } else {
            // save order et al.
            return $this->saveOrder($form_data, $return_data);
        }
    }

    public function saveOrder($form_data = array(), $return_data = array())
    {
        $session = Yii::$app->session->open();

        $modelUser = new User();
        $modelShopOrder = new ShopOrder();
        $modelPaymentLogs = new PaymentLogs();
        $modelEmailMap = new EmailMap();
        $post_data = array();
        $posted_data = array();
        $modelUserArr = array();
        $modelShopOrderArr = array();
        $modelShopOrderItemArr = array();
        $flag_new_user = true;
        $last_order_id = 1;

        $posted_data = $form_data;

        // validating response from fhtpay
        if (!isset($return_data) || empty($return_data)) {
            throw new \yii\web\BadRequestHttpException('Empty response from payment gateway.');
        }

        // get last order id inserted (for invoice number)
        if (ShopOrder::find()->orderBy(['id' => SORT_DESC])->exists()) {
            $last_order = ShopOrder::find()->orderBy(['id' => SORT_DESC])->one();
            $last_order_id = $last_order->id;
        } else {
            $last_order_id = 1;
        }

        $transaction = Yii::$app->db->beginTransaction();

        try {
            // Create/Update User
            if (User::find()->where(['email' => $posted_data['grEmail'], 'status' => 'active'])->exists()) {   // user already exists
                $flag_new_user = false;

                $modelUser = User::find()->where(['email' => $posted_data['grEmail'], 'status' => 'active'])->one();

                $post_data['User']['email'] = $modelUser->email;
                $post_data['User']['password_hash'] = $modelUser->password_hash;
                $post_data['User']['role_id'] = $modelUser->role_id;
                $post_data['User']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
                $post_data['User']['created_by'] = $modelUser->created_by;
                $post_data['User']['updated_by'] = $modelUser->id;

                $post_data['UserProfile']['first_name'] = $posted_data['first_name'];
                $post_data['UserProfile']['last_name'] = $posted_data['last_name'];
                $post_data['UserProfile']['ba_address'] = ((isset($posted_data['grAddress']) && !empty($posted_data['grAddress'])) ? $posted_data['grAddress'] : '--');
                $post_data['UserProfile']['ba_city'] = ((isset($posted_data['grCity']) && !empty($posted_data['grCity'])) ? $posted_data['grCity'] : '--');
                $post_data['UserProfile']['ba_country'] = ((isset($posted_data['grCountry']) && !empty($posted_data['grCountry'])) ? $posted_data['grCountry'] : '--');
                $post_data['UserProfile']['ba_zip_code'] = ((isset($posted_data['grZipCode']) && !empty($posted_data['grZipCode'])) ? $posted_data['grZipCode'] : NULL);
                $post_data['UserProfile']['sa_address'] = ((isset($posted_data['grAddress']) && !empty($posted_data['grAddress'])) ? $posted_data['grAddress'] : '--');
                $post_data['UserProfile']['sa_city'] = ((isset($posted_data['grCity']) && !empty($posted_data['grCity'])) ? $posted_data['grCity'] : '--');
                $post_data['UserProfile']['sa_country'] = ((isset($posted_data['grCountry']) && !empty($posted_data['grCountry'])) ? $posted_data['grCountry'] : '--');
                $post_data['UserProfile']['sa_zip_code'] = ((isset($posted_data['grZipCode']) && !empty($posted_data['grZipCode'])) ? $posted_data['grZipCode'] : NULL);
                $post_data['UserProfile']['phone'] = ((isset($posted_data['grphoneNumber']) && !empty($posted_data['grphoneNumber'])) ? $posted_data['grphoneNumber'] : NULL);
                $post_data['UserProfile']['created_by'] = $modelUser->created_by;
                $post_data['UserProfile']['updated_by'] = $modelUser->id;

                $user_id = $modelUser->saveUser($post_data, $modelUser->id);
            } else {    // create new user
                $next_user_id = User::find()->select('id')->orderBy(['id' => SORT_DESC])->asArray()->one()['id'];
                
                $post_data['User']['email'] = $posted_data['grEmail'];
                $post_data['User']['password_hash'] = Yii::$app->getSecurity()->generateRandomString(8);
                $post_data['User']['role_id'] = UserRole::getAttr(['id'], ['name' => 'user']);
                $post_data['User']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
                $post_data['User']['created_by'] = $next_user_id;
                $post_data['User']['updated_by'] = $next_user_id;

                $post_data['UserProfile']['first_name'] = $posted_data['first_name'];
                $post_data['UserProfile']['last_name'] = $posted_data['last_name'];
                $post_data['UserProfile']['gender'] = 'male';
                $post_data['UserProfile']['ba_address'] = ((isset($posted_data['grAddress']) && !empty($posted_data['grAddress'])) ? $posted_data['grAddress'] : '--');
                $post_data['UserProfile']['ba_city'] = ((isset($posted_data['grCity']) && !empty($posted_data['grCity'])) ? $posted_data['grCity'] : '--');
                $post_data['UserProfile']['ba_country'] = ((isset($posted_data['grCountry']) && !empty($posted_data['grCountry'])) ? $posted_data['grCountry'] : '--');
                $post_data['UserProfile']['ba_zip_code'] = ((isset($posted_data['grZipCode']) && !empty($posted_data['grZipCode'])) ? $posted_data['grZipCode'] : NULL);
                $post_data['UserProfile']['sa_address'] = ((isset($posted_data['grAddress']) && !empty($posted_data['grAddress'])) ? $posted_data['grAddress'] : '--');
                $post_data['UserProfile']['sa_city'] = ((isset($posted_data['grCity']) && !empty($posted_data['grCity'])) ? $posted_data['grCity'] : '--');
                $post_data['UserProfile']['sa_country'] = ((isset($posted_data['grCountry']) && !empty($posted_data['grCountry'])) ? $posted_data['grCountry'] : '--');
                $post_data['UserProfile']['sa_zip_code'] = ((isset($posted_data['grZipCode']) && !empty($posted_data['grZipCode'])) ? $posted_data['grZipCode'] : NULL);
                $post_data['UserProfile']['phone'] = ((isset($posted_data['grphoneNumber']) && !empty($posted_data['grphoneNumber'])) ? $posted_data['grphoneNumber'] : NULL);
                $post_data['UserProfile']['newsletters'] = 'no';
                $post_data['UserProfile']['created_by'] = $next_user_id;
                $post_data['UserProfile']['updated_by'] = $next_user_id;

                $user_id = $modelUser->saveUser($post_data);
            }

            // validating successful user creation
            if (!isset($user_id) || empty($user_id)) {
                throw new \yii\web\HttpException(422, 'User not created.');
            }

            $modelUserArr = User::find()->where(['id' => $user_id, 'status' => 'active'])->asArray()->one();

            $modelUserArr['password'] = $post_data['User']['password_hash'];

            // Create order
            $post_data['ShopOrder']['user_id'] = $user_id;
            $post_data['ShopOrder']['first_name'] = $posted_data['first_name'];
            $post_data['ShopOrder']['last_name'] = $posted_data['last_name'];
            $post_data['ShopOrder']['order_status_id'] = ShopOrderStatus::getAttr(['id'], ['name' => 'New']);
            $post_data['ShopOrder']['amount'] = (string) (isset($_SESSION['domain']['price']) ? $_SESSION['domain']['price'] : '0');
            $post_data['ShopOrder']['exchange_rate'] = '1.00';
            $post_data['ShopOrder']['invoice_number'] = Yii::$app->params['invoice_prefix'] . date('Ymd') . str_pad(mt_rand(0, 10000), 5, '0', STR_PAD_LEFT);
            $post_data['ShopOrder']['credit_note_number'] = NULL;
            $post_data['ShopOrder']['session_id'] = Yii::$app->session->getId();
            $post_data['ShopOrder']['order_date'] = date('Y-m-d');
            $post_data['ShopOrder']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
            $post_data['ShopOrder']['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['status'] = 'active';

            $post_data['ShopOrderItem'][0]['product_id'] = 1;
            $post_data['ShopOrderItem'][0]['product_name'] = $_SESSION['whois']['domain']['name'];
            $post_data['ShopOrderItem'][0]['cart_id'] = (isset($_SESSION['cart_id']) && !empty($_SESSION['cart_id'])) ? $_SESSION['cart_id'] : null;
            $post_data['ShopOrderItem'][0]['quantity'] = isset($_SESSION['domain']['validity']) ? $_SESSION['domain']['validity'] : 0;
            $post_data['ShopOrderItem'][0]['price'] = (string) (isset($_SESSION['domain']['price']) ? $_SESSION['domain']['price'] : '0');
            $post_data['ShopOrderItem'][0]['traits'] = isset($_SESSION['whois']) ? Json::encode(['whois_data' => $_SESSION['whois']]) : NULL;
            $post_data['ShopOrderItem'][0]['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrderItem'][0]['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrderItem'][0]['status'] = 'active';

            $order_id = $modelShopOrder->saveOrder($post_data);

            // validating successful order creation
            if (!isset($order_id) || empty($order_id)) {
                throw new \yii\web\HttpException(422, 'Order not created.');
            }

            $modelShopOrderArr = ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->asArray()->one();
            $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $order_id, 'status' => 'active'])->asArray()->one();

            // saving order id into session
            $_SESSION['order_id'] = $order_id;

            // Save fhtpay logs
            $modelPaymentLogs->order_id = $order_id;
            $modelPaymentLogs->payment_method_id = PaymentMethod::getAttr(['id'], ['name' => 'fhtpay']);
            $modelPaymentLogs->response = Json::encode($return_data);
            $modelPaymentLogs->created_by = $user_id;
            $modelPaymentLogs->updated_by = $user_id;

            if ($modelPaymentLogs->validate()) {
                $modelPaymentLogs->save();

                /*if (EmailMap::find()->where(['email' => $modelUser->email, 'status' => 'active'])->exists()) {
                    $modelEmailMap = EmailMap::find()->where(['email' => $modelUser->email, 'status' => 'active'])->one();

                    $modelEmailMap->user_id = $modelUser->id;

                    if ($modelEmailMap->validate()) {
                        $modelEmailMap->save();
                    }
                }*/

                $transaction->commit();

                // order confirmation email to customer
                Email::send(
                    'order-confirmation-customer', 
                    [
                        'html' => 'orderConfirmationCustomer-html', 
                        'text' => 'orderConfirmationCustomer-text'
                    ], 
                    [
                        'to' => $modelUserArr['email'], 
                        'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                    ], 
                    NULL, 
                    ['user_data' => [
                        'user' => $modelUserArr, 
                        'order' => $modelShopOrderArr, 
                        'order_item' => $modelShopOrderItemArr, 
                        'is_fallback' => ((isset($_SESSION['is_fallback']) && $_SESSION['is_fallback'] == 'true') ? true : false), 
                        'payment_gateway' => ((isset($_SESSION['is_fallback']) && $_SESSION['is_fallback'] == 'true') ? Yii::$app->params['payment_gateway_fallback'][$_SESSION['payment_gateway_fallback_index']] : Yii::$app->params['payment_gateway']), 
                    ]], 
                    Yii::$app->params['site_code'], 
                    Yii::$app->params['site_code']
                );

                return $order_id;
            } else {
                $transaction->rollBack();
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
