<?php

namespace common\modules\payment\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\payment\components\PaypalIPN;
use common\modules\payment\models\FormSquare;
use common\modules\payment\models\PaymentMethod;
use common\modules\payment\models\PaymentLogs;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreSite;
use common\modules\site\components\Email;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\user\models\UserRole;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopOrderStatus;
use common\modules\accounting\models\AccountingCurrency;
use common\modules\lms\components\LmsEvents;
use common\modules\lms\models\LmsCourse;

/**
 * Payment controller
 */
class PaypalController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['ipn'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['ipn'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [

                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    public function beforeAction($action) 
    {
        if (Yii::$app->controller->action->id == 'ipn') {
            $this->enableCsrfValidation = false;
        }

        return parent::beforeAction($action);
    }

    public function actionIpn()
    {
        $ipn = new PaypalIPN();
        $post_data = array();

        if (Yii::$app->params['mode'] == 'sandbox') {
            $ipn->useSandbox();
        }

        /* START: Log request params in file */
        if (isset($_POST, $_POST['txn_id'])) {
            $file = Yii::getAlias('@esslcerts_com') . '/logs/paypal/ipn/' . $_POST['txn_id'] . '.txt';
            $handle = fopen($file, 'w') or die('Cannot open file: ' . $file);

            fwrite($handle, print_r($_POST, true));

            fclose($handle);
        }
        /* END: Log request params in file */

        $verified = $ipn->verifyIPN();
        
        if ($verified) {
            // Process IPN
            // A list of variables are available here:
            // https://developer.paypal.com/webapps/developer/docs/classic/ipn/integration-guide/IPNandPDTVariables/

            // save user
            $modelUser = new User();

            if (User::find()->where(['email' => Yii::$app->request->post("payer_email"), 'status' => 'active'])->exists()) {
                $next_user_id = User::find()->select('id')->where(['email' => Yii::$app->request->post("payer_email"), 'status' => 'active'])->asArray()->one()['id'];
            } else {
                $next_user_id = User::find()->select('id')->orderBy(['id' => SORT_DESC])->asArray()->one()['id'];
            }
            
            $post_data['User']['email'] = Yii::$app->request->post("payer_email");
            $post_data['User']['password_hash'] = Yii::$app->request->post("payer_id");
            $post_data['User']['role_id'] = UserRole::getAttr(['id'], ['name' => 'user']);
            $post_data['User']['created_by'] = $next_user_id;
            $post_data['User']['updated_by'] = $next_user_id;
            $post_data['User']['site_id'] = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);

            $tmp = explode(' ', trim(Yii::$app->request->post("address_name")));

            $post_data['UserProfile']['first_name'] = $tmp[0];

            unset($tmp[0]);

            $post_data['UserProfile']['last_name'] = implode(' ', $tmp);
            $post_data['UserProfile']['gender'] = 'male';
            $post_data['UserProfile']['ba_address'] = Yii::$app->request->post("address_street") ? Yii::$app->request->post("address_street") : '--';
            $post_data['UserProfile']['ba_city'] = Yii::$app->request->post("address_city") ? Yii::$app->request->post("address_city") : '--';
            $post_data['UserProfile']['ba_country'] = Yii::$app->request->post("address_country") ? Yii::$app->request->post("address_country") : '--';
            $post_data['UserProfile']['ba_zip_code'] = Yii::$app->request->post("address_zip") ? Yii::$app->request->post("address_zip") : '--';
            $post_data['UserProfile']['sa_address'] = Yii::$app->request->post("address_street") ? Yii::$app->request->post("address_street") : '--';
            $post_data['UserProfile']['sa_city'] = Yii::$app->request->post("address_city") ? Yii::$app->request->post("address_city") : '--';
            $post_data['UserProfile']['sa_country'] = Yii::$app->request->post("address_country") ? Yii::$app->request->post("address_country") : '--';
            $post_data['UserProfile']['sa_zip_code'] = Yii::$app->request->post("address_zip") ? Yii::$app->request->post("address_zip") : '--';
            $post_data['UserProfile']['newsletters'] = 'yes';
            $post_data['UserProfile']['created_by'] = $next_user_id;
            $post_data['UserProfile']['updated_by'] = $next_user_id;

            if (User::find()->where(['email' => Yii::$app->request->post("payer_email"), 'status' => 'active'])->exists()) {
                $user_id = $modelUser->saveUser($post_data, User::find()->select('id')->where(['email' => Yii::$app->request->post("payer_email"), 'status' => 'active'])->asArray()->one()['id']);
            } else {
                $user_id = $modelUser->saveUser($post_data);
            }

            // save order
            $order_id = $this->saveOrder($_POST, $_POST, $user_id);

            // Reply with an empty 200 response to indicate to paypal the IPN was received correctly
            header("HTTP/1.1 200 OK");
        }
    }

    public function actionPay()
    {
        // save user
        $modelUser = new User();

        if (User::find()->where(['email' => Yii::$app->request->post("payer_email"), 'status' => 'active'])->exists()) {
            $next_user_id = User::find()->select('id')->where(['email' => Yii::$app->request->post("payer_email"), 'status' => 'active'])->asArray()->one()['id'];
        } else {
            $next_user_id = User::find()->select('id')->orderBy(['id' => SORT_DESC])->asArray()->one()['id'];
        }
        
        $post_data['User']['email'] = Yii::$app->request->post("payer_email");
        $post_data['User']['password_hash'] = Yii::$app->request->post("payer_id");
        $post_data['User']['role_id'] = UserRole::getAttr(['id'], ['name' => 'user']);
        $post_data['User']['created_by'] = $next_user_id;
        $post_data['User']['updated_by'] = $next_user_id;
        $post_data['User']['site_id'] = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);

        $tmp = explode(' ', trim(Yii::$app->request->post("address_name")));

        $post_data['UserProfile']['first_name'] = $tmp[0];

        unset($tmp[0]);

        $post_data['UserProfile']['last_name'] = implode(' ', $tmp);
        $post_data['UserProfile']['gender'] = 'male';
        $post_data['UserProfile']['ba_address'] = Yii::$app->request->post("address_street") ? Yii::$app->request->post("address_street") : '--';
        $post_data['UserProfile']['ba_city'] = Yii::$app->request->post("address_city") ? Yii::$app->request->post("address_city") : '--';
        $post_data['UserProfile']['ba_country'] = Yii::$app->request->post("address_country") ? Yii::$app->request->post("address_country") : '--';
        $post_data['UserProfile']['ba_zip_code'] = Yii::$app->request->post("address_zip") ? Yii::$app->request->post("address_zip") : '--';
        $post_data['UserProfile']['sa_address'] = Yii::$app->request->post("address_street") ? Yii::$app->request->post("address_street") : '--';
        $post_data['UserProfile']['sa_city'] = Yii::$app->request->post("address_city") ? Yii::$app->request->post("address_city") : '--';
        $post_data['UserProfile']['sa_country'] = Yii::$app->request->post("address_country") ? Yii::$app->request->post("address_country") : '--';
        $post_data['UserProfile']['sa_zip_code'] = Yii::$app->request->post("address_zip") ? Yii::$app->request->post("address_zip") : '--';
        $post_data['UserProfile']['newsletters'] = 'yes';
        $post_data['UserProfile']['created_by'] = $next_user_id;
        $post_data['UserProfile']['updated_by'] = $next_user_id;

        if (User::find()->where(['email' => Yii::$app->request->post("payer_email"), 'status' => 'active'])->exists()) {
            $user_id = $modelUser->saveUser($post_data, User::find()->select('id')->where(['email' => Yii::$app->request->post("payer_email"), 'status' => 'active'])->asArray()->one()['id']);
        } else {
            $user_id = $modelUser->saveUser($post_data);
        }

        // save order
        $this->saveOrder($_POST, json_decode($_POST['return_data']), $user_id);
    }

    public function saveOrder($form_data = array(), $return_data = array(), $user_id = null)
    {
        $session = Yii::$app->session->open();

        if ($form_data instanceof \yii\base\Event) {
            $return_data = $form_data->data->returnData;
            $form_data = $form_data->data->formData;
        }

        $modelUser = new User();
        $modelShopOrder = new ShopOrder();
        $modelPaymentLogs = new PaymentLogs();
        $post_data = array();
        $posted_data = array();
        $modelUserArr = array();
        $modelUserProfileArr = array();
        $modelShopOrderArr = array();
        $modelShopOrderItemArr = array();
        $flag_new_user = true;
        $last_order_id = 1;

        $posted_data = $form_data;

        // validating response from square
        if (!isset($return_data) || empty($return_data)) {
            throw new \yii\web\BadRequestHttpException('Empty response from payment gateway.');
        }

        // get last order id inserted (for invoice number)
        if (ShopOrder::find()->orderBy(['id' => SORT_DESC])->exists()) {
            $last_order = ShopOrder::find()->orderBy(['id' => SORT_DESC])->one();
            $last_order_id = $last_order->id;
        } else {
            $last_order_id = 1;
        }

        $transaction = Yii::$app->db->beginTransaction();

        try {
            // validating successful user creation
            if (!isset($user_id) || empty($user_id)) {
                throw new \yii\web\HttpException(422, 'User not created.');
            }

            $modelUserArr = User::find()->where(['id' => $user_id, 'status' => 'active'])->asArray()->one();
            $modelUserProfileArr = UserProfile::find()->where(['user_id' => $user_id, 'status' => 'active'])->asArray()->one();

            // Create order
            $post_data['ShopOrder']['user_id'] = $user_id;
            $post_data['ShopOrder']['first_name'] = $modelUserProfileArr['first_name'];
            $post_data['ShopOrder']['last_name'] = $modelUserProfileArr['last_name'];
            $post_data['ShopOrder']['order_status_id'] = ShopOrderStatus::getAttr(['id'], ['name' => 'New']);
            $post_data['ShopOrder']['voucher_id'] = (string) ((isset($_SESSION['cart']['voucher_id']) && !empty($_SESSION['cart']['voucher_id'])) ? $_SESSION['cart']['voucher_id'] : null);
            $post_data['ShopOrder']['amount'] = (string) ((isset($posted_data['mc_gross']) && !empty($posted_data['mc_gross'])) ? $posted_data['mc_gross'] : 0);
            $post_data['ShopOrder']['exchange_rate'] = AccountingCurrency::getAttr(['rate'], ['code' => 'usd']);
            $post_data['ShopOrder']['invoice_number'] = Yii::$app->params['invoice_prefix'] . date('Ymd') . str_pad(mt_rand(0, 10000), 5, '0', STR_PAD_LEFT);
            $post_data['ShopOrder']['credit_note_number'] = null;
            $post_data['ShopOrder']['session_id'] = Yii::$app->session->getId();
            $post_data['ShopOrder']['order_date'] = date('Y-m-d');
            $post_data['ShopOrder']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
            $post_data['ShopOrder']['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['status'] = 'active';

            $post_data['ShopOrderItem'][0]['product_id'] = ShopProduct::getAttr(['id'], ['price' => $post_data['ShopOrder']['amount']]);

            if (isset($posted_data['custom']) && !empty($posted_data['custom'])) {
                $post_data['ShopOrderItem'][0]['product_name'] = (string) $posted_data['custom'];
            }
            
            $post_data['ShopOrderItem'][0]['cart_id'] = (isset($_SESSION['cart']['db_sync']) && !empty($_SESSION['cart']['db_sync'])) ? $_SESSION['cart']['cart_id'] : null;
            $post_data['ShopOrderItem'][0]['quantity'] = 1;
            $post_data['ShopOrderItem'][0]['price'] = (string) $post_data['ShopOrder']['amount'];
            $post_data['ShopOrderItem'][0]['traits'] = (isset($_SESSION['whois']) && !empty($_SESSION['whois'])) ? (is_array($_SESSION['whois']) ? json_encode($_SESSION['whois']) : $_SESSION['whois']) : null;
            $post_data['ShopOrderItem'][0]['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrderItem'][0]['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrderItem'][0]['status'] = 'active';

            $order_id = $modelShopOrder->saveOrder($post_data);

            // validating successful order creation
            if (!isset($order_id) || empty($order_id)) {
                throw new \yii\web\HttpException(422, 'Order not created.');
            } else {
                // Create user-course subscription, in case of course order
                if (ShopProduct::getAttr(['code'], ['id' => 1]) == LmsCourse::getAttr(['code'], ['id' => 1])) {
                    $data['LmsUserCourse']['user_id'] = $user_id;
                    $data['LmsUserCourse']['course_id'] = LmsCourse::getAttr(['id'], ['code' => ShopProduct::getAttr(['code'], ['id' => $post_data['ShopOrderItem'][0]['product_id']])]);
                    $data['LmsUserCourse']['expiration_date'] = date('Y-m-d H:s:i', strtotime('+365 days'));
                    $data['LmsUserCourse']['valid'] = 'yes';

                    $eventLmsEvents = new LmsEvents;

                    $eventLmsEvents->eventModLmsCreateUserCourse($data);
                }
            }

            $modelShopOrderArr = ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->asArray()->one();
            $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $order_id, 'status' => 'active'])->asArray()->all();

            // saving order id into session
            $_SESSION['order_id'] = $order_id;

            // Save paypal logs
            $modelPaymentLogs->order_id = $order_id;
            $modelPaymentLogs->payment_method_id = PaymentMethod::getAttr(['id'], ['name' => 'paypal']);
            $modelPaymentLogs->response = \yii\helpers\Json::encode($return_data);
            $modelPaymentLogs->created_by = $user_id;
            $modelPaymentLogs->updated_by = $user_id;

            if ($modelPaymentLogs->validate()) {
                $modelPaymentLogs->save();

                $transaction->commit();

                // order confirmation email to customer
                Email::send(
                    'order-confirmation-customer', 
                    [
                        'html' => 'orderConfirmationCustomer-html', 
                        'text' => 'orderConfirmationCustomer-text'
                    ], 
                    [
                        'to' => $modelUserArr['email'], 
                        'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                    ], 
                    null, 
                    ['user_data' => [
                        'user' => $modelUserArr, 
                        'order' => $modelShopOrderArr, 
                        'order_item' => $modelShopOrderItemArr, 
                        'payment_gateway' => Yii::$app->params['payment_gateway'],
                        'is_fallback' => false,
                    ]], 
                    Yii::$app->params['site_code'], 
                    Yii::$app->params['site_code']
                );

                return $order_id;
            } else {
                $transaction->rollBack();
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function actionLog()
    {
        if (Yii::$app->request->isPost) {
            Yii::error('PayPal error dump: ' . Yii::$app->request->post('err'), Yii::$app->params['site_code']);

            return true;
        } else {
            return false;
        }
    }
}
