<?php

namespace common\modules\payment\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\helpers\Json;

use common\modules\payment\models\FormSquare;
use common\modules\payment\models\PaymentMethod;
use common\modules\payment\models\PaymentLogs;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreSite;
use common\modules\site\components\Email;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\user\models\UserRole;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopOrderStatus;
use common\modules\accounting\models\AccountingCurrency;
use common\modules\lms\components\LmsEvents;
use common\modules\lms\models\LmsCourse;

/**
 * Payment controller
 */
class SquareController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'pay'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'pay'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [

                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    public function beforeAction($action) 
    {
        if (Yii::$app->controller->action->id == 'pay') {
            $this->enableCsrfValidation = false;
        }

        return parent::beforeAction($action);
    }

    public function actionIndex($data)
    {
        $modelFormSquare = new FormSquare();

        $data = Json::decode($data);

        return $this->renderAjax('_form.php', [
            'modelFormSquare' => $modelFormSquare, 
            'amount' => (isset($data['amount']) ? $data['amount'] : 0), 
            'description' => (isset($data['description']) ? $data['description'] : ''), 
        ]);
    }

    public function actionPay($post = array(), $retry = false)
    {
        $session = Yii::$app->session->open();

        $_SESSION['gateway_call'] = 'true';
        
        $fullname = array();
        $first_name = '';
        $last_name = '';

        foreach ($_POST['FormSquare'] as $key => $value) {
            $_POST[$key] = $value;

            unset($_POST['FormSquare']);
        }

        if (!$retry) {
            $mandatory_keys = ['nonce', 'grPerName', 'grEmail', 'amount'];

            $_POST["description"] = '';
            $_POST["address_line1"] = '';
            $_POST["address_line2"] = '';
            $_POST["address_city"] = '';
            $_POST["address_state"] = '';
            $_POST["address_zip"] = '';
            $_POST["address_country"] = '';
        } else {
            $mandatory_keys = ['nonce', 'grPerName', 'grEmail', 'amount'];
        }
        
        if (!$_POST) {
            throw new \yii\web\BadRequestHttpException('Insufficient parameters to process payment.');
        }

        // validating mandatory fields
        foreach ($mandatory_keys as $mandatory_key) {
            if (!isset($_POST[$mandatory_key]) || empty($_POST[$mandatory_key])) {
                throw new \yii\web\BadRequestHttpException('Insufficient parameters to process payment.');
            }
        }

        $fullname = explode(' ', $_POST['grPerName']);

        if (count($fullname) < 2) { // last name not provided
            $first_name = $fullname[0];
            $last_name = '';
        } else {
            $first_name = $fullname[0];

            unset($fullname[0]);

            $last_name = implode(' ', $fullname);
        }

        $_POST['first_name'] = $first_name;
        $_POST['last_name'] = $last_name;

        // Initialize the authorization for Square
        \SquareConnect\Configuration::getDefaultConfiguration()->setAccessToken(Yii::$app->params['payment_gateways']['square'][Yii::$app->params['mode']]['access_token']);

        # Fail if the card form didn't send a value for `nonce` to the server
        $nonce = $_POST['nonce'];
        
        if (is_null($nonce)) {
            return $this->redirect('/index.php?r=shop/shop/thank-you&id=0&successful=false');
        }

        $transactions_api = new \SquareConnect\Api\TransactionsApi();

        # To learn more about splitting transactions with additional recipients,
        # see the Transactions API documentation on our [developer site]
        # (https://docs.connect.squareup.com/payments/transactions/overview#mpt-overview).
        $request_body = array (
            "card_nonce" => $nonce,
            # Monetary amounts are specified in the smallest unit of the applicable currency.
            # This amount is in cents. It's also hard-coded for $1.00, which isn't very useful.
            "amount_money" => array (
                "amount" => intval($_POST['amount'] * 100),
                "currency" => "USD"
            ),
            # Every payment you process with the SDK must have a unique idempotency key.
            # If you're unsure whether a particular payment succeeded, you can reattempt
            # it with the same idempotency key without worrying about double charging
            # the buyer.
            "idempotency_key" => uniqid()
        );

        # The SDK throws an exception if a Connect endpoint responds with anything besides
        # a 200-level HTTP code. This block catches any exceptions that occur from the request.
        try {
            $result = $transactions_api->charge(Yii::$app->params['payment_gateways']['square'][Yii::$app->params['mode']]['location_id'], $request_body);

            $result = json_decode($result->__toString(), true);

            if (isset($result['transaction']['id']) && !empty($result['transaction']['id'])) {
                $result = [
                    'trans_id' => $result['transaction']['id'], 
                    'response_code' => 'success',
                    'response' => $result, 
                ];
            } else {
                $result = [
                    'trans_id' => null, 
                    'response_code' => 'failure',
                    'response' => $result, 
                ];
            }
        } catch (\SquareConnect\ApiException $e) {
            $result = [
                'trans_id' => null, 
                'response_code' => 'failure',
                'response' => $e->getResponseBody(), 
            ];
        }

        if ($result['response_code'] != 'success') {
            return $this->redirect('/index.php?r=shop/shop/thank-you&id=0&successful=false');
        } else {
            if ($retry) {
                $_SESSION['is_fallback'] = 'true';
            }

            // save order et al.
            $order_id = $this->saveOrder($_POST, $result['response']);

            return $this->redirect('/index.php?r=shop/shop/thank-you&id=' . $order_id);
        }
    }

    public function saveOrder($form_data = array(), $return_data = array())
    {
        $session = Yii::$app->session->open();

        if ($form_data instanceof \yii\base\Event) {
            $return_data = $form_data->data->returnData;
            $form_data = $form_data->data->formData;
        }

        $modelUser = new User();
        $modelShopOrder = new ShopOrder();
        $modelPaymentLogs = new PaymentLogs();
        $post_data = array();
        $posted_data = array();
        $modelUserArr = array();
        $modelUserProfileArr = array();
        $modelShopOrderArr = array();
        $modelShopOrderItemArr = array();
        $flag_new_user = true;
        $last_order_id = 1;

        $posted_data = $form_data;

        // validating response from square
        if (!isset($return_data) || empty($return_data)) {
            throw new \yii\web\BadRequestHttpException('Empty response from payment gateway.');
        }

        // get last order id inserted (for invoice number)
        if (ShopOrder::find()->orderBy(['id' => SORT_DESC])->exists()) {
            $last_order = ShopOrder::find()->orderBy(['id' => SORT_DESC])->one();
            $last_order_id = $last_order->id;
        } else {
            $last_order_id = 1;
        }

        $transaction = Yii::$app->db->beginTransaction();

        try {
            $user_id = Yii::$app->user->identity->id;

            // validating successful user creation
            if (!isset($user_id) || empty($user_id)) {
                throw new \yii\web\HttpException(422, 'User not created.');
            }

            $modelUserArr = User::find()->where(['id' => $user_id, 'status' => 'active'])->asArray()->one();
            $modelUserProfileArr = UserProfile::find()->where(['user_id' => $user_id, 'status' => 'active'])->asArray()->one();

            // Create order
            $post_data['ShopOrder']['user_id'] = $user_id;
            $post_data['ShopOrder']['first_name'] = $modelUserProfileArr['first_name'];
            $post_data['ShopOrder']['last_name'] = $modelUserProfileArr['last_name'];
            $post_data['ShopOrder']['order_status_id'] = ShopOrderStatus::getAttr(['id'], ['name' => 'New']);
            $post_data['ShopOrder']['voucher_id'] = (string) ((isset($_SESSION['cart']['voucher_id']) && !empty($_SESSION['cart']['voucher_id'])) ? $_SESSION['cart']['voucher_id'] : null);
            $post_data['ShopOrder']['amount'] = (string) ((isset($_SESSION['cart']['cart_total']) && !empty($_SESSION['cart']['cart_total'])) ? $_SESSION['cart']['cart_total'] : 0);
            $post_data['ShopOrder']['exchange_rate'] = AccountingCurrency::getAttr(['rate'], ['code' => 'usd']);
            $post_data['ShopOrder']['invoice_number'] = Yii::$app->params['invoice_prefix'] . date('Ymd') . str_pad(mt_rand(0, 10000), 5, '0', STR_PAD_LEFT);
            $post_data['ShopOrder']['credit_note_number'] = null;
            $post_data['ShopOrder']['session_id'] = Yii::$app->session->getId();
            $post_data['ShopOrder']['order_date'] = date('Y-m-d');
            $post_data['ShopOrder']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
            $post_data['ShopOrder']['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['status'] = 'active';

            $post_data['ShopOrderItem'][0]['product_id'] = 1;
            $post_data['ShopOrderItem'][0]['cart_id'] = (isset($_SESSION['cart']['db_sync']) && !empty($_SESSION['cart']['db_sync'])) ? $_SESSION['cart']['cart_id'] : null;
            $post_data['ShopOrderItem'][0]['quantity'] = 365;
            $post_data['ShopOrderItem'][0]['price'] = (string) (ShopProduct::getAttr(['price'], ['id' => 1]) ? ShopProduct::getAttr(['price'], ['id' => 1]) : '0');
            $post_data['ShopOrderItem'][0]['traits'] = null;
            $post_data['ShopOrderItem'][0]['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrderItem'][0]['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrderItem'][0]['status'] = 'active';

            $order_id = $modelShopOrder->saveOrder($post_data);

            // validating successful order creation
            if (!isset($order_id) || empty($order_id)) {
                throw new \yii\web\HttpException(422, 'Order not created.');
            } else {
                // Create user-course subscription, in case of course order
                if (ShopProduct::getAttr(['code'], ['id' => 1]) == LmsCourse::getAttr(['code'], ['id' => 1])) {
                    $data['LmsUserCourse']['user_id'] = $user_id;
                    $data['LmsUserCourse']['course_id'] = LmsCourse::getAttr(['id'], ['code' => ShopProduct::getAttr(['code'], ['id' => $post_data['ShopOrderItem'][0]['product_id']])]);
                    $data['LmsUserCourse']['expiration_date'] = date('Y-m-d H:s:i', strtotime('+365 days'));
                    $data['LmsUserCourse']['valid'] = 'yes';

                    $eventLmsEvents = new LmsEvents;

                    $eventLmsEvents->eventModLmsCreateUserCourse($data);
                }
            }

            $modelShopOrderArr = ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->asArray()->one();
            $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $order_id, 'status' => 'active'])->asArray()->all();

            // saving order id into session
            $_SESSION['order_id'] = $order_id;

            // Save square logs
            $modelPaymentLogs->order_id = $order_id;
            $modelPaymentLogs->payment_method_id = PaymentMethod::getAttr(['id'], ['name' => 'square']);
            $modelPaymentLogs->response = Json::encode($return_data);
            $modelPaymentLogs->created_by = $user_id;
            $modelPaymentLogs->updated_by = $user_id;

            if ($modelPaymentLogs->validate()) {
                $modelPaymentLogs->save();

                $transaction->commit();

                // order confirmation email to customer
                Email::send(
                    'order-confirmation-customer', 
                    [
                        'html' => 'orderConfirmationCustomer-html', 
                        'text' => 'orderConfirmationCustomer-text'
                    ], 
                    [
                        'to' => $modelUserArr['email'], 
                        'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                    ], 
                    null, 
                    ['user_data' => [
                        'user' => $modelUserArr, 
                        'order' => $modelShopOrderArr, 
                        'order_item' => $modelShopOrderItemArr, 
                    ]], 
                    Yii::$app->params['site_code'], 
                    Yii::$app->params['site_code']
                );

                return $order_id;
            } else {
                $transaction->rollBack();
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
