<?php

namespace common\modules\payment\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\helpers\Json;
use linslin\yii2\curl;

use common\modules\payment\models\FormStripe;
use common\modules\payment\models\PaymentMethod;
use common\modules\payment\models\PaymentLogs;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreSite;
use common\modules\site\components\Email;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\user\models\UserRole;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopOrderStatus;
use common\modules\accounting\models\AccountingCurrency;

/**
 * Payment controller
 */
class StripeController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'pay'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'pay'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [

                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    public function actionIndex($data)
    {
        $modelFormStripe = new FormStripe();

        $data = Json::decode($data);

        return $this->renderAjax('_form.php', [
            'modelFormStripe' => $modelFormStripe, 
            'amount' => (isset($data['amount']) ? $data['amount'] : 0), 
            'description' => (isset($data['description']) ? $data['description'] : ''), 
        ]);
    }

    public function actionPay($ext_auth_key = '')
    {
        $session = Yii::$app->session->open();

        $_SESSION['gateway_call'] = 'true';

        if ($ext_auth_key == Yii::$app->params['payment_gateways'][Yii::$app->params['payment_gateway']][Yii::$app->params['mode']]['proxy']['auth_key']) {
            foreach ($_GET as $key => $value) {
                $_GET[$key] = urldecode($value);
            }

            $_POST['FormStripe'] = $_GET;
        }

        // flag to check the correct payment flow
        $_SESSION['mod_payment']['is_payment_initiated'] = 1;
        
        $fullname = array();
        $exp = array();
        $expMonth = '';
        $expYear = '';
        $first_name = '';
        $last_name = '';
        $mandatory_keys = ['cardHolderName', 'cardHolderEmail', 'cardNumber', 'cardExpiry', 'cardCVC', 'amount'];

        if (!isset($_POST['FormStripe'])) {
            throw new \yii\web\BadRequestHttpException('Insufficient parameters to process payment.');
        }

        // validating mandatory fields
        foreach ($mandatory_keys as $mandatory_key) {
            if (!isset($_POST['FormStripe'][$mandatory_key]) || empty($_POST['FormStripe'][$mandatory_key])) {
                throw new \yii\web\BadRequestHttpException('Insufficient parameters to process payment.');
            }
        }

        $fullname = explode(' ', $_POST['FormStripe']['cardHolderName']);

        if (count($fullname) < 2) {     // last name not provided
            $first_name = $fullname[0];
            $last_name = '--';
        } else {
            $first_name = $fullname[0];

            unset($fullname[0]);

            $last_name = implode(' ', $fullname);
        }

        $_POST['FormStripe']['first_name'] = $first_name;
        $_POST['FormStripe']['last_name'] = $last_name;
        $_POST['FormStripe']['email'] = $_POST['FormStripe']['cardHolderEmail'];

        // extracting month and year from card expiry date
        $exp = str_replace(' ', '', $_POST['FormStripe']['cardExpiry']);
        $exp = explode('/', $exp);
        $expMonth = $exp[0]; 
        $expYear = ((strlen($exp[1]) == 2) ? '20' . $exp[1] : $exp[1]);

        if (Yii::$app->params['payment_gateways'][Yii::$app->params['payment_gateway']][Yii::$app->params['mode']]['proxy']['enabled'] !== true) {
            \Stripe\Stripe::setApiKey(Yii::$app->params['payment_gateways'][Yii::$app->params['payment_gateway']][Yii::$app->params['mode']]['privateKey']);

            // generating stripe token
            $_POST['FormStripe']['stripeToken'] = \Stripe\Token::create(array(
                "card" => array(
                    "number" => $_POST['FormStripe']['cardNumber'], 
                    "exp_month" => $expMonth, 
                    "exp_year" => $expYear, 
                    "cvc" => $_POST['FormStripe']['cardCVC'], 
                    "name" => $_POST['FormStripe']['cardHolderName'], 
                    "address_line1" => (isset($_POST['FormStripe']['address']) ? $_POST['FormStripe']['address'] : ''), 
                    "address_line2" => (isset($_POST['FormStripe']['address_line2']) ? $_POST['FormStripe']['address_line2'] : ''), 
                    "address_city" => (isset($_POST['FormStripe']['city']) ? $_POST['FormStripe']['city'] : ''), 
                    "address_state" => (isset($_POST['FormStripe']['state']) ? $_POST['FormStripe']['state'] : ''), 
                    "address_zip" => (isset($_POST['FormStripe']['zip']) ? $_POST['FormStripe']['zip'] : ''), 
                    "address_country" => (isset($_POST['FormStripe']['country']) ? $_POST['FormStripe']['country'] : ''), 
                )
            ));

            // creating charge
            $charge = \Stripe\Charge::create(array(
                "amount" => intval($_POST['FormStripe']['amount'] * 100), 
                "currency" => 'usd', 
                "source" => $_POST['FormStripe']['stripeToken'], 
                "description" => (isset($_POST['FormStripe']['description']) ? $_POST['FormStripe']['description'] : ''), 
                "statement_descriptor" => ((isset($_POST['FormStripe']['statement_descriptor']) && !empty($_POST['FormStripe']['statement_descriptor'])) ? $_POST['FormStripe']['statement_descriptor'] : Globals::getMerchantDescriptor($_SESSION['whois']['domain']['name'])), 
            ));

            // fetching response from payment gateway
            $response = $charge->getLastResponse();

            if ($response->code != '200') {
                return Json::encode(['result' => 'failure', 'code' => $response->code, 'message' => 'Payment failed', 'payload' => []]);
            } else {
                if (Yii::$app->params['mod_payment']['save_order'] === true) {
                    // save order et al.
                    $tmp_order_id = $this->saveOrder($_POST['FormStripe'], $response->json);
                    
                    return Json::encode(['result' => 'success', 'code' => $response->code, 'message' => 'Payment successful', 'payload' => ['order_id' => $tmp_order_id]]);
                } else {
                    return Json::encode(['result' => 'success', 'code' => $response->code, 'message' => 'Payment successful', 'payload' => ['response' => $response->json]]);
                }
            }
        } else {
            $curl = new curl\Curl();

            $response = $curl->setGetParams([
                'ext_auth_key' => Yii::$app->params['payment_gateways'][Yii::$app->params['payment_gateway']][Yii::$app->params['mode']]['proxy']['auth_key'], 
                'cardHolderName' => urlencode($_POST['FormStripe']['cardHolderName']), 
                'cardHolderEmail' => urlencode($_POST['FormStripe']['cardHolderEmail']), 
                'cardNumber' => urlencode($_POST['FormStripe']['cardNumber']), 
                'cardExpiry' => urlencode($expMonth . '/' . $expYear), 
                'cardCVC' => urlencode($_POST['FormStripe']['cardCVC']), 
                'amount' => urlencode(intval($_POST['FormStripe']['amount'] * 100)), 
                'description' => urlencode(isset($_POST['FormStripe']['description']) ? $_POST['FormStripe']['description'] : ''), 
                'statement_descriptor' => urlencode((isset($_POST['FormStripe']['statement_descriptor']) && !empty($_POST['FormStripe']['statement_descriptor'])) ? $_POST['FormStripe']['statement_descriptor'] : Globals::getMerchantDescriptor($_SESSION['whois']['domain']['name'])), 
                'address_line1' => (isset($_POST['FormStripe']['address']) ? $_POST['FormStripe']['address'] : ''), 
                'address_line2' => (isset($_POST['FormStripe']['address_line2']) ? $_POST['FormStripe']['address_line2'] : ''), 
                'address_city' => (isset($_POST['FormStripe']['city']) ? $_POST['FormStripe']['city'] : ''), 
                'address_state' => (isset($_POST['FormStripe']['state']) ? $_POST['FormStripe']['state'] : ''), 
                'address_zip' => (isset($_POST['FormStripe']['zip']) ? $_POST['FormStripe']['zip'] : ''), 
                'address_country' => (isset($_POST['FormStripe']['country']) ? $_POST['FormStripe']['country'] : ''), 
            ])->get(Yii::$app->params['payment_gateways'][Yii::$app->params['payment_gateway']][Yii::$app->params['mode']]['proxy']['url']);

            $response = Json::decode($response, false);

            switch ($curl->responseCode) {
                case 200:
                    // success logic here
                    if ($response->code != '200') {
                        return Json::encode(['result' => 'failure', 'code' => $response->code, 'message' => 'Payment failed', 'payload' => []]);
                    } else {
                        // save order et al.
                        $tmp_order_id = $this->saveOrder($_POST['FormStripe'], $response->payload->response);

                        return Json::encode(['result' => 'success', 'code' => $response->code, 'message' => 'Payment successful', 'payload' => ['order_id' => $tmp_order_id]]);
                    }

                    break;

                default: 
                    return Json::encode(['result' => 'failure', 'code' => 500, 'message' => 'Payment failed', 'payload' => []]);

                    break;
            }
        }
    }

    public function saveOrder($form_data = array(), $return_data = array())
    {
        $session = Yii::$app->session->open();

        $modelUser = new User();
        $modelShopOrder = new ShopOrder();
        $modelPaymentLogs = new PaymentLogs();
        $post_data = array();
        $posted_data = array();
        $modelUserArr = array();
        $modelShopOrderArr = array();
        $last_order_id = 1;

        $posted_data = $form_data;

        // get last order id inserted (for invoice number)
        if (ShopOrder::find()->orderBy(['id' => SORT_DESC])->exists()) {
            $last_order = ShopOrder::find()->orderBy(['id' => SORT_DESC])->one();
            $last_order_id = $last_order->id;
        } else {
            $last_order_id = 1;
        }

        $transaction = Yii::$app->db->beginTransaction();

        try {
            // Create/Update User
            if (User::find()->where(['email' => $posted_data['email'], 'status' => 'active'])->exists()) {   // user already exists
                $modelUser = User::find()->where(['email' => $posted_data['email'], 'status' => 'active'])->one();

                $post_data['User'] = User::find()->where(['email' => $posted_data['email'], 'status' => 'active'])->asArray()->one();

                $post_data['User']['password_hash'] = Yii::$app->getSecurity()->generateRandomString(8);
                $post_data['User']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
                $post_data['User']['updated_by'] = $modelUser->id;

                $post_data['UserProfile'] = UserProfile::find()->where(['user_id' => $modelUser->id, 'status' => 'active'])->asArray()->one();

                $post_data['UserProfile']['first_name'] = $posted_data['first_name'];
                $post_data['UserProfile']['last_name'] = $posted_data['last_name'];
                $post_data['UserProfile']['ba_address'] = $post_data['UserProfile']['sa_address'] = isset($posted_data['address']) ? $posted_data['address'] : '--';
                $post_data['UserProfile']['ba_city'] = $post_data['UserProfile']['sa_city'] = isset($posted_data['city']) ? $posted_data['city'] : '--';
                $post_data['UserProfile']['ba_state'] = $post_data['UserProfile']['sa_state'] = isset($posted_data['state']) ? $posted_data['state'] : '--';
                $post_data['UserProfile']['ba_zip'] = $post_data['UserProfile']['sa_zip'] = isset($posted_data['zip']) ? $posted_data['zip'] : '--';
                $post_data['UserProfile']['ba_country'] = $post_data['UserProfile']['sa_country'] = isset($posted_data['country']) ? $posted_data['country'] : '--';
                $post_data['UserProfile']['updated_by'] = $modelUser->id;

                $user_id = $modelUser->saveUser($post_data, $modelUser->id);
            } else {    // create new user
                $next_user_id = User::find()->select('id')->orderBy(['id' => SORT_DESC])->asArray()->one()['id'];
                
                $post_data['User']['email'] = $posted_data['email'];
                $post_data['User']['password_hash'] = Yii::$app->getSecurity()->generateRandomString(8);
                $post_data['User']['role_id'] = UserRole::getAttr(['id'], ['name' => 'user']);
                $post_data['User']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
                $post_data['User']['created_by'] = $next_user_id;
                $post_data['User']['updated_by'] = $next_user_id;

                $post_data['UserProfile']['first_name'] = $posted_data['first_name'];
                $post_data['UserProfile']['last_name'] = $posted_data['last_name'];
                $post_data['UserProfile']['gender'] = 'male';
                $post_data['UserProfile']['ba_address'] = $post_data['UserProfile']['sa_address'] = isset($posted_data['address']) ? $posted_data['address'] : '--';
                $post_data['UserProfile']['ba_city'] = $post_data['UserProfile']['sa_city'] = isset($posted_data['city']) ? $posted_data['city'] : '--';
                $post_data['UserProfile']['ba_state'] = $post_data['UserProfile']['sa_state'] = isset($posted_data['state']) ? $posted_data['state'] : '--';
                $post_data['UserProfile']['ba_zip'] = $post_data['UserProfile']['sa_zip'] = isset($posted_data['zip']) ? $posted_data['zip'] : '--';
                $post_data['UserProfile']['ba_country'] = $post_data['UserProfile']['sa_country'] = isset($posted_data['country']) ? $posted_data['country'] : '--';
                $post_data['UserProfile']['newsletters'] = 'yes';
                $post_data['UserProfile']['created_by'] = $next_user_id;
                $post_data['UserProfile']['updated_by'] = $next_user_id;

                $user_id = $modelUser->saveUser($post_data);
            }

            // validating successful user creation
            if (!isset($user_id) || empty($user_id)) {
                throw new \yii\web\HttpException(422, 'User not created.');
            }

            $modelUserArr = User::find()->where(['id' => $user_id, 'status' => 'active'])->asArray()->one();
            $modelUserProfileArr = UserProfile::find()->where(['user_id' => $user_id, 'status' => 'active'])->asArray()->one();

            $modelUserArr['password'] = $post_data['User']['password_hash'];

            // Create order
            $post_data['ShopOrder']['user_id'] = $user_id;
            $post_data['ShopOrder']['first_name'] = $modelUserProfileArr['first_name'];
            $post_data['ShopOrder']['last_name'] = $modelUserProfileArr['last_name'];
            $post_data['ShopOrder']['order_status_id'] = ShopOrderStatus::getAttr(['id'], ['name' => 'New']);
            $post_data['ShopOrder']['voucher_id'] = (string) ((isset($_SESSION['cart']['voucher_id']) && !empty($_SESSION['cart']['voucher_id'])) ? $_SESSION['cart']['voucher_id'] : null);
            $post_data['ShopOrder']['amount'] = (string) ((isset($posted_data['amount']) && !empty($posted_data['amount'])) ? Globals::formatNumber($posted_data['amount'], $round = array('round' => false, 'precision' => 2)) : 0);
            $post_data['ShopOrder']['exchange_rate'] = AccountingCurrency::getAttr(['rate'], ['code' => 'usd']);
            $post_data['ShopOrder']['invoice_number'] = Yii::$app->params['invoice_prefix'] . date('Ymd') . str_pad(mt_rand(0, 10000), 5, '0', STR_PAD_LEFT);
            $post_data['ShopOrder']['credit_note_number'] = null;
            $post_data['ShopOrder']['session_id'] = Yii::$app->session->getId();
            $post_data['ShopOrder']['order_date'] = date('Y-m-d');
            $post_data['ShopOrder']['site_id'] = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
            $post_data['ShopOrder']['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrder']['status'] = 'active';

            $post_data['ShopOrderItem'][0]['product_id'] = (int) ShopProduct::getAttr(['id'], ['price' => Globals::formatNumber($post_data['ShopOrder']['amount'], $round = array('round' => false, 'precision' => 2)), 'available' => 'yes']);

            if (isset($_SESSION['whois']['domain']['name']) && !empty($_SESSION['whois']['domain']['name'])) {
                $post_data['ShopOrderItem'][0]['product_name'] = (string) $_SESSION['whois']['domain']['name'];
            }
            
            $post_data['ShopOrderItem'][0]['cart_id'] = (isset($_SESSION['cart']['db_sync']) && !empty($_SESSION['cart']['db_sync'])) ? $_SESSION['cart']['cart_id'] : null;
            $post_data['ShopOrderItem'][0]['price'] = (string) Globals::formatNumber($post_data['ShopOrder']['amount'], $round = array('round' => false, 'precision' => 2));

            $validity = array_flip(Yii::$app->params['prices']);

            $post_data['ShopOrderItem'][0]['quantity'] = $validity[$post_data['ShopOrderItem'][0]['price']];
            $post_data['ShopOrderItem'][0]['traits'] = (isset($_SESSION['whois']) && !empty($_SESSION['whois'])) ? json_encode($_SESSION['whois']) : null;
            $post_data['ShopOrderItem'][0]['created_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrderItem'][0]['updated_by'] = isset($user_id) ? $user_id : 1;
            $post_data['ShopOrderItem'][0]['status'] = 'active';

            $order_id = $modelShopOrder->saveOrder($post_data);

            // validating successful order creation
            if (!isset($order_id) || empty($order_id)) {
                throw new \yii\web\HttpException(422, 'Order not created.');
            }

            $modelShopOrderArr = ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->asArray()->one();
            $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $order_id, 'status' => 'active'])->asArray()->all();

            // saving order id into session
            $_SESSION['order_id'] = $order_id;

            // Save stripe logs
            $modelPaymentLogs->order_id = $order_id;
            $modelPaymentLogs->payment_method_id = PaymentMethod::getAttr(['id'], ['name' => 'stripe']);
            $modelPaymentLogs->response = Json::encode($return_data);
            $modelPaymentLogs->created_by = $user_id;
            $modelPaymentLogs->updated_by = $user_id;

            if ($modelPaymentLogs->validate()) {
                $modelPaymentLogs->save();

                $transaction->commit();

                // order confirmation email to customer
                Email::send(
                    'order-confirmation-customer', 
                    [
                        'html' => 'orderConfirmationCustomer-html', 
                        'text' => 'orderConfirmationCustomer-text'
                    ], 
                    [
                        'to' => $modelUserArr['email'], 
                        'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                    ], 
                    null, 
                    ['user_data' => [
                        'user' => $modelUserArr, 
                        'order' => $modelShopOrderArr, 
                        'order_item' => $modelShopOrderItemArr, 
                        'payment_gateway' => Yii::$app->params['payment_gateway'],
                        'is_fallback' => false,
                    ]], 
                    Yii::$app->params['site_code'], 
                    Yii::$app->params['site_code']
                );

                return $order_id;
            } else {
                $transaction->rollBack();
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
