<?php

namespace common\modules\payment\models;

use Yii;
use yii\db\Expression;
use Yadakhov\Json;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%payment_logs}}".
 *
 * @property int $id
 * @property int $order_id
 * @property int $payment_method_id
 * @property string $response
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 *
 * @property ShopOrder $order
 * @property User $createdBy
 * @property User $updatedBy
 * @property PaymentMethod $paymentMethod
 */
class PaymentLogs extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%payment_logs}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['order_id', 'payment_method_id'], 'required'],
            [['order_id', 'payment_method_id', 'created_by', 'updated_by'], 'integer'],
            [['response'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['order_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopOrder::className(), 'targetAttribute' => ['order_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['payment_method_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\payment\models\PaymentMethod::className(), 'targetAttribute' => ['payment_method_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'order_id' => 'Order ID',
            'payment_method_id' => 'Payment Method ID',
            'response' => 'Response',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrder()
    {
        return $this->hasOne(\common\modules\shop\models\ShopOrder::className(), ['id' => 'order_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPaymentMethod()
    {
        return $this->hasOne(\common\modules\payment\models\PaymentMethod::className(), ['id' => 'payment_method_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function behaviors()
    {
        return [
            [
                'class' => \yii\behaviors\TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
            [
                'class' => \yii\behaviors\BlameableBehavior::class,
                'createdByAttribute' => 'created_by',
                'updatedByAttribute' => 'updated_by',
                'defaultValue' => 1
            ],
        ];
    }

    public static function getTransactionId($order_id, $payment_gateway)
    {
        if (!isset($order_id, $payment_gateway) || empty($order_id) || empty($payment_gateway)) {
            return false;
        }

        $txn_id = '';

        $model = self::find()->where(['order_id' => $order_id])->one();

        switch ($payment_gateway) {
            case 'paypal':
                $json_response = (isset($model->response) && !empty($model->response)) ? new Json($model->response) : new Json('{}');

                $txn_id = $json_response->get('purchase_units.0.payments.captures.0.id');
                
                break;

            case 'stripe':
                $json_response = (isset($model->response) && !empty($model->response)) ? new Json($model->response) : new Json('{}');

                $txn_id = $json_response->get('id');
                
                break;

            case 'square':
                $json_response = (isset($model->response) && !empty($model->response)) ? new Json($model->response) : new Json('{}');

                $txn_id = $json_response->get('transaction.id');
                
                break;

            case 'authorizenet':
                $json_response = (isset($model->response) && !empty($model->response)) ? new Json($model->response) : new Json('{}');

                $txn_id = $json_response->get('trans_id');
                
                break;

            case 'fhtpay':
                $json_response = (isset($model->response) && !empty($model->response)) ? new Json($model->response) : new Json('{}');

                $txn_id = $json_response->get('tradeNo');
                
                break;
            
            default:
                $json_response = (isset($model->response) && !empty($model->response)) ? new Json($model->response) : new Json('{}');

                $txn_id = $json_response->get('transaction_id');

                break;
        }

        return $txn_id;
    }
}
