<?php

namespace common\modules\payment\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\payment\models\PaymentMethod;

/**
 * PaymentMethodSearch represents the model behind the search form of `common\modules\payment\models\PaymentMethod`.
 */
class PaymentMethodSearch extends PaymentMethod
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'display_name', 'description', 'url', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = PaymentMethod::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_payment_method.name', $this->name])
            ->andFilterWhere(['like', 'mod_payment_method.display_name', $this->display_name])
            ->andFilterWhere(['like', 'mod_payment_method.description', $this->description])
            ->andFilterWhere(['like', 'mod_payment_method.url', $this->url])
            ->andFilterWhere(['like', 'mod_payment_method.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_payment_method.status', $this->status])
            ->andFilterWhere(['like', 'mod_payment_method.id', $this->id])
            ->andFilterWhere(['like', 'mod_payment_method.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_payment_method.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_payment_method.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_payment_method.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
