<?php

namespace common\modules\payment\models;

use Yii;
use yii\db\Expression;
use yii\behaviors\TimestampBehavior;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%payment_rebill_data}}".
 *
 * @property int $id
 * @property int $user_id
 * @property string $email
 * @property string $card_name
 * @property string $card_number
 * @property string $card_expiry_month
 * @property string $card_expiry_year
 * @property string $card_cvc
 * @property string $source
 * @property string $metadata
 * @property int $payment_method_id
 * @property int $count
 * @property string $queue
 * @property string $chargeback
 * @property string $valid
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $user
 * @property PaymentMethod $paymentMethod
 */
class PaymentRebillData extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%payment_rebill_data}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'payment_method_id', 'count', 'created_by', 'updated_by'], 'integer'],
            [['card_name', 'card_number', 'card_expiry_month', 'card_expiry_year', 'card_cvc'], 'required'],
            [['metadata', 'queue', 'chargeback', 'valid', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['email', 'card_name', 'card_number', 'card_expiry_month', 'card_expiry_year', 'card_cvc', 'source'], 'string', 'max' => 255],
            [['card_number'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['payment_method_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\payment\models\PaymentMethod::className(), 'targetAttribute' => ['payment_method_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'email' => 'Email',
            'card_name' => 'Card Name',
            'card_number' => 'Card Number',
            'card_expiry_month' => 'Card Expiry Month',
            'card_expiry_year' => 'Card Expiry Year',
            'card_cvc' => 'Card Cvc',
            'source' => 'Source',
            'metadata' => 'Metadata',
            'payment_method_id' => 'Payment Method ID',
            'count' => 'Count',
            'queue' => 'Queue',
            'chargeback' => 'Chargeback',
            'valid' => 'Valid',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPaymentMethod()
    {
        return $this->hasOne(\common\modules\payment\models\PaymentMethod::className(), ['id' => 'payment_method_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function behaviors()
    {
        return [
            [
                'class' => TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }
}
