<?php

namespace common\modules\qms\controllers\backend;

use Yii;
use common\modules\qms\models\QmsDepartment;
use common\modules\qms\models\QmsDepartmentSearch;
use common\modules\assetManager\models\AssetManager;
use common\modules\cms\models\CmsEntityAsset;
use common\modules\core\models\FormFileUpload;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use yii\web\UploadedFile;

/**
 * DepartmentController implements the CRUD actions for QmsDepartment model.
 */
class DepartmentController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => ['@'],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all QmsDepartment models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new QmsDepartmentSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single QmsDepartment model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new QmsDepartment model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $session = Yii::$app->session;

        $model = new QmsDepartment();
        $modelFormFileUpload = new FormFileUpload();

        if ($model->load(Yii::$app->request->post())) {
            $modelFormFileUpload->file = UploadedFile::getInstance($modelFormFileUpload, 'file');
            
            if (!empty($modelFormFileUpload->file)) {
                echo "hello";die;
                $filename = $modelFormFileUpload->file->baseName . '.' . $modelFormFileUpload->file->extension;
                
                if ($tempRelativePath = $modelFormFileUpload->upload('contact', $modelFormFileUpload->file->baseName . '.' . $modelFormFileUpload->file->extension)) {
                    $transaction = Yii::$app->db->beginTransaction();

                    try {
                        if (!$session->has('uploadedFileName')) {
                            $transaction->rollBack();

                            throw new HttpException(500, 'Unable to save file.');
                        }

                        if (!file_exists(Yii::getAlias('@assets') . '/data/' . $filename)) {
                            throw new NotFoundHttpException('File does not exist: ' . Yii::getAlias('@assets') . '/data/' . $filename);
                        }
                    } catch (\Exception $e) {
                        $transaction->rollBack();

                        throw $e;
                    }
                }
            }
            if ($department_id = $model->saveDepartment(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $department_id]);
            }
        }
        // if ($model->load(Yii::$app->request->post()) && $model->save()) {
        //     return $this->redirect(['view', 'id' => $model->id]);
        // } 
        else {
            return $this->render('create', [
                'model' => $model,
                'modelFormFileUpload' => $modelFormFileUpload,
            ]);
        }
    }

    /**
     * Updates an existing QmsDepartment model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $session = Yii::$app->session;

        $modelFormFileUpload = new FormFileUpload();
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
            'modelFormFileUpload' => $modelFormFileUpload,
        ]);
    }

    /**
     * Deletes an existing QmsDepartment model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the QmsDepartment model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return QmsDepartment the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = QmsDepartment::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
