<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_asset}}".
 *
 * @property integer $id
 * @property integer $asset_type_id
 * @property string $date_purchased
 * @property string $description
 * @property integer $supplier_id
 * @property string $allocated_to
 * @property string $estimated_life
 * @property string $cost
 * @property string $asset_id
 * @property string $remarks
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterAssetType $assetType
 * @property QmsSupplier $supplier
 */
class QmsAsset extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_asset}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['asset_type_id', 'date_purchased', 'description', 'cost', 'asset_id'], 'required'],
            [['asset_type_id', 'supplier_id', 'created_by', 'updated_by'], 'integer'],
            [['date_purchased'], 'safe'],
            [['description', 'status'], 'string'],
            [['allocated_to', 'estimated_life', 'cost', 'asset_id', 'remarks'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['asset_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterAssetType::className(), 'targetAttribute' => ['asset_type_id' => 'id']],
            [['supplier_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsSupplier::className(), 'targetAttribute' => ['supplier_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'asset_type_id' => 'Asset Type ID',
            'date_purchased' => 'Date Purchased',
            'description' => 'Description',
            'supplier_id' => 'Supplier ID',
            'allocated_to' => 'Allocated To',
            'estimated_life' => 'Estimated Life',
            'cost' => 'Cost',
            'asset_id' => 'Asset ID',
            'remarks' => 'Remarks',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAssetType()
    {
        return $this->hasOne(QmsMasterAssetType::className(), ['id' => 'asset_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSupplier()
    {
        return $this->hasOne(QmsSupplier::className(), ['id' => 'supplier_id']);
    }
}
