<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsAsset;

/**
 * QmsAssetSearch represents the model behind the search form of `common\modules\qms\models\QmsAsset`.
 */
class QmsAssetSearch extends QmsAsset
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'asset_type_id', 'supplier_id', 'created_by', 'updated_by'], 'integer'],
            [['date_purchased', 'description', 'allocated_to', 'estimated_life', 'cost', 'asset_id', 'remarks', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsAsset::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'asset_type_id' => $this->asset_type_id,
            'date_purchased' => $this->date_purchased,
            'supplier_id' => $this->supplier_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'description', $this->description])
            ->andFilterWhere(['like', 'allocated_to', $this->allocated_to])
            ->andFilterWhere(['like', 'estimated_life', $this->estimated_life])
            ->andFilterWhere(['like', 'cost', $this->cost])
            ->andFilterWhere(['like', 'asset_id', $this->asset_id])
            ->andFilterWhere(['like', 'remarks', $this->remarks])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
