<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_breakdown_record}}".
 *
 * @property integer $id
 * @property integer $equipment_id
 * @property string $breakdown_date
 * @property string $breakdown_details
 * @property integer $breakdown_category_id
 * @property integer $department_id
 * @property integer $assigned_to_id
 * @property integer $breakdown_hour
 * @property string $date_of_breakdown_rectification
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsEquipment $equipment
 * @property QmsMasterBreakdownCategory $breakdownCategory
 * @property QmsDepartment $department
 * @property User $assignedTo
 */
class QmsBreakdownRecord extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_breakdown_record}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['equipment_id', 'breakdown_date'], 'required'],
            [['equipment_id', 'breakdown_category_id', 'department_id', 'assigned_to_id', 'breakdown_hour', 'created_by', 'updated_by'], 'integer'],
            [['breakdown_date', 'date_of_breakdown_rectification'], 'safe'],
            [['breakdown_details', 'status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['equipment_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsEquipment::className(), 'targetAttribute' => ['equipment_id' => 'id']],
            [['breakdown_category_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterBreakdownCategory::className(), 'targetAttribute' => ['breakdown_category_id' => 'id']],
            [['department_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsDepartment::className(), 'targetAttribute' => ['department_id' => 'id']],
            [['assigned_to_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['assigned_to_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'equipment_id' => 'Equipment ID',
            'breakdown_date' => 'Breakdown Date',
            'breakdown_details' => 'Breakdown Details',
            'breakdown_category_id' => 'Breakdown Category ID',
            'department_id' => 'Department ID',
            'assigned_to_id' => 'Assigned To ID',
            'breakdown_hour' => 'Breakdown Hour',
            'date_of_breakdown_rectification' => 'Date Of Breakdown Rectification',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEquipment()
    {
        return $this->hasOne(QmsEquipment::className(), ['id' => 'equipment_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBreakdownCategory()
    {
        return $this->hasOne(QmsMasterBreakdownCategory::className(), ['id' => 'breakdown_category_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartment()
    {
        return $this->hasOne(QmsDepartment::className(), ['id' => 'department_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAssignedTo()
    {
        return $this->hasOne(User::className(), ['id' => 'assigned_to_id']);
    }
}
