<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsBreakdownRecord;

/**
 * QmsBreakdownRecordSearch represents the model behind the search form of `common\modules\qms\models\QmsBreakdownRecord`.
 */
class QmsBreakdownRecordSearch extends QmsBreakdownRecord
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'equipment_id', 'breakdown_category_id', 'department_id', 'assigned_to_id', 'breakdown_hour', 'created_by', 'updated_by'], 'integer'],
            [['breakdown_date', 'breakdown_details', 'date_of_breakdown_rectification', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsBreakdownRecord::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'equipment_id' => $this->equipment_id,
            'breakdown_date' => $this->breakdown_date,
            'breakdown_category_id' => $this->breakdown_category_id,
            'department_id' => $this->department_id,
            'assigned_to_id' => $this->assigned_to_id,
            'breakdown_hour' => $this->breakdown_hour,
            'date_of_breakdown_rectification' => $this->date_of_breakdown_rectification,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'breakdown_details', $this->breakdown_details])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
