<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_compliance_obligation}}".
 *
 * @property integer $id
 * @property integer $compliance_obligation_type_id
 * @property string $compliance_obligation_source_type
 * @property string $compliance_obligation
 * @property string $applicable_section_rule
 * @property string $consequences_of_non_compliance
 * @property string $frequency
 * @property string $current_status
 * @property integer $responsibility_id
 * @property string $compliance_obligation_proof
 * @property string $compliance_obligation_document_name
 * @property string $compliance_obligation_document_number
 * @property string $evaluation_of_compliance_obligation
 * @property string $summary
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterComplianceObligationType $complianceObligationType
 * @property User $responsibility
 * @property User $approvingAuthority
 */
class QmsComplianceObligation extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_compliance_obligation}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['compliance_obligation_type_id', 'compliance_obligation'], 'required'],
            [['compliance_obligation_type_id', 'responsibility_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['compliance_obligation_source_type', 'compliance_obligation', 'applicable_section_rule', 'consequences_of_non_compliance', 'frequency', 'current_status', 'compliance_obligation_proof', 'compliance_obligation_document_name', 'evaluation_of_compliance_obligation', 'summary', 'approval_status', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['compliance_obligation_document_number'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['compliance_obligation_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterComplianceObligationType::className(), 'targetAttribute' => ['compliance_obligation_type_id' => 'id']],
            [['responsibility_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['responsibility_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'compliance_obligation_type_id' => 'Compliance Obligation Type ID',
            'compliance_obligation_source_type' => 'Compliance Obligation Source Type',
            'compliance_obligation' => 'Compliance Obligation',
            'applicable_section_rule' => 'Applicable Section Rule',
            'consequences_of_non_compliance' => 'Consequences Of Non Compliance',
            'frequency' => 'Frequency',
            'current_status' => 'Current Status',
            'responsibility_id' => 'Responsibility ID',
            'compliance_obligation_proof' => 'Compliance Obligation Proof',
            'compliance_obligation_document_name' => 'Compliance Obligation Document Name',
            'compliance_obligation_document_number' => 'Compliance Obligation Document Number',
            'evaluation_of_compliance_obligation' => 'Evaluation Of Compliance Obligation',
            'summary' => 'Summary',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getComplianceObligationType()
    {
        return $this->hasOne(QmsMasterComplianceObligationType::className(), ['id' => 'compliance_obligation_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getResponsibility()
    {
        return $this->hasOne(User::className(), ['id' => 'responsibility_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
