<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsComplianceObligation;

/**
 * QmsComplianceObligationSearch represents the model behind the search form of `common\modules\qms\models\QmsComplianceObligation`.
 */
class QmsComplianceObligationSearch extends QmsComplianceObligation
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'compliance_obligation_type_id', 'responsibility_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['compliance_obligation_source_type', 'compliance_obligation', 'applicable_section_rule', 'consequences_of_non_compliance', 'frequency', 'current_status', 'compliance_obligation_proof', 'compliance_obligation_document_name', 'compliance_obligation_document_number', 'evaluation_of_compliance_obligation', 'summary', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsComplianceObligation::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'compliance_obligation_type_id' => $this->compliance_obligation_type_id,
            'responsibility_id' => $this->responsibility_id,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'compliance_obligation_source_type', $this->compliance_obligation_source_type])
            ->andFilterWhere(['like', 'compliance_obligation', $this->compliance_obligation])
            ->andFilterWhere(['like', 'applicable_section_rule', $this->applicable_section_rule])
            ->andFilterWhere(['like', 'consequences_of_non_compliance', $this->consequences_of_non_compliance])
            ->andFilterWhere(['like', 'frequency', $this->frequency])
            ->andFilterWhere(['like', 'current_status', $this->current_status])
            ->andFilterWhere(['like', 'compliance_obligation_proof', $this->compliance_obligation_proof])
            ->andFilterWhere(['like', 'compliance_obligation_document_name', $this->compliance_obligation_document_name])
            ->andFilterWhere(['like', 'compliance_obligation_document_number', $this->compliance_obligation_document_number])
            ->andFilterWhere(['like', 'evaluation_of_compliance_obligation', $this->evaluation_of_compliance_obligation])
            ->andFilterWhere(['like', 'summary', $this->summary])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
