<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsContextOrganisation;

/**
 * QmsContextOrganisationSearch represents the model behind the search form of `common\modules\qms\models\QmsContextOrganisation`.
 */
class QmsContextOrganisationSearch extends QmsContextOrganisation
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'department_id', 'site_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['where_do_we_aim_to_be', 'what_do_we_do_what_makes_us_different', 'why_this_business_is_good_business', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsContextOrganisation::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'department_id' => $this->department_id,
            'site_id' => $this->site_id,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'where_do_we_aim_to_be', $this->where_do_we_aim_to_be])
            ->andFilterWhere(['like', 'what_do_we_do_what_makes_us_different', $this->what_do_we_do_what_makes_us_different])
            ->andFilterWhere(['like', 'why_this_business_is_good_business', $this->why_this_business_is_good_business])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
