<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;
use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%qms_department}}".
 *
 * @property integer $id
 * @property string $name
 * @property string $profile
 * @property integer $department_hod
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $departmentHod
 */
class QmsDepartment extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_department}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['name'], 'required'],
            [['profile', 'status'], 'string'],
            [['department_hod', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['name'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['department_hod'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['department_hod' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'profile' => 'Profile',
            'department_hod' => 'Department Hod',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartmentHod()
    {
        return $this->hasOne(User::className(), ['id' => 'department_hod']);
    }

    /**
    * To Delete the department details
    **/

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveDepartment($post_data, $department_id = 0)
    {   
        $session = Yii::$app->session;

        $modelDepartment = new QmsDepartment();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();
        
        try {
            if (isset($department_id) && $department_id > 0 && QmsDepartment::find()->where(['id' => $department_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelDepartment = QmsDepartment::findOne(['id' => $product_id, 'status' => 'active']);

                $modelDepartment->name = $post_data['QmsDepartment']['name'];
                $modelDepartment->profile = $post_data['QmsDepartment']['profile'];
                $modelDepartment->department_hod = $post_data['QmsDepartment']['department_hod'];
            } else {    // Create a new model
                $modelDepartment = new QmsDepartment();

                $modelDepartment->name = $post_data['QmsDepartment']['name'];
                $modelDepartment->profile = $post_data['QmsDepartment']['profile'];
                $modelDepartment->department_hod = $post_data['QmsDepartment']['department_hod'];
                $modelDepartment->status = 'active';
            }

            if ($modelDepartment->validate()) {
                $modelDepartment->save();
                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Department saved successfully.");

                    return $modelDepartment->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save Department " . Globals::renderErrors($modelDepartment->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save Department: " . Globals::renderErrors($modelDepartment->getErrors()));

                return false;
            }

        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
