<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_design_development}}".
 *
 * @property integer $id
 * @property integer $product_service_project_id
 * @property string $brief_description
 * @property string $scope_of_work
 * @property string $target_date
 * @property string $project_manager
 * @property string $project_manager_email
 * @property string $team_members
 * @property string $team_members_email
 * @property string $planning_date
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsProductServiceProject $productServiceProject
 * @property QmsDesignDevelopmentChanges[] $qmsDesignDevelopmentChanges
 */
class QmsDesignDevelopment extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_design_development}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['product_service_project_id', 'brief_description', 'scope_of_work', 'target_date', 'project_manager', 'team_members', 'planning_date'], 'required'],
            [['product_service_project_id', 'created_by', 'updated_by'], 'integer'],
            [['brief_description', 'scope_of_work', 'team_members', 'team_members_email', 'status'], 'string'],
            [['target_date', 'planning_date'], 'safe'],
            [['project_manager', 'project_manager_email'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['product_service_project_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsProductServiceProject::className(), 'targetAttribute' => ['product_service_project_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'product_service_project_id' => 'Product Service Project ID',
            'brief_description' => 'Brief Description',
            'scope_of_work' => 'Scope Of Work',
            'target_date' => 'Target Date',
            'project_manager' => 'Project Manager',
            'project_manager_email' => 'Project Manager Email',
            'team_members' => 'Team Members',
            'team_members_email' => 'Team Members Email',
            'planning_date' => 'Planning Date',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProductServiceProject()
    {
        return $this->hasOne(QmsProductServiceProject::className(), ['id' => 'product_service_project_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsDesignDevelopmentChanges()
    {
        return $this->hasMany(QmsDesignDevelopmentChanges::className(), ['design_and_development_change_details_id' => 'id']);
    }
}
