<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_documents}}".
 *
 * @property integer $id
 * @property integer $management_system_type_id
 * @property integer $department_id
 * @property string $document_name
 * @property string $document_number
 * @property string $revision_number
 * @property string $revision_date
 * @property string $document_category
 * @property string $document_type
 * @property string $document
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterManagementSystemType $managementSystemType
 * @property QmsDepartment $department
 * @property User $approvingAuthority
 */
class QmsDocuments extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_documents}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['management_system_type_id', 'department_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['department_id', 'document_name', 'revision_number'], 'required'],
            [['revision_date'], 'safe'],
            [['document_category', 'document_type', 'document', 'approval_status', 'status'], 'string'],
            [['document_name', 'document_number', 'revision_number'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
            [['department_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsDepartment::className(), 'targetAttribute' => ['department_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'management_system_type_id' => 'Management System Type ID',
            'department_id' => 'Department ID',
            'document_name' => 'Document Name',
            'document_number' => 'Document Number',
            'revision_number' => 'Revision Number',
            'revision_date' => 'Revision Date',
            'document_category' => 'Document Category',
            'document_type' => 'Document Type',
            'document' => 'Document',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartment()
    {
        return $this->hasOne(QmsDepartment::className(), ['id' => 'department_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
