<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_enquiry}}".
 *
 * @property integer $id
 * @property string $enquiry_date
 * @property string $details
 * @property string $enquiry_status
 * @property string $prospect_company
 * @property string $prospect_name
 * @property string $address
 * @property string $city
 * @property string $zip
 * @property string $phone
 * @property string $email
 * @property integer $country_id
 * @property string $website
 * @property string $remarks
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterCountry $country
 * @property QmsEnquiryFollowUp[] $qmsEnquiryFollowUps
 * @property QmsEnquiryInternalPurpose[] $qmsEnquiryInternalPurposes
 */
class QmsEnquiry extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_enquiry}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['enquiry_date', 'details', 'prospect_company', 'prospect_name', 'phone', 'email'], 'required'],
            [['enquiry_date'], 'safe'],
            [['details', 'address', 'remarks', 'status'], 'string'],
            [['country_id', 'created_by', 'updated_by'], 'integer'],
            [['enquiry_status', 'prospect_company', 'prospect_name', 'city', 'zip', 'phone', 'email', 'website'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['country_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterCountry::className(), 'targetAttribute' => ['country_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'enquiry_date' => 'Enquiry Date',
            'details' => 'Details',
            'enquiry_status' => 'Enquiry Status',
            'prospect_company' => 'Prospect Company',
            'prospect_name' => 'Prospect Name',
            'address' => 'Address',
            'city' => 'City',
            'zip' => 'Zip',
            'phone' => 'Phone',
            'email' => 'Email',
            'country_id' => 'Country ID',
            'website' => 'Website',
            'remarks' => 'Remarks',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCountry()
    {
        return $this->hasOne(QmsMasterCountry::className(), ['id' => 'country_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsEnquiryFollowUps()
    {
        return $this->hasMany(QmsEnquiryFollowUp::className(), ['enquiry_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsEnquiryInternalPurposes()
    {
        return $this->hasMany(QmsEnquiryInternalPurpose::className(), ['enquiry_id' => 'id']);
    }
}
