<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_enquiry_follow_up}}".
 *
 * @property integer $id
 * @property integer $enquiry_id
 * @property string $enquiry_date
 * @property string $details
 * @property string $prospect_name
 * @property string $prospect_company
 * @property string $follow_up_date
 * @property string $reminder_date
 * @property string $remarks
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsEnquiry $enquiry
 */
class QmsEnquiryFollowUp extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_enquiry_follow_up}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['enquiry_id', 'enquiry_date', 'details', 'prospect_name', 'prospect_company', 'follow_up_date', 'reminder_date', 'remarks'], 'required'],
            [['enquiry_id', 'created_by', 'updated_by'], 'integer'],
            [['enquiry_date', 'follow_up_date', 'reminder_date'], 'safe'],
            [['details', 'remarks', 'status'], 'string'],
            [['prospect_name', 'prospect_company'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['enquiry_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsEnquiry::className(), 'targetAttribute' => ['enquiry_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'enquiry_id' => 'Enquiry ID',
            'enquiry_date' => 'Enquiry Date',
            'details' => 'Details',
            'prospect_name' => 'Prospect Name',
            'prospect_company' => 'Prospect Company',
            'follow_up_date' => 'Follow Up Date',
            'reminder_date' => 'Reminder Date',
            'remarks' => 'Remarks',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEnquiry()
    {
        return $this->hasOne(QmsEnquiry::className(), ['id' => 'enquiry_id']);
    }
}
