<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsEnquiry;

/**
 * QmsEnquirySearch represents the model behind the search form of `common\modules\qms\models\QmsEnquiry`.
 */
class QmsEnquirySearch extends QmsEnquiry
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'country_id', 'created_by', 'updated_by'], 'integer'],
            [['enquiry_date', 'details', 'enquiry_status', 'prospect_company', 'prospect_name', 'address', 'city', 'zip', 'phone', 'email', 'website', 'remarks', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsEnquiry::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'enquiry_date' => $this->enquiry_date,
            'country_id' => $this->country_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'details', $this->details])
            ->andFilterWhere(['like', 'enquiry_status', $this->enquiry_status])
            ->andFilterWhere(['like', 'prospect_company', $this->prospect_company])
            ->andFilterWhere(['like', 'prospect_name', $this->prospect_name])
            ->andFilterWhere(['like', 'address', $this->address])
            ->andFilterWhere(['like', 'city', $this->city])
            ->andFilterWhere(['like', 'zip', $this->zip])
            ->andFilterWhere(['like', 'phone', $this->phone])
            ->andFilterWhere(['like', 'email', $this->email])
            ->andFilterWhere(['like', 'website', $this->website])
            ->andFilterWhere(['like', 'remarks', $this->remarks])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
