<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_equipment}}".
 *
 * @property integer $id
 * @property string $name
 * @property string $model_number
 * @property string $manufacturer_name
 * @property string $location
 * @property string $identification_number
 * @property string $manufacturer_identification_number
 * @property string $asset_number
 * @property string $date_of_purchase
 * @property string $approval_status
 * @property integer $approving_authority
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property QmsBreakdownRecord[] $qmsBreakdownRecords
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $approvingAuthority
 */
class QmsEquipment extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_equipment}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['name', 'location', 'date_of_purchase'], 'required'],
            [['date_of_purchase'], 'safe'],
            [['approval_status', 'status'], 'string'],
            [['approving_authority', 'created_by', 'updated_by'], 'integer'],
            [['name', 'model_number', 'manufacturer_name', 'location', 'identification_number', 'manufacturer_identification_number', 'asset_number'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['approving_authority'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'model_number' => 'Model Number',
            'manufacturer_name' => 'Manufacturer Name',
            'location' => 'Location',
            'identification_number' => 'Identification Number',
            'manufacturer_identification_number' => 'Manufacturer Identification Number',
            'asset_number' => 'Asset Number',
            'date_of_purchase' => 'Date Of Purchase',
            'approval_status' => 'Approval Status',
            'approving_authority' => 'Approving Authority',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsBreakdownRecords()
    {
        return $this->hasMany(QmsBreakdownRecord::className(), ['equipment_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority']);
    }
}
