<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_external_issues}}".
 *
 * @property integer $id
 * @property integer $issue_type_id
 * @property string $how_it_affects_your_organisation
 * @property integer $management_system_type_id
 * @property integer $site_id
 * @property integer $department_id
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterIssueType $issueType
 * @property QmsMasterManagementSystemType $managementSystemType
 * @property QmsMasterSite $site
 * @property QmsDepartment $department
 * @property User $approvingAuthority
 */
class QmsExternalIssues extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_external_issues}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['issue_type_id', 'how_it_affects_your_organisation', 'management_system_type_id', 'approval_status'], 'required'],
            [['issue_type_id', 'management_system_type_id', 'site_id', 'department_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['how_it_affects_your_organisation', 'approval_status', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['issue_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterIssueType::className(), 'targetAttribute' => ['issue_type_id' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterSite::className(), 'targetAttribute' => ['site_id' => 'id']],
            [['department_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsDepartment::className(), 'targetAttribute' => ['department_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'issue_type_id' => 'Issue Type ID',
            'how_it_affects_your_organisation' => 'How It Affects Your Organisation',
            'management_system_type_id' => 'Management System Type ID',
            'site_id' => 'Site ID',
            'department_id' => 'Department ID',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getIssueType()
    {
        return $this->hasOne(QmsMasterIssueType::className(), ['id' => 'issue_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(QmsMasterSite::className(), ['id' => 'site_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartment()
    {
        return $this->hasOne(QmsDepartment::className(), ['id' => 'department_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
