<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_incoming_material}}".
 *
 * @property integer $id
 * @property string $incoming_material
 * @property integer $incoming_material_category_id
 * @property integer $management_system_type_id
 * @property string $remarks
 * @property string $identification_number
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property QmsIncomingInspectionQualityPlan[] $qmsIncomingInspectionQualityPlans
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterIncomingMaterialCategory $incomingMaterialCategory
 * @property QmsMasterManagementSystemType $managementSystemType
 */
class QmsIncomingMaterial extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_incoming_material}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['incoming_material', 'management_system_type_id'], 'required'],
            [['incoming_material_category_id', 'management_system_type_id', 'created_by', 'updated_by'], 'integer'],
            [['remarks', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['incoming_material', 'identification_number'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['incoming_material_category_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterIncomingMaterialCategory::className(), 'targetAttribute' => ['incoming_material_category_id' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'incoming_material' => 'Incoming Material',
            'incoming_material_category_id' => 'Incoming Material Category ID',
            'management_system_type_id' => 'Management System Type ID',
            'remarks' => 'Remarks',
            'identification_number' => 'Identification Number',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsIncomingInspectionQualityPlans()
    {
        return $this->hasMany(QmsIncomingInspectionQualityPlan::className(), ['purchased_product_service_incoming_material_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getIncomingMaterialCategory()
    {
        return $this->hasOne(QmsMasterIncomingMaterialCategory::className(), ['id' => 'incoming_material_category_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }
}
