<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_interested_party}}".
 *
 * @property integer $id
 * @property integer $management_system_type_id
 * @property string $stakeholder_interested_party
 * @property string $requirements_needs_expectations
 * @property string $impact_on_the_organiastion
 * @property integer $department_id
 * @property string $department_impact
 * @property integer $site_id
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterManagementSystemType $managementSystemType
 * @property QmsDepartment $department
 * @property QmsMasterSite $site
 * @property User $approvingAuthority
 * @property QmsOfflineCommunication[] $qmsOfflineCommunications
 */
class QmsInterestedParty extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_interested_party}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['management_system_type_id', 'stakeholder_interested_party', 'requirements_needs_expectations', 'impact_on_the_organiastion'], 'required'],
            [['management_system_type_id', 'department_id', 'site_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['stakeholder_interested_party', 'requirements_needs_expectations', 'impact_on_the_organiastion', 'department_impact', 'approval_status', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
            [['department_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsDepartment::className(), 'targetAttribute' => ['department_id' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterSite::className(), 'targetAttribute' => ['site_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'management_system_type_id' => 'Management System Type ID',
            'stakeholder_interested_party' => 'Stakeholder Interested Party',
            'requirements_needs_expectations' => 'Requirements Needs Expectations',
            'impact_on_the_organiastion' => 'Impact On The Organiastion',
            'department_id' => 'Department ID',
            'department_impact' => 'Department Impact',
            'site_id' => 'Site ID',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartment()
    {
        return $this->hasOne(QmsDepartment::className(), ['id' => 'department_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(QmsMasterSite::className(), ['id' => 'site_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsOfflineCommunications()
    {
        return $this->hasMany(QmsOfflineCommunication::className(), ['interested_party_id' => 'id']);
    }
}
