<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsInterestedParty;

/**
 * QmsInterestedPartySearch represents the model behind the search form of `common\modules\qms\models\QmsInterestedParty`.
 */
class QmsInterestedPartySearch extends QmsInterestedParty
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'management_system_type_id', 'department_id', 'site_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['stakeholder_interested_party', 'requirements_needs_expectations', 'impact_on_the_organiastion', 'department_impact', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsInterestedParty::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'management_system_type_id' => $this->management_system_type_id,
            'department_id' => $this->department_id,
            'site_id' => $this->site_id,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'stakeholder_interested_party', $this->stakeholder_interested_party])
            ->andFilterWhere(['like', 'requirements_needs_expectations', $this->requirements_needs_expectations])
            ->andFilterWhere(['like', 'impact_on_the_organiastion', $this->impact_on_the_organiastion])
            ->andFilterWhere(['like', 'department_impact', $this->department_impact])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
