<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_internal_audit_schedule}}".
 *
 * @property integer $id
 * @property string $internal_audit_name
 * @property integer $master_internal_audit_name_id
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterInternalAuditPlan $masterInternalAuditName
 * @property User $approvingAuthority
 */
class QmsInternalAuditSchedule extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_internal_audit_schedule}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['internal_audit_name'], 'required'],
            [['master_internal_audit_name_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['approval_status', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['internal_audit_name'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['master_internal_audit_name_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterInternalAuditPlan::className(), 'targetAttribute' => ['master_internal_audit_name_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'internal_audit_name' => 'Internal Audit Name',
            'master_internal_audit_name_id' => 'Master Internal Audit Name ID',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMasterInternalAuditName()
    {
        return $this->hasOne(QmsMasterInternalAuditPlan::className(), ['id' => 'master_internal_audit_name_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
