<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_kpi}}".
 *
 * @property integer $id
 * @property integer $management_system_type_id
 * @property integer $department_id
 * @property integer $site_id
 * @property integer $process_id
 * @property string $what_to_monitor
 * @property string $how_to_monitor
 * @property string $when_to_monitor
 * @property string $target
 * @property string $unit
 * @property string $best_direction_for_tracking
 * @property string $how_kpi_should_be_aggregated
 * @property integer $responsibility_id
 * @property string $when_to_analyse_results
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterManagementSystemType $managementSystemType
 * @property QmsDepartment $department
 * @property QmsMasterSite $site
 * @property QmsProcessIdentification $process
 * @property User $responsibility
 * @property User $approvingAuthority
 */
class QmsKpi extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_kpi}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['management_system_type_id', 'what_to_monitor', 'how_to_monitor', 'when_to_monitor', 'target'], 'required'],
            [['management_system_type_id', 'department_id', 'site_id', 'process_id', 'responsibility_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['unit', 'best_direction_for_tracking', 'how_kpi_should_be_aggregated', 'when_to_analyse_results', 'approval_status', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['what_to_monitor', 'how_to_monitor', 'when_to_monitor', 'target'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
            [['department_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsDepartment::className(), 'targetAttribute' => ['department_id' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterSite::className(), 'targetAttribute' => ['site_id' => 'id']],
            [['process_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsProcessIdentification::className(), 'targetAttribute' => ['process_id' => 'id']],
            [['responsibility_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['responsibility_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'management_system_type_id' => 'Management System Type ID',
            'department_id' => 'Department ID',
            'site_id' => 'Site ID',
            'process_id' => 'Process ID',
            'what_to_monitor' => 'What To Monitor',
            'how_to_monitor' => 'How To Monitor',
            'when_to_monitor' => 'When To Monitor',
            'target' => 'Target',
            'unit' => 'Unit',
            'best_direction_for_tracking' => 'Best Direction For Tracking',
            'how_kpi_should_be_aggregated' => 'How Kpi Should Be Aggregated',
            'responsibility_id' => 'Responsibility ID',
            'when_to_analyse_results' => 'When To Analyse Results',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartment()
    {
        return $this->hasOne(QmsDepartment::className(), ['id' => 'department_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(QmsMasterSite::className(), ['id' => 'site_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProcess()
    {
        return $this->hasOne(QmsProcessIdentification::className(), ['id' => 'process_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getResponsibility()
    {
        return $this->hasOne(User::className(), ['id' => 'responsibility_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
