<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsKpi;

/**
 * QmsKpiSearch represents the model behind the search form of `common\modules\qms\models\QmsKpi`.
 */
class QmsKpiSearch extends QmsKpi
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'management_system_type_id', 'department_id', 'site_id', 'process_id', 'responsibility_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['what_to_monitor', 'how_to_monitor', 'when_to_monitor', 'target', 'unit', 'best_direction_for_tracking', 'how_kpi_should_be_aggregated', 'when_to_analyse_results', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsKpi::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'management_system_type_id' => $this->management_system_type_id,
            'department_id' => $this->department_id,
            'site_id' => $this->site_id,
            'process_id' => $this->process_id,
            'responsibility_id' => $this->responsibility_id,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'what_to_monitor', $this->what_to_monitor])
            ->andFilterWhere(['like', 'how_to_monitor', $this->how_to_monitor])
            ->andFilterWhere(['like', 'when_to_monitor', $this->when_to_monitor])
            ->andFilterWhere(['like', 'target', $this->target])
            ->andFilterWhere(['like', 'unit', $this->unit])
            ->andFilterWhere(['like', 'best_direction_for_tracking', $this->best_direction_for_tracking])
            ->andFilterWhere(['like', 'how_kpi_should_be_aggregated', $this->how_kpi_should_be_aggregated])
            ->andFilterWhere(['like', 'when_to_analyse_results', $this->when_to_analyse_results])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
