<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_life_cycle_management}}".
 *
 * @property integer $id
 * @property integer $product_service_projects_id
 * @property integer $stages_of_product_id
 * @property string $description
 * @property string $control_measure
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsProductServiceProject $productServiceProjects
 * @property QmsMasterProductStage $stagesOfProduct
 * @property User $approvingAuthority
 */
class QmsLifeCycleManagement extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_life_cycle_management}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['product_service_projects_id', 'stages_of_product_id'], 'required'],
            [['product_service_projects_id', 'stages_of_product_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['description', 'control_measure', 'approval_status', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['product_service_projects_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsProductServiceProject::className(), 'targetAttribute' => ['product_service_projects_id' => 'id']],
            [['stages_of_product_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterProductStage::className(), 'targetAttribute' => ['stages_of_product_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'product_service_projects_id' => 'Product Service Projects ID',
            'stages_of_product_id' => 'Stages Of Product ID',
            'description' => 'Description',
            'control_measure' => 'Control Measure',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProductServiceProjects()
    {
        return $this->hasOne(QmsProductServiceProject::className(), ['id' => 'product_service_projects_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getStagesOfProduct()
    {
        return $this->hasOne(QmsMasterProductStage::className(), ['id' => 'stages_of_product_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
