<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_master_internal_audit_plan}}".
 *
 * @property integer $id
 * @property string $master_internal_audit_name
 * @property string $audit_cycle_from_month
 * @property string $audit_cycle_to_month
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property QmsInternalAuditSchedule[] $qmsInternalAuditSchedules
 * @property User $updatedBy
 * @property User $approvingAuthority
 * @property User $createdBy
 */
class QmsMasterInternalAuditPlan extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_master_internal_audit_plan}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['master_internal_audit_name', 'audit_cycle_from_month', 'audit_cycle_to_month'], 'required'],
            [['audit_cycle_from_month', 'audit_cycle_to_month'], 'safe'],
            [['approval_status', 'status'], 'string'],
            [['approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['master_internal_audit_name'], 'string', 'max' => 255],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'master_internal_audit_name' => 'Master Internal Audit Name',
            'audit_cycle_from_month' => 'Audit Cycle From Month',
            'audit_cycle_to_month' => 'Audit Cycle To Month',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsInternalAuditSchedules()
    {
        return $this->hasMany(QmsInternalAuditSchedule::className(), ['master_internal_audit_name_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }
}
