<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_mrm}}".
 *
 * @property integer $id
 * @property integer $management_system_type_id
 * @property string $date_of_mrm
 * @property integer $attendee_id
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterManagementSystemType $managementSystemType
 * @property User $attendee
 * @property User $approvingAuthority
 */
class QmsMrm extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_mrm}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['management_system_type_id', 'date_of_mrm', 'attendee_id'], 'required'],
            [['management_system_type_id', 'attendee_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['date_of_mrm'], 'safe'],
            [['approval_status', 'status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
            [['attendee_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['attendee_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'management_system_type_id' => 'Management System Type ID',
            'date_of_mrm' => 'Date Of Mrm',
            'attendee_id' => 'Attendee ID',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAttendee()
    {
        return $this->hasOne(User::className(), ['id' => 'attendee_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
