<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_offline_communication}}".
 *
 * @property integer $id
 * @property string $communication_date
 * @property integer $communication_trigger_id
 * @property integer $interested_party_id
 * @property string $remarks
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterCommunicationTrigger $communicationTrigger
 * @property QmsInterestedParty $interestedParty
 */
class QmsOfflineCommunication extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_offline_communication}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['communication_date', 'communication_trigger_id', 'interested_party_id', 'remarks'], 'required'],
            [['communication_date'], 'safe'],
            [['communication_trigger_id', 'interested_party_id', 'created_by', 'updated_by'], 'integer'],
            [['remarks', 'status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['communication_trigger_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterCommunicationTrigger::className(), 'targetAttribute' => ['communication_trigger_id' => 'id']],
            [['interested_party_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsInterestedParty::className(), 'targetAttribute' => ['interested_party_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'communication_date' => 'Communication Date',
            'communication_trigger_id' => 'Communication Trigger ID',
            'interested_party_id' => 'Interested Party ID',
            'remarks' => 'Remarks',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCommunicationTrigger()
    {
        return $this->hasOne(QmsMasterCommunicationTrigger::className(), ['id' => 'communication_trigger_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getInterestedParty()
    {
        return $this->hasOne(QmsInterestedParty::className(), ['id' => 'interested_party_id']);
    }
}
