<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_offline_communication_policy}}".
 *
 * @property integer $id
 * @property integer $communication_trigger_id
 * @property string $what_to_communicate
 * @property string $whom_to_communicate_internal
 * @property string $whom_to_communicate_external
 * @property string $how_to_communicate
 * @property string $responsibility
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $communicationTrigger
 * @property User $approvingAuthority
 */
class QmsOfflineCommunicationPolicy extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_offline_communication_policy}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['communication_trigger_id', 'what_to_communicate', 'whom_to_communicate_internal', 'whom_to_communicate_external', 'how_to_communicate', 'responsibility'], 'required'],
            [['communication_trigger_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['what_to_communicate', 'whom_to_communicate_internal', 'whom_to_communicate_external', 'how_to_communicate', 'approval_status', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['responsibility'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['communication_trigger_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['communication_trigger_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'communication_trigger_id' => 'Communication Trigger ID',
            'what_to_communicate' => 'What To Communicate',
            'whom_to_communicate_internal' => 'Whom To Communicate Internal',
            'whom_to_communicate_external' => 'Whom To Communicate External',
            'how_to_communicate' => 'How To Communicate',
            'responsibility' => 'Responsibility',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCommunicationTrigger()
    {
        return $this->hasOne(User::className(), ['id' => 'communication_trigger_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
