<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsOfflineCommunicationPolicy;

/**
 * QmsOfflineCommunicationPolicySearch represents the model behind the search form of `common\modules\qms\models\QmsOfflineCommunicationPolicy`.
 */
class QmsOfflineCommunicationPolicySearch extends QmsOfflineCommunicationPolicy
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'communication_trigger_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['what_to_communicate', 'whom_to_communicate_internal', 'whom_to_communicate_external', 'how_to_communicate', 'responsibility', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsOfflineCommunicationPolicy::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'communication_trigger_id' => $this->communication_trigger_id,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'what_to_communicate', $this->what_to_communicate])
            ->andFilterWhere(['like', 'whom_to_communicate_internal', $this->whom_to_communicate_internal])
            ->andFilterWhere(['like', 'whom_to_communicate_external', $this->whom_to_communicate_external])
            ->andFilterWhere(['like', 'how_to_communicate', $this->how_to_communicate])
            ->andFilterWhere(['like', 'responsibility', $this->responsibility])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
