<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_operation_planning}}".
 *
 * @property integer $id
 * @property integer $product_service_project_id
 * @property string $brief_description
 * @property string $requirement_analysis
 * @property integer $operation_planning_process_id
 * @property string $acceptance_criteria
 * @property string $human_resource
 * @property string $interested_parties
 * @property string $exceptions
 * @property string $knowledge_required
 * @property string $knowledge_made_available
 * @property string $infrstructure_resource
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsProductServiceProject $productServiceProject
 * @property QmsProcessIdentification $operationPlanningProcess
 * @property User $approvingAuthority
 */
class QmsOperationPlanning extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_operation_planning}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['product_service_project_id', 'brief_description', 'requirement_analysis'], 'required'],
            [['product_service_project_id', 'operation_planning_process_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['brief_description', 'requirement_analysis', 'acceptance_criteria', 'human_resource', 'interested_parties', 'exceptions', 'knowledge_required', 'knowledge_made_available', 'infrstructure_resource', 'approval_status', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['product_service_project_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsProductServiceProject::className(), 'targetAttribute' => ['product_service_project_id' => 'id']],
            [['operation_planning_process_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsProcessIdentification::className(), 'targetAttribute' => ['operation_planning_process_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'product_service_project_id' => 'Product Service Project ID',
            'brief_description' => 'Brief Description',
            'requirement_analysis' => 'Requirement Analysis',
            'operation_planning_process_id' => 'Operation Planning Process ID',
            'acceptance_criteria' => 'Acceptance Criteria',
            'human_resource' => 'Human Resource',
            'interested_parties' => 'Interested Parties',
            'exceptions' => 'Exceptions',
            'knowledge_required' => 'Knowledge Required',
            'knowledge_made_available' => 'Knowledge Made Available',
            'infrstructure_resource' => 'Infrstructure Resource',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProductServiceProject()
    {
        return $this->hasOne(QmsProductServiceProject::className(), ['id' => 'product_service_project_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOperationPlanningProcess()
    {
        return $this->hasOne(QmsProcessIdentification::className(), ['id' => 'operation_planning_process_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
