<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsOperationPlanning;

/**
 * QmsOperationPlanningSearch represents the model behind the search form of `common\modules\qms\models\QmsOperationPlanning`.
 */
class QmsOperationPlanningSearch extends QmsOperationPlanning
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'product_service_project_id', 'operation_planning_process_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['brief_description', 'requirement_analysis', 'acceptance_criteria', 'human_resource', 'interested_parties', 'exceptions', 'knowledge_required', 'knowledge_made_available', 'infrstructure_resource', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsOperationPlanning::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'product_service_project_id' => $this->product_service_project_id,
            'operation_planning_process_id' => $this->operation_planning_process_id,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'brief_description', $this->brief_description])
            ->andFilterWhere(['like', 'requirement_analysis', $this->requirement_analysis])
            ->andFilterWhere(['like', 'acceptance_criteria', $this->acceptance_criteria])
            ->andFilterWhere(['like', 'human_resource', $this->human_resource])
            ->andFilterWhere(['like', 'interested_parties', $this->interested_parties])
            ->andFilterWhere(['like', 'exceptions', $this->exceptions])
            ->andFilterWhere(['like', 'knowledge_required', $this->knowledge_required])
            ->andFilterWhere(['like', 'knowledge_made_available', $this->knowledge_made_available])
            ->andFilterWhere(['like', 'infrstructure_resource', $this->infrstructure_resource])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
