<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_process_identification}}".
 *
 * @property integer $id
 * @property integer $management_system_type_id
 * @property integer $department_id
 * @property string $process_name
 * @property string $purpose
 * @property string $scope
 * @property string $definition
 * @property string $revision_number
 * @property string $revision_date
 * @property string $outsourced
 * @property integer $accountable_person_id
 * @property string $what_knowledge_is_required
 * @property string $how_the_knowledge_would_be_made_available
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property QmsKpi[] $qmsKpis
 * @property QmsOperationPlanning[] $qmsOperationPlannings
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterManagementSystemType $managementSystemType
 * @property QmsDepartment $department
 * @property User $accountablePerson
 * @property User $approvingAuthority
 * @property QmsProcessValidation[] $qmsProcessValidations
 */
class QmsProcessIdentification extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_process_identification}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['management_system_type_id', 'department_id', 'accountable_person_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['department_id', 'process_name', 'revision_date', 'accountable_person_id'], 'required'],
            [['purpose', 'scope', 'definition', 'what_knowledge_is_required', 'how_the_knowledge_would_be_made_available', 'approval_status', 'status'], 'string'],
            [['revision_date'], 'safe'],
            [['process_name', 'revision_number', 'outsourced'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
            [['department_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsDepartment::className(), 'targetAttribute' => ['department_id' => 'id']],
            [['accountable_person_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['accountable_person_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'management_system_type_id' => 'Management System Type ID',
            'department_id' => 'Department ID',
            'process_name' => 'Process Name',
            'purpose' => 'Purpose',
            'scope' => 'Scope',
            'definition' => 'Definition',
            'revision_number' => 'Revision Number',
            'revision_date' => 'Revision Date',
            'outsourced' => 'Outsourced',
            'accountable_person_id' => 'Accountable Person ID',
            'what_knowledge_is_required' => 'What Knowledge Is Required',
            'how_the_knowledge_would_be_made_available' => 'How The Knowledge Would Be Made Available',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsKpis()
    {
        return $this->hasMany(QmsKpi::className(), ['process_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsOperationPlannings()
    {
        return $this->hasMany(QmsOperationPlanning::className(), ['operation_planning_process_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartment()
    {
        return $this->hasOne(QmsDepartment::className(), ['id' => 'department_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccountablePerson()
    {
        return $this->hasOne(User::className(), ['id' => 'accountable_person_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsProcessValidations()
    {
        return $this->hasMany(QmsProcessValidation::className(), ['department_process_id' => 'id']);
    }
}
