<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsProcessIdentification;

/**
 * QmsProcessIdentificationSearch represents the model behind the search form of `common\modules\qms\models\QmsProcessIdentification`.
 */
class QmsProcessIdentificationSearch extends QmsProcessIdentification
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'management_system_type_id', 'department_id', 'accountable_person_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['process_name', 'purpose', 'scope', 'definition', 'revision_number', 'revision_date', 'outsourced', 'what_knowledge_is_required', 'how_the_knowledge_would_be_made_available', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsProcessIdentification::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'management_system_type_id' => $this->management_system_type_id,
            'department_id' => $this->department_id,
            'revision_date' => $this->revision_date,
            'accountable_person_id' => $this->accountable_person_id,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'process_name', $this->process_name])
            ->andFilterWhere(['like', 'purpose', $this->purpose])
            ->andFilterWhere(['like', 'scope', $this->scope])
            ->andFilterWhere(['like', 'definition', $this->definition])
            ->andFilterWhere(['like', 'revision_number', $this->revision_number])
            ->andFilterWhere(['like', 'outsourced', $this->outsourced])
            ->andFilterWhere(['like', 'what_knowledge_is_required', $this->what_knowledge_is_required])
            ->andFilterWhere(['like', 'how_the_knowledge_would_be_made_available', $this->how_the_knowledge_would_be_made_available])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
