<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_process_validation}}".
 *
 * @property integer $id
 * @property integer $department_id
 * @property integer $department_process_id
 * @property string $validation_date
 * @property string $re_validation_date
 * @property string $brief_process_description
 * @property string $name_of_the_machine_equipment_to_be_used
 * @property string $name_of_the_qualified_person
 * @property string $description_of_process_steps
 * @property string $operator_qualification_competence
 * @property string $validation_result
 * @property string $re_validation_due_date
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsDepartment $department
 * @property QmsProcessIdentification $departmentProcess
 */
class QmsProcessValidation extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_process_validation}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['department_id', 'validation_date', 'name_of_the_qualified_person', 'validation_result'], 'required'],
            [['department_id', 'department_process_id', 'created_by', 'updated_by'], 'integer'],
            [['validation_date', 're_validation_date', 're_validation_due_date'], 'safe'],
            [['brief_process_description', 'name_of_the_machine_equipment_to_be_used', 'name_of_the_qualified_person', 'description_of_process_steps', 'operator_qualification_competence', 'validation_result', 'status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['department_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsDepartment::className(), 'targetAttribute' => ['department_id' => 'id']],
            [['department_process_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsProcessIdentification::className(), 'targetAttribute' => ['department_process_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'department_id' => 'Department ID',
            'department_process_id' => 'Department Process ID',
            'validation_date' => 'Validation Date',
            're_validation_date' => 'Re Validation Date',
            'brief_process_description' => 'Brief Process Description',
            'name_of_the_machine_equipment_to_be_used' => 'Name Of The Machine Equipment To Be Used',
            'name_of_the_qualified_person' => 'Name Of The Qualified Person',
            'description_of_process_steps' => 'Description Of Process Steps',
            'operator_qualification_competence' => 'Operator Qualification Competence',
            'validation_result' => 'Validation Result',
            're_validation_due_date' => 'Re Validation Due Date',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartment()
    {
        return $this->hasOne(QmsDepartment::className(), ['id' => 'department_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartmentProcess()
    {
        return $this->hasOne(QmsProcessIdentification::className(), ['id' => 'department_process_id']);
    }
}
