<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsProcessValidation;

/**
 * QmsProcessValidationSearch represents the model behind the search form of `common\modules\qms\models\QmsProcessValidation`.
 */
class QmsProcessValidationSearch extends QmsProcessValidation
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'department_id', 'department_process_id', 'created_by', 'updated_by'], 'integer'],
            [['validation_date', 're_validation_date', 'brief_process_description', 'name_of_the_machine_equipment_to_be_used', 'name_of_the_qualified_person', 'description_of_process_steps', 'operator_qualification_competence', 'validation_result', 're_validation_due_date', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsProcessValidation::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'department_id' => $this->department_id,
            'department_process_id' => $this->department_process_id,
            'validation_date' => $this->validation_date,
            're_validation_date' => $this->re_validation_date,
            're_validation_due_date' => $this->re_validation_due_date,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'brief_process_description', $this->brief_process_description])
            ->andFilterWhere(['like', 'name_of_the_machine_equipment_to_be_used', $this->name_of_the_machine_equipment_to_be_used])
            ->andFilterWhere(['like', 'name_of_the_qualified_person', $this->name_of_the_qualified_person])
            ->andFilterWhere(['like', 'description_of_process_steps', $this->description_of_process_steps])
            ->andFilterWhere(['like', 'operator_qualification_competence', $this->operator_qualification_competence])
            ->andFilterWhere(['like', 'validation_result', $this->validation_result])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
