<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_product_service_project}}".
 *
 * @property integer $id
 * @property string $type
 * @property string $name
 * @property integer $management_system_type_id
 * @property string $remarks
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property QmsDesignDevelopment[] $qmsDesignDevelopments
 * @property QmsDesignDevelopmentChanges[] $qmsDesignDevelopmentChanges
 * @property QmsFinalInspectionQualityPlan[] $qmsFinalInspectionQualityPlans
 * @property QmsFinalInspectionQualityRecords[] $qmsFinalInspectionQualityRecords
 * @property QmsInProcessInspectionRecords[] $qmsInProcessInspectionRecords
 * @property QmsInProcessQualityPlan[] $qmsInProcessQualityPlans
 * @property QmsLifeCycleManagement[] $qmsLifeCycleManagements
 * @property QmsOperationPlanning[] $qmsOperationPlannings
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterManagementSystemType $managementSystemType
 */
class QmsProductServiceProject extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_product_service_project}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['type', 'remarks', 'status'], 'string'],
            [['name', 'management_system_type_id'], 'required'],
            [['management_system_type_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['name'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'type' => 'Type',
            'name' => 'Name',
            'management_system_type_id' => 'Management System Type ID',
            'remarks' => 'Remarks',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsDesignDevelopments()
    {
        return $this->hasMany(QmsDesignDevelopment::className(), ['product_service_project_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsDesignDevelopmentChanges()
    {
        return $this->hasMany(QmsDesignDevelopmentChanges::className(), ['product_service_project_name_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsFinalInspectionQualityPlans()
    {
        return $this->hasMany(QmsFinalInspectionQualityPlan::className(), ['product_service_project_name_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsFinalInspectionQualityRecords()
    {
        return $this->hasMany(QmsFinalInspectionQualityRecords::className(), ['product_service_project_name_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsInProcessInspectionRecords()
    {
        return $this->hasMany(QmsInProcessInspectionRecords::className(), ['product_service_project_name_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsInProcessQualityPlans()
    {
        return $this->hasMany(QmsInProcessQualityPlan::className(), ['product_service_project_name_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsLifeCycleManagements()
    {
        return $this->hasMany(QmsLifeCycleManagement::className(), ['product_service_projects_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsOperationPlannings()
    {
        return $this->hasMany(QmsOperationPlanning::className(), ['product_service_project_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }
}
