<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_purchase_order}}".
 *
 * @property integer $id
 * @property string $number
 * @property string $date_of_purchase
 * @property integer $supplier_id
 * @property string $supplier_address
 * @property string $expected_delivery_date
 * @property string $supplier_contact
 * @property string $actual_delivery_date
 * @property string $total_po_price
 * @property string $supplier_rating
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsSupplier $supplier
 */
class QmsPurchaseOrder extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_purchase_order}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['number', 'date_of_purchase', 'supplier_id', 'expected_delivery_date'], 'required'],
            [['date_of_purchase', 'expected_delivery_date', 'actual_delivery_date'], 'safe'],
            [['supplier_id', 'created_by', 'updated_by'], 'integer'],
            [['supplier_address', 'status'], 'string'],
            [['number', 'supplier_contact', 'total_po_price', 'supplier_rating'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['supplier_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsSupplier::className(), 'targetAttribute' => ['supplier_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'number' => 'Number',
            'date_of_purchase' => 'Date Of Purchase',
            'supplier_id' => 'Supplier ID',
            'supplier_address' => 'Supplier Address',
            'expected_delivery_date' => 'Expected Delivery Date',
            'supplier_contact' => 'Supplier Contact',
            'actual_delivery_date' => 'Actual Delivery Date',
            'total_po_price' => 'Total Po Price',
            'supplier_rating' => 'Supplier Rating',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSupplier()
    {
        return $this->hasOne(QmsSupplier::className(), ['id' => 'supplier_id']);
    }
}
