<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsPurchaseOrder;

/**
 * QmsPurchaseOrderSearch represents the model behind the search form of `common\modules\qms\models\QmsPurchaseOrder`.
 */
class QmsPurchaseOrderSearch extends QmsPurchaseOrder
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'supplier_id', 'created_by', 'updated_by'], 'integer'],
            [['number', 'date_of_purchase', 'supplier_address', 'expected_delivery_date', 'supplier_contact', 'actual_delivery_date', 'total_po_price', 'supplier_rating', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsPurchaseOrder::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'date_of_purchase' => $this->date_of_purchase,
            'supplier_id' => $this->supplier_id,
            'expected_delivery_date' => $this->expected_delivery_date,
            'actual_delivery_date' => $this->actual_delivery_date,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'number', $this->number])
            ->andFilterWhere(['like', 'supplier_address', $this->supplier_address])
            ->andFilterWhere(['like', 'supplier_contact', $this->supplier_contact])
            ->andFilterWhere(['like', 'total_po_price', $this->total_po_price])
            ->andFilterWhere(['like', 'supplier_rating', $this->supplier_rating])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
