<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_resource_planning}}".
 *
 * @property integer $id
 * @property string $planning_duration
 * @property string $planning_date
 * @property string $resource_planning_for
 * @property integer $resource_type_id
 * @property string $description
 * @property string $quantity
 * @property string $approximate_cost_to_company
 * @property string $required_date
 * @property string $benefits
 * @property string $remarks
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterResourceType $resourceType
 * @property User $approvingAuthority
 */
class QmsResourcePlanning extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_resource_planning}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['planning_duration', 'resource_planning_for', 'description', 'benefits', 'remarks', 'approval_status', 'status'], 'string'],
            [['planning_date', 'resource_planning_for', 'resource_type_id', 'description', 'quantity', 'approximate_cost_to_company', 'required_date'], 'required'],
            [['planning_date', 'required_date'], 'safe'],
            [['resource_type_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['quantity', 'approximate_cost_to_company'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['resource_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterResourceType::className(), 'targetAttribute' => ['resource_type_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'planning_duration' => 'Planning Duration',
            'planning_date' => 'Planning Date',
            'resource_planning_for' => 'Resource Planning For',
            'resource_type_id' => 'Resource Type ID',
            'description' => 'Description',
            'quantity' => 'Quantity',
            'approximate_cost_to_company' => 'Approximate Cost To Company',
            'required_date' => 'Required Date',
            'benefits' => 'Benefits',
            'remarks' => 'Remarks',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getResourceType()
    {
        return $this->hasOne(QmsMasterResourceType::className(), ['id' => 'resource_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
