<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsResourcePlanning;

/**
 * QmsResourcePlanningSearch represents the model behind the search form of `common\modules\qms\models\QmsResourcePlanning`.
 */
class QmsResourcePlanningSearch extends QmsResourcePlanning
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'resource_type_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['planning_duration', 'planning_date', 'resource_planning_for', 'description', 'quantity', 'approximate_cost_to_company', 'required_date', 'benefits', 'remarks', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsResourcePlanning::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'planning_date' => $this->planning_date,
            'resource_type_id' => $this->resource_type_id,
            'required_date' => $this->required_date,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'planning_duration', $this->planning_duration])
            ->andFilterWhere(['like', 'resource_planning_for', $this->resource_planning_for])
            ->andFilterWhere(['like', 'description', $this->description])
            ->andFilterWhere(['like', 'quantity', $this->quantity])
            ->andFilterWhere(['like', 'approximate_cost_to_company', $this->approximate_cost_to_company])
            ->andFilterWhere(['like', 'benefits', $this->benefits])
            ->andFilterWhere(['like', 'remarks', $this->remarks])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
