<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_risk_opportunity}}".
 *
 * @property integer $id
 * @property integer $risk_and_opportunity_type_id
 * @property integer $management_system_type_id
 * @property string $risk_opportunity_source_or_activity
 * @property integer $risk_category_id
 * @property string $risk_description
 * @property string $risk_impact
 * @property string $risk_rating
 * @property string $risk_probability
 * @property integer $assigned_to_id
 * @property integer $department_id
 * @property string $actions_proposed
 * @property string $treatment_type
 * @property string $proposed_review_date
 * @property string $proposed_completion_date
 * @property string $is_it_an_opportunity
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property QmsMasterRiskOpportunityType $riskAndOpportunityType
 * @property QmsMasterManagementSystemType $managementSystemType
 * @property QmsMasterRiskCategory $riskCategory
 * @property User $assignedTo
 * @property QmsDepartment $department
 * @property User $approvingAuthority
 */
class QmsRiskOpportunity extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_risk_opportunity}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['risk_and_opportunity_type_id', 'risk_opportunity_source_or_activity', 'risk_category_id', 'risk_description', 'is_it_an_opportunity'], 'required'],
            [['risk_and_opportunity_type_id', 'management_system_type_id', 'risk_category_id', 'assigned_to_id', 'department_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['risk_opportunity_source_or_activity', 'risk_description', 'risk_impact', 'risk_probability', 'actions_proposed', 'treatment_type', 'approval_status', 'status'], 'string'],
            [['proposed_review_date', 'proposed_completion_date'], 'safe'],
            [['risk_rating', 'is_it_an_opportunity'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['risk_and_opportunity_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterRiskOpportunityType::className(), 'targetAttribute' => ['risk_and_opportunity_type_id' => 'id']],
            [['management_system_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterManagementSystemType::className(), 'targetAttribute' => ['management_system_type_id' => 'id']],
            [['risk_category_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsMasterRiskCategory::className(), 'targetAttribute' => ['risk_category_id' => 'id']],
            [['assigned_to_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['assigned_to_id' => 'id']],
            [['department_id'], 'exist', 'skipOnError' => true, 'targetClass' => QmsDepartment::className(), 'targetAttribute' => ['department_id' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'risk_and_opportunity_type_id' => 'Risk And Opportunity Type ID',
            'management_system_type_id' => 'Management System Type ID',
            'risk_opportunity_source_or_activity' => 'Risk Opportunity Source Or Activity',
            'risk_category_id' => 'Risk Category ID',
            'risk_description' => 'Risk Description',
            'risk_impact' => 'Risk Impact',
            'risk_rating' => 'Risk Rating',
            'risk_probability' => 'Risk Probability',
            'assigned_to_id' => 'Assigned To ID',
            'department_id' => 'Department ID',
            'actions_proposed' => 'Actions Proposed',
            'treatment_type' => 'Treatment Type',
            'proposed_review_date' => 'Proposed Review Date',
            'proposed_completion_date' => 'Proposed Completion Date',
            'is_it_an_opportunity' => 'Is It An Opportunity',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getRiskAndOpportunityType()
    {
        return $this->hasOne(QmsMasterRiskOpportunityType::className(), ['id' => 'risk_and_opportunity_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getManagementSystemType()
    {
        return $this->hasOne(QmsMasterManagementSystemType::className(), ['id' => 'management_system_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getRiskCategory()
    {
        return $this->hasOne(QmsMasterRiskCategory::className(), ['id' => 'risk_category_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAssignedTo()
    {
        return $this->hasOne(User::className(), ['id' => 'assigned_to_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDepartment()
    {
        return $this->hasOne(QmsDepartment::className(), ['id' => 'department_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }
}
