<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsRiskOpportunity;

/**
 * QmsRiskOpportunitySearch represents the model behind the search form of `common\modules\qms\models\QmsRiskOpportunity`.
 */
class QmsRiskOpportunitySearch extends QmsRiskOpportunity
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'risk_and_opportunity_type_id', 'management_system_type_id', 'risk_category_id', 'assigned_to_id', 'department_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['risk_opportunity_source_or_activity', 'risk_description', 'risk_impact', 'risk_rating', 'risk_probability', 'actions_proposed', 'treatment_type', 'proposed_review_date', 'proposed_completion_date', 'is_it_an_opportunity', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsRiskOpportunity::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'risk_and_opportunity_type_id' => $this->risk_and_opportunity_type_id,
            'management_system_type_id' => $this->management_system_type_id,
            'risk_category_id' => $this->risk_category_id,
            'assigned_to_id' => $this->assigned_to_id,
            'department_id' => $this->department_id,
            'proposed_review_date' => $this->proposed_review_date,
            'proposed_completion_date' => $this->proposed_completion_date,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'risk_opportunity_source_or_activity', $this->risk_opportunity_source_or_activity])
            ->andFilterWhere(['like', 'risk_description', $this->risk_description])
            ->andFilterWhere(['like', 'risk_impact', $this->risk_impact])
            ->andFilterWhere(['like', 'risk_rating', $this->risk_rating])
            ->andFilterWhere(['like', 'risk_probability', $this->risk_probability])
            ->andFilterWhere(['like', 'actions_proposed', $this->actions_proposed])
            ->andFilterWhere(['like', 'treatment_type', $this->treatment_type])
            ->andFilterWhere(['like', 'is_it_an_opportunity', $this->is_it_an_opportunity])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
