<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_supplier}}".
 *
 * @property integer $id
 * @property string $name
 * @property string $address
 * @property string $email
 * @property string $phone
 * @property string $supplier_contact_name
 * @property string $supplier_contact_phone
 * @property string $supplier_category
 * @property string $product_service_details
 * @property integer $number_of_employees
 * @property integer $annual_turnover_millions
 * @property string $certification_type
 * @property string $other_certification_type
 * @property string $existing_major_customers
 * @property string $govt_or_tax_registration_number
 * @property string $major_registration_criterias
 * @property string $supplier_rating
 * @property string $sample_approval_details
 * @property string $is_active
 * @property string $approval_status
 * @property integer $approving_authority_id
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property QmsAsset[] $qmsAssets
 * @property QmsPurchaseOrder[] $qmsPurchaseOrders
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $approvingAuthority
 * @property QmsSupplierReevaluation[] $qmsSupplierReevaluations
 */
class QmsSupplier extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_supplier}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['name', 'address', 'email', 'phone', 'supplier_contact_name', 'supplier_rating'], 'required'],
            [['address', 'supplier_category', 'product_service_details', 'certification_type', 'existing_major_customers', 'govt_or_tax_registration_number', 'major_registration_criterias', 'sample_approval_details', 'approval_status', 'status'], 'string'],
            [['number_of_employees', 'annual_turnover_millions', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['name', 'email', 'phone', 'supplier_contact_name', 'supplier_contact_phone', 'other_certification_type', 'supplier_rating', 'is_active'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['approving_authority_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['approving_authority_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'address' => 'Address',
            'email' => 'Email',
            'phone' => 'Phone',
            'supplier_contact_name' => 'Supplier Contact Name',
            'supplier_contact_phone' => 'Supplier Contact Phone',
            'supplier_category' => 'Supplier Category',
            'product_service_details' => 'Product Service Details',
            'number_of_employees' => 'Number Of Employees',
            'annual_turnover_millions' => 'Annual Turnover Millions',
            'certification_type' => 'Certification Type',
            'other_certification_type' => 'Other Certification Type',
            'existing_major_customers' => 'Existing Major Customers',
            'govt_or_tax_registration_number' => 'Govt Or Tax Registration Number',
            'major_registration_criterias' => 'Major Registration Criterias',
            'supplier_rating' => 'Supplier Rating',
            'sample_approval_details' => 'Sample Approval Details',
            'is_active' => 'Is Active',
            'approval_status' => 'Approval Status',
            'approving_authority_id' => 'Approving Authority ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsAssets()
    {
        return $this->hasMany(QmsAsset::className(), ['supplier_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsPurchaseOrders()
    {
        return $this->hasMany(QmsPurchaseOrder::className(), ['supplier_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getApprovingAuthority()
    {
        return $this->hasOne(User::className(), ['id' => 'approving_authority_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsSupplierReevaluations()
    {
        return $this->hasMany(QmsSupplierReevaluation::className(), ['supplier_id' => 'id']);
    }
}
