<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsSupplierReevaluation;

/**
 * QmsSupplierReevaluationSearch represents the model behind the search form of `common\modules\qms\models\QmsSupplierReevaluation`.
 */
class QmsSupplierReevaluationSearch extends QmsSupplierReevaluation
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'supplier_id', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['supplier_address', 'suplier_rating', 'suplier_contact', 'evaluation_analysis', 'evaluation_date', 'next_evaluation_date', 'supplier_score', 'remarks', 'parameter_name_consistency_in_product_service_quality', 'parameter_name_quality_of_product_service', 'parameter_name_market_credibility', 'paramter_name_terms_and_condition_competitiveness', 'parameter_name_adherence_to_delivery_schedule', 'parameter_name_pricing_competitiveness', 'parameter_name_quality_of_support_services', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsSupplierReevaluation::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'supplier_id' => $this->supplier_id,
            'evaluation_date' => $this->evaluation_date,
            'next_evaluation_date' => $this->next_evaluation_date,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'supplier_address', $this->supplier_address])
            ->andFilterWhere(['like', 'suplier_rating', $this->suplier_rating])
            ->andFilterWhere(['like', 'suplier_contact', $this->suplier_contact])
            ->andFilterWhere(['like', 'evaluation_analysis', $this->evaluation_analysis])
            ->andFilterWhere(['like', 'supplier_score', $this->supplier_score])
            ->andFilterWhere(['like', 'remarks', $this->remarks])
            ->andFilterWhere(['like', 'parameter_name_consistency_in_product_service_quality', $this->parameter_name_consistency_in_product_service_quality])
            ->andFilterWhere(['like', 'parameter_name_quality_of_product_service', $this->parameter_name_quality_of_product_service])
            ->andFilterWhere(['like', 'parameter_name_market_credibility', $this->parameter_name_market_credibility])
            ->andFilterWhere(['like', 'paramter_name_terms_and_condition_competitiveness', $this->paramter_name_terms_and_condition_competitiveness])
            ->andFilterWhere(['like', 'parameter_name_adherence_to_delivery_schedule', $this->parameter_name_adherence_to_delivery_schedule])
            ->andFilterWhere(['like', 'parameter_name_pricing_competitiveness', $this->parameter_name_pricing_competitiveness])
            ->andFilterWhere(['like', 'parameter_name_quality_of_support_services', $this->parameter_name_quality_of_support_services])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
