<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsSupplier;

/**
 * QmsSupplierSearch represents the model behind the search form of `common\modules\qms\models\QmsSupplier`.
 */
class QmsSupplierSearch extends QmsSupplier
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'number_of_employees', 'annual_turnover_millions', 'approving_authority_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'address', 'email', 'phone', 'supplier_contact_name', 'supplier_contact_phone', 'supplier_category', 'product_service_details', 'certification_type', 'other_certification_type', 'existing_major_customers', 'govt_or_tax_registration_number', 'major_registration_criterias', 'supplier_rating', 'sample_approval_details', 'is_active', 'approval_status', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsSupplier::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'number_of_employees' => $this->number_of_employees,
            'annual_turnover_millions' => $this->annual_turnover_millions,
            'approving_authority_id' => $this->approving_authority_id,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'name', $this->name])
            ->andFilterWhere(['like', 'address', $this->address])
            ->andFilterWhere(['like', 'email', $this->email])
            ->andFilterWhere(['like', 'phone', $this->phone])
            ->andFilterWhere(['like', 'supplier_contact_name', $this->supplier_contact_name])
            ->andFilterWhere(['like', 'supplier_contact_phone', $this->supplier_contact_phone])
            ->andFilterWhere(['like', 'supplier_category', $this->supplier_category])
            ->andFilterWhere(['like', 'product_service_details', $this->product_service_details])
            ->andFilterWhere(['like', 'certification_type', $this->certification_type])
            ->andFilterWhere(['like', 'other_certification_type', $this->other_certification_type])
            ->andFilterWhere(['like', 'existing_major_customers', $this->existing_major_customers])
            ->andFilterWhere(['like', 'govt_or_tax_registration_number', $this->govt_or_tax_registration_number])
            ->andFilterWhere(['like', 'major_registration_criterias', $this->major_registration_criterias])
            ->andFilterWhere(['like', 'supplier_rating', $this->supplier_rating])
            ->andFilterWhere(['like', 'sample_approval_details', $this->sample_approval_details])
            ->andFilterWhere(['like', 'is_active', $this->is_active])
            ->andFilterWhere(['like', 'approval_status', $this->approval_status])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
