<?php

namespace common\modules\qms\models;

use Yii;
use yii\db\Expression;
use common\modules\user\models\User;

/**
 * This is the model class for table "{{%qms_work_order}}".
 *
 * @property integer $id
 * @property integer $work_order_owner_id
 * @property string $work_order_title
 * @property string $work_order_description
 * @property string $proposal_completion_date
 * @property string $remarks
 * @property string $is_from_enquiry
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property QmsFinalInspectionQualityRecords[] $qmsFinalInspectionQualityRecords
 * @property QmsInProcessInspectionRecords[] $qmsInProcessInspectionRecords
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $workOrderOwner
 */
class QmsWorkOrder extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '{{%qms_work_order}}';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['work_order_owner_id', 'work_order_title', 'work_order_description', 'proposal_completion_date'], 'required'],
            [['work_order_owner_id', 'created_by', 'updated_by'], 'integer'],
            [['work_order_title', 'work_order_description', 'remarks', 'is_from_enquiry', 'status'], 'string'],
            [['proposal_completion_date'], 'safe'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['work_order_owner_id'], 'exist', 'skipOnError' => true, 'targetClass' => User::className(), 'targetAttribute' => ['work_order_owner_id' => 'id']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'work_order_owner_id' => 'Work Order Owner ID',
            'work_order_title' => 'Work Order Title',
            'work_order_description' => 'Work Order Description',
            'proposal_completion_date' => 'Proposal Completion Date',
            'remarks' => 'Remarks',
            'is_from_enquiry' => 'Is From Enquiry',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsFinalInspectionQualityRecords()
    {
        return $this->hasMany(QmsFinalInspectionQualityRecords::className(), ['work_order_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getQmsInProcessInspectionRecords()
    {
        return $this->hasMany(QmsInProcessInspectionRecords::className(), ['work_order_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getWorkOrderOwner()
    {
        return $this->hasOne(User::className(), ['id' => 'work_order_owner_id']);
    }
}
