<?php

namespace common\modules\qms\models;

use Yii;
use yii\base\Model;
use yii\data\ActiveDataProvider;
use common\modules\qms\models\QmsWorkOrder;

/**
 * QmsWorkOrderSearch represents the model behind the search form of `common\modules\qms\models\QmsWorkOrder`.
 */
class QmsWorkOrderSearch extends QmsWorkOrder
{
    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'work_order_owner_id', 'created_by', 'updated_by'], 'integer'],
            [['work_order_title', 'work_order_description', 'proposal_completion_date', 'remarks', 'is_from_enquiry', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = QmsWorkOrder::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'work_order_owner_id' => $this->work_order_owner_id,
            'proposal_completion_date' => $this->proposal_completion_date,
            'created_by' => $this->created_by,
            'updated_by' => $this->updated_by,
            'created_at' => $this->created_at,
            'updated_at' => $this->updated_at,
        ]);

        $query->andFilterWhere(['like', 'work_order_title', $this->work_order_title])
            ->andFilterWhere(['like', 'work_order_description', $this->work_order_description])
            ->andFilterWhere(['like', 'remarks', $this->remarks])
            ->andFilterWhere(['like', 'is_from_enquiry', $this->is_from_enquiry])
            ->andFilterWhere(['like', 'status', $this->status]);

        return $dataProvider;
    }
}
