<?php

use yii\bootstrap4\Html;
use yii\bootstrap4\ActiveForm;
use kartik\select2\Select2;
use kartik\date\DatePicker;

use common\modules\user\models\User;
use common\modules\qms\models\QmsRiskOpportunity;
use common\modules\qms\models\QmsMasterManagementSystemType;
use common\modules\qms\models\QmsMasterRiskCategory;
use common\modules\qms\models\QmsDepartment;

/* @var $this yii\web\View */
/* @var $model common\modules\qms\models\QmsRiskOpportunity */
/* @var $form yii\bootstrap4\ActiveForm */
?>

<div class="qms-risk-opportunity-form">

    <?php $form = ActiveForm::begin(); ?>

    <?= $form->field($model, 'risk_and_opportunity_type_id')->widget(Select2::classname(), [
        'data' => QmsRiskOpportunity::getAll($k = 'id', $v = 'name'),
        'options' => ['placeholder' => 'Select Risk And Opportunity'],
        'pluginOptions' => [
            'allowClear' => true
        ],
    ])->label('Risk And Opportunity'); ?>

    <?= $form->field($model, 'management_system_type_id')->widget(Select2::classname(), [
        'data' => QmsMasterManagementSystemType::getAll($k = 'id', $v = 'name'),
        'options' => ['placeholder' => 'Select Management System Type'],
        'pluginOptions' => [
            'allowClear' => true
        ],
    ])->label('Management System Type'); ?>

    <?= $form->field($model, 'risk_opportunity_source_or_activity')->textarea(['rows' => 6]) ?>

    <?= $form->field($model, 'risk_category_id')->widget(Select2::classname(), [
        'data' => QmsMasterRiskCategory::getAll($k = 'id', $v = 'name'),
        'options' => ['placeholder' => 'Select Risk Category'],
        'pluginOptions' => [
            'allowClear' => true
        ],
    ])->label('Risk Category'); ?>

    <?= $form->field($model, 'risk_description')->textarea(['rows' => 6]) ?>

    <?= $form->field($model, 'risk_impact')->dropDownList([ 'low' => 'Low', 'medium' => 'Medium', 'high' => 'High', ], ['prompt' => '']) ?>

    <?= $form->field($model, 'risk_rating')->textInput(['maxlength' => true]) ?>

    <?= $form->field($model, 'risk_probability')->dropDownList([ 'low' => 'Low', 'medium' => 'Medium', 'high' => 'High', ], ['prompt' => '']) ?>

    <?= $form->field($model, 'assigned_to_id')->widget(Select2::classname(), [
        'data' => User::getAll($k = 'id', $v = 'email'),
        'options' => ['placeholder' => 'Select Assigned To'],
        'pluginOptions' => [
            'allowClear' => true
        ],
    ])->label('Assigned To'); ?>

    <?= $form->field($model, 'department_id')->widget(Select2::classname(), [
        'data' => QmsDepartment::getAll($k = 'id', $v = 'name'),
        'options' => ['placeholder' => 'Select Department'],
        'pluginOptions' => [
            'allowClear' => true
        ],
    ])->label('Department'); ?>

    <?= $form->field($model, 'actions_proposed')->textarea(['rows' => 6]) ?>

    <?= $form->field($model, 'treatment_type')->dropDownList([ 'elimination' => 'Elimination', 'reduction' => 'Reduction', 'retention' => 'Retention', 'transfer' => 'Transfer', ], ['prompt' => '']) ?>

    <?= $form->field($model, 'proposed_review_date')->widget(DatePicker::classname(), [
        'type' => DatePicker::TYPE_COMPONENT_APPEND,
        'pluginOptions' => [
            'autoclose' => true,
            'format' => 'yyyy-mm-dd'
        ]
    ]); ?>

    <?= $form->field($model, 'proposed_completion_date')->widget(DatePicker::classname(), [
        'type' => DatePicker::TYPE_COMPONENT_APPEND,
        'pluginOptions' => [
            'autoclose' => true,
            'format' => 'yyyy-mm-dd'
        ]
    ]); ?>

    <?= $form->field($model, 'is_it_an_opportunity')->textInput(['maxlength' => true]) ?>

    <?= $form->field($model, 'approval_status')->dropDownList([ 'approved' => 'Approved', 'rejected' => 'Rejected', 'pending' => 'Pending', ], ['prompt' => '']) ?>

    <?= $form->field($model, 'approving_authority_id')->textInput() ?>

    <div class="form-group">
        <?= Html::submitButton($model->isNewRecord ? 'Create' : 'Update', ['class' => $model->isNewRecord ? 'btn btn-success' : 'btn btn-primary']) ?>
    </div>

    <?php ActiveForm::end(); ?>

</div>
