<?php

namespace common\modules\shop\components;

use Yii;
use yii\base\Component;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreValue;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopVoucher;
use Yadakhov\Json;

class Cart extends Component 
{
	public static function getCartSubtotal() 
	{
		$cart_subtotal = 0;
		$cart_session = $_SESSION['cart'];


		foreach ($cart_session as $cart_session_key => $cart_session_value) {

			if (!is_array($cart_session_value)) {
				continue;
			}

			$cart_subtotal = $cart_subtotal + ($cart_session_value['price'] * $cart_session_value['quantity']);
		}

		return $cart_subtotal;
	}

	public static function getCartTotal() 
	{
		$cart_total = 0;
		$discount = 0;
		$cart_session = $_SESSION['cart'];

		foreach ($cart_session as $cart_session_key => $cart_session_value) {
			if (!is_array($cart_session_value)) {
				continue;
			}

			$cart_total = $cart_total + ($cart_session_value['price'] * $cart_session_value['quantity']);
		}

		if (isset($_SESSION['cart']['voucher_id']) && !empty($_SESSION['cart']['voucher_id'])) {
			$discount = self::calculateVoucherDiscount($_SESSION['cart']['voucher_id'], $cart_total);
		}

		$cart_total = $cart_total - $discount;

		$cart_total = Globals::formatNumber($cart_total, ['round' => true, 'precision' => 2]);

		return $cart_total;
	}

	public static function calculateVoucherDiscount($voucher_id, $price)
	{
		$voucher_discount = 0;
		$modelShopVoucher = new ShopVoucher();

		if (ShopVoucher::find()->where(['id' => $voucher_id, 'status' => 'active'])->exists()) {
			$modelShopVoucher = ShopVoucher::findOne(['id' => $voucher_id, 'status' => 'active']);

			if ($modelShopVoucher->discount_percent == 'yes') {
				$voucher_discount = ($modelShopVoucher->discount * $price) / 100;
			} else {
				$voucher_discount = $modelShopVoucher->discount;
			}
		}
		
		$voucher_discount = Globals::formatNumber($voucher_discount, ['round' => true, 'precision' => 2]);

		return $voucher_discount;
	}

	public static function applyVoucher($voucher_code) 
	{
		if (!ShopVoucher::find()->where(['code' => $voucher_code, 'status' => 'active'])->exists()) {
			return ['error', 'Voucher not found.'];
		}

		$modelShopVoucher = ShopVoucher::findOne(['code' => $voucher_code, 'status' => 'active']);

		$voucher_metadata = new Json($modelShopVoucher->metadata);
		
		if (!Globals::isDateInRange(date('Y-m-d'), $modelShopVoucher->start_date, $modelShopVoucher->end_date)) {
			return ['error', 'Voucher not active.'];
		}

		if (isset($_SESSION['cart']['discount']) && $_SESSION['cart']['discount'] > 0) {
			return ['error', 'Voucher already applied.'];
		}

		if (!empty($voucher_metadata->get('is_limited')) && $voucher_metadata->get('is_limited') == 'yes') {
			if (ShopOrder::find()->where(['voucher_id' => $modelShopVoucher->id])->count() >= $voucher_metadata->get('voucher_count')) {
				return ['error', 'Vouchers exhausted.'];
			}
		}

		if (isset($_SESSION['cart']['size']) && $_SESSION['cart']['size'] > 0) {
			$_SESSION['cart']['voucher_id'] = $modelShopVoucher->id;
			$_SESSION['cart']['discount'] = self::calculateVoucherDiscount($modelShopVoucher->id, self::getCartTotal());
			$_SESSION['cart']['cart_total'] = $_SESSION['cart']['cart_total'] - $_SESSION['cart']['discount'];

			return ['success', 'Voucher applied successfully.'];
		} else {
			return ['error', 'Cart is empty.'];
		}
	}
}
