<?php

namespace common\modules\shop\components;

use Yii;
use yii\base\Component;
use yii\helpers\ArrayHelper;
use yii\web\HttpException;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreValue;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\shop\models\ShopProduct;
use common\modules\accounting\models\AccountingCurrency;
use common\modules\shop\models\ShopCategory;
use common\modules\shop\models\ShopSubcategory;
use common\modules\lms\models\LmsCourse;
use Nuxia\BusinessDayManipulator\Manipulator;
use Nuxia\BusinessDayManipulator\LocalizedManipulator;

class Product extends Component 
{
	public static function getProductPrice($product_id, $currency_code = 'usd') 
	{
		if (!isset($product_id) || empty($product_id)) {
			throw new HttpException(500, 'Unable to display product price.');
		}

		if (!ShopProduct::find()->where(['id' => $product_id, 'status' => 'active'])->exists()) {
			throw new HttpException(404, 'Product not found.');
		}

		$modelProduct = ShopProduct::find()->where(['id' => $product_id, 'status' => 'active'])->one();

		// Return product's fixed-price, if defined
		if (isset($modelProduct->price) && is_numeric($modelProduct->price) && $modelProduct->price > 0) {
			return Globals::formatNumber(($modelProduct->price * AccountingCurrency::getAttr(['rate'], ['code' => (isset(Yii::$app->params['currency_code']) && !empty(Yii::$app->params['currency_code'])) ? Yii::$app->params['currency_code'] : $currency_code])), ['round' => true, 'precision' => 2]);
		}

		$product_cost = 0;
		$profit = 0;
		$product_price = 0;
		$exchange_rate = 1;
		
		$product_cost = $modelProduct->price;

		$product_price = $product_cost;
				
		$product_price = $product_price * AccountingCurrency::getAttr(['rate'], ['code' => $currency_code]);

		return Globals::formatNumber($product_price, ['round' => true, 'precision' => 2]);
	}

	public static function getPreviousProduct($product_id) 
	{
		if (!isset($product_id) || empty($product_id) || !ShopProduct::find()->where(['id' => $product_id, 'available' => 'yes', 'status' => 'active'])->exists()) {
			throw new HttpException(500, 'Unable to fetch previous product.');
		}

		$modelCurrentProduct = ShopProduct::find()->where(['id' => $product_id, 'available' => 'yes', 'status' => 'active'])->one();
		$first_product_id = ShopProduct::find()->where(['subcategory_id' => $modelCurrentProduct->subcategory_id, 'available' => 'yes', 'status' => 'active'])->orderBy(['id' => SORT_ASC])->scalar();

		$current_product_id = $modelCurrentProduct->id;

		$current_product_id--;

		while (!ShopProduct::find()->where(['id' => $current_product_id, 'subcategory_id' => $modelCurrentProduct->subcategory_id, 'available' => 'yes', 'status' => 'active'])->exists()) {
			$current_product_id--;

			if ($current_product_id < $first_product_id) {
				return false;
			}
		}

		$modelProduct = ShopProduct::find()->where(['id' => $current_product_id, 'available' => 'yes', 'status' => 'active'])->one();

		return $modelProduct;
	}

	public static function getNextProduct($product_id) 
	{
		if (!isset($product_id) || empty($product_id) || !ShopProduct::find()->where(['id' => $product_id, 'available' => 'yes', 'status' => 'active'])->exists()) {
			throw new HttpException(500, 'Unable to fetch next product.');
		}

		$modelCurrentProduct = ShopProduct::find()->where(['id' => $product_id, 'available' => 'yes', 'status' => 'active'])->one();
		$last_product_id = ShopProduct::find()->where(['subcategory_id' => $modelCurrentProduct->subcategory_id, 'available' => 'yes', 'status' => 'active'])->orderBy(['id' => SORT_DESC])->scalar();

		$current_product_id = $modelCurrentProduct->id;

		$current_product_id++;

		while (!ShopProduct::find()->where(['id' => $current_product_id, 'subcategory_id' => $modelCurrentProduct->subcategory_id, 'available' => 'yes', 'status' => 'active'])->exists()) {
			$current_product_id++;

			if ($current_product_id > $last_product_id) {
				return false;
			}
		}

		$modelProduct = ShopProduct::find()->where(['id' => $current_product_id, 'available' => 'yes', 'status' => 'active'])->one();

		return $modelProduct;
	}

	public static function getProductDetails($model) 
	{
        //Getting Entity Class Id of product
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'product', 'status' => 'active']);
        $entityIdArray = array();
        $attrArray = array();
        
        //getting array of selected product ids
        foreach ($model as $key => $value) {
            array_push($entityIdArray, $value['id']);
        }
        //Finding details of selected Product
        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'entity_id' => $entityIdArray, 'status' => 'active'])->asArray()->all();   

        $modelValue = array();

        $attrArray = CoreEntityAttribute::getAttributeClassesOfEntityClass('product');
        
        foreach ($tmpArray as $key => $value) {
            if(array_key_exists($value['attribute_class_id'], $attrArray)) {
                $modelValue[$value['entity_id']][$attrArray[$value['attribute_class_id']]] = $value['value'];
            }
        }
        
        $subcategoryArr = ShopSubcategory::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'subcategory', $attribute_class = 'name');
        $subcategoryList = array_values($subcategoryArr);

        foreach($model as $key => $value) {
        	$modelValue[$value['id']]['id'] = $value['id'];
            $modelValue[$value['id']]['featured_image'] = $value['featured_image'];
            $modelValue[$value['id']]['price'] = $value['price'];
            $modelValue[$value['id']]['code'] = $value['code'];
            // $modelValue[$value['id']]['traits'] = $value['traits'];
            // $modelValue[$value['id']]['category'] = $value['category'];
            $modelValue[$value['id']]['subcategory'] = $subcategoryArr[$value['subcategory_id']];
            $modelValue[$value['id']]['category'] = CoreValue::getEntityValue('category', 'name', ShopSubcategory::getAttr(['category_id'], ['id' => $value['subcategory_id']]));

            if (LmsCourse::find()->where(['code' => $value['code'], 'status' => 'active'])->exists()) {
            	$modelValue[$value['id']]['course_metadata'] = json_decode(LmsCourse::getAttr(['metadata'], ['code' => $value['code']]), true);
            }
        }

        return $modelValue;
    }
}
