<?php

namespace common\modules\shop\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopProductSearch;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\cms\models\CmsEntityAsset;

/**
 * ProductController implements the CRUD actions for ShopProduct model.
 */
class ProductController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all ShopProduct models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ShopProductSearch();
        $modelCoreValue = new CoreValue();
        $modelCoreValueCategory = new CoreValue();
        $modelEntityClass = new CoreEntityClass();
        $modelAttributeClass = new CoreAttributeClass();

        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'product', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $modelCoreValue = array();

        foreach ($tmpArray as $key => $value) {
            $modelCoreValue[$value['entity_id']] = $value['value'];
        }

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'category', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $modelCoreValueCategory = array();

        foreach ($tmpArray as $key => $value) {
            $modelCoreValueCategory[$value['entity_id']] = $value['value'];
        }

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'subcategory', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $modelCoreValueSubcategory = array();

        foreach ($tmpArray as $key => $value) {
            $modelCoreValueSubcategory[$value['entity_id']] = $value['value'];
        }

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'modelCoreValue' => $modelCoreValue,
            'modelCoreValueCategory' => $modelCoreValueCategory,
            'modelCoreValueSubcategory' => $modelCoreValueSubcategory,
        ]);
    }

    /**
     * Displays a single ShopProduct model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new ShopProduct model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new ShopProduct();
        $modelCoreValue = new CoreValue();
        $modelCoreLanguage = new CoreLanguage();
        $modelProductImage = [new CmsEntityAsset];
        $attributes = array();
        $data = array();
        $product_traits = array();

        $modelCoreLanguage = CoreLanguage::find()->where(['status' => 'active'])->all();
		
        foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass('product') as $attributeClassKey => $attributeClassValue) {
            $tmp = array();
            
            foreach ($modelCoreLanguage as $langKey => $langValue) {
                $tmp[$langValue->id] = '';            
            }
            
            $$attributeClassValue = $tmp;
            $attributes[] = $attributeClassValue;
        }

        $traitsArray = \common\modules\shop\models\ShopTrait::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'trait', $attribute_class = 'name');
        
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        foreach($traitsArray as $key => $name){

            $product_traits[$key]['name'] = $name;

            $tmpArray = \common\modules\shop\models\ShopTraitValue::find()->where(['trait_id' => $key, 'status' => 'active'])->asArray()->all();
            $traitValueArr = array();
			
            foreach($tmpArray as $tmpkey => $value) {
                array_push($traitValueArr, $value['id']);
            }
			
            $traitValueArr = array_values($traitValueArr);

            $traitNamesArray = CoreValue::find()->select('entity_id, value')->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'entity_id' => $traitValueArr ,'status' => 'active'])->asArray()->all();
            
            foreach($traitNamesArray as $k => $v) {
                $product_traits[$key]['value'][$v['entity_id']]['name'] = $v['value'];
                $product_traits[$key]['value'][$v['entity_id']]['is_selected'] = 0;
            }
        }  

        if (Yii::$app->request->isPost) {
            if ($product_id = $model->saveProduct(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $product_id]);
            } else {
                $data['model'] = $model;
                $data['modelCoreValue'] = $modelCoreValue;
                $data['product_traits'] = $product_traits;
                $data['modelProductImage'] = (empty($modelProductImage)) ? [new CmsEntityAsset] : $modelProductImage;
                $data['modelCoreLanguage'] = $modelCoreLanguage;

                foreach ($attributes as $key => $value) {
                    $data[$value] = $$value;
                }

                return $this->render('create', [
                    'data' => $data,
                ]);    
            }
        } else {
            $data['model'] = $model;
            $data['modelCoreValue'] = $modelCoreValue;
            $data['product_traits'] = $product_traits;
            $data['modelProductImage'] = (empty($modelProductImage)) ? [new CmsEntityAsset] : $modelProductImage;
            $data['modelCoreLanguage'] = $modelCoreLanguage;

            foreach ($attributes as $key => $value) {
                $data[$value] = $$value;
            }

            return $this->render('create', [
                'data' => $data,
            ]);
        }
    }

    /**
     * Updates an existing ShopProduct model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $modelCoreValue = new CoreValue();
        $modelCoreLanguage = new CoreLanguage();
        $attributes = array();
        $data = array();
        $product_traits = array();
        $modelProductImage = [new CmsEntityAsset];
        $modelShopProductTraitValue = new \common\modules\shop\models\ShopProductTraitValue();
        
        $model = $this->findModel($id);
        $modelCoreLanguage = CoreLanguage::find()->where(['status' => 'active'])->all();

        $traitsArray = \common\modules\shop\models\ShopTrait::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'trait', $attribute_class = 'name');
        
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        foreach($traitsArray as $key => $name){

            $product_traits[$key]['name'] = $name;

            if(\common\modules\shop\models\ShopProductTraitValue::find()->where(['product_id' => $id, 'trait_id' => $key , 'status' => 'active'])->exists()) {
                $product_traits[$key]['is_selected'] = 1;
            } else {
                $product_traits[$key]['is_selected'] = 0;
            }

            $tmpArray = \common\modules\shop\models\ShopTraitValue::find()->where(['trait_id' => $key, 'status' => 'active'])->asArray()->all();
            $traitValueArr = array();
			
            foreach($tmpArray as $tmpkey => $value) {
                array_push($traitValueArr, $value['id']);
            }
			
            $traitValueArr = array_values($traitValueArr);

            $traitNamesArray = CoreValue::find()->select('entity_id, value')->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'entity_id' => $traitValueArr ,'status' => 'active'])->asArray()->all();
            
            foreach($traitNamesArray as $k => $v) {
                $product_traits[$key]['value'][$v['entity_id']]['name'] = $v['value'];

                if(\common\modules\shop\models\ShopProductTraitValue::find()->where(['product_id' => $id, 'trait_value_id' => $v['entity_id'], 'status' => 'active'])->exists()) {

                    $product_traits[$key]['value'][$v['entity_id']]['is_selected'] = 1;
                } else {
                    $product_traits[$key]['value'][$v['entity_id']]['is_selected'] = 0;
                }
            }
        }   
        
        foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass('product') as $attributeClassKey => $attributeClassValue) {
            $$attributeClassValue = array();
            $tmp = array();

            $temp = CoreValue::getEntityValue('product', $attributeClassValue, $id);
            
            if (!isset($temp)) {
                continue;
            }
            
            foreach ($modelCoreLanguage as $langKey => $langValue) {
                $tmp[$langValue->id] = CoreValue::getEntityValue('product', $attributeClassValue, $id, $langValue->id);            
            }
            
            $$attributeClassValue = $tmp;
            $attributes[] = $attributeClassValue;
        }

        $modelEntityClass = CoreEntityClass::findOne(['name' => 'product', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'status' => 'active'])->asArray()->all();
        
        $valueArray = array();

        foreach ($tmpArray as $key => $value) {
            $valueArray[$value['entity_id']] = $value['value'];
        }

        $modelProductImage = CmsEntityAsset::findAll(['entity_class_id' => $modelEntityClass->id, 'entity_id' => $id, 'status' => 'active']);

        if (Yii::$app->request->isPost) {
            if ($product_id = $model->saveProduct(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $product_id]);
            } else {
                $data['model'] = $model;
                $data['modelCoreValue'] = $modelCoreValue;
                $data['valueArray'] = $valueArray;
                $data['product_traits'] = $product_traits;
                $data['modelProductImage'] = (empty($modelProductImage)) ? [new CmsEntityAsset] : $modelProductImage;
                $data['modelCoreLanguage'] = $modelCoreLanguage;
				
                foreach ($attributes as $key => $value) {
                    $data[$value] = $$value;
                }

                return $this->render('update', [
                    'data' => $data,
                ]);    
            }
        } else {
            $data['model'] = $model;
            $data['modelCoreValue'] = $modelCoreValue;
            $data['valueArray'] = $valueArray;
            $data['product_traits'] = $product_traits;
            $data['modelProductImage'] = (empty($modelProductImage)) ? [new CmsEntityAsset] : $modelProductImage;
            $data['modelCoreLanguage'] = $modelCoreLanguage;
			
            foreach ($attributes as $key => $value) {
                $data[$value] = $$value;
            }
                
            return $this->render('update', [
                'data' => $data,
            ]);
        }
    }

    /**
     * Deletes an existing ShopProduct model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the ShopProduct model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return ShopProduct the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = ShopProduct::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
