<?php

namespace common\modules\shop\controllers\backend;

use Yii;
use yii\data\ActiveDataProvider;
use yii\data\ArrayDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;

use common\modules\shop\models\ShopTrait;
use common\modules\shop\models\ShopTraitSearch;
use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * TraitController implements the CRUD actions for ShopTrait model.
 */
class TraitController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'create', 'update', 'delete'],
                'rules' => [
                    [
                        'allow' => true,
                        'actions' => ['index', 'view', 'create', 'update', 'delete'],
                        'roles' => ['@'],
                        'matchCallback' => function ($rule, $action) {
                            return \common\modules\user\components\GenesysUser::isAllowed(Yii::$app->user->identity->role_id, Yii::$app->controller->module->id . '/' . Yii::$app->controller->id);
                        },
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all ShopTrait models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ShopTraitSearch();
        $modelCoreEntityClass = new CoreEntityClass();
        $modelCoreAttributeClass = new CoreAttributeClass();
		$modelCoreValue = new CoreValue();
        
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'trait', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $modelCoreValue = array();

        foreach ($tmpArray as $key => $value) {
            $modelCoreValue[$value['entity_id']] = $value['value'];
        }

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
            'modelCoreValue' => $modelCoreValue,
        ]);
    }

    /**
     * Displays a single ShopTrait model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new ShopTrait model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new ShopTrait();
        $modelCoreValue = new CoreValue();
        $modelCoreLanguage = new CoreLanguage();
        $attributes = array();
        $data = array();

        $modelCoreLanguage = CoreLanguage::find()->where(['status' => 'active'])->all();

        foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass('trait') as $attributeClassKey => $attributeClassValue) {
            $tmp = array();
            
            foreach ($modelCoreLanguage as $langKey => $langValue) {
                $tmp[$langValue->id] = '';            
            }
            
            $$attributeClassValue = $tmp;
            $attributes[] = $attributeClassValue;
        }
        
        if (Yii::$app->request->isPost) {
            if ($trait_id = $model->saveTrait(Yii::$app->request->post())) {
                return $this->redirect(['view', 'id' => $trait_id]);
            } else {
                $data['model'] = $model;
                $data['modelCoreValue'] = $modelCoreValue;
                $data['modelCoreLanguage'] = $modelCoreLanguage;
                
                foreach ($attributes as $key => $value) {
                    $data[$value] = $$value;
                }

                return $this->render('create', [
                    'data' => $data,
                ]);    
            }
        } else {
            $data['model'] = $model;
            $data['modelCoreValue'] = $modelCoreValue;
            $data['modelCoreLanguage'] = $modelCoreLanguage;
            
            foreach ($attributes as $key => $value) {
                $data[$value] = $$value;
            }


            return $this->render('create', [
                'data' => $data,
            ]);
        }
    }

    /**
     * Updates an existing ShopTrait model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $modelCoreValue = new CoreValue();
        $modelCoreLanguage = new CoreLanguage();
        $attributes = array();
        $data = array();

        $model = $this->findModel($id);
        $modelCoreLanguage = CoreLanguage::find()->where(['status' => 'active'])->all();
        
        foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass('trait') as $attributeClassKey => $attributeClassValue) {
            $$attributeClassValue = array();
            $tmp = array();

            $temp = CoreValue::getEntityValue('trait', $attributeClassValue, $id);
            
            if (!isset($temp)) {
                continue;
            }
            
            foreach ($modelCoreLanguage as $langKey => $langValue) {
                $tmp[$langValue->id] = CoreValue::getEntityValue('trait', $attributeClassValue, $id, $langValue->id);            
            }
            
            $$attributeClassValue = $tmp;
            $attributes[] = $attributeClassValue;
        }

        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'trait', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        $tmpArray = CoreValue::find()->where(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id, 'status' => 'active'])->asArray()->all();

        $valueArray = array();

        foreach ($tmpArray as $key => $value) {
            $valueArray[$value['entity_id']] = $value['value'];
        }

        if (Yii::$app->request->isPost) {
            if ($user_id = $model->saveTrait(Yii::$app->request->post(), $id)) {
                return $this->redirect(['view', 'id' => $user_id]);
            } else {
                $data['model'] = $model;
                $data['modelCoreValue'] = $modelCoreValue;
                $data['valueArray'] = $valueArray;
                $data['modelCoreLanguage'] = $modelCoreLanguage;

                foreach ($attributes as $key => $value) {
                    $data[$value] = $$value;
                }

                return $this->render('update', [
                    'data' => $data,
                ]);    
            }
        } else {
            $data['model'] = $model;
            $data['modelCoreValue'] = $modelCoreValue;
            $data['valueArray'] = $valueArray;
            $data['modelCoreLanguage'] = $modelCoreLanguage;

            foreach ($attributes as $key => $value) {
                $data[$value] = $$value;
            }
                
            return $this->render('update', [
                'data' => $data,
            ]);
        }
    }

    /**
     * Deletes an existing ShopTrait model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->softDelete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the ShopTrait model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return ShopTrait the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = ShopTrait::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
