<?php

namespace common\modules\shop\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\shop\components\Product;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopProductTrait;
use common\modules\shop\models\ShopProductTraitValue;
use common\modules\shop\models\ShopCategory;
use common\modules\shop\models\ShopSubcategory;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;

/**
 * Default controller for the `shop` module
 */
class CategoryController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['view'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'view' => ['get', 'post'],
                ],
            ],
        ];
    }

    public function actionView($category_id = null, $subcategory_id = null)
    {
        if (!ShopCategory::find()->where(['id' => $category_id, 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Category not found.', 404);
        }

        $modelSubcategory = new ShopSubcategory();
        $modelProduct = $modelProductTmp = new ShopProduct();
        $query_array = array();
        $subcategory_ids = array();
        $product_ids = array();
        $page_names = array();

        $model = ShopCategory::findOne(['id' => $category_id, 'status' => 'active']);
        $query_array['mod_shop_product.status'] = 'active';
        $query_array['available'] = 'yes';

        if (isset($category_id) && ShopCategory::find()->where(['id' => $category_id, 'status' => 'active'])->exists()) {
            $page_names[] = CoreValue::getEntityValue('category', 'name', $category_id);
            $modelSubcategory = ShopSubcategory::find()->where(['category_id' => $category_id, 'status' => 'active'])->asArray()->all();

            foreach ($modelSubcategory as $modelSubcategoryKey => $modelSubcategoryValue) {
                $subcategory_ids[] = $modelSubcategoryValue['id'];
            }
        }

        if (isset($subcategory_id) && ShopSubcategory::find()->where(['id' => $subcategory_id, 'status' => 'active'])->exists()) {
            $page_names[] = CoreValue::getEntityValue('subcategory', 'name', $subcategory_id);
            $subcategory_ids[] = $subcategory_id;
        }

        $modelProductTmp = ShopProduct::find()->where($query_array)->all();
        $query = ShopProduct::find()->where($query_array);
        
        if (!empty($subcategory_ids)) {
            $query->andWhere(['IN', 'subcategory_id', $subcategory_ids]);
        }

        $count = $query->count();
        $pagination = new \yii\data\Pagination(['totalCount' => $count, 'pageSize' => 3]);
        $modelProduct = $query->offset($pagination->offset)->limit($pagination->limit)->orderBy('code')->all();

        $productDetails = Product::getProductDetails($modelProduct);
        
        $subcategoryArr = ShopSubcategory::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'subcategory', $attribute_class = 'name');
        $subcategoryList = array_values($subcategoryArr);

        return $this->render('view', [
            'model' => $model,
            'modelProduct' => $modelProduct,
            'pagination' => $pagination,
            'page_names' => !empty($page_names) ? $page_names : ['Products'],
            'productDetails' => $productDetails,
            'subcategoryList' => $subcategoryList
        ]);
    }
}
