<?php

namespace common\modules\shop\controllers\frontend;

use Yii;
use yii\web\Controller;
use Dompdf\Dompdf;
use Dompdf\Options;

use common\modules\core\components\Globals;
use common\modules\user\models\User;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopRefund2;
use common\modules\shop\models\FormRefund2;
use common\modules\payment\models\PaymentLogs;

/**
 * Order controller
 */
class OrderController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => [],
                'rules' => [
                    [
                        'actions' => [''],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [''],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [

                ],
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionThankYou($id, $successful = true)
    {
        $session = Yii::$app->session->open();

        if ($successful === true) {
            $modelShopOrderArr = array();
            $modelShopOrderItemArr = array();
            $modelPaymentLogsArr = array();
            $payment_gateway = Yii::$app->params['payment_gateway'];

            if (!isset($_SESSION['gateway_call']) || $_SESSION['gateway_call'] != 'true') {
                throw new \yii\web\HttpException(422, 'Payment gateway not called.');
            }

            unset($_SESSION['gateway_call']);

            if (!ShopOrder::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
                throw new HttpException(422, 'Order not found.');
            }

            $modelShopOrderArr = ShopOrder::find()->where(['id' => $id, 'status' => 'active'])->one();
            $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $modelShopOrderArr['id'], 'status' => 'active'])->one();
            $modelPaymentLogsArr = PaymentLogs::find()->where(['order_id' => $id])->one();

            return $this->render('thank-you', [
                'modelShopOrderArr' => $modelShopOrderArr,
                'modelShopOrderItemArr' => $modelShopOrderItemArr,
                'modelPaymentLogsArr' => $modelPaymentLogsArr,
                'is_fallback' => ((isset($_SESSION['is_fallback']) && $_SESSION['is_fallback'] == 'true') ? true : false), 
                'payment_gateway' => ((isset($_SESSION['is_fallback']) && $_SESSION['is_fallback'] == 'true') ? Yii::$app->params['payment_gateway_fallback'][$_SESSION['payment_gateway_fallback_index']] : Yii::$app->params['payment_gateway']), 
            ]);
        } else {
            return $this->render('thank-you-failure', [
                'message' => 'Payment Declined', 
                'reason' => 'There was a problem processing your payment.', 
            ]);
        }
    }

    public function actionRefund2($order_id = '', $trans_id = '')
    {
        $model = new FormRefund2();
        $modelShopRefund2 = new ShopRefund2();
        $modelShopOrder = new ShopOrder();

        if (Yii::$app->request->isPost) {
            $modelShopOrder = new ShopOrder();

            $transaction = Yii::$app->db->beginTransaction();

            try {
                $modelShopRefund2->email = Yii::$app->request->post('FormRefund2')['email'];
                $modelShopRefund2->card_last_digits = (isset(Yii::$app->request->post('FormRefund2')['card_last_digits']) && Yii::$app->request->post('FormRefund2')['card_last_digits']) ? Yii::$app->request->post('FormRefund2')['card_last_digits'] : 'NA';
                $modelShopRefund2->domain = Yii::$app->request->post('FormRefund2')['domain'];
                $modelShopRefund2->transaction_id = Yii::$app->request->post('FormRefund2')['transaction_id'];
                $modelShopRefund2->initiated_by = 'user';
                $modelShopRefund2->ip_address = Yii::$app->request->getUserIP();
                $modelShopRefund2->approved = 'no';
                $modelShopRefund2->funnel = Yii::$app->params['site_code'];
                $modelShopRefund2->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                $modelShopRefund2->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;

                if ($modelShopRefund2->validate()) {
                    $modelShopRefund2->save();

                    $transaction->commit();

                    $data['email'] = Yii::$app->request->post('FormRefund2')['email'];
                    $data['card_last_digits'] = Yii::$app->request->post('FormRefund2')['card_last_digits'];
                    $data['domain'] = Yii::$app->request->post('FormRefund2')['domain'];
                    $data['transaction_id'] = Yii::$app->request->post('FormRefund2')['transaction_id'];
                    $data['initiated_by'] = 'user';

                    // cancellation request email to customer
                    \common\modules\site\components\Email::send(
                        'order-cancellation-customer', 
                        [
                            'html' => 'orderCancellationCustomer-html', 
                            'text' => 'orderCancellationCustomer-text'
                        ], 
                        [
                            'to' => $modelShopRefund2->email, 
                            'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                        ], 
                        NULL, 
                        ['data' => $data], 
                        Yii::$app->params['site_code'], 
                        Yii::$app->params['site_code']
                    );

                    Yii::$app->session->setFlash('success', "Refund request sent successfully.");

                    return $this->render('refund-2', [
                        'trans_id' => $trans_id, 
                        'model' => $model,
                        'modelShopOrder' => $modelShopOrder,
                        'order_id' => $order_id,
                    ]);
                } else {
                    Yii::$app->session->setFlash('error', "Unable to send refund request " . Globals::renderErrors($modelShopRefund2->getErrors()));

                    return $this->render('refund-2', [
                        'trans_id' => $trans_id, 
                        'model' => $model,
                        'modelShopOrder' => $modelShopOrder,
                        'order_id' => $order_id,
                    ]);
                }
            } catch (\Exception $e) {
                $transaction->rollBack();

                throw $e;
            }
        }

        return $this->render('refund-2', [
            'trans_id' => $trans_id, 
            'model' => $model,
            'modelShopOrder' => $modelShopOrder,
            'order_id' => $order_id,
        ]);
    }

    public function actionOrders()
    {
        if (Yii::$app->user->isGuest) {
            \yii\helpers\Url::remember();

            return $this->redirect('/login', 302);
        }

        $modelShopOrder = ShopOrder::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->all();
        
        return $this->render('orders', [
            'modelShopOrder' => $modelShopOrder,
        ]);
    }

    public function actionOrder($id)
    {
        $modelShopOrder = ShopOrder::find()->where(['id' => $id, 'status' => 'active'])->one();

        return $this->renderAjax('order', [
            'modelShopOrder' => $modelShopOrder, 
        ]);
    }

    public function actionInvoice($id)
    {
        $modelShopOrder = new ShopOrder();
        $modelUser = new User();
        $payment_gateway = Yii::$app->params['payment_gateway'];

        $modelShopOrder = ShopOrder::findOne(['id' => $id, 'status' => 'active']);
        $modelUser = User::findOne(['id' => $modelShopOrder->user_id, 'status' => 'active']);

        if (!isset($modelShopOrder->id) || empty($modelShopOrder->id)) {
            throw new NotFoundHttpException('Order does not exist.');
        }

        if (!isset($modelUser->id) || empty($modelUser->id)) {
            throw new NotFoundHttpException('User does not exist.');
        }

        if (PaymentLogs::find()->where(['order_id' => $id])->exists()) {
            $payment_gateway = 'fhtpay';
        } elseif (PaymentLogs::find()->where(['order_id' => $id])->exists()) {
            $payment_gateway = 'stripe';
        } elseif (PaymentLogs::find()->where(['order_id' => $id])->exists()) {
            $payment_gateway = 'authorizenet';
        }
        
        // instantiate and use the dompdf class
        $dompdf = new Dompdf();
        $options = new Options();

        $options->setIsRemoteEnabled(true);
        $options->setIsPhpEnabled(true);
        
        $dompdf->setOptions($options);
        $dompdf->loadHtml($this->renderPartial('invoice', [
            'modelShopOrder' => $modelShopOrder,
            'modelUser' => $modelUser,
            'payment_gateway' => $payment_gateway,
        ]));
        $dompdf->setPaper('A4', 'portrait');    // (Optional) Setup the paper size and orientation      
        @$dompdf->render();  // Render the HTML as PDF
        $canvas = $dompdf->getCanvas();

        $canvas->page_script('
            $font = $fontMetrics->getFont("helvetica", "bold");
            $font = $fontMetrics->getFont("helvetica", "normal");

            $pdf->text(35, 800, Yii::$app->params[\'site\'][\'name\'] . " - ' . Yii::$app->params['site']['invoice_address'] . '", $font, 10, array(0,0,0));
        ');
        
        $dompdf->stream("dompdf_out", array("Attachment" => false));    // Output the generated PDF to Browser

        exit(0);
    }
}
