<?php

namespace common\modules\shop\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\core\components\Globals;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopRefund2;
use common\modules\shop\models\FormRefund2;

/**
 * Refund2 controller for the `shop` module
 */
class Refund2Controller extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'index' => ['get', 'post'],
                ],
            ],
        ];
    }

    public function actionIndex($order_id = '', $trans_id = '')
    {
        $model = new FormRefund2();
        $modelShopRefund2 = new ShopRefund2();
        $modelShopOrder = new ShopOrder();

        if (Yii::$app->request->isPost) {
            $modelShopOrder = new ShopOrder();

            $transaction = Yii::$app->db->beginTransaction();

            try {
                $modelShopRefund2->email = Yii::$app->request->post('FormRefund2')['email'];
                $modelShopRefund2->card_last_digits = Yii::$app->request->post('FormRefund2')['card_last_digits'];
                $modelShopRefund2->domain = Yii::$app->request->post('FormRefund2')['domain'];
                $modelShopRefund2->transaction_id = Yii::$app->request->post('FormRefund2')['transaction_id'];
                $modelShopRefund2->initiated_by = 'user';
                $modelShopRefund2->ip_address = Yii::$app->request->getUserIP();
                $modelShopRefund2->approved = 'no';
                $modelShopRefund2->funnel = Yii::$app->params['site_code'];
                $modelShopRefund2->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                $modelShopRefund2->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;

                if ($modelShopRefund2->validate()) {
                    $modelShopRefund2->save();

                    $transaction->commit();

                    $data['email'] = Yii::$app->request->post('FormRefund2')['email'];
                    $data['card_last_digits'] = Yii::$app->request->post('FormRefund2')['card_last_digits'];
                    $data['domain'] = Yii::$app->request->post('FormRefund2')['domain'];
                    $data['transaction_id'] = Yii::$app->request->post('FormRefund2')['transaction_id'];
                    $data['initiated_by'] = 'user';

                    // cancellation request email to customer
                    \common\modules\site\components\Email::send(
                        'mario-' . Yii::$app->request->post('FormRefund2')['service'] . '-order-cancellation-customer', 
                        [
                            'html' => 'mario-' . Yii::$app->request->post('FormRefund2')['service'] . '-order-cancellation-customer-html', 
                            'text' => 'mario-' . Yii::$app->request->post('FormRefund2')['service'] . '-order-cancellation-customer-text'
                        ], 
                        [
                            'to' => $modelShopRefund2->email, 
                            'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                        ], 
                        NULL, 
                        ['data' => $data], 
                        Yii::$app->params['site_code'], 
                        Yii::$app->params['site_code']
                    );

                    Yii::$app->session->setFlash('success', "Refund request sent successfully.");

                    return $this->render('index', [
                        'trans_id' => $trans_id, 
                        'model' => $model,
                        'modelShopOrder' => $modelShopOrder,
                        'order_id' => $order_id,
                    ]);
                } else {
                    Yii::$app->session->setFlash('error', "Unable to send refund request " . Globals::renderErrors($modelShopRefund2->getErrors()));

                    return $this->render('index', [
                        'trans_id' => $trans_id, 
                        'model' => $model,
                        'modelShopOrder' => $modelShopOrder,
                        'order_id' => $order_id,
                    ]);
                }
            } catch (\Exception $e) {
                $transaction->rollBack();

                throw $e;
            }
        }

        return $this->render('index', [
            'trans_id' => $trans_id, 
            'model' => $model,
            'modelShopOrder' => $modelShopOrder,
            'order_id' => $order_id,
        ]);
    }
}