<?php

namespace common\modules\shop\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\helpers\ArrayHelper;

use common\modules\shop\components\Product;
use common\modules\shop\components\Cart;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopProductTrait;
use common\modules\shop\models\ShopProductTraitValue;
use common\modules\shop\models\ShopCategory;
use common\modules\shop\models\ShopSubcategory;
use common\modules\shop\models\ShopCart;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopVoucher;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreSite;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\site\models\SiteReview;
use common\modules\payment\models\PaymentMethod;
use common\modules\payment\models\PaymentLogs;

/**
 * Default controller for the `shop` module
 */
class ShopController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'view', 'cart', 'checkout', 'thank-you', 'product-added'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['checkout', 'thank-you'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'view', 'cart', 'product-added'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'products' => ['get', 'post'],
                    'view' => ['get', 'post'],
                    'cart' => ['get', 'post'],
                    'checkout' => ['get', 'post'],
                    'thank-you' => ['get', 'post'],
                    'product-added' => ['get', 'post'],
                ],
            ],
        ];
    }

    public function beforeAction($action) {
        $this->enableCsrfValidation = false;

        return parent::beforeAction($action);
    }

    public function actionIndex($category_id = null, $subcategory_id = null)
    {
        $modelSubcategory = new ShopSubcategory();
        $modelProduct = $modelProductTmp = new ShopProduct();
        $query_array = array();
        $subcategory_ids = array();
        $product_ids = array();
        $page_names = array();

        $query_array['mod_shop_product.status'] = 'active';
        $query_array['available'] = 'yes';

        if (isset($category_id) && ShopCategory::find()->where(['id' => $category_id, 'status' => 'active'])->exists()) {
            $page_names[] = CoreValue::getEntityValue('category', 'name', $category_id);
            $modelSubcategory = ShopSubcategory::find()->where(['category_id' => $category_id, 'status' => 'active'])->asArray()->all();

            foreach ($modelSubcategory as $modelSubcategoryKey => $modelSubcategoryValue) {
                $subcategory_ids[] = $modelSubcategoryValue['id'];
            }
        }

        if (isset($subcategory_id) && ShopSubcategory::find()->where(['id' => $subcategory_id, 'status' => 'active'])->exists()) {
            $page_names[] = CoreValue::getEntityValue('subcategory', 'name', $subcategory_id);
            $subcategory_ids[] = $subcategory_id;
        }

        $modelProductTmp = ShopProduct::find()->where($query_array)->all();
        $query = ShopProduct::find()->where($query_array);
        
        if (!empty($subcategory_ids)) {
            $query->andWhere(['IN', 'subcategory_id', $subcategory_ids]);
        }

        $count = $query->count();
        $pagination = new \yii\data\Pagination(['totalCount' => $count, 'pageSize' => 12]);
        $modelProduct = $query->offset($pagination->offset)->limit($pagination->limit)->orderBy('code')->all();

        $products = Product::getProductDetails($modelProduct);
        
        $subcategoryArr = ShopSubcategory::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'subcategory', $attribute_class = 'name');
        $subcategoryList = array_values($subcategoryArr);

        return $this->render('index', [
            'modelProduct' => $modelProduct,
            'pagination' => $pagination,
            'page_names' => !empty($page_names) ? $page_names : ['Products'],
            'products' => $products,
            'subcategoryList' => $subcategoryList
        ]);
    }

    public function actionView($id)
    {
        if (!ShopProduct::find()->where(['id' => $id, 'available' => 'yes', 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Product not found.', 404);
        }

        $modelSiteReview = new SiteReview();

        $query_array = array();
        $product_traits = array();
        $rating_avg = 0;

        $query_array['id'] = $id;
        $query_array['status'] = 'active';

        if (isset($query_array['code'])) {
            unset($query_array['id']);
        }

        $model = ShopProduct::findOne($query_array);

        $modelShopProductTraitValue = ShopProductTraitValue::find()->select('trait_id, trait_value_id')->where(['product_id' => $id, 'status' => 'active'])->distinct()->asArray()->all();
		
        if (!empty($modelShopProductTraitValue)) {
            $traitArr = ArrayHelper::getColumn($modelShopProductTraitValue, 'trait_id');
            $traitValueArr = ArrayHelper::getColumn($modelShopProductTraitValue, 'trait_value_id');

            $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait', 'status' => 'active']);
            $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);
            $traits = CoreValue::find()->select('entity_id, value')->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'entity_id' => array_unique($traitArr) , 'status' => 'active'])->asArray()->all();

            $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
            $traitValueNamesArray = CoreValue::find()->select('entity_id, value')->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'entity_id' => $traitValueArr , 'status' => 'active'])->asArray()->all();

            foreach ($modelShopProductTraitValue as $key => $value) { 
                foreach ($traits as $trait) {
                    if ($value['trait_id'] == $trait['entity_id']) {
                        $product_traits[$value['trait_id']]['name'] = $trait['value'];
                    }
                }

                foreach ($traitValueNamesArray as $k => $v) {
                    if ($value['trait_value_id'] == $v['entity_id']) {
                        $product_traits[$value['trait_id']]['value'][$v['entity_id']] = $v['value'];
                    }
                }
            }
        } else {
            
        }
        
        $modelPreviousProduct = new ShopProduct();
        $modelNextProduct = new ShopProduct();

        $productArr = Product::getProductDetails(array($model));
        $product = array();

        foreach ($productArr as $key => $value) {
            $product = $value;
        }

        $modelPreviousProduct = Product::getPreviousProduct($model->id);
        $modelNextProduct = Product::getNextProduct($model->id);

        $reviews = SiteReview::find()->where(['site_id' => CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]), 'entity_class_id' => CoreEntityClass::getAttr(['id'], ['name' => 'product']), 'entity_id' => $id ,'status' => 'active', 'visible' => 'yes'])->asArray()->all();

        foreach ($reviews as $review) {
            $rating_avg += $review['rating'];
        }

        if (count($reviews) > 0) {
            $rating_avg = floatval($rating_avg / count($reviews));
        } else {
            $rating_avg = 0;
        }

        return $this->render('view', [
            'model' => $model,
            'product' => $product,
            'product_traits' => $product_traits,
            'reviews' => $reviews,
            'rating_avg' => $rating_avg,
        ]);
    }

    public function actionCustomize($id)
    {
        if (!ShopProduct::find()->where(['id' => $id, 'available' => 'yes', 'status' => 'active'])->exists()) {
            throw new \yii\web\NotFoundHttpException('Product not found.', 404);
        }

        $query_array = array();
        $product_traits = array();

        $query_array['id'] = $id;
        $query_array['status'] = 'active';

        if (isset($query_array['code'])) {
            unset($query_array['id']);
        }

        $model = ShopProduct::findOne($query_array);

        $modelShopProductTrait = ShopProductTraitValue::find()->select('trait_id,trait_value_id')->where(['product_id' => $id, 'status' => 'active'])->distinct()->asArray()->all();
        
        if (!empty($modelShopProductTrait)) {
            $traitArr = ArrayHelper::getColumn($modelShopProductTrait, 'trait_id');
            $traitValueArr = ArrayHelper::getColumn($modelShopProductTrait, 'trait_value_id');

            $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait', 'status' => 'active']);
            $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);
            $traitNamesArray = CoreValue::find()->select('entity_id, value')->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'entity_id' => array_unique($traitArr) , 'status' => 'active'])->asArray()->all();

            $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
            $traitValueNamesArray = CoreValue::find()->select('entity_id, value')->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'entity_id' => $traitValueArr , 'status' => 'active'])->asArray()->all();

            foreach ($modelShopProductTrait as $key => $value) { 
                foreach ($traitNamesArray as $id => $val) {
                    if ($value['trait_id'] == $val['entity_id']) {
                        $product_traits[$value['trait_id']]['name'] = $val['value'];
                    }
                }
                foreach ($traitValueNamesArray as $k => $v) {
                    if ($value['trait_value_id'] == $v['entity_id']) {
                        $product_traits[$value['trait_id']]['value'][$v['entity_id']] = $v['value'];
                    }
                }
            }
        }   
        
        $modelPreviousProduct = new ShopProduct();
        $modelNextProduct = new ShopProduct();

        $productArr = Product::getProductDetails(array($model));
        $productDetails = array();
        
        foreach ($productArr as $key => $value) {
            $productDetails = $value;
        }

        return $this->render('customize', [
            'model' => $model,
            'productDetails' => $productDetails,
            'product_traits' => $product_traits,
        ]);
    }

    public function actionSearch($category_id = null, $subcategory_id = null)
    {
        $modelSubcategory = new ShopSubcategory();
        $modelProduct = $modelProductTmp = new ShopProduct();
        $query_array = array();
        $subcategory_ids = array();
        $product_ids = array();
        $page_names = array();

        $query_array['mod_shop_product.status'] = 'active';
        $query_array['available'] = 'yes';

        if (isset($category_id) && ShopCategory::find()->where(['id' => $category_id, 'status' => 'active'])->exists()) {
            $page_names[] = CoreValue::getEntityValue('category', 'name', $category_id);
            $modelSubcategory = ShopSubcategory::find()->where(['category_id' => $category_id, 'status' => 'active'])->asArray()->all();

            foreach ($modelSubcategory as $modelSubcategoryKey => $modelSubcategoryValue) {
                $subcategory_ids[] = $modelSubcategoryValue['id'];
            }
        }

        if (isset($subcategory_id) && ShopSubcategory::find()->where(['id' => $subcategory_id, 'status' => 'active'])->exists()) {
            $page_names[] = CoreValue::getEntityValue('subcategory', 'name', $subcategory_id);
            $subcategory_ids[] = $subcategory_id;
        }

        $modelProductTmp = ShopProduct::find()->where($query_array)->all();
        $query = ShopProduct::find()->where($query_array);
        
        if (!empty($subcategory_ids)) {
            $query->andWhere(['IN', 'subcategory_id', $subcategory_ids]);
        }

        $count = $query->count();
        $pagination = new \yii\data\Pagination(['totalCount' => $count, 'pageSize' => 9]);
        $modelProduct = $query->offset($pagination->offset)->limit($pagination->limit)->orderBy('code')->all();

        $productDetails = Product::getProductDetails($modelProduct);
        
        $subcategoryArr = ShopSubcategory::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'subcategory', $attribute_class = 'name');
        $subcategoryList = array_values($subcategoryArr);

        return $this->render('search-result', [
            'modelProduct' => $modelProduct,
            'pagination' => $pagination,
            'page_names' => !empty($page_names) ? $page_names : ['Products'],
            'productDetails' => $productDetails,
            'subcategoryList' => $subcategoryList
        ]);
    }

    public function actionCart()
    {
        $session = Yii::$app->session->open();

        $breakFlag = false;
        $modelShopCart = new ShopCart();
        
        $this->refreshCartSessArray();
        
        $this->cartSessDbSync();
        
        if (Yii::$app->request->isPost) {
            if (Yii::$app->request->post('action') == 'update') {
                $_SESSION['cart'][Yii::$app->request->post('id')]['quantity'] = Yii::$app->request->post('quantity');

                $_SESSION['cart']['cart_total'] = Cart::getCartTotal();

                if (!Yii::$app->user->isGuest) {
                    $modelShopCart->saveCart($_SESSION['cart'], 'update', Yii::$app->request->post('id'));
                }
            } else if (Yii::$app->request->post('action') == 'remove') {
                unset($_SESSION['cart'][Yii::$app->request->post('id')]);

                $_SESSION['cart']['size'] = count($_SESSION['cart']) - 5;

                $_SESSION['cart']['cart_total'] = Cart::getCartTotal();

                if (!Yii::$app->user->isGuest) {
                    $modelShopCart->saveCart($_SESSION['cart'], 'remove', Yii::$app->request->post('id'));
                }
            } else if (Yii::$app->request->post('action') == 'apply-voucher') {
                if (!empty(Yii::$app->request->post('voucher_code'))) {
                    $tmp = Cart::applyVoucher(Yii::$app->request->post('voucher_code'));

                    Yii::$app->session->setFlash($tmp[0], $tmp[1]);
                } else {
                    Yii::$app->session->setFlash('error', "Invalid voucher code entered.");
                }
            } else {
                if (isset($_SESSION['cart']['db_sync'])) {
                    $_SESSION['cart']['db_sync'] = $_SESSION['cart']['db_sync'];
                } else {
                    $_SESSION['cart']['db_sync'] = 0;
                }

                if (isset($_SESSION['cart']['size'])) {
                    $_SESSION['cart']['size'] = $_SESSION['cart']['size'] + 1;
                } else {
                    $_SESSION['cart']['size'] = 1;
                }

                if (isset($_SESSION['cart']['voucher_id'])) {
                    $_SESSION['cart']['voucher_id'] = $_SESSION['cart']['voucher_id'];
                } else {
                    $_SESSION['cart']['voucher_id'] = 0;
                }

                if (isset($_SESSION['cart']['discount'])) {
                    $_SESSION['cart']['discount'] = $_SESSION['cart']['discount'];
                } else {
                    $_SESSION['cart']['discount'] = 0;
                }
                
                $_SESSION['cart']['cart_total'] = Cart::getCartTotal();

                $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['user_id'] = Yii::$app->user->isGuest ? '' : Yii::$app->user->identity->id;
                $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['product_id'] = Yii::$app->request->post('product_id');
                $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['order_id'] = '';
                $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['quantity'] = empty(Yii::$app->request->post('quantity')) ? '1' : Yii::$app->request->post('quantity');
                $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['price'] = Globals::formatNumber(Yii::$app->request->post('price'),  ['round' => true, 'precision' => 2]);
                $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['ip_address'] = Yii::$app->request->getUserIP();

                $_SESSION['cart']['cart_total'] = Cart::getCartTotal();
                
                if (!Yii::$app->user->isGuest) {
                    $modelShopCart->saveCart($_SESSION['cart']);

                    if (ShopCart::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
                        for ($i = 0; $i < $_SESSION['cart']['size']; $i++) { 
                            unset($_SESSION['cart'][$i]);
                        }
                
                        // loading cart from db to session
                        $modelShopCart = ShopCart::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->all();

                        if (isset($_SESSION['cart']['size'])) {
                            $counter = $_SESSION['cart']['size'];
                        } else {
                            $counter = -1;
                        }

                        foreach ($modelShopCart as $cartKey => $cartValue) {
                            $counter++;

                            $_SESSION['cart']['size'] = $counter + 1;
                            $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['cart_id'] = $cartValue['id'];
                            $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['user_id'] = $cartValue['user_id'];
                            $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['product_id'] = $cartValue['product_id'];
                            $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['order_id'] = $cartValue['order_id'];
                            $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['quantity'] = $cartValue['quantity'];
                            $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['price'] = Globals::formatNumber($cartValue['price'], ['round' => true, 'precision' => 2]);
                            $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['ip_address'] = $cartValue['ip_address'];
                        }

                        $_SESSION['cart']['db_sync'] = 1;

                        if (isset($_SESSION['cart']['voucher_id'])) {
                            $_SESSION['cart']['voucher_id'] = $_SESSION['cart']['voucher_id'];
                        } else {
                            $_SESSION['cart']['voucher_id'] = 0;
                        }

                        if (isset($_SESSION['cart']['discount'])) {
                            $_SESSION['cart']['discount'] = $_SESSION['cart']['discount'];
                        } else {
                            $_SESSION['cart']['discount'] = 0;
                        }

                        $_SESSION['cart']['cart_total'] = Cart::getCartTotal();
                    }
                }
            }
        }

        $this->refreshCartSessArray();
		
        return $this->render('cart');
    }

    public function actionCheckout()
    {
        if (Yii::$app->user->isGuest) {
            \yii\helpers\Url::remember('/index.php?r=shop/shop/cart');   // redirect user to cart page after login

            return $this->redirect('/index.php?r=user/user/login', 302);
        }
        
        $session = Yii::$app->session->open();

        $modelPaymentMethod = new PaymentMethod();

        $modelPaymentMethod = PaymentMethod::find()->where(['status' => 'active'])->asArray()->all();

        if (isset($_SESSION['cart'])) {
            $_SESSION['cart']['payment_flag'] = 'true';
            $_SESSION['cart']['payment_method_id'] = PaymentMethod::getAttr(['id'], ['name' => Yii::$app->params['payment_gateway']]);

            // TODO: Send checkout notification to admin
        }

        return $this->render('checkout', [
            'modelPaymentMethod' => $modelPaymentMethod,
        ]);
    }

    public function actionThankYou($id, $successful = true)
    {
        $session = Yii::$app->session->open();

        if ($successful === true) {
            $modelShopOrderArr = array();
            $modelShopOrderItemArr = array();
            $modelPaymentLogsArr = array();
            $payment_gateway = Yii::$app->params['payment_gateway'];

            if (!isset($_SESSION['gateway_call']) || $_SESSION['gateway_call'] != 'true') {
                throw new \yii\web\HttpException(422, 'Payment gateway not called.');
            }

            unset($_SESSION['gateway_call']);

            if (!ShopOrder::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
                throw new \yii\web\HttpException(422, 'Order not found.');
            }

            $modelShopOrderArr = ShopOrder::find()->where(['id' => $id, 'status' => 'active'])->asArray()->one();
            $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $modelShopOrderArr['id'], 'status' => 'active'])->asArray()->one();
            $modelPaymentLogsArr = PaymentLogs::find()->where(['order_id' => $id])->asArray()->one();

            return $this->render('thank-you', [
                'message' => 'Payment Successful', 
                'modelShopOrderArr' => $modelShopOrderArr,
                'modelShopOrderItemArr' => $modelShopOrderItemArr,
                'modelPaymentLogsArr' => $modelPaymentLogsArr,
                'is_fallback' => ((isset($_SESSION['is_fallback']) && $_SESSION['is_fallback'] == 'true') ? true : false), 
                'payment_gateway' => ((isset($_SESSION['is_fallback']) && $_SESSION['is_fallback'] == 'true') ? Yii::$app->params['payment_gateway_fallback'][$_SESSION['payment_gateway_fallback_index']] : Yii::$app->params['payment_gateway']), 
            ]);
        } else {
            return $this->render('thank-you-failure', [
                'message' => 'Payment Declined', 
                'reason' => 'There was a problem processing your payment.', 
            ]);
        }
    }

    public function actionThankYouPaypal($amt, $cc, $item_name, $item_number, $st, $tx)
    {
        $session = Yii::$app->session->open();

        if (isset($amt, $cc, $item_name, $item_number, $st, $tx)) {
            /*if (!isset($_SESSION['gateway_call']) || $_SESSION['gateway_call'] != 'true') {
                throw new \yii\web\HttpException(422, 'Payment gateway not called.');
            }

            unset($_SESSION['gateway_call']);*/

            return $this->render('thank-you', [
                'message' => 'Your payment is currently being processed.', 
            ]);
        } else {
            return $this->render('thank-you-failure', [
                'message' => 'Payment Declined', 
                'reason' => 'There was a problem processing your payment.', 
            ]);
        }
    }

    public function createOrder()
    {
        $session = Yii::$app->session;

        $modelOrder = new Order();
        $modelShopCart = new Cart();
        $modelOrderArr = array();
        $modelOrderItemArr = array();
        $post_data = array();

        $modelOrderArr['user_id'] = Yii::$app->user->identity->id;
        $modelOrderArr['first_name'] = UserProfile::getAttr(['first_name'], ['user_id' => Yii::$app->user->identity->id]);
        $modelOrderArr['last_name'] = UserProfile::getAttr(['last_name'], ['user_id' => Yii::$app->user->identity->id]);

        if (isset($_GET['st'])) {  // PayPal
            $modelOrderArr['order_status_id'] = OrderStatus::getAttr(['id'], ['name' => Yii::$app->params['pdt_status'][$_GET['st']]]);
        } else {    // MultiSafepay, Bank Transfer, Cash on Delivery
            $modelOrderArr['order_status_id'] = OrderStatus::getAttr(['id'], ['name' => 'New']);
        }

        $modelOrderArr['tax_id'] = Tax::getAttr(['id'], ['applicable' => 'yes', 'store_id' => Store::getAttr(['id'], ['code' => Yii::$app->params['store_code']])]);
        $modelOrderArr['voucher_id'] = isset($_SESSION['cart']['voucher_id']) && !empty($_SESSION['cart']['voucher_id']) ? $_SESSION['cart']['voucher_id'] : NULL;
        $modelOrderArr['amount'] = isset($_SESSION['cart']['cart_total']) ? Globals::formatNumber($_SESSION['cart']['cart_total'], ['round' => true, 'precision' => 2]) : 0;
        $modelOrderArr['exchange_rate'] = ExchangeRate::getAttr(['rate'], ['currency_code' => Yii::$app->params['currency_code']]);
        $modelOrderArr['payment_method_id'] = isset($_SESSION['cart']['payment_method_id']) ? $_SESSION['cart']['payment_method_id'] : NULL;
        $modelOrderArr['network_id'] = Network::getAttr(['id'], ['name' => 'Infinity ' . strtoupper(Yii::$app->params['store_code'])]);
        $modelOrderArr['invoice_number'] = NULL;
        $modelOrderArr['credit_note_number'] = NULL;
        $modelOrderArr['proforma_invoice'] = isset($_SESSION['cart']['proforma_invoice_name']) ? $_SESSION['cart']['proforma_invoice_name'] : NULL;
        $modelOrderArr['store_id'] = Store::getAttr(['id'], ['code' => Yii::$app->params['store_code']]);
        $modelOrderArr['session_id'] = NULL;
        $modelOrderArr['order_date'] = date('Y-m-d');

        $post_data['Order'] = $modelOrderArr;

        $modelShopCart = Cart::find()->where(['user_id' => Yii::$app->user->identity->id, 'store_id' => Store::getAttr(['id'], ['code' => Yii::$app->params['store_code']]), 'status' => 'active'])->asArray()->all();

        foreach ($_SESSION['cart'] as $sessionCartKey => $sessionCartValue) {
            if (!is_array($sessionCartValue)) {
                continue;
            }

            $modelOrderItemArrTmp = array();

            $modelOrderItemArrTmp['order_id'] = 0;
            $modelOrderItemArrTmp['product_id'] = $sessionCartValue['product_id'];
            $modelOrderItemArrTmp['product_name'] = Value::getEntityValue('product', 'name', $sessionCartValue['product_id'], Yii::$app->params['language_code']);
            $modelOrderItemArrTmp['cart_id'] = isset($sessionCartValue['cart_id']) ? $sessionCartValue['cart_id'] : NULL;
            $modelOrderItemArrTmp['quantity'] = isset($sessionCartValue['quantity']) ? $sessionCartValue['quantity'] : 0;
            $modelOrderItemArrTmp['price'] = isset($sessionCartValue['price']) ? Yii::$app->Globals->formatNumber($sessionCartValue['price'], ['round' => true, 'precision' => 2]) : 0;
            $modelOrderItemArrTmp['ring_size_id'] = isset($sessionCartValue['ring_size_id']) ? $sessionCartValue['ring_size_id'] : NULL;
            $modelOrderItemArrTmp['ring_size'] = isset($sessionCartValue['ring_size']) ? $sessionCartValue['ring_size'] : NULL;
            $modelOrderItemArrTmp['carat'] = isset($sessionCartValue['carat']) ? Yii::$app->Globals->formatNumber($sessionCartValue['carat'], ['round' => false]) : NULL;
            $modelOrderItemArrTmp['clarity_id'] = isset($sessionCartValue['clarity_id']) ? $sessionCartValue['clarity_id'] : NULL;
            $modelOrderItemArrTmp['clarity'] = isset($sessionCartValue['clarity']) ? $sessionCartValue['clarity'] : NULL;
            $modelOrderItemArrTmp['chain_id'] = isset($sessionCartValue['chain_id']) ? $sessionCartValue['chain_id'] : NULL;
            $modelOrderItemArrTmp['engraving'] = isset($sessionCartValue['engraving']) ? $sessionCartValue['engraving'] : NULL;
            $modelOrderItemArrTmp['special_request'] = isset($sessionCartValue['special_request']) ? $sessionCartValue['special_request'] : NULL;
            
            $modelOrderItemArr[$sessionCartKey] = $modelOrderItemArrTmp;
        }

        $post_data['OrderItem'] = $modelOrderItemArr;

        $tmp_order_id = $modelOrder->saveOrder($post_data, $order_id = 0);

        return $tmp_order_id;
    }

    public function cartSessDbSync()
    {
        $session = Yii::$app->session;

        $modelShopCart = new ShopCart();
        
        // saving cart session into db, then loading cart from db to session
        if (!Yii::$app->user->isGuest && (!isset($_SESSION['cart']['db_sync']) || $_SESSION['cart']['db_sync'] == 0)) {
            // saving cart session into db
            if (isset($_SESSION['cart']['size']) && $_SESSION['cart']['size'] > 0) {
                $modelShopCart->saveCart($_SESSION['cart']);

                for ($i = 0; $i < $_SESSION['cart']['size']; $i++) { 
                    unset($_SESSION['cart'][$i]);
                }
            }

            // saving cart session into db, then loading cart from db to session
            if (ShopCart::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
                // loading cart from db to session
                $modelShopCart = ShopCart::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->all();

                if (isset($_SESSION['cart']['size'])) {
                    $counter = $_SESSION['cart']['size'];
                } else {
                    $counter = -1;
                }

                foreach ($modelShopCart as $cartKey => $cartValue) {
                    $counter++;

                    $_SESSION['cart']['size'] = $counter + 1;
                    $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['cart_id'] = $cartValue['id'];
                    $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['user_id'] = $cartValue['user_id'];
                    $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['product_id'] = $cartValue['product_id'];
                    $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['order_id'] = $cartValue['order_id'];
                    $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['quantity'] = $cartValue['quantity'];
                    $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['price'] = Globals::formatNumber($cartValue['price'], ['round' => true, 'precision' => 2]);
                    $_SESSION['cart'][$_SESSION['cart']['size'] - 1]['ip_address'] = $cartValue['ip_address'];
                }

                $_SESSION['cart']['db_sync'] = 1;
                $_SESSION['cart']['voucher_id'] = 0;
                $_SESSION['cart']['discount'] = 0;
                $_SESSION['cart']['cart_total'] = Cart::getCartTotal();
            }

            $this->refreshCartSessArray();
        }
    }

    public function refreshCartSessArray()
    {
        $session = Yii::$app->session;

        // re-indexing the numeric keys of cart array
        if (isset($_SESSION['cart']['size']) && $_SESSION['cart']['size'] > 0) {
            $_SESSION['cart'] = array_merge($_SESSION['cart']);
            $_SESSION['cart']['size'] = count($_SESSION['cart']) - 5;
            $_SESSION['cart']['discount'] = Cart::calculateVoucherDiscount($_SESSION['cart']['voucher_id'], Cart::getCartSubtotal());
            $_SESSION['cart']['cart_total'] = Cart::getCartTotal();
        }
    }

    public function actionProductAdded() 
	{
        $session = Yii::$app->session;

        $productAdded = '0';
        $product_id = Yii::$app->request->post('product_id');
		
        if (isset($_SESSION['cart']['size']) && $_SESSION['cart']['size'] > 0) {
            foreach ($_SESSION['cart'] as $cartKey => $cartValue) {
                if (!is_array($cartValue)) { continue; }
				
                if ($cartValue['product_id'] == $product_id) {
                    $productAdded = '1';
					
                    return $productAdded;
                }
            }
        }
		
		return $productAdded;
    }
}
