<?php

namespace common\modules\shop\controllers\frontend;

use Yii;
use yii\web\Controller;

use common\modules\shop\models\ShopWishlist;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\components\Product;

class WishlistController extends Controller
{

    public $enableCsrfValidation = false;

    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'delete','add-to-wishlist'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['index', 'delete'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['add-to-wishlist'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'index' => ['get'],
                    'delete' => ['get'],
                ],
            ],
        ];
    }

    public function actionIndex()
    {
        if (Yii::$app->user->isGuest) {
            \yii\helpers\Url::remember();

            return $this->redirect('sign-in.html', 302);
        }

        $modelArrWishlist = new ShopWishlist();
        $wishListDetails = array();
        $productIdArr = array();
        $productDetails = array();

        $modelArrWishlist = ShopWishlist::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->all();

        foreach($modelArrWishlist as $key => $value) {
            array_push($productIdArr, $value['product_id']);
        }

        $productIdArr = array_unique($productIdArr);

        $productModel = ShopProduct::find()->where(['id' => $productIdArr])->all();

        $productDetails = Product::getProductDetails($productModel);

        return $this->render('index', [
            'modelArrWishlist' => $modelArrWishlist,
            'productDetails' => $productDetails
        ]);
    }

    public function actionDelete($id)
    {
        if (!isset($id) || empty($id)) {
            throw new \yii\web\BadRequestHttpException('Invalid request for wishlist item.', 400);
        }

        $modelArrWishlist = new ShopWishlist();

        if (ShopWishlist::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
            $modelArrWishlist = ShopWishlist::find()->where(['id' => $id, 'status' => 'active'])->one();

            $modelArrWishlist->softDelete();
        }

        $modelArrWishlist = ShopWishlist::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->all();

        return $this->render('index', [
            'modelArrWishlist' => $modelArrWishlist
        ]);
    }

    public function actionAddToWishlist() 
    {
        $wishlistModel = new ShopWishlist();
        $result = array();

        if (Yii::$app->request->isPost) {
            if (!empty(Yii::$app->request->Post('productId'))) {
                $productId = Yii::$app->request->Post('productId');
                $userId = Yii::$app->user->identity->id;

                if (ShopWishlist::find()->where(['product_id' => $productId, 'user_id' => $userId, 'status' => 'active'])->exists() && Yii::$app->request->Post('action') == 'add') {
                    $result['status'] = 0;
                    $result['message'] = 'Already added in your Wishlist';
                } else if (ShopWishlist::find()->where(['product_id' => $productId, 'user_id' => $userId, 'status' => 'active'])->exists() && Yii::$app->request->Post('action') == 'remove') {
                    $wishlistModel->saveWishlist($productId,'remove');
                    $result['status'] = 1;
                    $result['message'] = 'Product has been removed from your wishlist...';
                } else {
                    $wishlistModel->saveWishlist($productId,'add');

                    $result['status'] = 1;
                    $result['message'] = 'Product has been added in your wishlist...';
                }
            } else {
                $result['status'] = 0;
                $result['message'] = 'Cannot be added... Please try again later...';
            }

            return json_encode($result);
        }
        
        return false;
    }
}
