<?php

namespace common\modules\shop\models;

use Yii;
use yii\db\Expression;
use yii\behaviors\TimestampBehavior;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%shop_cart}}".
 *
 * @property int $id
 * @property int $user_id
 * @property int $product_id
 * @property int $order_id
 * @property int $quantity
 * @property int $voucher_id
 * @property string $price
 * @property string $trait
 * @property int $site_id
 * @property string $ip_address
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property User $user
 * @property ShopProduct $product
 * @property ShopOrder $order
 * @property ShopVoucher $voucher
 * @property CoreSite $site
 * @property ShopOrderItem[] $shopOrderItems
 */
class ShopCart extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%shop_cart}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'product_id', 'quantity', 'price', 'site_id'], 'required'],
            [['user_id', 'product_id', 'order_id', 'quantity', 'voucher_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['trait', 'metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['price', 'ip_address'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['product_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopProduct::className(), 'targetAttribute' => ['product_id' => 'id']],
            [['order_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopOrder::className(), 'targetAttribute' => ['order_id' => 'id']],
            [['voucher_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopVoucher::className(), 'targetAttribute' => ['voucher_id' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::className(), 'targetAttribute' => ['site_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'product_id' => 'Product ID',
            'order_id' => 'Order ID',
            'quantity' => 'Quantity',
            'voucher_id' => 'Voucher ID',
            'price' => 'Price',
            'trait' => 'Trait',
            'site_id' => 'Site ID',
            'ip_address' => 'Ip Address',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProduct()
    {
        return $this->hasOne(\common\modules\shop\models\ShopProduct::className(), ['id' => 'product_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrder()
    {
        return $this->hasOne(\common\modules\shop\models\ShopOrder::className(), ['id' => 'order_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getVoucher()
    {
        return $this->hasOne(\common\modules\shop\models\ShopVoucher::className(), ['id' => 'voucher_id']);
    }

    public function getVoucherValues()
    {
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'voucher', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'voucher_id'])->andOnCondition(['vv.entity_class_id' => $modelEntityClass->id, 'vv.attribute_class_id' => $modelAttributeClass->id]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::className(), ['id' => 'site_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopOrderItems()
    {
        return $this->hasMany(\common\modules\shop\models\ShopOrderItem::className(), ['cart_id' => 'id']);
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            [
                'class' => TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }
	
    public function saveCart($cart_session, $action = '', $id = '')
    {
        $breakFlag = false;
        $modelShopCart = new self();

        $transaction = Yii::$app->db->beginTransaction();

        if (isset($action, $id) && $action == 'update') {   // update cart
            if (self::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
                $modelShopCart = self::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->all();

                $cart_id = $modelShopCart[$id]['id'];

                $modelShopCart = self::findOne(['id' => $cart_id, 'status' => 'active']);
                $modelShopCart->quantity = $cart_session[$id]['quantity'];

                if ($modelShopCart->validate()) {
                    $modelShopCart->save();
                } else {
                    $breakFlag = true;
                }
            } else {
                $breakFlag = true;

                return false;
            }
        } else if (isset($action, $id) && $action == 'remove') {    // remove from cart
            if (self::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->exists()) {
                $modelShopCart = self::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->all();

                $cart_id = $modelShopCart[$id]['id'];

                $modelShopCart = self::findOne(['id' => $cart_id, 'status' => 'active']);
                
                $modelShopCart->softDelete();
            } else {
                $breakFlag = true;

                return false;
            }
        } elseif (isset($action) && $action == 'log') {
            $modelShopCart->user_id = $cart_session['ShopCart']['user_id'];
            $modelShopCart->product_id = $cart_session['ShopCart']['product_id'];
            $modelShopCart->order_id = $cart_session['ShopCart']['order_id'];
            $modelShopCart->quantity = $cart_session['ShopCart']['quantity'];
            $modelShopCart->voucher_id = $cart_session['ShopCart']['voucher_id'];
            $modelShopCart->price = $cart_session['ShopCart']['price'];
            $modelShopCart->ip_address = $cart_session['ShopCart']['ip_address'];
            $modelShopCart->site_id = $cart_session['ShopCart']['site_id'];
            $modelShopCart->created_by = $cart_session['ShopCart']['created_by'];
            $modelShopCart->updated_by = $cart_session['ShopCart']['updated_by'];
            $modelShopCart->status = $cart_session['ShopCart']['status'];

            if ($modelShopCart->validate()) {
                $modelShopCart->save();
            } else {
                $breakFlag = true;
            }
        } else {    // add to cart
            foreach ($cart_session as $sessionKey => $sessionValue) {
                if (!is_array($sessionValue)) {
                    continue;
                }

                $modelShopCart->user_id = Yii::$app->user->identity->id;
                $modelShopCart->product_id = $sessionValue['product_id'];
                $modelShopCart->order_id = isset($sessionValue['order_id']) ? $sessionValue['order_id'] : NULL;
                $modelShopCart->quantity = $sessionValue['quantity'];
                $modelShopCart->voucher_id = isset($sessionValue['voucher_id']) && !empty($sessionValue['voucher_id']) ? $sessionValue['voucher_id'] : NULL;
                $modelShopCart->price = $sessionValue['price'];
                $modelShopCart->ip_address = isset($sessionValue['ip_address']) ? $sessionValue['ip_address'] : NULL;
                $modelShopCart->site_id = \common\modules\core\models\CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
                $modelShopCart->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                $modelShopCart->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
                $modelShopCart->status = 'active';

                if ($modelShopCart->validate()) {
                    $modelShopCart->save();
                } else {
                    $breakFlag = true;

                    break;
                }
            }
        }

        if (!$breakFlag) {
            $transaction->commit();

            return true;
        } else {
            $transaction->rollBack();

            Yii::$app->session->setFlash('error', "Unable to save cart " . Globals::renderErrors($modelShopCart->getErrors()));

            return false;
        }
    }
}
