<?php

namespace common\modules\shop\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\shop\models\ShopCart;

/**
 * ShopCartSearch represents the model behind the search form of `common\modules\shop\models\ShopCart`.
 */
class ShopCartSearch extends ShopCart
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'product_id', 'order_id', 'quantity', 'voucher_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['price', 'trait', 'ip_address', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = ShopCart::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        $query->andFilterWhere(['mod_shop_cart.site_id' => $this->site_id]);

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_shop_cart.price', $this->price])
            ->andFilterWhere(['like', 'mod_shop_cart.trait', $this->trait])
            ->andFilterWhere(['like', 'mod_shop_cart.ip_address', $this->ip_address])
            ->andFilterWhere(['like', 'mod_shop_cart.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_shop_cart.status', $this->status])
            ->andFilterWhere(['like', 'mod_shop_cart.id', $this->id])
            ->andFilterWhere(['like', 'mod_shop_cart.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_shop_cart.product_id', $this->product_id])
            ->andFilterWhere(['like', 'mod_shop_cart.order_id', $this->order_id])
            ->andFilterWhere(['like', 'mod_shop_cart.quantity', $this->quantity])
            ->andFilterWhere(['like', 'mod_shop_cart.voucher_id', $this->voucher_id])
            ->andFilterWhere(['like', 'mod_shop_cart.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_shop_cart.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_shop_cart.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_shop_cart.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
