<?php

namespace common\modules\shop\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%shop_category}}".
 *
 * @property int $id
 * @property int $featured_image
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property AssetManager $featuredImage
 * @property ShopSubcategory[] $shopSubcategories
 */
class ShopCategory extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%shop_category}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['featured_image', 'created_by', 'updated_by'], 'integer'],
            [[], 'required'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['featured_image' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'featured_image' => 'Featured Image',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'featured_image']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopSubcategories()
    {
        return $this->hasMany(\common\modules\shop\models\ShopSubcategory::className(), ['category_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'category', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['entity_class_id' => $modelCoreEntityClass->id, 'attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'category', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveCategory($post_data, $category_id = 0)
    {
        $session = Yii::$app->session;

        $modelShopCategory = new ShopCategory();
        $modelCoreValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($category_id) && $category_id > 0 && ShopCategory::find()->where(['id' => $category_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelShopCategory = ShopCategory::findOne(['id' => $category_id, 'status' => 'active']);

                $modelShopCategory->featured_image = $post_data['ShopCategory']['featured_image'];
            } else {    // Create a new model
                $modelShopCategory = new ShopCategory();

                $modelShopCategory->featured_image = $post_data['ShopCategory']['featured_image'];
                $modelShopCategory->status = 'active';
            }
                
            if ($modelShopCategory->validate()) {
                $modelShopCategory->save();

                foreach ($post_data['ShopCategory'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }
 
                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();

                            if ($key == 'url') {                                
                                if (!isset($codeValue) || empty($codeValue)) {
                                    $codeValue = (string) 'index.php?r=shop/category/view&id=' . $modelShopCategory->id;
                                }                                
                            }


                            $modelCoreValue = CoreValue::setEntityValue('category', $key, $modelShopCategory->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }
                }

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Category saved successfully.");

                    return $modelShopCategory->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Globals::renderErrors($modelCoreValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save category: " . Globals::renderErrors($modelShopCategory->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
