<?php

namespace common\modules\shop\models;

use Yii;
use yii\db\Expression;
use yii\behaviors\TimestampBehavior;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;
use common\modules\user\models\UserProfile;

/**
 * This is the model class for table "{{%shop_order}}".
 *
 * @property int $id
 * @property int $user_id
 * @property string $first_name
 * @property string $last_name
 * @property int $order_status_id
 * @property int $voucher_id
 * @property string $amount
 * @property string $exchange_rate
 * @property string $invoice_number
 * @property string $credit_note_number
 * @property string $session_id
 * @property string $order_date
 * @property int $site_id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property PaymentLogs[] $paymentLogs
 * @property ShopCart[] $shopCarts
 * @property ShopLogsOrderStatus[] $shopLogsOrderStatuses
 * @property User $user
 * @property ShopOrderStatus $orderStatus
 * @property ShopVoucher $voucher
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreSite $site
 * @property ShopOrderItem[] $shopOrderItems
 * @property ShopRefund[] $shopRefunds
 */
class ShopOrder extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%shop_order}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'first_name', 'last_name', 'order_status_id', 'amount', 'order_date', 'site_id'], 'required'],
            [['user_id', 'order_status_id', 'voucher_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['order_date'], 'safe'],
            [['metadata', 'status'], 'string'],
            [['first_name', 'last_name', 'amount', 'exchange_rate', 'invoice_number', 'credit_note_number', 'session_id'], 'string', 'max' => 255],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['order_status_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopOrderStatus::className(), 'targetAttribute' => ['order_status_id' => 'id']],
            [['voucher_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopVoucher::className(), 'targetAttribute' => ['voucher_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::className(), 'targetAttribute' => ['site_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'first_name' => 'First Name',
            'last_name' => 'Last Name',
            'order_status_id' => 'Order Status ID',
            'voucher_id' => 'Voucher ID',
            'amount' => 'Amount',
            'exchange_rate' => 'Exchange Rate',
            'invoice_number' => 'Invoice Number',
            'credit_note_number' => 'Credit Note Number',
            'session_id' => 'Session ID',
            'order_date' => 'Order Date',
            'site_id' => 'Site ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPaymentLogs()
    {
        return $this->hasMany(\common\modules\payment\models\PaymentLogs::className(), ['order_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopCarts()
    {
        return $this->hasMany(\common\modules\shop\models\ShopCart::className(), ['order_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopLogsOrderStatuses()
    {
        return $this->hasMany(\common\modules\shop\models\ShopLogsOrderStatus::className(), ['order_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrderStatus()
    {
        return $this->hasOne(\common\modules\shop\models\ShopOrderStatus::className(), ['id' => 'order_status_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getVoucher()
    {
        return $this->hasOne(\common\modules\shop\models\ShopVoucher::className(), ['id' => 'voucher_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::className(), ['id' => 'site_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopOrderItems()
    {
        return $this->hasMany(\common\modules\shop\models\ShopOrderItem::className(), ['order_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopRefunds()
    {
        return $this->hasMany(\common\modules\shop\models\ShopRefund::className(), ['order_id' => 'id']);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                if (ShopOrderItem::find()->where(['order_id' => $this->id, 'status' => 'active'])->exists()) {
                    $modelShopOrderItem = ShopOrderItem::find()->where(['order_id' => $this->id, 'status' => 'active'])->all();

                    foreach ($modelShopOrderItem as $key => $value) {
                        $value->softDelete();
                    }
                }

                $transaction->commit();
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            [
                'class' => TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }
	
    public function saveOrder($post_data, $order_id = 0)
    {
        $session = Yii::$app->session;

        $modelShopOrder = new ShopOrder();
        $modelShopOrderItem = new ShopOrderItem();
        $modelShopCart = new ShopCart();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();
		
        try {
            if (isset($order_id) && $order_id > 0 && ShopOrder::find()->where(['id' => $order_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelShopOrder = new ShopOrder();

                $modelShopOrder = ShopOrder::findOne(['id' => $order_id, 'status' => 'active']);

                $modelShopOrder->load($post_data);

                $modelShopOrder->first_name = UserProfile::getAttr(['first_name'], ['user_id' => $post_data['ShopOrder']['user_id']]);
                $modelShopOrder->last_name = UserProfile::getAttr(['last_name'], ['user_id' => $post_data['ShopOrder']['user_id']]);
                $modelShopOrder->session_id = \common\modules\user\models\UserRole::getAttr(['name'], ['id' => \common\modules\user\models\User::getAttr(['role_id'], ['id' => Yii::$app->user->identity->id])]) == 'user' ? Yii::$app->session->getId() : NULL;
                $modelShopOrder->site_id = $post_data['ShopOrder']['site_id'];
                $modelShopOrder->created_by = isset($post_data['ShopOrder']['user_id']) ? $post_data['ShopOrder']['user_id'] : 1;
                $modelShopOrder->updated_by = isset($post_data['ShopOrder']['user_id']) ? $post_data['ShopOrder']['user_id'] : 1;
            } else {    // Create a new model
                $modelShopOrder = new ShopOrder();

                $modelShopOrder->load($post_data);

                $modelShopOrder->first_name = UserProfile::getAttr(['first_name'], ['user_id' => $post_data['ShopOrder']['user_id']]);
                $modelShopOrder->last_name = UserProfile::getAttr(['last_name'], ['user_id' => $post_data['ShopOrder']['user_id']]);

                if (!Yii::$app->user->isGuest) {
                    $modelShopOrder->session_id = \common\modules\user\models\UserRole::getAttr(['name'], ['id' => \common\modules\user\models\User::getAttr(['role_id'], ['id' => Yii::$app->user->identity->id])]) == 'user' ? Yii::$app->session->getId() : NULL;
                }
				
                $modelShopOrder->site_id = $post_data['ShopOrder']['site_id'];
                $modelShopOrder->created_by = isset($post_data['ShopOrder']['user_id']) ? $post_data['ShopOrder']['user_id'] : 1;
                $modelShopOrder->updated_by = isset($post_data['ShopOrder']['user_id']) ? $post_data['ShopOrder']['user_id'] : 1;
                $modelShopOrder->status = 'active';
            }

            if ($modelShopOrder->validate()) {
                $modelShopOrder->save();

                foreach (ShopOrderItem::findAll(['order_id' => $modelShopOrder->id, 'status' => 'active']) as $orderItemKey => $orderItemValue) {
                    $orderItemValue->softDelete();
                }

                foreach ($post_data['ShopOrderItem'] as $orderItemKey => $orderItemValue) {
                    $traitsArray = array();
					
                    if (isset($post_data['Trait'][$orderItemKey]) && !empty($post_data['Trait'][$orderItemKey])) {
                        foreach ($post_data['Trait'][$orderItemKey] as $key => $value) {
                            $trait_id = $key;
                            $options = $value;
                            $traitsArray[$trait_id] = $options;
                        }
                    }

                    $traitsJson = json_encode($traitsArray);
                    
                    if (isset($orderItemValue['id']) && $orderItemValue['id'] > 0 && ShopOrderItem::find()->where(['id' => $orderItemValue['id'], 'status' => 'active'])->exists()) {   // Model already exists
                        $modelShopOrderItem = new ShopOrderItem();

                        $modelShopOrderItem = ShopOrderItem::findOne(['id' => $orderItemValue['id'], 'status' => 'active']);

                        $modelShopOrderItem->order_id = $modelShopOrder->id;
                        $modelShopOrderItem->product_id = $orderItemValue['product_id'];

                        if (isset($orderItemValue['product_name']) && !empty($orderItemValue['product_name'])) {
                            $modelShopOrderItem->product_name = $orderItemValue['product_name'];
                        } else {
                            $modelShopOrderItem->product_name = CoreValue::getEntityValue('product', 'name', $orderItemValue['product_id']);
                        }
						
                        $modelShopOrderItem->cart_id = isset($orderItemValue['cart_id']) ? $orderItemValue['cart_id'] : NULL;
                        $modelShopOrderItem->quantity = $orderItemValue['quantity'];
                        $modelShopOrderItem->price = Globals::formatNumber($orderItemValue['price'], ['round' => true, 'precision' => 2]);
                        
                        if (isset($orderItemValue['traits']) && !empty($orderItemValue['traits'])) {
                            $modelShopOrderItem->traits = $orderItemValue['traits'];
                        } else {
                            $modelShopOrderItem->traits = $traitsJson;
                        }
                        
                        $modelShopOrderItem->created_by = $modelShopOrder->created_by;
                        $modelShopOrderItem->updated_by = $modelShopOrder->updated_by;
                        $modelShopOrderItem->status = 'active';
                    } else {    // Create a new model
                        $modelShopOrderItem = new ShopOrderItem();
                        
                        $modelShopOrderItem->order_id = $modelShopOrder->id;
                        $modelShopOrderItem->product_id = $orderItemValue['product_id'];

                        if (isset($orderItemValue['product_name']) && !empty($orderItemValue['product_name'])) {
                            $modelShopOrderItem->product_name = $orderItemValue['product_name'];
                        } else {
                            $modelShopOrderItem->product_name = CoreValue::getEntityValue('product', 'name', $orderItemValue['product_id']);
                        }
						
                        $modelShopOrderItem->cart_id = isset($orderItemValue['cart_id']) ? $orderItemValue['cart_id'] : NULL;
                        $modelShopOrderItem->quantity = $orderItemValue['quantity'];
                        $modelShopOrderItem->price = Globals::formatNumber($orderItemValue['price'], ['round' => true, 'precision' => 2]);
                        
                        if (isset($orderItemValue['traits']) && !empty($orderItemValue['traits'])) {
                            $modelShopOrderItem->traits = $orderItemValue['traits'];
                        } else {
                            $modelShopOrderItem->traits = $traitsJson;
                        }
                        
                        $modelShopOrderItem->created_by = $modelShopOrder->created_by;
                        $modelShopOrderItem->updated_by = $modelShopOrder->updated_by;
                        $modelShopOrderItem->status = 'active';
                    }

                    // Associating cart with order
                    if (isset($orderItemValue['cart_id'])) {
                        $modelCart = \common\modules\shop\models\ShopCart::findOne(['id' => $orderItemValue['cart_id']]);

                        $modelCart->order_id = $modelShopOrder->id;

                        if ($modelCart->validate()) {
                            $modelCart->save();
                        }
                    }

                    // Removing order-item from cart
                    if (isset($orderItemValue['cart_id'])) {
                        $modelCart = \common\modules\shop\models\ShopCart::findOne(['id' => $orderItemValue['cart_id']]);

                        $modelCart->softDelete();
                    }

                    if ($modelShopOrderItem->validate()) {
                        $modelShopOrderItem->save();
                    } else {
                        $breakFlag = true;

                        Yii::$app->session->setFlash('error', "Unable to save order item " . Globals::renderErrors($modelShopOrderItem->getErrors()));

                        break;
                    }
                }
            } else {
                $breakFlag = true;

                Yii::$app->session->setFlash('error', "Unable to save order: " . Globals::renderErrors($modelShopOrder->getErrors()));
            }

            if (!$breakFlag) {
                $transaction->commit();

                Yii::$app->session->setFlash('success', "Order saved successfully.");

                return $modelShopOrder->id;
            } else {
                $transaction->rollBack();

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
