<?php

namespace common\modules\shop\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\shop\models\ShopOrder;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\user\models\UserRole;

/**
 * ShopOrderSearch represents the model behind the search form of `common\modules\shop\models\ShopOrder`.
 */
class ShopOrderSearch extends ShopOrder
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'order_status_id', 'voucher_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['first_name', 'last_name', 'amount', 'exchange_rate', 'invoice_number', 'credit_note_number', 'session_id', 'order_date', 'metadata', 'created_at', 'updated_at', 'status', 'email', 'orderStatus'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['email', 'orderStatus']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = ShopOrder::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['user', 'orderStatus']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['email'] = ['asc' => ['mod_user.email' => SORT_ASC], 'desc' => ['mod_user.email' => SORT_DESC]];
        $dataProvider->sort->attributes['orderStatus'] = ['asc' => ['mod_shop_order_status.name' => SORT_ASC], 'desc' => ['mod_shop_order_status.name' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_shop_order.first_name', $this->first_name])
            ->andFilterWhere(['like', 'mod_user.email', $this->email])
            ->andFilterWhere(['like', 'mod_shop_order_status.name', $this->orderStatus])
            ->andFilterWhere(['like', 'mod_shop_order.last_name', $this->last_name])
            ->andFilterWhere(['like', 'mod_shop_order.amount', $this->amount])
            ->andFilterWhere(['like', 'mod_shop_order.exchange_rate', $this->exchange_rate])
            ->andFilterWhere(['like', 'mod_shop_order.invoice_number', $this->invoice_number])
            ->andFilterWhere(['like', 'mod_shop_order.credit_note_number', $this->credit_note_number])
            ->andFilterWhere(['like', 'mod_shop_order.session_id', $this->session_id])
            ->andFilterWhere(['like', 'mod_shop_order.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_shop_order.status', $this->status])
            ->andFilterWhere(['like', 'mod_shop_order.id', $this->id])
            ->andFilterWhere(['like', 'mod_shop_order.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_shop_order.order_status_id', $this->order_status_id])
            ->andFilterWhere(['like', 'mod_shop_order.voucher_id', $this->voucher_id])
            ->andFilterWhere(['like', 'mod_shop_order.order_date', $this->order_date])
            ->andFilterWhere(['like', 'mod_shop_order.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_shop_order.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_shop_order.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_shop_order.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_shop_order.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
