<?php

namespace common\modules\shop\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%shop_product}}".
 *
 * @property int $id
 * @property string $code
 * @property int $subcategory_id
 * @property int $featured_image
 * @property string $price
 * @property string $available
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property ShopCart[] $shopCarts
 * @property ShopOrderItem[] $shopOrderItems
 * @property ShopSubcategory $subcategory
 * @property AssetManager $featuredImage
 * @property User $createdBy
 * @property User $updatedBy
 * @property ShopProductTraitValue[] $shopProductTraitValues
 * @property ShopWishlist[] $shopWishlists
 */
class ShopProduct extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%shop_product}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['code', 'subcategory_id', 'featured_image', 'price'], 'required'],
            [['subcategory_id', 'featured_image', 'created_by', 'updated_by'], 'integer'],
            [['available', 'metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['code', 'price'], 'string', 'max' => 255],
            [['code'], 'unique'],
            [['subcategory_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopSubcategory::className(), 'targetAttribute' => ['subcategory_id' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['featured_image' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'code' => 'Code',
            'subcategory_id' => 'Subcategory ID',
            'featured_image' => 'Featured Image',
            'price' => 'Price',
            'available' => 'Available',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopCarts()
    {
        return $this->hasMany(\common\modules\shop\models\ShopCart::className(), ['product_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopOrderItems()
    {
        return $this->hasMany(\common\modules\shop\models\ShopOrderItem::className(), ['product_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSubcategory()
    {
        return $this->hasOne(\common\modules\shop\models\ShopSubcategory::className(), ['id' => 'subcategory_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'featured_image']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopProductTraitValues()
    {
        return $this->hasMany(\common\modules\shop\models\ShopProductTraitValue::className(), ['product_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopWishlists()
    {
        return $this->hasMany(\common\modules\shop\models\ShopWishlist::className(), ['product_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'product', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['v.entity_class_id' => $modelEntityClass->id, 'v.attribute_class_id' => $modelAttributeClass->id]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCategoryValues()
    {
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'category', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'category_id'])->viaTable('mod_shop_subcategory', ['id' => 'subcategory_id'])->andOnCondition(['cv.entity_class_id' => $modelEntityClass->id, 'cv.attribute_class_id' => $modelAttributeClass->id]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSubcategoryValues()
    {
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'subcategory', 'status' => 'active']);
        $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'subcategory_id'])->andOnCondition(['sv.entity_class_id' => $modelEntityClass->id, 'sv.attribute_class_id' => $modelAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelEntityClass = CoreEntityClass::findOne(['name' => 'product', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveProduct($post_data, $product_id = 0)
    {   
        $session = Yii::$app->session;

        $modelProduct = new ShopProduct();
        $modelValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();
		
		try {
            if (isset($product_id) && $product_id > 0 && ShopProduct::find()->where(['id' => $product_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelProduct = ShopProduct::findOne(['id' => $product_id, 'status' => 'active']);

                $modelProduct->code = $post_data['ShopProduct']['code'];
                $modelProduct->subcategory_id = $post_data['ShopProduct']['subcategory_id'];
                $modelProduct->featured_image = $post_data['ShopProduct']['featured_image'];
                $modelProduct->price = Globals::formatNumber($post_data['ShopProduct']['price'], ['round' => true, 'precision' => 2]);
                $modelProduct->available = $post_data['ShopProduct']['available'];
                $modelProduct->metadata = isset($post_data['ShopProductMetadata']) ? Globals::setJson($post_data['ShopProductMetadata']) : '';
            } else {    // Create a new model
                $modelProduct = new ShopProduct();

                $modelProduct->code = $post_data['ShopProduct']['code'];
                $modelProduct->subcategory_id = $post_data['ShopProduct']['subcategory_id'];
                $modelProduct->featured_image = $post_data['ShopProduct']['featured_image'];
                $modelProduct->price = Globals::formatNumber($post_data['ShopProduct']['price'], ['round' => true, 'precision' => 2]);
                $modelProduct->available = $post_data['ShopProduct']['available'];
                $modelProduct->metadata = isset($post_data['ShopProductMetadata']) ? Globals::setJson($post_data['ShopProductMetadata']) : '';
                $modelProduct->status = 'active';
            }

            if ($modelProduct->validate()) {
                $modelProduct->save();

                foreach ($post_data['ShopProduct'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }
					
                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();

                            if ($key == 'url') {                                
                                if (!isset($codeValue) || empty($codeValue)) {
                                    $codeValue = (string) 'index.php?r=shop/shop/view&id=' . $modelProduct->id;
                                }                                
                            }

                            $modelCoreValue = CoreValue::setEntityValue('product', $key, $modelProduct->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }
                }     

                //Saving traits
                if(isset($post_data['Trait']['trait']) && !empty($post_data['Trait']['trait'])) {
                    $traitValueIdArr = array();
                    foreach ($post_data['Trait']['trait'] as $key => $value) {
                        if(isset($post_data['Trait'][$value])) {
                            foreach($post_data['Trait'][$value] as $seq => $traitValueId) {
                                array_push($traitValueIdArr, $traitValueId);
                                $modelProductTraitsValue = new ShopProductTraitValue();
                                $modelProductTraitsValue->product_id = $modelProduct->id;
                                $modelProductTraitsValue->trait_id = $value;
                                $modelProductTraitsValue->status = 'active';
                                $modelProductTraitsValue->trait_value_id = $traitValueId;

                                if ($modelProductTraitsValue->validate()) {
                                    $modelProductTraitsValue->save();
                                } else {
                                    $transaction->rollBack();

                                    $breakFlag = true;
                                }
                            }
                        }
                    }
                    // Deleting left-out product Traits
                    if (!$breakFlag) {
                        foreach (ShopProductTraitValue::find()->where(['product_id' => $modelProduct->id, 'status' => 'active'])->andWhere(['NOT IN', 'trait_value_id', $traitValueIdArr])->orWhere(['product_id' => $modelProduct->id, 'trait_value_id' => null, 'status' => 'active'])->all() as $key => $value) {
                            $value->softDelete();
                        }
                    }
                }

                // Saving product images
                $is_Save = \common\modules\cms\models\CmsEntityAsset::saveEntityAssets('product', $modelProduct->id, $post_data['CmsEntityAsset']);
               
                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Product saved successfully.");

                    return $modelProduct->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Globals::renderErrors($modelValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save product: " . Globals::renderErrors($modelProduct->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
