<?php

namespace common\modules\shop\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\shop\models\ShopProduct;

/**
 * ShopProductSearch represents the model behind the search form of `common\modules\shop\models\ShopProduct`.
 */
class ShopProductSearch extends ShopProduct
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'subcategory_id', 'featured_image', 'created_by', 'updated_by'], 'integer'],
            [['code', 'price', 'available', 'metadata', 'created_at', 'updated_at', 'status', 'name', 'category', 'subcategory'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['name', 'category', 'subcategory']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = ShopProduct::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['values AS v', 'categoryValues AS cv', 'subcategoryValues AS sv']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['name'] = ['asc' => ['v.value' => SORT_ASC], 'desc' => ['v.value' => SORT_DESC]];
        $dataProvider->sort->attributes['category'] = ['asc' => ['cv.value' => SORT_ASC], 'desc' => ['cv.value' => SORT_DESC]];
        $dataProvider->sort->attributes['subcategory'] = ['asc' => ['sv.value' => SORT_ASC], 'desc' => ['sv.value' => SORT_DESC]];
        

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_shop_product.code', $this->code])
            ->andFilterWhere(['like', 'v.value', $this->name])
            ->andFilterWhere(['like', 'cv.value', $this->category])
            ->andFilterWhere(['like', 'sv.value', $this->subcategory])
            ->andFilterWhere(['like', 'mod_shop_product.price', $this->price])
            ->andFilterWhere(['like', 'mod_shop_product.available', $this->available])
            ->andFilterWhere(['like', 'mod_shop_product.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_shop_product.status', $this->status])
            ->andFilterWhere(['like', 'mod_shop_product.id', $this->id])
            ->andFilterWhere(['like', 'mod_shop_product.subcategory_id', $this->subcategory_id])
            ->andFilterWhere(['like', 'mod_shop_product.featured_image', $this->featured_image])
            ->andFilterWhere(['like', 'mod_shop_product.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_shop_product.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_shop_product.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_shop_product.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
