<?php

namespace common\modules\shop\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%shop_subcategory}}".
 *
 * @property int $id
 * @property int $category_id
 * @property int $featured_image
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property ShopProduct[] $shopProducts
 * @property ShopCategory $category
 * @property User $createdBy
 * @property User $updatedBy
 * @property AssetManager $featuredImage
 */
class ShopSubcategory extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%shop_subcategory}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['category_id'], 'required'],
            [['category_id', 'featured_image', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['category_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopCategory::className(), 'targetAttribute' => ['category_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['featured_image' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'category_id' => 'Category ID',
            'featured_image' => 'Featured Image',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopProducts()
    {
        return $this->hasMany(\common\modules\shop\models\ShopProduct::className(), ['subcategory_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCategory()
    {
        return $this->hasOne(\common\modules\shop\models\ShopCategory::className(), ['id' => 'category_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'featured_image']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'subcategory', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['v.entity_class_id' => $modelCoreEntityClass->id, 'v.attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCategoryValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'category', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'category_id'])->andOnCondition(['cv.entity_class_id' => $modelCoreEntityClass->id, 'cv.attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'subcategory', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveSubcategory($post_data, $subcategory_id = 0)
    {
        $session = Yii::$app->session;

        $modelSubcategory = new ShopSubcategory();
        $modelValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($subcategory_id) && $subcategory_id > 0 && ShopSubcategory::find()->where(['id' => $subcategory_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelSubcategory = ShopSubcategory::findOne(['id' => $subcategory_id, 'status' => 'active']);

                $modelSubcategory->category_id = $post_data['ShopSubcategory']['category_id'];
                $modelSubcategory->featured_image = $post_data['ShopSubcategory']['featured_image'];
            } else {    // Create a new model
                $modelSubcategory = new ShopSubcategory();

                $modelSubcategory->category_id = $post_data['ShopSubcategory']['category_id'];
                $modelSubcategory->featured_image = $post_data['ShopSubcategory']['featured_image'];
                $modelSubcategory->status = 'active';
            }
            
            if ($modelSubcategory->validate()) {
                $modelSubcategory->save();

                foreach ($post_data['ShopSubcategory'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }
					
                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();

                            if ($key == 'url') {                                
                                if (!isset($codeValue) || empty($codeValue)) {
                                    $codeValue = (string) 'index.php?r=shop/subcategory/view&id=' . $modelSubcategory->id;
                                }                                
                            }





                            $modelCoreValue = CoreValue::setEntityValue('subcategory', $key, $modelSubcategory->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }
                }

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Subcategory saved successfully.");

                    return $modelSubcategory->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Globals::renderErrors($modelValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save subcategory: " . Globals::renderErrors($modelSubcategory->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
