<?php

namespace common\modules\shop\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\shop\models\ShopSubcategory;

/**
 * ShopSubcategorySearch represents the model behind the search form of `common\modules\shop\models\ShopSubcategory`.
 */
class ShopSubcategorySearch extends ShopSubcategory
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'category_id', 'featured_image', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at', 'status', 'name', 'category'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['name', 'category']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = ShopSubcategory::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['values AS v', 'categoryValues AS cv']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['name'] = ['asc' => ['v.value' => SORT_ASC], 'desc' => ['v.value' => SORT_DESC]];
        $dataProvider->sort->attributes['category'] = ['asc' => ['cv.value' => SORT_ASC], 'desc' => ['cv.value' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_shop_subcategory.status', $this->status])
            ->andFilterWhere(['like', 'mod_shop_subcategory.id', $this->id])
            ->andFilterWhere(['like', 'mod_shop_subcategory.category_id', $this->category_id])
            ->andFilterWhere(['like', 'mod_shop_subcategory.featured_image', $this->featured_image])
            ->andFilterWhere(['like', 'v.value', $this->name])
            ->andFilterWhere(['like', 'cv.value', $this->category])
            ->andFilterWhere(['like', 'mod_shop_subcategory.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_shop_subcategory.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_shop_subcategory.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_shop_subcategory.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
