<?php

namespace common\modules\shop\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreValue;

/**
 * This is the model class for table "{{%shop_trait_value}}".
 *
 * @property int $id
 * @property int $trait_id
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property ShopProductTraitValue[] $shopProductTraitValues
 * @property User $createdBy
 * @property User $updatedBy
 * @property ShopTrait $trait
 */
class ShopTraitValue extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%shop_trait_value}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['trait_id'], 'required'],
            [['trait_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['trait_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopTrait::className(), 'targetAttribute' => ['trait_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'trait_id' => 'Trait ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopProductTraitValues()
    {
        return $this->hasMany(\common\modules\shop\models\ShopProductTraitValue::className(), ['trait_value_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTrait()
    {
        return $this->hasOne(\common\modules\shop\models\ShopTrait::className(), ['id' => 'trait_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'id'])->andOnCondition(['v.entity_class_id' => $modelCoreEntityClass->id, 'v.attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTraitValues()
    {
        $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'trait', 'status' => 'active']);
        $modelCoreAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);

        return $this->hasOne(CoreValue::className(), ['entity_id' => 'trait_id'])->andOnCondition(['cv.entity_class_id' => $modelCoreEntityClass->id, 'cv.attribute_class_id' => $modelCoreAttributeClass->id]);
    }

    public function softDelete()
    {
        $transaction = Yii::$app->db->beginTransaction();

        try {
            $this->status = 'inactive';
            
            if ($this->validate()) {
                $this->save();

                $modelCoreEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);

                if (CoreValue::deleteEntityValue(['entity_class_id' => $modelCoreEntityClass->id, 'entity_id' => $this->id])) {
                    $transaction->commit();
                } else {
                    throw new \yii\web\HttpException(500, 'Unable to delete.');                    
                }
            } else {
                $transaction->rollBack();

                throw new \yii\web\HttpException(500, 'Unable to delete.');
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function saveTraitValue($post_data, $trait_value_id = 0)
    {
        $session = Yii::$app->session;

        $modelTraitValue = new ShopTraitValue();
        $modelValue = new CoreValue();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($trait_value_id) && $trait_value_id > 0 && ShopTraitValue::find()->where(['id' => $trait_value_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelTraitValue = ShopTraitValue::findOne(['id' => $trait_value_id, 'status' => 'active']);

                $modelTraitValue->trait_id = $post_data['ShopTraitValue']['trait_id'];
            } else {    // Create a new model
                $modelTraitValue = new ShopTraitValue();

                $modelTraitValue->trait_id = $post_data['ShopTraitValue']['trait_id'];
                $modelTraitValue->status = 'active';
            }
            
            if ($modelTraitValue->validate()) {
                $modelTraitValue->save();

                foreach ($post_data['ShopTraitValue'] as $key => $value) {
                    if (!CoreAttributeClass::doesAttributeClassExist($key)) {
                        continue;
                    }
					
                    if (is_array($value) && array_key_exists('value', $value)) {
                        foreach ($value['value'] as $code => $codeValue) {
                            $modelCoreValue = new CoreValue();

                            $modelCoreValue = CoreValue::setEntityValue('trait_value', $key, $modelTraitValue->id, $codeValue, $code);

                            if ($modelCoreValue->validate()) {
                                $modelCoreValue->save();
                            } else {
                                $transaction->rollBack();

                                $breakFlag = true;
                            }
                        }                        
                    }
                }

                if (!$breakFlag) {
                    $transaction->commit();

                    Yii::$app->session->setFlash('success', "Trait value saved successfully.");

                    return $modelTraitValue->id;
                } else {
                    $transaction->rollBack();

                    Yii::$app->session->setFlash('error', "Unable to save value " . Globals::renderErrors($modelValue->getErrors()));

                    return false;
                }
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save trait values: " . Globals::renderErrors($modelTraitValue->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
