<?php

namespace common\modules\shop\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\shop\models\ShopTraitValue;

/**
 * ShopTraitValueSearch represents the model behind the search form of `common\modules\shop\models\ShopTraitValue`.
 */
class ShopTraitValueSearch extends ShopTraitValue
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'trait_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at', 'status','name','trait'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), ['name', 'trait']);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = ShopTraitValue::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith(['values AS v', 'traitValues AS cv']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $dataProvider->sort->attributes['name'] = ['asc' => ['v.value' => SORT_ASC], 'desc' => ['v.value' => SORT_DESC]];
        $dataProvider->sort->attributes['trait'] = ['asc' => ['cv.value' => SORT_ASC], 'desc' => ['cv.value' => SORT_DESC]];

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_shop_trait_value.status', $this->status])
            ->andFilterWhere(['like', 'mod_shop_trait_value.id', $this->id])
            ->andFilterWhere(['like', 'mod_shop_trait_value.trait_id', $this->trait_id])
            ->andFilterWhere(['like', 'v.value', $this->name])
            ->andFilterWhere(['like', 'cv.value', $this->trait])
            ->andFilterWhere(['like', 'mod_shop_trait_value.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_shop_trait_value.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_shop_trait_value.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_shop_trait_value.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
