<?php

namespace common\modules\shop\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%shop_voucher}}".
 *
 * @property int $id
 * @property string $code
 * @property string $discount
 * @property string $discount_percent
 * @property string $start_date
 * @property string $end_date
 * @property int $featured_image
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property ShopCart[] $shopCarts
 * @property ShopOrder[] $shopOrders
 * @property User $createdBy
 * @property User $updatedBy
 * @property AssetManager $featuredImage
 */
class ShopVoucher extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%shop_voucher}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['code', 'discount', 'start_date', 'end_date'], 'required'],
            [['discount_percent', 'metadata', 'status'], 'string'],
            [['start_date', 'end_date'], 'safe'],
            [['featured_image', 'created_by', 'updated_by'], 'integer'],
            [['code', 'discount'], 'string', 'max' => 255],
            [['code'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['featured_image'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['featured_image' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'code' => 'Code',
            'discount' => 'Discount',
            'discount_percent' => 'Discount Percent',
            'start_date' => 'Start Date',
            'end_date' => 'End Date',
            'featured_image' => 'Featured Image',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopCarts()
    {
        return $this->hasMany(\common\modules\shop\models\ShopCart::className(), ['voucher_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getShopOrders()
    {
        return $this->hasMany(\common\modules\shop\models\ShopOrder::className(), ['voucher_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFeaturedImage()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'featured_image']);
    }

    public function saveVoucher($post_data, $voucher_id = 0)
    {
        $session = Yii::$app->session;

        $modelShopVoucher = new self();
        $breakFlag = false;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($voucher_id) && $voucher_id > 0 && self::find()->where(['id' => $voucher_id, 'status' => 'active'])->exists()) {   // Model already exists
                $modelShopVoucher = self::findOne(['id' => $voucher_id, 'status' => 'active']);

                $modelShopVoucher->load($post_data);

                $modelShopVoucher->metadata = isset($post_data['ShopVoucher']['ShopVoucherMetadata']) ? Globals::setJson($post_data['ShopVoucher']['ShopVoucherMetadata']) : '';
            } else {    // Create a new model
                $modelShopVoucher = new ShopVoucher();

                $modelShopVoucher->load($post_data);

                $modelShopVoucher->metadata = isset($post_data['ShopVoucher']['ShopVoucherMetadata']) ? Globals::setJson($post_data['ShopVoucher']['ShopVoucherMetadata']) : '';
                $modelShopVoucher->status = 'active';
            }
            
            if ($modelShopVoucher->validate()) {
                $modelShopVoucher->save();

                $transaction->commit();

                Yii::$app->session->setFlash('success', "Voucher saved successfully.");

                return $modelShopVoucher->id;
            } else {
                $transaction->rollBack();

                Yii::$app->session->setFlash('error', "Unable to save voucher: " . Globals::renderErrors($modelShopVoucher->getErrors()));

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
