<?php

namespace common\modules\shop\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%shop_wishlist}}".
 *
 * @property int $id
 * @property int $user_id
 * @property int $product_id
 * @property int $site_id
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $user
 * @property ShopProduct $product
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreSite $site
 */
class ShopWishlist extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%shop_wishlist}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'product_id', 'site_id'], 'required'],
            [['user_id', 'product_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['status'], 'string'],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['product_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\shop\models\ShopProduct::className(), 'targetAttribute' => ['product_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::className(), 'targetAttribute' => ['site_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'product_id' => 'Product ID',
            'site_id' => 'Site ID',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProduct()
    {
        return $this->hasOne(\common\modules\shop\models\ShopProduct::className(), ['id' => 'product_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::className(), ['id' => 'site_id']);
    }

    public function saveWishlist($productId = '', $action = '') 
    {
        $breakFlag = false;
        $modelWishlist = new self();

        $transaction = Yii::$app->db->beginTransaction();

        if (isset($action, $productId) && $action == 'remove') {
            if (self::find()->where(['user_id' => Yii::$app->user->identity->id, 'product_id' => $productId, 'status' => 'active'])->exists()) {
                $modelWishlist = self::find()->where(['user_id' => Yii::$app->user->identity->id, 'product_id' => $productId, 'status' => 'active'])->asArray()->all();

                foreach($modelWishlist as $key => $value) {
                    $wishlist_id = $value['id'];
                    $modelWishlist = self::findOne(['id' => $wishlist_id, 'status' => 'active']);
                
                    $modelWishlist->softDelete();
                }                
            } else {
                $breakFlag = true;
                
                return false;
            }

        } else {
            if(isset($productId) && !empty($productId)) {
                $modelWishlist->user_id = Yii::$app->user->identity->id;
                $modelWishlist->product_id = $productId;
                $modelWishlist->status = 'active';

                if ($modelWishlist->validate()) {
                    $modelWishlist->save();
                } else {
                    $breakFlag = true;
                }
            }
        }
        if (!$breakFlag) {
            $transaction->commit();

            return true;
        } else {
            $transaction->rollBack();

            Yii::$app->session->setFlash('error', "Unable to save Wishlist " . Globals::renderErrors($modelWishlist->getErrors()));

            return false;
        }
    }
}
