<?php

namespace common\modules\shop\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\shop\models\ShopWishlist;

/**
 * ShopWishlistSearch represents the model behind the search form of `common\modules\shop\models\ShopWishlist`.
 */
class ShopWishlistSearch extends ShopWishlist
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'product_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = ShopWishlist::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_shop_wishlist.status', $this->status])
            ->andFilterWhere(['like', 'mod_shop_wishlist.id', $this->id])
            ->andFilterWhere(['like', 'mod_shop_wishlist.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_shop_wishlist.product_id', $this->product_id])
            ->andFilterWhere(['like', 'mod_shop_wishlist.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_shop_wishlist.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_shop_wishlist.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_shop_wishlist.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_shop_wishlist.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
