<?php

use yii\bootstrap4\Html;
use yii\bootstrap4\ActiveForm;

use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\core\models\CoreSite;
use common\modules\user\models\User;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopTrait;
use common\modules\shop\models\ShopProductTraitValue;
use common\modules\shop\models\ShopOrderStatus;
use common\modules\accounting\models\AccountingCurrency;
use common\modules\shop\models\ShopVoucher;

/* @var $this yii\web\View */
/* @var $model common\modules\shop\models\ShopOrder */
/* @var $form yii\bootstrap4\ActiveForm */

$data_tmp = $data;

extract($data_tmp);
?>

<div class="shop-order-form">

    <?php $form = ActiveForm::begin(['id' => 'dynamic-form']); ?>

    <?= $form->field($model, 'user_id')->widget(\kartik\select2\Select2::classname(), [
        'data' => User::getAll($k = 'id', $v = 'email'),
        'options' => ['placeholder' => 'Select User'],
        'pluginOptions' => [
            'allowClear' => true
        ],
    ])->label('User'); ?>

    <?= $form->field($model, 'site_id')->dropDownList(CoreSite::getAll($k = 'id', $v = 'name'), ['prompt' => 'Select Site'])->label('Site') ?>
	
    <?= $form->field($model, 'order_status_id')->dropDownList(ShopOrderStatus::getAll($k = 'id', $v = 'name'), ['prompt' => 'Select Order Status'])->label('Order Status') ?>

    <?= $form->field($model, 'voucher_id')->dropDownList($vouchers, ['prompt' => 'Select Voucher'])->label('Voucher') ?>

    <?= $form->field($model, 'amount')->textInput(['maxlength' => true]) ?>

    <?= $form->field($model, 'exchange_rate')->dropDownList(AccountingCurrency::getAll($k = 'rate', $v = 'name'), ['prompt' => 'Select Exchange Rate'])->label('Exchange Rate') ?>

    <?= $form->field($model, 'invoice_number')->textInput(['maxlength' => true]) ?>

    <?= $form->field($model, 'credit_note_number')->textInput(['maxlength' => true]) ?>

    <?= $form->field($model, 'order_date')->widget(\kartik\date\DatePicker::classname(), [
        'type' => \kartik\date\DatePicker::TYPE_COMPONENT_APPEND,
        'pluginOptions' => [
            'autoclose' => true,
            'format' => 'yyyy-mm-dd'
        ]
    ]); ?>

    <?php \wbraganca\dynamicform\DynamicFormWidget::begin([
        'widgetContainer' => 'dynamicform_wrapper',     // required: only alphanumeric characters plus "_" [A-Za-z0-9_]
        'widgetBody' => '.container-items',     // required: css class selector
        'widgetItem' => '.item',    // required: css class
        'limit' => 10,  // the maximum times, an element can be cloned (default 999)
        'min' => 1,     // 0 or 1 (default 1)
        'insertButton' => '.add-item',  // css class
        'deleteButton' => '.remove-item',   // css class
        'model' => $modelOrderItem[0],
        'formId' => 'dynamic-form',
        'formFields' => [
            'product_name',
            'price',
            'quantity'
        ],
    ]); ?>

    <div class="card mt-5">
        <div class="card-header">
            <i class="fa fa-shopping-cart"></i> Order Items
            <button type="button" class="float-right add-item btn btn-success btn-sm"><i class="fas fa-plus"></i> Add</button>
        </div>
        <div class="card-body container-items"><!-- widgetContainer -->
            <?php foreach ($modelOrderItem as $orderItemKey => $orderItemValue): ?>
                <div class="item card bg-light"><!-- widgetBody -->
                    <div class="card-header">
                        <span class="card-title-address">Order Item: <?= ($orderItemKey + 1) ?></span>
                        <button type="button" class="float-right remove-item btn btn-danger btn-sm"><i class="fas fa-minus"></i></button>
                    </div>
                    <div class="card-body">
                        <?php
                        $product_traits = array();
						
                        // necessary for update action.
                        if (!$orderItemValue->isNewRecord) {
                            echo Html::activeHiddenInput($orderItemValue, "[{$orderItemKey}]id");
							
							$product_traits = array();

                            // Gettig traits of this product
                            if (isset($orderItemValue->product_id) && $orderItemValue->product_id !== null) {
                                $productid = $orderItemValue->product_id;
                                $modelShopProductTraitValue = new ShopProductTraitValue();

                                $modelShopProductTrait = ShopProductTraitValue::find()->select('trait_id,trait_value_id')->where(['product_id' => $productid, 'status' => 'active'])->distinct()->asArray()->all();

                                if (!empty($modelShopProductTrait)) {
                                    $traitValueArr = \yii\helpers\ArrayHelper::getColumn($modelShopProductTrait,'trait_value_id');
                                    $modelEntityClass = CoreEntityClass::findOne(['name' => 'trait_value', 'status' => 'active']);
                                    $modelAttributeClass = CoreAttributeClass::findOne(['name' => 'name', 'status' => 'active']);
                                    $traitNamesArray = CoreValue::find()->select('entity_id, value')->where(['entity_class_id' => $modelEntityClass->id, 'attribute_class_id' => $modelAttributeClass->id, 'entity_id' => $traitValueArr ,'status' => 'active'])->asArray()->all();

                                    foreach($modelShopProductTrait as $key => $value) {                
                                        foreach($traitNamesArray as $k => $v){
                                            if ($value['trait_value_id'] == $v['entity_id']) {
                                                $product_traits[$value['trait_id']][$v['entity_id']] = $v['value'];
                                            }
                                        }
                                    }
                                    $traitValueNamesArr = \yii\helpers\ArrayHelper::map($modelShopProductTrait,'entity_id','value');
                                }      
                            }
                        }
                        ?>

                        <?= $form->field($orderItemValue, "[{$orderItemKey}]product_id")->widget(\kartik\select2\Select2::classname(), [
                            'data' => ShopProduct::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'product', $attribute_class = 'name'),
                            'language' => 'en',
                            'options' => ['placeholder' => 'Select Product', 'onchange' => 'setTrait(this.value,this.id)', 'class' => $orderItemKey],
                            'pluginOptions' => [
                                'allowClear' => true,
                            ],
                        ])->label('Product'); ?>

                        <?= $form->field($orderItemValue, "[{$orderItemKey}]product_name")->textInput(['maxlength' => true]) ?>

                        <?= $form->field($orderItemValue, "[{$orderItemKey}]quantity")->textInput(['maxlength' => true]) ?>

                        <?= $form->field($orderItemValue, "[{$orderItemKey}]price")->textInput(['maxlength' => true]) ?>
                        
                        <div id="traits-list">
                            <?php
                            echo '<label class="control-label clearfix" for="trait-product_trait_id">List of Product Traits</label>';

                            $orderTrait = $orderItemValue['traits'];
                            $orderTraitArr = json_decode($orderTrait, true);

                            $traitsArray = ShopTrait::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'trait', $attribute_class = 'name');

                            foreach($traitsArray as $key => $value) {
                                $tmp = '';

                                if (isset($orderTraitArr) && array_key_exists($key, $orderTraitArr)) {
                                    $tmp = $orderTraitArr[$key];
                                }
                                $traitValueArray = array();
								
								if (isset($product_traits) && !empty($product_traits) && $product_traits !== null && isset($product_traits[$key])) {
                                    $traitValueArray = $product_traits[$key];
                                }

                                echo '<div class="trait-checkbox-wrap form-group">';

                                $isdisabled = true;
                                $selectedTrait = "false";
								
                                echo '<label class="control-label clearfix trait-label" for="trait-product_trait_id">  Select ' . $value . '</label>';
                                echo Html::dropDownList('Trait[' . $orderItemKey . '][' . $key . ']', $tmp, $traitValueArray, ['prompt' => 'Select option', 'class' => 'form-control']);

                                echo '</div>';
                            }
                            ?>
                        </div>
                    </div>
                </div>
            <?php endforeach; ?>
        </div>
    </div>
    
    <?php \wbraganca\dynamicform\DynamicFormWidget::end(); ?>

    <div class="form-group">
        <?= Html::submitButton('Save', ['class' => 'btn btn-success']) ?>
    </div>

    <?php ActiveForm::end(); ?>

</div>

<script type="text/javascript">
    function setTrait(product_id, orderItem) {
        var orderItemId = orderItem.match("shoporderitem-(.*)-product_id");
        orderItemId = orderItemId[1];
		
		console.log(product_id + orderItemId);
		
		$.ajax({
           url: "<?php echo \Yii::$app->getUrlManager()->createUrl('/shop/order/ajax') ?>",
           type: 'POST',
           async: false,
           data: {product_id: product_id},
           success: function (test) {
                var traits = JSON.parse(test);
				
                console.log(traits);
				
                if (jQuery.isEmptyObject(traits)){
                    $('#traits-list').find('.trait-checkbox-wrap').each(function() {
                        console.log($(this).find('select').html('<option>Select Option</option>'));
                        
                    });
                } else {
                    $.each(traits, function( index, value ) {
                        $('select[name="Trait[' + orderItemId + '][' + index + ']"]').html('');
                        $.each(value, function( id, name ) {
                            $('select[name="Trait[' + orderItemId + '][' + index + ']"]').append($('<option></option>').val(id).html(name));
                        });
                    });
                    
                }
            },
            error: function (exception) {
                console.log(exception);
            }
        });
    }
</script>

<?php 
$script = <<< JS
$(".dynamicform_wrapper").on("beforeInsert", function(e, item) {

});

$(".dynamicform_wrapper").on("afterInsert", function(e, item) {
    $(".dynamicform_wrapper .card-title-address").each(function(index) {
        $(this).html("Order Item: " + (index + 1))
    });
});

$(".dynamicform_wrapper").on("beforeDelete", function(e, item) {
    if (!confirm("Are you sure you want to delete this order item?")) {
        return false;
    }

    return true;
});

$(".dynamicform_wrapper").on("afterDelete", function(e) {
    $(".dynamicform_wrapper .card-title-address").each(function(index) {
        $(this).html("Order Item: " + (index + 1))
    });
});

$(".dynamicform_wrapper").on("limitReached", function(e, item) {
    alert("Can not add more order items.");
});
JS;

$this->registerJs($script);
?>
