<?php

namespace common\modules\site\components;

use Yii;
use yii\base\Component;
use yii\helpers\Url;

use common\modules\core\models\CoreSite;
use common\modules\site\models\SiteMail;

class Email extends Component 
{
    
    private static $apiKey;

	public static function send(
        $type = 'generic', 
        $template = array(), 
        $rsvp = array(), 
        $subject = null, 
        $data = array(), 
        $app_alias = null, 
        $funnel = null, 
        $campaign = false, 
        $transport = 'smtp-1',
        $marketing = false
    )
    {
        if (!isset($rsvp['to']) || empty($rsvp['to'])) {
            $rsvp['to'] = Yii::$app->params['email']['to'][Yii::$app->params['mode']];
        }

        // All emails to be sent to developer's email in sandbox mode
        if (Yii::$app->params['mode'] == 'sandbox') {
            $rsvp['to'] = Yii::$app->params['email']['to'][Yii::$app->params['mode']];
        }

        if (!isset($rsvp['from']) || empty($rsvp['from'])) {
            $rsvp['from'] = Yii::$app->params['email']['from'][Yii::$app->params['mode']];
        }

        if (!isset($subject) || empty($subject)) {
            $subject = Yii::$app->params['email']['subject'][$type];
        }

        if (!isset($site_id) || empty($site_id)) {
            $site_id = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
        }

        $template_arr = [
            'html' => $template['html'], 
            'text' => $template['text']
        ];

        if ($template['html'] == false) {
            unset($template_arr['html']);
        }

        if(isset(Yii::$app->params['email']['sendgrid']['api_key'][$transport])) {
            self::$apiKey = Yii::$app->params['email']['sendgrid']['api_key'][$transport];
        } else {
            self::$apiKey = null;
        }

        if (!$campaign) {
			$modelSiteMail = new SiteMail();
			
			$modelSiteMail->type = $type;
			$modelSiteMail->email_to = $rsvp['to'];
			$modelSiteMail->email_from = $rsvp['from'];
			$modelSiteMail->email_cc = isset($rsvp['cc']) ? $rsvp['cc'] : null;
			$modelSiteMail->email_bcc = isset($rsvp['bcc']) ? $rsvp['bcc'] : null;
			$modelSiteMail->subject = $subject;
			$modelSiteMail->body = Yii::$app->mailer->render('@common/views/mail/' . $type, ['data' => $data]);
			$modelSiteMail->ip_address = ((Yii::$app instanceof \yii\web\Application) ? Yii::$app->request->getUserIP() : null);
			$modelSiteMail->site_id = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
			$modelSiteMail->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
			$modelSiteMail->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;

			if ($modelSiteMail->validate()) {
				$modelSiteMail->save();
			}

			// send a copy to developer
            if (isset(Yii::$app->params['email']['developer']) && !empty(Yii::$app->params['email']['developer'])) {
                Yii::$app->mailer->useTransport($transport)->compose(
                $template_arr, 
                [
                    'data' => $data
                ])
                ->setFrom($rsvp['from'])
                ->setTo(Yii::$app->params['email']['developer'])
                ->setSubject($subject)
                ->send();
            }

            // send a copy to the tester
            if (isset(Yii::$app->params['email']['tester']) && !empty(Yii::$app->params['email']['tester'])) {
                Yii::$app->mailer->useTransport($transport)->compose(
                $template_arr, 
                [
                    'data' => $data
                ])
                ->setFrom($rsvp['from'])
                ->setTo(Yii::$app->params['email']['tester'])
                ->setSubject($subject)
                ->send();
            }
		}

        if ($marketing === false) { // send transactional email
            return Yii::$app->mailer->useTransport($transport)->compose(
                $template_arr, 
            	[
            		'data' => $data
            	])
                ->setFrom($rsvp['from'])
                ->setTo($rsvp['to'])
                ->setSubject($subject)
                ->send();
        } else {    // send marketing email
            // TODO: Delete contact
            $contacts_delete = self::sgContactsDelete();

            // TODO: Add contact
            $contacts_upsert = self::sgContactsUpsert([
                'email' => (Yii::$app->params['mode'] == 'live') ? $rsvp['to'] : Yii::$app->params['email']['developer']
            ]);

            // TODO: Create single send
            if ($contacts_upsert === false || !isset($template['html']) || !isset($template['text']) || !isset($rsvp['sg_sender_id'])) {
                $single_send_create = false;
            } else {
                $single_send_create = self::sgSingleSendCreate(
                    $template,
                    $data['data'], 
                    $subject, 
                    $unsubscribe_url = '#',
                    $sender_id = $rsvp['sg_sender_id']
                );

                $single_send_create = json_decode($single_send_create);
            }

            // TODO: Schedule single send
            if ($single_send_create === false) {
                $single_send_schedule = false;
            } else {
                $single_send_schedule = self::sgSingleSendSchedule($single_send_create->id);
            }
        }
    }

    public function sgContactsUpsert($contacts = array(), $list_ids = array())
    {
        $sg = new \SendGrid(self::$apiKey);
        $request_body = json_decode('{
            "list_ids": 
            ' . json_encode($list_ids) . '
            ,
            "contacts": [
            ' . json_encode($contacts) . '
            ]
        }');

        try {
            $response = $sg->client->marketing()->contacts()->put($request_body);

            print $response->statusCode() . "\n";
            print_r($response->headers());
            print $response->body() . "\n";

            if ($response->statusCode() == 202) {
                return $response->body();
            } else {
                return false;
            }
        } catch (Exception $ex) {
            echo 'Caught exception: '. $ex->getMessage();
        }
    }

    public function sgContactsDelete()
    {
        $sg = new \SendGrid(self::$apiKey);
        $query_params = json_decode('{
            "delete_all_contacts": "true"
        }');

        try {
            $response = $sg->client->marketing()->contacts()->delete(null, $query_params);
            
            print $response->statusCode() . "\n";
            print_r($response->headers());
            print $response->body() . "\n";
            
            if ($response->statusCode() == 202) {
                return true;
            } else {
                return false;
            }
        } catch (Exception $ex) {
            echo 'Caught exception: '. $ex->getMessage();
        }
    }

    public function sgSingleSendCreate(
        $template,
        $data = array(),
        $subject = '',
        $unsubscribe_url = '#',
        $sender_id
    )
    {
        $sg = new \SendGrid(self::$apiKey);
        $request_body = [
            'name' => ucfirst($data['user_profile']['first_name'] . ' ' . $data['user_profile']['last_name']),
            'send_to' => [
                'all' => true,
            ],
            'email_config' => [
                'subject' => $subject,
                'html_content' => Yii::$app->view->renderFile('@common/mail/' . $template['html'] . '.php', ['data' => $data]),
                'plain_content' => Yii::$app->view->renderFile('@common/mail/' . $template['text'] . '.php', ['data' => $data]),
                'custom_unsubscribe_url' => $unsubscribe_url,
                'sender_id' => $sender_id,
            ],
        ];

        try {
            $response = $sg->client->marketing()->singlesends()->post($request_body);
            
            print $response->statusCode() . "\n";
            print_r($response->headers());
            print $response->body() . "\n";

            if ($response->statusCode() == 201) {
                return $response->body();
            } else {
                return false;
            }
        } catch (Exception $ex) {
            echo 'Caught exception: '. $ex->getMessage();
        }
    }

    public function sgSingleSendSchedule($single_send_id)
    {
        $sg = new \SendGrid(self::$apiKey);
        $request_body = json_decode('{
            "send_at": "now"
        }');

        try {
            $response = $sg->client->marketing()->singlesends()->_($single_send_id)->schedule()->put($request_body);
            
            print $response->statusCode() . "\n";
            print_r($response->headers());
            print $response->body() . "\n";
            
            if ($response->statusCode() == 200) {
                return $response->body();
            } else {
                return false;
            }
        } catch (Exception $ex) {
            echo 'Caught exception: '. $ex->getMessage();
        }
    }
}
