<?php

namespace common\modules\site\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\helpers\Json;

use common\modules\site\models\FormContact;
use common\modules\site\models\FormNewsletters;
use common\modules\site\models\FormReview;
use common\modules\site\models\SiteContact;
use common\modules\site\models\SiteNewsletters;
use common\modules\site\models\SiteReview;
use common\modules\core\components\Globals;
use common\modules\core\components\Shortcode;
use common\modules\core\components\GenesysRestClient;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\core\models\CoreSite;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\user\models\UserLogsLogin;
use common\modules\user\models\FormRegister;
use common\modules\user\models\FormLogin;
use common\modules\user\models\FormPasswordResetRequest;
use common\modules\user\models\FormResetPassword;
use common\modules\cms\models\CmsBlog;
use common\modules\cms\models\CmsTestimonial;
use common\modules\cms\models\CmsEvent;
use common\modules\cms\models\CmsEntityVideo;
use common\modules\shop\components\Product;
use common\modules\shop\models\ShopCategory;
use common\modules\shop\models\ShopSubcategory;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopCart;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\payment\models\FormStripe;
use common\modules\payment\models\FormFhtpay;
use common\modules\payment\models\FormSquare;
use common\modules\payment\models\PaymentLogs;
use common\modules\mailer\models\MailerCampaign;
use common\components\WhoisComponent;

/**
 * Dncorp2 controller
 */
class Dncorp2Controller extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'about', 'contact', 'user-logged-in', 'thank-you', 'newsletters', 'review'],
                'rules' => [
                    [
                        'actions' => [''],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => [''],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'about', 'contact', 'user-logged-in', 'thank-you', 'newsletters', 'review'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'index' => ['get', 'post'],
                    'about' => ['get'],
                    'contact' => ['get', 'post'],
                    'user-logged-in' => ['get', 'post'],
                ],
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function actionIndex($n = '', $r = 'a', $t = '', $p = null)
    {
        $session = Yii::$app->session->open();

        // Track email campaign
        if (isset($t) && !empty($t) && isset($p) && !empty($p) && $p == 'v0') {
            $restClient = new GenesysRestClient('', 'v1', 'http://api.v1.parker.com');

            $response = $restClient->callApi('domains/track', 'POST', '', ['t' => $t]);
        } else if (isset($p) && is_int($p) === true && $p >= 0) {
            $col_name = Yii::$app->params['whois_scraper_data'][$p];

            if (isset($t) && !empty($t)) {
                $config = Yii::$app->getComponents();

                $$col_name = (new \MongoDB\Client($config['mongodb']['dsn']))->whoisrestapi->$col_name;

                $updateResult = $$col_name->updateOne(
                    ['timestamp' => (string) $t],
                    ['$set' => [
                            'timestamp_ack' => (string) time(), 
                            'updated_at' => date('Y-m-d H:s:i')
                        ]
                    ]
                );
            }
        } else {
            if (!isset($p) && isset($t) && !empty($t)) {
                $modelMailerCampaign = new MailerCampaign();

                if (MailerCampaign::find()->where(['timestamp' => $t])->exists()) {
                    $modelMailerCampaign = MailerCampaign::find()->where(['timestamp' => $t])->one();

                    $modelMailerCampaign->timestamp_click = (string) time();
                    $modelMailerCampaign->clicks = $modelMailerCampaign->clicks + 1;
                    $modelMailerCampaign->ip_address = Yii::$app->request->userIP;
                    $modelMailerCampaign->referrer_url = Yii::$app->request->referrer;
                    $modelMailerCampaign->user_agent = Yii::$app->request->userAgent;

                    if ($modelMailerCampaign->validate()) {
                        $modelMailerCampaign->save();
                    }
                }
            }
        }
        
        if (!isset($_SESSION['price_bracket_index']) || empty($_SESSION['price_bracket_index'])) {
            $_SESSION['price_bracket_index'] = rand(0, count(Yii::$app->params['prices']) - 1);
        }
        
        $tt = ['a' => '1', 'b' => '2', 'c' => '3', 'd' => '5', 'e' => '10'];
        $registration_periods = [
            1 => date('Y-m-d', strtotime('+1 years')), 
            2 => date('Y-m-d', strtotime('+2 years')),
            3 => date('Y-m-d', strtotime('+3 years')),
            5 => date('Y-m-d', strtotime('+5 years')),
            10 => date('Y-m-d', strtotime('+10 years')),
        ];

        $r = str_replace(array_keys($tt), $tt, $r);

        $this->actionSaveSessDomain(['validity' => $r, 'price' => Yii::$app->params['prices'][$r]]);

        if ((!isset($n) || empty($n)) && !Yii::$app->request->cookies->has('n')) {  // domain name neither supplied nor found in cookie
            // return $this->render('index', [
            //     'n' => $n,
            //     'r' => $r,
            //     'flag_domain' => false,
            //     'modelFormFhtpay' => new FormFhtpay(),
            //     'modelFormSquare' => new FormSquare(),
            //     'registration_periods' => $registration_periods,
            // ]);
            return $this->redirect('/user-dashboard', 302);
        } else {
            if (isset($n, $r) && !empty($n) && !empty($r)) {
                if (!Yii::$app->request->cookies->has('n')) {
                    // setcookie('n', $n, time() + (10 * 365 * 24 * 60 * 60));
                    Yii::$app->response->cookies->add(new \yii\web\Cookie(['name' => 'n', 'value' => $n]));
                }

                if (!Yii::$app->request->cookies->has('r')) {
                    // setcookie('r', $r, time() + (10 * 365 * 24 * 60 * 60));
                    Yii::$app->response->cookies->add(new \yii\web\Cookie(['name' => 'r', 'value' => $r]));
                }
            } else if (isset($n) && !empty($n)) {
                // setcookie('n', $n, time() + (10 * 365 * 24 * 60 * 60));
                Yii::$app->response->cookies->add(new \yii\web\Cookie(['name' => 'n', 'value' => $n]));
            } else {
                $n = Yii::$app->request->cookies->getValue('n');
                $r = Yii::$app->request->cookies->getValue('r');
            }

            return $this->render('index', [
                'n' => $n,
                'r' => $r,
                'flag_domain' => true,
                'modelFormStripe' => new FormStripe(),
                'modelFormFhtpay' => new FormFhtpay(),
                'modelFormSquare' => new FormSquare(),
                'registration_periods' => $registration_periods,
            ]);
        }
    }

    /**
     * Displays about page.
     *
     * @return mixed
     */
    public function actionAbout()
    {
        return $this->render('about');
    }

    /**
     * Displays contact page.
     *
     * @return mixed
     */
    public function actionContact()
    {
        $model = new FormContact();
        $modelSiteContact = new SiteContact();

        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            $modelSiteContact->name = $model->name;
            $modelSiteContact->email = $model->email;
            $modelSiteContact->subject = $model->subject;
            $modelSiteContact->message = $model->body;
            $modelSiteContact->site_id = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
            $modelSiteContact->ip_address = Yii::$app->request->getUserIP();
            $modelSiteContact->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
            $modelSiteContact->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;

            if ($modelSiteContact->validate()) {
                $modelSiteContact->save();
            }

            if ($model->sendEmail(Yii::$app->params['email']['contact'][Yii::$app->params['mode']])) {
                Yii::$app->session->setFlash('success', 'Thank you for contacting us. We will respond to you as soon as possible.');
            } else {
                Yii::$app->session->setFlash('error', 'There was an error sending your message.');
            }

            return $this->refresh();
        } else {
            return $this->render('contact', [
                'model' => $model,
            ]);
        }
    }

    public function actionUserLoggedIn() 
	{
        $userLogged = 1;

        if (Yii::$app->user->isGuest) {
            $userLogged = 0;
        }

        return $userLogged;
    }

    public function actionWhoisData()
    {
        $request = Yii::$app->request;

        unset($_SESSION['whois']);

        if ($request->isPost) {
            $whois_data = WhoisComponent::lookup($request->post('domain'));

            // explicitly insert domain name into whois-data array, in case it is not fetched from the whois-service as desired
            if (!isset($whois_data['domain']['name']) || empty($whois_data['domain']['name'])) {
                $whois_data['domain']['name'] = $request->post('domain');
            }
        }

        $_SESSION['whois']['domain']['name'] = (isset($whois_data['domain']['name']) && !empty($whois_data['domain']['name'])) ? $whois_data['domain']['name'] : null;

        return $this->renderAjax('_whois-data', [
            'whois_data' => $whois_data,
        ]);
    }

    public function actionSaveSessWhois()
    {

    }

    public function actionSaveSessWhoisData()
    {
        $session = Yii::$app->session->open();
        $request = Yii::$app->request;

        if ($request->isPost) {
            $_SESSION['cart_id'] = $this->saveCart();
            
            unset($_SESSION['whois']);

            $_SESSION['whois']['domain']['name'] = $request->post('whois_data')['domain']['name'];

            foreach ($request->post('whois_data')['registrant_contact'] as $key => $value) {
                $_SESSION['whois']['registrant_contact'][$key] = $value;
            }

            if (isset($request->post('whois_data')['administrative_contact'])) {
                foreach ($request->post('whois_data')['administrative_contact'] as $key => $value) {
                    $_SESSION['whois']['administrative_contact'][$key] = $value;
                }
            }

            if (isset($request->post('whois_data')['technical_contact'])) {
                foreach ($request->post('whois_data')['technical_contact'] as $key => $value) {
                    $_SESSION['whois']['technical_contact'][$key] = $value;
                }
            }

            return true;
        }
    }

    public function actionSaveSessDomain($post = array())
    {
        $session = Yii::$app->session->open();
        $request = Yii::$app->request;

        $_SESSION['domain']['validity'] = isset($_SESSION['domain']['validity']) ? $_SESSION['domain']['validity'] : null;
        $_SESSION['domain']['price'] = isset($_SESSION['domain']['price']) ? $_SESSION['domain']['price'] : null;

        if ($request->isPost) {
            $_SESSION['domain']['validity'] = $request->post('validity');
            $_SESSION['domain']['price'] = $request->post('price');
            
            // $_SESSION['cart_id'] = $this->saveCart();
            
            return 'ok';
        } else {
            $_SESSION['domain']['validity'] = $post['validity'];
            $_SESSION['domain']['price'] = $post['price'];

            // $_SESSION['cart_id'] = $this->saveCart();
            
            return 'ok';
        }
    }

    public function saveCart()
    {
        $session = Yii::$app->session->open();
        $request = Yii::$app->request;

        $modelShopCart = new ShopCart();

        if (!isset($_SESSION['domain']['validity'])) {
            return false;
        }

        if (!isset($_SESSION['domain']['price'])) {
            return false;
        }

        $modelShopCart->user_id = !Yii::$app->user->isGuest ? Yii::$app->user->identity->id : 1;
        $modelShopCart->product_id = (int) ShopProduct::getAttr(['id'], ['price' => Globals::formatNumber($_SESSION['domain']['price'], $round = array('round' => false, 'precision' => 2))]);
        $modelShopCart->quantity = $_SESSION['domain']['validity'];
        $modelShopCart->price = (string) $_SESSION['domain']['price'];
        $modelShopCart->trait = (string) (isset($_SESSION['whois']) ? json_encode($_SESSION['whois']) : null);
        $modelShopCart->site_id = CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
        $modelShopCart->ip_address = Yii::$app->request->getUserIP();
        $modelShopCart->created_by = !Yii::$app->user->isGuest ? Yii::$app->user->identity->id : 1;
        $modelShopCart->updated_by = !Yii::$app->user->isGuest ? Yii::$app->user->identity->id : 1;
        $modelShopCart->status = 'active';

        if ($modelShopCart->validate()) {
            $modelShopCart->save();

            return $modelShopCart->id;
        }

        return false;
    }

    public function actionThankYou($id, $successful = true)
    {
        $session = Yii::$app->session->open();

        if ($successful === true) {
            $modelShopOrderArr = array();
            $modelShopOrderItemArr = array();
            $modelPaymentLogsArr = array();
            $payment_gateway = Yii::$app->params['payment_gateway'];

            if (!isset($_SESSION['gateway_call']) || $_SESSION['gateway_call'] != 'true') {
                throw new \yii\web\HttpException(422, 'Payment gateway not called.');
            }

            unset($_SESSION['gateway_call']);

            if (!ShopOrder::find()->where(['id' => $id, 'status' => 'active'])->exists()) {
                throw new HttpException(422, 'Order not found.');
            }

            $modelShopOrderArr = ShopOrder::find()->where(['id' => $id, 'status' => 'active'])->asArray()->one();
            $modelShopOrderItemArr = ShopOrderItem::find()->where(['order_id' => $modelShopOrderArr['id'], 'status' => 'active'])->asArray()->one();
            $modelPaymentLogsArr = PaymentLogs::find()->where(['order_id' => $id])->asArray()->one();

            return $this->render('thank-you', [
                'modelShopOrderArr' => $modelShopOrderArr,
                'modelShopOrderItemArr' => $modelShopOrderItemArr,
                'modelPaymentLogsArr' => $modelPaymentLogsArr,
                'is_fallback' => ((isset($_SESSION['is_fallback']) && $_SESSION['is_fallback'] == 'true') ? true : false), 
                'payment_gateway' => ((isset($_SESSION['is_fallback']) && $_SESSION['is_fallback'] == 'true') ? Yii::$app->params['payment_gateway_fallback'][$_SESSION['payment_gateway_fallback_index']] : Yii::$app->params['payment_gateway']), 
            ]);
        } else {
            return $this->render('thank-you-failure', [
                'message' => 'Payment Declined', 
                'reason' => 'There was a problem processing your payment.', 
            ]);
        }
    }

    public function actionAboutUs()
    {
        return $this->renderAjax('_about-us');
    }

    public function actionContactUs()
    {
        return $this->renderAjax('_contact-us');
    }

    public function actionListings()
    {
        return $this->renderAjax('_listings');
    }

    public function actionTermsAndConditions()
    {
        return $this->render('terms-and-conditions');
    }

    public function actionPrivacyPolicy()
    {
        return $this->render('privacy-policy');
    }

    public function actionUserDashboard()
    {
        if (Yii::$app->user->isGuest) {
            \yii\helpers\Url::remember();

            return $this->redirect('/login', 302);
        }

        $modelUser = User::find()->where(['id' => Yii::$app->user->identity->id, 'status' => 'active'])->one();

        $modelUserLogsLogin = UserLogsLogin::find()->where(['user' => Yii::$app->user->identity->email])->orderBy(['created_at' => SORT_DESC])->one();
        
        return $this->render('user-dashboard', [
            'modelUser' => $modelUser,
            'modelUserLogsLogin' => $modelUserLogsLogin,
        ]);           
    }

    public function actionServices($auth = '')
    {
        return $this->render('services', [
            'auth' => $auth, 
        ]);
    }

    public function actionNewsletters()
    {
        $modelFormNewsletters = new FormNewsletters();

        if ($modelFormNewsletters->load(Yii::$app->request->post())) {
            if ($modelFormNewsletters->subscribe()) {
                if (Yii::$app->request->isAjax) {
                    return Json::encode([
                        'result' => 'ok', 
                        'payload' => [
                            'redirect_url' => '/course-preview.html', 
                        ]
                    ]);
                }
            }
        }
    }

    public function actionReview()
    {
        $model = new FormReview();
        $modelSiteReview = new SiteReview();

        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            $modelSiteReview->name = $model->name;
            $modelSiteReview->email = $model->email;
            
            if (!Yii::$app->user->isGuest) {
                $modelSiteReview->user_id = Yii::$app->user->identity->id;
            }

            $modelSiteReview->bio = $model->bio;
            $modelSiteReview->rating_scale = (string) Yii::$app->params['rating_scale'];
            $modelSiteReview->rating = $model->rating;
            $modelSiteReview->review = $model->review;
            $modelSiteReview->visible = 'no';
            $modelSiteReview->site_id = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
            $modelSiteReview->ip_address = Yii::$app->request->getUserIP();
            $modelSiteReview->user_agent = Yii::$app->request->getUserAgent();
            $modelSiteReview->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
            $modelSiteReview->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
            $modelSiteReview->status = 'active';

            if ($modelSiteReview->validate()) {
                $modelSiteReview->save();

                Yii::$app->session->setFlash('success', 'Thank you for your feedback.');
            } else {
                Yii::$app->session->setFlash('error', 'There was an error submitting your feedback.');
            }

            return $this->refresh();
        } else {
            return $this->render('review', [
                'model' => $model,
            ]);
        }
    }
}
