<?php

namespace common\modules\site\controllers\frontend;

use Yii;

use common\modules\user\models\User;
use common\modules\shop\models\ShopOrder;

class SeoController extends \yii\web\Controller
{
	/**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['seo-stats'],
                'rules' => [
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['seo-stats'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [],
            ],
        ];
    }

    public function actionIndex($auth = '')
    {
        if (!Yii::$app->user->isGuest) {
            $auth = Yii::$app->user->identity->auth_key;

            $modelShopOrder = ShopOrder::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->one();
        } else {
            if (!User::find()->where(['auth_key' => $auth, 'status' => 'active'])->exists()) {
                throw new \yii\web\HttpException(401, 'Authentication key not found.');
            }

            $modelShopOrder = ShopOrder::find()->where(['user_id' => User::getAttr(['id'], ['auth_key' => $auth]), 'status' => 'active'])->one();
        }

        return $this->render('index', [
            'modelShopOrder' => $modelShopOrder, 
            'auth' => $auth, 
        ]);
    }

    public function actionSeoStats($domain)
    {
        return $this->render('seo-stats', [
            'domain' => $domain, 
        ]);
    }

    public function actionSeoReports($report = 'keyword-position-checker', $auth = '')
    {
        $file_exists = false;

        if (!Yii::$app->user->isGuest) {
            $auth = Yii::$app->user->identity->auth_key;

            $modelShopOrder = ShopOrder::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->one();
        } else {
            if (!User::find()->where(['auth_key' => $auth, 'status' => 'active'])->exists()) {
                throw new \yii\web\HttpException(401, 'Authentication key not found.');
            }

            $modelShopOrder = ShopOrder::find()->where(['user_id' => User::getAttr(['id'], ['auth_key' => $auth]), 'status' => 'active'])->one();
        }

        if (file_exists(Yii::getAlias('@assets') . '/uploads/sp-reports/' . $auth . '/' . $report . '.pdf') || file_exists(Yii::getAlias('@assets') . '/uploads/sp-reports/' . $auth . '/' . $report . '.zip')) {
            $file_exists = true;
        }

        return $this->render($report, [
            'report' => $report, 
            'auth' => $auth, 
            'modelShopOrder' => $modelShopOrder, 
            'file_exists' => $file_exists, 
        ]);
    }
}
