<?php

namespace common\modules\site\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\helpers\Json;

use common\modules\site\models\FormContact;
use common\modules\site\models\FormNewsletters;
use common\modules\site\models\FormReview;
use common\modules\site\models\SiteContact;
use common\modules\site\models\SiteNewsletters;
use common\modules\site\models\SiteReview;
use common\modules\core\components\Globals;
use common\modules\core\components\Shortcode;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\core\models\CoreSite;
use common\modules\user\models\UserProfile;
use common\modules\user\models\FormRegister;
use common\modules\user\models\FormLogin;
use common\modules\user\models\FormPasswordResetRequest;
use common\modules\user\models\FormResetPassword;
use common\modules\cms\models\CmsBlog;
use common\modules\cms\models\CmsTestimonial;
use common\modules\cms\models\CmsEvent;
use common\modules\cms\models\CmsEntityVideo;
use common\modules\shop\components\Product;
use common\modules\shop\models\ShopCategory;
use common\modules\shop\models\ShopSubcategory;
use common\modules\shop\models\ShopProduct;
use common\modules\mailer\models\MailerCampaign;

/**
 * Site controller
 */
class SiteController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['index', 'about', 'contact', 'user-logged-in', 'thank-you', 'newsletters', 'review'],
                'rules' => [
                    [
                        'actions' => [''],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['thank-you'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['index', 'about', 'contact', 'user-logged-in', 'thank-you', 'newsletters', 'review'],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'index' => ['get', 'post'],
                    'about' => ['get'],
                    'contact' => ['get', 'post'],
                    'user-logged-in' => ['get', 'post'],
                ],
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    /**
     * Displays homepage.
     *
     * @return mixed
     */
    public function actionIndex()
    {
        $modelProduct = new ShopProduct();
        $modelSiteContact = new SiteContact();
        $modelFormContact = new FormContact();
        $product_list = array();
        $blog_list = array();
		$testimonialArray = array();
		$eventsArray = array();
        $news = array();

        $categoryList = ShopCategory::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'category', $attribute_class = 'name');

        $modelShopProduct = ShopProduct::findAll(['status' => 'active']);
        $products = Product::getProductDetails($modelShopProduct);

        // Getting product details
        if (ShopProduct::find()->where(['available' => 'yes', 'status' => 'active'])->exists()) {
            $productValue = ShopProduct::find()->where(['available' => 'yes', 'status' => 'active'])->limit(8)->orderBy('created_at')->all();
            $product_list = Product::getProductDetails($productValue);
        }

        // Getting blog details
        $modelBlog = CmsBlog::find()->where(['status' => 'active'])->limit(4)->asArray()->all();

        if (!empty($modelBlog)) {
            foreach ($modelBlog as $key => $value) {
                $blog_list_tmp['name'] = CoreValue::getEntityValue('blog', 'name', $value['id']);
                $blog_list_tmp['url'] = CoreValue::getEntityValue('blog', 'url', $value['id']);
                $blog_list_tmp['content'] = Shortcode::stripContent(CoreValue::getEntityValue('blog', 'blog_content', $value['id']), 200);
                $blog_list_tmp['featured_image'] = $value['featured_image'];
                $blog_list_tmp['created_at'] = $value['created_at'];
                $blog_list_tmp['author'] = UserProfile::getAttr(['first_name'], ['user_id' => $value['created_by']]) . ' ' . UserProfile::getAttr(['last_name'], ['user_id' => $value['created_by']]);

                array_push($blog_list, $blog_list_tmp);
            }
        }

        // Getting events started
        $now = date('Y-m-d H:i:s');
        $modelEvents = CmsEvent::find()->where(['status' => 'active'])->andWhere(['>', 'end',$now])->limit(5)->asArray()->all();        
        
        if(!empty($modelEvents)) {
            foreach ($modelEvents as $key => $value) {
                $eventDetailsArray['name'] = CoreValue::getEntityValue('event', 'name', $value['id']);
                $eventDetailsArray['url'] = CoreValue::getEntityValue('event', 'url', $value['id']);
                array_push($eventsArray, $eventDetailsArray);
            }   
        }

        // Getting testimonial details
        $testimoialModel = CmsTestimonial::find()->where(['status' => 'active'])->orderBy(['created_at' => SORT_DESC])->limit(5)->asArray()->all();
        $modelEntityClass = CoreEntityClass::findOne(['name' => 'testimonial', 'status' => 'active']);

        if(!empty($testimoialModel)) {
            foreach ($testimoialModel as $key => $value) {
                $detailsArray['name'] = CoreValue::getEntityValue('testimonial', 'name', $value['id']);
                $detailsArray['role'] = CoreValue::getEntityValue('testimonial', 'testimonial_role', $value['id']);
                $detailsArray['company'] = CoreValue::getEntityValue('testimonial', 'testimonial_company', $value['id']);

                $detailsArray['content'] = '';
                $detailsArray['video'] = '';
                $detailsArray['type'] = $value['type'];
                if($value['type'] == 'video') {
                    $detailsArray['video'] = CmsEntityVideo::getAttr(['embed_url'], ['entity_class_id' => $modelEntityClass->id, 'entity_id' => $value['id']]);
                } else {
                    $detailsArray['content'] = CoreValue::getEntityValue('testimonial', 'testimonial_content', $value['id']);
                }
                $detailsArray['image'] = $value['featured_image'];
                array_push($testimonialArray, $detailsArray);
            }   
        }

        $news = \common\modules\cms\models\CmsNews::getData([], [], true, 'cms_news');

        return $this->render('index', [
            'categoryList' => $categoryList,
            'products' => $products,
            'productDetails' => $product_list,
            'blogDetails' => $blog_list,
			'testimonialArray' => $testimonialArray,
			'eventsArray'=> $eventsArray,
            'news' => $news,
            'modelFormNewsletters' => new FormNewsletters(),
            'modelSiteContact' => $modelSiteContact,
            'modelFormContact' => $modelFormContact,
        ]);
    }

    /**
     * Displays about page.
     *
     * @return mixed
     */
    public function actionAbout()
    {
        return $this->render('about');
    }

    /**
     * Displays contact page.
     *
     * @return mixed
     */
    public function actionContact()
    {
        $model = new FormContact();
        $modelSiteContact = new SiteContact();

        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            $modelSiteContact->name = $model->name;
            $modelSiteContact->email = $model->email;
            $modelSiteContact->phone = isset($model->phone) ? $model->phone : null;
            $modelSiteContact->subject = $model->subject;
            $modelSiteContact->message = $model->body;
            $modelSiteContact->site_id = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
            $modelSiteContact->ip_address = Yii::$app->request->getUserIP();
            $modelSiteContact->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
            $modelSiteContact->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;

            if ($modelSiteContact->validate()) {
                $modelSiteContact->save();
            }

            if ($model->sendEmail(Yii::$app->params['email']['contact'][Yii::$app->params['mode']])) {
                Yii::$app->session->setFlash('success', 'Thank you for contacting us. We will respond to you as soon as possible.');
            } else {
                Yii::$app->session->setFlash('error', 'There was an error sending your message.');
            }

            return $this->refresh();
        } else {
            return $this->render('contact', [
                'model' => $model,
            ]);
        }
    }

    public function actionTermsAndConditions()
    {
        return $this->render('terms-and-conditions');
    }

    public function actionPrivacyPolicy()
    {
        return $this->render('privacy-policy');
    }

    public function actionThankYou($msg = '') 
	{
        return $this->render('thankYou',[
            'message' => $msg
        ]);
    }

    public function actionUserLoggedIn() 
	{
        $userLogged = 1;

        if (Yii::$app->user->isGuest) {
            $userLogged = 0;
        }

        return $userLogged;
    }

    public function actionNewsletters()
    {
        $modelFormNewsletters = new FormNewsletters();

        if ($modelFormNewsletters->load(Yii::$app->request->post())) {
            if ($modelFormNewsletters->subscribe()) {
                if (Yii::$app->request->isAjax) {
                    return Json::encode([
                        'result' => 'ok', 
                        'payload' => [
                            'redirect_url' => '/course-preview.html', 
                        ]
                    ]);
                }
            }
        }
    }

    public function actionReview()
    {
        $model = new FormReview();
        $modelSiteReview = new SiteReview();

        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            $modelSiteReview->name = $model->name;
            $modelSiteReview->email = $model->email;
            
            if (!Yii::$app->user->isGuest) {
                $modelSiteReview->user_id = Yii::$app->user->identity->id;
            }

            $modelSiteReview->bio = $model->bio;
            $modelSiteReview->rating_scale = (string) Yii::$app->params['rating_scale'];
            $modelSiteReview->rating = $model->rating;
            $modelSiteReview->review = $model->review;
            $modelSiteReview->visible = 'no';
            $modelSiteReview->entity_class_id = $model->entity_class_id;
            $modelSiteReview->entity_id = $model->entity_id;
            $modelSiteReview->site_id = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
            $modelSiteReview->ip_address = Yii::$app->request->getUserIP();
            $modelSiteReview->user_agent = Yii::$app->request->getUserAgent();
            $modelSiteReview->created_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
            $modelSiteReview->updated_by = Yii::$app->user->isGuest ? 1 : Yii::$app->user->identity->id;
            $modelSiteReview->status = 'active';

            if ($modelSiteReview->validate()) {
                $modelSiteReview->save();

                Yii::$app->session->setFlash('success', 'Thank you for your feedback.');
            } else {
                Yii::$app->session->setFlash('error', 'There was an error submitting your feedback.');
            }

            if (isset($model->entity_id) && isset($model->entity_class_id)) {
                if($model->entity_class_id == CoreEntityClass::getAttr(['id'], ['name' => 'product'])) {
                    return $this->redirect('/index.php?r=shop/shop/view&id=' . $model->entity_id);
                }
            }

            return $this->refresh();
        } elseif ($model->load(Yii::$app->request->post()) && !$model->validate() && isset($model->entity_id) && isset($model->entity_class_id)) {
            if($model->entity_class_id == CoreEntityClass::getAttr(['id'], ['name' => 'product'])) {
                return $this->redirect('/index.php?r=shop/shop/view&id=' . $model->entity_id);
            }
            
            Yii::$app->session->setFlash('error', 'There is something wrong in form validation.');

            $this->redirect(\Yii::$app->request->getReferrer());
        } else {
            return $this->render('review', [
                'model' => $model,
            ]);
        }
    }
}
