<?php

namespace common\modules\site\models;

use Yii;
use yii\base\Model;
use common\modules\site\components\Email;

/**
 * FormContact is the model behind the contact form.
 */
class FormContact extends Model
{
    public $name;
    public $email;
    public $phone;
    public $subject;
    public $body;
    public $reCaptcha;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            // name, email, subject and body are required
            [['name', 'email', 'subject', 'body'], 'required'],
            // email has to be a valid email address
            ['email', 'email'],
            [['reCaptcha'], \himiklab\yii2\recaptcha\ReCaptchaValidator::className()],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'reCaptcha' => 'CAPTCHA',
        ];
    }

    /**
     * Sends an email to the specified email address using the information collected by this model.
     *
     * @param string $email the target email address
     * @return bool whether the email was sent
     */
    public function sendEmail($email)
    {
        $data = array();

        $data['name'] = $this->name;
        $data['email'] = $this->email;
        $data['phone'] = $this->phone;
        $data['body'] = $this->body;

        return Email::send(
            'contact', 
            [
                'html' => 'contact-html', 
                'text' => 'contact-text'
            ], 
            [
                'to' => Yii::$app->params['email']['contact'][Yii::$app->params['mode']], 
                'from' => $this->email
            ], 
            $this->subject, 
            ['data' => $data]
        );
    }
}
