<?php

namespace common\modules\site\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\site\models\SiteContact;

/**
 * SiteContactSearch represents the model behind the search form of `common\modules\site\models\SiteContact`.
 */
class SiteContactSearch extends SiteContact
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'email', 'phone', 'subject', 'message', 'ip_address', 'metadata', 'created_at', 'updated_at'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = SiteContact::find();
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        $query->andFilterWhere(['mod_site_contact.site_id' => $this->site_id]);

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_site_contact.name', $this->name])
            ->andFilterWhere(['like', 'mod_site_contact.email', $this->email])
            ->andFilterWhere(['like', 'mod_site_contact.phone', $this->phone])
            ->andFilterWhere(['like', 'mod_site_contact.subject', $this->subject])
            ->andFilterWhere(['like', 'mod_site_contact.message', $this->message])
            ->andFilterWhere(['like', 'mod_site_contact.ip_address', $this->ip_address])
            ->andFilterWhere(['like', 'mod_site_contact.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_site_contact.id', $this->id])
            ->andFilterWhere(['like', 'mod_site_contact.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_site_contact.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_site_contact.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_site_contact.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
