<?php

namespace common\modules\site\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\site\models\SiteMail;

/**
 * SiteMailSearch represents the model behind the search form of `common\modules\site\models\SiteMail`.
 */
class SiteMailSearch extends SiteMail
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['type', 'email_to', 'email_from', 'email_cc', 'email_bcc', 'subject', 'body', 'ip_address', 'created_at', 'updated_at'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = SiteMail::find();
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        $query->andFilterWhere(['mod_site_mail.site_id' => $this->site_id]);

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_site_mail.type', $this->type])
            ->andFilterWhere(['like', 'mod_site_mail.email_to', $this->email_to])
            ->andFilterWhere(['like', 'mod_site_mail.email_from', $this->email_from])
            ->andFilterWhere(['like', 'mod_site_mail.email_cc', $this->email_cc])
            ->andFilterWhere(['like', 'mod_site_mail.email_bcc', $this->email_bcc])
            ->andFilterWhere(['like', 'mod_site_mail.subject', $this->subject])
            ->andFilterWhere(['like', 'mod_site_mail.body', $this->body])
            ->andFilterWhere(['like', 'mod_site_mail.ip_address', $this->ip_address])
            ->andFilterWhere(['like', 'mod_site_mail.id', $this->id])
            ->andFilterWhere(['like', 'mod_site_mail.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_site_mail.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_site_mail.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_site_mail.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
