<?php

namespace common\modules\site\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\site\models\SiteNewsletters;

/**
 * SiteNewslettersSearch represents the model behind the search form of `common\modules\site\models\SiteNewsletters`.
 */
class SiteNewslettersSearch extends SiteNewsletters
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'email', 'phone', 'ip_address', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = SiteNewsletters::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_site_newsletters.name', $this->name])
            ->andFilterWhere(['like', 'mod_site_newsletters.email', $this->email])
            ->andFilterWhere(['like', 'mod_site_newsletters.phone', $this->phone])
            ->andFilterWhere(['like', 'mod_site_newsletters.ip_address', $this->ip_address])
            ->andFilterWhere(['like', 'mod_site_newsletters.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_site_newsletters.status', $this->status])
            ->andFilterWhere(['like', 'mod_site_newsletters.id', $this->id])
            ->andFilterWhere(['like', 'mod_site_newsletters.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_site_newsletters.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_site_newsletters.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_site_newsletters.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_site_newsletters.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_site_newsletters.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
