<?php

namespace common\modules\site\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%site_notification}}".
 *
 * @property int $id
 * @property string $channel
 * @property string $class
 * @property string $key
 * @property string $message
 * @property string $route
 * @property string $is_read
 * @property int $user_id
 * @property int $entity_class_id
 * @property int $entity_id
 * @property string $app_alias
 * @property int $site_id
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 *
 * @property CoreSite $site
 * @property CoreEntityClass $entityClass
 * @property User $user
 * @property User $createdBy
 * @property User $updatedBy
 */
class SiteNotification extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%site_notification}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['channel', 'class', 'key', 'message', 'route'], 'required'],
            [['message', 'route', 'is_read', 'metadata'], 'string'],
            [['user_id', 'entity_class_id', 'entity_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['channel', 'class', 'key', 'app_alias'], 'string', 'max' => 255],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::className(), 'targetAttribute' => ['site_id' => 'id']],
            [['entity_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreEntityClass::className(), 'targetAttribute' => ['entity_class_id' => 'id']],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'channel' => 'Channel',
            'class' => 'Class',
            'key' => 'Key',
            'message' => 'Message',
            'route' => 'Route',
            'is_read' => 'Is Read',
            'user_id' => 'User ID',
            'entity_class_id' => 'Entity Class ID',
            'entity_id' => 'Entity ID',
            'app_alias' => 'App Alias',
            'site_id' => 'Site ID',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }

    public function behaviors()
    {
        return [
            [
                'class' => \yii\behaviors\TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::className(), ['id' => 'site_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEntityClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreEntityClass::className(), ['id' => 'entity_class_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    public function saveNotification($post_data, $notification_id = 0)
    {
        $session = Yii::$app->session;

        $modelSiteNotification = new self();

        $transaction = Yii::$app->db->beginTransaction();

        try {
            if (isset($notification_id) && $notification_id > 0 && self::find()->where(['id' => $notification_id])->exists()) {   // Model already exists
                $modelSiteNotification = self::findOne(['id' => $notification_id]);

                $modelSiteNotification->load($post_data);

                if (isset($post_data['User']['updated_by'])) {
                    $modelSiteNotification->updated_by = $post_data['User']['updated_by'];
                } else {
                    $modelSiteNotification->updated_by = Yii::$app->user->id;
                }

                $modelSiteNotification->metadata = isset($post_data['SiteNotification']['SiteNotificationMetadata']) ? Globals::setJson($post_data['SiteNotification']['SiteNotificationMetadata']) : '';
            } else {    // Create a new model
                $modelSiteNotification->load($post_data);

                $modelSiteNotification->is_read = 'no';

                if (!isset($post_data['SiteNotification']['site_id']) || empty($post_data['SiteNotification']['site_id'])) {
                    $modelSiteNotification->site_id = (int) \common\modules\core\models\CoreSite::getAttr(['id'], ['code' => Yii::$app->params['site_code']]);
                }

                if (isset($post_data['SiteNotification']['created_by'], $post_data['SiteNotification']['updated_by'])) {
                    $modelSiteNotification->created_by = $post_data['SiteNotification']['created_by'];
                    $modelSiteNotification->updated_by = $post_data['SiteNotification']['updated_by'];
                } else {
                    $modelSiteNotification->created_by = Yii::$app->user->id;
                    $modelSiteNotification->updated_by = Yii::$app->user->id;
                }

                $modelSiteNotification->metadata = isset($post_data['SiteNotification']['SiteNotificationMetadata']) ? Globals::setJson($post_data['SiteNotification']['SiteNotificationMetadata']) : '';
            }

            if ($modelSiteNotification->validate()) {
                $modelSiteNotification->save();

                $transaction->commit();

                return $modelSiteNotification->id;
            } else {
                $transaction->rollBack();

                return false;
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public static function markRead($id)
    {
        $post_data = array();
        $modelSiteNotification = new self;

        $post_data['SiteNotification']['is_read'] = 'yes';

        if (self::find()->where(['id' => $id])->exists()) {
            $modelSiteNotification = self::find()->where(['id' => $id])->one();

            return $modelSiteNotification->saveNotification($post_data, $id);
        } else {
            return false;
        }
    }
}
