<?php

namespace common\modules\site\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\site\models\SiteNotification;

/**
 * SiteNotificationSearch represents the model behind the search form of `common\modules\site\models\SiteNotification`.
 */
class SiteNotificationSearch extends SiteNotification
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'entity_class_id', 'entity_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['channel', 'class', 'key', 'message', 'route', 'is_read', 'app_alias', 'metadata', 'created_at', 'updated_at'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = SiteNotification::find();
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_site_notification.channel', $this->channel])
            ->andFilterWhere(['like', 'mod_site_notification.class', $this->class])
            ->andFilterWhere(['like', 'mod_site_notification.key', $this->key])
            ->andFilterWhere(['like', 'mod_site_notification.message', $this->message])
            ->andFilterWhere(['like', 'mod_site_notification.route', $this->route])
            ->andFilterWhere(['like', 'mod_site_notification.is_read', $this->is_read])
            ->andFilterWhere(['like', 'mod_site_notification.app_alias', $this->app_alias])
            ->andFilterWhere(['like', 'mod_site_notification.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_site_notification.id', $this->id])
            ->andFilterWhere(['like', 'mod_site_notification.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_site_notification.entity_class_id', $this->entity_class_id])
            ->andFilterWhere(['like', 'mod_site_notification.entity_id', $this->entity_id])
            ->andFilterWhere(['like', 'mod_site_notification.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_site_notification.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_site_notification.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_site_notification.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_site_notification.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
