<?php

namespace common\modules\site\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\site\models\SiteReview;

/**
 * SiteReviewSearch represents the model behind the search form of `common\modules\site\models\SiteReview`.
 */
class SiteReviewSearch extends SiteReview
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'user_id', 'entity_class_id', 'entity_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'email', 'bio', 'rating_scale', 'rating', 'review', 'visible', 'ip_address', 'user_agent', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = SiteReview::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_site_review.name', $this->name])
            ->andFilterWhere(['like', 'mod_site_review.email', $this->email])
            ->andFilterWhere(['like', 'mod_site_review.bio', $this->bio])
            ->andFilterWhere(['like', 'mod_site_review.rating_scale', $this->rating_scale])
            ->andFilterWhere(['like', 'mod_site_review.rating', $this->rating])
            ->andFilterWhere(['like', 'mod_site_review.review', $this->review])
            ->andFilterWhere(['like', 'mod_site_review.visible', $this->visible])
            ->andFilterWhere(['like', 'mod_site_review.ip_address', $this->ip_address])
            ->andFilterWhere(['like', 'mod_site_review.user_agent', $this->user_agent])
            ->andFilterWhere(['like', 'mod_site_review.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_site_review.status', $this->status])
            ->andFilterWhere(['like', 'mod_site_review.id', $this->id])
            ->andFilterWhere(['like', 'mod_site_review.user_id', $this->user_id])
            ->andFilterWhere(['like', 'mod_site_review.entity_class_id', $this->entity_class_id])
            ->andFilterWhere(['like', 'mod_site_review.entity_id', $this->entity_id])
            ->andFilterWhere(['like', 'mod_site_review.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_site_review.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_site_review.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_site_review.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_site_review.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
