<?php

namespace common\modules\site\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%site_url}}".
 *
 * @property int $id
 * @property string $name
 * @property string $uuid
 * @property int $entity_class_id
 * @property int $entity_id
 * @property int $affiliate_id
 * @property int $site_id
 * @property string $redirect_type
 * @property string $redirect_url
 * @property string $relative_url
 * @property string $absolute_url
 * @property string $params
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreEntityClass $entityClass
 * @property Affiliate $affiliate
 * @property CoreSite $site
 */
class SiteUrl extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%site_url}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['uuid', 'site_id'], 'required'],
            [['entity_class_id', 'entity_id', 'affiliate_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['redirect_type', 'metadata', 'status'], 'string'],
            [['created_at', 'updated_at'], 'safe'],
            [['name', 'uuid', 'redirect_url', 'relative_url', 'absolute_url', 'params'], 'string', 'max' => 255],
            [['uuid'], 'unique'],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['entity_class_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreEntityClass::className(), 'targetAttribute' => ['entity_class_id' => 'id']],
            [['affiliate_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\affiliate\models\Affiliate::className(), 'targetAttribute' => ['affiliate_id' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::className(), 'targetAttribute' => ['site_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'uuid' => 'Uuid',
            'entity_class_id' => 'Entity Class ID',
            'entity_id' => 'Entity ID',
            'affiliate_id' => 'Affiliate ID',
            'site_id' => 'Site ID',
            'redirect_type' => 'Redirect Type',
            'redirect_url' => 'Redirect Url',
            'relative_url' => 'Relative Url',
            'absolute_url' => 'Absolute Url',
            'params' => 'Params',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getEntityClass()
    {
        return $this->hasOne(\common\modules\core\models\CoreEntityClass::className(), ['id' => 'entity_class_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAffiliate()
    {
        return $this->hasOne(\common\modules\affiliate\models\Affiliate::className(), ['id' => 'affiliate_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::className(), ['id' => 'site_id']);
    }
}