<?php

namespace common\modules\site\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;

use common\modules\site\models\SiteUrl;

/**
 * SiteUrlSearch represents the model behind the search form of `common\modules\site\models\SiteUrl`.
 */
class SiteUrlSearch extends SiteUrl
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'entity_class_id', 'entity_id', 'affiliate_id', 'site_id', 'created_by', 'updated_by'], 'integer'],
            [['name', 'uuid', 'redirect_type', 'redirect_url', 'relative_url', 'absolute_url', 'params', 'metadata', 'created_at', 'updated_at', 'status'], 'safe'],
        ];
    }

    public function attributes()
    {
        // add related fields to searchable attributes
        return array_merge(parent::attributes(), []);
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
		$query = SiteUrl::find()->where(parent::tableName() . '.status = :status', ['status' => 'active']);
	
        // add conditions that should always apply here
        $query->joinWith([]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => [
                'defaultOrder' => [
                    'id' => SORT_DESC, 
                ],
            ],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 'mod_site_url.name', $this->name])
            ->andFilterWhere(['like', 'mod_site_url.uuid', $this->uuid])
            ->andFilterWhere(['like', 'mod_site_url.redirect_type', $this->redirect_type])
            ->andFilterWhere(['like', 'mod_site_url.redirect_url', $this->redirect_url])
            ->andFilterWhere(['like', 'mod_site_url.relative_url', $this->relative_url])
            ->andFilterWhere(['like', 'mod_site_url.absolute_url', $this->absolute_url])
            ->andFilterWhere(['like', 'mod_site_url.params', $this->params])
            ->andFilterWhere(['like', 'mod_site_url.metadata', $this->metadata])
            ->andFilterWhere(['like', 'mod_site_url.status', $this->status])
            ->andFilterWhere(['like', 'mod_site_url.id', $this->id])
            ->andFilterWhere(['like', 'mod_site_url.entity_class_id', $this->entity_class_id])
            ->andFilterWhere(['like', 'mod_site_url.entity_id', $this->entity_id])
            ->andFilterWhere(['like', 'mod_site_url.affiliate_id', $this->affiliate_id])
            ->andFilterWhere(['like', 'mod_site_url.site_id', $this->site_id])
            ->andFilterWhere(['like', 'mod_site_url.created_by', $this->created_by])
            ->andFilterWhere(['like', 'mod_site_url.updated_by', $this->updated_by])
            ->andFilterWhere(['like', 'mod_site_url.created_at', $this->created_at])
            ->andFilterWhere(['like', 'mod_site_url.updated_at', $this->updated_at]);

        return $dataProvider;
    }
}
