<?php

$this->title = 'Dashboard';
$this->params['breadcrumbs'] = [['label' => $this->title]];

$dashboard['widgets']['SmallBox'] = [
    [
        'text' => 'Users',
        'icon' => 'fas fa-users',
        'theme' => 'gradient-info',
        'model' => '\common\modules\user\models\User',
        'select' => [],
        'where' => ['status' => 'active'],
        'fetch' => 'count',
        'linkUrl' => ['/user/user/index']
    ],
    [
        'text' => 'Orders',
        'icon' => 'fas fa-shopping-cart',
        'theme' => 'gradient-success',
        'model' => '\common\modules\shop\models\ShopOrder',
        'select' => [],
        'where' => ['status' => 'active'],
        'fetch' => 'count',
        'linkUrl' => ['/shop/order/index']
    ],
    [
        'text' => 'Amount',
        'icon' => 'fas fa-shopping-cart',
        'theme' => 'gradient-warning',
        'model' => '\common\modules\shop\models\ShopOrder',
        'select' => ['SUM(`amount`)'],
        'where' => ['status' => 'active'],
        'fetch' => 'scalar',
        'linkUrl' => ['/shop/order/index']
    ],
    [
        'text' => 'Visitors',
        'icon' => 'fas fa-eye',
        'theme' => 'gradient-danger',
        'model' => '\common\modules\logs\models\LogsVisitor',
        'select' => [],
        'where' => [],
        'fetch' => 'count',
        'linkUrl' => ['/logs/visitor/index']
    ],
];

$dashboard['widgets']['InfoBox'] = [
    [
        'text' => 'Leads',
        'icon' => 'fas fa-filter',
        'model' => '\common\modules\mailer\models\MailerContact',
        'select' => [],
        'where' => ['status' => 'active'],
        'fetch' => 'count',
    ],
    [
        'text' => 'Conversions',
        'icon' => 'fas fa-funnel-dollar',
        'model' => '\common\modules\mailer\models\MailerContact',
        'select' => ['JSON_EXTRACT(`metadata`, "$.lead_status")'],
        'where' => ['status' => 'active', 'JSON_EXTRACT(`metadata`, "$.lead_status")' => 'converted'],
        'fetch' => 'count',
    ],
];

?>

<div class="container-fluid">
    <div class="row">
        <?php foreach ($dashboard['widgets']['SmallBox'] as $widgets_small_box) { ?>
            <?php $fetch = $widgets_small_box['fetch']; ?>

            <div class="col-lg-3 col-md-6 col-sm-6 col-12">
                <?= \hail812\adminlte\widgets\SmallBox::widget([
                    'title' => intval($widgets_small_box['model']::find()->select($widgets_small_box['select'])->where($widgets_small_box['where'])->$fetch()),
                    'text' => $widgets_small_box['text'],
                    'icon' => $widgets_small_box['icon'],
                    'theme' => $widgets_small_box['theme'],
                    'linkUrl' => $widgets_small_box['linkUrl'],
                ]) ?>
            </div>
        <?php } ?>
    </div>

    <div class="row">
        <?php foreach ($dashboard['widgets']['InfoBox'] as $widgets_info_box) { ?>
            <?php $fetch = $widgets_info_box['fetch']; ?>

            <div class="col-md-3 col-sm-6 col-12">
                <?= \hail812\adminlte\widgets\InfoBox::widget([
                    'text' => $widgets_info_box['text'],
                    'number' => intval($widgets_info_box['model']::find()->select($widgets_info_box['select'])->where($widgets_info_box['where'])->$fetch()),
                    'icon' => $widgets_info_box['icon'],
                ]) ?>
            </div>
        <?php } ?>
    </div>

    <div class="row">
        <!-- Interval: Last 30 Days -->
        <?php
        $interval = 'daily';
        $where = ['between', 'created_at', date('Y-m-d', strtotime('today - 31 days')), date('Y-m-d', strtotime('tomorrow'))];
        $chart_data = \common\modules\shop\models\ShopOrder::getChartDataInterval($interval, false, [], $where);
        ?>

        <?php if (!empty($chart_data)) { ?>
            <div class="col-lg-6">
                <div class="card">
                    <div class="card-header border-0">
                        <div class="d-flex justify-content-between">
                            <h3 class="card-title"><?= ucfirst($interval) ?> Orders <small>(Last 30 Days)</small></h3>
                        </div>
                    </div>
                    <div class="card-body">
                        <div class="d-flex">
                            <p class="d-flex flex-column">
                                <span class="text-bold text-lg"><?= \common\modules\shop\models\ShopOrder::find()->where($where)->andWhere(['status' => 'active'])->count() ?></span>
                                <span>Orders</span>
                            </p>
                            <p class="ml-auto d-flex flex-column text-right">
                                <span class="text-bold text-lg"><?= count($chart_data) ?></span>
                                <span class="text-muted">Days</span>
                            </p>
                        </div>

                        <div class="position-relative mb-4">
                            <?= \common\widgets\ChartJsBar::widget([
                                'id' => 'ChartJsBarOrders' . ucfirst($interval) . 'Last30Days',
                                'label' => 'Orders',
                                'labels' => array_keys($chart_data),
                                'data' => array_values($chart_data),
                                'minBars' => 0,
                                'maxBars' => 30,
                            ]) ?>
                        </div>
                    </div>
                </div>
            </div>
        <?php } ?>

        <!-- Interval: Last 30 Days -->
        <?php
        $interval = 'daily';
        $where = ['between', 'created_at', date('Y-m-d', strtotime('today - 31 days')), date('Y-m-d', strtotime('tomorrow'))];
        $chart_data = \common\modules\shop\models\ShopOrder::getChartDataInterval($interval, true, ['SUM(`amount`) AS `amount`'], $where);
        ?>

        <?php if (!empty($chart_data)) { ?>
            <div class="col-lg-6">
                <div class="card">
                    <div class="card-header border-0">
                        <div class="d-flex justify-content-between">
                            <h3 class="card-title"><?= ucfirst($interval) ?> Orders <small>(Last 30 Days)</small></h3>
                        </div>
                    </div>
                    <div class="card-body">
                        <div class="d-flex">
                            <p class="d-flex flex-column">
                                <span class="text-bold text-lg"><?= \common\modules\shop\models\ShopOrder::find()->where($where)->andWhere(['status' => 'active'])->count() ?></span>
                                <span>Orders</span>
                            </p>
                            <p class="ml-auto d-flex flex-column text-center">
                                <span class="text-bold text-lg"><i class="fas fa-dollar-sign"></i> <?= array_sum($chart_data['amount']) ?></span>
                                <span class="text-muted">Amount</span>
                            </p>
                            <p class="ml-auto d-flex flex-column text-right">
                                <span class="text-bold text-lg"><?= count($chart_data['created_at']) ?></span>
                                <span class="text-muted">Days</span>
                            </p>
                        </div>

                        <div class="position-relative mb-4">
                            <?= \common\widgets\ChartJsLineMultiAxis::widget([
                                'id' => 'ChartJsLineMultiAxisOrders' . ucfirst($interval) . 'Last30Days',
                                'label' => ['Orders', 'Amount'],
                                'labels' => $chart_data['created_at'],
                                'data' => [$chart_data['count'], $chart_data['amount']],
                                'minBars' => 0,
                                'maxBars' => 30,
                            ]) ?>
                        </div>
                    </div>
                </div>
            </div>
        <?php } ?>

        <!-- Interval: Last 30 Days -->
        <?php
        $interval = 'daily';
        $where = ['between', 'created_at', date('Y-m-d', strtotime('today - 31 days')), date('Y-m-d', strtotime('tomorrow'))];
        $chart_data = \common\modules\user\models\User::getChartDataInterval($interval, false, [], $where);
        ?>

        <?php if (!empty($chart_data)) { ?>
            <div class="col-lg-6">
                <div class="card">
                    <div class="card-header border-0">
                        <div class="d-flex justify-content-between">
                            <h3 class="card-title"><?= ucfirst($interval) ?> Users <small>(Last 30 Days)</small></h3>
                        </div>
                    </div>
                    <div class="card-body">
                        <div class="d-flex">
                            <p class="d-flex flex-column">
                                <span class="text-bold text-lg"><?= \common\modules\user\models\User::find()->where($where)->andWhere(['status' => 'active'])->count() ?></span>
                                <span>Users</span>
                            </p>
                            <p class="ml-auto d-flex flex-column text-right">
                                <span class="text-bold text-lg"><?= count($chart_data) ?></span>
                                <span class="text-muted">Days</span>
                            </p>
                        </div>

                        <div class="position-relative mb-4">
                            <?= \common\widgets\ChartJsBar::widget([
                                'id' => 'ChartJsBarUsers' . ucfirst($interval) . 'Last30Days',
                                'label' => 'Users',
                                'labels' => array_keys($chart_data),
                                'data' => array_values($chart_data),
                                'minBars' => 0,
                                'maxBars' => 30,
                            ]) ?>
                        </div>
                    </div>
                </div>
            </div>
        <?php } ?>

        <!-- Interval: Last 30 Days -->
        <?php
        $interval = 'daily';
        $where = ['between', 'created_at', date('Y-m-d', strtotime('today - 31 days')), date('Y-m-d', strtotime('tomorrow'))];
        $chart_data = \common\modules\logs\models\LogsVisitor::getChartDataInterval($interval, false, [], $where, true);
        ?>

        <?php if (!empty($chart_data)) { ?>
            <div class="col-lg-6">
                <div class="card">
                    <div class="card-header border-0">
                        <div class="d-flex justify-content-between">
                            <h3 class="card-title"><?= ucfirst($interval) ?> Visitors <small>(Last 30 Days)</small></h3>
                        </div>
                    </div>
                    <div class="card-body">
                        <div class="d-flex">
                            <p class="d-flex flex-column">
                                <span class="text-bold text-lg"><?= \common\modules\logs\models\LogsVisitor::find()->where($where)->count() ?></span>
                                <span>Visitors</span>
                            </p>
                            <p class="ml-auto d-flex flex-column text-right">
                                <span class="text-bold text-lg"><?= count($chart_data) ?></span>
                                <span class="text-muted">Days</span>
                            </p>
                        </div>

                        <div class="position-relative mb-4">
                            <?= \common\widgets\ChartJsBar::widget([
                                'id' => 'ChartJsBarVisitorLogs' . ucfirst($interval) . 'Last30Days',
                                'label' => 'Visitors',
                                'labels' => array_keys($chart_data),
                                'data' => array_values($chart_data),
                                'minBars' => 0,
                                'maxBars' => 30,
                            ]) ?>
                        </div>
                    </div>
                </div>
            </div>
        <?php } ?>
    </div>
</div>
