<?php

namespace common\modules\user\controllers\frontend;

use Yii;
use yii\web\Controller;
use yii\web\BadRequestHttpException;
use yii\base\InvalidParamException;
use yii\helpers\Url;
use yii\helpers\ArrayHelper;
use Yadakhov\Json;

use common\modules\user\models\User;
use common\modules\user\models\UserSearch;
use common\modules\user\models\UserProfile;
use common\modules\user\models\UserLogsLogin;
use common\modules\user\models\FormRegister;
use common\modules\user\models\FormLogin;
use common\modules\user\models\FormPasswordResetRequest;
use common\modules\user\models\FormResetPassword;
use common\modules\user\models\FormProfile;
use common\modules\user\models\FormChangePassword;
use common\modules\core\components\GenesysRestClient;
use common\modules\site\components\Email;
use common\modules\shop\components\Product;
use common\modules\shop\components\ShopEvents;
use common\modules\shop\models\ShopProduct;
use common\modules\shop\models\ShopTrait;
use common\modules\shop\models\ShopTraitValue;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\shop\models\ShopOrderStatus;
use common\modules\shop\models\ShopWishlist;
use common\modules\payment\models\FormSquare;
use common\modules\lms\models\LmsCourseActivity;
use common\modules\lms\models\LmsUserCourse;

class UserController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['register', 'login', 'request-password-reset', 'reset-password', 'logout', 'dashboard', 'change-password', 'profile'],
                'rules' => [
                    [
                        'actions' => ['register', 'login', 'request-password-reset', 'reset-password'],
                        'allow' => true,
                        'roles' => ['?'],
                    ],
                    [
                        'actions' => ['logout', 'dashboard', 'change-password', 'profile'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => [],
                        'allow' => true,
                        'roles' => ['?', '@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => \yii\filters\VerbFilter::className(),
                'actions' => [
                    'register' => ['get', 'post'],
                    'login' => ['get', 'post'],
                    'request-password-reset' => ['get', 'post'],
                    'reset-password' => ['get', 'post'],
                    'logout' => ['get', 'post'],
                    'dashboard' => ['get'],
                    'change-password' => ['get', 'post'],
                    'profile' => ['get', 'post'],
                ],
            ],
        ];
    }

    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    public function beforeAction($action)
    {
        if ($action->id == 'checkout') {
            if (Yii::$app->user->isGuest) {
                Url::remember();

                Yii::$app->user->loginUrl = ['login'];
            }
        }

        return parent::beforeAction($action);
    }

    public function actionRegister($product = '')
    {
        $model = new FormRegister();
        $modelFormSquare = new FormSquare();

        if ($model->load(Yii::$app->request->post())) {
            if ($user = $model->signup()) {
                Email::send(
                    'registration-user', 
                    [
                        'html' => 'registration-user-html', 
                        'text' => 'registration-user-text'
                    ], 
                    [
                        'to' => $user->email, 
                        'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
                    ], 
                    NULL, 
                    ['data' => $user], 
                    Yii::$app->params['site_code'], 
                    Yii::$app->params['site_code']
                );

                if (Yii::$app->getUser()->login($user)) {
                    if (Yii::$app->request->post('redirect-url') !== null && !empty(Yii::$app->request->post('redirect-url'))) {
						$redirect = Yii::$app->request->post('redirect-url');
						$user_id = Yii::$app->user->id;
						
                        if (Yii::$app->request->isAjax) {
                            if (Yii::$app->request->post('cart-total') !== null && Yii::$app->request->post('cart-total') <= 0) {
                                $data = array();
                                $eventShopEvents = new ShopEvents;

                                $eventShopEvents->eventModShopCreateOrder($data, 'offline_payment');
                            }

                            return json_encode([
                                'result' => 'ok', 
                                'payload' => [
                                    'redirect_url' => $redirect
                                ]
                            ]);
                        } else {
                            return Yii::$app->response->redirect([$redirect]);
                        }
                    }
					
					$previousUrl = Url::previous();
            
                    if (isset($previousUrl) && !empty($previousUrl)) {
                        if (Yii::$app->request->isAjax) {
                            if (Yii::$app->request->post('cart-total') !== null && Yii::$app->request->post('cart-total') <= 0) {
                                $data = array();
                                $eventShopEvents = new ShopEvents;

                                $eventShopEvents->eventModShopCreateOrder($data, 'offline_payment');
                            }

                            return json_encode([
                                'result' => 'ok', 
                                'payload' => [
                                    'redirect_url' => $previousUrl
                                ]
                            ]);
                        } else {
                            return $this->redirect($previousUrl);
                        }
                    }

                    if (Yii::$app->request->isAjax) {
                        if (Yii::$app->request->post('cart-total') !== null && Yii::$app->request->post('cart-total') <= 0) {
                            $data = array();
                            $eventShopEvents = new ShopEvents;

                            $eventShopEvents->eventModShopCreateOrder($data, 'offline_payment');
                        }

                        return json_encode([
                            'result' => 'ok', 
                            'payload' => [
                                'redirect_url' => $this->goHome()
                            ]
                        ]);
                    } else {
                        return $this->goHome();
                    }
                }
            }
        }

        return $this->render('register', [
            'model' => $model,
            'modelFormSquare' => $modelFormSquare,
        ]);
    }

    public function actionRegisterRest()
    {
        $model = new FormRegister();
        $modelUser = new User();
        $restClient = new GenesysRestClient();

        if (Yii::$app->request->isPost) {
            $response = new Json($restClient->callApi('users', 'POST', '', Yii::$app->request->post('FormRegister')));

            if (!empty($response->get('data'))) {
                $modelUser = User::find()->where(['id' => $response->get('data'), 'status' => 'active'])->one();

                Yii::$app->session->setFlash('success', "Registration successful.");

                if (isset($modelUser->access_token) && !empty($modelUser->access_token)) {
                    return $this->redirect('/exam-enrollment/' . $modelUser->access_token, 302);
                } else {
                    return $this->redirect('/login', 302);
                }
            } else {
                Yii::$app->session->setFlash('error', "Registration failed.");

                return $this->redirect('/register', 302);
            }
        } else {
            return $this->render('register', [
                'model' => $model,
            ]);
        }
    }

    public function actionLogin()
    {
        if (!Yii::$app->user->isGuest) {
            $previousUrl = Url::previous();
            
            if (isset($previousUrl) && !empty($previousUrl)) {
                return $this->redirect($previousUrl);
            }

            return $this->goHome();
        }

        $model = new FormLogin();

        if ($model->load(Yii::$app->request->post()) && $model->login()) {
            $previousUrl = Url::previous();
            
            if (isset($previousUrl) && !empty($previousUrl)) {
                return $this->redirect($previousUrl);
            }

            return $this->goBack();
        } else {
            return $this->render('login', [
                'model' => $model,
            ]);
        }
    }

    public function actionLoginRest()
    {
        $model = new FormLogin();
        $modelLmsUserCourse = new LmsUserCourse();
        $restClient = new GenesysRestClient();

        if (Yii::$app->request->isPost) {
            $response = new Json($restClient->callApi('users/login', 'POST', '', Yii::$app->request->post('FormLogin')));

            /*if (!LmsUserCourse::find()->where(['user_id' => User::getAttr(['id'], ['access_token' => $response->get('data')['access_token']]), 'status' => 'active'])->andWhere(['IS NOT', 'metadata', null])->exists()) {
                return $this->redirect('/exam-enrollment/' . $response->get('data')['access_token'], 302);
            }*/

            if (strpos(Url::base(true), 'feedback') !== false) {
                return $this->redirect('/quiz-home/' . $response->get('data')['access_token'], 302);
            } else {
                if (LmsUserCourse::find()->where(['user_id' => User::getAttr(['id'], ['access_token' => $response->get('data')['access_token']]), 'status' => 'active'])->andWhere(['IS NOT', 'metadata', null])->exists()) {
                    $modelLmsUserCourse = LmsUserCourse::find()->where(['user_id' => User::getAttr(['id'], ['access_token' => $response->get('data')['access_token']]), 'status' => 'active'])->andWhere(['IS NOT', 'metadata', null])->all();

                    $user_course_metadata = $tmp_user_course_metadata = array();
                    $tmp_exam_date = date('Y-m-d');

                    foreach ($modelLmsUserCourse as $key => $value) {
                        $tmp_user_course_metadata = json_decode($value->metadata, true);

                        if (isset($tmp_user_course_metadata['exam_date']) && $tmp_user_course_metadata['exam_date'] >= date('Y-m-d')) {
                            if ($tmp_user_course_metadata['exam_date'] <= $tmp_exam_date) {
                                $tmp_exam_date = $tmp_user_course_metadata['exam_date'];

                                $user_course_metadata = $tmp_user_course_metadata;
                            }
                        }
                    }

                    if (isset($user_course_metadata['exam']) && LmsCourseActivity::find()->where(['id' => $user_course_metadata['exam'], 'status' => 'active'])->exists()) {
                        return $this->redirect('/launch-quiz/' . $response->get('data')['access_token'] . '/' . $user_course_metadata['exam'], 302);
                    }
                }

                return $this->redirect('/launch-quiz/' . $response->get('data')['access_token'] . '/265', 302);
                // die('Examination over!');
            }
        } else {
            return $this->render('login', [
                'model' => $model,
            ]);
        }
    }

    public function actionLogout()
    {
        Yii::$app->user->logout();

        return $this->goHome();
    }

    public function actionRequestPasswordReset()
    {
        $model = new FormPasswordResetRequest();
        
        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            if ($model->sendEmail()) {
                Yii::$app->session->setFlash('success', 'Check your email for further instructions.');

                return $this->goHome();
            } else {
                Yii::$app->session->setFlash('error', 'Sorry, we are unable to reset password for email provided.');
            }
        }

        return $this->render('request-password-reset', [
            'model' => $model,
        ]);
    }

    public function actionResetPassword($token)
    {
        try {
            $model = new FormResetPassword($token);
        } catch (InvalidParamException $e) {
            throw new BadRequestHttpException($e->getMessage());
        }

        if ($model->load(Yii::$app->request->post()) && $model->validate() && $model->resetPassword()) {
            Yii::$app->session->setFlash('success', 'New password was saved.');

            return $this->goHome();
        }

        return $this->render('reset-password', [
            'model' => $model,
        ]);
    }

    public function actionAccount()
    {
        if (Yii::$app->user->isGuest) {
            Url::remember();
            return $this->redirect('login', 302);
        }

        $orderDetails = array();
        $modelUser = User::findOne(['id' => Yii::$app->user->identity->id, 'status' => 'active']);
        $modelUserProfile = UserProfile::findOne(['user_id' => Yii::$app->user->identity->id, 'status' => 'active']);
        $modelOrder = ShopOrder::find()->where(['user_id' => Yii::$app->user->identity->id, 'status' => 'active'])->asArray()->all();

        if ($modelOrder !== null && !empty($modelOrder)) {
            $orderIdArr = ArrayHelper::getColumn($modelOrder,'id');
            $modelOrderItem = ShopOrderItem::find()->where(['order_id' => $orderIdArr, 'status' => 'active'])->asArray()->all();
            $productArr = ArrayHelper::getColumn($modelOrderItem,'product_id');
            $modelProduct = ShopProduct::find()->where(['id' => $productArr,'status' => 'active'])->all();
            $productDetails = Product::getProductDetails($modelProduct);
            $orderStatusArr = ArrayHelper::getColumn($modelOrder,'order_status_id');
            $modelOrderStatus = ShopOrderStatus::find()->where(['id' => $orderStatusArr, 'status' => 'active'])->asArray()->all();
            $orderStatusArr = ArrayHelper::index($modelOrderStatus, 'id');
            $traitArr = ArrayHelper::index($modelOrderItem,'id');
            $traitDetails = ShopTrait::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'trait', $attribute_class = 'name');
            $traitValueDetails = ShopTraitValue::getAll($k = 'id', $v = 'name', $eav = true, $entity_class = 'trait_value', $attribute_class = 'name');

            if ($productDetails !== null && !empty($productDetails)) {
                foreach ($modelOrder as $key => $value) {
                    foreach ($modelOrderItem as $k => $v) {
                        if ($value['id'] == $v['order_id']) {
                            $orderDetails[$value['id']]['invoice'] = $value['invoice_number'];
                            $orderDetails[$value['id']]['status'] = $orderStatusArr[$value['order_status_id']]['name'];
                            $orderDetails[$value['id']]['order_date'] = $value['order_date'];
                            $orderDetails[$value['id']]['product'][$v['id']]['quantity'] = $v['quantity'];
                            $orderDetails[$value['id']]['product'][$v['id']]['price'] = $v['price'];                            
                            $orderDetails[$value['id']]['product'][$v['id']]['product_info'] = $productDetails[$v['product_id']];
                            
                            $traitValue = array();

                            if (!empty($v['traits']) && $v['traits'] !== null) {
                                $traits = json_decode($v['traits'], true);

                                foreach ($traits as $traitsid => $traitsValue) {
                                    if (array_key_exists($traitsid, $traitDetails) && array_key_exists($traitsValue, $traitValueDetails)) {
                                        $traitValue[$traitDetails[$traitsid]] = $traitValueDetails[$traitsValue];
                                    }
                                }
                            }

                            $orderDetails[$value['id']]['product'][$v['id']]['traits'] = $traitValue;
                        }                        
                    }
                }               
            }
        }
        
        if (Yii::$app->request->isPost) {
            if ($user_id = $modelUser->saveUser(Yii::$app->request->post(), Yii::$app->user->identity->id)) {
                return $this->redirect([
                    'account', 
                    'modelUser' => $modelUser,
                    'orderDetails' => $orderDetails,
                    'modelUserProfile' => $modelUserProfile,
                ]);
            } else {
                return $this->render('account', [
                    'modelUser' => $modelUser,
                    'orderDetails' => $orderDetails,
                    'modelUserProfile' => $modelUserProfile,
                ]);
            }
        } else {
            return $this->render('account', [
                'modelUser' => $modelUser,
                'orderDetails' => $orderDetails,
                'modelUserProfile' => $modelUserProfile,
            ]);
        }
        
        return $this->render('account', [
            'model' => $model,
            'modelUser' => $modelUser,
            'orderDetails' => $orderDetails,
            'modelUserProfile' => $modelUserProfile,
        ]);
    }

    public function actionUserDashboard()
    {
        if (Yii::$app->user->isGuest) {
            Url::remember();

            return $this->redirect('/login', 302);
        }

        $modelUser = User::find()->where(['id' => Yii::$app->user->identity->id, 'status' => 'active'])->one();

        $modelUserLogsLogin = UserLogsLogin::find()->where(['user' => Yii::$app->user->identity->email])->orderBy(['created_at' => SORT_DESC])->one();
        
        return $this->render('user-dashboard', [
            'modelUser' => $modelUser,
            'modelUserLogsLogin' => $modelUserLogsLogin,
        ]);           
    }

    public function actionDashboard()
    {
        if (Yii::$app->user->isGuest) {
            Url::remember();

            return $this->redirect('/login', 302);
        }

        $modelUser = User::find()->where(['id' => Yii::$app->user->identity->id, 'status' => 'active'])->one();
    	$modelUserProfile = UserProfile::find()->where(['user_id' => $modelUser->id, 'status' => 'active'])->one();
    	$modelLmsUserCourse = LmsUserCourse::find()->where(['user_id' => $modelUser->id, 'status' => 'active'])->all();
        
        return $this->render('dashboard', [
            'modelUser' => $modelUser,
            'modelUserProfile' => $modelUserProfile,
            'modelLmsUserCourse' => $modelLmsUserCourse,
        ]);
    }

    public function actionChangePassword()
    {
        if (Yii::$app->user->isGuest) {
            Url::remember();

            return $this->redirect('/login', 302);
        }

        try {
            $model = new FormChangePassword(Yii::$app->user->identity->auth_key);
        } catch (InvalidParamException $e) {
            throw new BadRequestHttpException($e->getMessage());
        }

        if (Yii::$app->request->isPost) {
            if ($model->load(Yii::$app->request->post()) && $model->validate() && $model->changePassword()) {
                Yii::$app->session->setFlash('success', 'New password was saved.');
            } else {
                Yii::$app->session->setFlash('success', 'Unable to save new password.');
            }
        }

        return $this->render('change-password', [
            'model' => $model,
        ]);
    }

    public function actionProfile()
    {
        if (Yii::$app->user->isGuest) {
            Url::remember();

            return $this->redirect('login', 302);
        }

        try {
            $model = new FormProfile(Yii::$app->user->identity->auth_key);
        } catch (InvalidParamException $e) {
            throw new BadRequestHttpException($e->getMessage());
        }

        if (Yii::$app->request->isPost) {
            if ($model->load(Yii::$app->request->post()) && $model->validate() && $model->updateProfile()) {
                Yii::$app->session->setFlash('success', 'User profile was updated.');
            } else {
                Yii::$app->session->setFlash('danger', 'Unable to update user profile.');
            }
        }
        
        return $this->render('profile', [
            'model' => $model,
        ]);
    }
}
