<?php

namespace common\modules\user\models;

use yii\base\Model;
use yii\base\InvalidParamException;

use common\modules\user\models\UserIdentity;

/**
 * Password change form
 */
class FormChangePassword extends Model
{

    public $password;
    public $password_confirm;

    /**
     * @var \common\modules\user\models\UserIdentity
     */
    private $_user;


    /**
     * Creates a form model given a authentication key.
     *
     * @param string $auth_key
     * @param array $config name-value pairs that will be used to initialize the object properties
     * @throws \yii\base\InvalidParamException if authentication key is empty or not valid
     */
    public function __construct($auth_key, $config = [])
    {
        if (empty($auth_key) || !is_string($auth_key)) {
            throw new InvalidParamException('Authentication key cannot be blank.');
        }

        $this->_user = UserIdentity::findByAuthKey($auth_key);

        if (!$this->_user) {
            throw new InvalidParamException('Wrong authentication key.');
        }
        
        parent::__construct($config);
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            ['password', 'required'],
            ['password', 'string', 'min' => 6],

            ['password_confirm', 'required'],
            ['password_confirm', 'compare', 'compareAttribute' => 'password'],
        ];
    }

    /**
     * Changes password.
     *
     * @return bool if password was changed.
     */
    public function changePassword()
    {
        $user = $this->_user;
        $user->setPassword($this->password);

        return $user->save(false);
    }
}
