<?php

namespace common\modules\user\models;

use Yii;
use yii\base\Model;
use common\modules\user\models\UserLogsLogin;
use common\modules\core\models\CoreSite;

/**
 * Login form
 */
class FormLogin extends Model
{
    public $email;
    public $password;
    public $rememberMe = true;
    public $role_id;

    private $_user;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            // email and password are both required
            [['email', 'password'], 'required'],
            ['email', 'email'],
            // rememberMe must be a boolean value
            ['rememberMe', 'boolean'],
            // password is validated by validatePassword()
            ['password', 'validatePassword'],
        ];
    }

    /**
     * Validates the password.
     * This method serves as the inline validation for password.
     *
     * @param string $attribute the attribute currently being validated
     * @param array $params the additional name-value pairs given in the rule
     */
    public function validatePassword($attribute, $params)
    {
        if (!$this->hasErrors()) {
            $user = $this->getUser();
            if (!$user || !$user->validatePassword($this->password) || $user->role_id != $this->role_id) {
                $this->addError($attribute, 'Incorrect email or password.');
            }
        }
    }

    /**
     * Logs in a user using the provided email and password.
     *
     * @return bool whether the user is logged in successfully
     */
    public function login($app_name = 'frontend')
    {
        $modelUserLogsLogin = new UserLogsLogin();

        switch ($app_name) {
            case 'frontend':
                if ($tmp = UserIdentity::isUser($this->email)) {
                    $this->role_id = $tmp;
					
                    $site_id = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);

                    if ($this->validate()) {
                        $modelUserLogsLogin->saveLogsLogin($this->email, 'yes', 'frontend', $site_id);

                        return Yii::$app->user->login($this->getUser(), $this->rememberMe ? 3600 * 24 * 30 : 0);
                    } else {
                        $modelUserLogsLogin->saveLogsLogin($this->email, 'no', 'frontend', $site_id);

                        return false;
                    }
                } else {
                    $site_id = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);
                    
					$modelUserLogsLogin->saveLogsLogin($this->email, 'no', 'frontend', $site_id);

                    return false;
                }

                break;

            case 'backend':
                if ($tmp = (UserIdentity::isAdmin($this->email) || 
                    UserIdentity::isTester($this->email) || 
                    UserIdentity::isEditor($this->email) || 
                    UserIdentity::isSalesRep($this->email) || 
                    UserIdentity::isTeacher($this->email) || 
                    UserIdentity::isTenant($this->email))
                ) {
                    $this->role_id = $tmp;

                    if ($this->validate()) {
                        $modelUserLogsLogin->saveLogsLogin($this->email, 'yes', 'backend');

                        return Yii::$app->user->login($this->getUser(), $this->rememberMe ? 3600 : 0);
                    } else {
                        $modelUserLogsLogin->saveLogsLogin($this->email, 'no', 'backend');

                        return false;
                    }
                } else {
                    $modelUserLogsLogin->saveLogsLogin($this->email, 'no', 'backend');

                    return false;
                }

                break;
            
            default:
                $modelUserLogsLogin->saveLogsLogin($this->email, 'no', 'frontend');

                return false;

                break;
        }
    }

    /**
     * Finds user by [[email]]
     *
     * @return User|null
     */
    protected function getUser()
    {
        if ($this->_user === null) {
            $this->_user = UserIdentity::findByEmail($this->email);
        }
		
        return $this->_user;
    }
}
