<?php

namespace common\modules\user\models;

use Yii;
use yii\base\Model;
use common\modules\site\components\Email;
use common\modules\user\models\UserIdentity;

/**
 * Password reset request form
 */
class FormPasswordResetRequest extends Model
{
    public $email;


    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            ['email', 'trim'],
            ['email', 'required'],
            ['email', 'email'],
            ['email', 'exist',
                'targetClass' => '\common\modules\user\models\UserIdentity',
                'filter' => ['status' => UserIdentity::STATUS_ACTIVE],
                'message' => 'There is no user with such email.'
            ],
        ];
    }

    /**
     * Sends an email with a link, for resetting the password.
     *
     * @return bool whether the email was send
     */
    public function sendEmail()
    {
        /* @var $user UserIdentity */
        $user = UserIdentity::findOne([
            'status' => UserIdentity::STATUS_ACTIVE,
            'email' => $this->email,
        ]);

        if (!$user) {
            return false;
        }
        
        if (!UserIdentity::isPasswordResetTokenValid($user->password_reset_token)) {
            $user->generatePasswordResetToken();
            if (!$user->save()) {
                return false;
            }
        }

        $user = UserIdentity::find()->where(['status' => UserIdentity::STATUS_ACTIVE, 'email' => $this->email])->asArray()->one();

        return Email::send(
            'password-reset-customer', 
            [
                'html' => 'passwordResetToken-html', 
                'text' => 'passwordResetToken-text'
            ], 
            [
                'to' => $this->email, 
                'from' => Yii::$app->params['email']['from'][Yii::$app->params['mode']]
            ], 
            NULL, 
            ['user' => $user]
        );
    }
}
