<?php

namespace common\modules\user\models;

use Yii;
use yii\base\Model;

use common\modules\user\models\User;
use common\modules\user\models\UserRole;
use common\modules\core\models\CoreSite;

/**
 * Profile form
 */
class FormProfile extends Model
{
    
    public $email;

    public $first_name;
    public $last_name;
    public $gender;
    public $date_of_birth;
    public $avatar;
    
    public $ba_address;
    public $ba_city;
    public $ba_country;
    public $ba_zip_code;

    public $sa_address;
    public $sa_city;
    public $sa_country;
    public $sa_zip_code;
    
    public $phone;

    public $newsletters;

    /**
     * @var \common\modules\user\models\User
     */
    private $_user;

    /**
     * @var \common\modules\user\models\UserProfile
     */
    private $_userProfile;

    /**
     * Creates a form model given a authentication key.
     *
     * @param string $auth_key
     * @param array $config name-value pairs that will be used to initialize the object properties
     * @throws \yii\base\InvalidParamException if authentication key is empty or not valid
     */
    public function __construct($auth_key, $config = [])
    {
        if (empty($auth_key) || !is_string($auth_key)) {
            throw new InvalidParamException('Authentication key cannot be blank.');
        }

        $this->_user = User::find()->where(['auth_key' => $auth_key, 'status' => 'active'])->one();
        $this->_userProfile = UserProfile::find()->where(['user_id' => $this->_user->id, 'status' => 'active'])->one();

        if (!$this->_user) {
            throw new InvalidParamException('Wrong authentication key.');
        }

        $this->email = $this->_user->email;

        $this->first_name = $this->_userProfile->first_name;
        $this->last_name = $this->_userProfile->last_name;
        $this->gender = $this->_userProfile->gender;
        $this->date_of_birth = $this->_userProfile->date_of_birth;
        $this->avatar = $this->_userProfile->avatar;

        $this->ba_address = $this->_userProfile->ba_address;
        $this->ba_city = $this->_userProfile->ba_city;
        $this->ba_country = $this->_userProfile->ba_country;
        $this->ba_zip_code = $this->_userProfile->ba_zip_code;

        $this->sa_address = $this->_userProfile->sa_address;
        $this->sa_city = $this->_userProfile->sa_city;
        $this->sa_country = $this->_userProfile->sa_country;
        $this->sa_zip_code = $this->_userProfile->sa_zip_code;

        $this->phone = $this->_userProfile->phone;

        $this->newsletters = $this->_userProfile->newsletters;
        
        parent::__construct($config);
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            ['email', 'trim'],
            ['email', 'required'],
            ['email', 'email'],
            ['email', 'string', 'max' => 255],

            ['first_name', 'filter', 'filter' => 'trim'],
            ['first_name', 'required'],
            ['first_name', 'string', 'min' => 2, 'max' => 255],

            ['last_name', 'filter', 'filter' => 'trim'],
            ['last_name', 'required'],
            ['last_name', 'string', 'min' => 2, 'max' => 255],

            ['gender', 'required'],
			
			['ba_address', 'filter', 'filter' => 'trim'],
            ['ba_address', 'required', 'message' => 'Billing Address cannot be blank.'],
            ['ba_address', 'string', 'min' => 2, 'max' => 255],

            ['ba_city', 'filter', 'filter' => 'trim'],
            ['ba_city', 'required', 'message' => 'Billing City cannot be blank.'],
            ['ba_city', 'string', 'min' => 2, 'max' => 255],

            ['ba_country', 'filter', 'filter' => 'trim'],
            ['ba_country', 'required', 'message' => 'Billing Country cannot be blank.'],
            ['ba_country', 'string', 'min' => 2, 'max' => 255],

            ['ba_zip_code', 'filter', 'filter' => 'trim'],

            ['sa_address', 'filter', 'filter' => 'trim'],
            ['sa_address', 'required', 'message' => 'Shipping Address cannot be blank.'],
            ['sa_address', 'string', 'min' => 2, 'max' => 255],

            ['sa_city', 'filter', 'filter' => 'trim'],
            ['sa_city', 'required', 'message' => 'Shipping City cannot be blank.'],
            ['sa_city', 'string', 'min' => 2, 'max' => 255],

            ['sa_country', 'filter', 'filter' => 'trim'],
            ['sa_country', 'required', 'message' => 'Shipping Country cannot be blank.'],
            ['sa_country', 'string', 'min' => 2, 'max' => 255],

            ['sa_zip_code', 'filter', 'filter' => 'trim'],

            ['phone', 'filter', 'filter' => 'trim'],

            ['newsletters', 'required'],
        ];
    }

    /**
     * Updates user profile.
     *
     * @return User::id|null the user-id or false if update fails
     */
    public function updateProfile()
    {
        $post_data = array();
        $modelUser = new User();

        $modelUser = User::find()->where(['email' => Yii::$app->request->post("FormProfile")['email']])->one();

        $user_id = User::find()->select('id')->where(['email' => Yii::$app->request->post("FormProfile")['email']])->asArray()->one()['id'];

        $post_data['User']['password_hash'] = isset($modelUser->password_hash) ? $modelUser->password_hash : null;
        $post_data['User']['UserMetadata'] = isset($modelUser->metadata) ? $modelUser->metadata : null;
        $post_data['User']['updated_by'] = $modelUser->id;

        $post_data['UserProfile']['first_name'] = isset(Yii::$app->request->post("FormProfile")['first_name']) ? Yii::$app->request->post("FormProfile")['first_name'] : null;
        $post_data['UserProfile']['last_name'] = isset(Yii::$app->request->post("FormProfile")['last_name']) ? Yii::$app->request->post("FormProfile")['last_name'] : null;
        $post_data['UserProfile']['gender'] = isset(Yii::$app->request->post("FormProfile")['gender']) ? Yii::$app->request->post("FormProfile")['gender'] : 'male';
        $post_data['UserProfile']['date_of_birth'] = isset(Yii::$app->request->post("FormProfile")['date_of_birth']) ? Yii::$app->request->post("FormProfile")['date_of_birth'] : null;
        $post_data['UserProfile']['ba_address'] = isset(Yii::$app->request->post("FormProfile")['ba_address']) ? Yii::$app->request->post("FormProfile")['ba_address'] : '--';
        $post_data['UserProfile']['ba_city'] = isset(Yii::$app->request->post("FormProfile")['ba_city']) ? Yii::$app->request->post("FormProfile")['ba_city'] : '--';
        $post_data['UserProfile']['ba_country'] = isset(Yii::$app->request->post("FormProfile")['ba_country']) ? Yii::$app->request->post("FormProfile")['ba_country'] : '--';
        $post_data['UserProfile']['ba_zip_code'] = isset(Yii::$app->request->post("FormProfile")['ba_zip_code']) ? Yii::$app->request->post("FormProfile")['ba_zip_code'] : '--';
        $post_data['UserProfile']['sa_address'] = isset($modelUser->userProfile->sa_address) ? $modelUser->userProfile->sa_address : '--';
        $post_data['UserProfile']['sa_city'] = isset($modelUser->userProfile->sa_city) ? $modelUser->userProfile->sa_city : '--';
        $post_data['UserProfile']['sa_country'] = isset($modelUser->userProfile->sa_country) ? $modelUser->userProfile->sa_country : '--';
        $post_data['UserProfile']['sa_zip_code'] = isset($modelUser->userProfile->sa_zip_code) ? $modelUser->userProfile->sa_zip_code : '--';
        $post_data['UserProfile']['phone'] = isset(Yii::$app->request->post("FormProfile")['phone']) ? Yii::$app->request->post("FormProfile")['phone'] : null;
        $post_data['UserProfile']['newsletters'] = isset(Yii::$app->request->post("FormProfile")['newsletters']) ? Yii::$app->request->post("FormProfile")['newsletters'] : 'no';
        $post_data['UserProfile']['updated_by'] = $modelUser->id;
        
        $user_id = $modelUser->saveUser($post_data, $modelUser->id);
        
        return $user_id ? $user_id : null;
    }
}
