<?php

namespace common\modules\user\models;

use Yii;
use yii\base\Model;
use common\modules\user\models\User;
use common\modules\user\models\UserRole;
use common\modules\core\models\CoreSite;

/**
 * Register form
 */
class FormRegister extends Model
{
    public $email;
    public $password_hash;
    public $confirm_password_hash;

    public $first_name;
    public $last_name;
    public $gender;
    public $date_of_birth;
    public $avatar;
    
    public $ba_address;
    public $ba_city;
    public $ba_country;
    public $ba_zip_code;

    public $sa_address;
    public $sa_city;
    public $sa_country;
    public $sa_zip_code;
    
    public $phone;
    public $mobile;

    public $newsletters;

    public $reCaptcha;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            ['email', 'trim'],
            ['email', 'required'],
            ['email', 'email'],
            ['email', 'string', 'max' => 255],
            ['email', 'unique', 'targetClass' => '\common\modules\user\models\User', 'message' => 'This email address has already been taken.'],

            ['password_hash', 'required'],
            ['password_hash', 'string', 'min' => 6],

            ['confirm_password_hash', 'required'],
            ['confirm_password_hash', 'compare', 'compareAttribute' => 'password_hash'],

            ['first_name', 'filter', 'filter' => 'trim'],
            ['first_name', 'required'],
            ['first_name', 'string', 'min' => 2, 'max' => 255],

            ['last_name', 'filter', 'filter' => 'trim'],
            ['last_name', 'required'],
            ['last_name', 'string', 'min' => 2, 'max' => 255],
			
			['ba_address', 'filter', 'filter' => 'trim'],
            ['ba_address', 'required', 'message' => 'Billing Address cannot be blank.'],
            ['ba_address', 'string', 'min' => 2, 'max' => 255],

            ['ba_city', 'filter', 'filter' => 'trim'],
            ['ba_city', 'required', 'message' => 'Billing City cannot be blank.'],
            ['ba_city', 'string', 'min' => 2, 'max' => 255],

            ['ba_country', 'filter', 'filter' => 'trim'],
            ['ba_country', 'required', 'message' => 'Billing Country cannot be blank.'],
            ['ba_country', 'string', 'min' => 2, 'max' => 255],

            ['ba_zip_code', 'filter', 'filter' => 'trim'],

            ['sa_address', 'filter', 'filter' => 'trim'],
            ['sa_address', 'required', 'message' => 'Shipping Address cannot be blank.'],
            ['sa_address', 'string', 'min' => 2, 'max' => 255],

            ['sa_city', 'filter', 'filter' => 'trim'],
            ['sa_city', 'required', 'message' => 'Shipping City cannot be blank.'],
            ['sa_city', 'string', 'min' => 2, 'max' => 255],

            ['sa_country', 'filter', 'filter' => 'trim'],
            ['sa_country', 'required', 'message' => 'Shipping Country cannot be blank.'],
            ['sa_country', 'string', 'min' => 2, 'max' => 255],

            ['sa_zip_code', 'filter', 'filter' => 'trim'],

            ['phone', 'filter', 'filter' => 'trim'],

            ['newsletters', 'required'],

            [['reCaptcha'], \himiklab\yii2\recaptcha\ReCaptchaValidator::className()],
        ];
    }

    /**
     * Signs user up.
     *
     * @return User|null the saved model or null if saving fails
     */
    public function signup()
    {
        if (!$this->validate()) {
            return null;
        }

        $post_data = array();
        $user = new User();
        $next_user_id = User::find()->select('id')->orderBy(['id' => SORT_DESC])->asArray()->one()['id'];
        
        $post_data['User']['email'] = isset(Yii::$app->request->post("FormRegister")['email']) ? Yii::$app->request->post("FormRegister")['email'] : null;
        $post_data['User']['password_hash'] = isset(Yii::$app->request->post("FormRegister")['password_hash']) ? Yii::$app->request->post("FormRegister")['password_hash'] : null;
        $post_data['User']['role_id'] = UserRole::getAttr(['id'], ['name' => 'user']);
        $post_data['User']['UserMetadata'] = isset(Yii::$app->request->post("FormRegister")['UserMetadata']) ? Yii::$app->request->post("FormRegister")['UserMetadata'] : null;
        $post_data['User']['created_by'] = $next_user_id;
        $post_data['User']['updated_by'] = $next_user_id;
        $post_data['User']['site_id'] = CoreSite::getAttr(['id'], ['code' => (isset(Yii::$app->params['site_code']) ? Yii::$app->params['site_code'] : 'genesys')]);

        $post_data['UserProfile']['first_name'] = isset(Yii::$app->request->post("FormRegister")['first_name']) ? Yii::$app->request->post("FormRegister")['first_name'] : null;
        $post_data['UserProfile']['last_name'] = isset(Yii::$app->request->post("FormRegister")['last_name']) ? Yii::$app->request->post("FormRegister")['last_name'] : null;
        $post_data['UserProfile']['gender'] = isset(Yii::$app->request->post("FormRegister")['gender']) ? Yii::$app->request->post("FormRegister")['gender'] : 'male';
        $post_data['UserProfile']['date_of_birth'] = isset(Yii::$app->request->post("FormRegister")['date_of_birth']) ? Yii::$app->request->post("FormRegister")['date_of_birth'] : null;
        $post_data['UserProfile']['ba_address'] = isset(Yii::$app->request->post("FormRegister")['ba_address']) ? Yii::$app->request->post("FormRegister")['ba_address'] : null;
        $post_data['UserProfile']['ba_city'] = isset(Yii::$app->request->post("FormRegister")['ba_city']) ? Yii::$app->request->post("FormRegister")['ba_city'] : null;
        $post_data['UserProfile']['ba_country'] = isset(Yii::$app->request->post("FormRegister")['ba_country']) ? Yii::$app->request->post("FormRegister")['ba_country'] : null;
        $post_data['UserProfile']['ba_zip_code'] = isset(Yii::$app->request->post("FormRegister")['ba_zip_code']) ? Yii::$app->request->post("FormRegister")['ba_zip_code'] : null;
        $post_data['UserProfile']['sa_address'] = isset(Yii::$app->request->post("FormRegister")['sa_address']) ? Yii::$app->request->post("FormRegister")['sa_address'] : null;
        $post_data['UserProfile']['sa_city'] = isset(Yii::$app->request->post("FormRegister")['sa_city']) ? Yii::$app->request->post("FormRegister")['sa_city'] : null;
        $post_data['UserProfile']['sa_country'] = isset(Yii::$app->request->post("FormRegister")['sa_country']) ? Yii::$app->request->post("FormRegister")['sa_country'] : null;
        $post_data['UserProfile']['sa_zip_code'] = isset(Yii::$app->request->post("FormRegister")['sa_zip_code']) ? Yii::$app->request->post("FormRegister")['sa_zip_code'] : null;
        $post_data['UserProfile']['phone'] = isset(Yii::$app->request->post("FormRegister")['phone']) ? Yii::$app->request->post("FormRegister")['phone'] : null;
        $post_data['UserProfile']['newsletters'] = isset(Yii::$app->request->post("FormRegister")['newsletters']) ? Yii::$app->request->post("FormRegister")['newsletters'] : 'no';
        $post_data['UserProfile']['created_by'] = $next_user_id;
        $post_data['UserProfile']['updated_by'] = $next_user_id;
        
        $user_id = $user->saveUser($post_data);
        
        return $user_id ? UserIdentity::findIdentity($user_id) : null;
    }
}
