<?php

namespace common\modules\user\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%user_logs_login}}".
 *
 * @property int $id
 * @property string $user
 * @property string $app_name
 * @property string $success
 * @property int $site_id
 * @property string $ip_address
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 *
 * @property User $createdBy
 * @property User $updatedBy
 * @property CoreSite $site
 */
class UserLogsLogin extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%user_logs_login}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user'], 'required'],
            [['success'], 'string'],
            [['site_id', 'created_by', 'updated_by'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
            [['user', 'app_name', 'ip_address'], 'string', 'max' => 255],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            [['site_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\core\models\CoreSite::className(), 'targetAttribute' => ['site_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user' => 'User',
            'app_name' => 'App Name',
            'success' => 'Success',
            'site_id' => 'Site ID',
            'ip_address' => 'Ip Address',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSite()
    {
        return $this->hasOne(\common\modules\core\models\CoreSite::className(), ['id' => 'site_id']);
    }

    public function behaviors()
    {
        return [
            [
                'class' => \yii\behaviors\TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }

    public function saveLogsLogin($user, $success, $app_name, $site_id = '')
    {
        $modelUserLogsLogin = new self();

        $modelUserLogsLogin->user = $user;
        $modelUserLogsLogin->app_name = $app_name;
        $modelUserLogsLogin->success = $success;
        $modelUserLogsLogin->site_id = $site_id;
        $modelUserLogsLogin->ip_address = Yii::$app->request->getUserIP();
        $modelUserLogsLogin->created_by = isset(Yii::$app->user->identity->id) ? Yii::$app->user->identity->id : 1;
        $modelUserLogsLogin->updated_by = isset(Yii::$app->user->identity->id) ? Yii::$app->user->identity->id : 1;

        if ($modelUserLogsLogin->validate()) {
            $modelUserLogsLogin->save();
        }
    }
}