<?php

namespace common\modules\user\models;

use Yii;
use yii\db\Expression;

use common\modules\core\components\Globals;

/**
 * This is the model class for table "{{%user_profile}}".
 *
 * @property int $id
 * @property int $user_id
 * @property string $first_name
 * @property string $last_name
 * @property string $gender
 * @property string $date_of_birth
 * @property int $avatar
 * @property string $ba_address
 * @property string $ba_city
 * @property string $ba_country
 * @property string $ba_zip_code
 * @property string $sa_address
 * @property string $sa_city
 * @property string $sa_country
 * @property string $sa_zip_code
 * @property string $phone
 * @property string $newsletters
 * @property string $metadata
 * @property int $created_by
 * @property int $updated_by
 * @property string $created_at
 * @property string $updated_at
 * @property string $status
 *
 * @property User $user
 * @property AssetManager $avatar0
 * @property User $createdBy
 * @property User $updatedBy
 */
class UserProfile extends \common\modules\core\models\GenesysActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%user_profile}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['user_id', 'first_name', 'last_name', 'gender', 'newsletters'], 'required'],
            [['user_id', 'avatar', 'created_by', 'updated_by'], 'integer'],
            [['gender', 'newsletters', 'metadata', 'status'], 'string'],
            [['date_of_birth'], 'safe'],
            [['first_name', 'last_name', 'ba_address', 'ba_city', 'ba_country', 'ba_zip_code', 'sa_address', 'sa_city', 'sa_country', 'sa_zip_code', 'phone'], 'string', 'max' => 255],
            [['user_id'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['user_id' => 'id']],
            [['avatar'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\assetManager\models\AssetManager::className(), 'targetAttribute' => ['avatar' => 'id']],
            [['created_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['created_by' => 'id']],
            [['updated_by'], 'exist', 'skipOnError' => true, 'targetClass' => \common\modules\user\models\User::className(), 'targetAttribute' => ['updated_by' => 'id']],
            ['ba_address', 'filter', 'filter' => 'trim'],
            ['ba_address', 'required', 'message' => 'Billing Address cannot be blank.'],
            ['ba_address', 'string', 'min' => 2, 'max' => 255],

            ['ba_city', 'filter', 'filter' => 'trim'],
            ['ba_city', 'required', 'message' => 'Billing City cannot be blank.'],
            ['ba_city', 'string', 'min' => 2, 'max' => 255],

            ['ba_country', 'filter', 'filter' => 'trim'],
            ['ba_country', 'required', 'message' => 'Billing Country cannot be blank.'],
            ['ba_country', 'string', 'min' => 2, 'max' => 255],

            ['ba_zip_code', 'filter', 'filter' => 'trim'],

            ['sa_address', 'filter', 'filter' => 'trim'],
            ['sa_address', 'required', 'message' => 'Shipping Address cannot be blank.'],
            ['sa_address', 'string', 'min' => 2, 'max' => 255],

            ['sa_city', 'filter', 'filter' => 'trim'],
            ['sa_city', 'required', 'message' => 'Shipping City cannot be blank.'],
            ['sa_city', 'string', 'min' => 2, 'max' => 255],

            ['sa_country', 'filter', 'filter' => 'trim'],
            ['sa_country', 'required', 'message' => 'Shipping Country cannot be blank.'],
            ['sa_country', 'string', 'min' => 2, 'max' => 255],

            ['sa_zip_code', 'filter', 'filter' => 'trim'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'user_id' => 'User ID',
            'first_name' => 'First Name',
            'last_name' => 'Last Name',
            'gender' => 'Gender',
            'date_of_birth' => 'Date Of Birth',
            'avatar' => 'Avatar',
            'ba_address' => 'Ba Address',
            'ba_city' => 'Ba City',
            'ba_country' => 'Ba Country',
            'ba_zip_code' => 'Ba Zip Code',
            'sa_address' => 'Sa Address',
            'sa_city' => 'Sa City',
            'sa_country' => 'Sa Country',
            'sa_zip_code' => 'Sa Zip Code',
            'phone' => 'Phone',
            'newsletters' => 'Newsletters',
            'metadata' => 'Metadata',
            'created_by' => 'Created By',
            'updated_by' => 'Updated By',
            'created_at' => 'Created At',
            'updated_at' => 'Updated At',
            'status' => 'Status',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUser()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'user_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAvatar0()
    {
        return $this->hasOne(\common\modules\assetManager\models\AssetManager::className(), ['id' => 'avatar']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getCreatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'created_by']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUpdatedBy()
    {
        return $this->hasOne(\common\modules\user\models\User::className(), ['id' => 'updated_by']);
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            [
                'class' => \yii\behaviors\TimestampBehavior::className(),
                'createdAtAttribute' => 'created_at',
                'updatedAtAttribute' => 'updated_at',
                'value' => new Expression('NOW()'),
            ],
        ];
    }

    public static function getAllUsernames()
    {
        $result = array();
        $model = self::findAll(['status' => 'active']);

        foreach ($model as $modelKey => $modelValue) {
            if (isset($modelValue->user_id) && (isset($modelValue->first_name)) && (isset($modelValue->last_name))) {
                $result[$modelValue->user_id] = $modelValue->first_name . ' ' . $modelValue->last_name;
            }
        }

        return $result;
    }
}
