<?php

namespace common\modules\user\models;

use Yii;

/**
 * Our data model extends yii\base\Model class so we can get easy to use and yet 
 * powerful Yii 2 validation mechanism.
 */
class UserSearch extends \yii\base\Model
{
    /**
     * We plan to get two columns in our grid that can be filtered.
     * Add more if required. You don't have to add all of them.
     */
    public $id;
    public $email;
    public $auth_key;
    public $access_token;
    public $role_id;
    public $site_id;
    public $metadata_product;
    public $metadata_organization;
    public $metadata_mailer_campaign;
    public $metadata_mailer_campaign_identifier_name;
    public $metadata_mailer_campaign_timestamp;
    public $created_by;
    public $updated_by;
    public $created_at;
    public $updated_at;
    public $first_name;
    public $last_name;
    public $gender;
    public $date_of_birth;
    public $ba_address;
    public $ba_city;
    public $ba_country;
    public $ba_zip_code;
    public $sa_address;
    public $sa_city;
    public $sa_country;
    public $sa_zip_code;
    public $phone;
    public $newsletters;
    public $status;

    /**
     * Here we can define validation rules for each filtered column.
     * See http://www.yiiframework.com/doc-2.0/guide-input-validation.html
     * for more information about validation.
     */
    public function rules()
    {
        return [
            [['id', 'email', 'auth_key', 'access_token', 'role_id', 'site_id', 'metadata_product', 'metadata_organization', 'metadata_mailer_campaign', 'metadata_mailer_campaign_identifier_name', 'metadata_mailer_campaign_timestamp', 'created_by', 'updated_by', 'created_at', 'updated_at', 'status', 'first_name', 'last_name', 'gender', 'date_of_birth', 'ba_address', 'ba_city', 'ba_country', 'ba_zip_code', 'sa_address', 'sa_city', 'sa_country', 'sa_zip_code', 'phone', 'newsletters'], 'string'],
        ];
    }

    /**
     * In this example we keep this special property to know if columns should be 
     * filtered or not. See search() method below.
     */
    private $_filtered = false;

    /**
     * This method returns ArrayDataProvider.
     * Filtered and sorted if required.
     */
    public function search($params)
    {
        /**
         * $params is the array of GET parameters passed in the actionExample().
         * These are being loaded and validated.
         * If validation is successful _filtered property is set to true to prepare
         * data source. If not - data source is displayed without any filtering.
         */
        if ($this->load($params) && $this->validate()) {
            $this->_filtered = true;
        }

        return new \yii\data\ArrayDataProvider([
            // ArrayDataProvider here takes the actual data source
            'allModels' => $this->getData(),
            'key' => 'id',
            'sort' => [
                // we want our columns to be sortable:
                'attributes' => ['id', 'email', 'auth_key', 'access_token', 'role_id', 'site_id', 'metadata_product', 'metadata_organization', 'metadata_mailer_campaign', 'metadata_mailer_campaign_identifier_name', 'metadata_mailer_campaign_timestamp', 'created_by', 'updated_by', 'created_at', 'updated_at', 'status', 'first_name', 'last_name', 'gender', 'date_of_birth', 'ba_address', 'ba_city', 'ba_country', 'ba_zip_code', 'sa_address', 'sa_city', 'sa_country', 'sa_zip_code', 'phone', 'newsletters'],
                'defaultOrder' => ['id' => SORT_DESC],
            ],
        ]);
    }

    /**
     * Here we are preparing the data source and applying the filters
     * if _filtered property is set to true.
     */
    protected function getData()
    {
        $data = User::find()->with('userProfile')->where(['status' => 'active'])->asArray()->all();

        foreach ($data as $k => $v) {
            $user_profile = $data[$k]['userProfile'];

            unset($data[$k]['userProfile']);

            $data[$k] = $data[$k] + $user_profile;

            $json_arr = json_decode($v['metadata'], true);

            foreach (Yii::$app->params['user_metadata'] as $key => $value) {
                $data[$k]['metadata_' . $key] = null;
            }

            if (!is_array($json_arr)) {
                continue;
            }

            foreach ($json_arr as $_k => $_v) {
                $data[$k]['metadata_' . $_k] = $_v;
            }
        }

        if ($this->_filtered) {
            $data = array_filter($data, function ($value) {
                $conditions = [true];

                if (!empty($this->id)) {
                    $conditions[] = stripos($value['id'], $this->id) !== false;
                }

                if (!empty($this->email)) {
                    $conditions[] = stripos($value['email'], $this->email) !== false;
                }

                if (!empty($this->auth_key)) {
                    $conditions[] = stripos($value['auth_key'], $this->auth_key) !== false;
                }

                if (!empty($this->access_token)) {
                    $conditions[] = stripos($value['access_token'], $this->access_token) !== false;
                }

                if (!empty($this->role_id)) {
                    $conditions[] = stripos(\common\modules\user\models\UserRole::getAttr(['name'], ['id' => $value['role_id']]), \common\modules\user\models\UserRole::getAttr(['name'], ['id' => $this->role_id])) !== false;
                }

                if (!empty($this->site_id)) {
                    $conditions[] = stripos(\common\modules\core\models\CoreSite::getAttr(['name'], ['id' => $value['site_id']]), \common\modules\core\models\CoreSite::getAttr(['name'], ['id' => $this->site_id])) !== false;
                }

                if (!empty($this->metadata_product)) {
                    $conditions[] = stripos($value['metadata_product'], $this->metadata_product) !== false;
                }

                if (!empty($this->metadata_organization)) {
                    $conditions[] = stripos($value['metadata_organization'], $this->metadata_organization) !== false;
                }

                if (!empty($this->metadata_mailer_campaign)) {
                    $conditions[] = stripos($value['metadata_mailer_campaign'], $this->metadata_mailer_campaign) !== false;
                }

                if (!empty($this->metadata_mailer_campaign_identifier_name)) {
                    $conditions[] = stripos($value['metadata_mailer_campaign_identifier_name'], $this->metadata_mailer_campaign_identifier_name) !== false;
                }

                if (!empty($this->metadata_mailer_campaign_timestamp)) {
                    $conditions[] = stripos($value['metadata_mailer_campaign_timestamp'], $this->metadata_mailer_campaign_timestamp) !== false;
                }

                if (!empty($this->created_by)) {
                    $conditions[] = stripos(\common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $value['created_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $value['created_by']]), $this->created_by) !== false;
                }

                if (!empty($this->updated_by)) {
                    $conditions[] = stripos(\common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $value['updated_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $value['updated_by']]), $this->updated_by) !== false;
                }

                if (!empty($this->created_at)) {
                    $conditions[] = stripos($value['created_at'], $this->created_at) !== false;
                }

                if (!empty($this->updated_at)) {
                    $conditions[] = stripos($value['updated_at'], $this->updated_at) !== false;
                }

                if (!empty($this->first_name)) {
                    $conditions[] = stripos($value['first_name'], $this->first_name) !== false;
                }

                if (!empty($this->last_name)) {
                    $conditions[] = stripos($value['last_name'], $this->last_name) !== false;
                }

                if (!empty($this->gender)) {
                    $conditions[] = stripos($value['gender'], $this->gender) !== false;
                }

                if (!empty($this->date_of_birth)) {
                    $conditions[] = stripos($value['date_of_birth'], $this->date_of_birth) !== false;
                }

                if (!empty($this->ba_address)) {
                    $conditions[] = stripos($value['ba_address'], $this->ba_address) !== false;
                }

                if (!empty($this->ba_city)) {
                    $conditions[] = stripos($value['ba_city'], $this->ba_city) !== false;
                }

                if (!empty($this->ba_country)) {
                    $conditions[] = stripos($value['ba_country'], $this->ba_country) !== false;
                }

                if (!empty($this->ba_zip_code)) {
                    $conditions[] = stripos($value['ba_zip_code'], $this->ba_zip_code) !== false;
                }

                if (!empty($this->sa_address)) {
                    $conditions[] = stripos($value['sa_address'], $this->sa_address) !== false;
                }

                if (!empty($this->sa_city)) {
                    $conditions[] = stripos($value['sa_city'], $this->sa_city) !== false;
                }

                if (!empty($this->sa_country)) {
                    $conditions[] = stripos($value['sa_country'], $this->sa_country) !== false;
                }

                if (!empty($this->sa_zip_code)) {
                    $conditions[] = stripos($value['sa_zip_code'], $this->sa_zip_code) !== false;
                }

                if (!empty($this->phone)) {
                    $conditions[] = stripos($value['phone'], $this->phone) !== false;
                }

                if (!empty($this->newsletters)) {
                    $conditions[] = stripos($value['newsletters'], $this->newsletters) !== false;
                }

                return array_product($conditions);
            });
        }

        return $data;
    }
}
