<?php

use yii\bootstrap4\Html;
use kartik\grid\GridView;
use kartik\dynagrid\DynaGrid;

use common\modules\core\components\Globals;
use common\modules\core\models\CoreSite;
use common\modules\user\models\UserProfile;
use common\modules\user\models\UserRole;
/* @var $this yii\web\View */
/* @var $searchModel common\modules\user\models\UserSearch */
/* @var $dataProvider yii\data\ActiveDataProvider */

$this->title = 'Users';
$this->params['breadcrumbs'][] = $this->title;

$columns = [
    ['class' => 'kartik\grid\SerialColumn', 'order' => DynaGrid::ORDER_FIX_LEFT],

    [
        'attribute' => 'id',
        'label' => 'User ID',
        'value' => 'id'
    ],
    'email:email',
    [
        'attribute' => 'first_name', 
        'value' => function ($model, $key, $index, $widget) {
            return UserProfile::getAttr(['first_name'], ['user_id' => $model['id']]);
        },
    ],
    [
        'attribute' => 'last_name', 
        'value' => function ($model, $key, $index, $widget) {
            return UserProfile::getAttr(['last_name'], ['user_id' => $model['id']]);
        },
    ],
    [
        'attribute' => 'gender',
        'value' => function ($model, $key, $index, $widget) {
            return ucfirst($model['gender']);
        },
        'filter' => ['male' => 'Male', 'female' => 'Female'],
        'visible' => false
    ],
    [
        'attribute' => 'date_of_birth', 
        'visible' => false
    ],
    // 'password_hash',
    [
        'attribute' => 'auth_key', 
        'visible' => false
    ],
    // 'password_reset_token',
    [
        'attribute' => 'access_token', 
        'visible' => false
    ],
    [
        'attribute' => 'role_id',
        'label' => 'Role',
        'value' => function ($model, $key, $index, $widget) {
            return UserRole::getAttr(['name'], ['id' => $model['role_id']]);
        },
        'filter' => UserRole::getAll($k = 'id', $v = 'name'),
    ],
    [
        'attribute' => 'site_id',
        'label' => 'Site',
        'value' => function ($model, $key, $index, $widget) {
            return \common\modules\core\models\CoreSite::getAttr(['name'], ['id' => $model['site_id']]);
        },
        'filter' => \common\modules\core\models\CoreSite::getAll($k = 'id', $v = 'name'),
        'visible' => false
    ],
    [
        'attribute' => 'ba_address', 
        'label' => 'Billing Address', 
        'value' => 'ba_address', 
        'visible' => false
    ],
    [
        'attribute' => 'ba_city', 
        'label' => 'Billing City', 
        'value' => 'ba_city', 
        'visible' => false
    ],
    [
        'attribute' => 'ba_country', 
        'label' => 'Billing Country', 
        'value' => 'ba_country', 
        'visible' => false
    ],
    [
        'attribute' => 'ba_zip_code', 
        'label' => 'Billing Zip Code', 
        'value' => 'ba_zip_code', 
        'visible' => false
    ],
    [
        'attribute' => 'sa_address', 
        'label' => 'Shipping Address', 
        'value' => 'sa_address', 
        'visible' => false
    ],
    [
        'attribute' => 'sa_city', 
        'label' => 'Shipping City', 
        'value' => 'sa_city', 
        'visible' => false
    ],
    [
        'attribute' => 'sa_country', 
        'label' => 'Shipping Country', 
        'value' => 'sa_country', 
        'visible' => false
    ],
    [
        'attribute' => 'sa_zip_code', 
        'label' => 'Shipping Zip Code', 
        'value' => 'sa_zip_code', 
        'visible' => false
    ],
    [
        'attribute' => 'phone', 
        'label' => 'Phone', 
        'value' => 'phone', 
        'visible' => false
    ],
    [
        'attribute' => 'newsletters',
        'value' => function ($model, $key, $index, $widget) {
            return ucfirst($model['newsletters']);
        },
        'filter' => ['yes' => 'Yes', 'no' => 'No'],
        'visible' => false
    ],
    // 'metadata:ntext',
    [
        'attribute' => 'created_by',
        'value' => function ($model, $key, $index, $widget) {
            return \common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $model['created_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $model['created_by']]);
        },
        'visible' => false
    ],
    [
        'attribute' => 'updated_by',
        'value' => function ($model, $key, $index, $widget) {
            return \common\modules\user\models\UserProfile::getAttr(['first_name'], ['user_id' => $model['updated_by']]) . ' ' . \common\modules\user\models\UserProfile::getAttr(['last_name'], ['user_id' => $model['updated_by']]);
        },
        'visible' => false
    ],
    'created_at',
    'updated_at',

    ['class' => 'kartik\grid\ActionColumn', 'order' => DynaGrid::ORDER_FIX_RIGHT],
];

$columns = array_merge($columns, \common\modules\core\components\Globals::dynagridMetadataColumns('user_metadata', []));
?>

<div class="container-fluid user-index">
    <div class="row">
        <div class="col-md-12">
            <div class="card">
                <div class="card-body">
                    <div class="row mb-2">
                        <div class="col-md-12">
                            <?= Html::a('Create User', ['create'], ['class' => 'btn btn-success']) ?>
                        </div>
                    </div>

                    <?= DynaGrid::widget([
                        'columns' => $columns,
                        'storage' => DynaGrid::TYPE_SESSION,
                        'theme' => 'simple-condensed',
                        'allowPageSetting' => true,
                        'allowThemeSetting' => false,
                        'allowFilterSetting' => false,
                        'allowSortSetting' => false,
                        'gridOptions' => [
                            'dataProvider' => $dataProvider,
                            'filterModel' => $searchModel,
                            'showPageSummary' => false,
                            'floatHeader' => false,
                            'pjax' => true,
                            'responsiveWrap' => true,
                            'panel' => [
                                'after' => false
                            ],
                            'toolbar' =>  [
                                ['content' => 
                                    Html::a('<i class="fas fa-plus"></i>', ['create'], [
                                        'type' => 'button', 
                                        'title' => 'Add User', 
                                        'class' => 'btn btn-success', 
                                        'data-pjax' => 0, 
                                    ]) . 
                                    Html::a('<i class="fas fa-redo"></i>', ['index'], [
                                        'type' => 'button', 
                                        'title' => 'Reset Grid', 
                                        'class' => 'btn btn-outline-secondary', 
                                        'data-pjax' => 0, 
                                    ]) . 
                                    '{dynagrid}' . 
                                    '{export}'
                                ],
                            ]
                        ],
                        'options' => ['id' => 'dynagrid-mailer-user']
                    ]);
                    ?>
                </div>
                <!--.card-body-->
            </div>
            <!--.card-->
        </div>
        <!--.col-md-12-->
    </div>
    <!--.row-->
</div>
