<?php

use yii\bootstrap4\Html;
use yii\helpers\Inflector;

use common\modules\core\models\CoreLanguage;
use common\modules\core\models\CoreEntityAttribute;
use common\modules\core\models\CoreValue;
use common\modules\user\models\UserProfile;

/* @var $this yii\web\View */
/* @var $model common\modules\user\models\User */

$entity_value = CoreValue::getEntityValue('user', 'name', $model->id);

if (isset($entity_value) && empty($entity_value)) {
    unset($entity_value);
}

$this->title = 'View User: ' . (isset($entity_value) ? $entity_value : $model->id);
$this->params['breadcrumbs'][] = ['label' => 'Users', 'url' => ['index']];
$this->params['breadcrumbs'][] = (isset($entity_value) ? $entity_value : $model->id);
?>

<div class="container-fluid user-view">
    <div class="card">
        <div class="card-body">
            <div class="row">
                <div class="col-md-12">
                    <p>
                        <?= Html::a('Update', ['update', 'id' => $model->id], ['class' => 'btn btn-primary']) ?>
                        <?= Html::a('Delete', ['delete', 'id' => $model->id], [
                            'class' => 'btn btn-danger',
                            'data' => [
                                'confirm' => 'Are you sure you want to delete this item?',
                                'method' => 'post',
                            ],
                        ]) ?>
                    </p>

                    <?php
                    $attributes = array();

                    $attributes[] = ['attribute' => 'id', 'label' => 'User ID'];
                    $attributes[] = 'email:email';
                    $attributes[] = 'userProfile.first_name';
                    $attributes[] = 'userProfile.last_name';
                    $attributes[] = 'userProfile.gender';
                    $attributes[] = 'userProfile.date_of_birth';
                    $attributes[] = 'password_hash';
                    $attributes[] = 'auth_key';
                    $attributes[] = 'password_reset_token';
                    $attributes[] = 'access_token';
                    $attributes[] = ['label' => 'Role', 'attribute' => 'role.name'];
                    $attributes[] = ['attribute' => 'site.name', 'label' => 'Site'];
                    $attributes[] = ['label' => 'Billing Address', 'attribute' => 'userProfile.ba_address'];
                    $attributes[] = ['label' => 'Billing City', 'attribute' => 'userProfile.ba_city'];
                    $attributes[] = ['label' => 'Billing Country', 'attribute' => 'userProfile.ba_country'];
                    $attributes[] = ['label' => 'Billing Zip Code', 'attribute' => 'userProfile.ba_zip_code'];
                    $attributes[] = ['label' => 'Shipping Address', 'attribute' => 'userProfile.sa_address'];
                    $attributes[] = ['label' => 'Shipping City', 'attribute' => 'userProfile.sa_city'];
                    $attributes[] = ['label' => 'Shipping Country', 'attribute' => 'userProfile.sa_country'];
                    $attributes[] = ['label' => 'Shipping Zip Code', 'attribute' => 'userProfile.sa_zip_code'];
                    $attributes[] = 'userProfile.phone';
                    $attributes[] = 'userProfile.newsletters';
                    $attributes[] = 'metadata:ntext';
                    $attributes[] = [
                        'attribute' => 'created_by',
                        'label' => 'Created By',
                        'value' => UserProfile::getAttr(['first_name'], ['user_id' => $model->created_by]) . ' ' . UserProfile::getAttr(['last_name'], ['user_id' => $model->created_by])
                    ];
                    $attributes[] = [
                        'attribute' => 'updated_by',
                        'label' => 'Updated By',
                        'value' => UserProfile::getAttr(['first_name'], ['user_id' => $model->updated_by]) . ' ' . UserProfile::getAttr(['last_name'], ['user_id' => $model->updated_by])
                    ];
                    $attributes[] = 'created_at';
                    $attributes[] = 'updated_at';

                    foreach (CoreEntityAttribute::getAttributeClassesOfEntityClass('user') as $attributeClassKey => $attributeClassValue) {
                        foreach (CoreLanguage::find()->where(['status' => 'active'])->all() as $languageKey => $languageValue) {
                            if (Yii::$app->params['attributeFormFields'][$attributeClassValue] == 'wysiwyg') {
                                $attributes[] = [
                                    'attribute' => $attributeClassValue . '_' . $languageValue->code,
                                    'label' => Inflector::titleize(ucfirst($attributeClassValue), true) . ' (' . strtoupper($languageValue->code) . ')',
                                    'value' => CoreValue::getEntityValue('user', $attributeClassValue, $model->id, $languageValue->id),
                                    'format' => 'html',
                                ];
                            } else {
                                $attributes[] = [
                                    'attribute' => $attributeClassValue . '_' . $languageValue->code,
                                    'label' => Inflector::titleize(ucfirst($attributeClassValue), true) . ' (' . strtoupper($languageValue->code) . ')',
                                    'value' => CoreValue::getEntityValue('user', $attributeClassValue, $model->id, $languageValue->id)
                                ];
                            }
                        }
                    }
                    ?>

                    <?= \yii\widgets\DetailView::widget([
                        'model' => $model,
                        'attributes' => $attributes,
                    ]) ?>
                </div>
                <!--.col-md-12-->
            </div>
            <!--.row-->
        </div>
        <!--.card-body-->
    </div>
    <!--.card-->
</div>