<?php 

use yii\helpers\Inflector;
use yii\bootstrap4\Html;

use common\modules\core\components\Globals;

$model_name = \yii\helpers\StringHelper::basename(get_class($model));

if (isset(Yii::$app->params[$metadata]) && !empty(Yii::$app->params[$metadata])) {
    $tmpData = '';
    $modelMetadata = array();
    
    if (!empty($model->metadata)) {
        $modelMetadata = Globals::getArray($model->metadata);           
    }
?>
    <?php
    foreach (Yii::$app->params[$metadata] as $field => $detailsArr) {
        $is_true = false;
        
        if (array_key_exists('type', $detailsArr)) {
            if ($detailsArr['mode'] == 'required') {
                $is_true = true;
            }
    ?>
            <div class="form-group">
                <label for="<?= strtolower($model_name) ?>-<?= $field ?>"><?= Inflector::titleize(ucfirst($field), true) ?></label>

                <br>

                <?php 
                $tmpData = isset($modelMetadata[$field]) ? $modelMetadata[$field] : '';

                switch ($detailsArr['type']) {
                    case 'input_text':
                        echo Html::input('text', $model_name . '[' . $model_name . 'Metadata][' . $field . ']', isset($tmpData) ? $tmpData : '', ['class' => 'form-control', 'required' => $is_true]);
                        
                        break;

                    case 'textarea':
                        echo Html::textarea($model_name . '[' . $model_name . 'Metadata][' . $field . ']', isset($tmpData) ? $tmpData : '', ['class' => 'form-control', 'rows' => 6, 'required' => $is_true]);

                        break;

                    case 'wysiwyg':
                        echo \dosamigos\tinymce\TinyMce::widget([
                            'name' => $model_name . '[' . $model_name . 'Metadata][' . $field . ']',
                            'value' => (isset($tmpData) && !empty($tmpData)) ? $tmpData : '',
                            'options' => ['rows' => 3],
                            'language' => 'en_GB',
                            'clientOptions' => [
                                'plugins' => [
                                    "advlist autolink lists link charmap print preview anchor hr",
                                    "searchreplace visualblocks code fullscreen",
                                    "insertdatetime media table contextmenu paste"
                                ],
                                'toolbar' => "undo redo | styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image"
                            ]
                        ]);
                        
                        break;

                    case 'select':
                        echo Html::dropDownList($model_name . '[' . $model_name . 'Metadata][' . $field . ']', (isset($tmpData) && !empty($tmpData)) ? $tmpData : [], $detailsArr['data'], ['prompt' => 'Select ' . Inflector::titleize(ucfirst($field), true), 'class' => 'form-control']);

                        break;

                    case 'select2':
                        echo \kartik\select2\Select2::widget([
                            'name' => $model_name . '[' . $model_name . 'Metadata][' . $field . ']',
                            'value' => (isset($tmpData) && !empty($tmpData)) ? $tmpData : [],   // initial value
                            'data' => $detailsArr['data'],
                            'theme' => \kartik\select2\Select2::THEME_BOOTSTRAP,
                            'maintainOrder' => true,
                            'hideSearch' => false,
                            'toggleAllSettings' => [
                                'selectLabel' => '<i class="glyphicon glyphicon-ok-circle"></i> Select All',
                                'unselectLabel' => '<i class="glyphicon glyphicon-remove-circle"></i> Deselect All',
                                'selectOptions' => ['class' => 'text-success'],
                                'unselectOptions' => ['class' => 'text-danger'],
                            ],
                            'options' => ['placeholder' => 'Select options', 'multiple' => isset($detailsArr['multiple']) ? $detailsArr['multiple'] : false, 'class'=> 'traits-options-select'],
                            'pluginOptions' => [
                                'tags' => true,
                                'maximumInputLength' => 100,
                                'allowClear' => true,
                            ],
                        ]);
                        
                        break;

                    case 'date':
                        echo \kartik\date\DatePicker::widget([
                            'name' => $model_name . '[' . $model_name . 'Metadata][' . $field . ']',
                            'value' => (isset($tmpData) && !empty($tmpData)) ? $tmpData : null,
                            // 'type' => DatePicker::TYPE_COMPONENT_APPEND,
                            'pluginOptions' => [
                                'autoclose' => true,
                                'format' => 'yyyy-mm-dd'
                            ]
                        ]);

                        break;

                    case 'datetime':
                        echo kartik\datetime\DateTimePicker::widget([
                            'name' => $model_name . '[' . $model_name . 'Metadata][' . $field . ']',
                            'value' => (isset($tmpData) && !empty($tmpData)) ? $tmpData : null,
                            'pluginOptions' => [
                                'autoclose' => true,
                                'format' => 'yyyy-mm-dd hh:ii:ss',
                            ]
                        ]);

                        break;

                    case 'json':
                        for ($i = 0; $i < 33; $i++) {
                            echo Html::label('Question #' . intval($i + 1), null, ['style' => 'font-size: 0.7em;']);

                            echo Html::textarea($model_name . '[' . $model_name . 'Metadata][' . $field . '][' . $i . ']', isset($tmpData[$i]) ? json_encode($tmpData[$i], JSON_PRETTY_PRINT) : '', ['class' => 'form-control', 'rows' => 6]);

                            echo '</div>';

                            if ($i < 33 - 1) {
                                echo '<div class="form-group">';
                            }
                        }

                        break;
                    
                    default:
                        echo Html::input('text', $model_name . '[' . $model_name . 'Metadata][' . $field . ']', isset($tmpData) ? $tmpData : '', ['class' => 'form-control']);
                        
                        break;
                }
                ?>
            </div>
        <?php
        }
    }
}

?>
