<?php

namespace common\widgets;

use Yii;
use yii\helpers\Html;
use yii\helpers\ArrayHelper;
use yii\helpers\Url;
use yii\helpers\Json;
use yii\db\Query;
use webzop\notifications\NotificationsAsset;

use common\modules\site\models\SiteNotification;

class NotificationsNavbar extends \yii\base\Widget
{

    public $maxNotifications = 7;
    public $userId;

    protected $modelSiteNotification;

    public function init()
    {
        parent::init();

        $this->modelSiteNotification = new SiteNotification();

        $this->modelSiteNotification = SiteNotification::find()->where(['user_id' => $this->userId, 'is_read' => 'no'])->orderBy(['updated_at' => SORT_DESC])->all();
    }

    /**
     * @inheritdoc
     */
    public function run()
    {
        $counter = 1;

        echo '
        <li class="nav-item dropdown">
            <a class="nav-link" data-toggle="dropdown" href="#">
                <i class="far fa-bell"></i>
                <span class="badge badge-warning navbar-badge">' . self::getUnreadCount() . '</span>
            </a>
            <div class="dropdown-menu dropdown-menu-lg dropdown-menu-right">
                <span class="dropdown-header">' . self::getUnreadCount() . ' Notification(s)</span>
                <div class="dropdown-divider"></div>
        ';

        foreach ($this->modelSiteNotification as $key => $value) {
            $counter++;

            $route = unserialize($value->route);

            if ($counter > $this->maxNotifications) {
                break;
            }

            $timeOrigin = new \DateTime($value->updated_at);
            $timeTarget = new \DateTime();
            $interval = $timeTarget->diff($timeOrigin);

            $interval_y = $interval->format('%Y');
            $interval_m = $interval->format('%m');
            $interval_d = $interval->format('%d');
            $interval_h = $interval->format('%H');
            $interval_i = $interval->format('%i');
            $interval_s = $interval->format('%s');

            if (!empty($interval_y) && intval($interval_y) > 0) {
                $interval = $interval_y . ((intval($interval_y) > 1) ? ' years' : ' year');
            } else if (!empty($interval_m) && intval($interval_m) > 0) {
                $interval = $interval_m . ((intval($interval_m) > 1) ? ' months' : ' month');
            } else if (!empty($interval_d) && intval($interval_d) > 0) {
                $interval = $interval_d . ((intval($interval_d) > 1) ? ' days' : ' day');
            } else if (!empty($interval_h) && intval($interval_h) > 0) {
                $interval = $interval_h . ((intval($interval_h) > 1) ? ' hours' : ' hour');
            } else if (!empty($interval_i) && intval($interval_i) > 0) {
                $interval = $interval_i . ((intval($interval_i) > 1) ? ' minutes' : ' minute');
            } else if (!empty($interval_s) && intval($interval_s) > 0) {
                $interval = $interval_s . ((intval($interval_s) > 1) ? ' seconds' : ' second');
            } else {
                $interval = '0 seconds';
            }

            echo '
            <a href="' . Url::toRoute([$route[0], 'id' => $route['id'], 'notif' => $value->id]) . '" class="dropdown-item" style="display: inline-block;">
                <i class="fas fa-' . Yii::$app->params['fontawesome']['v5'][$value->class] . ' mr-2"></i> 
                <span style="white-space: normal;">' . $value->message . '</span>
                <span class="float-right text-muted text-sm">' . $interval . ' ago</span>
            </a>

            <div class="dropdown-divider clearfix"></div>
            ';
        }

        echo '
                <a href="' . Url::toRoute(['/site/notification/index']) . '" class="dropdown-item dropdown-footer">See All Notifications</a>
            </div>
        </li>
        ';
    }

    public static function getUnreadCount()
    {
        $count = 0;

        if (!Yii::$app->user->isGuest) {
            $count = SiteNotification::find()->where([
                'user_id' => Yii::$app->user->identity->id, 
                'is_read' => 'no', 
            ])->count();
        }

        return $count;
    }
}
