<?php

namespace console\components;

use Yii;
use common\modules\site\components\Email;
use common\modules\site\components\GenesysEmail;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\mailer\models\MailerCampaign;
use common\modules\mailer\models\MailerContact;

class EmailCampaign extends \yii\base\Component
{

    private $campaign;
    private $campaign_version;
    private $uuid;
    private $identifier_name;
    private $transport;
    private $site_id;
    private $funnel;
    private $iteration;
    private $text_only;
    private $marketing;
    private $product;
    private $designation;

    public function __construct($vars, $config = [])
    {
        foreach ($vars as $var_k => $var_v) {
            $this->$var_k = $var_v;
        }

        parent::__construct($config);
    }

    public function send()
    {
        $modelMailerCampaign = new MailerCampaign();
        $modelMailerContact = new MailerContact();
        $data = $user_ids = array();
        $count = 0;

        $this->funnel = ($this->funnel == 'null') ? null : $this->funnel;
        $this->iteration = ($this->iteration == 'false') ? false : $this->iteration;
        $this->text_only = ($this->text_only == 'true') ? true : false;
        $this->marketing = ($this->marketing == 'true') ? true : false;

        if (MailerCampaign::find()->where(['campaign' => $this->campaign, 'identifier_name' => $this->identifier_name])->exists()) {    // a previous record exists in 'mod_mailer_campaign'
            if ($this->iteration !== false) { // Subsequent iteration
                $modelMailerCampaign = MailerCampaign::find()
                    ->where(['campaign' => $this->campaign, 'identifier_name' => $this->identifier_name, 'count' => intval($this->iteration - 1)])
                    ->andWhere(['IS', 'timestamp_click', new \yii\db\Expression('NULL')])
                    ->orderBy(['identifier' => SORT_ASC])
                    ->one();
            } else {    // First iteration
                $modelMailerCampaign = MailerCampaign::find()
                    ->where(['campaign' => $this->campaign, 'identifier_name' => $this->identifier_name])
                    ->orderBy(['identifier' => SORT_DESC])
                    ->one();
            }

            switch ($this->identifier_name) {
                case 'mailer_contact_id':
                    if ($this->iteration !== false) { // Subsequent iteration
                        $modelMailerContact = MailerContact::find()
                            ->where(['id' => $modelMailerCampaign->identifier, 'subscribed' => 'yes', 'status' => 'active'])
                            ->one();
                    } else {    // First iteration
                        $modelMailerContact = MailerContact::find()
                            ->where(['subscribed' => 'yes', 'status' => 'active'])
                            ->andWhere(['>', 'id', $modelMailerCampaign->identifier])
                            ->one();
                    }

                    $identifier_tmp = $modelMailerContact->id;

                    break;

                case 'user_id':
                    $modelUser = User::find()->where(['role_id' => 2, 'status' => 'active'])->andWhere(['>', 'id', $modelMailerCampaign->identifier])->one();

                    $identifier_tmp = $modelUser->id;

                    break;

                case 'order_id':
                    if ($this->iteration !== false) { // Subsequent iteration
                        do {
                            if (!isset($modelMailerCampaign->identifier)) {
                                return false;
                            }

                            $modelShopOrder = ShopOrder::find()
                                ->where(['id' => $modelMailerCampaign->identifier])
                                ->one();
                            
                            $modelMailerCampaign = MailerCampaign::find()
                                ->where(['campaign' => $this->campaign, 'identifier_name' => $this->identifier_name, 'count' => intval($this->iteration - 1)])
                                ->andWhere(['IS', 'timestamp_click', new \yii\db\Expression('NULL')])
                                ->andWhere(['>', 'id', $modelMailerCampaign->identifier])
                                ->orderBy(['identifier' => SORT_ASC])
                                ->one();
                        } while (isset($modelShopOrder->user_id, $modelMailerCampaign->identifier) && $this->isRepeatUser($modelShopOrder->user_id));
                    } else {    // First iteration
                        if (!isset($modelMailerCampaign->identifier)) {
                            return false;
                        }

                        $identifier_temp = $modelMailerCampaign->identifier;

                        do {
                            $modelShopOrder = ShopOrder::find()
                                ->where(['order_status_id' => 1, 'status' => 'active'])
                                ->andWhere(['>', 'id', $identifier_temp])
                                ->one();
                            
                            $identifier_temp++;
                        } while (isset($modelShopOrder->user_id) && $this->isRepeatUser($modelShopOrder->user_id));
                    }

                    if (isset($modelShopOrder->id) && !empty($modelShopOrder->id)) {
                        $identifier_tmp = $modelShopOrder->id;
                    } else {
                        return false;
                    }

                    break;
                
                default:
                    $modelMailerContact = MailerContact::find()->where(['subscribed' => 'yes', 'status' => 'active'])->andWhere(['>', 'id', $modelMailerCampaign->identifier])->one();

                    $identifier_tmp = $modelMailerContact->id;

                    break;
            }
        } else {    // no previous record exists in 'mod_mailer_campaign'
            switch ($this->identifier_name) {
                case 'mailer_contact_id':
                    $modelMailerContact = MailerContact::find()->where(['subscribed' => 'yes', 'status' => 'active'])->one();

                    $identifier_tmp = $modelMailerContact->id;

                    break;

                case 'user_id':
                    $modelUser = User::find()->where(['role_id' => 2, 'status' => 'active'])->one();

                    $identifier_tmp = $modelUser->id;

                    break;

                case 'order_id':
                    $modelShopOrder = ShopOrder::find()->where(['order_status_id' => 1, 'status' => 'active'])->one();

                    $identifier_tmp = $modelShopOrder->id;

                    break;
                
                default:
                    $modelMailerContact = MailerContact::find()->where(['subscribed' => 'yes', 'status' => 'active'])->one();

                    $identifier_tmp = $modelMailerContact->id;
                    
                    break;
            }
        }

        if (MailerCampaign::find()->where(['campaign' => $this->campaign, 'identifier_name' => $this->identifier_name, 'identifier' => $identifier_tmp])->exists()) {
            $modelMailerCampaign = MailerCampaign::find()->where(['campaign' => $this->campaign, 'identifier_name' => $this->identifier_name, 'identifier' => $identifier_tmp])->one();

            $count = $modelMailerCampaign->count;
        } else {
            $modelMailerCampaign = new MailerCampaign();

            $modelMailerCampaign->campaign = $this->campaign;
            $modelMailerCampaign->identifier_name = $this->identifier_name;
            $modelMailerCampaign->identifier = $identifier_tmp;
        }

        $modelMailerCampaign->count = $count + 1;
        $modelMailerCampaign->timestamp = (string) time();
        $modelMailerCampaign->in_queue = 'no';
        $modelMailerCampaign->site_id = $this->site_id;
        $modelMailerCampaign->created_by = 1;
        $modelMailerCampaign->updated_by = 1;

        if ($modelMailerCampaign->validate()) {
            $modelMailerCampaign->save();

            $email_from = Yii::$app->params['email']['from'][Yii::$app->params['mode']];

            switch ($this->identifier_name) {
                case 'mailer_contact_id':
                    $data['mailer_contact'] = MailerContact::find()->where(['id' => $identifier_tmp])->asArray()->one();
                    $data['mailer_contact']['metadata'] = json_decode($data['mailer_contact']['metadata'], true);

                    $email_to = $data['mailer_contact']['email'];

                    break;

                case 'user_id':
                    $data['user'] = User::find()->where(['id' => $identifier_tmp])->asArray()->one();
                    $data['user_profile'] = UserProfile::find()->where(['user_id' => $identifier_tmp])->asArray()->one();

                    $email_to = $data['user']['email'];

                    break;

                case 'order_id':
                    $data['order'] = ShopOrder::find()->where(['id' => $identifier_tmp])->asArray()->one();
                    $data['order_item'] = ShopOrderItem::find()->where(['order_id' => $identifier_tmp])->asArray()->one();
                    $data['user'] = User::find()->where(['id' => $data['order']['user_id']])->asArray()->one();
                    $data['user_profile'] = UserProfile::find()->where(['user_id' => $data['order']['user_id']])->asArray()->one();

                    $email_to = $data['user']['email'];

                    if (isset($this->funnel) && !empty($this->funnel)) {
                        $data['funnel'] = $this->funnel;
                        
                        $email_from = 'noreply@' . str_replace('_', '.', $this->funnel);
                    }

                    break;
                
                default:
                    $data['mailer_contact'] = MailerContact::find()->where(['id' => $identifier_tmp])->asArray()->one();
                    $data['mailer_contact']['metadata'] = json_decode($data['mailer_contact']['metadata'], true);

                    $email_to = $data['mailer_contact']['email'];
                    
                    break;
            }
            
            $data['campaign'] = $this->campaign;
            $data['identifier_name'] = $this->identifier_name;
            $data['timestamp'] = $modelMailerCampaign->timestamp;

            $tmp_subject = '';

            if ($this->funnel == 'esslcerts_com' && $this->campaign == 'renewal-ssl') {
                $tmp_subject = '[Expiration Notice] ' . strtoupper($data['mailer_contact']['metadata']['product_name']) . ' Certificate';
            } else if ($this->funnel == 'inetdomainportal_com' && $this->campaign == 'renewal') {
                $tmp_subject = 'SERVICE TERMINATION NOTICE: ' . strtoupper($data['order_item']['product_name']) . ' Expiration';
            }

            return Email::send(
                'email-campaign/email-campaign-' . $this->campaign, 
                [
                    'html' => ($this->text_only === true) ? false : 'email-campaign/email-campaign-' . $this->campaign . '-html', 
                    'text' => 'email-campaign/email-campaign-' . $this->campaign . '-text'
                ], 
                [
                    'to' => (Yii::$app->params['mode'] == 'live') ? $email_to : Yii::$app->params['email']['developer'], 
                    'from' => $email_from, 
                    'sg_sender_id' => !empty(Yii::$app->params['email']['sendgrid']['sender_id']) ? Yii::$app->params['email']['sendgrid']['sender_id'] : null,
                ], 
                ((!isset($this->funnel) || empty($this->funnel)) ? Yii::$app->params['email']['subject']['email-campaign-' . $this->campaign] : $tmp_subject), 
                ['data' => $data], 
                'email_campaign_' . str_replace('-', '_', $this->campaign), 
                $this->funnel, 
                true, 
                $this->transport,
                $this->marketing
            );
        }
    }

    public function sendContact($where = [])
    {
        $modelMailerCampaign = new MailerCampaign;
        $modelMailerContact = new MailerContact;
        $data = array();
        $count = 0;

        if (MailerCampaign::find()
            ->where([
                'campaign' => $this->campaign,
                'uuid' => $this->uuid,
                'identifier_name' => $this->identifier_name
            ])
            ->exists()
        ) { // Log(s) exist for this combination of campaign and contacts
            if ($this->iteration !== false) { // Subsequent iteration
                $modelMailerCampaign = MailerCampaign::find()
                    ->where([
                        'campaign' => $this->campaign,
                        'uuid' => $this->uuid,
                        'identifier_name' => $this->identifier_name,
                        'count' => intval($this->iteration - 1)
                    ])
                    ->andWhere(['IS', 'timestamp_click', new \yii\db\Expression('NULL')])
                    ->orderBy(['identifier' => SORT_ASC])
                    ->one();

                $modelMailerContact = MailerContact::find()
                    ->where([
                        'id' => $modelMailerCampaign->identifier, 
                        'subscribed' => 'yes', 
                        'status' => 'active'
                    ])
                    ->andWhere($where)
                    ->one();
            } else {    // First iteration
                $modelMailerCampaign = MailerCampaign::find()
                    ->select('identifier')
                    ->where([
                        'campaign' => $this->campaign,
                        'uuid' => $this->uuid,
                        'identifier_name' => $this->identifier_name
                    ])
                    ->column();

                $modelMailerContact = MailerContact::find()
                    ->where([
                        'subscribed' => 'yes', 
                        'status' => 'active'
                    ])
                    ->andWhere(['not in', 'id', $modelMailerCampaign])
                    ->andWhere($where)
                    ->one();
            }
        } else {    // No log exists for this combination of campaign and contacts
            $modelMailerContact = MailerContact::find()
                ->where([
                    'subscribed' => 'yes', 
                    'status' => 'active'
                ])
                ->andWhere($where)
                ->one();
        }

        if (!isset($modelMailerContact->id) || empty($modelMailerContact->id)) {
            return false;
        }

        $identifier_tmp = $modelMailerContact->id;

        if (MailerCampaign::find()
            ->where([
                'campaign' => $this->campaign,
                'uuid' => $this->uuid,
                'identifier_name' => $this->identifier_name,
                'identifier' => $identifier_tmp
            ])
            ->exists()
        ) {
            $modelMailerCampaign = MailerCampaign::find()
                ->where([
                    'campaign' => $this->campaign,
                    'uuid' => $this->uuid,
                    'identifier_name' => $this->identifier_name,
                    'identifier' => $identifier_tmp
                ])
                ->one();

            $count = $modelMailerCampaign->count;
        } else {
            $modelMailerCampaign = new MailerCampaign;

            $modelMailerCampaign->campaign = $this->campaign;
            $modelMailerCampaign->uuid = $this->uuid;
            $modelMailerCampaign->identifier_name = $this->identifier_name;
            $modelMailerCampaign->identifier = $identifier_tmp;
        }

        $modelMailerCampaign->count = $count + 1;
        $modelMailerCampaign->timestamp = (string) time();
        $modelMailerCampaign->in_queue = 'no';
        $modelMailerCampaign->site_id = $this->site_id;
        $modelMailerCampaign->created_by = 1;
        $modelMailerCampaign->updated_by = 1;

        if ($modelMailerCampaign->validate()) {
            $modelMailerCampaign->save();

            $data['mailer_contact'] = MailerContact::find()->where(['id' => $identifier_tmp])->asArray()->one();
            $data['mailer_contact']['metadata'] = json_decode($data['mailer_contact']['metadata'], true);
            $data['campaign'] = $this->campaign;
            $data['uuid'] = $this->uuid;
            $data['identifier'] = $identifier_tmp;
            $data['identifier_name'] = $this->identifier_name;
            $data['timestamp'] = $modelMailerCampaign->timestamp;

            $email_to = $data['mailer_contact']['email'];
            $email_from = Yii::$app->params['email']['from'][Yii::$app->params['mode']];

            return GenesysEmail::send(
                'email-campaign/' . $this->campaign . '/' . $this->campaign_version, 
                [
                    'html' => ($this->text_only === true) ? false : 'email-campaign/' . $this->campaign . '/' . $this->campaign_version . '-html', 
                    'text' => 'email-campaign/' . $this->campaign . '/' . $this->campaign_version . '-text'
                ], 
                [
                    'to' => (Yii::$app->params['mode'] == 'live') ? $email_to : Yii::$app->params['email']['developer'], 
                    'from' => $email_from, 
                    'sg_sender_id' => !empty(Yii::$app->params['email']['sendgrid']['sender_id']) ? Yii::$app->params['email']['sendgrid']['sender_id'] : null,
                ], 
                $this->getSubject($data), 
                ['data' => $data], 
                null, 
                $this->funnel, 
                true, 
                $this->transport,
                $this->marketing
            );
        }

        return false;
    }

    public function sendOrder($where = [])
    {
        $modelMailerCampaign = new MailerCampaign();
        $modelMailerContact = new MailerContact();
        $data = array();
        $count = 0;

        if (MailerCampaign::find()
            ->where([
                'campaign' => $this->campaign, 
                'identifier_name' => $this->identifier_name
            ])
            ->exists()
        ) { // Log(s) exist for this combination of campaign and contacts
            if ($this->iteration !== false) {   // Subsequent iteration
                $modelMailerCampaign = MailerCampaign::find()
                    ->where([
                        'campaign' => $this->campaign, 
                        'uuid' => $this->uuid, 
                        'identifier_name' => $this->identifier_name, 
                        'count' => intval($this->iteration - 1)
                    ])
                    ->andWhere(['IS', 'timestamp_click', new \yii\db\Expression('NULL')])
                    ->orderBy(['identifier' => SORT_ASC])
                    ->one();

                do {
                    if (!isset($modelMailerCampaign->identifier)) {
                        return false;
                    }

                    $modelShopOrder = ShopOrder::find()
                        ->where([
                            'id' => $modelMailerCampaign->identifier
                        ])
                        ->one();
                    
                    $modelMailerCampaign = MailerCampaign::find()
                        ->where([
                            'campaign' => $this->campaign, 
                            'uuid' => $this->uuid, 
                            'identifier_name' => $this->identifier_name, 
                            'count' => intval($this->iteration - 1)
                        ])
                        ->andWhere(['IS', 'timestamp_click', new \yii\db\Expression('NULL')])
                        ->andWhere(['>', 'id', $modelMailerCampaign->identifier])
                        ->orderBy(['identifier' => SORT_ASC])
                        ->one();
                } while (isset($modelShopOrder->user_id, $modelMailerCampaign->identifier) && $this->isRepeatUser($modelShopOrder->user_id));
            } else {    // First iteration
                $modelMailerCampaign = MailerCampaign::find()
                    ->where([
                        'campaign' => $this->campaign, 
                        'uuid' => $this->uuid, 
                        'identifier_name' => $this->identifier_name
                    ])
                    ->orderBy(['identifier' => SORT_DESC])
                    ->one();

                if (!isset($modelMailerCampaign->identifier)) {
                    return false;
                }

                $identifier_temp = $modelMailerCampaign->identifier;

                do {
                    $modelShopOrder = ShopOrder::find()
                        ->where([
                            'order_status_id' => 1, 
                            'status' => 'active'
                        ])
                        ->andWhere(['>', 'id', $identifier_temp])
                        ->one();
                    
                    $identifier_temp++;
                } while (isset($modelShopOrder->user_id) && $this->isRepeatUser($modelShopOrder->user_id));
            }
            
            if (!isset($modelShopOrder->id) || empty($modelShopOrder->id)) {
                return false;
            }
        } else {    // No log exists for this combination of campaign and contacts
            $modelShopOrder = ShopOrder::find()
                ->where([
                    'order_status_id' => 1, 
                    'status' => 'active'
                ])
                ->one();
        }

        $identifier_tmp = $modelShopOrder->id;

        if (MailerCampaign::find()
            ->where([
                'campaign' => $this->campaign, 
                'uuid' => $this->uuid, 
                'identifier_name' => $this->identifier_name, 
                'identifier' => $identifier_tmp
            ])
            ->exists()
        ) {
            $modelMailerCampaign = MailerCampaign::find()
                ->where([
                    'campaign' => $this->campaign, 
                    'uuid' => $this->uuid, 
                    'identifier_name' => $this->identifier_name, 
                    'identifier' => $identifier_tmp
                ])
                ->one();

            $count = $modelMailerCampaign->count;
        } else {
            $modelMailerCampaign = new MailerCampaign();

            $modelMailerCampaign->campaign = $this->campaign;
            $modelMailerCampaign->uuid = $this->uuid;
            $modelMailerCampaign->identifier_name = $this->identifier_name;
            $modelMailerCampaign->identifier = $identifier_tmp;
        }

        $modelMailerCampaign->count = $count + 1;
        $modelMailerCampaign->timestamp = (string) time();
        $modelMailerCampaign->in_queue = 'no';
        $modelMailerCampaign->site_id = $this->site_id;
        $modelMailerCampaign->created_by = 1;
        $modelMailerCampaign->updated_by = 1;

        if ($modelMailerCampaign->validate()) {
            $modelMailerCampaign->save();

            $email_from = Yii::$app->params['email']['from'][Yii::$app->params['mode']];

            $data['order'] = ShopOrder::find()->where(['id' => $identifier_tmp])->asArray()->one();
            $data['order_item'] = ShopOrderItem::find()->where(['order_id' => $identifier_tmp])->asArray()->one();
            $data['user'] = User::find()->where(['id' => $data['order']['user_id']])->asArray()->one();
            $data['user_profile'] = UserProfile::find()->where(['user_id' => $data['order']['user_id']])->asArray()->one();
            $data['campaign'] = $this->campaign;
            $data['uuid'] = $this->uuid;
            $data['identifier'] = $identifier_tmp;
            $data['identifier_name'] = $this->identifier_name;
            $data['timestamp'] = $modelMailerCampaign->timestamp;

            $email_to = $data['user']['email'];

            if (isset($this->funnel) && !empty($this->funnel)) {
                $data['funnel'] = $this->funnel;
                
                $email_from = 'noreply@' . str_replace('_', '.', $this->funnel);
            }

            return GenesysEmail::send(
                'email-campaign/' . $this->campaign . '/' . $this->campaign_version, 
                [
                    'html' => ($this->text_only === true) ? false : 'email-campaign/' . $this->campaign . '/' . $this->campaign_version . '-html', 
                    'text' => 'email-campaign/' . $this->campaign . '/' . $this->campaign_version . '-text'
                ], 
                [
                    'to' => (Yii::$app->params['mode'] == 'live') ? $email_to : Yii::$app->params['email']['developer'], 
                    'from' => $email_from, 
                    'sg_sender_id' => !empty(Yii::$app->params['email']['sendgrid']['sender_id']) ? Yii::$app->params['email']['sendgrid']['sender_id'] : null
                ], 
                $this->getSubject($data), 
                ['data' => $data], 
                null, 
                $this->funnel, 
                true, 
                $this->transport, 
                $this->marketing
            );
        }
    }

    private function getSubject($data)
    {
        switch ($this->campaign) {
            case 'dnc-renewal':
                $subject = 'SERVICE TERMINATION NOTICE: ' . strtoupper($data['order_item']['product_name']) . ' Expiration';

                break;

            case 'essl-renewal':
                $subject = '[Expiration Notice] ' . strtoupper($data['mailer_contact']['metadata']['product_name']) . ' Certificate';

                break;

            case 'lean-awareness-workshop':
                $subject = 'FREE 3-Hours Workshop Custom-made for ' . strtoupper($data['mailer_contact']['metadata']['organization']) . '\'s Rapid Growth (valid until ' . date('Y-m-d', strtotime('+30 days')) . ')';

                break;

            case 'cpps':
                $subject = '25% Limited Period Discount for ' . strtoupper($data['mailer_contact']['first_name'] . ' ' . $data['mailer_contact']['last_name']) . '\'s CPPS Training and Certification (valid until ' . date('Y-m-d', strtotime('+2 days')) . ')';

                break;

            default:
                $subject = Yii::$app->params['email']['subject']['email-campaign'][$this->campaign][$this->campaign_version];

                break;
        }

        return $subject;
    }

    private function isRepeatUser($user_id)
    {
        return MailerCampaign::find()
            ->joinWith('order')
            ->where([
                'mod_mailer_campaign.campaign' => $this->campaign, 
                'mod_mailer_campaign.uuid' => $this->uuid, 
                'mod_mailer_campaign.identifier_name' => $this->identifier_name, 
                'mod_mailer_campaign.count' => ($this->iteration === false) ? 1 : $this->iteration,
                'mod_shop_order.user_id' => $user_id,
            ])
            ->exists();
    }
}
