<?php

namespace console\controllers;

use Yii;
use yii\console\Controller;
use common\modules\core\components\Globals;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\mailer\models\MailerContact;

class ContactController extends Controller
{

    public function actionImport($type = 'mailer_contact', $dir_name = '/data/', $filename = 'mailer_contact.xlsx', $tag = null)
    {
        $modelMailerContact = new MailerContact();
        $loopCounter = 0;
        $total = 0;

        $files = \yii\helpers\FileHelper::findFiles(Yii::getAlias('@assets') . $dir_name);

        foreach ($files as $file) {
            $filename = explode('/', $file);
            $filename = end($filename);

            echo "\nImporting contacts from the file: " . $filename . "\n";

            if (!file_exists(Yii::getAlias('@assets') . $dir_name . $filename)) {
                throw new NotFoundHttpException('File does not exist: ' . Yii::getAlias('@assets') . $dir_name . $filename);
            }

            $data = \moonland\phpexcel\Excel::import(Yii::getAlias('@assets') . $dir_name . $filename, [
                'setFirstRecordAsKeys' => true, 
            ]);

            if (isset($data[0]) && !isset($data[0]['email'])) {
                $data = $data[0];
            }

            $total = count($data);

            $loopCounter = 0;

            try {
                foreach ($data as $key => $value) {
                    $loopCounter++;

                    $user_email = '';

                    echo Globals::cliProgress($loopCounter, $total, "\e[33mImporting " . $loopCounter . " contacts\e[39m");

                    switch ($type) {
                        case 'mailer_contact':
                            $user_email = strtolower(trim(isset($value['email']) ? $value['email'] : ''));

                            if (!isset($value['email']) || empty($value['email']) || !filter_var($value['email'], FILTER_VALIDATE_EMAIL) || MailerContact::find()->where(['email' => $user_email])->exists()) {
                                continue 2;
                            }

                            $contact_id = $modelMailerContact->importMailerContact($value, $tag);

                            if (!$contact_id) {
                                $loopCounter--;
                            }

                            break;

                        case 'user':
                            $user_email = strtolower(trim(isset($value['email']) ? $value['email'] : ''));

                            if (!isset($value['email']) || empty($value['email']) || !filter_var($value['email'], FILTER_VALIDATE_EMAIL) || User::find()->where(['email' => $user_email])->exists()) {
                                continue 2;
                            }

                            $user_id = $modelMailerContact->importUser($value);

                            if (!$user_id) {
                                $loopCounter--;
                            }

                            break;
                        
                        default:
                            
                            break;
                    }

                    /* START: Janitor working... */
                    $var_keys = array_keys(get_defined_vars());
                    $except = ['type', 'tag', 'modelMailerContact', 'total', 'key', 'value', 'loopCounter', 'files', 'file', 'dir_name'];

                    for ($i = 0; $i < sizeOf($var_keys); $i++) {
                        $temp = $var_keys[$i];

                        if (in_array($temp, $except)) {
                            continue;
                        }

                        unset($$temp);
                    }

                    unset($i, $var_keys, $temp);

                    gc_collect_cycles();

                    usleep(1000);
                    /* END: Janitor working... */
                }

                echo "\n\n\e[32m" . $loopCounter . " contacts imported successfully.\e[39m\n\n";
            } catch (\Exception $e) {
                echo "\n\n\e[31mImporting contacts failed.\e[39m\n\n";

                throw $e;
            }
        }
    }
}
