<?php

namespace console\controllers;

use Yii;
use yii\console\Controller;
use common\modules\site\components\Email;
use common\modules\user\models\User;
use common\modules\user\models\UserProfile;
use common\modules\shop\models\ShopOrder;
use common\modules\shop\models\ShopOrderItem;
use common\modules\mailer\models\MailerCampaign;
use common\modules\mailer\models\MailerContact;

class EmailCampaignController extends Controller
{

    public function actionIndex(
        $campaign = null, 
        $identifier_name = 'mailer_contact_id', 
        $transport = 'smtp-1', 
        $site_id = 1, 
        $funnel = null, 
        $iteration = false, 
        $text_only = false,
        $marketing = false
    )
    {
        $modelMailerCampaign = new MailerCampaign();
        $modelMailerContact = new MailerContact();
        $data = $user_ids = array();
        $count = 0;

        $funnel = ($funnel == 'null') ? null : $funnel;
        $iteration = ($iteration == 'false') ? false : $iteration;
        $text_only = ($text_only == 'true') ? true : false;
        $marketing = ($marketing == 'true') ? true : false;

        if (MailerCampaign::find()->where(['campaign' => $campaign, 'identifier_name' => $identifier_name])->exists()) {    // a previous record exists in 'mod_mailer_campaign'
            if ($iteration !== false) { // Subsequent iteration
                $modelMailerCampaign = MailerCampaign::find()
                    ->where(['campaign' => $campaign, 'identifier_name' => $identifier_name, 'count' => intval($iteration - 1)])
                    ->andWhere(['IS', 'timestamp_click', new \yii\db\Expression('NULL')])
                    ->orderBy(['identifier' => SORT_ASC])
                    ->one();
            } else {    // First iteration
                $modelMailerCampaign = MailerCampaign::find()
                    ->where(['campaign' => $campaign, 'identifier_name' => $identifier_name])
                    ->orderBy(['identifier' => SORT_DESC])
                    ->one();
            }

            switch ($identifier_name) {
                case 'mailer_contact_id':
                    if ($iteration !== false) { // Subsequent iteration
                        $modelMailerContact = MailerContact::find()
                            ->where(['id' => $modelMailerCampaign->identifier, 'subscribed' => 'yes', 'status' => 'active'])
                            ->one();
                    } else {    // First iteration
                        $modelMailerContact = MailerContact::find()
                            ->where(['subscribed' => 'yes', 'status' => 'active'])
                            ->andWhere(['>', 'id', $modelMailerCampaign->identifier])
                            ->one();
                    }

                    $identifier_tmp = $modelMailerContact->id;

                    break;

                case 'user_id':
                    $modelUser = User::find()->where(['role_id' => 2, 'status' => 'active'])->andWhere(['>', 'id', $modelMailerCampaign->identifier])->one();

                    $identifier_tmp = $modelUser->id;

                    break;

                case 'order_id':
                    if ($iteration !== false) { // Subsequent iteration
                        do {
                            if (!isset($modelMailerCampaign->identifier)) {
                                return false;
                            }

                            $modelShopOrder = ShopOrder::find()
                                ->where(['id' => $modelMailerCampaign->identifier])
                                ->one();
                            
                            $modelMailerCampaign = MailerCampaign::find()
                                ->where(['campaign' => $campaign, 'identifier_name' => $identifier_name, 'count' => intval($iteration - 1)])
                                ->andWhere(['IS', 'timestamp_click', new \yii\db\Expression('NULL')])
                                ->andWhere(['>', 'id', $modelMailerCampaign->identifier])
                                ->orderBy(['identifier' => SORT_ASC])
                                ->one();
                        } while (isset($modelShopOrder->user_id, $modelMailerCampaign->identifier) && self::isRepeatUser($modelShopOrder->user_id, $campaign, $identifier_name, $iteration));
                    } else {    // First iteration
                        if (!isset($modelMailerCampaign->identifier)) {
                            return false;
                        }

                        $identifier_temp = $modelMailerCampaign->identifier;

                        do {
                            $modelShopOrder = ShopOrder::find()
                                ->where(['order_status_id' => 1, 'status' => 'active'])
                                ->andWhere(['>', 'id', $identifier_temp])
                                ->one();
                            
                            $identifier_temp++;
                        } while (isset($modelShopOrder->user_id) && self::isRepeatUser($modelShopOrder->user_id, $campaign, $identifier_name, $iteration));
                    }

                    if (isset($modelShopOrder->id) && !empty($modelShopOrder->id)) {
                        $identifier_tmp = $modelShopOrder->id;
                    } else {
                        return false;
                    }

                    break;
                
                default:
                    $modelMailerContact = MailerContact::find()->where(['subscribed' => 'yes', 'status' => 'active'])->andWhere(['>', 'id', $modelMailerCampaign->identifier])->one();

                    $identifier_tmp = $modelMailerContact->id;

                    break;
            }
        } else {    // no previous record exists in 'mod_mailer_campaign'
            switch ($identifier_name) {
                case 'mailer_contact_id':
                    $modelMailerContact = MailerContact::find()->where(['subscribed' => 'yes', 'status' => 'active'])->one();

                    $identifier_tmp = $modelMailerContact->id;

                    break;

                case 'user_id':
                    $modelUser = User::find()->where(['role_id' => 2, 'status' => 'active'])->one();

                    $identifier_tmp = $modelUser->id;

                    break;

                case 'order_id':
                    $modelShopOrder = ShopOrder::find()->where(['order_status_id' => 1, 'status' => 'active'])->one();

                    $identifier_tmp = $modelShopOrder->id;

                    break;
                
                default:
                    $modelMailerContact = MailerContact::find()->where(['subscribed' => 'yes', 'status' => 'active'])->one();

                    $identifier_tmp = $modelMailerContact->id;
                    
                    break;
            }
        }

        if (MailerCampaign::find()->where(['campaign' => $campaign, 'identifier_name' => $identifier_name, 'identifier' => $identifier_tmp])->exists()) {
            $modelMailerCampaign = MailerCampaign::find()->where(['campaign' => $campaign, 'identifier_name' => $identifier_name, 'identifier' => $identifier_tmp])->one();

            $count = $modelMailerCampaign->count;
        } else {
            $modelMailerCampaign = new MailerCampaign();

            $modelMailerCampaign->campaign = $campaign;
            $modelMailerCampaign->identifier_name = $identifier_name;
            $modelMailerCampaign->identifier = $identifier_tmp;
        }

        $modelMailerCampaign->count = $count + 1;
        $modelMailerCampaign->timestamp = (string) time();
        $modelMailerCampaign->in_queue = 'no';
        $modelMailerCampaign->site_id = $site_id;
        $modelMailerCampaign->created_by = 1;
        $modelMailerCampaign->updated_by = 1;

        if ($modelMailerCampaign->validate()) {
            $modelMailerCampaign->save();

            $email_from = Yii::$app->params['email']['from'][Yii::$app->params['mode']];

            switch ($identifier_name) {
                case 'mailer_contact_id':
                    $data['mailer_contact'] = MailerContact::find()->where(['id' => $identifier_tmp])->asArray()->one();
                    $data['mailer_contact']['metadata'] = json_decode($data['mailer_contact']['metadata'], true);

                    $email_to = $data['mailer_contact']['email'];

                    break;

                case 'user_id':
                    $data['user'] = User::find()->where(['id' => $identifier_tmp])->asArray()->one();
                    $data['user_profile'] = UserProfile::find()->where(['user_id' => $identifier_tmp])->asArray()->one();

                    $email_to = $data['user']['email'];

                    break;

                case 'order_id':
                    $data['order'] = ShopOrder::find()->where(['id' => $identifier_tmp])->asArray()->one();
                    $data['order_item'] = ShopOrderItem::find()->where(['order_id' => $identifier_tmp])->asArray()->one();
                    $data['user'] = User::find()->where(['id' => $data['order']['user_id']])->asArray()->one();
                    $data['user_profile'] = UserProfile::find()->where(['user_id' => $data['order']['user_id']])->asArray()->one();

                    $email_to = $data['user']['email'];

                    if (isset($funnel) && !empty($funnel)) {
                        $data['funnel'] = $funnel;
                        
                        $email_from = 'noreply@' . str_replace('_', '.', $funnel);
                    }

                    break;
                
                default:
                    $data['mailer_contact'] = MailerContact::find()->where(['id' => $identifier_tmp])->asArray()->one();
                    $data['mailer_contact']['metadata'] = json_decode($data['mailer_contact']['metadata'], true);

                    $email_to = $data['mailer_contact']['email'];
                    
                    break;
            }
            
            $data['campaign'] = $campaign;
            $data['identifier_name'] = $identifier_name;
            $data['timestamp'] = $modelMailerCampaign->timestamp;

            $tmp_subject = '';

            if ($funnel == 'esslcerts_com' && $campaign == 'renewal-ssl') {
                $tmp_subject = '[Expiration Notice] ' . strtoupper($data['mailer_contact']['metadata']['product_name']) . ' Certificate';
            } else if ($funnel == 'inetdomainportal_com' && $campaign == 'renewal') {
                $tmp_subject = 'SERVICE TERMINATION NOTICE: ' . strtoupper($data['order_item']['product_name']) . ' Expiration';
            }

            return Email::send(
                'email-campaign/email-campaign-' . $campaign, 
                [
                    'html' => ($text_only === true) ? false : 'email-campaign/email-campaign-' . $campaign . '-html', 
                    'text' => 'email-campaign/email-campaign-' . $campaign . '-text'
                ], 
                [
                    'to' => (Yii::$app->params['mode'] == 'live') ? $email_to : Yii::$app->params['email']['developer'], 
                    'from' => $email_from, 
                    'sg_sender_id' => !empty(Yii::$app->params['email']['sendgrid']['sender_id']) ? Yii::$app->params['email']['sendgrid']['sender_id'] : null,
                ], 
                ((!isset($funnel) || empty($funnel)) ? Yii::$app->params['email']['subject']['email-campaign-' . $campaign] : $tmp_subject), 
                ['data' => $data], 
                'email_campaign_' . str_replace('-', '_', $campaign), 
                $funnel, 
                true, 
                $transport,
                $marketing
            );
        }
    }

    private static function isRepeatUser($user_id, $campaign, $identifier_name, $iteration)
    {
        return MailerCampaign::find()
            ->joinWith('order')
            ->where([
                'mod_mailer_campaign.campaign' => $campaign, 
                'mod_mailer_campaign.identifier_name' => $identifier_name, 
                'mod_mailer_campaign.count' => ($iteration === false) ? 1 : $iteration,
                'mod_shop_order.user_id' => $user_id,
            ])
            ->exists();
    }
}
