<?php

namespace console\controllers;

use Yii;
use yii\console\ExitCode;
use yii\helpers\Console;
use yii\helpers\FileHelper;

class GenesysController extends \yii\console\Controller
{

    public function printMsg($type = '', $msg = '', $constant = '')
    {
        switch ($type) {
            case 'enter':
                $this->stdout("ENTER: $msg\n", Console::BOLD);

                break;

            case 'exit':
                $this->stdout("EXIT: $msg\n", Console::BOLD);
                
                break;

            case 'janitor':
                $this->stdout("Janitor working...\n", Console::BLINK);
                
                break;

            case 'info':
                $this->stdout("[" . date('Y-m-d H:i:s') . "] " . "$msg\n", Console::FG_CYAN);
                
                break;

            case 'success':
                $this->stderr("[" . date('Y-m-d H:i:s') . "] " . "$msg\n", Console::FG_GREEN);
                
                break;

            case 'error':
                $this->stderr("[" . date('Y-m-d H:i:s') . "] " . "$msg\n", Console::FG_RED);
                
                break;
            
            default:
                if (!empty($msg)) {
                    if (!empty($constant)) {
                        $this->stdout("$msg\n", $constant);
                    }

                    $this->stdout("$msg\n");
                }

                break;
        }
    }

    public static function getDsnAttribute($attribute, $dsn)
    {
        if (preg_match('/' . $attribute . '=([^;]*)/', $dsn, $match)) {
            return $match[1];
        } else {
            return null;
        }
    }

    public function actionPurge()
    {
        FileHelper::removeDirectory(Yii::getAlias('@frontend') . '/runtime', ['traverseSymlinks' => true]);
        FileHelper::removeDirectory(Yii::getAlias('@frontend') . '/web/assets', ['traverseSymlinks' => true]);
        FileHelper::createDirectory(Yii::getAlias('@frontend') . '/runtime');
        FileHelper::createDirectory(Yii::getAlias('@frontend') . '/web/assets');

        FileHelper::removeDirectory(Yii::getAlias('@backend') . '/runtime', ['traverseSymlinks' => true]);
        FileHelper::removeDirectory(Yii::getAlias('@backend') . '/web/assets', ['traverseSymlinks' => true]);
        FileHelper::createDirectory(Yii::getAlias('@backend') . '/runtime');
        FileHelper::createDirectory(Yii::getAlias('@backend') . '/web/assets');

        FileHelper::removeDirectory(Yii::getAlias('@console') . '/runtime', ['traverseSymlinks' => true]);
        FileHelper::createDirectory(Yii::getAlias('@console') . '/runtime');

        FileHelper::removeDirectory(Yii::getAlias('@assets') . '/runtime', ['traverseSymlinks' => true]);
        FileHelper::removeDirectory(Yii::getAlias('@assets') . '/web/assets', ['traverseSymlinks' => true]);
        FileHelper::createDirectory(Yii::getAlias('@assets') . '/runtime');
        FileHelper::createDirectory(Yii::getAlias('@assets') . '/web/assets');

        FileHelper::removeDirectory(Yii::getAlias('@api') . '/runtime', ['traverseSymlinks' => true]);
        FileHelper::removeDirectory(Yii::getAlias('@api') . '/web/assets', ['traverseSymlinks' => true]);
        FileHelper::createDirectory(Yii::getAlias('@api') . '/runtime');
        FileHelper::createDirectory(Yii::getAlias('@api') . '/web/assets');

        FileHelper::removeDirectory(Yii::getAlias('@opexms_com') . '/runtime', ['traverseSymlinks' => true]);
        FileHelper::removeDirectory(Yii::getAlias('@opexms_com') . '/web/assets', ['traverseSymlinks' => true]);
        FileHelper::createDirectory(Yii::getAlias('@opexms_com') . '/runtime');
        FileHelper::createDirectory(Yii::getAlias('@opexms_com') . '/web/assets');

        Yii::$app->db->createCommand()->truncateTable('mod_core_log')->execute();
        Yii::$app->db->createCommand()->truncateTable('mod_core_session')->execute();
    }

    public function actionSetCollation()
    {
        $db = Yii::$app->getDb();

        $result = $db->createCommand('ALTER DATABASE ' . self::getDsnAttribute('dbname', $db->dsn) . ' CHARACTER SET = utf8mb4 COLLATE = utf8mb4_unicode_ci')->execute();

        if (!is_integer($result)) {
            echo "Unable change collation of database\n";

            return;
        }

        $tables = $db->createCommand('SHOW TABLES')->queryAll();
        
        foreach ($tables as $table) {
            $result = $db->createCommand('ALTER TABLE ' . $table['tables_in_' . self::getDsnAttribute('dbname', $db->dsn)] . ' CONVERT TO CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci')->execute();

            if (!is_integer($result)) {
                echo "Unable change collation of " . $table['tables_in_' . self::getDsnAttribute('dbname', $db->dsn)] . "\n";

                return;
            }
        }

        echo "Collation changed successfully\n"; 
    }

    public function actionResetAutoIncrement()
    {
        $db = Yii::$app->getDb();

        $tables = $db->createCommand('SHOW TABLES')->queryAll();
        
        foreach ($tables as $table) {
            $result = $db->createCommand('ALTER TABLE ' . $table['tables_in_' . self::getDsnAttribute('dbname', $db->dsn)] . ' AUTO_INCREMENT=1')->execute();

            if (!is_integer($result)) {
                echo "Unable reset auto-increment of " . $table['tables_in_' . self::getDsnAttribute('dbname', $db->dsn)] . "\n";

                return;
            }
        }

        echo "Auto-increment reset successfully\n"; 
    }

    public function actionEmailCampaign(
        $campaign = null, 
        $uuid = null,
        $campaign_version = 'v1', 
        $identifier_name = 'mailer_contact_id', 
        $transport = 'smtp-1', 
        $site_id = 1, 
        $funnel = null, 
        $iteration = false, 
        $text_only = false, 
        $marketing = false
    )
    {
        if (!isset($uuid) || empty($uuid)) {
            return ExitCode::USAGE;
        }

        $funnel = ($funnel == 'null') ? null : $funnel;
        $iteration = ($iteration == 'false') ? false : $iteration;
        $text_only = ($text_only == 'true') ? true : false;
        $marketing = ($marketing == 'true') ? true : false;

        $emailCampaign = new \console\components\EmailCampaign(get_defined_vars());

        if ($emailCampaign->sendContact()) {
            return ExitCode::OK;
        } else {
            return ExitCode::UNSPECIFIED_ERROR;
        }
    }
}
