<?php

namespace console\controllers;

use Yii;
use yii\console\Controller;
use yii\helpers\FileHelper;
use common\modules\core\components\Globals;
use common\modules\core\models\CoreKeyword;
use common\modules\core\models\CoreValue;
use common\modules\assetManager\models\AssetManager;
use common\modules\cms\models\CmsBlog;

class WordpressController extends Controller
{
    public function actionImportPosts($filename = '')
    {
        $file = Yii::getAlias('@assets') . '/data/' . $filename;

        if (!file_exists($file)) {
            throw new NotFoundHttpException('File does not exist: ' . $file);
        }

        $dom = new \DomDocument;

        $dom->load($file);

        $items = $dom->getElementsByTagName('item');

        $posts = array();

        $count = 0;

        foreach ($items as $item) {
            $posts[$count]['title'] = $item->getElementsByTagName('title')->item(0)->nodeValue;
            $posts[$count]['link'] =  str_replace('/', '', str_replace('https://theceoviews.com/', '', $item->getElementsByTagName('link')->item(0)->nodeValue));
            $posts[$count]['description'] = $item->getElementsByTagName('description')->item(0)->nodeValue;
            $posts[$count]['content'] = nl2br($item->getElementsByTagName('encoded')->item(0)->nodeValue);
            $posts[$count]['post_date'] = date('Y-m-d h:s:i', strtotime($item->getElementsByTagName('post_date')->item(0)->nodeValue));
            $posts[$count]['post_name'] = $item->getElementsByTagName('post_name')->item(0)->nodeValue;
            $posts[$count]['status'] = $item->getElementsByTagName('status')->item(0)->nodeValue;

            $count_tmp = 0;

            foreach ($item->getElementsByTagName('category') as $category) {
                $posts[$count]['category'][$count_tmp] = $category->nodeValue;

                $count_tmp++;
            }

            $count_tmp = 0;

            foreach ($item->getElementsByTagName('meta_key') as $meta_key) {
                if ($meta_key->nodeValue == '_thumbnail_id') {
                    $count_temp = 0;

                    foreach ($item->getElementsByTagName('meta_value') as $meta_value) {
                        if ($count_tmp == $count_temp) {
                            $posts[$count]['_thumbnail_id'][$count_tmp] = $meta_value->nodeValue;
                        }

                        $count_temp++;
                    }
                }

                $count_tmp++;
            }

            $count++;
        }

        $modelCmsBlog = new CmsBlog();
    	
        $flagBreak = false;
    	$loopCounter = 0;

        $total = count($posts);
        $last_inserted = CmsBlog::find()->orderBy(['id' => SORT_DESC])->one();
        $last_inserted_id = isset($last_inserted->id) && !empty($last_inserted->id) ? $last_inserted->id : 0;
        $batch_size = $total;
        $current_batch_limit = $batch_size + $last_inserted_id;

    	$transaction = Yii::$app->db->beginTransaction();
        
        try {
            foreach ($posts as $post) {
            	$loopCounter++;

                if ($loopCounter <= $last_inserted_id) {
                    continue;
                } 

                if ($loopCounter > $current_batch_limit) {
                    $loopCounter--;

                    break;
                }

                $post_data = array();

                $post_data['CmsBlog']['name']['value'][1] = $post['title'];
                $post_data['CmsBlog']['title']['value'][1] = $post['title'];
                $post_data['CmsBlog']['meta_description']['value'][1] = $post['description'];
                $post_data['CmsBlog']['url']['value'][1] = $post['link'];
                $post_data['CmsBlog']['blog_content']['value'][1] = $post['content'];

                $modelCoreKeyword = new CoreKeyword();

                // Save keywords
                foreach ($post['category'] as $k => $v) {
                    if (!CoreKeyword::find()->where(['name' => $v, 'status' => 'active'])->exists()) {
                        $modelCoreKeyword = new CoreKeyword();
                        
                        $modelCoreKeyword->name = $v;
                        $modelCoreKeyword->status = 'active';

                        if ($modelCoreKeyword->validate()) {
                            $modelCoreKeyword->save();
                        } else {
                            $flagBreak = true;

                            break;
                        }
                    } else {
                        $modelCoreKeyword = CoreKeyword::find()->where(['name' => $v, 'status' => 'active'])->one();
                    }

                    $post_data['CmsEntityKeyword']['keyword_id'][$k] = $modelCoreKeyword->id;
                }

                $post_data['CmsBlog']['featured_image'] = intval($post['_thumbnail_id']);

            	$modelCmsBlog = new CmsBlog();

                if (!$modelCmsBlog->saveBlog($post_data)) {
                    $transaction->rollBack();

                    echo "\n\n\e[31mUnable to save blog " . Globals::renderErrors($modelCmsBlog->getErrors()) . "\e[39m\n\n";

                    $flagBreak = true;

                    break;
                }

                echo Globals::cliProgress($loopCounter, $total, "\e[33mMigrating blog #" . $loopCounter . "\e[39m");

                /* START: Janitor working... */
		        $var_keys = array_keys(get_defined_vars());
		        $except = ['dom', 'items', 'posts', 'count', 'total', 'transaction', 'last_inserted_id', 'current_batch_limit', 'flagBreak', 'loopCounter'];

		        for ($i = 0; $i < sizeOf($var_keys); $i++) {
		            $temp = $var_keys[$i];

		            if (in_array($temp, $except)) {
		                continue;
		            }

		            unset($$temp);
		        }

		        unset($i, $var_keys, $temp);

		        gc_collect_cycles();

		        usleep(100);
		        /* END: Janitor working... */
            }

            if (!$flagBreak) {
            	$transaction->commit();

				echo "\n\n\e[32m" . $loopCounter . " blogs saved successfully.\e[39m\n\n";
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function actionImportMedia($filename = '')
    {
        $file = Yii::getAlias('@assets') . '/data/' . $filename;

        if (!file_exists($file)) {
            throw new NotFoundHttpException('File does not exist: ' . $file);
        }

        $dom = new \DomDocument;

        $dom->load($file);

        $items = $dom->getElementsByTagName('item');

        $posts = array();

        $count = 0;

        foreach ($items as $item) {
            $posts[$count]['title'] = $item->getElementsByTagName('title')->item(0)->nodeValue;
            $posts[$count]['post_date'] = date('Y-m-d h:s:i', strtotime($item->getElementsByTagName('post_date')->item(0)->nodeValue));
            $posts[$count]['guid'] = $item->getElementsByTagName('guid')->item(0)->nodeValue;
            $posts[$count]['description'] = $item->getElementsByTagName('description')->item(0)->nodeValue;
            $posts[$count]['attachment_url'] = $item->getElementsByTagName('attachment_url')->item(0)->nodeValue;

            $count_tmp = 0;

            foreach ($item->getElementsByTagName('meta_key') as $meta_key) {
                if ($meta_key->nodeValue == '_wp_attached_file') {
                    $count_temp = 0;

                    foreach ($item->getElementsByTagName('meta_value') as $meta_value) {
                        if ($count_tmp == $count_temp) {
                            $posts[$count]['_wp_attached_file'][] = $meta_value->nodeValue;
                        }

                        $count_temp++;
                    }
                }

                if ($meta_key->nodeValue == '_wp_attachment_metadata') {
                    $count_temp = 0;

                    foreach ($item->getElementsByTagName('meta_value') as $meta_value) {
                        if ($count_tmp == $count_temp) {
                            $posts[$count]['_wp_attachment_metadata'][] = $meta_value->nodeValue;
                        }

                        $count_temp++;
                    }
                }

                $count_tmp++;
            }

            $count++;
        }

        $flagBreak = false;
        $loopCounter = 0;

        $total = count($posts);
        $last_inserted = AssetManager::find()->orderBy(['id' => SORT_DESC])->one();
        $last_inserted_id = isset($last_inserted->id) && !empty($last_inserted->id) ? $last_inserted->id : 0;
        $batch_size = $total;
        $current_batch_limit = $batch_size + $last_inserted_id;

        $transaction = Yii::$app->db->beginTransaction();

        try {
            foreach ($posts as $post) {
                $loopCounter++;

                if ($loopCounter <= $last_inserted_id) {
                    continue;
                } 

                if ($loopCounter > $current_batch_limit) {
                    $loopCounter--;

                    break;
                }

                $modelAssetManager = new AssetManager();

                $modelAssetManager->description = $post['description'];
                $modelAssetManager->app_alias = 'assets';

                $modelAssetManager->path = str_replace('http://theceoviews.com/wp-content', '', $post['guid']);
                $modelAssetManager->path = str_replace('https://theceoviews.com/wp-content', '', $modelAssetManager->path);

                $path_tmp = explode('/', $modelAssetManager->path);
                unset($path_tmp[count($path_tmp) - 1]);
                $path_tmp = implode('/', $path_tmp);

                $url_tmp = explode('/', $post['attachment_url']);
                
                $filename_tmp = $url_tmp[count($url_tmp) - 1];
                $filename_tmp = explode('.', $filename_tmp);
                $filename_tmp_ext = $filename_tmp[count($filename_tmp) - 1];
                unset($filename_tmp[count($filename_tmp) - 1]);
                $filename_tmp = implode('.', $filename_tmp);
                
                unset($url_tmp[count($url_tmp) - 1]);
                
                $url_tmp = implode('/', $url_tmp);

                $modelAssetManager->name = $filename_tmp . '.' . $filename_tmp_ext;

                $_wp_attachment_metadata = unserialize($post['_wp_attachment_metadata'][0]);

                exec('sudo mkdir -p ' . Yii::getAlias('@assets') . '/' . $path_tmp);

                exec('sudo wget -O ' . Yii::getAlias('@assets') . $path_tmp . '/' . $filename_tmp . '-thumb.' . $filename_tmp_ext . ' ' . $post['attachment_url']);
                exec('sudo wget -O ' . Yii::getAlias('@assets') . $path_tmp . '/' . $filename_tmp . '-small.' . $filename_tmp_ext . ' ' . $post['attachment_url']);
                exec('sudo wget -O ' . Yii::getAlias('@assets') . $path_tmp . '/' . $filename_tmp . '-medium.' . $filename_tmp_ext . ' ' . $post['attachment_url']);
                exec('sudo wget -O ' . Yii::getAlias('@assets') . $path_tmp . '/' . $filename_tmp . '-large.' . $filename_tmp_ext . ' ' . $post['attachment_url']);
                exec('sudo wget -O ' . Yii::getAlias('@assets') . $modelAssetManager->path . ' ' . $post['attachment_url']);

                $modelAssetManager->mime_type = mime_content_type(Yii::getAlias('@assets') . $modelAssetManager->path);
                $modelAssetManager->size = filesize(Yii::getAlias('@assets') . $modelAssetManager->path);

                $_wp_attachment_metadata_tmp['small'] = $path_tmp . '/' . $filename_tmp . '-small.' . $filename_tmp_ext;
                $_wp_attachment_metadata_tmp['medium'] = $path_tmp . '/' . $filename_tmp . '-medium.' . $filename_tmp_ext;
                $_wp_attachment_metadata_tmp['large'] = $path_tmp . '/' . $filename_tmp . '-large.' . $filename_tmp_ext;

                $modelAssetManager->thumbs = serialize($_wp_attachment_metadata_tmp);
                $modelAssetManager->created_by = 1;
                $modelAssetManager->updated_by = 1;
                $modelAssetManager->status = 'active';

                if ($modelAssetManager->validate()) {
                    $modelAssetManager->save();
                } else {
                    $transaction->rollBack();

                    echo "\n\n\e[31mUnable to save asset " . Globals::renderErrors($modelAssetManager->getErrors()) . "\e[39m\n\n";

                    $flagBreak = true;

                    break;
                }

                echo Globals::cliProgress($loopCounter, $total, "\e[33mMigrating assets #" . $loopCounter . "\e[39m");

                /* START: Janitor working... */
                $var_keys = array_keys(get_defined_vars());
                $except = ['dom', 'items', 'posts', 'count', 'total', 'transaction', 'last_inserted_id', 'current_batch_limit', 'flagBreak', 'loopCounter'];

                for ($i = 0; $i < sizeOf($var_keys); $i++) {
                    $temp = $var_keys[$i];

                    if (in_array($temp, $except)) {
                        continue;
                    }

                    unset($$temp);
                }

                unset($i, $var_keys, $temp);

                gc_collect_cycles();

                usleep(100);
                /* END: Janitor working... */
            }

            if (!$flagBreak) {
                $transaction->commit();

                echo "\n\n\e[32m" . $loopCounter . " assets saved successfully.\e[39m\n\n";
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }

    public function actionMapMedia($filename_posts = '', $filename_media = '')
    {
        $file_media = Yii::getAlias('@assets') . '/data/' . $filename_media;

        if (!file_exists($file_media)) {
            throw new NotFoundHttpException('File does not exist: ' . $file_media);
        }

        $dom_media = new \DomDocument;

        $dom_media->load($file_media);

        $items_media = $dom_media->getElementsByTagName('item');

        $assets = array();

        foreach ($items_media as $item_media) {
            $item_media_name = $item_media->getElementsByTagName('attachment_url')->item(0)->nodeValue;
            $item_media_name = explode('/', $item_media_name);

            $assets[$item_media->getElementsByTagName('post_id')->item(0)->nodeValue] = $item_media_name[count($item_media_name) - 1];
        }

        $file_posts = Yii::getAlias('@assets') . '/data/' . $filename_posts;

        if (!file_exists($file_posts)) {
            throw new NotFoundHttpException('File does not exist: ' . $file_posts);
        }

        $dom_posts = new \DomDocument;

        $dom_posts->load($file_posts);

        $items_posts = $dom_posts->getElementsByTagName('item');

        $posts = array();

        foreach ($items_posts as $item_post) {
            $count_tmp = 0;

            foreach ($item_post->getElementsByTagName('meta_key') as $meta_key) {
                if ($meta_key->nodeValue == '_thumbnail_id') {
                    $count_temp = 0;

                    foreach ($item_post->getElementsByTagName('meta_value') as $meta_value) {
                        if ($count_tmp == $count_temp) {
                            $posts[] = $meta_value->nodeValue;
                        }

                        $count_temp++;
                    }
                }

                $count_tmp++;
            }
        }

        $modelCmsBlog = new CmsBlog();

        $flagBreak = false;
        $loopCounter = 0;

        $total = count($posts);

        $transaction = Yii::$app->db->beginTransaction();
        try {
            foreach ($posts as $k => $v) {
                $loopCounter++;

                $modelCmsBlog = CmsBlog::find()->where(['id' => $k + 1])->one();

                $modelCmsBlog->featured_image = intval(AssetManager::getAttr(['id'], ['name' => $assets[$v]]));

                if ($modelCmsBlog->validate()) {
                    $modelCmsBlog->save();
                } else {var_dump($k); var_dump($v); die();
                    $transaction->rollBack();

                    echo "\n\n\e[31mUnable to map asset " . Globals::renderErrors($modelCmsBlog->getErrors()) . "\e[39m\n\n";

                    $flagBreak = true;

                    break;

                }

                echo Globals::cliProgress($loopCounter, $total, "\e[33mMapping asset #" . $loopCounter . "\e[39m");

                /* START: Janitor working... */
                $var_keys = array_keys(get_defined_vars());
                $except = ['posts', 'assets', 'total', 'transaction', 'flagBreak', 'loopCounter'];

                for ($i = 0; $i < sizeOf($var_keys); $i++) {
                    $temp = $var_keys[$i];

                    if (in_array($temp, $except)) {
                        continue;
                    }

                    unset($$temp);
                }

                unset($i, $var_keys, $temp);

                gc_collect_cycles();

                usleep(100);
                /* END: Janitor working... */
            }

            if (!$flagBreak) {
                $transaction->commit();

                echo "\n\n\e[32m" . $loopCounter . " assets mapped successfully.\e[39m\n\n";
            }
        } catch (\Exception $e) {
            $transaction->rollBack();

            throw $e;
        }
    }
}
