<?php

use \yii\web\Request;
use yii\helpers\Url;
use common\modules\core\models\CoreEntityClass;
use common\modules\core\models\CoreAttributeClass;
use common\modules\core\models\CoreLanguage;
use common\modules\cms\models\CmsPage;
use common\modules\cms\models\CmsBlog;
use common\modules\cms\models\CmsTeamMember;
use common\modules\cms\models\CmsEvent;
use common\modules\cms\models\CmsAlbum;
use himiklab\sitemap\behaviors\SitemapBehavior;

$params = array_merge(
    require __DIR__ . '/../../common/config/params.php',
    require __DIR__ . '/../../common/config/params-local.php',
    require __DIR__ . '/params.php',
    require __DIR__ . '/params-local.php'
);

$baseUrl = str_replace('/frontend/web', '', (new Request)->getBaseUrl());

return [
    'id' => 'app-frontend',
    'name' => 'GeneSys',
    'basePath' => dirname(__DIR__),
    'bootstrap' => [
        'log', 
        function($app) {
            $eventLogsEvents = new \common\modules\logs\components\LogsEvents;
            $eventLogsEvents->eventModLogsUserVisits();

            return;
        }
    ],
    'modules' => [
        'sitemap' => [
            'class' => 'himiklab\sitemap\Sitemap',
            'models' => [
                [
                    'class' => 'common\modules\core\models\CoreValue',
                    'behaviors' => [
                        'sitemap' => [
                            'class' => SitemapBehavior::className(),
                            'scope' => function ($model) {
                                $model->where(['attribute_class_id' => 6, 'language_id' => CoreLanguage::getAttr(['id'], ['code' => Yii::$app->params['language_code']])])->orderBy('entity_class_id');
                            },
                            'dataClosure' => function ($model) {
                                $pageCoreEntityClass = CoreEntityClass::findOne(['name' => 'page', 'status' => 'active']);
                                $blogCoreEntityClass = CoreEntityClass::findOne(['name' => 'blog', 'status' => 'active']);
                                $teamMemberCoreEntityClass = CoreEntityClass::findOne(['name' => 'team_member', 'status' => 'active']);
                                $eventCoreEntityClass = CoreEntityClass::findOne(['name' => 'event', 'status' => 'active']);
                                $albumCoreEntityClass = CoreEntityClass::findOne(['name' => 'album', 'status' => 'active']);

                                if ($model->entity_class_id == $pageCoreEntityClass->id) {
                                    if (CmsPage::getAttr(['status'], ['id' => $model->entity_id]) == 'active') {
                                        return [
                                            'loc' => Url::to($model->value, true),
                                            'lastmod' => strtotime($model->updated_at),
                                            'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY,
                                            'priority' => 0.40
                                        ];
                                    }
                                } else if ($model->entity_class_id == $blogCoreEntityClass->id) {
                                    if (CmsBlog::getAttr(['status'], ['id' => $model->entity_id]) == 'active') {
                                        return [
                                            'loc' => Url::to('case-studies/' . $model->value, true),
                                            'lastmod' => strtotime($model->updated_at),
                                            'changefreq' => SitemapBehavior::CHANGEFREQ_WEEKLY,
                                            'priority' => 0.60
                                        ];
                                    }
                                } else if ($model->entity_class_id == $teamMemberCoreEntityClass->id) {
                                    if (CmsTeamMember::getAttr(['status'], ['id' => $model->entity_id]) == 'active') {
                                        return [
                                            'loc' => Url::to('member/' . $model->value, true),
                                            'lastmod' => strtotime($model->updated_at),
                                            'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY,
                                            'priority' => 0.40
                                        ];
                                    }
                                } else if ($model->entity_class_id == $eventCoreEntityClass->id) {
                                    if (CmsEvent::getAttr(['status'], ['id' => $model->entity_id]) == 'active') {
                                        return [
                                            'loc' => Url::to('event/' . $model->value, true),
                                            'lastmod' => strtotime($model->updated_at),
                                            'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY,
                                            'priority' => 0.40
                                        ];
                                    }
                                } else if ($model->entity_class_id == $albumCoreEntityClass->id) {
                                    if (CmsAlbum::getAttr(['status'], ['id' => $model->entity_id]) == 'active') {
                                        return [
                                            'loc' => Url::to('gallery/' . $model->value, true),
                                            'lastmod' => strtotime($model->updated_at),
                                            'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY,
                                            'priority' => 0.40
                                        ];
                                    }
                                } else {
                                    return [
                                        'loc' => Url::to($model->value, true),
                                        'lastmod' => strtotime($model->updated_at),
                                        'changefreq' => SitemapBehavior::CHANGEFREQ_DAILY,
                                        'priority' => 0.80
                                    ];
                                }
                            }
                        ],
                    ],
                ],
            ],
            'urls'=> [
                ['loc' => '/', 'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY, 'priority' => 0.40], 
                ['loc' => '/contact', 'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY, 'priority' => 0.40], 
                ['loc' => '/blog', 'changefreq' => SitemapBehavior::CHANGEFREQ_WEEKLY, 'priority' => 0.60], 
                ['loc' => '/case-studies', 'changefreq' => SitemapBehavior::CHANGEFREQ_WEEKLY, 'priority' => 0.60], 
                ['loc' => '/faqs', 'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY, 'priority' => 0.40], 
                ['loc' => '/testimonials', 'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY, 'priority' => 0.40], 
                ['loc' => '/our-team', 'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY, 'priority' => 0.40], 
                ['loc' => '/events', 'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY, 'priority' => 0.40], 
                ['loc' => '/gallery', 'changefreq' => SitemapBehavior::CHANGEFREQ_MONTHLY, 'priority' => 0.40], 
            ],
            'enableGzip' => true, // default is false
            'cacheExpire' => 1, // 1 second. Default is 24 hours
        ],
    ],
    'components' => [
        'request' => [
            'baseUrl' => $baseUrl,
            'csrfParam' => '_csrf-frontend',
        ],
        'user' => [
            'identityClass' => 'common\modules\user\models\UserIdentity',
            'enableAutoLogin' => true,
            'identityCookie' => ['name' => '_identity-frontend', 'httpOnly' => true],
        ],
        'session' => [
            'class' => 'yii\web\DbSession',
            'sessionTable' => 'mod_core_session',
            // this is the name of the session cookie used for login on the frontend
            'name' => 'advanced-frontend',
        ],
        'log' => [
            'traceLevel' => YII_DEBUG ? 3 : 0,
            'targets' => [
                [
                    'class' => 'yii\log\FileTarget',
                    'levels' => ['error', 'warning'],
                ],
            ],
        ],
        'urlManager' => [
            'enablePrettyUrl' => false,
            'showScriptName' => false,
            'rules' => [
            ],
        ],
        'errorHandler' => [
            'errorAction' => 'site/site/error',
        ],
        'view' => [
            'theme' => [
                'basePath' => '@assets/web/themes/genesys',
                'baseUrl' => '@assets/web/themes/genesys',
                'pathMap' => [
                    '@common/modules/site/views/frontend' => '@assets/web/themes/genesys',
                    '@common/modules/user/views/frontend' => '@assets/web/themes/genesys',
                    '@common/modules/cms/views/frontend' => '@assets/web/themes/genesys',
                ],
            ],
        ],
        'reCaptcha' => [
            'name' => 'reCaptcha',
            'class' => 'himiklab\yii2\recaptcha\ReCaptcha',
            'siteKey' => '6Le9GyQUAAAAABzuN2oF54k58MBNAEigFtj5jrbm',
            'secret' => '6Le9GyQUAAAAANzY0uWJHTtdrJefzeUs6E-yd3uK',
        ],
    ],
    'params' => $params,
    'layoutPath' => '@assets/web/themes/genesys/layouts',
];
